/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2006/07/22 17:11:45 $
 * $Author: sid4u $
 */

#ifndef _BSOCKET_H_
#define _BSOCKET_H_

#include <glib.h>

#include "buffer.h"
#include "l_socket.h"
#include "module.h"

G_BEGIN_DECLS

/** @defgroup BSocketModule Buffered socket class
 * @ingroup SocketClasses
 */
/*@{*/

/** @name Buffered socket status */
/*@{*/
#define ST_INPUT_OVERFLOW   100  ///< input buffer overflow
#define ST_CONSUME_ERROR    101  ///< error consuming input
/*@}*/

typedef struct _b_socket_t b_socket_t;

/** Buffer consume function
 * @param s the socket.
 * @return string to end the socket with that message (see
 * socket_done()) or NULL to keep receiving/sending
 */
typedef const char* (*BConsumeInputFunc)(b_socket_t* s);
typedef int (*BReadFunc)(b_socket_t*);
typedef int (*BWriteFunc)(b_socket_t*);
/** Prepare function.
 * @param s the socket.
 * @return TRUE on success, FALSE on error.
 */
typedef gboolean (*BPrepareFunc)(b_socket_t* s);

typedef struct _b_socket_priv_t b_socket_priv_t;

#define BSOCK(s) ((b_socket_t*)(s))
/** The buffered socket class */
struct _b_socket_t {
  socket_t _socket;       ///< the base class
  b_socket_priv_t* priv;  ///< private members
};

module_t* bsocket_module_instance();

void     b_socket_init(b_socket_t* s, const char* name, int timeout);
gboolean b_socket_end(b_socket_t* s, const char* message);
void     b_socket_destroy(b_socket_t* s);
void     b_socket_move(b_socket_t* dest, b_socket_t* src, gboolean copy_buffer);
void b_socket_start_receiving(b_socket_t* s, unsigned int max_size);
void b_socket_start_sending  (b_socket_t* s);
void b_socket_set_limit(b_socket_t* s, unsigned int max_size);

void b_socket_close_when_sent(b_socket_t* s, int status);
buffer_t* b_socket_get_out_buffer(b_socket_t* s, unsigned space);
buffer_t* b_socket_get_in_buffer(b_socket_t* s);
buffer_t* b_socket_set_in_buffer(b_socket_t* s, buffer_t* buffer);
buffer_t* b_socket_set_out_buffer(b_socket_t* s, buffer_t* buffer);
void b_socket_set_read_write(b_socket_t* s, BReadFunc read, BWriteFunc write);
void b_socket_set_consume(b_socket_t* s, BConsumeInputFunc func);
void b_socket_set_prepare_out(b_socket_t* s, BPrepareFunc func);

/*@}*/

G_END_DECLS

#endif
