/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _BUDDY_H_
#define _BUDDY_H_

#include <glib.h>

#include "attribute.h"
#include "module.h"
#include "user.h"

G_BEGIN_DECLS

/** \defgroup BuddyModule Support for buddies
 * @ingroup FileSharingFramework
 */
/*@{*/

/** The buddy class */
typedef struct {
  attr_container_t _ac;  ///< the base class
  char* description;
  GList* users;
  int online;   // dont access directly
} buddy_t;

/** Buddy callback */
typedef void (*BuddyFunc)(buddy_t* buddy);

module_t* buddy_module_instance();

void buddy_register_update(BuddyFunc func);
void buddy_propagate_update(buddy_t* buddy);

buddy_t* buddy_new(const char* nick);
void buddy_set_description(buddy_t* buddy, const char* des);
void buddy_remove_user(buddy_t* buddy, user_t* user);
void buddy_add_user(buddy_t* buddy, user_t* user);
void buddy_foreach_user(buddy_t* buddy, int ID, UserForeach func, void* data);
buddy_t* buddy_find_from_user(user_t* user);
gboolean buddy_has_user(buddy_t* buddy, int ID);
const char* buddy_get_filename();

/*@}*/

G_END_DECLS

#endif
