/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2006/10/29 00:52:57 $
 * $Author: sgop $
 */

#ifndef _CHAT_H_
#define _CHAT_H_

#include <glib.h>

#include "object.h"
#include "module.h"

G_BEGIN_DECLS

/** \defgroup ChatModule Chatting with lopster
 * @ingroup FileSharingFramework
 */
/*@{*/

/** @name Message hide flags */
/*@{*/
#define HIDE_NONE        0x00 ///< no hide
#define HIDE_SERVER      0x01 ///< hide server messages
#define HIDE_GLOBAL      0x02 ///< hide global messages
#define HIDE_PARTS       0x04 ///< hide channel parts
#define HIDE_JOINS       0x08 ///< hide channel joins
#define HIDE_MOTD        0x10 ///< hide motd message
#define HIDE_MASK        0x1f ///< mask for all flags
/*@}*/

/** @name Message pipe flags */
/*@{*/
#define PIPE_NONE        0x00 ///< no pipe
#define PIPE_PRIVATE     0x01 ///< pipe private messages
#define PIPE_GLOBAL      0x02 ///< pipe global messages
#define PIPE_OPERATOR    0x04 ///< pipe moderator messages
#define PIPE_CHAN_OP     0x08 ///< pipe channel op messages
#define PIPE_MASK        0x0f ///< mask for all flags.
/*@}*/

/** @name Message layout flags
 * @see chat_print()
 */
/*@{*/
#define PAD_LEFT         0x01 ///< pad with space to the left
#define PAD_RIGHT        0x02 ///< pad with space to the right
#define PAD_HILIGHT      0x04 ///< check message for highlights
/*@}*/

#define LFP    80

typedef struct _chat_page_t chat_page_t;

/** Function to decide what to do when closing a page.
 * @param page the page
 * @return TRUE to destroy the page, FALSE to keep the page
 */
typedef gboolean (*ChatCloseFunc)(chat_page_t* page);
/** Expand function for the chat input.
 * @param page the page
 * @param data the current input of the user
 * @return the expanded input for the user
 */
typedef char* (*ChatExpandFunc)(chat_page_t* page, const char* data);

#define PAGE(arg) ((chat_page_t*)(arg))
/** The chat page class. */
struct _chat_page_t {
  object_t _object;       ///< the base class
  char* label;
  char* title;
  int ID;                 ///< the ID of the plugin this page belongs to
  ChatCloseFunc close;    ///< the close function
  ChatExpandFunc expand;  ///< the expand function
  gboolean active;        ///< whether the page is active.
};


/** @name Controlling pages */
/*@{*/
module_t* chat_module_instance();
void chat_page_init(chat_page_t* page, const char* name, const char* label,
                    const char* title, int ID);
chat_page_t* chat_page_new(const char* name, const char* label, const char* title);
void chat_page_destroy(chat_page_t* page);

void chat_page_close(chat_page_t* page);
void chat_page_set_active(chat_page_t* page, gboolean active);
void chat_page_set_label(chat_page_t* page, const char* label);
void chat_page_set_title(chat_page_t* page, const char* title);
chat_page_t* chat_page_get_current(void);
/*@}*/

/** @name Printing messages */
/*@{*/
void chat_set_buffer(int num);
void chat_print(chat_page_t* page, ...);

void server_message (const char* sender, const char* message);
void motd_message   (const char* sender, const char* message);
void global_message (const char* sender, const char* user, const char* message);

void user_message   (chat_page_t* page, const char* sender, const char* type, const char* user, const char* message);
void normal_message (chat_page_t* page, const char* sender, const char* type, const char* message);
void emote_message  (chat_page_t* page, const char* sender, const char* type, const char* user, const char* message);
void private_message(chat_page_t* page, const char* sender, const char* type, const char* user, const char* message);
/*@}*/

/*@}*/

void chat_page_set_current(chat_page_t* page);

G_END_DECLS
#endif
