/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _COMMANDS_H_
#define _COMMANDS_H_

#include <glib.h>

#include "chat.h"
#include "module.h"

G_BEGIN_DECLS

/** \defgroup CommandModule Supporting commands for chat pages.
 * To be able to process commands that were typed in in a chat page
 * you have to install a command handler using
 * command_handler_install(). Lopster searches for the right command
 * and then passes it (including the given arguments) to the plugin
 * that provide that command.
 *
 * @ingroup FileSharingFramework
 */
/*@{*/

typedef struct _command_t command_t;

/** Command handler 
 * @param page the chat page the command was typed in
 * @param command the command that was typed in.
 * @param args the command arguments
 */
typedef void (*CommandHandler)(chat_page_t* page, command_t* command, char* args);

typedef void (*CommandForeachFunc)(command_t* command, void* data);

/** The command class */
struct _command_t {
  char* name;		     ///< name of the command
  char* args;		     ///< argument specification
  char* description; ///< description of the command and its arguments
  int id;            ///< id of the network that provides this command
};
#define COMMAND(c) ((command_t*)(c))

module_t* command_module_instance();

/** Print an error and return if a condition is met.
 * @param val the condition
 * @param page the chat page
 * @param text the error message.
 */
#define COMMAND_ERROR(val, page, text) \
  if (val) { command_error(page, text); return; }

void command_init(command_t* command, int id, const char* name,
                  const char* syntax, const char* des);
void command_handler_install(int ID, CommandHandler handler);
void command_error(chat_page_t* page, const char* message);

/*@}*/

void command_handle(chat_page_t* page, char *data);
void command_foreach(CommandForeachFunc func, int id, void* data);
command_t* command_search(const char *iname, int id);

char* command_expand(chat_page_t* page, const char* command);

void        command_history_update(const char* str);
const char* command_history_up(void);
const char* command_history_down(void);

G_END_DECLS
#endif
