/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _COMPLETION_H_
#define _COMPLETION_H_

#include <glib/glist.h>

G_BEGIN_DECLS

/** @defgroup CompletionModule Autocomplete in a list of objects
 * Complete a string by searching in a list of objects.
 *
 * @ingroup LopsterLibs
 */
/*@{*/

typedef struct _completion_t completion_t;

/** Function to convert an object into a string, which then is used
 * for comparison
 * @param object the object to compare
 * @return the string representation of the object
 */
typedef gchar* (*CompletionFunc)(gpointer object);
/** Function to compare two strings.
 * @param s1 first string
 * @param s2 second string
 * @param n how many characters to compare
 * @return similar to strcmp(), 0 if equal, -1 if s1 lower than s2, 1 otherwise
 */
typedef gint (*CompletionStrncmpFunc)(const gchar *s1, const gchar *s2, gsize n);

completion_t* completion_new(CompletionFunc func);
void completion_set_items(completion_t* cmp, GList** items);
GList* completion_complete(completion_t* cmp, const gchar* prefix, gchar** new_prefix);
void completion_set_compare(completion_t* cmp, CompletionStrncmpFunc strncmp_func);
void completion_free(completion_t* cmp);

/*@}*/

G_END_DECLS

#endif

