/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.9 $
 * $Date: 2007/05/23 20:21:53 $
 * $Author: sid4u $
 */

#include "dynamic.h"
#ifdef WIN32
#include "lcorealias.h"
#endif

typedef struct {
  module_t _module;
  GList* dlls;
} dynamic_module_t;

static dynamic_module_t* DynamicModule = NULL;

/** Open a shared library.
 * @param filename the full pathname of the library.
 * @param flags the GLib flags for opening (g_module_open())
 * @return the opened library or NULL
 */
GModule* lopster_dll_open(const char* filename, int flags)
{
    GModule* module = g_module_open(filename, flags);
    g_assert(DynamicModule);
    if (module)
    {
        g_log("DYNAMIC", G_LOG_LEVEL_DEBUG, "opening '%s'", filename);
        DynamicModule->dlls = g_list_append(DynamicModule->dlls, module);
    }
    return module;
}

/** Close a shared library. You normally don't need to close the
 * shared libraries manually. They are closed automatically on exit.
 * @param module the module.
 */
void lopster_dll_close(GModule* module)
{
    g_assert(DynamicModule);
    DynamicModule->dlls = g_list_remove(DynamicModule->dlls, module);
    g_log("DYNAMIC", G_LOG_LEVEL_DEBUG, "closing '%s'", g_module_name(module));
    g_module_close(module);
}

static void dynamic_exit()
{
/*   while (DynamicModule->dlls) { */
/*     GModule* module = DynamicModule->dlls->data; */
/*     lopster_dll_close(module); */
/*   } */
    DynamicModule->dlls = NULL;
    DynamicModule = NULL;
}

static void dynamic_module_init(dynamic_module_t* module)
{
    module_init(MODULE(module), "Dynamic Module",
                NULL, NULL, (ModuleFunc)dynamic_exit);
    module->dlls = NULL;
}


/** This module is one of the very few modules that is ready for use
 * as soon as you've requested it.  No need to wait for
 * module_init()
 * @return the module instance
 * @see module_require()
 */
module_t* dynamic_module_instance()
{
    if (!DynamicModule)
    {
        DynamicModule = g_malloc0(sizeof(*DynamicModule));
        dynamic_module_init(DynamicModule);
    }
    return MODULE(DynamicModule);
}

#ifdef WIN32
# define _DYNAMIC_C_
# include "lcorealiasdef.c"
#endif
