/* Copyright (C) 2004 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _EXEC_H_
#define _EXEC_H_

#include <glib.h>

#include "buffer.h"

G_BEGIN_DECLS

#define EXEC_INACTIVE     0
#define EXEC_QUEUED       1
#define EXEC_RUNNING      2
#define EXEC_FAILED       3
#define EXEC_ABORTED      4
#define EXEC_SUCCESS      5

typedef struct _exec_t exec_t;

typedef gboolean (*ExecReadFunc)(exec_t*, void*);
typedef void (*ExecEndFunc)(exec_t*, void*);

struct _exec_t {
  char** argv;  // NULL terminated list of arguments
  unsigned status;
  int internal_id;
  GPid pid;

  ExecReadFunc read;   // called when process writes data
  void* read_data;
  int read_fd;
  int read_input;
  buffer_t* buffer;

  ExecEndFunc end;     // called when process ends
  void* end_data;
};

exec_t* exec_new(char** argv);
void exec_start(exec_t* exec, ExecEndFunc end, void* data);
void exec_set_output_func(exec_t* exec, ExecReadFunc func, void* data);

G_END_DECLS
#endif
