/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.3 $
 * $Date: 2006/07/22 17:11:46 $
 * $Author: sid4u $
 */

#ifndef _FILE_SEGMENT_H_
#define _FILE_SEGMENT_H_

#define SEG_ERR_NONE   0x00
#define SEG_ERR_NEXT   0x01
#define SEG_ERR_PREV   0x02
#define SEG_ERR_HASH   0x04

#include "utils.h"

G_BEGIN_DECLS


typedef struct _segment_t segment_t;
typedef struct _segments_t segments_t;

typedef void (*SegmentsForeach)(segment_t* segment, void* data);

struct _segment_t {
  file_size_t start;
  file_size_t size;
  file_size_t stop;
  unsigned error;
};


segment_t* segment_new(file_size_t start, file_size_t stop,
                       file_size_t size, unsigned error);
void segment_destroy(segment_t* segment);
void segment_print(segment_t* segment);

segments_t* segments_new(void);
void segments_destroy(segments_t* segments);
void segments_prepend(segments_t* segments, segment_t* segment);
void segments_append(segments_t* segments, segment_t* segment);
segments_t* segments_insert(segments_t* segments, file_size_t start,
                            file_size_t stop, file_size_t size, unsigned error);
segments_t* segments_create_complement(segments_t* segments, file_size_t size);
void segments_foreach(segments_t* segments, SegmentsForeach func, void* data);
int segments_cover(segments_t* segments, file_size_t start, file_size_t stop);
gboolean segments_intersect(segments_t* segments, file_size_t start, file_size_t stop);
unsigned segments_count(segments_t* segments);
segment_t* segments_cut(segments_t* segments,	file_size_t start, file_size_t stop);
segment_t* segments_get_first(segments_t* segments);
GList* segments_get_list(segments_t* segments);

G_END_DECLS

#endif
