/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2006/07/22 17:11:46 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* FLAC streaminfo access.
 * Based on ogg_ext.c 
 * (sgop@users.sf.net) MODIFIED: not based much on ogg_ext.c anymore
 */

#ifdef HAVE_FLAC

#include <string.h>

#include <FLAC/metadata.h>
#include <FLAC/format.h>

#include <glib.h>

#include "library.h"
#include "files.h"

void file_parse_flac(thread_t* thread, lib_node_t* node) {
  FLAC__StreamMetadata streaminfo;
  file_t* file;
  char* temp;
  int bitrate;
  int frequency;
  int duration;
  attribute_t* attr;
  
  file = FILE_NODE(node)->file;

  temp = get_suffix(file->filename);
  if (!temp) return;
  if (strcasecmp(temp, "flac")) return;

  attr = attr_find(ATTRC(file), "file", "Bitrate");
  if (attr && attr->refreshed >= node->mod_time) return;

  temp = locale_from_utf8(file->localname);
  if (!temp) return;
  
  if (!FLAC__metadata_get_streaminfo(temp, &streaminfo) ||
      (frequency = streaminfo.data.stream_info.sample_rate) <= 0 ||
      (duration = (time_t)(streaminfo.data.stream_info.total_samples / frequency)) <= 0 ||
      (bitrate = (int)(file->size / duration * 8 / 1000)) <= 0) {
    
    g_log("FLAC", G_LOG_LEVEL_WARNING,
          "error reading header information from");
    g_log("FLAC", G_LOG_LEVEL_WARNING,
          "  '%s'", file->localname);
    g_log("FLAC", G_LOG_LEVEL_WARNING,
          "Are you sure that this is a flac file?");
  } else {
    thread_lock(thread);
    attr_store(ATTRC(file), "file", "Bitrate", bitrate);
    attr_store(ATTRC(file), "file", "Frequency", frequency);
    attr_store(ATTRC(file), "file", "Duration", duration);
    thread_unlock(thread);
  }
  g_free(temp);
}

#endif
