/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2006/07/22 17:11:58 $
 * $Author: sid4u $
 */

#include "gtkhtabs.h"

static void gtk_htabs_class_init   (GtkHtabsClass  *klass);
static void gtk_htabs_init         (GtkHtabs       *box);
static void gtk_htabs_size_request (GtkWidget      *widget,
				    GtkRequisition *requisition);
static void gtk_htabs_size_allocate(GtkWidget      *widget,
				    GtkAllocation  *allocation);


GType gtk_htabs_get_type(void) {
  static GType htabs_type = 0;

  if (!htabs_type) {
    static const GTypeInfo htabs_info = {
      sizeof (GtkHtabsClass),
      NULL,		/* base_init */
      NULL,		/* base_finalize */
      (GClassInitFunc) gtk_htabs_class_init,
      NULL,		/* class_finalize */
      NULL,		/* class_data */
      sizeof (GtkHtabs),
      0,		/* n_preallocs */
      (GInstanceInitFunc) gtk_htabs_init,
      0
    };
    
    htabs_type = g_type_register_static
      (GTK_TYPE_BOX, "GtkHtabs", &htabs_info, 0);
  }

  return htabs_type;
}

static void gtk_htabs_class_init(GtkHtabsClass *class) {
  GtkWidgetClass *widget_class;

  widget_class = (GtkWidgetClass*) class;

  widget_class->size_request = gtk_htabs_size_request;
  widget_class->size_allocate = gtk_htabs_size_allocate;
}

static void gtk_htabs_init(GtkHtabs *htabs) {
  (void)htabs;
}

GtkWidget* gtk_htabs_new(gboolean homogeneous, gint spacing) {
  GtkHtabs *htabs;

  htabs = g_object_new(GTK_TYPE_HTABS, NULL);

  GTK_BOX(htabs)->spacing = spacing;
  GTK_BOX(htabs)->homogeneous = homogeneous ? TRUE : FALSE;

  return GTK_WIDGET (htabs);
}


static void
gtk_htabs_size_request(GtkWidget* widget, GtkRequisition* req) {
  GtkBox *box;
  GtkBoxChild *child;
  GList *children;
  gint nvis_children;
  gint width;
  
  box = GTK_BOX(widget);
  req->width = 0;
  req->height = 0;
  nvis_children = 0;

  children = box->children;
  while (children) {
    child = children->data;
    children = children->next;
    
    if (GTK_WIDGET_VISIBLE(child->widget)) {
      GtkRequisition child_req;
      gtk_widget_size_request(child->widget, &child_req);
      
      width = 10 + child->padding * 2;  // min size 10
      req->width = MAX(req->width, width);
      
      req->height = MAX(req->height, child_req.height);
      
      nvis_children += 1;
    }
  }

  if (nvis_children > 0) {
    req->width *= nvis_children;
    req->width += (nvis_children - 1) * box->spacing;
  }

  req->width += GTK_CONTAINER(box)->border_width * 2;
  req->height += GTK_CONTAINER(box)->border_width * 2;
}

static void
gtk_htabs_size_allocate(GtkWidget* widget, GtkAllocation* alloc) {
  GtkBox *box;
  GtkBoxChild *child;
  GList *children;
  GtkAllocation child_alloc;
  gint nvis_children;
  gint width;
  gint extra;
  gint x;
  GtkTextDirection direction;

  box = GTK_BOX (widget);
  widget->allocation = *alloc;

  direction = gtk_widget_get_direction(widget);
  
  nvis_children = 0;
  children = box->children;

  while (children) {
    child = children->data;
    children = children->next;
    
    if (GTK_WIDGET_VISIBLE(child->widget)) nvis_children++;
  }

#define D_WIDTH 150
  if (!nvis_children) return;

  width = (alloc->width - GTK_CONTAINER(box)->border_width*2 -
	   (nvis_children - 1) * box->spacing);
  extra = MIN(width / nvis_children, D_WIDTH);

  x = alloc->x + GTK_CONTAINER(box)->border_width;
  child_alloc.y = alloc->y + GTK_CONTAINER (box)->border_width;
  child_alloc.height =
    MAX (1, (gint)alloc->height - (gint)GTK_CONTAINER (box)->border_width * 2);

  children = box->children;
  while (children) {
    child = children->data;
    children = children->next;

    if (GTK_WIDGET_VISIBLE(child->widget)) {
      /*
      GtkRequisition child_req;
      gtk_widget_size_request(child->widget, &child_req);
      if (child_req.width < extra) extra = child_req.width;
      */

      nvis_children--;
      width -= extra;
      
      child_alloc.width = extra;
      child_alloc.x = x;
      

      if (direction == GTK_TEXT_DIR_RTL)
	child_alloc.x = alloc->x + alloc->width - (child_alloc.x - alloc->x) - child_alloc.width;
      gtk_widget_size_allocate(child->widget, &child_alloc);
      
      // update the x position
      x += child_alloc.width + box->spacing;
      // update the extra value, which might increase a fuew pixels
      if (nvis_children == 0) extra = MIN(width, D_WIDTH);
      else extra = MIN(width / nvis_children, D_WIDTH);
    }
  }
}
