/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#include <stdlib.h>
#include <glib.h>

#include "renderer_download.h"

static void
gtk_cell_renderer_download_init(GtkCellRendererDownload* celltext);
static void
gtk_cell_renderer_download_class_init(GtkCellRendererDownloadClass* class);
static void
gtk_cell_renderer_download_finalize(GObject* object);

static void
gtk_cell_renderer_download_get_property(GObject* object, guint param_id,
				      GValue* value, GParamSpec* pspec);
static void
gtk_cell_renderer_download_set_property(GObject* object, guint param_id,
				  const GValue* value, GParamSpec* pspec);
static void
gtk_cell_renderer_download_get_size(GtkCellRenderer* cell,
				  GtkWidget* widget,
				  GdkRectangle* cell_area,
				  gint* x_offset, gint* y_offset,
				  gint* width, gint* height);
static void
gtk_cell_renderer_download_render(GtkCellRenderer* cell,
				GdkWindow* window,
				GtkWidget* widget,
				GdkRectangle* background_area,
				GdkRectangle* cell_area,
				GdkRectangle* expose_area,
				guint flags);

enum {
  PROP_0,
  PROP_DOWNLOAD,
  PROP_COLOR_BACKGROUND,
  PROP_COLOR_BORDER,
  PROP_COLOR_ACTIVE,
  PROP_COLOR_FINISHED,
  PROP_COLOR_PROGRESS,
};

enum {
  COLOR_BACK_GROUND,
  COLOR_BORDER,
  COLOR_ACTIVE,
  COLOR_FINISHED,
  COLOR_PROGRESS,
  COLOR_SIZE,
};

struct _GtkCellRendererDownloadPriv {
  download_t* download;
  source_t* source;
  GdkColor color[5]; //back,border,active,finished,progress
  gboolean color_set[5];
};

static gpointer parent_class;

GType gtk_cell_renderer_download_get_type(void) {
  static GType cell_download_type = 0;

  if (!cell_download_type) {
    static const GTypeInfo cell_download_info = {
      sizeof (GtkCellRendererDownloadClass),
      NULL,		/* base_init */
      NULL,		/* base_finalize */
      (GClassInitFunc) gtk_cell_renderer_download_class_init,
      NULL,		/* class_finalize */
      NULL,		/* class_data */
      sizeof (GtkCellRendererDownload),
      0,              /* n_preallocs */
      (GInstanceInitFunc) gtk_cell_renderer_download_init,
      0,
    };

    cell_download_type =
      g_type_register_static(GTK_TYPE_CELL_RENDERER,
                             "GtkCellRendererDownload",
                             &cell_download_info, 0);
  }

  return cell_download_type;
}

static void
gtk_cell_renderer_download_init(GtkCellRendererDownload *celldownload) {
    
  celldownload->priv = (GtkCellRendererDownloadPriv*)g_new0(GtkCellRendererDownloadPriv, 1);
  celldownload->priv->download = NULL;
  celldownload->priv->source = NULL;
  celldownload->priv->color_set[COLOR_BACK_GROUND] = FALSE;
  celldownload->priv->color_set[COLOR_BORDER] = FALSE;
  celldownload->priv->color_set[COLOR_ACTIVE] = FALSE;
  celldownload->priv->color_set[COLOR_FINISHED] = FALSE;
  celldownload->priv->color_set[COLOR_PROGRESS] = FALSE;
}

static void
gtk_cell_renderer_download_class_init (GtkCellRendererDownloadClass *class) {
  GObjectClass *object_class = G_OBJECT_CLASS (class);
  GtkCellRendererClass *cell_class = GTK_CELL_RENDERER_CLASS(class);

  parent_class = g_type_class_peek_parent(class);
  
  object_class->finalize = gtk_cell_renderer_download_finalize;
  
  object_class->get_property = gtk_cell_renderer_download_get_property;
  object_class->set_property = gtk_cell_renderer_download_set_property;

  cell_class->get_size = gtk_cell_renderer_download_get_size;
  cell_class->render = gtk_cell_renderer_download_render;
  
  g_object_class_install_property
    (object_class, PROP_DOWNLOAD,
     g_param_spec_pointer("download", "Download",
                          "the download", G_PARAM_READWRITE));

  g_object_class_install_property
    (object_class, PROP_COLOR_BACKGROUND,
     g_param_spec_boxed("background-color", "Background Color",
                        "Background color of the download bar.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));
  
  g_object_class_install_property
    (object_class, PROP_COLOR_BORDER,
     g_param_spec_boxed("border-color", "Border Color",
                        "Border color of the download bar.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));
  
  g_object_class_install_property
    (object_class, PROP_COLOR_FINISHED,
     g_param_spec_boxed("finished-color", "Finished segments",
                        "Color for finsihed segments.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));
  
  g_object_class_install_property
    (object_class, PROP_COLOR_ACTIVE,
     g_param_spec_boxed("active-color", "Active segments",
                        "Color for active segments.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));
  
  g_object_class_install_property
    (object_class, PROP_COLOR_PROGRESS,
     g_param_spec_boxed("progress-color", "Progress Color",
                        "Progress color of the download bar.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));

}

static void
gtk_cell_renderer_download_get_property(GObject* object, guint param_id,
                                        GValue* value, GParamSpec* pspec) {
  GtkCellRendererDownload* cell = GTK_CELL_RENDERER_DOWNLOAD(object);

  switch (param_id) {
  case PROP_DOWNLOAD:
    return g_value_set_pointer(value, NULL);
    break;
  case PROP_COLOR_BACKGROUND: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_BACK_GROUND].red;
    color.green = cell->priv->color[COLOR_BACK_GROUND].green;
    color.blue = cell->priv->color[COLOR_BACK_GROUND].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  case PROP_COLOR_BORDER: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_BORDER].red;
    color.green = cell->priv->color[COLOR_BORDER].green;
    color.blue = cell->priv->color[COLOR_BORDER].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  case PROP_COLOR_ACTIVE: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_ACTIVE].red;
    color.green = cell->priv->color[COLOR_ACTIVE].green;
    color.blue = cell->priv->color[COLOR_ACTIVE].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  case PROP_COLOR_FINISHED: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_FINISHED].red;
    color.green = cell->priv->color[COLOR_FINISHED].green;
    color.blue = cell->priv->color[COLOR_FINISHED].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  case PROP_COLOR_PROGRESS: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_PROGRESS].red;
    color.green = cell->priv->color[COLOR_PROGRESS].green;
    color.blue = cell->priv->color[COLOR_PROGRESS].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
    break;
  }
}

static void set_bg_color(GtkCellRendererDownload* cell, int id, GdkColor* color) {
  if (color) {
    if (!cell->priv->color_set[id]) {
      cell->priv->color_set[id] = TRUE;
    }
    cell->priv->color[id].red   = color->red;
    cell->priv->color[id].green = color->green;
    cell->priv->color[id].blue  = color->blue;
  } else {
    if (cell->priv->color_set[id]) {
      cell->priv->color_set[id] = FALSE;
    }
  }
  //  g_object_notify(G_OBJECT(cell), ColorPropName[id]);
}

static void
gtk_cell_renderer_download_set_property(GObject* object,
                                        guint param_id,
                                        const GValue* value,
                                        GParamSpec* pspec) {
  GtkCellRendererDownload* celldownload = 
    GTK_CELL_RENDERER_DOWNLOAD (object);

  switch (param_id) {
  case PROP_DOWNLOAD: {
    rend_dl_data_t* data = g_value_get_pointer(value);
    celldownload->priv->download = data?data->download:NULL;
    celldownload->priv->source = data?data->source:NULL;
    break;
  }
  case PROP_COLOR_BACKGROUND:
    set_bg_color(celldownload, COLOR_BACK_GROUND, g_value_get_boxed(value));
    break;
  case PROP_COLOR_BORDER:
    set_bg_color(celldownload, COLOR_BORDER, g_value_get_boxed(value));
    break;
  case PROP_COLOR_ACTIVE:
    set_bg_color(celldownload, COLOR_ACTIVE, g_value_get_boxed(value));
    break;
  case PROP_COLOR_FINISHED:
    set_bg_color(celldownload, COLOR_FINISHED, g_value_get_boxed(value));
    break;
  case PROP_COLOR_PROGRESS:
    set_bg_color(celldownload, COLOR_PROGRESS, g_value_get_boxed(value));
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
    break;
  }
  g_object_notify(object, "download");
}

static void
gtk_cell_renderer_download_get_size(GtkCellRenderer* cell,
                                    GtkWidget* widget,
                                    GdkRectangle* cell_area,
                                    gint* x_offset, gint* y_offset,
                                    gint* width, gint* height) {
  (void)cell;
  (void)widget;
  (void)cell_area;
  (void)x_offset;
  (void)y_offset;
  if (width) *width = 8;
  if (height) *height = 8;
}

GtkCellRenderer* gtk_cell_renderer_download_new(void) {
  return GTK_CELL_RENDERER(g_object_new(gtk_cell_renderer_download_get_type(), NULL));
}

static void
gtk_cell_renderer_download_render(GtkCellRenderer* cell,
                                  GdkWindow* window,
                                  GtkWidget* widget,
                                  GdkRectangle* background_area,
                                  GdkRectangle* cell_area,
                                  GdkRectangle* expose_area,
                                  guint flags) {
  GtkCellRendererDownload* cd = (GtkCellRendererDownload*)cell;
  GdkGC *gc;
  file_size_t prog, todo;
  gint x, w, h, x2;
      
  (void)widget;
  (void)background_area;
  (void)expose_area;
  (void)flags;

  gc = gdk_gc_new(window);
  
  todo = prog = 0;

  w = cell_area->width;
  h = cell_area->height;
  
  if (cd->priv->download || (cd->priv->source && cd->priv->source->cur_segment)) {
    // the background
    if (cd->priv->color_set[COLOR_BACK_GROUND]) {
      gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_BACK_GROUND]);
      gdk_draw_rectangle(window, gc, TRUE, cell_area->x, cell_area->y, w, h);
    }
    if (cd->priv->color_set[COLOR_BORDER]) {
      gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_BORDER]);
      gdk_draw_rectangle(window, gc, FALSE, cell_area->x, cell_area->y, w-1, h-1);
    }
  }

  w -= 2;
  h -= 2;

  if (cd->priv->download) {
    GList* dlist;
    download_t* download = cd->priv->download;
    file_size_t size = FILE(download)->size;
    
    todo = size;

    for (dlist = segments_get_list(download->segments_done); dlist; dlist = dlist->next) {
      segment_t* segment = dlist->data;
      prog += (segment->stop-segment->start);
      if (cd->priv->color_set[COLOR_FINISHED]) {
        x = segment->start*w/size;
        x2 = (segment->stop-1)*w/size +1;
        gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_FINISHED]);
        gdk_draw_rectangle(window, gc, TRUE, 
                           cell_area->x + x+1, cell_area->y+1,
                           x2-x, h);
      }
    }
    
    for (dlist = download->segments_used; dlist; dlist = dlist->next) {
      dl_segment_t* s = dlist->data;
      segment_t* segment = s->segment;
      
      x = segment->start*w/size;
      if (cd->priv->color_set[COLOR_ACTIVE]) {
        if (segment->start + segment->size < segment->stop) {
          x2 = (segment->stop-1)*w/size +1;
          gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_ACTIVE]);
          gdk_draw_rectangle(window, gc, TRUE, 
                             cell_area->x + x+1, cell_area->y+1,
                             x2-x, h);
        }
      }
      if (segment->size > 0) {
        prog += segment->size;
        if (cd->priv->color_set[COLOR_FINISHED]) {
          x2 = (segment->start+segment->size-1)*w/size +1;
          gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_FINISHED]);
          gdk_draw_rectangle(window, gc, TRUE, 
                             cell_area->x + x+1, cell_area->y+1,
                             x2-x, h);
        }
      }
    }
  } else if (cd->priv->source) {
    GList* dlist;
    source_t* source = cd->priv->source;
    download_t* download = source->download;
    file_size_t size = FILE(download)->size;

    for (dlist = source->used_segments; dlist; dlist = dlist->next) {
      segment_t* segment = dlist->data;
      
      x = segment->start*w/size;
      if (cd->priv->color_set[COLOR_ACTIVE]) {
        if (segment->start + segment->size < segment->stop) {
          x2 = (segment->stop-1)*w/size +1;
          gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_ACTIVE]);
          gdk_draw_rectangle(window, gc, TRUE, 
                             cell_area->x + x+1, cell_area->y+1,
                             x2-x, h);
        }
      }
      if (cd->priv->color_set[COLOR_FINISHED]) {
        if (segment->size > 0) {
          x2 = (segment->start+segment->size-1)*w/size +1;
          gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_FINISHED]);
          gdk_draw_rectangle(window, gc, TRUE, 
                             cell_area->x + x+1, cell_area->y+1,
                             x2-x, h);
        }
      }
    }
    if (source->cur_segment) {
      segment_t* segment = source->cur_segment;
      todo = segment->stop - segment->start;
      prog = segment->size;

      x = segment->start*w/size;
      if (cd->priv->color_set[COLOR_ACTIVE]) {
        if (segment->start + segment->size < segment->stop) {
          x2 = (segment->stop-1)*w/size +1;
          gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_ACTIVE]);
          gdk_draw_rectangle(window, gc, TRUE, 
                             cell_area->x + x+1, cell_area->y+1,
                             x2-x, h);
        }
      }
      if (cd->priv->color_set[COLOR_FINISHED]) {
        if (segment->size > 0) {
          x2 = (segment->start+segment->size-1)*w/size +1;
          gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_FINISHED]);
          gdk_draw_rectangle(window, gc, TRUE, 
                             cell_area->x + x+1, cell_area->y+1,
                             x2-x, h);
        }
      }
    }
  }
  if (cd->priv->color_set[COLOR_PROGRESS]) {
    if (prog > 0 && todo > 0) {
      x = 0;
      x2 = (prog-1)*w / todo +1;
      gdk_gc_set_rgb_fg_color(gc, &cd->priv->color[COLOR_PROGRESS]);
      gdk_draw_rectangle(window, gc, TRUE, 
                         cell_area->x + x+1, cell_area->y+1,
                         x2-x, h/4);
    }
  }
  g_object_unref(G_OBJECT(gc));
}

static void gtk_cell_renderer_download_finalize(GObject *object) {
  GtkCellRendererDownload* celldownload = GTK_CELL_RENDERER_DOWNLOAD(object);

  g_free(celldownload->priv);
  
  (*G_OBJECT_CLASS(parent_class)->finalize)(object);
}
