/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.7 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#include <stdlib.h>
#include <glib.h>

#include "renderer_upload.h"
#include "lcore/upload.h"

static void gtk_cell_renderer_upload_init(GtkCellRendererUpload* celltext);
static void gtk_cell_renderer_upload_class_init(GtkCellRendererUploadClass* class);
static void gtk_cell_renderer_upload_finalize(GObject* object);

static void gtk_cell_renderer_upload_get_property(GObject* object, guint param_id,
                                                  GValue* value, GParamSpec* pspec);
static void gtk_cell_renderer_upload_set_property(GObject* object, guint param_id,
                                                  const GValue* value, GParamSpec* pspec);
static void gtk_cell_renderer_upload_get_size(GtkCellRenderer* cell,
                                              GtkWidget* widget,
                                              GdkRectangle* cell_area,
                                              gint* x_offset, gint* y_offset,
                                              gint* width, gint* height);
static void gtk_cell_renderer_upload_render(GtkCellRenderer* cell,
                                            GdkWindow* window,
                                            GtkWidget* widget,
                                            GdkRectangle* background_area,
                                            GdkRectangle* cell_area,
                                            GdkRectangle* expose_area,
                                            guint flags);

enum {
  PROP_0,
  PROP_UPLOAD,
  PROP_COLOR_BACKGROUND,
  PROP_COLOR_BORDER,
  PROP_COLOR_FINISHED,
  PROP_COLOR_ACTIVE,
  PROP_COLOR_PROGRESS,
};

enum {
  COLOR_BACK_GROUND,
  COLOR_BORDER,
  COLOR_ACTIVE,
  COLOR_FINISHED,
  COLOR_PROGRESS,
  COLOR_SIZE,
};

struct _GtkCellRendererUploadPriv {
  upload_t* upload;
  GdkColor color[5]; //back,border,active,finished,progress
  gboolean color_set[5];
};

static gpointer parent_class;

GType gtk_cell_renderer_upload_get_type(void) {
  static GType cell_upload_type = 0;

  if (!cell_upload_type) {
    static const GTypeInfo cell_upload_info = {
      sizeof (GtkCellRendererUploadClass),
      NULL,		/* base_init */
      NULL,		/* base_finalize */
      (GClassInitFunc) gtk_cell_renderer_upload_class_init,
      NULL,		/* class_finalize */
      NULL,		/* class_data */
      sizeof (GtkCellRendererUpload),
      0,              /* n_preallocs */
      (GInstanceInitFunc) gtk_cell_renderer_upload_init,
      0,
    };

    cell_upload_type =
      g_type_register_static(GTK_TYPE_CELL_RENDERER,
                             "GtkCellRendererUpload",
                             &cell_upload_info, 0);
  }

  return cell_upload_type;
}

static void
gtk_cell_renderer_upload_init(GtkCellRendererUpload *cellupload) {
    
  cellupload->priv = (GtkCellRendererUploadPriv*)g_new0(GtkCellRendererUploadPriv, 1);
  cellupload->priv->upload = NULL;
  cellupload->priv->color_set[COLOR_BACK_GROUND] = FALSE;
  cellupload->priv->color_set[COLOR_BORDER] = FALSE;
  cellupload->priv->color_set[COLOR_ACTIVE] = FALSE;
  cellupload->priv->color_set[COLOR_FINISHED] = FALSE;
  cellupload->priv->color_set[COLOR_PROGRESS] = FALSE;
}

static void
gtk_cell_renderer_upload_class_init (GtkCellRendererUploadClass *class) {
  GObjectClass *object_class = G_OBJECT_CLASS (class);
  GtkCellRendererClass *cell_class = GTK_CELL_RENDERER_CLASS(class);

  parent_class = g_type_class_peek_parent (class);
  
  object_class->finalize = gtk_cell_renderer_upload_finalize;
  
  object_class->get_property = gtk_cell_renderer_upload_get_property;
  object_class->set_property = gtk_cell_renderer_upload_set_property;

  cell_class->get_size = gtk_cell_renderer_upload_get_size;
  cell_class->render = gtk_cell_renderer_upload_render;
  
  g_object_class_install_property
    (object_class, PROP_UPLOAD,
     g_param_spec_pointer("upload", "Upload",
                          "the upload", G_PARAM_READWRITE));

  g_object_class_install_property
    (object_class, PROP_COLOR_BACKGROUND,
     g_param_spec_boxed("background-color", "Background Color",
                        "Background color of the download bar.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));
  
  g_object_class_install_property
    (object_class, PROP_COLOR_BORDER,
     g_param_spec_boxed("border-color", "Border Color",
                        "Border color of the download bar.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));
  
  g_object_class_install_property
    (object_class, PROP_COLOR_FINISHED,
     g_param_spec_boxed("finished-color", "Finished segments",
                        "Color for finsihed segments.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));
  
  g_object_class_install_property
    (object_class, PROP_COLOR_ACTIVE,
     g_param_spec_boxed("active-color", "Active segments",
                        "Color for active segments.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));
  
  g_object_class_install_property
    (object_class, PROP_COLOR_PROGRESS,
     g_param_spec_boxed("progress-color", "Progress Color",
                        "Progress color of the download bar.",
                        GDK_TYPE_COLOR, G_PARAM_READWRITE));

}

static void
gtk_cell_renderer_upload_get_property(GObject* object,
                                      guint param_id,
                                      GValue* value,
                                      GParamSpec* pspec) {
  GtkCellRendererUpload* cell = GTK_CELL_RENDERER_UPLOAD(object);

  switch (param_id) {
  case PROP_UPLOAD:
    return g_value_set_pointer(value, NULL);
    break;
  case PROP_COLOR_BACKGROUND: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_BACK_GROUND].red;
    color.green = cell->priv->color[COLOR_BACK_GROUND].green;
    color.blue = cell->priv->color[COLOR_BACK_GROUND].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  case PROP_COLOR_BORDER: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_BORDER].red;
    color.green = cell->priv->color[COLOR_BORDER].green;
    color.blue = cell->priv->color[COLOR_BORDER].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  case PROP_COLOR_ACTIVE: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_ACTIVE].red;
    color.green = cell->priv->color[COLOR_ACTIVE].green;
    color.blue = cell->priv->color[COLOR_ACTIVE].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  case PROP_COLOR_FINISHED: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_FINISHED].red;
    color.green = cell->priv->color[COLOR_FINISHED].green;
    color.blue = cell->priv->color[COLOR_FINISHED].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  case PROP_COLOR_PROGRESS: {
    GdkColor color;
    color.red = cell->priv->color[COLOR_PROGRESS].red;
    color.green = cell->priv->color[COLOR_PROGRESS].green;
    color.blue = cell->priv->color[COLOR_PROGRESS].blue;
    g_value_set_boxed(value, &color);
    break;
  }
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
    break;
  }
}

static void set_bg_color(GtkCellRendererUpload* cell, int id, GdkColor* color) {
  if (color) {
    if (!cell->priv->color_set[id]) {
      cell->priv->color_set[id] = TRUE;
    }
    cell->priv->color[id].red   = color->red;
    cell->priv->color[id].green = color->green;
    cell->priv->color[id].blue  = color->blue;
  } else {
    if (cell->priv->color_set[id]) {
      cell->priv->color_set[id] = FALSE;
    }
  }
  //  g_object_notify(G_OBJECT(cell), ColorPropName[id]);
}

static void
gtk_cell_renderer_upload_set_property(GObject* object,
                                      guint param_id,
                                      const GValue* value,
                                      GParamSpec* pspec) {
  GtkCellRendererUpload* cellupload = 
    GTK_CELL_RENDERER_UPLOAD (object);

  switch (param_id) {
  case PROP_UPLOAD:
    cellupload->priv->upload = g_value_get_pointer(value);
    break;
  case PROP_COLOR_BACKGROUND:
    set_bg_color(cellupload, COLOR_BACK_GROUND, g_value_get_boxed(value));
    break;
  case PROP_COLOR_BORDER:
    set_bg_color(cellupload, COLOR_BORDER, g_value_get_boxed(value));
    break;
  case PROP_COLOR_ACTIVE:
    set_bg_color(cellupload, COLOR_ACTIVE, g_value_get_boxed(value));
    break;
  case PROP_COLOR_FINISHED:
    set_bg_color(cellupload, COLOR_FINISHED, g_value_get_boxed(value));
    break;
  case PROP_COLOR_PROGRESS:
    set_bg_color(cellupload, COLOR_PROGRESS, g_value_get_boxed(value));
    break;
  default:
    G_OBJECT_WARN_INVALID_PROPERTY_ID(object, param_id, pspec);
    break;
  }
  g_object_notify(object, "upload");
}

static void
gtk_cell_renderer_upload_get_size(GtkCellRenderer* cell,
                                  GtkWidget* widget,
                                  GdkRectangle* cell_area,
                                  gint* x_offset, gint* y_offset,
                                  gint* width, gint* height) {
  (void)cell;
  (void)widget;
  (void)cell_area;
  (void)x_offset;
  (void)y_offset;
  if (width) *width = 8;
  if (height) *height = 8;
}

GtkCellRenderer* gtk_cell_renderer_upload_new(void) {
  return GTK_CELL_RENDERER(g_object_new(gtk_cell_renderer_upload_get_type(), NULL));
}

static void
gtk_cell_renderer_upload_render(GtkCellRenderer* cell,
                                GdkWindow* window,
                                GtkWidget* widget,
                                GdkRectangle* background_area,
                                GdkRectangle* cell_area,
                                GdkRectangle* expose_area,
                                guint flags) {
  GtkCellRendererUpload* cu =
    (GtkCellRendererUpload*)cell;
      
  (void)widget;
  (void)background_area;
  (void)expose_area;
  (void)flags;
 
  if (cu->priv->upload && cu->priv->upload->cur_segment) {
    GdkGC* gc = gdk_gc_new(window);
    GList* dlist;
    upload_t* upload = cu->priv->upload;
    file_size_t size = upload->file->size;
    gint x, w, h;
    gint x2;
    segment_t* seg;
    unsigned buf = buffer_avail(b_socket_get_out_buffer(BSOCK(upload), 0));

    w = cell_area->width;
    h = cell_area->height;

    // the background
    if (cu->priv->color_set[COLOR_BACK_GROUND]) {
      gdk_gc_set_rgb_fg_color(gc, &cu->priv->color[COLOR_BACK_GROUND]);
      gdk_draw_rectangle(window, gc, TRUE, cell_area->x, cell_area->y, w, h);
    }
    if (cu->priv->color_set[COLOR_BORDER]) {
      gdk_gc_set_rgb_fg_color(gc, &cu->priv->color[COLOR_BORDER]);
      gdk_draw_rectangle(window, gc, FALSE, cell_area->x, cell_area->y, w-1, h-1);
    }
    
    w -= 2;
    h -= 2;

    for (dlist = segments_get_list(upload->finished_segments); dlist; dlist = dlist->next) {
      segment_t* segment = dlist->data;
      if (segment->size <= 0) continue;
      
      if (cu->priv->color_set[COLOR_FINISHED]) {
        x = segment->start*w/size;
        x2 = (segment->stop-1)*w/size +1;
        gdk_gc_set_rgb_fg_color(gc, &cu->priv->color[COLOR_FINISHED]);
        gdk_draw_rectangle(window, gc, TRUE, 
                           cell_area->x + x+1, cell_area->y+1,
                           x2-x, h);
      }
    }
    for (dlist = segments_get_list(upload->segments); dlist; dlist = dlist->next) {
      segment_t* segment = dlist->data;
      file_size_t ssize;

      if (segment == upload->cur_segment) {
        ssize = segment->size - buf;
      } else {
        ssize = segment->size;
      }
      x = segment->start*w/size;
      if (cu->priv->color_set[COLOR_ACTIVE]) {
        if (segment->start + ssize < segment->stop) {
          x2 = (segment->stop-1)*w/size +1;
          gdk_gc_set_rgb_fg_color(gc, &cu->priv->color[COLOR_ACTIVE]);
          gdk_draw_rectangle(window, gc, TRUE, 
                             cell_area->x + x+1, cell_area->y+1,
                             x2-x, h);
        }
      }
      if (cu->priv->color_set[COLOR_FINISHED]) {
        if (ssize > 0) {
          x2 = (segment->start+ssize-1)*w/size +1;
          gdk_gc_set_rgb_fg_color(gc, &cu->priv->color[COLOR_FINISHED]);
          gdk_draw_rectangle(window, gc, TRUE, 
                             cell_area->x + x+1, cell_area->y+1,
                             x2-x, h);
        }
      }
    }
    if (cu->priv->color_set[COLOR_PROGRESS]) {
      x = 0;
      seg = upload->cur_segment;
      if (seg->size-buf > 0) {
        x2 = (seg->size-buf-1)*w / (seg->stop - seg->start) +1;
        gdk_gc_set_rgb_fg_color(gc, &cu->priv->color[COLOR_PROGRESS]);
        gdk_draw_rectangle(window, gc, TRUE, 
                           cell_area->x + x+1, cell_area->y+1,
                           x2-x, h/4);
      }
    }
    g_object_unref(G_OBJECT(gc));
  }
}

static void
gtk_cell_renderer_upload_finalize(GObject *object) {
  GtkCellRendererUpload* cellupload = GTK_CELL_RENDERER_UPLOAD(object);

  g_free(cellupload->priv);
  
  (*G_OBJECT_CLASS(parent_class)->finalize)(object);
}
