/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.11 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>

#include <gtk/gtk.h>

#include "lcore/version.h"
#include "lcore/utils.h"
#include "lcore/mtypes.h"
#include "lcore/preferences.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "ui_about.h"
#include "ui_main.h"

static GtkWidget* HelpWin = NULL;

void on_browse(GtkAboutDialog* about, const char* link) {
  (void)about;
  mtype_browse(link);
}
void on_email(GtkAboutDialog* about, const char* link) {
  (void)about;
  mtype_mail(link);
}

void ui_about_show(GtkWindow* parent,
                   const char* name, const char* version, const char* copyright,
                   const char* license, const char* website, const char** authors,
                   const char* comments, const char* icon) {
  GtkWidget* win;
  GtkAboutDialog* aboutwin;

  gtk_about_dialog_set_url_hook
    ((GtkAboutDialogActivateLinkFunc)on_browse, NULL, NULL);
  gtk_about_dialog_set_email_hook
    ((GtkAboutDialogActivateLinkFunc)on_email, NULL, NULL);

  win = gtk_about_dialog_new();
  aboutwin = GTK_ABOUT_DIALOG(win);
  if (name)
    gtk_about_dialog_set_program_name(aboutwin, name);
  if (version)
    gtk_about_dialog_set_version(aboutwin, version);
  if (copyright)
    gtk_about_dialog_set_copyright(aboutwin, copyright);
  if (license)
    gtk_about_dialog_set_license(aboutwin, license);
  if (website)
    gtk_about_dialog_set_website(aboutwin, website);
  if (authors)
    gtk_about_dialog_set_authors(aboutwin, authors);
  if (comments)
    gtk_about_dialog_set_comments(aboutwin, comments);
  if (icon)
    gtk_about_dialog_set_logo(aboutwin, create_pixbuf(icon));

  gtk_about_dialog_set_translator_credits(aboutwin, _("translator-credits"));

  g_signal_connect_swapped(win, "response", G_CALLBACK(gtk_widget_destroy), win);

  if (parent)
  {
      gtk_window_set_transient_for(GTK_WINDOW(win), parent);
      gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
  }

  gtk_widget_show(win);
}


void ui_about_help()
{
    GtkWidget* win;
    GtkWidget* vbox;
    GtkWidget* hbox;
    GtkWidget* hhbox;
    GtkWidget* item;
    GtkWidget* label;
    GtkWidget* swindow;

    if (HelpWin) {
        gtk_window_present(GTK_WINDOW(HelpWin));
        return;
    }
  
    HelpWin = win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    g_signal_connect_swapped(G_OBJECT(win), "destroy",
                             G_CALLBACK(g_nullify_pointer), &HelpWin);
  
    gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
    gtk_window_set_title(GTK_WINDOW(win), _("Help"));
    gtk_window_set_role(GTK_WINDOW(win), "Help");
    gtk_window_set_default_size(GTK_WINDOW(win), 500, 400);
/*   gtk_window_set_resizable(GTK_WINDOW(win), TRUE); */


    vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
    gtk_container_add(GTK_CONTAINER(win), vbox);

    swindow = gtk_scrolled_window_new (NULL, NULL);
    gtk_box_pack_start(GTK_BOX(vbox), swindow, TRUE, TRUE, 0);
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(swindow), GTK_SHADOW_NONE);

    item = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

    hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

    item = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
    gtk_container_add(GTK_CONTAINER(hbox), item);
    g_signal_connect_swapped(G_OBJECT(item), "clicked",
                             G_CALLBACK(gtk_widget_destroy), win);
  
    vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
    gtk_scrolled_window_add_with_viewport
        (GTK_SCROLLED_WINDOW(swindow), GTK_WIDGET(vbox));
    gtk_viewport_set_shadow_type
        (GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(swindow))), GTK_SHADOW_NONE);


    label = gtk_label_new(_("This is a beta version! Expect the unexpected. --sgop"));
    gtk_widget_set_name(label, "LightLabel");
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);


    // getting help
    item = ui_create_section(_("Getting help"), NULL, &hbox, &hhbox);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
  
    label = gtk_label_new
        (_("Get help at one of the forums. Other sources might follow in the future."));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

    label = ui_create_url_label
        (_("visit forums"), "http://lopster.sourceforge.net/board/");
    gtk_box_pack_end(GTK_BOX(hhbox), label, FALSE, FALSE, 0);



    // bug reports
    item = ui_create_section(_("Bugs"), NULL, &hbox, &hhbox);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
  
    label = gtk_label_new
        (_("Report any bug you find and attach a callstack if possible. "
           "Please first take a few minutes to evaluate whether the bug was reported earlier."));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

    label = ui_create_url_label
        (_("report a bug"), "http://sourceforge.net/tracker/?group_id=8472");
    gtk_box_pack_end(GTK_BOX(hhbox), label, FALSE, FALSE, 0);


    // feature requests
    item = ui_create_section(_("Feature requests"), NULL, &hbox, &hhbox);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
  
    label = gtk_label_new
        (_("If you are missing a feature don't hesitate to ask for it, "
           "but please note that this software is still alpha and a lot of things "
           "will still change. So for the moment please only request basic features."));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

    label = ui_create_url_label
        (_("request a feature"), "http://sourceforge.net/tracker/?group_id=8472");
    gtk_box_pack_end(GTK_BOX(hhbox), label, FALSE, FALSE, 0);

  
    // 
    item = ui_create_section(_("Contribute to Lopster"), NULL, &hbox, &hhbox);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
  
    label = gtk_label_new
        (_("Everyone who wants to participate in the development of Lopster is welcome "
           "to join the discussion at Lopster's forums. There you can actively influence "
           "the direction Lopster goes."));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

    label = ui_create_url_label
        (_("join discussion"), "http://lopster.sourceforge.net/board/");
    gtk_box_pack_end(GTK_BOX(hhbox), label, FALSE, FALSE, 0);

  
    if (ui_main_get_win())
    {
        gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
        gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
    }

    gtk_widget_show_all(win);
}
