/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.10 $
 * $Date: 2006/10/29 00:52:56 $
 * $Author: sgop $
 */

#include <gtk/gtk.h>

#include "ui_filetree.h"
#include "ui_buddy.h"
#include "ui_main.h"
#include "ui_support.h"
#include "ui_layout.h"
#include "ui_treeview.h"
#include "lcore/browse.h"
#include "lcore/download.h"
#include "lcore/l_i18n.h"

static void ui_browse_hide(browse_t* browse)
{
    tree_gui_t* data = object_get_data(OBJ(browse), "FileTreePage");
    if (!data) return;

    // disconnect guidata from gui
    ui_filetree_assign_new_tree(data, NULL);

    ui_layout_unmonitor("core:browse:paned:pos", data->paned);
    ui_layout_unmonitor("core:browse:col:filename:size", data->col_file[0]);
    ui_layout_unmonitor("core:browse:col:filename:vis", data->col_file[0]);
    ui_layout_unmonitor("core:browse:col:filesize:size", data->col_file[1]);
    ui_layout_unmonitor("core:browse:col:filesize:vis", data->col_file[1]);
    ui_layout_unmonitor("core:browse:col:meta:size", data->col_file[2]);
    ui_layout_unmonitor("core:browse:col:meta:vis", data->col_file[2]);
    
    ui_buddy_remove_subpage(data->main_link);

    g_free(data->search_token);
    data->search_token = NULL;

    // free guidata
    g_free(data);
}

void ui_browse_show(browse_t* browse)
{
    GtkWidget* hpaned;
    GtkWidget* item;
    GtkWidget* hbox;
    tree_gui_t* data;
    const char* str;
    char* str2;
    
    if (object_get_data(OBJ(browse), "FileTreePage")) return;
    data = g_malloc0(sizeof(*data));
    
    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    
    item = ui_create_close_button(_("close this page"));
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    g_signal_connect_swapped(G_OBJECT(item), "clicked",
                             (GCallback)object_kill, browse);
    
    str = ui_main_get_plugin_stock(browse->user?browse->user->ID:-1);
    if (str)
    {
        item = gtk_image_new_from_stock(str, GTK_ICON_SIZE_MENU);
        gtk_widget_show(item);
        gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    }
    str2 = g_markup_printf_escaped(_("<b>Browsing %s</b>"), OBJ(browse)->name);
    item = gtk_label_new(str2);
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    g_free(str2);
    
    hpaned = ui_filetree_create_page(data, GTK_BOX(hbox), NULL);
    
    ui_layout_monitor("core:browse:paned:pos", data->paned);
    ui_layout_monitor("core:browse:col:filename:size", data->col_file[0]);
    ui_layout_monitor("core:browse:col:filename:vis", data->col_file[0]);
    ui_layout_monitor("core:browse:col:filesize:size", data->col_file[1]);
    ui_layout_monitor("core:browse:col:filesize:vis", data->col_file[1]);
    ui_layout_monitor("core:browse:col:meta:size", data->col_file[2]);
    ui_layout_monitor("core:browse:col:meta:vis", data->col_file[2]);
    
    ui_filetree_assign_new_tree(data, FILE_TREE(browse));
    
    {
        action_item_t* actions =
            action_item_search("Browse file actions");
        ui_tv_handle_button(GTK_TREE_VIEW(data->file_view),
                            1, &actions, browse);
    }
    
    ui_buddy_add_subpage(data->main_link);
    
    object_notify_d(OBJ(browse), (ObjectFunc)ui_browse_hide, NULL);
}
