/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.19 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#include <gtk/gtk.h>

#include "lcore/plugin.h"
#include "lcore/l_i18n.h"
#include "ui_buddy.h"
#include "ui_main.h"
#include "ui_support.h"
#include "ui_filetree.h"
#include "ui_layout.h"
#include "ui_preferences.h"
#include "ui_treeview.h"
#include "ui_tooltips.h"
#ifdef WIN32
#include "lgtk2corealias.h"
#endif

#define UI_STORE_ONLINE   0
#define UI_STORE_OFFLINE  1
#define UI_STORE_BROWSE   2

static GtkNotebook* Notebook    = NULL;
static GtkBox*      BuddyAddBox = NULL;
static GtkWidget*   EditWin     = NULL;
static GtkEntry*    BuddyEntry  = NULL;
static GtkEntry*    BuddyEntry2 = NULL;
static GtkWidget*   InfoPage    = NULL;
static GtkBox*  BuddyInfoParent = NULL;
static GList*   ActiveContents  = NULL;

static const char* ListLabel[3] = {
  N_("Online buddies"), 
  N_("Offline buddies"),
  N_("Browsed users")
};
static const char* ListLayout[3] = {
  "core:buddy:online:exp", 
  "core:buddy:offline:exp",
  "core:buddy:browses:exp"
};

typedef struct {
  int ID;
  BuddyEditFunc edit_func;
  BuddyApplyFunc apply_func;
  BuddyShowFunc show_func;
} buddy_funcs_t;

typedef struct {
  void* object;
  GtkWidget* main;
  GtkWidget* sub;
  GtkWidget* label;
} ui_info_t;


static GList* BuddyFuncs = NULL;

static GtkListStore* ui_buddy_get_store(int which)
{
    static GtkListStore* OnlineStore  = NULL;
    static GtkListStore* OfflineStore = NULL;
    static GtkListStore* BrowseStore  = NULL;
    switch (which) {
    case UI_STORE_ONLINE:
        if (!OnlineStore)
        {
            OnlineStore = gtk_list_store_new
                (2, G_TYPE_POINTER, G_TYPE_STRING);
            gtk_tree_sortable_set_sort_column_id
                (GTK_TREE_SORTABLE(OnlineStore), 1, GTK_SORT_ASCENDING);
        }
        return OnlineStore;
    case UI_STORE_OFFLINE:
        if (!OfflineStore)
        {
            OfflineStore = gtk_list_store_new
                (2, G_TYPE_POINTER, G_TYPE_STRING);
            gtk_tree_sortable_set_sort_column_id
                (GTK_TREE_SORTABLE(OfflineStore), 1, GTK_SORT_ASCENDING);
        }
        return OfflineStore;
    case UI_STORE_BROWSE:
        if (!BrowseStore)
        {
            BrowseStore = gtk_list_store_new
                (2, G_TYPE_POINTER, G_TYPE_STRING);
            gtk_tree_sortable_set_sort_column_id
                (GTK_TREE_SORTABLE(BrowseStore), 1, GTK_SORT_ASCENDING);
        }
        return BrowseStore;
    default:
        g_assert(FALSE);
        return NULL;
    }
}

void ui_buddy_register_funcs(int ID,
                             BuddyEditFunc edit_func,
                             BuddyApplyFunc apply_func,
                             BuddyShowFunc show_func
                             ) {
  buddy_funcs_t* funcs;

  g_assert(edit_func && apply_func && show_func);
  g_assert(ID >= 0 && ID < plugin_get_number());

  g_log("GTK_BUDDY", G_LOG_LEVEL_DEBUG,
        "registering buddy funcs for '%s'", plugin_get_name(plugin_get(ID)));

  funcs = g_malloc(sizeof(*funcs));
  funcs->ID = ID;
  funcs->edit_func = edit_func;
  funcs->apply_func = apply_func;
  funcs->show_func = show_func;
  BuddyFuncs = g_list_append(BuddyFuncs, funcs);
}

static void on_apply_buddy(buddy_t* buddy) {
  const char* text;
  GList* dlist;

  g_assert(EditWin && BuddyEntry);

  text = gtk_entry_get_text(BuddyEntry);
  if (!text || *text == 0) return;

  if (!buddy) {
    buddy = buddy_new(text);
  } else {
    object_set_name(OBJ(buddy), text);
  }

  text = gtk_entry_get_text(BuddyEntry2);
  if (text && *text)
    buddy_set_description(buddy, text);
  else
    buddy_set_description(buddy, NULL);

  for (dlist = BuddyFuncs; dlist; dlist = dlist->next) {
    buddy_funcs_t* funcs = dlist->data;
    funcs->apply_func(buddy);
  }

  buddy_propagate_update(buddy);

  gtk_widget_destroy(EditWin);
  EditWin = NULL;
}

static void on_edit_buddy(buddy_t* buddy)
{
    GtkWidget* win;
    GtkWidget* vbox;
    GtkWidget* vbox2;
    GtkWidget* hbox;
    GtkWidget* button;
    GtkWidget* entry;
    GtkWidget* swindow;
    GList* dlist;
    GtkWidget* table;
    GtkWidget* item;

    if (BuddyAddBox) return;  // already editing a buddy

    win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    if (buddy)
        gtk_window_set_title(GTK_WINDOW (win), _("Edit buddy"));
    else
        gtk_window_set_title(GTK_WINDOW (win), _("New buddy"));
    gtk_window_set_default_size(GTK_WINDOW(win), 500, 300);
    gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
    if (ui_main_get_win())
    {
        gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
        gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
    }

    g_signal_connect_swapped(G_OBJECT(win), "delete_event",
                             G_CALLBACK(gtk_widget_destroy), win);
  
    vbox = gtk_vbox_new(FALSE, 10);
    gtk_widget_show(vbox);
    gtk_container_add(GTK_CONTAINER(win), vbox);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);

    vbox2 = ui_create_section(_("General"), NULL, &hbox, NULL);
    gtk_widget_show(vbox2);
    gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, FALSE, 0);

    table = gtk_table_new(1, 2, FALSE);
    gtk_widget_show(table);
    gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 0);
    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  
    //
    entry = gtk_entry_new();
    if (buddy) gtk_entry_set_text(GTK_ENTRY(entry), OBJ(buddy)->name);
    BuddyEntry = GTK_ENTRY(entry);

    item = ui_create_headed_item(_("name of buddy"), entry);
    gtk_widget_show(item);
    gtk_table_attach(GTK_TABLE(table), item, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);


    //
    entry = gtk_entry_new();
    if (buddy && buddy->description)
        gtk_entry_set_text(GTK_ENTRY(entry), buddy->description);
    BuddyEntry2 = GTK_ENTRY(entry);

    item = ui_create_headed_item(_("description"), entry);
    gtk_widget_show(item);
    gtk_table_attach(GTK_TABLE(table), item, 1, 2, 0, 1,
                     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);


    swindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_box_pack_start(GTK_BOX(vbox), swindow, TRUE, TRUE, 0);
    gtk_widget_show(swindow);
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(swindow), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type
        (GTK_SCROLLED_WINDOW(swindow), GTK_SHADOW_NONE);

    vbox2 = gtk_vbox_new(FALSE, 5);
    gtk_widget_show(vbox2);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swindow), vbox2);
    gtk_viewport_set_shadow_type
        (GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(swindow))), GTK_SHADOW_NONE);

    // FIXME: add monitor to destroy the window if the buddy gets deleted
    BuddyAddBox = GTK_BOX(vbox2);
    g_signal_connect(G_OBJECT(vbox2), "destroy",
                     G_CALLBACK(gtk_widget_destroyed), &BuddyAddBox);

    hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_container_add(GTK_CONTAINER(hbox), button);
    gtk_widget_show(button);
    g_signal_connect_swapped(G_OBJECT(button), "clicked",
                             G_CALLBACK(gtk_widget_destroy), win);
  
    button = gtk_button_new_from_stock(GTK_STOCK_OK);
    gtk_container_add(GTK_CONTAINER(hbox), button);
    gtk_widget_show(button);
    g_signal_connect_swapped(G_OBJECT(button), "clicked",
                             G_CALLBACK(on_apply_buddy), buddy);

    EditWin = win;

    for (dlist = BuddyFuncs; dlist; dlist = dlist->next) {
        buddy_funcs_t* funcs = dlist->data;
        GtkWidget* widget;

        widget = funcs->edit_func(buddy);
        if (!widget) continue;
    
        vbox = ui_create_section(plugin_get_name(plugin_get(funcs->ID)),
                                 NULL, &hbox, NULL);
        gtk_widget_show(vbox);
        gtk_box_pack_start(BuddyAddBox, vbox, FALSE, FALSE, 0);

        gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0);
    }
  
    gtk_widget_show(win);
}

static ui_info_t* ui_info_search(void* object) {
  GList* dlist;
  
  for (dlist = ActiveContents; dlist; dlist = dlist->next) {
    ui_info_t* info = dlist->data;
    if (info->object == object) return info;
  }
  return NULL;
}

/* static gboolean on_show_hide(ui_info_t* info) { */
/*   if (GTK_WIDGET_VISIBLE(info->sub)) { */
/*     gtk_widget_hide(info->sub); */
/*   } else { */
/*     gtk_widget_show(info->sub); */
/*   } */
/*   return TRUE; */
/* } */

static void ui_info_show_only(ui_info_t* info) {
  GList* dlist;
  
  for (dlist = ActiveContents; dlist; dlist = dlist->next) {
    ui_info_t* i = dlist->data;
    if (i == info) 
      gtk_widget_show(info->sub);
    else if (GTK_WIDGET_VISIBLE(i->sub))
      gtk_widget_hide(i->sub);
  }
}

static gboolean on_info_hide(ui_info_t* info) {
  ActiveContents = g_list_remove(ActiveContents, info);
  gtk_widget_destroy(info->main);
  g_free(info);
  return TRUE;
}

static ui_info_t* ui_info_add(void* object, const char* type, const char* lab, ...) {
  ui_info_t* info;
  GtkWidget* vbox0;
  GtkWidget* hbox;
  GtkWidget* label;
  GtkWidget* item;

  va_list ap;
  char* text;

  info = g_malloc0(sizeof(*info));
  info->object = object;
  ActiveContents = g_list_append(ActiveContents, info);
  info->main = vbox0 = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox0);
  gtk_box_pack_start(BuddyInfoParent, vbox0, FALSE, FALSE, 0);
    
  item = gtk_event_box_new();
  gtk_widget_show(item);
  gtk_widget_set_name(item, "HeaderBox");
  gtk_box_pack_start(GTK_BOX(vbox0), item, FALSE, FALSE, 3);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_container_add(GTK_CONTAINER(item), hbox);
/*   gtk_box_pack_start(GTK_BOX(vbox0), hbox, FALSE, FALSE, 3); */
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 1);

  item = ui_create_close_button(_("Hide"));
  gtk_widget_show(item);
  g_signal_connect_swapped(G_OBJECT(item), "clicked",
                           G_CALLBACK(on_info_hide), info);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);


/*   label = gtk_label_new(type); */
/*   gtk_widget_show(label); */
/*   gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0); */
/*   gtk_misc_set_padding(GTK_MISC(label), 5, 0); */
  
  info->label = label = gtk_label_new(l_printf("<b>%s</b>", lab));
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

  va_start(ap, lab);
  while ((text = va_arg(ap, char*)) != NULL) {
    GCallback cb = va_arg(ap, void*);
    label = ui_create_event_label(text, cb, object, TRUE);
    gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  }
  va_end(ap);

  return info;
}

static void ui_buddy_show_details(buddy_t* buddy, gboolean hide) {
  GtkWidget* vbox0;
  GtkWidget* vbox;
  GtkWidget* label;
  GList* dlist;
  ui_info_t* info;
  gboolean shown = TRUE;

  if (!BuddyInfoParent) return;
  info = ui_info_search(buddy);
  
  if (!info) {
    info = ui_info_add(buddy, _("Buddy"), OBJ(buddy)->name, 
                       _("delete"), G_CALLBACK(object_kill),
                       _("edit"), G_CALLBACK(on_edit_buddy),
                       NULL);
  } else {
    if (!GTK_WIDGET_VISIBLE(info->sub)) shown = FALSE;
    gtk_widget_destroy(info->sub);
  }
  vbox0 = info->main;

  if (buddy->online > 0)
    gtk_widget_set_name(info->label, "LightLabel");
  else
    gtk_widget_set_name(info->label, NULL);
  
  info->sub = vbox = gtk_vbox_new(FALSE, 5);
  if (shown) gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(vbox0), vbox, FALSE, FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  
  if (buddy->description) {
    label = gtk_label_new(buddy->description);
    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
  }


  for (dlist = BuddyFuncs; dlist; dlist = dlist->next) {
    buddy_funcs_t* funcs = dlist->data;
    GtkWidget* frame;
    GtkWidget* widget;
    GtkWidget* hbox;

    widget = funcs->show_func(buddy);
    if (!widget) continue;
    
    frame = ui_create_section
      (plugin_get_name(plugin_get(funcs->ID)), NULL, &hbox, NULL);
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
    gtk_widget_show(widget);
    gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 0);
  }
  if (hide) ui_info_show_only(info);
}

static void ui_buddy_switch_subpage(GtkWidget* main) {
  int index = gtk_notebook_page_num(Notebook, main);
  if (index >= 0) gtk_notebook_set_current_page(Notebook, index);
}

static gboolean on_buddy_button(GtkTreeView *tv, GdkEventButton *event) {
  GtkWidget *pop;
  GtkTreePath* path;
  GtkTreeIter iter;
  buddy_t* buddy;
  GtkTreeModel* model;
  GtkTreeSelection* select;

  if (event->window != gtk_tree_view_get_bin_window(tv))
    return FALSE;

  if (event->type != GDK_BUTTON_PRESS) return FALSE;

  model = gtk_tree_view_get_model(tv);

  if (!gtk_tree_view_get_path_at_pos(tv, event->x, event->y,
				     &path, NULL, NULL, NULL)) return FALSE;
  pop = NULL;
  
  select = gtk_tree_view_get_selection(tv);
  gtk_tree_model_get_iter(model, &iter, path);
  gtk_tree_path_free(path);
  gtk_tree_model_get(model, &iter, 0, &buddy, -1);
  gtk_tree_selection_select_iter(select, &iter);

  if (event->button == 1) {
    if (buddy) {
      ui_buddy_show_details(buddy, FALSE/*TRUE*/);
      ui_buddy_switch_subpage(InfoPage);
    }
    return TRUE;
  }
  return FALSE;
}

void ui_buddy_add_subpage(GtkWidget* main) {
  gtk_notebook_set_current_page
    (Notebook, gtk_notebook_append_page(Notebook, main, NULL));
}

void ui_buddy_remove_subpage(GtkWidget* main) {
  int index = gtk_notebook_page_num(Notebook, main);
  if (index != -1) gtk_notebook_remove_page(Notebook, index);
}

static gboolean on_browsed_button(GtkTreeView *tv, GdkEventButton *event)
{
    GtkWidget* pop;
    GtkTreePath* path;
    GtkTreeIter iter;
    browse_t* browse;
    GtkTreeModel* model;
    GtkTreeSelection* select;
    
    if (event->window != gtk_tree_view_get_bin_window(tv))
        return FALSE;
    
    if (event->type != GDK_BUTTON_PRESS) return FALSE;
    
    model = gtk_tree_view_get_model(tv);
    
    if (!gtk_tree_view_get_path_at_pos(tv, event->x, event->y,
                                       &path, NULL, NULL, NULL)) return FALSE;
    pop = NULL;
    
    select = gtk_tree_view_get_selection(tv);
    gtk_tree_model_get_iter(model, &iter, path);
    gtk_tree_path_free(path);
    gtk_tree_model_get(model, &iter, 0, &browse, -1);
    gtk_tree_selection_select_iter(select, &iter);
    
    if (event->button == 1)
    {
        tree_gui_t* d = object_get_data(OBJ(browse), "FileTreePage");
        if (d) ui_buddy_switch_subpage(d->main_link);
        return TRUE;
    }
    return FALSE;
}


static void on_expander_changed(GtkExpander* exp) {
  gboolean expand = gtk_expander_get_expanded(exp);
  GtkWidget* parent = gtk_widget_get_parent(GTK_WIDGET(exp));

  // why do we get TRUE if the epxander just got collapsed?
  expand = !expand;

  gtk_box_set_child_packing(GTK_BOX(parent), GTK_WIDGET(exp),
                            expand, expand, 0, GTK_PACK_START);
}

static GtkWidget* on_buddy_tip(buddy_t* buddy, unsigned depth) {
  GtkWidget* table;
  int line = 0;
  GList* dlist;

  (void)depth;

  table = gtk_table_new(1, 2, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);

  ui_table_add_line(GTK_TABLE(table), &line, _("Buddy"), OBJ(buddy)->name);

  if (buddy->description)
    ui_table_add_line(GTK_TABLE(table), &line, _("Description"), buddy->description);

  ui_table_add_sep(GTK_TABLE(table), &line);

  for (dlist = buddy->users; dlist; dlist = dlist->next) {
    user_t* user = dlist->data;
    plugin_t* plugin = plugin_get(user->ID);
    GtkWidget* label1 = gtk_label_new(NULL);
    GtkWidget* label2 = gtk_label_new(user->print(user));

    gtk_label_set_text(GTK_LABEL(label1), 
                       l_printf("%s:", plugin_get_name(plugin)));
    gtk_misc_set_alignment(GTK_MISC(label1), 1, 0.5);
    gtk_misc_set_alignment(GTK_MISC(label2), 0, 0.5);
    gtk_widget_set_name(label2, "OnlineLabel");
    if (user->online) gtk_widget_set_state(label2, GTK_STATE_ACTIVE);
    else gtk_widget_set_state(label2, GTK_STATE_NORMAL);
    ui_table_add_item(GTK_TABLE(table), &line, label1, label2);
    //FIXME: we could display more info
  }
  gtk_widget_show(table);
  return table;
}

static GtkWidget* _create_buddy_list(unsigned index, GCallback func) {
  GtkWidget* exp;
  GtkWidget* sw;
  GtkListStore* store;
  GtkWidget* tv;
  GtkTreeViewColumn* column;

  exp = gtk_expander_new(l_printf("<b>%s</b>", _(ListLabel[index])));
  gtk_expander_set_use_markup(GTK_EXPANDER(exp), TRUE);
  gtk_expander_set_expanded(GTK_EXPANDER(exp), TRUE);
  ui_layout_monitor(ListLayout[index], exp);
   
  g_signal_connect(G_OBJECT(exp), "activate",
                   G_CALLBACK(on_expander_changed), NULL);

  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type
     (GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
  gtk_container_add(GTK_CONTAINER(exp), sw);
  gtk_widget_show(sw);

  store = ui_buddy_get_store(index);
  tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
  gtk_container_add(GTK_CONTAINER(sw), tv);
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tv), FALSE);
  gtk_widget_show(tv);
  if (index == UI_STORE_ONLINE || index == UI_STORE_OFFLINE) {
      gtk_fancytips_new(GTK_TREE_VIEW(tv), (FancyTipsFunc)on_buddy_tip,
                        GTK_FANCY_POS_LEFT);
  }

  //  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
  //  gtk_tree_selection_set_mode (select, GTK_SELECTION_MULTIPLE);
  
  gtk_tree_sortable_set_sort_column_id
    (GTK_TREE_SORTABLE(store), 1, GTK_SORT_ASCENDING);
  
  column = ui_tv_column_new
    ("dummy", gtk_cell_renderer_text_new(), "text", 1, TRUE, NULL);
  ui_tv_config_default_col(column, 1, NULL, TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  g_signal_connect(G_OBJECT (tv), "button-press-event", func, NULL);
  return exp;
}

GtkWidget* ui_create_buddy_content() {
  GtkWidget* sw;
  GtkWidget* vbox0;
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* hpaned;
  GtkWidget* item;
  GtkWidget* notebook;


  vbox0 = gtk_vbox_new(FALSE, 5);

  vbox = gtk_vbox_new(FALSE, 5); //forward instance
  gtk_widget_show(vbox);
  ui_layout_monitor("core:buddy:sidebar:vis", vbox);
  
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox0), hbox, FALSE, FALSE, 0);

  item = gtk_image_new_from_stock(STOCK_LOPSTER_HOTLIST, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  item = ui_create_header_label(_("Buddies and Browses"));
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  
  item = ui_create_event_label
    (_("add new buddy"), G_CALLBACK(on_edit_buddy), NULL, TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  item = ui_create_show_hide_label(_("show sidebar"), _("hide sidebar"), vbox);
  gtk_widget_show(item);
  gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  item = gtk_hseparator_new();
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(vbox0), item, FALSE, FALSE, 0);


  hpaned = gtk_hpaned_new();
  gtk_widget_show(hpaned);
  ui_layout_monitor("core:buddy:paned:pos", hpaned);
  gtk_box_pack_start(GTK_BOX(vbox0), hpaned, TRUE, TRUE, 0);

  gtk_paned_pack2(GTK_PANED(hpaned), vbox, FALSE, TRUE);

  item = _create_buddy_list(0, G_CALLBACK(on_buddy_button));
  gtk_widget_show(item);
  if (gtk_expander_get_expanded(GTK_EXPANDER(item)))
    gtk_box_pack_start(GTK_BOX(vbox), item, TRUE, TRUE, 0);
  else
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  item = _create_buddy_list(1, G_CALLBACK(on_buddy_button));
  gtk_widget_show(item);
  if (gtk_expander_get_expanded(GTK_EXPANDER(item)))
    gtk_box_pack_start(GTK_BOX(vbox), item, TRUE, TRUE, 0);
  else
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  item = _create_buddy_list(2, G_CALLBACK(on_browsed_button));
  gtk_widget_show(item);
  if (gtk_expander_get_expanded(GTK_EXPANDER(item)))
    gtk_box_pack_start(GTK_BOX(vbox), item, TRUE, TRUE, 0);
  else
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  // create notebook
  notebook = gtk_notebook_new();
  gtk_widget_show(notebook);
  Notebook = GTK_NOTEBOOK(notebook);
  gtk_notebook_set_show_tabs(Notebook, FALSE);
  gtk_notebook_set_show_border(Notebook, FALSE);
  //  g_signal_connect(G_OBJECT(notebook), "switch_page",
  //		   G_CALLBACK(on_search_switch_page), select);
  gtk_paned_pack1(GTK_PANED(hpaned), notebook, TRUE, TRUE);
  

  // create static info window and add to notebook
  //FIXME: i dont like this page, think about it.
  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show(sw);
  gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_NONE);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);
  BuddyInfoParent = GTK_BOX(vbox);
  //  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);

  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), vbox);
  gtk_viewport_set_shadow_type(GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(sw))), GTK_SHADOW_NONE);


  InfoPage = sw;
  ui_buddy_add_subpage(sw);

  gtk_widget_show(vbox0);
  return vbox0;
}

static void ui_buddy_hide_info(buddy_t* buddy) {
  ui_info_t* info;

  info = ui_info_search(buddy);
  if (info) on_info_hide(info);
}

#define VALID_ITER(iter, store) (iter->user_data != NULL && store->stamp == iter->stamp)
static void ui_buddy_hide(buddy_t* buddy)
{
    GtkTreeIter* iter;
    GtkListStore* store;
    
    g_assert(buddy);
    
    iter = object_get_data(OBJ(buddy), "BuddyEntry");
    if (!iter) return;
    
    store = ui_buddy_get_store(UI_STORE_ONLINE);
    if (!VALID_ITER(iter, store)) 
        store = ui_buddy_get_store(UI_STORE_OFFLINE);
    
    gtk_list_store_remove(store, iter);
    
    object_free_data_common(OBJ(buddy), "BuddyEntry");
}


static void ui_buddy_show(buddy_t* buddy);
static void ui_buddy_update(buddy_t* buddy)
{
    GtkTreeIter* iter;
    gboolean online = FALSE;
    GtkListStore* store;
    
    g_assert(buddy);
    
    iter = object_get_data(OBJ(buddy), "BuddyEntry");
    if (!iter) return;
    
    store = ui_buddy_get_store(UI_STORE_OFFLINE);
    if (!VALID_ITER(iter, store))
    {
        store = ui_buddy_get_store(UI_STORE_ONLINE);
        online = TRUE;
    }

    if ((!online && buddy->online) || (online && !buddy->online))
    {
        // changes lists
        ui_buddy_hide(buddy);
        ui_buddy_show(buddy);
    }
    else
    {
        GtkListStore* store =
            ui_buddy_get_store(buddy->online?UI_STORE_ONLINE:UI_STORE_OFFLINE);
        // just update
        gtk_list_store_set(store, iter,
                           0, buddy,
                           1, OBJ(buddy)->name,
                           -1);
    }
    // re-display details if shown
    if (ui_info_search(buddy)) ui_buddy_show_details(buddy, FALSE);
}

static void ui_buddy_show(buddy_t* buddy)
{
    GtkTreeIter* iter;
    GtkListStore* store;
    
    g_assert(buddy);
    
    if (object_get_data(OBJ(buddy), "BuddyEntry")) return;

    object_set_data(OBJ(buddy), "BuddyEntry", iter = g_malloc(sizeof(*iter)));

    if (buddy->online) store = ui_buddy_get_store(UI_STORE_ONLINE);
    else store = ui_buddy_get_store(UI_STORE_OFFLINE);
    gtk_list_store_append(store, iter);
  
    ui_buddy_update(buddy);
}

void ui_buddy_new(buddy_t* buddy)
{
    object_notify_d(OBJ(buddy), (ObjectFunc)ui_buddy_hide_info, NULL);
    object_notify_d(OBJ(buddy), (ObjectFunc)ui_buddy_hide, NULL);
    object_monitor_d(OBJ(buddy), (ObjectFunc)ui_buddy_update, NULL);
    ui_buddy_show(buddy);
}



static void ui_browse_link_hide(browse_t* browse)
{
    GtkListStore* store = ui_buddy_get_store(UI_STORE_BROWSE);
    GtkTreeIter* iter = object_get_data(OBJ(browse), "BrowseEntry");

    if (iter)
    {
        gtk_list_store_remove(store, iter);
        object_free_data_common(OBJ(browse), "BrowseEntry");
    }
}

void ui_browse_link_new(browse_t* browse)
{
    GtkListStore* store = ui_buddy_get_store(UI_STORE_BROWSE);
    GtkTreeIter* iter;

    object_set_data(OBJ(browse), "BrowseEntry", iter = g_malloc0(sizeof(*iter)));
    
    gtk_list_store_append(store, iter);
    gtk_list_store_set(store, iter,
                       0, browse,
                       1, OBJ(browse)->name,
                       -1);
    object_notify_d(OBJ(browse), (ObjectFunc)ui_browse_link_hide, NULL);
}

#ifdef WIN32
#define _UI_BUDDY_C_
#include "lgtk2corealiasdef.c"
#endif
