/* Copyright (C) 2005-2006 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.19 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>

#include "lcore/l_time.h"
#include <gtk/gtk.h>

#include "lcore/chat.h"
#include "lcore/preferences.h"
#include "lcore/commands.h"
#include "lcore/l_base.h"
#include "lcore/utils.h"
#include "lcore/plugin.h"
#include "lcore/string_list.h"
#include "lcore/mtypes.h"
#include "lcore/l_i18n.h"
#include "ui_chat.h"
#include "ui_main.h"
#include "ui_support.h"
#include "ui_style.h"
#include "ui_notebook.h"
#ifdef WIN32
#include "lgtk2corealias.h"
#endif

static GHashTable* PrintFuncList = NULL;
static GtkTextTagTable* TagTable = NULL;
static ui_nb_t* NoteBook = NULL;
static gboolean MonitorChanges = TRUE;
static char* SPrefix = NULL;
static char* CPrefix = NULL;
static char* PPrefix = NULL;
static char* JPrefix = NULL;
static char* TPrefix = NULL;
static GtkTextTag* UrlTag = NULL;
static GtkTextTag* NickTag = NULL;
static GdkCursor* HandCursor = NULL;

static GdkCursor* ui_chat_get_hand_cursor() {
  if (!HandCursor) HandCursor = gdk_cursor_new(GDK_HAND2);
  return HandCursor;
}

void ui_chat_register_print_func(const char* name, PrintFunc func) {
  PrintFunc f2;
  g_assert(name);
  g_assert(func);

  f2 = g_hash_table_lookup(PrintFuncList, name);
  if (f2) {
    g_log("UI_CHAT", G_LOG_LEVEL_WARNING,
          "print function '%s' already defined, overriding", name);
  }
  g_hash_table_insert(PrintFuncList, g_strdup(name), func);
}

static void print_ts(GtkTextBuffer* buffer, GtkTextIter* iter)
{
#ifdef WIN32
    time_t wtime;
    struct tm* ltime;
#else
    struct tm* ltime = localtime(&CurrentTime);
#endif
    const char* res;
#ifdef WIN32
    wtime = CurrentTime;
    ltime = localtime(&wtime);
#endif
    gtk_text_buffer_insert_with_tags_by_name
        (buffer, iter, "(", 1, "c_timestamp", "t_hour", NULL);

    res = l_printf("%02d:", ltime->tm_hour);
    gtk_text_buffer_insert_with_tags_by_name
        (buffer, iter, res, -1, "c_timestamp", "t_hour", NULL);
    res = l_printf("%02d", ltime->tm_min);
    gtk_text_buffer_insert_with_tags_by_name
        (buffer, iter, res, -1, "c_timestamp", "t_min", NULL);
    res = l_printf(":%02d", ltime->tm_sec);
    gtk_text_buffer_insert_with_tags_by_name
        (buffer, iter, res, -1, "c_timestamp", "t_sec", NULL);

    gtk_text_buffer_insert_with_tags_by_name
        (buffer, iter, ")", 1, "c_timestamp", "t_hour", NULL);
}

static void
ui_chat_print_client(GtkTextBuffer* buffer, GtkTextIter* iter, va_list ap) {
  (void)ap;
  // prints timestamp and client prefix
  // no args
  print_ts(buffer, iter);
  if (CPrefix)
    gtk_text_buffer_insert_with_tags_by_name
      (buffer, iter, CPrefix, -1, "c_cprefix", NULL);
}
static void
ui_chat_print_server(GtkTextBuffer* buffer, GtkTextIter* iter, va_list ap) {
  (void)ap;
  // prints timestamp and server prefix
  // no args
  print_ts(buffer, iter);
  if (SPrefix)
    gtk_text_buffer_insert_with_tags_by_name
      (buffer, iter, SPrefix, -1, "c_sprefix", NULL);
}
static void
ui_chat_print_join(GtkTextBuffer* buffer, GtkTextIter* iter, va_list ap) {
  (void)ap;
  // prints timestamp and join prefix
  // no args
  print_ts(buffer, iter);
  if (JPrefix)
    gtk_text_buffer_insert_with_tags_by_name
      (buffer, iter, JPrefix, -1, "c_jprefix", NULL);
}
static void
ui_chat_print_part(GtkTextBuffer* buffer, GtkTextIter* iter,
		   va_list ap) {
  (void)ap;
  // prints timestamp and part prefix
  // no args
  print_ts(buffer, iter);
  if (PPrefix)
    gtk_text_buffer_insert_with_tags_by_name
      (buffer, iter, PPrefix, -1, "c_pprefix", NULL);
}
static void
ui_chat_print_other(GtkTextBuffer* buffer, GtkTextIter* iter,
		    va_list ap) {
  (void)ap;
  // prints timestamp and server prefix
  // no args
  print_ts(buffer, iter);
  if (TPrefix)
    gtk_text_buffer_insert_with_tags_by_name
      (buffer, iter, TPrefix, -1, "c_tprefix", NULL);
}
static void
ui_chat_print_proto(GtkTextBuffer* buffer, GtkTextIter* iter,
		     va_list ap) {
  char* text;

  // prints [string]
  text = va_arg(ap, void*);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, "[", 1, "c_bracket", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, text, -1, "c_text", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, "]", 1, "c_bracket", NULL);
}
static void
ui_chat_print_nl(GtkTextBuffer* buffer, GtkTextIter* iter,
		 va_list ap) {
  (void)ap;
  // prints new line
  // no args
  gtk_text_buffer_insert(buffer, iter, "\n", -1);
}
/*
static GdkColor* _get_nick_color(const char *nick, int mode) {
  static GdkColor fore;
  char* hex1 = "11223344556";
  char* hex2 = "566778899aa";
  char* hex3 = "abbccddeeff";
  char* hex;
  char ncolor[8] = "#000000";
  int i1;
  unsigned long sum = 0;
  const char* pos;

  if (!nick) return &fore;

  if (mode == 1)      hex = hex1;
  else if (mode == 2) hex = hex2;
  else if (mode == 3) hex = hex3;
  else                hex = hex2;

  pos = nick;
  while (*pos) {
    if (sum) sum *= *pos;
    else sum = *pos;
    pos++;
  }
  for (i1 = 1; i1 < 7; i1++) {
    ncolor[i1] = hex[sum % 11];
    sum /= 11;
  }

  gdk_color_parse(ncolor, &fore);

  return &fore;
}
*/
static GdkColor* _get_nick_color(const char *nick, int mode) {
  guint hash = g_str_hash(l_printf("%sabcdefg", nick?nick:""));
  static GdkColor fore;
  char* hex1 = "01234567";
  char* hex2 = "456789ab";
  char* hex3 = "89abcdef";
  char* hex;
  char ncolor[8] = "#000000";
  int i1;

  if (mode == 1)      hex = hex1;
  else if (mode == 2) hex = hex2;
  else if (mode == 3) hex = hex3;
  else                hex = hex2;

  for (i1 = 1; i1 < 7; i1++) {
    ncolor[i1] = hex[hash%8];
    hash >>= 3;
  }

  gdk_color_parse(ncolor, &fore);

  return &fore;
}
static void _set_colored(GtkTextTag* tag, int* setting) {
  if (!strncmp(tag->name, "n_", 2)) {
    if (*setting) {
      GdkColor* color = _get_nick_color(tag->name+2, *setting);
      g_object_set(G_OBJECT(tag), "foreground-gdk", color, NULL);
    } else {
      g_object_set(G_OBJECT(tag), "foreground-set", FALSE, NULL);
    }
  }
}
static const char* _get_nick_tag(GtkTextBuffer* buffer, 
				 const char* nick) {
  GtkTextTagTable* table = gtk_text_buffer_get_tag_table(buffer);
  const char* tag_name = l_printf("n_%s", nick);
  GtkTextTag* tag = gtk_text_tag_table_lookup(table, tag_name);
  if (!tag) {
    int setting = pref_get_ref("/core/chat/colored_nicks", int);
    tag = gtk_text_tag_new(tag_name);
    gtk_text_tag_table_add(TagTable, GTK_TEXT_TAG(tag));
    _set_colored(tag, &setting);
  }
  return tag_name;
}
static void
ui_chat_print_nick_text(GtkTextBuffer* buffer, GtkTextIter* iter,
			va_list ap) {
  char* nick = va_arg(ap, char*);
  char* text = va_arg(ap, char*);
  // prints text in nick color, nick is not printed
  // char* nick, char* text
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, text, -1, "c_nick", _get_nick_tag(buffer, nick), NULL);
}
static void
ui_chat_print_nick(GtkTextBuffer* buffer, GtkTextIter* iter,
		   va_list ap) {
  char* nick;
  char* text;
  // prints nick in nick color, text is 2 bytes with brackets
  // char* nick, char* text
  nick = va_arg(ap, char*);
  text = va_arg(ap, char*);
  if (text && text[0])
    gtk_text_buffer_insert_with_tags_by_name
      (buffer, iter, text, 1, "c_bracket", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, nick, -1, "c_nick", _get_nick_tag(buffer, nick), NULL);
  if (text && text[1])
    gtk_text_buffer_insert_with_tags_by_name
      (buffer, iter, text+1, 1, "c_bracket", NULL);
}
static void
ui_chat_print_url(GtkTextBuffer* buffer, GtkTextIter* iter, va_list ap) {
  char* text;
  // prints an url
  text = va_arg(ap, char*);  // FIXME
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, text, -1, "c_url", NULL);
}

void ui_chat_register_tag(GtkTextTag* tag) {
  if (tag->name && !strncmp(tag->name, "n_", 2)) {
    g_log("UI_CHAT", G_LOG_LEVEL_WARNING,
          "tags may not start with 'n_'");
    return;
  }
  gtk_text_tag_table_add(TagTable, GTK_TEXT_TAG(tag));
}

static void on_style_changed(const char* style, GtkTextTag* tag) {
  if (ui_style_sets(style, "Text")) {
    g_object_set(G_OBJECT(tag), "foreground-gdk",
                 ui_style_get_color(style, "Text"), NULL);
  } else {
    g_object_set(G_OBJECT(tag), "foreground-set", FALSE, NULL);
  }
  if (ui_style_sets(style, "Background")) {
    g_object_set(G_OBJECT(tag), "background-gdk",
                 ui_style_get_color(style, "Background"), NULL);
  } else {
    g_object_set(G_OBJECT(tag), "background-set", FALSE, NULL);
  }
  if (ui_style_sets(style, "Font")) {
    g_object_set(G_OBJECT(tag), "font",
                 ui_style_get_font(style, "Font"), NULL);
  } else {
    g_object_set(G_OBJECT(tag),
                 "family-set", FALSE, "size-set", FALSE,
                 "variant-set", FALSE, "weight-set", FALSE,
                 "style-set", FALSE, NULL);
  }
  if (ui_style_sets(style, "Underline")) {
    g_object_set(G_OBJECT(tag), "underline", 
                 ui_style_get_underline(style, "Underline"), NULL);
  } else {
    g_object_set(G_OBJECT(tag), "underline-set", FALSE, NULL);
  }
}

static void on_prefix_changed(pref_node_t* node, char** prefix) {
  g_free(*prefix);
  *prefix = g_strdup((char*)(node->value));
}
GtkTextTag* ui_chat_register_tag_from_style(const char* name, const char* style) {
  GtkTextTag* tag = gtk_text_tag_new(name);
  //  g_object_set(G_OBJECT(tag), "family", "alexandria", NULL);
  // apply style
  on_style_changed(style, tag);
  // register the tag.
  ui_chat_register_tag(tag);
  // monitor style
  ui_style_observe(style, (StyleFunc)on_style_changed, tag);

  return tag;
}

static void on_timestamp_changed(pref_node_t* node, GtkTextTag* tag) {
  int* value = node->value;
  gboolean hour = (*value == TIME_HOUR_MIN || *value == TIME_HOUR_MIN_SEC);
  gboolean min = (*value == TIME_HOUR_MIN || *value == TIME_HOUR_MIN_SEC);
  gboolean sec = (*value == TIME_HOUR_MIN_SEC);

  if (!strcmp(tag->name, "t_hour")) {
    g_object_set(G_OBJECT(tag), "invisible-set", !hour, NULL);
  } else if (!strcmp(tag->name, "t_min")) {
    g_object_set(G_OBJECT(tag), "invisible-set", !min, NULL);
  } else if (!strcmp(tag->name, "t_sec")) {
    g_object_set(G_OBJECT(tag), "invisible-set", !sec, NULL);
  }
}
static GtkTextTag*
ui_chat_register_timestamp_tag(const char* name, const char* pref) {
  GtkTextTag* tag = gtk_text_tag_new(name);

  g_object_set(G_OBJECT(tag), "invisible", TRUE, NULL);
  ui_chat_register_tag(tag);
  on_timestamp_changed(pref_search(pref, TRUE), tag);
  pref_observe(pref, (ObjectFunc)on_timestamp_changed, tag);
  
  return tag;
}

static void on_colored_changed(pref_node_t* node) {
  int* setting = node->value;
  gtk_text_tag_table_foreach
    (TagTable, (GtkTextTagTableForeach)_set_colored, setting);
}

static char*
ui_get_tag_bounds(GtkTextBuffer* buffer, GtkTextIter* iter, GtkTextTag* tag) {
  GtkTextIter s_tag = *iter;
  GtkTextIter e_tag = *iter;
  
  while (!gtk_text_iter_begins_tag(&s_tag, tag))
    gtk_text_iter_backward_char(&s_tag);
  while (!gtk_text_iter_ends_tag(&e_tag, tag))
    gtk_text_iter_forward_char(&e_tag);

  return gtk_text_buffer_get_text(buffer, &s_tag, &e_tag, FALSE);
}

static gboolean
on_url_event(GtkTextTag* tag, GtkTextView* view, GdkEvent* event, GtkTextIter* iter) {
  GtkTextBuffer* buffer = gtk_text_iter_get_buffer(iter);
  
  (void)view;
	if (event->type == GDK_BUTTON_RELEASE) {
    GdkEventButton* bevent = (GdkEventButton*)event;
		if (bevent->button == 1 || bevent->button == 2) {
			GtkTextIter s_iter, e_iter;
			if (!gtk_text_buffer_get_selection_bounds(buffer, &s_iter, &e_iter)) {
        char* text = ui_get_tag_bounds(buffer, iter, tag);
        mtype_browse(text);
        g_free(text);
      }
    } else if (bevent->button == 3) {
      // popup menu?
    }
  }
  return FALSE;
}


void ui_chat_init() {
  
  if (!TagTable) {
    PrintFuncList = g_hash_table_new
      ((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal);

    // the colors used by the gtk text tags (default colors)
    ui_style_register(N_("Style settings for chat pages"), "chat:timestamp",
                      N_("Chat timestamps"),
                      N_("Text"), STYLE_COLOR, TRUE, 0xde06, 0x9032, 0x7dd7,
                      N_("Background"), STYLE_COLOR, TRUE, 0xfdf3, 0xefb8, 0xe724,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:text",
                      N_("Normal chat text"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x75aa, 0x75aa, 0x75aa,
                      N_("Background"), STYLE_COLOR, FALSE, 0x0000, 0x0000, 0x0000,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:private",
                      N_("Private chat text"),
                      N_("Text"), STYLE_COLOR, FALSE, 0x6b5a, 0x6b5a, 0x6b5a,
                      N_("Background"), STYLE_COLOR, TRUE, 0xffff, 0xbee6, 0x9e59,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:nick",
                      N_("Style for nicknames"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x0000, 0x5555, 0xaaaa,
                      N_("Background"), STYLE_COLOR, FALSE, 0x0000, 0x0000, 0x0000,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:client",
                      N_("Client messages"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x3dc4, 0x753a, 0x73e6,
                      N_("Background"), STYLE_COLOR, FALSE, 0x0000, 0x0000, 0x0000,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:server",
                      N_("Server messages"),
                      N_("Text"), STYLE_COLOR, TRUE, 0xa757, 0x37b4, 0x34e7,
                      N_("Background"), STYLE_COLOR, FALSE, 0x0000, 0x0000, 0x0000,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:cprefix",
                      N_("Prefix for client messages"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x4b1d, 0x7f52, 0xcdc5,
                      N_("Background"), STYLE_COLOR, TRUE, 0x4b1d, 0x7f52, 0xcdc5,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:sprefix",
                      N_("Prefix for server messages"),
                      N_("Text"), STYLE_COLOR, TRUE, 0xf2d1, 0x4722, 0x2c9e,
                      N_("Background"), STYLE_COLOR, TRUE, 0xf2d1, 0x4722, 0x2c9e,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:jprefix",
                      N_("Prefix for join messages"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x439c, 0xa85b, 0x9c64,
                      N_("Background"), STYLE_COLOR, TRUE, 0x439c, 0xa85b, 0x9c64,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:pprefix",
                      N_("Prefix for part messages"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x439c, 0xa85b, 0x9c64,
                      N_("Background"), STYLE_COLOR, TRUE, 0x439c, 0xa85b, 0x9c64,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:tprefix",
                      N_("Prefix for normal chat lines"),
                      N_("Text"), STYLE_COLOR, TRUE, 0xfdf3, 0xefb8, 0xe724,
                      N_("Background"), STYLE_COLOR, TRUE, 0xfdf3, 0xefb8, 0xe724,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:url",
                      N_("Urls in chat messages"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x0000, 0x7ad0, 0xb6d8,
                      N_("Background"), STYLE_COLOR, FALSE, 0x0000, 0x0000, 0x0000,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, TRUE, TRUE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:bracket",
                      N_("Style of chat brackets"),
                      N_("Text"), STYLE_COLOR, TRUE, 0xe94f, 0x5097, 0x0000,
                      N_("Background"), STYLE_COLOR, FALSE, 0x0000, 0x0000, 0x0000,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:join",
                      N_("Channel join messages"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x439c, 0xa85b, 0x9c64,
                      N_("Background"), STYLE_COLOR, FALSE, 0x0000, 0x0000, 0x0000,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:part",
                      N_("Channel part messages"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x439c, 0xa85b, 0x9c64,
                      N_("Background"), STYLE_COLOR, FALSE, 0x0000, 0x0000, 0x0000,
                      N_("Font"), STYLE_FONT, FALSE, NULL,
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      NULL);
    ui_style_register(N_("Style settings for chat pages"), "chat:hilight",
                      N_("Hilighted text"),
                      N_("Text"), STYLE_COLOR, TRUE, 0x0000, 0x0000, 0x0000,
                      N_("Background"), STYLE_COLOR, FALSE, 0x0000, 0x0000, 0x0000,
                      N_("Font"), STYLE_FONT, TRUE, "Luxi Mono Bold 10",
                      N_("Underline"), STYLE_UNDERLINE, FALSE, FALSE,
                      STYLE_NONE);

    ui_style_register(N_("Style settings for chat pages"), "chat:user:opped",
                      N_("Users who have operator status"),
                      N_("Text"),       STYLE_COLOR, TRUE,  0x3caf, 0xaaaa, 0x0000,
                      N_("Background"), STYLE_COLOR, FALSE, 0xcccc, 0xcccc, 0xcccc,
                      N_("Font"),       STYLE_FONT,  FALSE, NULL,
                      NULL);

    // the gtk text tags used by the print functions
    // NOTE: these tags are ordered by priority, so most important
    // at the end (c_hilight)
    TagTable = GTK_TEXT_TAG_TABLE(gtk_text_tag_table_new());
    ui_chat_register_tag_from_style("c_text",      "chat:text");
    ui_chat_register_tag_from_style("c_timestamp", "chat:timestamp");
    ui_chat_register_tag_from_style("c_client",    "chat:client");
    ui_chat_register_tag_from_style("c_server",    "chat:server");
    ui_chat_register_tag_from_style("c_join",      "chat:join");
    ui_chat_register_tag_from_style("c_part",      "chat:part");
    ui_chat_register_tag_from_style("c_sprefix",   "chat:sprefix");
    ui_chat_register_tag_from_style("c_cprefix",   "chat:cprefix");
    ui_chat_register_tag_from_style("c_tprefix",   "chat:tprefix");
    ui_chat_register_tag_from_style("c_jprefix",   "chat:jprefix");
    ui_chat_register_tag_from_style("c_pprefix",   "chat:pprefix");
    ui_chat_register_tag_from_style("c_bracket",   "chat:bracket");
    NickTag = ui_chat_register_tag_from_style("c_nick",      "chat:nick");
    ui_chat_register_tag_from_style("c_priv_msg",  "chat:private");
    ui_chat_register_tag_from_style("c_hilight",   "chat:hilight");
    UrlTag = ui_chat_register_tag_from_style("c_url",       "chat:url");
		g_signal_connect(G_OBJECT(UrlTag), "event", G_CALLBACK(on_url_event), NULL);

    ui_chat_register_timestamp_tag("t_hour", "/core/chat/timestamp");
    ui_chat_register_timestamp_tag("t_min",  "/core/chat/timestamp");
    ui_chat_register_timestamp_tag("t_sec",  "/core/chat/timestamp");


    // finally the print functions, which is the frontend to the user
    ui_chat_register_print_func("client",   ui_chat_print_client);
    ui_chat_register_print_func("server",   ui_chat_print_server);
    ui_chat_register_print_func("join",     ui_chat_print_join);
    ui_chat_register_print_func("part",     ui_chat_print_part);
    ui_chat_register_print_func("proto",    ui_chat_print_proto);
    ui_chat_register_print_func("stamp",    ui_chat_print_other);
    ui_chat_register_print_func("nl",       ui_chat_print_nl);
    ui_chat_register_print_func("nick_text",ui_chat_print_nick_text);
    ui_chat_register_print_func("nick",     ui_chat_print_nick);
    ui_chat_register_print_func("url",      ui_chat_print_url);
 
    SPrefix = g_strdup(pref_get_ptr("/core/chat/prefix/server", char));
    pref_observe("/core/chat/prefix/server",
                 (ObjectFunc)on_prefix_changed, &SPrefix);
    CPrefix = g_strdup(pref_get_ptr("/core/chat/prefix/client", char));
    pref_observe("/core/chat/prefix/client",
                 (ObjectFunc)on_prefix_changed, &CPrefix);
    JPrefix = g_strdup(pref_get_ptr("/core/chat/prefix/join", char));
    pref_observe("/core/chat/prefix/join",
                 (ObjectFunc)on_prefix_changed, &JPrefix);
    PPrefix = g_strdup(pref_get_ptr("/core/chat/prefix/part", char));
    pref_observe("/core/chat/prefix/part",
                 (ObjectFunc)on_prefix_changed, &PPrefix);
    TPrefix = g_strdup(pref_get_ptr("/core/chat/prefix/text", char));
    pref_observe("/core/chat/prefix/text",
                 (ObjectFunc)on_prefix_changed, &TPrefix);
    pref_observe("/core/chat/colored_nicks",
                 (ObjectFunc)on_colored_changed, NULL);
  }
}


static void on_chat_entry_activate(GtkEntry* entry) {
  char *text;

  //  deactivate_auto_afk();
  
  text = g_strdup(gtk_entry_get_text(entry));
  if (*text == 0) {
    g_free(text);
    return;
  }

  command_handle(chat_page_get_current(), text);

  g_free(text);
  gtk_entry_set_text(entry, "");
}

static void on_chat_entry_changed(GtkEntry* entry) {
  if (MonitorChanges)
    command_history_update(gtk_entry_get_text(entry));
}

static gboolean
on_chat_entry_key_press_event(GtkEntry* entry,
			      GdkEventKey* event) {
  char *text;
  chat_page_t* cur;

  cur = chat_page_get_current();

  if (event->type == GDK_KEY_PRESS) {
    /*
    if ((event->keyval == GDK_r) && (event->state & 8) && 
	(global.user_reply.user)) {
      
      if (global.user_reply.net)
	i1 = g_list_index(global.net_active, 
			  global.user_reply.net);
      else i1 = -1;

      if (i1 >= 0) {
	text = g_strdup_printf("/msg :%d %s ", i1+1,
			       global.user_reply.user);
	gtk_entry_set_text(entry, text);
	g_free(text);
      }
      gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
      return TRUE;
      } else */
    if (event->keyval == 65289) {	// tab
      int pos;
      char* t;

      pos = gtk_editable_get_position(GTK_EDITABLE(entry));
      t = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, pos);
      text = command_expand(cur, t);
      g_free(t);
      if (text) {
	gtk_editable_delete_text(GTK_EDITABLE(entry), 0, pos);
	pos = 0;
	gtk_editable_insert_text(GTK_EDITABLE(entry), text, -1, &pos);
	gtk_editable_set_position(GTK_EDITABLE(entry), pos);
	g_free(text);
      }
    } else if (event->keyval == 65362 || event->keyval == 65364) {
      const char* t;

      if (event->keyval == 65362) 
	t = command_history_up();
      else if (event->keyval == 65364)
	t = command_history_down();
      else 
	t = NULL;
      if (t) {
	MonitorChanges = FALSE;
	gtk_entry_set_text(entry, t);
	gtk_editable_set_position(GTK_EDITABLE(entry), -1);
	MonitorChanges = TRUE;
      } else {
	gdk_beep();
      }
    } else {
      return FALSE;
    }
    g_signal_stop_emission_by_name(G_OBJECT(entry), "key_press_event");
  }
  

  /*
  text = gtk_entry_get_text(entry);
  if (!g_strcasecmp(text, "/reply") && (event->keyval == 32)) {
    if (global.user_reply.user && global.user_reply.net) {
      int i1 = g_list_index(global.net_active, 
			    global.user_reply.net);
      sprintf(t, "/msg :%d %s", i1+1, global.user_reply.user);
    } else
      sprintf(t, "/msg");
    gtk_entry_set_text(GTK_ENTRY(widget), t);
  }
  if (!g_strcasecmp(text, "/reply ")) {
    if (global.user_reply.user && global.user_reply.net) {
      int i1 = g_list_index(global.net_active, 
			    global.user_reply.net);
      sprintf(t, "/msg :%d %s", i1+1, global.user_reply.user);
    } else
      sprintf(t, "/msg ");
    gtk_entry_set_text(GTK_ENTRY(widget), t);
  }
  */
  return TRUE;
}

GtkWidget* ui_create_chat_content() {
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* entry;
  ui_nb_t* nb;

  vbox = gtk_vbox_new(FALSE, 2);
  gtk_widget_show (vbox);

  NoteBook = nb =
    ui_create_notebook((TabChangedFunc)chat_page_set_current,
                       GTK_ORIENTATION_HORIZONTAL, TRUE);
  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(nb->notebook), TRUE, TRUE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(nb->tab_box), FALSE, FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 2);
  gtk_widget_show (hbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  entry = gtk_entry_new();
  gtk_widget_show (entry);
  gtk_entry_set_max_length(GTK_ENTRY(entry), 0);
  gtk_box_pack_start(GTK_BOX (hbox), entry, TRUE, TRUE, 0);
  g_signal_connect(G_OBJECT(entry), "activate",
                   G_CALLBACK(on_chat_entry_activate),
                   NULL);
  g_signal_connect(G_OBJECT(entry), "changed",
                   G_CALLBACK(on_chat_entry_changed),
                   NULL);
  g_signal_connect(G_OBJECT(entry), "key-press-event",
                   G_CALLBACK(on_chat_entry_key_press_event),
                   NULL);
  
  return vbox;
}

void ui_chat_page_remove(chat_page_t* page)
{
    chat_gui_t* data = object_get_data(OBJ(page), "ChatPage");

    if (data->view) g_free(data->view);
    if (data->buffer) g_free(data->buffer);
    
    ui_notebook_remove_page(NoteBook, page);
    
    object_free_data_common(OBJ(page), "ChatPage");
}

static void ui_chat_page_update(chat_page_t* page)
{
    chat_gui_t* data = object_get_data(OBJ(page), "ChatPage");

    if (!data) return;

    // update label
    if (data->label &&
        strcmp(page->label, gtk_label_get_label(data->label)))
    {
        gtk_label_set_text(data->label, page->label);
    }

    // update title
    if (page->title)
    {
        char* temp;

        if (page->ID >= 0)
            temp = auto_text(g_markup_printf_escaped
                             ("<b>%s - %s</b>", plugin_get_name(plugin_get(page->ID)), page->title));
        else
            temp = auto_text(g_markup_printf_escaped("<b>%s</b>", page->title));
        

        if (strcmp(temp, gtk_label_get_label(data->title)))
        {
            gtk_label_set_markup(data->title, temp);
        }

        if ((GTK_WIDGET_SENSITIVE(data->title)!=FALSE) != (page->active!=FALSE))
        {
            gtk_widget_set_sensitive(GTK_WIDGET(data->title), page->active);
        }
    }
}

void ui_chat_page_add(chat_page_t* page, GtkWidget* controls,
                      GtkWidget* add, ...) {
  va_list ap;
  chat_gui_t* data;
  GtkWidget* widget;
  GtkWidget* vbox0;
  GtkWidget* vbox;
  GtkWidget* hbox0;
  GtkWidget* hbox;
  GtkWidget* item;
  GtkWidget* button;

  if (!page) return;
  data = object_get_data(OBJ(page), "ChatPage");
  if (!data) return;
  
  if (!data->toplevel) return;
  
  vbox0 = gtk_vbox_new(FALSE, 2);
  gtk_widget_show(vbox0);
  data->real_toplevel = vbox0;

  hbox0 = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox0);
  gtk_box_pack_start(GTK_BOX(vbox0), hbox0, FALSE, FALSE, 0);
  
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox0), vbox, FALSE, FALSE, 0);
  
  button = ui_create_close_button(_("close page"));
  gtk_widget_show(button);
  gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);
  g_signal_connect_swapped(G_OBJECT(button), "clicked",
                           G_CALLBACK(chat_page_close), page);
  

  vbox = gtk_vbox_new(FALSE, 2);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox0), vbox, TRUE, TRUE, 0);
  
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

  if (page->ID >= 0) {
    const char* stock = ui_main_get_plugin_stock(page->ID);
    if (stock) {
      item = gtk_image_new_from_stock(stock, GTK_ICON_SIZE_MENU);
      gtk_widget_show(item);
      gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    }
    item = gtk_label_new
      (auto_text(g_markup_printf_escaped
                 ("<b>%s - %s</b>", plugin_get_name(plugin_get(page->ID)), page->title)));
  } else {
    item = gtk_label_new(auto_text(g_markup_printf_escaped("<b>%s</b>", page->title)));
  }
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 0);
  data->title = GTK_LABEL(item);

  if (!page->active) gtk_widget_set_sensitive(item, FALSE);

  if (add) {
    gtk_box_pack_start(GTK_BOX(vbox), add, FALSE, FALSE, 0);
  }
  if (controls) {
    GtkWidget* frame = gtk_vbox_new(FALSE, 5);
    GtkWidget* sep;

    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 5);

    sep = gtk_hseparator_new();
    gtk_widget_show(sep);
    gtk_box_pack_start(GTK_BOX(frame), sep, FALSE, FALSE, 0);
    
    gtk_box_pack_start(GTK_BOX(frame), controls, FALSE, FALSE, 0);
    gtk_widget_show(controls);
    //    gtk_widget_show(frame);
    
    item = ui_create_show_hide_label(_("show controls"), _("hide controls"), frame);
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  }

  va_start(ap, add);
  while ((widget = va_arg(ap, GtkWidget*)) != NULL) {
    gtk_widget_show(widget);
    gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 0);
  }
  va_end(ap);


  g_object_set_data(G_OBJECT(vbox0), "page", page);
  gtk_box_pack_start(GTK_BOX(vbox0), data->toplevel, TRUE, TRUE, 0);

  hbox = gtk_hbox_new(FALSE, 5);
  if (page->ID >= 0) {
    const char* stock = ui_main_get_plugin_stock(page->ID);
    if (stock) {
      item = gtk_image_new_from_stock(stock, GTK_ICON_SIZE_MENU);
      gtk_widget_show(item);
      gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    }
  } else {
    item = gtk_image_new_from_stock(STOCK_LOPSTER_ICON, GTK_ICON_SIZE_MENU);
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  }
  item = gtk_label_new(page->label);
  gtk_widget_show(item);
  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 0);
  data->label = GTK_LABEL(item);

  ui_notebook_add_page(NoteBook, vbox0, hbox, page);

  object_monitor_d(OBJ(page), (ObjectFunc)ui_chat_page_update, NULL);

  return;
}

GtkTextBuffer* ui_chat_new_buffer() {
  GtkTextBuffer* buffer =
    GTK_TEXT_BUFFER(gtk_text_buffer_new(TagTable));
  GtkTextIter iter;

  gtk_text_buffer_get_iter_at_offset(buffer, &iter, -1);
  gtk_text_buffer_create_mark(buffer, "buffer_end", &iter, FALSE);

  return buffer;
}

/*
static gboolean
on_text_view_expose(GtkTextView* view, GdkEventExpose* event, GtkTextBuffer* buffer) {
  GdkRectangle rect;
  GdkWindow* win;
  GtkTextIter l_start,l_end, it;
  gint l_top1,l_top2;
  PangoLayout* l;
  gchar* pomstr;
  gint numlines,w,i;

  win = gtk_text_view_get_window(view, GTK_TEXT_WINDOW_LEFT);
  if (win != event->window) return FALSE;

  gtk_text_view_get_visible_rect(view, &rect);
  gtk_text_view_get_line_at_y(view, &l_start, rect.y, &l_top1);
  gtk_text_view_get_line_at_y(view, &l_end, rect.y+rect.height, &l_top2);
  l = gtk_widget_create_pango_layout(GTK_WIDGET(view), "");

  numlines = gtk_text_buffer_get_line_count(gtk_text_view_get_buffer(view));
  pomstr = g_strdup_printf("%d", MAX(99,numlines));
  pango_layout_set_text(l,pomstr,-1);
  g_free(pomstr);
  pango_layout_get_pixel_size(l,&w,NULL);
  gtk_text_view_set_border_window_size(view,GTK_TEXT_WINDOW_LEFT,w+4);   

  it = l_start;
  for (i = gtk_text_iter_get_line(&l_start); i <= gtk_text_iter_get_line(&l_end); i++) {
    gtk_text_iter_set_line(&it, i);
    gtk_text_view_get_line_yrange(view,&it,&w,NULL);      
    gtk_text_view_buffer_to_window_coords(view,GTK_TEXT_WINDOW_LEFT,0,w,NULL,&w);
    pomstr = g_strdup_printf("%d", i+1);
    pango_layout_set_text(l, pomstr, -1);
    gtk_paint_layout(GTK_WIDGET(view)->style, win, GTK_WIDGET_STATE(GTK_WIDGET(view)),
		     FALSE, NULL, GTK_WIDGET(view), NULL, 2, w, l);
    g_free(pomstr);
  }
  g_object_unref(G_OBJECT(l));
  return TRUE;
}
*/

static void on_view_cursor(GtkTextView* view, GdkWindow* win, double ex, double ey) {
  GtkTextIter iter;
  int x, y;

	gtk_text_view_window_to_buffer_coords(view, GTK_TEXT_WINDOW_WIDGET, ex, ey, &x, &y);
	gtk_text_view_get_iter_at_location(view, &iter, x, y);

  if (gtk_text_iter_has_tag(&iter, UrlTag)) {
    gdk_window_set_cursor(win, ui_chat_get_hand_cursor());
    return;
  } else if (gtk_text_iter_has_tag(&iter, NickTag)) {
    gdk_window_set_cursor(win, NULL);
  } else {
    gdk_window_set_cursor(win, NULL);
  }
}
static gboolean on_view_motion(GtkTextView* view, GdkEventMotion* event) {
	gdk_window_get_pointer(GTK_WIDGET(view)->window, NULL, NULL, NULL);
  on_view_cursor(view, event->window, event->x, event->y);
	return FALSE;
}

static gboolean on_view_enter(GtkTextView* view, GdkEventCrossing* event) {
  on_view_cursor(view, event->window, event->x, event->y);
	return FALSE;
}
static gboolean on_view_leave(GtkTextView* view) {
  gdk_window_set_cursor(GTK_WIDGET(view)->window, NULL);
	return FALSE;
}

GtkWidget* ui_chat_new_text_view(GtkTextBuffer* buffer) {
  GtkWidget* text = gtk_text_view_new_with_buffer(buffer);

  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(text), GTK_WRAP_WORD_CHAR);
  gtk_text_view_set_editable(GTK_TEXT_VIEW(text), FALSE);
  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(text), FALSE);

  //FIXME: test
  //  gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(text), GTK_TEXT_WINDOW_LEFT, 70);
  //  g_signal_connect(G_OBJECT(text), "expose-event", G_CALLBACK(on_text_view_expose), buffer);
  //FIXME: test end

  gtk_widget_set_events(text,
                        gtk_widget_get_events(text) | 
                        GDK_BUTTON_PRESS_MASK |
                        //                        GDK_KEY_PRESS_MASK |
                        GDK_ENTER_NOTIFY_MASK |
                        GDK_LEAVE_NOTIFY_MASK);


  g_signal_connect(G_OBJECT(text), "motion-notify-event",
                   G_CALLBACK(on_view_motion), NULL);
  g_signal_connect(G_OBJECT(text), "enter-notify-event",
                   G_CALLBACK(on_view_enter), NULL);
  g_signal_connect(G_OBJECT(text), "leave-notify-event",
                   G_CALLBACK(on_view_leave), NULL);

  //  gtk_widget_set_name(text, "ChatTextView");
  return text;
}

void ui_chat_create_simple_page(chat_page_t* page) {
  GtkWidget* sw;
  GtkWidget* text;
  chat_gui_t* data = g_malloc0(sizeof(chat_gui_t));

  object_set_data(OBJ(page), "ChatPage", data);

  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(sw);
  gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
  data->toplevel = sw;

  data->no_buffers = 1;
  data->buffer = g_malloc0(sizeof(GtkTextBuffer)*data->no_buffers);
  data->view = g_malloc0(sizeof(GtkTextView)*data->no_buffers);

  data->buffer[0] = ui_chat_new_buffer();
  text = ui_chat_new_text_view(data->buffer[0]);
  gtk_widget_show(text);
  //  gtk_text_view_set_indent(GTK_TEXT_VIEW(text), -50);

  gtk_container_add(GTK_CONTAINER(sw), text);
  data->view[0] = GTK_TEXT_VIEW(text);
  gtk_text_view_set_wrap_mode(data->view[0], GTK_WRAP_WORD);
  gtk_text_view_set_editable(data->view[0], FALSE);
  gtk_text_view_set_cursor_visible(data->view[0], FALSE);

  //  gtk_widget_set_style(text, global.scheme->style);
  
  ui_chat_page_add(page, NULL, NULL, NULL);
  
  object_notify_d(OBJ(page), (ObjectFunc)ui_chat_page_remove, NULL);

  /*
  gtk_notebook_set_page(GTK_NOTEBOOK(notebook),
			gtk_notebook_page_num(GTK_NOTEBOOK(notebook),
			page->toplevel));
  */
  /*
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (on_chat_close_clicked),
                      page);
  */
  return;
}

static void
ui_chat_buffer_check_for_high(GtkTextBuffer* buffer, 
                              GtkTextIter* citer, GtkTextIter* iter) {
  char* text;
  const GList* list;

  //FIXME: do utf8 processing
  text = gtk_text_buffer_get_text(buffer, citer, iter, FALSE);
  list = string_list_get_items("core:chat:hilight");
  while (list) {
    const char* item = list->data;
    if (strcasestr(text, item)) {
      gtk_text_buffer_apply_tag_by_name
        (buffer, "c_hilight", citer, iter);
      break;
    }
    list = list->next;
  }
  g_free(text);
}

static int _is_url(char* word, int len) {
  char* dot;
  
  if (!g_ascii_isalnum(*word)) return 0;

  dot = word;
  while ((dot = strchr(dot, '.'))) {
    dot++;
    if (*dot == '.') return 0;
  }
  
  if (len < 5) return 0;
  
  if (!strncasecmp(word, "irc://", 6)) return 1;
  if (!strncasecmp(word, "irc.", 4)) return 1;
  if (!strncasecmp(word, "ftp.", 4)) return 1;
  if (!strncasecmp(word, "ftp:", 4)) return 1;
  if (!strncasecmp(word, "www.", 4)) return 1;
  if (!strncasecmp(word, "http:", 5)) return 1;
  if (!strncasecmp(word, "https:", 6)) return 1;
  
  if (!strncasecmp(word + len - 5, ".html", 5)) return 1;
  if (!strncasecmp(word + len - 5, ".php3", 5)) return 1;
  if (!strncasecmp(word + len - 4, ".htm", 4)) return 1;
  if (!strncasecmp(word + len - 4, ".php", 4)) return 1;
  if (!strncasecmp(word + len - 4, ".org", 4)) return 1;
  if (!strncasecmp(word + len - 4, ".net", 4)) return 1;
  if (!strncasecmp(word + len - 4, ".com", 4)) return 1;
  if (!strncasecmp(word + len - 4, ".edu", 4)) return 1;
  
  if (len > 5) {
    if (word[len-3] == '.' && g_ascii_isalpha(word[len-2]) && 
	g_ascii_isalpha(word[len-1]))
      return 1;
  }
  return 0;
}

static void
ui_chat_buffer_check_for_url(GtkTextBuffer* buffer, 
			     GtkTextIter* citer, GtkTextIter* iter) {
  char* text;
  char* pos;
  int start = gtk_text_iter_get_offset(citer);
  int stop = start;
  char* prev;

  pos = text = gtk_text_iter_get_slice(citer, iter);
  
  while (pos) {
    char* next = g_utf8_strchr(pos, -1, g_utf8_get_char(" "));
    int len;

    // end the word
    if (next) *next = 0;

    len = g_utf8_strlen(pos, -1);
    stop = start + len;
    
    while (*pos == ' ' || *pos == '.' || *pos == '(') {
      pos++; start++; len--;
    }
    if (next) prev = next;
    else if (len > 0) prev = g_utf8_offset_to_pointer(pos, len-1);
    else prev = NULL;
    while (len > 0) {
      if (*prev == '.' || *prev == ')') {
	*prev = 0; len--;
      } else {
	break;
      }
      if (len > 0) prev = g_utf8_prev_char(prev);
    }
    
    if (len > 0 && _is_url(pos, len)) {
      GtkTextIter iter1;
      GtkTextIter iter2;
      gtk_text_buffer_get_iter_at_offset(buffer, &iter1, start);
      gtk_text_buffer_get_iter_at_offset(buffer, &iter2, start+len);
      gtk_text_buffer_apply_tag_by_name
      	(buffer, "c_url", &iter1, &iter2);
    }
    pos = next?next+1:NULL;
    start = stop+1;
  }
  g_free(text);
}

void ui_chat_print(chat_page_t* page, int index, va_list ap)
{
    chat_gui_t* data;
    char* tag;
    GtkTextBuffer* buffer;
    GtkTextIter iter;
    GtkTextMark* mark;
    int padding;
    char* text;
    gboolean at_bottom = TRUE;
    GtkTextIter citer;
    
    if (!page) return;
    data = object_get_data(OBJ(page), "ChatPage");
    if (!data || index < 0 || index >= data->no_buffers) return;
    
    buffer = data->buffer[index];
    gtk_text_buffer_get_iter_at_offset(buffer, &iter, -1);
    
    
    { // check whether we are at bottom
        GdkRectangle rect;
        gint y, height;
        
        gtk_text_view_get_visible_rect(data->view[index], &rect);
        gtk_text_view_get_line_yrange(data->view[index], &iter,
                                      &y, &height);
        if (y + height > rect.y + rect.height) at_bottom = FALSE;
    }
    


    while ((tag = va_arg(ap, char*)) != NULL)
    {
        PrintFunc pfunc;
        
        if (tag[0] != '+')
        {
            g_log("UI_CHAT", G_LOG_LEVEL_WARNING,
                  "ignoring chat tag [%s]", tag);
            continue;
        }
        tag++;
        padding = *tag++;
        
        mark = gtk_text_buffer_create_mark(buffer, NULL, &iter, TRUE);
        
        if (padding & PAD_LEFT)
        {
            gtk_text_buffer_insert(buffer, &iter, " ", 1);
        }
        
        pfunc = g_hash_table_lookup(PrintFuncList, tag);
        if (pfunc)
        {
            pfunc(buffer, &iter, ap);
        }
        else
        {
            // no func found, assume string
            text = va_arg(ap, char*);
            if (text && *text)
            {
                unsigned len = strlen(text);
                while (len && text[len-1] == '\n') len--;
                gtk_text_buffer_insert_with_tags_by_name
                    (buffer, &iter, text, len, tag, NULL);
            }
        }
        
        if (padding & PAD_RIGHT)
        {
            gtk_text_buffer_insert(buffer, &iter, " ", 1);
        }
        
        gtk_text_buffer_get_iter_at_mark(buffer, &citer, mark);
        
        // hilight some of the inserted text if requested so.
        if (padding & PAD_HILIGHT)
        {
            ui_chat_buffer_check_for_high(buffer, &citer, &iter);
        }
        
        ui_chat_buffer_check_for_url(buffer, &citer, &iter);
        
        gtk_text_buffer_delete_mark(buffer, mark);
    }
    
    if (at_bottom)
    {
        mark = gtk_text_buffer_get_mark(buffer, "buffer_end");
        gtk_text_view_scroll_mark_onscreen(data->view[index], mark);
    }
}

gboolean ui_chat_page_printable(chat_page_t* page)
{
    chat_gui_t* data = object_get_data(OBJ(page), "ChatPage");
    return (data && data->no_buffers > 0 && data->buffer[0] != NULL);
}

void ui_chat_switch_page(chat_page_t* page) {
  ui_notebook_switch_to_data(NoteBook, page);
  ui_switch_main(TAB_CHAT);
}

#ifdef WIN32
#define _UI_CHAT_C_
#include "lgtk2corealiasdef.c"
#endif
