/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
a * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.18 $
 * $Date: 2007/11/21 20:15:25 $
 * $Author: sgop $
 */

#include <gtk/gtk.h>

#include "ui_main.h"
#include "ui_support.h"
#include "ui_treeview.h"
#include "ui_style.h"
#include "ui_tooltips.h"
#include "ui_layout.h"
#include "ui_download.h"
#include "renderer_download.h"
#include "lcore/download.h"
#include "lcore/search.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"

static GtkTreeView*  DownloadView   = NULL;
static GList* Downloads = NULL;
static rate_t* TheRate = NULL;
static volume_t* TheVolume = NULL;

/* typedef struct { */
/*     GtkListStore* store; */
/*     GtkTreeViewColumn* col[2]; */
/* } ui_details_t; */

typedef struct {
  rend_dl_data_t _data;
  GtkTreeIter iter;
} ui_ds_t;

/* static void ui_details_destroy(ui_details_t* data) */
/* { */
/*     g_object_unref(data->store); */
/*     g_free(data); */
/* } */

#define COL_DATA         0
#define COL_FILENAME     1
#define COL_STATUS       2
#define COL_PROGRESS     3
#define COL_RATE         4
#define COL_ETA          5
#define COL_SOURCE       6
#define COL_SOURCE_PIX   7

GtkTreeStore* ui_download_get_store(void)
{
    static GtkTreeStore* DownloadStore  = NULL;
    if (!DownloadStore)
    {
        DownloadStore = gtk_tree_store_new
            (8,
             G_TYPE_POINTER,  // data pointer
             G_TYPE_STRING,   // filename
             G_TYPE_STRING,   // status
             G_TYPE_POINTER,  // progress // download_t* source_t*
             G_TYPE_STRING,   // rate
             G_TYPE_STRING,   // eta
             G_TYPE_STRING,   // source
             G_TYPE_STRING);  // plugin pix
        gtk_tree_sortable_set_sort_func
            (GTK_TREE_SORTABLE(DownloadStore),
             2, (GtkTreeIterCompareFunc)sort_file_by_size,
             DownloadStore, NULL);
    }
    return DownloadStore;
}

static void ui_style_changed(const char* name, GObject* rend) {
  (void)name;
  g_object_set(rend,
               "background-color", ui_style_get_color("transfer:bar", "Background"),
               "border-color",   ui_style_get_color("transfer:bar", "Border"),
               "active-color",   ui_style_get_color("transfer:bar", "In-progress"),
               "finished-color", ui_style_get_color("transfer:bar", "Finished"),
               "progress-color", ui_style_get_color("transfer:bar", "Percentage"),
               NULL);
}

static GtkWidget* on_download_tip(void* object, unsigned depth) {
  GtkWidget* table;
  int line = 0;

  table = gtk_table_new(1, 2, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);

  if (depth == 1) {
    download_t* download = object;
    char* str;
    
    ui_table_add_file(GTK_TABLE(table), &line, FILE(download));
    ui_table_add_sep(GTK_TABLE(table), &line);
    
    str = make_string_from_list(download->search_keys, " ");
    ui_table_add_line(GTK_TABLE(table), &line, _("Search keys"), str);
    g_free(str);
    
    ui_table_add_line(GTK_TABLE(table), &line, _("Sources"),
                      l_printf("%d", download->no_source));
  } else if (depth == 2) {
    source_t* source = object;
 
    ui_table_add_file(GTK_TABLE(table), &line, source->file);
    ui_table_add_sep(GTK_TABLE(table), &line);
    ui_table_add_user(GTK_TABLE(table), &line, source->user);

    // DEBUG
    ui_table_add_sep(GTK_TABLE(table), &line);
    ui_table_add_line(GTK_TABLE(table), &line, _("Status"),
                      l_printf("%d", source->status));
    ui_table_add_line(GTK_TABLE(table), &line, _("Scheduled"),
                      source->scheduled?_("yes"):_("no"));
    ui_table_add_line(GTK_TABLE(table), &line, _("Timer"),
                      l_printf("%d", source->timeout));
    // DEBUG END
  }

  gtk_widget_show(table);
  return table;
}

static GtkWidget* on_source_tip(void* object, unsigned depth)
{
    GtkWidget* table;
    source_t* source = object;
    int line = 0;

    table = gtk_table_new(1, 2, FALSE);
    gtk_table_set_col_spacings(GTK_TABLE(table), 10);

    ui_table_add_file(GTK_TABLE(table), &line, source->file);
    ui_table_add_sep(GTK_TABLE(table), &line);
    ui_table_add_user(GTK_TABLE(table), &line, source->user);

    // DEBUG
    ui_table_add_sep(GTK_TABLE(table), &line);
    ui_table_add_line(GTK_TABLE(table), &line, _("Status"),
                      l_printf("%d", source->status));
    ui_table_add_line(GTK_TABLE(table), &line, _("Scheduled"),
                      source->scheduled?_("yes"):_("no"));
    ui_table_add_line(GTK_TABLE(table), &line, _("Timer"),
                      l_printf("%d", source->timeout));
    // DEBUG END

    gtk_widget_show(table);
    return table;
}

static void on_limit_changed(limit_t* limit, GtkLabel* label) {
  int l = limit_get_limit(limit);
  gtk_label_set_text(label, print_speed(&l));
}

static void on_band_changed(limit_t* limit, GtkLabel* label) {
  int r = rate_get_rate(TheRate);
  gtk_label_set_text(label, print_speed(&r));
  (void)limit;
}

static void on_load_changed(volume_t* volume, GtkLabel* label) {
  file_size_t size = volume_get_sum(volume);
  gtk_label_set_text(label, print_filesize(&size));
}



GtkWidget* ui_create_download_content() {
  GtkWidget* swindow;
  GtkWidget* vbox0;
  GtkWidget* hbox;
  GtkTreeStore* store;
  GtkWidget* tv;
  GtkTreeViewColumn* column;
  GtkTreeSelection* select;
  GtkWidget* da;
  GtkCellRenderer* rend;
  GtkWidget* item;
  rate_t* rate = TheRate = downloads_get_rate();
  volume_t* volume = TheVolume = volume_find("Download Bandwidth Limit");

  //////////////
  vbox0 = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox0);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox0), hbox, FALSE, FALSE, 2);

  item = gtk_image_new_from_stock(STOCK_LOPSTER_DOWNLOAD, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  item = ui_create_header_label(_("Downloads"));
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  

  {
    GtkWidget* hbox2 = gtk_hbox_new(FALSE, 10);
    gtk_widget_show(hbox2);
    gtk_box_pack_start(GTK_BOX(hbox), hbox2, TRUE, FALSE, 5);
  
    item = gtk_label_new(_("<b>Bandwidth:</b>"));
    gtk_widget_show(item);
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

    item = gtk_label_new("");
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
    object_monitor_d(OBJ(volume), (ObjectFunc)on_band_changed, item);

    item = gtk_label_new(_("<b>Limit:</b>"));
    gtk_widget_show(item);
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

    item = gtk_label_new("");
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
    object_monitor_d(OBJ(volume), (ObjectFunc)on_limit_changed, item);

    item = gtk_label_new(_("<b>Downloaded:</b>"));
    gtk_widget_show(item);
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

    item = gtk_label_new("");
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
    object_monitor_d(OBJ(volume), (ObjectFunc)on_load_changed, item);

    object_set_dirty(OBJ(volume));
  }  


  swindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start(GTK_BOX(vbox0), swindow, TRUE, TRUE, 0);
  gtk_widget_show(swindow);
  gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW(swindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type
    (GTK_SCROLLED_WINDOW(swindow), GTK_SHADOW_IN);

  store = ui_download_get_store();
  tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
  gtk_widget_show(tv);
  DownloadView = GTK_TREE_VIEW(tv);
  gtk_container_add(GTK_CONTAINER(swindow), tv);
  g_object_set(G_OBJECT(tv), "fixed-height-mode", TRUE, NULL);

  gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(tv), TRUE);
  //  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW (tv), FALSE);
  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
  gtk_tree_selection_set_mode (select, GTK_SELECTION_MULTIPLE);
  /*
  g_signal_connect(G_OBJECT(select), "changed",
		   G_CALLBACK(on_download_select), NULL);
  */
  gtk_fancytips_new(GTK_TREE_VIEW(tv), (FancyTipsFunc)on_download_tip,
                    GTK_FANCY_POS_MOUSE);


  column = ui_tv_column_new
    (_("Filename"), gtk_cell_renderer_text_new(), "text", 1, TRUE, NULL);
  ui_tv_config_default_col(column, 1, "core:download:col:filename", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  column = ui_tv_column_new
    (_("Status"), gtk_cell_renderer_text_new(), "text", COL_STATUS, TRUE, NULL);
  ui_tv_config_default_col(column, COL_STATUS, "core:download:col:status", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  column = ui_tv_column_new
    (_("Source"),
     gtk_cell_renderer_pixbuf_new(), "stock-id", COL_SOURCE_PIX, FALSE,
     gtk_cell_renderer_text_new(), "text", COL_SOURCE, TRUE, NULL);
  ui_tv_config_default_col(column, COL_SOURCE, "core:download:col:source", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  rend = gtk_cell_renderer_download_new();
  ui_style_changed(NULL, G_OBJECT(rend));
  ui_style_observe("transfer:bar", (StyleFunc)ui_style_changed, rend);
  
  column = ui_tv_column_new(_("Progress"), rend, "download", COL_PROGRESS, TRUE, NULL);
  ui_tv_config_default_col(column, COL_PROGRESS, "core:download:col:progress", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  column = ui_tv_column_new
    (_("Rate"), gtk_cell_renderer_text_new(), "text", COL_RATE, TRUE, NULL);
  ui_tv_config_default_col(column, COL_RATE, "core:download:col:rate", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  column = ui_tv_column_new
    (_("ETA"), gtk_cell_renderer_text_new(), "text", COL_ETA, TRUE, NULL);
  ui_tv_config_default_col(column, COL_ETA, "core:download:col:eta", TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  
  {
    action_item_t* actions[2];
    actions[0] =  action_item_search("Download actions");
    actions[1] =  action_item_search("Source actions");

    ui_tv_handle_button(GTK_TREE_VIEW(tv), 2, actions, NULL);
  }

  da = ui_create_bandwidth_bar("core:band:download", volume, rate);
  gtk_box_pack_start(GTK_BOX(vbox0), da, FALSE, FALSE, 0);
  ui_layout_monitor("core:download:bandwidth:vis", da);

  item = ui_create_show_hide_label(_("show bandwidth"), _("hide bandwidth"), da);
  gtk_widget_show(item);
  gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  item = ui_create_event_label
    (_("clear finished"), G_CALLBACK(download_clear_finished), NULL, TRUE);
  gtk_widget_show(item);
  gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  return vbox0;
}

static void ui_download_update(download_t* download)
{
    ui_ds_t* data = object_get_data(OBJ(download), "DownloadEntry");
    GtkTreeStore* store = ui_download_get_store();
    file_t* file = FILE(download);
    file_size_t todo = file->size - download_get_progress(download);
  
    if (!data) return;
    
    if (download->no_active)
    {
        int rate = download_get_rate(download);
        int left = rate>0?todo/rate:0;
        
        gtk_tree_store_set(
                store, &data->iter,
                COL_STATUS, l_printf(_("%s to go"), print_filesize(&todo)),
                COL_RATE, print_speed(&rate),
                COL_ETA, print_duration_unit(&left),
                COL_SOURCE, l_printf(_("%d/%d active"), download->no_active, download->no_source),
                -1);
    }
    else
    {
        gtk_tree_store_set(
                store, &data->iter,
                COL_STATUS, l_printf(_("%s to go"), print_filesize(&todo)),
                COL_RATE, "",
                COL_ETA, "",
                COL_SOURCE, l_printf(_("%d sources"), download->no_source),
                -1);
    }
}

static void ui_download_hide(download_t* download)
{
    ui_ds_t* data = object_get_data(OBJ(download), "DownloadEntry");

    Downloads = g_list_remove(Downloads, download);
    
    if (data)
    {
        gtk_tree_store_remove(ui_download_get_store(), &data->iter);
        object_free_data_common(OBJ(download), "DownloadEntry");
        // set sources as hidden since we removed them with the download.
        g_list_foreach(download->sources, (GFunc)object_free_data_common, "SourceEntry");
    }
}

void ui_download_new(download_t* download)
{
    ui_ds_t* data = object_get_data(OBJ(download), "DownloadEntry");
    GtkTreeStore* store = ui_download_get_store();
    file_t* file;

    if (data) return;

    object_set_data(OBJ(download), "DownloadEntry", data = g_malloc(sizeof(*data)));
    data->_data.download = download;
    data->_data.source = NULL;
    gtk_tree_store_append(store, &data->iter, NULL);
    Downloads = g_list_append(Downloads, download);
  
    object_notify_d(OBJ(download), (ObjectFunc)ui_download_hide, NULL);
    object_monitor_d(OBJ(download), (ObjectFunc)ui_download_update, NULL);

    file = FILE(download);
    gtk_tree_store_set(
            store, &data->iter,
            COL_DATA, download,
            COL_FILENAME, file->filename,
            COL_PROGRESS, data,
            COL_SOURCE_PIX, STOCK_LOPSTER_ICON,
            -1);
    ui_download_update(download);
}

static void ui_source_update(source_t* source)
{
    ui_ds_t* data = object_get_data(OBJ(source), "SourceEntry");
    const char* status = NULL;

    if (!data) return;

    if (source->scheduled)
    {
        status = _("queued (locally)");
    }
    else if (source->status == SOURCE_REMOTE)
    {
        if (source->remote)
            status = l_printf(_("queued (%d)"), source->remote);
        else
            status = _("queued (remotely)");
    }
    else if (source->status == SOURCE_STARTED)
    {
        status = _("requested");
    }
    
    if (source->cur_segment)
    {
        segment_t* seg = source->cur_segment;
        int rate = socket_get_rate(SOCK(source));
        file_size_t size = seg->stop - seg->start - seg->size;
        int left = rate>0?size/rate:0;
        
        gtk_tree_store_set(
                ui_download_get_store(), &data->iter,
                COL_STATUS, status?status:l_printf(_("%s to go"), print_filesize(&size)),
                COL_RATE, print_speed(&rate),
                COL_ETA, print_duration_unit(&left),
                -1);
    }
    else
    {
        gtk_tree_store_set(
                ui_download_get_store(), &data->iter,
                COL_STATUS, status,
                COL_RATE, NULL,
                COL_ETA, NULL,
                -1);
    }
}

static void ui_source_hide(source_t* source)
{
    ui_ds_t* data = object_get_data(OBJ(source), "SourceEntry");

    if (!data) return;
    gtk_tree_store_remove(ui_download_get_store(), &data->iter);
    object_free_data_common(OBJ(source), "SourceEntry");
}

static void ui_source_show(source_t* source)
{
    ui_ds_t* pdata;
    ui_ds_t* data;
    
    if (!source->download) return;
    pdata = object_get_data(OBJ(source->download), "DownloadEntry");
    if (!pdata) return;
    if (object_get_data(OBJ(source), "SourceEntry")) return;

    object_set_data(OBJ(source), "SourceEntry", data = g_malloc0(sizeof(*data)));
    data->_data.download = NULL;
    data->_data.source = source;
    gtk_tree_store_append(ui_download_get_store(), 
                          &data->iter, &pdata->iter);
  
    gtk_tree_store_set(
            ui_download_get_store(), &data->iter,
            COL_DATA, source,
            COL_FILENAME, source->file->filename,
            COL_PROGRESS, data,
            COL_SOURCE, source->user->print(source->user),
            COL_SOURCE_PIX, ui_main_get_plugin_stock(source->user->ID),
            -1);
    
    ui_source_update(source);
}

void ui_source_new(source_t* source)
{
    if (!source->download) return;
    
    object_notify_d(OBJ(source), (ObjectFunc)ui_source_hide, NULL);
    object_monitor_d(OBJ(source), (ObjectFunc)ui_source_update, NULL);
    ui_source_show(source);
}
		     
/* static void ui_dsource_hide(source_t* source, ui_details_t* data) */
/* { */
/*     GtkTreeIter* iter = object_get_data(OBJ(source), "DetailsEntry"); */
/*     object_unnotify_d(OBJ(source), (ObjectFunc)ui_dsource_hide, data); */
/*     if (iter)  */
/*     { */
/*         gtk_list_store_remove(data->store, iter); */
/*         object_free_data_common(OBJ(source), "SourceEntry"); */
/*     } */
/* } */


void ui_download_periodic_update(void)
{
    GList* dlist;
    GList* dlist2;
    
    for (dlist = Downloads; dlist; dlist = dlist->next)
    {
        download_t* download = dlist->data;
        if (download->segments_used) ui_download_update(download);
        for (dlist2 = download->sources; dlist2; dlist2 = dlist2->next) {
            source_t* source = dlist2->data;
            if (source->used_segments) ui_source_update(source);
        }
    }
}

static void on_download_killed(download_t* download, GtkWidget* win)
{
    object_unnotify_d(OBJ(download), (ObjectFunc)on_download_killed, win);
    gtk_widget_destroy(win);
}

static void on_details_cancel(GtkWidget* win)
{
    download_t* download = g_object_get_data(G_OBJECT(win), "download");
    object_unnotify_d(OBJ(download), (ObjectFunc)on_download_killed, win);
    gtk_widget_destroy(win);
}

static gboolean on_details_close(GtkWidget* win)
{
    download_t* download = g_object_get_data(G_OBJECT(win), "download");
/*     ui_details_t* data = g_object_get_data(G_OBJECT(win), "data"); */
    
/*     ui_layout_unmonitor("core:source:col:source:size", data->col[0]); */
/*     ui_layout_unmonitor("core:source:col:source:vis", data->col[0]); */
/*     ui_layout_unmonitor("core:source:col:filename:size", data->col[1]); */
/*     ui_layout_unmonitor("core:source:col:filename:vis", data->col[1]); */

/*     g_list_foreach(download->sources, (GFunc)ui_dsource_hide, data); */
    object_unnotify_d(OBJ(download), (ObjectFunc)on_download_killed, win);
    gtk_widget_destroy(win);

    return TRUE;
}

static void on_change_file(GtkEntry* entry)
{
    GtkWindow* win = g_object_get_data(G_OBJECT(entry), "win");
    const char* c = gtk_entry_get_text(entry);
    char* name = ui_select_file(_("Choose file"), (c && *c)?c:NULL, win);

    if (name) 
    {
        gtk_entry_set_text(entry, name);
        g_free(name);
    }
}

static void on_details_apply(GtkWidget* win)
{
    download_t* download = g_object_get_data(G_OBJECT(win), "download");
    GtkEntry* entry = g_object_get_data(G_OBJECT(win), "keys");
    GSList* list = NULL;
    
    make_list_from_string(&list, gtk_entry_get_text(entry), " ");
    if (!list)
    {
        lopster_notify(LOPSTER_MESSAGE_INFO, _("Invalid search keys"),
                       _("Please specify search keys"));
        return;
    }
    g_slist_foreach(download->search_keys, (GFunc)g_free, NULL);
    g_slist_free(download->search_keys);
    download->search_keys = list;
    
    on_details_cancel(win);
}

void ui_download_show_details(download_t* download)
{
    GtkWidget* win;
    GtkWidget* vbox;
    GtkWidget* vbox2;
    GtkWidget* hbox;
    GtkWidget* hhbox;
    GtkWidget* table;
    GtkWidget* entry;
    GtkWidget* item;
    GtkWidget* label;
    GtkWidget* button;
    GtkWidget* sw;
    GtkWidget* tv;
    GtkTreeViewColumn* column;
/*     ui_details_t* data; */

    win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW (win), _("Download Details"));
    gtk_window_set_default_size(GTK_WINDOW(win), 800, 500);
    gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
    if (ui_main_get_win())
    {
        gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
        gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
    }
    g_signal_connect(G_OBJECT(win), "delete_event",
                     G_CALLBACK(on_details_close), download);
    object_notify_d(OBJ(download), (ObjectFunc)on_download_killed, win);
    g_object_set_data(G_OBJECT(win), "download", download);

/*     data = g_malloc0(sizeof(*data)); */
/*     g_object_set_data_full(G_OBJECT(win), "data", data, */
/*                            (GDestroyNotify)ui_details_destroy); */
    
    vbox = gtk_vbox_new(FALSE, 10);
    gtk_widget_show(vbox);
    gtk_container_add(GTK_CONTAINER(win), vbox);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  
    //////////////////////
    vbox2 = ui_create_section(_("General"), NULL, &hbox, NULL);
    gtk_widget_show(vbox2);
    gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, FALSE, 0);
    
    table = gtk_table_new(1, 2, FALSE);
    gtk_widget_show(table);
    gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 0);
    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    gtk_table_set_row_spacings(GTK_TABLE(table), 3);
    
    // filename
    entry = gtk_entry_new();
    gtk_widget_show(entry);
    gtk_entry_set_text(GTK_ENTRY(entry), FILE(download)->localname);
    g_object_set_data(G_OBJECT(entry), "win", win);
    gtk_widget_set_sensitive(entry, FALSE);
    g_object_set_data(G_OBJECT(win), "filename", entry);
    
    item = ui_create_advanced_headed_item(_("filename"), entry, &hhbox);
    gtk_widget_show(item);
    
    label = ui_create_event_label
        (_("change"), G_CALLBACK(on_change_file), entry, TRUE);
    gtk_widget_show(label);
    gtk_box_pack_end(GTK_BOX(hhbox), label, FALSE, FALSE, 0);
    ui_event_label_set_sensitive(label, FALSE);
    
    gtk_table_attach(GTK_TABLE(table), item, 0, 1, 0, 1,
                     GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);
    
    // search keys
    entry = gtk_entry_new();
    gtk_widget_show(entry);
    {
        char* str = make_string_from_list(download->search_keys, " ");
        gtk_entry_set_text(GTK_ENTRY(entry), str);
        g_free(str);
    }
    item = ui_create_headed_item(_("search keys"), entry);
    gtk_widget_show(item);
    g_object_set_data(G_OBJECT(win), "keys", entry);
    
    gtk_table_attach(GTK_TABLE(table), item, 0, 1, 1, 2,
                     GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);
    
    
    //////////////////////
    
/*     vbox2 = ui_create_section(_("Sources"), NULL, &hbox, NULL); */
/*     gtk_widget_show(vbox2); */
/*     gtk_box_pack_start(GTK_BOX(vbox), vbox2, TRUE, TRUE, 0); */
    

/*     sw = gtk_scrolled_window_new (NULL, NULL); */
/*     gtk_widget_show(sw); */
/*     gtk_scrolled_window_set_policy */
/*         (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC); */
/*     gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN); */
/*     gtk_box_pack_start(GTK_BOX(hbox), sw, TRUE, TRUE, 0); */
    
/*     data->store = gtk_list_store_new( */
/*             4, */
/*             G_TYPE_POINTER,  // data pointer */
/*             G_TYPE_STRING,   // filename */
/*             G_TYPE_STRING,   // source */
/*             G_TYPE_STRING);  // stock id */

/*     tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(data->store)); */
/*     gtk_container_add(GTK_CONTAINER(sw), tv); */
/*     gtk_widget_show(tv); */

/*     gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(tv), TRUE); */
/*     gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tv), TRUE); */
/*     select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv)); */
/*     gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE); */

/*     gtk_fancytips_new(GTK_TREE_VIEW(tv), (FancyTipsFunc)on_source_tip, */
/*                       GTK_FANCY_POS_MOUSE); */

/*     data->col[0] = column = ui_tv_column_new */
/*         (_("Source"), */
/*          gtk_cell_renderer_pixbuf_new(), "stock-id", 3, FALSE, */
/*          gtk_cell_renderer_text_new(), "text", 2, TRUE, NULL); */
/*     ui_tv_config_default_col(column, 6, "core:source:col:source", FALSE); */
/*     gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column); */

/*     data->col[1] = column = ui_tv_column_new */
/*         (_("Filename"), gtk_cell_renderer_text_new(), "text", 1, TRUE, NULL); */
/*     ui_tv_config_default_col(column, 1, "core:source:col:filename", TRUE); */
/*     gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column); */

/*     { */
/*         action_item_t* actions[1]; */
/*         actions[0] =  action_item_search("Source actions"); */

/*         ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, actions, NULL); */
/*     } */

/*     { */
/*         GList* dlist; */
/*         for (dlist = download->sources; dlist; dlist = dlist->next) */
/*         { */
/*             source_t* source = dlist->data; */
/*             GtkTreeIter* iter = g_malloc(sizeof(*iter)); */

/*             object_notify_d(OBJ(source), (ObjectFunc)ui_dsource_hide, data); */

/*             gtk_list_store_append(data->store, iter); */
/*             object_set_data(OBJ(source), "DetailsEntry", iter); */
/*             gtk_list_store_set(data->store, iter, */
/*                                0, source, */
/*                                1, source->file->filename, */
/*                                2, source->user->print(source->user), */
/*                                3, ui_main_get_plugin_stock(source->user->ID), */
/*                                -1); */
/*         } */
/*     } */

    hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
    gtk_widget_show(hbox);
    gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_container_add(GTK_CONTAINER(hbox), button);
    gtk_widget_show(button);
    g_signal_connect_swapped(G_OBJECT(button), "clicked",
                             G_CALLBACK(on_details_cancel), win);
    
    button = gtk_button_new_from_stock(GTK_STOCK_OK);
    gtk_container_add(GTK_CONTAINER(hbox), button);
    gtk_widget_show(button);
    g_signal_connect_swapped(G_OBJECT(button), "clicked",
                             G_CALLBACK(on_details_apply), win);
    
    gtk_widget_show(win);
}
