/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.17 $
 * $Date: 2007/04/29 09:21:47 $
 * $Author: sgop $
 */

#include <stdio.h>
#include <string.h>

#include <gtk/gtk.h>

#include "lcore/mtypes.h"
#include "lcore/file_tree.h"
#include "lcore/utils.h"
#include "lcore/l_i18n.h"
#include "ui_filetree.h"
#include "ui_support.h"
#include "ui_main.h"
#include "ui_style.h"
#include "ui_treeview.h"
#include "ui_tooltips.h"

static GdkPixbuf* share_none = NULL;
static GdkPixbuf* share_part = NULL;
static GdkPixbuf* share_all  = NULL;

static const char* TNames(int no)
{
    switch (no) {
    case 0:  return _("Show unshared");
    case 1:  return _("Show shared");
    case 2:  return _("Show new");
    default: return _("Show all");
    };
}


static const char* MNames(file_tree_t* tree)
{
    const char* fill;
    int num = tree->fci[tree->show_media][tree->show_time];
    
    if (tree->show_time == 0) fill = _(" unshared");
    else if (tree->show_time == 1) fill = _(" shared");
    else if (tree->show_time == 2) fill = _(" new");
    else fill = "";
    
    if (tree->show_time == 0)
    {
        switch (tree->show_media)
        {
        case MEDIA_FOLDER:
            return l_printf(ngettext("%d unshared folder", "%d unshared folders", num), num);
        case MEDIA_AUDIO:
            return l_printf(ngettext("%d unshared audio file", "%d unshared audio files", num), num);
        case MEDIA_VIDEO:
            return l_printf(ngettext("%d unshared video file", "%d unshared video files", num), num);
        case MEDIA_APPLICATION:
            return l_printf(ngettext("%d unshared application", "%d unshared applications", num), num);
        case MEDIA_IMAGE:
            return l_printf(ngettext("%d unshared image", "%d unshared images", num), num);
        case MEDIA_DOCUMENT:
            return l_printf(ngettext("%d unshared document", "%d unshared documents", num), num);
        case MEDIA_UNKNOWN:
            return l_printf(ngettext("%d unshared other file", "%d unshared other files", num), num);
        default:
            return l_printf(ngettext("%d unshared file", "%d unshared files", num), num);
        };
    }
    else if (tree->show_time == 1)
    {
        switch (tree->show_media)
        {
        case MEDIA_FOLDER:
            return l_printf(ngettext("%d shared folder", "%d shared folders", num), num);
        case MEDIA_AUDIO:
            return l_printf(ngettext("%d shared audio file", "%d shared audio files", num), num);
        case MEDIA_VIDEO:
            return l_printf(ngettext("%d shared video file", "%d shared video files", num), num);
        case MEDIA_APPLICATION:
            return l_printf(ngettext("%d shared application", "%d shared applications", num), num);
        case MEDIA_IMAGE:
            return l_printf(ngettext("%d shared image", "%d shared images", num), num);
        case MEDIA_DOCUMENT:
            return l_printf(ngettext("%d shared document", "%d shared documents", num), num);
        case MEDIA_UNKNOWN:
            return l_printf(ngettext("%d shared other file", "%d shared other files", num), num);
        default:
            return l_printf(ngettext("%d shared file", "%d shared files", num), num);
        };
    }
    else if (tree->show_time == 2)
    {
        switch (tree->show_media)
        {
        case MEDIA_FOLDER:
            return l_printf(ngettext("%d new folder", "%d new folders", num), num);
        case MEDIA_AUDIO:
            return l_printf(ngettext("%d new audio file", "%d new audio files", num), num);
        case MEDIA_VIDEO:
            return l_printf(ngettext("%d new video file", "%d new video files", num), num);
        case MEDIA_APPLICATION:
            return l_printf(ngettext("%d new application", "%d new applications", num), num);
        case MEDIA_IMAGE:
            return l_printf(ngettext("%d new image", "%d new images", num), num);
        case MEDIA_DOCUMENT:
            return l_printf(ngettext("%d new document", "%d new documents", num), num);
        case MEDIA_UNKNOWN:
            return l_printf(ngettext("%d new other file", "%d new other files", num), num);
        default:
            return l_printf(ngettext("%d new file", "%d new files", num), num);
        };
    }
    else
    {
        switch (tree->show_media)
        {
        case MEDIA_FOLDER:
            return l_printf(ngettext("%d folder", "%d folders", num), num);
        case MEDIA_AUDIO:
            return l_printf(ngettext("%d audio file", "%d audio files", num), num);
        case MEDIA_VIDEO:
            return l_printf(ngettext("%d video file", "%d video files", num), num);
        case MEDIA_APPLICATION:
            return l_printf(ngettext("%d application", "%d applications", num), num);
        case MEDIA_IMAGE:
            return l_printf(ngettext("%d image", "%d images", num), num);
        case MEDIA_DOCUMENT:
            return l_printf(ngettext("%d document", "%d documents", num), num);
        case MEDIA_UNKNOWN:
            return l_printf(ngettext("%d other file", "%d other files", num), num);
        default:
            return l_printf(ngettext("%d file", "%d files", num), num);
        };
    }
}

/** gets pixmap for node */
static void
ui_node_get_pixbuf(file_tree_t* tree, file_node_t* node, GdkPixbuf** pix) {

  if (!share_none) share_none = create_pixbuf("share_none.png");
  if (!share_part) share_part = create_pixbuf("share_part.png");
  if (!share_all)  share_all  = create_pixbuf("share_all.png");

  if (node->fci[tree->show_media][1] == 0) {
    *pix = share_none;
  } else if (tree->show_time == 0) {
    *pix = share_none;
  } else if (tree->show_time == 1) {
    *pix = share_all;
  } else if (node->fci[tree->show_media][1] <
             node->fci[tree->show_media][3]) {
    *pix = share_part;
  } else {
    *pix = share_all;
  }
}

static gboolean ui_unset(GtkTreeModel* model, GtkTreePath* path, GtkTreeIter* iter)
{
    file_node_t* node;
    (void)path;
    gtk_tree_model_get(model, iter, 0, &node, -1);
    object_free_data_common(OBJ(node), "FileNodeEntry");
    return FALSE;
}

static void ui_filetree_hide_folder(file_tree_t* tree)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    GtkTreeModel* model;

    if (!data || !data->folder_tree) return;
  
    // mark as hidden
    model = GTK_TREE_MODEL(data->folder_tree);
    gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)ui_unset,  NULL);
    
    gtk_tree_store_clear(data->folder_tree);
}

static void ui_filetree_hide_childs(file_tree_t* tree)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    GtkTreeModel* model;
    
    if (!data || !data->file_list) return;
  
    // mark as hidden
    model = GTK_TREE_MODEL(data->file_list);
    gtk_tree_model_foreach(model, (GtkTreeModelForeachFunc)ui_unset, NULL);
  
    gtk_list_store_clear(data->file_list);
    data->current_node = NULL;
}

static void ui_filetree_node_update(file_node_t* node, file_tree_t* tree)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    GtkTreeIter* iter = object_get_data(OBJ(node), "FileNodeEntry");
    GdkPixbuf* pix;

    if (!data || !iter) return;

    node->flags &= ~FN_FLAG_UPDATE;
    
    ui_node_get_pixbuf(tree, node, &pix);
    
    if (node->file)
    {
        gtk_list_store_set(data->file_list, iter,
                           2, pix,
                           3, (node->fci[tree->show_media][2]>0)?TRUE:FALSE,
                           4, print_filesize(&(node->file->size)),
                           5, attr_print_all(ATTRC(node->file)),
                           -1);
    }
    else
    {
        gtk_tree_store_set(data->folder_tree, iter,
                           2, pix,
                           3, (node->fci[tree->show_media][2]>0)?TRUE:FALSE,
                           -1);
    }
}

static void ui_filetree_node_hide(file_node_t* node, file_tree_t* tree)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    GtkTreeIter* iter = object_get_data(OBJ(node), "FileNodeEntry");

    if (!data || !iter) return;

    object_unnotify_d(OBJ(node), (ObjectFunc)ui_filetree_node_hide, tree);
    
    // and hide the node
    if (node->file && data->file_list)
    {
        gtk_list_store_remove(data->file_list, iter);
        object_free_data_common(OBJ(node), "FileNodeEntry");
    }
    else if (!node->file && data->folder_tree)
    {
        gtk_tree_store_remove(data->folder_tree, iter);
        object_free_data_common(OBJ(node), "FileNodeEntry");
    }
}

static void ui_filetree_node_show(file_node_t* node, file_tree_t* tree) {
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    GtkTreeIter* iter = object_get_data(OBJ(node), "FileNodeEntry");
    GdkPixbuf* pix;

    if (!data || iter) return;

    if (!data->file_list || !data->folder_tree) return;

    ui_node_get_pixbuf(tree, node, &pix);
  
    if (node->file)
    {
        object_set_data(OBJ(node), "FileNodeEntry", iter = g_malloc(sizeof(*iter)));
        gtk_list_store_append(data->file_list, iter);
        gtk_list_store_set(data->file_list, iter,
                           0, node,
                           1, OBJ(node)->name,
                           -1);
    }
    else if (!node->file)
    {
        GtkTreeIter* parent;

        if (!node->parent) parent = NULL;
        else if (!(parent = object_get_data(OBJ(node->parent), "FileNodeEntry"))) return;

        object_set_data(OBJ(node), "FileNodeEntry", iter = g_malloc(sizeof(*iter)));
        gtk_tree_store_append (data->folder_tree, iter, parent);
        gtk_tree_store_set(data->folder_tree, iter,
                           0, node,
                           1, OBJ(node)->name,
                           -1);
    }

    ui_filetree_node_update(node, tree);
  
    object_notify_d(OBJ(node), (ObjectFunc)ui_filetree_node_hide, tree);
}

static void ui_filetree_show_childs(file_tree_t* tree, file_node_t* node)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    file_node_t* child;
    
    if (!data || !node) return;

    child = node->child;
    while (child)
    {
        if (child->file && child->fci[tree->show_media][tree->show_time])
            ui_filetree_node_show(child, tree);
        child = child->next;
    }
    data->current_node = node;
}

static gboolean 
on_tree_button(GtkTreeView *tv, GdkEventButton *event, tree_gui_t* data)
{
    GtkTreePath* path;
    GtkTreeIter iter;
    file_node_t* node;
    GtkTreeModel* model;
    GtkTreeSelection* select;
    file_tree_t* tree = data->tree;
    
    if (event->window != gtk_tree_view_get_bin_window(tv))
        return FALSE;
    
    if (event->type != GDK_BUTTON_PRESS) return FALSE;
    if (event->button != 1) return FALSE;
    // avoid un/selects if we have mods pressed.
    //  if (event->state != 0) return FALSE;
    if (!tree) return FALSE;
    
    model = gtk_tree_view_get_model(tv);
    
    if (!gtk_tree_view_get_path_at_pos(tv, event->x, event->y,
                                       &path, NULL, NULL, NULL)) return FALSE;
    
    select = gtk_tree_view_get_selection(tv);
    gtk_tree_model_get_iter(model, &iter, path);
    gtk_tree_path_free(path);
    gtk_tree_model_get(model, &iter, 0, &node, -1);
    if (!gtk_tree_selection_iter_is_selected(select, &iter))
    {
        gtk_tree_selection_select_iter(select, &iter);
    }
    
    ui_filetree_hide_childs(tree);
    if (node) ui_filetree_show_childs(tree, node);
    
    return FALSE;
}


/*
static void
on_tree_selection_changed(GtkTreeSelection *selection,
			  tree_gui_t* data) {
  GtkTreeIter iter;
  GtkTreeModel *model;
  file_node_t* node;
  file_tree_t* tree = data->tree;

  if (gtk_tree_selection_get_selected(selection, &model, &iter)) {
    gtk_tree_model_get(model, &iter, 0, &node, -1);
    ui_filetree_hide_childs(tree);
    if (node) ui_filetree_show_childs(tree, node);
  }
}
*/

static gboolean on_hide_progress(tree_gui_t* data)
{
    if (data->hide_timeout) g_source_remove(data->hide_timeout);
    data->hide_timeout = 0;
    gtk_widget_hide(data->bottom_box);
    return FALSE;
}

static void ui_file_tree_show_bottom(tree_gui_t* data, int timeout)
{
    if (!GTK_WIDGET_VISIBLE(data->bottom_box))
        gtk_widget_show(data->bottom_box);
    
    if (data->hide_timeout) g_source_remove(data->hide_timeout);
    data->hide_timeout =
        g_timeout_add(timeout*1000, (GSourceFunc)on_hide_progress, data);
}

void ui_file_tree_update_label(file_tree_t* tree, const char* label)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    
    if (!data) return;
    gtk_label_set_text(GTK_LABEL(data->status_label), label?label:"");
    
    ui_file_tree_show_bottom(data, 30);
}

void ui_file_tree_set_progress(file_tree_t* tree, double progress)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    
    if (!data) return;
    
    gtk_progress_bar_set_fraction
        (GTK_PROGRESS_BAR(data->progress), progress>1.?1.:progress);
    
    ui_file_tree_show_bottom(data, 10);
}

void ui_file_tree_set_progress_name(file_tree_t* tree, const char* name)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    
    if (!data) return;

    gtk_progress_bar_set_text(GTK_PROGRESS_BAR(data->progress), name?name:"");
    
    ui_file_tree_show_bottom(data, 30);
}

void ui_file_tree_set_abort(file_tree_t* tree, GFunc func, void* user_data)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    
    if (!data) return;

    data->abort_func = func;
    data->abort_data = user_data;
    
    if (data->abort_signal)
    {
        g_signal_handler_disconnect(data->abort_button, data->abort_signal);
        data->abort_signal = 0;
    }
    if (!data->abort_func)
    {
        gtk_widget_set_sensitive(data->abort_button, FALSE);
    }
    else
    {
        gtk_widget_set_sensitive(data->abort_button, TRUE);
        data->abort_signal =
            g_signal_connect_swapped(G_OBJECT(data->abort_button), "clicked",
                                     G_CALLBACK(data->abort_func), data->abort_data);
    }
}

static void node_vis_action(file_tree_t* tree, file_node_t* node)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    file_node_t* child;
    int sm = tree->show_media;
    int st = tree->show_time;
    gboolean hide = FALSE;

    if (node)
    {
        if (object_get_data(OBJ(node), "FileNodeEntry"))
        {
            if (!node->file && !node->fci[sm][st])
                hide = TRUE;
            else if (node->file && !node->fci[sm][st] && data->current_node == node->parent)
                hide = TRUE;
            else if (node->flags & FN_FLAG_UPDATE)
                ui_filetree_node_update(node, tree);
        }
        else
        {
            if (!node->file && node->fci[sm][st])
                ui_filetree_node_show(node, tree);
            else if (node->file && node->fci[sm][st] && data->current_node == node->parent)
                ui_filetree_node_show(node, tree);
        }
        child = node->child;
    }
    else
    {
        child = tree->files;
    }
    
    while (child)
    {
        file_node_t* temp = child->next;
        node_vis_action(tree, child);
        child = temp;
    }

    if (hide) ui_filetree_node_hide(node, tree);
}

static void ui_filetree_update(file_tree_t* tree)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
  
    if (!data) return;

    node_vis_action(tree, NULL);

    if (data->sum_label1)
    {
        const char* m = MNames(tree);
        char* str = l_printf("<b>%s</b>", m);
        gtk_label_set_markup(data->sum_label1, str);
    }
    if (data->sum_label2)
    {
        char* str = l_printf("<b>%s</b>",
                             print_filesize(&(tree->fsi[tree->show_media][tree->show_time])));
        gtk_label_set_markup(data->sum_label2, str);
    }
}

static void ui_filetree_hide(file_tree_t* tree)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");

    ui_filetree_hide_folder(tree);
    ui_filetree_hide_childs(tree);

    if (data->hide_timeout) g_source_remove(data->hide_timeout);
    data->hide_timeout = 0;
    if (data->search_timeout) g_source_remove(data->search_timeout);
    data->search_timeout = 0;
    
    object_free_data(OBJ(tree), "FileTreePage", NULL);
    data->tree = NULL;
    data->current_node = NULL;
    object_unmonitor_d(OBJ(tree), (ObjectFunc)ui_filetree_update, NULL);
}


static void on_time_activate(GObject* menuitem, file_tree_t* tree) {
  int num = GPOINTER_TO_INT(g_object_get_data(menuitem, "num"));
  tree->show_time = num;
  object_set_dirty(OBJ(tree));
}

static GtkWidget* on_file_tree_files1(tree_gui_t* data) {
  GtkWidget* popup;
  GtkWidget* item;
  int i2;
  file_tree_t* tree = data->tree;

  popup = gtk_menu_new();

  for (i2 = 0; i2 <= 3; i2++) {
    const char* m = TNames(i2);
    char* l = ngettext("%d item", "%d items", tree->fci[tree->show_media][i2]);
    
    if (i2 == 3) {
      item = gtk_menu_item_new();
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup), item);
      gtk_widget_set_sensitive(item, FALSE);
    }

    l = l_printf(l, tree->fci[tree->show_media][i2]);
    item = gtk_image_menu_item_new_with_label(l_printf("%s (%s)", m, l));

    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup), item);
    g_object_set_data(G_OBJECT(item), "num", GINT_TO_POINTER(i2));
    if (tree->fci[tree->show_media][i2] == 0)
      gtk_widget_set_sensitive(item, FALSE);
    else
      g_signal_connect(G_OBJECT(item), "activate",
                       G_CALLBACK(on_time_activate), tree);
  }
  return popup;
}

static void on_mime_activate(GObject* menuitem, file_tree_t* tree) {
  int num = GPOINTER_TO_INT(g_object_get_data(menuitem, "num"));
  tree->show_media = num;
  object_set_dirty(OBJ(tree));
}

static GtkWidget* on_file_tree_files2(tree_gui_t* data) {
  GtkWidget* popup;
  GtkWidget* item;
  int i1;
  file_tree_t* tree = data->tree;


  popup = gtk_menu_new();

  for (i1 = 0; i1 <= MEDIA_ANY; i1++) {
    const char* m = _(int2media(i1, 1));
    char* l = ngettext("%d item", "%d items", tree->fci[i1][tree->show_time]);

    if (i1 == MEDIA_ANY) {
      item = gtk_menu_item_new();
      gtk_widget_show(item);
      gtk_container_add(GTK_CONTAINER(popup), item);
      gtk_widget_set_sensitive(item, FALSE);
    }


    l = l_printf(l, tree->fci[i1][tree->show_time]);
    item = gtk_image_menu_item_new_with_label(l_printf("%s (%s)", m, l));

    gtk_image_menu_item_set_image
      (GTK_IMAGE_MENU_ITEM(item),
       gtk_image_new_from_stock(MediaStock[i1], GTK_ICON_SIZE_SMALL_TOOLBAR));
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup), item);
    g_object_set_data(G_OBJECT(item), "num", GINT_TO_POINTER(i1));
    if (tree->fci[i1][tree->show_time] == 0)
      gtk_widget_set_sensitive(item, FALSE);
    else
      g_signal_connect(G_OBJECT(item), "activate",
                       G_CALLBACK(on_mime_activate), tree);
  }

  return popup;
}

static gint sort_filenode_by_size(GtkTreeModel* model,
                                  GtkTreeIter* a, GtkTreeIter* b) {
  file_node_t* r1;
  file_node_t* r2;
  
  gtk_tree_model_get(model, a, 0, &r1, -1);
  gtk_tree_model_get(model, b, 0, &r2, -1);

  if (!r2->file) return !r1->file;
  if (!r1->file) return -1;

  if (r1->file->size < r2->file->size) return -1;
  if (r1->file->size > r2->file->size) return 1;
  return 0;
}

static file_node_t*
search_cb(file_tree_t* tree, file_node_t* node, tree_gui_t* data) {
  
  if (!tree || !node || !node->file) return NULL;
  if (node->fci[tree->show_media][tree->show_time] == 0) return NULL;

  if (data->search_token[0] != 0) {
    char* file = g_utf8_strdown(node->file->localname, -1);
    if (pattern_match(data->search_token, file))
      ui_filetree_node_show(node, tree);
    g_free(file);
  }
  /*
  if (data->search_token[0] == 0) {
    ui_filetree_node_show(tree, node);
  } else if (data->search_token[1] == 0) {
    if (strchr(node->file->longname, data->search_token[0]))
      ui_filetree_node_show(tree, node);
  } else {
    if (strcasestr(node->file->longname, data->search_token))
      ui_filetree_node_show(tree, node);
  }
  */
  return NULL;
}

static gboolean file_tree_search_files(file_tree_t* tree)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    
    data->search_timeout = 0;
    
    // clear the files list
    ui_filetree_hide_childs(tree);
  
    if (tree->files && data->search_token && data->search_token[0])
    {
        file_tree_action(tree, NULL, (FileNodeTraverse)search_cb, data);
    }
    
    return FALSE;
}

static void on_file_tree_search_changed(GtkEntry* entry, tree_gui_t* data) {
  const char* text;
  file_tree_t* tree;
  
  tree = data->tree;

  text = gtk_entry_get_text(entry);
  
  if (data->search_token) g_free(data->search_token);
  // we dont annoy users to enter '*pattern*' if there is no
  // wildcard
  if (text && *text && !strchr(text, '*') && !strchr(text, '?'))
    data->search_token = 
      g_utf8_strdown(l_printf("*%s*", text), -1);
  else
    data->search_token = g_utf8_strdown(text, -1);

  if (data->search_timeout)
    g_source_remove(data->search_timeout);

  data->search_timeout = 
    g_timeout_add(1000/(strlen(text)+1),
                  (GSourceFunc)file_tree_search_files, tree);
  return;
}


static GtkWidget* on_node_tip(file_node_t* node, unsigned depth) {
  (void)depth;
  if (node->file) {
    GtkWidget* table = gtk_table_new(1, 2, FALSE);
    int line = 0;

    gtk_table_set_col_spacings(GTK_TABLE(table), 10);
    ui_table_add_file(GTK_TABLE(table), &line, node->file);
    gtk_widget_show(table);
    return table;
  } else {
    return NULL;
  }
}


GtkWidget* ui_filetree_create_page(tree_gui_t* data, GtkBox* header, GtkBox* hline)
{
    GtkWidget* vbox0;
    GtkWidget* hpaned;
    GtkWidget* hbox;
    GtkWidget* button;
    GtkWidget* sw;
    GtkWidget* item;
    GtkWidget* progressbar;
    GtkTreeStore* store;
    GtkListStore* store2;
    GtkWidget* tv;
    GtkTreeViewColumn *column;
    GtkTreeSelection *select;
    GtkCellRenderer* rend;
    
    
    vbox0 = gtk_vbox_new(FALSE, 0);
    data->main_link = vbox0;
    
    if (!header)
    {
        hbox = gtk_hbox_new(FALSE, 5);
        gtk_widget_show(hbox);
        header = GTK_BOX(hbox);
    }
    if (!hline)
    {
        hline = header;
    }
    gtk_box_pack_start(GTK_BOX(vbox0), GTK_WIDGET(header), FALSE, FALSE, 0);
  

    hbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox);
    gtk_box_pack_start(hline, hbox, FALSE, FALSE, 10);

    item = gtk_label_new("");
    data->sum_label1 = GTK_LABEL(item);
    
    button = ui_create_popup_button
        (item, POS_LEFT_DOWN, (PopupFunction)on_file_tree_files2, data);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    
    item = gtk_label_new("");
    data->sum_label2 = GTK_LABEL(item);
    
    button = ui_create_popup_button
        (item, POS_RIGHT_DOWN, (PopupFunction)on_file_tree_files1, data);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(hline, hbox, TRUE, FALSE, 10);

    item = gtk_label_new(_("Search"));
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

    item = gtk_entry_new();
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    data->search_entry = item;
    g_signal_connect(G_OBJECT(data->search_entry), "changed",
                     G_CALLBACK(on_file_tree_search_changed), data);
    
    

    hpaned = gtk_hpaned_new();
    gtk_widget_show(hpaned);
    gtk_paned_set_position(GTK_PANED(hpaned), 199);
    gtk_box_pack_start(GTK_BOX(vbox0), hpaned, TRUE, TRUE, 0);
    data->paned = hpaned;
    
    // folders
    sw = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show(sw);
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
    gtk_paned_pack1(GTK_PANED(hpaned), sw, FALSE, TRUE);
    
    store = gtk_tree_store_new(4,
                               G_TYPE_POINTER, // data pointer
                               G_TYPE_STRING,  // folder
                               G_TYPE_OBJECT,
                               G_TYPE_BOOLEAN); // bg color
    data->folder_tree = store;
    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), 1, GTK_SORT_ASCENDING);
    
    tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL(store));
    data->folder_view = GTK_TREE_VIEW(tv);
    gtk_widget_show(tv);
    gtk_container_add(GTK_CONTAINER(sw), tv);
    gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tv), TRUE);
    //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
    
    select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
    //  gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
    gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
    
    
    // FIXME: 
    /*
      data->signal =
      g_signal_connect(G_OBJECT(select), "changed",
      G_CALLBACK(on_tree_selection_changed), data);
    */
    g_signal_connect(G_OBJECT(tv), "button-press-event",
                     G_CALLBACK(on_tree_button), data);
    
    rend = gtk_cell_renderer_pixbuf_new();
    g_object_set(G_OBJECT(rend), "xpad", 3, NULL);
    
    column = ui_tv_column_new
        (_("Folder"),
         rend/*gtk_cell_renderer_pixbuf_new()*/, "pixbuf", 2, FALSE,
         gtk_cell_renderer_text_new(), "text", 1, FALSE,
         NULL);
    ui_tv_config_default_col(column, 1, NULL, TRUE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
    
    ui_tv_config_line_marker(GTK_TREE_VIEW(tv), "filetree:recent", 3);
    
    
    // files
    sw = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show(sw);
    gtk_paned_pack2(GTK_PANED(hpaned), sw, TRUE, TRUE);
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
    
    store2 = gtk_list_store_new(6,
                                G_TYPE_POINTER, // data pointer
                                G_TYPE_STRING,  // filename
                                G_TYPE_OBJECT,  // pix
                                G_TYPE_BOOLEAN, // background
                                G_TYPE_STRING,  // size
                                G_TYPE_STRING   // meta data
                               );
    gtk_tree_sortable_set_sort_func
        (GTK_TREE_SORTABLE(store2),
         2, (GtkTreeIterCompareFunc)sort_filenode_by_size,
         NULL, NULL);
    data->file_list = store2;

    tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store2));
    data->file_view = GTK_TREE_VIEW(tv);
    gtk_container_add(GTK_CONTAINER(sw), tv);
    gtk_widget_show(tv);
    gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW (tv), TRUE);
    g_object_set(G_OBJECT(tv), "fixed-height-mode", TRUE, NULL);
    //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
    gtk_fancytips_new(GTK_TREE_VIEW(tv), (FancyTipsFunc)on_node_tip, 
                      GTK_FANCY_POS_MOUSE);
  
    select = gtk_tree_view_get_selection(GTK_TREE_VIEW (tv));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);

    rend = gtk_cell_renderer_pixbuf_new();
    g_object_set(G_OBJECT(rend), "xpad", 3, NULL);

    column = ui_tv_column_new
        (_("Filename"),
         rend/*gtk_cell_renderer_pixbuf_new()*/, "pixbuf", 2, FALSE,
         gtk_cell_renderer_text_new(), "text", 1, TRUE,
         NULL);
    ui_tv_config_default_col(column, 1, NULL, FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
    data->col_file[0] = column;
  
    column = ui_tv_column_new
        (_("Filesize"), gtk_cell_renderer_text_new(), "text", 4, TRUE, NULL);
    ui_tv_config_default_col(column, 4, NULL, FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
    data->col_file[1] = column;

    column = ui_tv_column_new
        (_("Meta"), gtk_cell_renderer_text_new(), "text", 5, TRUE, NULL);
    ui_tv_config_default_col(column, 5, NULL, TRUE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    data->col_file[2] = column;

    ui_tv_config_line_marker(GTK_TREE_VIEW(tv), "filetree:recent", 3);


    data->bottom_box = hbox = gtk_hbox_new (FALSE, 5);
/*   gtk_widget_show(hbox); */
    gtk_box_pack_start(GTK_BOX(vbox0), hbox, FALSE, FALSE, 0);

    item = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_widget_show(item);
    gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    data->abort_button = item;
    gtk_widget_set_sensitive(item, FALSE);

    progressbar = gtk_progress_bar_new();
    gtk_widget_show(progressbar);
    gtk_box_pack_end(GTK_BOX(hbox), progressbar, FALSE, FALSE, 0);
    data->progress = progressbar;

    item = gtk_label_new("");
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 3);
    gtk_misc_set_alignment(GTK_MISC(item), 1.0, 0.5);
    gtk_widget_set_size_request(item, 10, -1);
    data->status_label = item;


    gtk_widget_show(vbox0);
    return vbox0;
}

void ui_filetree_scroll_to_parent(file_tree_t* tree, file_node_t* node)
{
    file_node_t* parent = node->parent;
    GtkTreePath* path;
    GtkTreeIter* iter;
    GtkTreeModel* model;
    GtkTreeSelection* select;
    tree_gui_t* data;
    
    return;  //FIXME
    iter = object_get_data(OBJ(parent), "FileNodeEntry");
    if (!iter) return;
    data = object_get_data(OBJ(tree), "FileTreePage");

    // select the parent
    select = gtk_tree_view_get_selection(data->folder_view);
    
    // block signal handler to prevent file list to be reshown
    g_signal_handler_block(G_OBJECT(select), data->signal);
    gtk_tree_selection_select_iter(select, iter);
    g_signal_handler_unblock(G_OBJECT(select), data->signal);
    
    // scroll to parent
    model = gtk_tree_view_get_model(data->folder_view);
    path = gtk_tree_model_get_path(model, iter);
    if (!path) return;
    gtk_tree_view_expand_to_path(data->folder_view, path);
    gtk_tree_view_scroll_to_cell(data->folder_view, path, NULL,
                                 TRUE, 0.5, 0);
    gtk_tree_path_free(path);
}

static void ui_filetree_show(file_tree_t* tree)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");

    if (!data) return;
    data->tree = tree;
    ui_filetree_update(tree);
    object_monitor_d(OBJ(tree), (ObjectFunc)ui_filetree_update, NULL);
}

static void ui_filetree_expand(file_tree_t* tree, file_node_t* node)
{
    tree_gui_t* data = object_get_data(OBJ(tree), "FileTreePage");
    
    if (!data) return;

    if (node)
    {
        GtkTreeIter* iter = object_get_data(OBJ(node), "FileNodeEntry");
        GtkTreePath* path;
        
        if (!iter) return;
        path = gtk_tree_model_get_path(GTK_TREE_MODEL(data->folder_tree), iter);
        if (path)
        {
            gtk_tree_view_expand_row(data->folder_view, path, TRUE);
            gtk_tree_path_free(path);
        }
    }
    else
    {
        gtk_tree_view_expand_all(data->folder_view);
    }
}

void ui_filetree_assign_new_tree(tree_gui_t* data, file_tree_t* tree)
{
    if (data->tree == tree) return;

    if (data->tree) ui_filetree_hide(data->tree);

    if (tree)
    {
        object_set_data(OBJ(tree), "FileTreePage", data);
        ui_filetree_show(tree);
        ui_filetree_expand(tree, NULL);
    }
}

