/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.12 $
 * $Date: 2007/04/29 09:21:47 $
 * $Author: sgop $
 */

#include <string.h>

#include "lcore/preferences.h"
#include "lcore/mtypes.h"
#include "lcore/xml_handler.h"
#include "lcore/chat.h"
#include "lcore/library.h"
#include "lcore/string_list.h"
#include "lcore/plugin.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "lcore/legacy.h"
#include "lcore/download.h"
#include "ui_init.h"
#include "ui_support.h"

#ifdef WIN32
#include "lgtk2corealias.h"
#endif

typedef struct {
  GtkToggleButton* import;
} ui_lopster1_t;

typedef struct {
  GtkEntry* nick;
  GtkEntry* pass;
} ui_basic_t;

typedef struct {
  GtkEntry* incomplete;
  GtkEntry* download[MEDIA_ANY];
} ui_path_t;

typedef struct {
  GtkToggleButton* timport;
  GtkToggleButton* tnothing;
  gboolean done;
} ui_library_t;

typedef struct {
  GtkToggleButton* timport;
  GtkToggleButton* tnothing;
  gboolean done;
} ui_download_t;

typedef struct {
  PageCreateFunc create;
  char* des;
} ui_init_t;


static ui_dialog_t* Dialog = NULL;

/*
static int media_lopster_1to2(int type) {
  if (type == 0) return MEDIA_NONE;
  if (type == 1) return MEDIA_AUDIO;
  if (type == 2) return MEDIA_AUDIO;
  if (type == 3) return MEDIA_VIDEO;
  if (type == 4) return MEDIA_APPLICATION;
  if (type == 5) return MEDIA_IMAGE;
  if (type == 6) return MEDIA_TEXT;
  if (type == 7) return MEDIA_FOLDER; // should not happen
  return MEDIA_NONE;
}
*/

static GtkWidget* ui_init_create_page(ui_page_t* page) {
  ui_init_t* data = ui_page_get_data(page);

  if (data->des) {
    GtkWidget* item;
    GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(vbox);
  
    item = gtk_label_new(data->des);
    g_free(data->des);
    gtk_label_set_line_wrap(GTK_LABEL(item), TRUE);
    gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

    item = gtk_hseparator_new();
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 10);

    item = data->create(page);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

    return vbox;
  } else {
    return data->create(page);
  }
}

void ui_init_add(const char* title, const char* des,
                 PageCreateFunc create, PageChangeFunc enter, PageChangeFunc leave) {
  ui_init_t* data = g_malloc0(sizeof(*data));
  ui_page_t* page;
  
  data->create = create;
  data->des = g_strdup(des);

  if (!Dialog) {
    Dialog = ui_dialog_new(_("Startup Wizard"), FALSE);
    ui_dialog_set_modal(Dialog, TRUE);
  }
  page = ui_page_new(title, ui_init_create_page, enter, leave);
  ui_page_set_data(page, data);
  
  ui_dialog_add_page(Dialog, page);
  ui_dialog_start(Dialog);
}


static GtkWidget* ui_lopster1_create(ui_page_t* page) {
  GtkWidget* toggle;
  GtkWidget* vbox;
  ui_lopster1_t* data = g_malloc(sizeof(*data));

  ui_page_set_data(page, data);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);

  toggle = gtk_radio_button_new_with_label
    (NULL, _("Import basic settings from Lopster 1"));
  data->import = GTK_TOGGLE_BUTTON(toggle);
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);
  gtk_toggle_button_set_active(data->import, TRUE);

  toggle = gtk_radio_button_new_with_label
    (gtk_radio_button_get_group(GTK_RADIO_BUTTON(toggle)), _("Do nothing"));
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);

  return vbox;
}

static const char* ui_lopster1_enter(ui_page_t* page, gboolean next) {
  (void)page;
  (void)next;
  return NULL;
}

static const char* ui_lopster1_leave(ui_page_t* page, gboolean next) {
  ui_lopster1_t* data = ui_page_get_data(page);
  const char* text;

  if (!next) return NULL;
  if (!gtk_toggle_button_get_active(data->import)) return NULL;

  if ((text = legacy_pref_get_value("Name")) != NULL)
    pref_store_str("/core/user/nick", text);

  if ((text = legacy_pref_get_value("Password")) != NULL)
    pref_store_str("/core/user/pass", text);

  if ((text = legacy_pref_get_value("Linespeed")) != NULL) {
    int speed = atoi(text);
    if (speed <= 0)      pref_store("/core/line/type", LINE_OTHER);
    else if (speed <= 4) pref_store("/core/line/type", LINE_MODEM);
    else if (speed <= 6) pref_store("/core/line/type", LINE_ISDN);
    else if (speed <= 7) pref_store("/core/line/type", LINE_CABLE);
    else if (speed <= 8) pref_store("/core/line/type", LINE_DSL);
    else if (speed <= 9) pref_store("/core/line/type", LINE_T1);
    else if (speed <=10) pref_store("/core/line/type", LINE_T3);
    else                 pref_store("/core/line/type", LINE_OTHER);
  }

  if ((text = legacy_pref_get_value("NapigatorUseProxy")) != NULL)
    pref_store_str("/core/http/proxy/enabled", text);
  else if ((text = legacy_pref_get_value("NapListUseProxy")) != NULL)
    pref_store_str("/core/http/proxy/enabled", text);

  if ((text = legacy_pref_get_value("NapigatorProxyHost")) != NULL)
    pref_store_str("/core/http/proxy/host", text);
  else if ((text = legacy_pref_get_value("NapListProxyHost")) != NULL)
    pref_store_str("/core/http/proxy/host", text);
  
  if ((text = legacy_pref_get_value("NapigatorProxyPort")) != NULL)
    pref_store_str("/core/http/proxy/port", text);
  else if ((text = legacy_pref_get_value("NapListProxyPort")) != NULL)
    pref_store_str("/core/http/proxy/port", text);
  
  if ((text = legacy_pref_get_value("AllowedPorts")) != NULL)
    pref_store_str("/core/listen/range", text);
  
  if ((text = legacy_pref_get_value("Firewall")) != NULL)
    pref_store_str("/core/listen/firewalled", text);
  
  if ((text = legacy_pref_get_value("RejoinWhenKicked")) != NULL)
    pref_store_str("/core/chat/auto_rejoin", text);
  
  if ((text = legacy_pref_get_value("Incomplete")) != NULL)
    pref_store_str("/core/path/incomplete", text);
  
  if ((text = legacy_pref_get_value("Max_Downloads")) != NULL)
    pref_store_str("/core/download/limit", text);
  
  if ((text = legacy_pref_get_value("Default_Downloads")) != NULL)
    pref_store_str("/core/download/limit_per_user", text);
  
  if ((text = legacy_pref_get_value("DownloadBandwidth")) != NULL) {
    char* temp = l_strdup(text);
    char* pos = strchr(temp, ':');
    if (pos) *pos = 0;
    pref_store_str("/core/download/bandlimit", text);
  }
  
  if ((text = legacy_pref_get_value("Max_Uploads")) != NULL)
    pref_store_str("/core/upload/limit", text);
  
  if ((text = legacy_pref_get_value("Default_Uploads")) != NULL)
    pref_store_str("/core/upload/limit_per_user", text);
  
  if ((text = legacy_pref_get_value("UploadBandwidth")) != NULL) {
    char* temp = l_strdup(text);
    char* pos = strchr(temp, ':');
    if (pos) *pos = 0;
    pref_store_str("/core/upload/bandlimit", text);
  }
  
  {
    int hide = HIDE_NONE;
    if ((text = legacy_pref_get_value("ShowJoins")) != NULL &&
        strcmp(text, "0"))
      hide |= HIDE_JOINS;
    if ((text = legacy_pref_get_value("ShowParts")) != NULL &&
        strcmp(text, "0"))
      hide |= HIDE_PARTS;
    pref_store("/core/chat/hide", hide);
  }
  
  if ((text = legacy_pref_get_value("Piping")) != NULL) {
    int pipe = PIPE_NONE;
    int old_pipe = atoi(text);
    if (old_pipe & 0x20) pipe |= PIPE_PRIVATE;
    if (old_pipe & 0x08) pipe |= PIPE_GLOBAL;
    if (old_pipe & 0x04) pipe |= PIPE_OPERATOR;
    if (old_pipe & 0x02) pipe |= PIPE_CHAN_OP;
    pref_store("/core/chat/pipe", pipe);
  }  
  
  if ((text = legacy_pref_get_value("TimeStamps")) != NULL) {
    int value = atoi(text);
    if (value == 0) pref_store("/core/chat/timestamp", TIME_NONE);
    else if (value == 1) pref_store("/core/chat/timestamp", TIME_HOUR_MIN);
    else if (value == 2) pref_store("/core/chat/timestamp", TIME_HOUR_MIN_SEC);
  }

  if ((text = legacy_pref_get_value("Highlight")) != NULL) {
    char* temp = l_strdup(text);
    char* item = arg(temp, 2);
    while (item) {
      string_list_add("core:chat:hilight", item);
      item = arg(NULL, 2);
    }
  }
    
  if ((text = legacy_pref_get_value("AFKReason")) != NULL) {
    char* temp = l_strdup(text);
    char* item = arg(temp, 2);
    while (item) {
      string_list_add("core:user:away", item);
      item = arg(NULL, 2);
    }
  }
  
  return NULL;
}

static GtkWidget* ui_basic_create(ui_page_t* page) {
  GtkWidget* hbox;
  GtkWidget* item;
  GtkWidget* entry;
  ui_basic_t* data = g_malloc(sizeof(*data));

  ui_page_set_data(page, data);

  hbox = gtk_hbox_new(TRUE, 5);
  gtk_widget_show(hbox);

  entry = gtk_entry_new();
  data->nick = GTK_ENTRY(entry);
  item = ui_create_headed_item(_("default username"), entry);
  gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 5);

  entry = gtk_entry_new();
  //  gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
  data->pass = GTK_ENTRY(entry);
  item = ui_create_headed_item(_("default password"), entry);
  gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 5);

  return hbox;
}

static const char* ui_basic_enter(ui_page_t* page, gboolean next) {

  if (next) {
    ui_basic_t* data = ui_page_get_data(page);
    gtk_entry_set_text(data->nick, pref_get_ptr("/core/user/nick", char));
    gtk_entry_set_text(data->pass, pref_get_ptr("/core/user/pass", char));
  }
  return NULL;
}

static const char* ui_basic_leave(ui_page_t* page, gboolean next) {
  ui_basic_t* data = ui_page_get_data(page);
  const char* text;

  if (!next) return NULL;
  text = gtk_entry_get_text(data->nick);
  if (!text || *text == 0)
    return _("You need to specify a username!");
  if (strchr(text, ' '))
    return _("The username may not contain any spaces!");

  text = gtk_entry_get_text(data->pass);
  if (!text || *text == 0)
    return _("You need to specify a password!");
  if (strchr(text, ' '))
    return _("The password may not contain any spaces!");
  
  text = gtk_entry_get_text(data->nick);
  pref_store("/core/user/nick", text);
  text = gtk_entry_get_text(data->pass);
  pref_store("/core/user/pass", text);
  return NULL;
}

static void on_browse_folder(GtkEntry* entry)
{
    const char* c = gtk_entry_get_text(entry);
    char* name = 
        ui_select_folder(_("Choose folder"), (c && *c)?c:NULL, NULL);

    if (name) 
    {
        gtk_entry_set_text(entry, name);
        g_free(name);
    }
}

static GtkWidget* ui_path_create(ui_page_t* page) {
  GtkWidget* vbox;
  GtkWidget* item;
  GtkWidget* entry;
  ui_path_t* data = g_malloc0(sizeof(*data));
  int i1;

  ui_page_set_data(page, data);

  vbox = gtk_vbox_new(TRUE, 5);
  gtk_widget_show(vbox);

  entry = gtk_entry_new();
  data->incomplete = GTK_ENTRY(entry);

  item = ui_create_functional_headed_item
    (_("path for incomplete files"), entry,
     _("change"), G_CALLBACK(on_browse_folder));
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  for (i1 = MEDIA_UNKNOWN; i1 < MEDIA_FOLDER; i1++) {
    const char* path = legacy_path_get(i1);
    entry = gtk_entry_new();
    data->download[i1] = GTK_ENTRY(entry);
    
    if (path) gtk_entry_set_text(data->download[i1], path);
    
    item = ui_create_functional_headed_item
      (l_printf(_("path for %s"), int2media(i1, 1)), entry,
       _("change"), G_CALLBACK(on_browse_folder));
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
  }

  return vbox;
}

static const char* ui_path_enter(ui_page_t* page, gboolean next) {

  if (next) {
    ui_path_t* data = ui_page_get_data(page);
    gtk_entry_set_text(data->incomplete, pref_get_ptr("/core/path/incomplete", char));
  }

  return NULL;
}

static const char* ui_path_leave(ui_page_t* page, gboolean next) {
  ui_path_t* data = ui_page_get_data(page);
  const char* text;
  int i1;

  if (!next) return NULL;

  text = gtk_entry_get_text(data->incomplete);
  if (!text || *text == 0) //FIXME: check dir
    return _("Specify a valid path for incomplete files");

  text = gtk_entry_get_text(data->download[MEDIA_UNKNOWN]);
  if (!text || *text == 0) //FIXME: check dir
    return _("Specify a valid path for unknown files");

  
  text = gtk_entry_get_text(data->incomplete);
  pref_store("/core/path/incomplete", text);

  for (i1 = MEDIA_UNKNOWN; i1 < MEDIA_FOLDER; i1++) {
    text = gtk_entry_get_text(data->download[i1]);
    mtype_set_folder(i1, text);
  }
  return NULL;
}




static GtkWidget* ui_library_create(ui_page_t* page) {
  GtkWidget* vbox;
  GtkWidget* toggle;
  ui_library_t* data = g_malloc0(sizeof(*data));

  ui_page_set_data(page, data);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);

  toggle = gtk_radio_button_new_with_label
    (NULL, _("Import library from Lopster 1"));
  data->timport = GTK_TOGGLE_BUTTON(toggle);
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);

  toggle = gtk_radio_button_new_with_label
    (gtk_radio_button_get_group(GTK_RADIO_BUTTON(toggle)), _("Setup library later"));
  data->tnothing = GTK_TOGGLE_BUTTON(toggle);
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);

  gtk_toggle_button_set_active(data->timport, TRUE);
  return vbox;
}

static const char* ui_library_enter(ui_page_t* page, gboolean next) {
  ui_library_t* data = ui_page_get_data(page);
  (void)next;
  
  if (data->done) {
    gtk_widget_set_sensitive(GTK_WIDGET(data->timport), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(data->tnothing), FALSE);
    return _("Library was already imported");
  }
  return NULL;
}

static const char* ui_library_import(library_t* lib, FILE* file) {
  char line[2048];
  char *s1, *s2;
  int lines, i1;
  
  if (!mfgets(line, sizeof(line), file))
    return _("Failed to import library from Lopster 1");
  
  s1 = arg(line, 0);
  s2 = arg(NULL, 0);
  if (!s2)
    return _("Failed to import library from Lopster 1");

  if (!strcmp(s1, "LIBRARY")) {
    if (!mfgets(line, sizeof(line), file)) 
      return _("Failed to import library from Lopster 1");
    s1 = arg(line, 0);
    s2 = arg(NULL, 0);
    if (!s2)
      return _("Failed to import library from Lopster 1");
  }
  
  if (strcmp(s1, "Section"))
    return _("Failed to import library from Lopster 1");
  if (strcmp(s2, "[Folders]"))
    return _("Failed to import library from Lopster 1");

  if (!mfgets(line, sizeof(line), file))
    return _("Failed to import library from Lopster 1");

  s1 = arg(line, 0);
  s2 = arg(NULL, 0);
  if (!s2)
    return _("Failed to import library from Lopster 1");
  if (strcmp(s1, "Lines"))
    return _("Failed to import library from Lopster 1");
  lines = atoi(s2);
  for (i1 = 0; i1 < lines; i1++) {
    if (!mfgets(line, sizeof(line), file))
      return _("Failed to import library from Lopster 1");
    s1 = arg(line, 2);
    s2 = arg(NULL, 2);
    if (!s2)
      return _("Failed to import library from Lopster 1");
    
    //    folder->type = media_lopster_1to2(atoi(s2));
    lib->shared_folder = g_list_append(lib->shared_folder, g_strdup(s1));
  }
  return NULL;
}

static const char* ui_library_leave(ui_page_t* page, gboolean next) {
  ui_library_t* data = ui_page_get_data(page);
  library_t* lib = library_get_current();

  if (!next) return NULL;
  if (data->done) return NULL;
  if (lib->shared_folder) return NULL;

  if (gtk_toggle_button_get_active(data->timport)) {
    const char* fname = legacy_pref_get_value("AutoSave");
    char* filename = l_printf
      (SDIR("%s") SDIR(".lopster") SDIR("share") "%s", g_get_home_dir(), fname);
    FILE* file;
    const char* res;

    file = fopen(filename, "r");
    if (!file)
      return _("Failed to import library from Lopster 1");
    
    res = ui_library_import(lib, file);
    fclose(file);
    if (res) return res;

    data->done = TRUE;
  }
  return NULL;
}

static GtkWidget* ui_download_create(ui_page_t* page) {
  GtkWidget* vbox;
  GtkWidget* toggle;
  ui_download_t* data = g_malloc0(sizeof(*data));

  ui_page_set_data(page, data);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);

  toggle = gtk_radio_button_new_with_label
    (NULL, _("Continue incomplete downloads from Lopster 1"));
  data->timport = GTK_TOGGLE_BUTTON(toggle);
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);

  toggle = gtk_radio_button_new_with_label
    (gtk_radio_button_get_group(GTK_RADIO_BUTTON(toggle)), 
     _("Leave incomplete downloads at Lopster 1"));
  data->tnothing = GTK_TOGGLE_BUTTON(toggle);
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);

  gtk_toggle_button_set_active(data->tnothing, TRUE);

  return vbox;
}

static const char* ui_download_enter(ui_page_t* page, gboolean next) {
  ui_download_t* data = ui_page_get_data(page);
  (void)next;
  
  if (data->done) {
    gtk_widget_set_sensitive(GTK_WIDGET(data->timport), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(data->tnothing), FALSE);
    return _("Incomplete downloads already imported");
  }
  return NULL;
}

static const char* ui_download_import(FILE* file) {
  download_import_from_lopster1(file);
  return NULL;
}

static const char* ui_download_leave(ui_page_t* page, gboolean next) {
  ui_download_t* data = ui_page_get_data(page);

  if (!next) return NULL;
  if (data->done) return NULL;

  if (gtk_toggle_button_get_active(data->timport)) {
    char* filename = l_printf
      (SDIR("%s") SDIR(".lopster") "incomplete.list", g_get_home_dir());
    FILE* file;
    const char* res;

    file = fopen(filename, "r");
    if (!file) return _("Failed to import incomplete downloads from Lopster 1");
    
    res = ui_download_import(file);
    fclose(file);
    if (res) return res;
    
    // truncate the file
    fopen(filename, "w");
    fclose(file);
    
    data->done = TRUE;
  }
  return NULL;
}


void ui_init_init() {
  
  if (!lopster_xml_loaded("core:prefs")) {
    if (legacy_pref_available()) {
      ui_init_add(_("Import from Lopster 1"),
                  _("If you wish, Lopster 2 can import some basic settings "
                    "from your Lopster 1 installation. You then can adjust "
                    "a few settings in the following pages. Later you will "
                    "be able to change any setting in the preferences dialog."),
                  ui_lopster1_create, ui_lopster1_enter, ui_lopster1_leave);
    }
    ui_init_add(_("Basic settings"),
                _("Setup a core username and password. These settings are "
                  "required because they are used whenever a sub-module "
                  "doesn't set an own username or password.\n"
                  "Note: No spaces are allowed within the entries!"),
                ui_basic_create, ui_basic_enter, ui_basic_leave);

    ui_init_add(_("Download paths"),
                _("Specify a path where lopster should store files that are "
                  "downloaded."),
                ui_path_create, ui_path_enter, ui_path_leave);
  }

  if (legacy_pref_available()) {

    if (!lopster_xml_loaded("core:library")) {
      const char* fname = legacy_pref_get_value("AutoSave");
      if (fname) {
        char* filename = l_printf
          (SDIR("%s") SDIR(".lopster") SDIR("share") "%s", g_get_home_dir(), fname);
        if (g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
          ui_init_add(_("Library"),
                      _("If you import the library from Lopster 1 you need to "
                        "refresh the library when you are done with this dialog."),
                      ui_library_create, ui_library_enter, ui_library_leave);
        }
      }
    }
    
    if (plugin_get_old_version(NULL) == 0 || !lopster_xml_loaded("core:downloads")) {
      ui_init_add(_("Downloads"),
                  _("If you wish, Lopster 2 can import your incomplete downloads "
                    "from your Lopster 1 installation. If you import them "
                    "you will not be able to resume the files in Lopster 1 anymore."),
                  ui_download_create, ui_download_enter, ui_download_leave);
    }

  }
}

#ifdef WIN32
#define _UI_INIT_C_
#include "lgtk2corealiasdef.c"
#endif
