/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _UI_LAYOUT_H_
#define _UI_LAYOUT_H_

#include <gtk/gtk.h>

/** @defgroup UILayoutModule Keep track of the GUI layout.
 * With this framework you can easily save/load the layout of GUI
 * components. You must register a layout using ui_layout_register()
 * or one of its predefined wrappers and then monitor the components
 * from which the layout should be saved.
 *
 * @ingroup UserInterface
 */
/*@{*/

/** The layout getter. This encodes the layout of an object into a
 * string.
 * @param object the object of interest.
 * @return a dynamically allocated string containing the layout
 */
typedef char* (*LayoutGetFunc)(void* object);
/** The layout setter. This decodes a string and sets the layout of an
 * object accordingly.
 * @param object the object of interest.
 * @param layout the layout as string
 */
typedef void (*LayoutSetFunc)(void* object, const char* layout);


/** @name Registering layouts */
/*@{*/
void ui_layout_register      (const char* name, LayoutGetFunc get, LayoutSetFunc set);
void ui_layout_register_paned(const char* name);
void ui_layout_register_col  (const char* name);
void ui_layout_register_vis  (const char* name);
void ui_layout_register_exp  (const char* name);
void ui_layout_register_win  (const char* name);
void ui_layout_register_spin (const char* name);
void ui_layout_register_int  (const char* name);
void ui_layout_register_bool (const char* name);
//void ui_layout_register_win_size(const char* name);
//void ui_layout_register_win_pos(const char* name);
//void ui_layout_register_col_size(const char* name);
//void ui_layout_register_col_vis(const char* name);
/*@}*/

/** @name Using layouts */
/*@{*/
void ui_layout_monitor  (const char* name, void* object);
void ui_layout_unmonitor(const char* name, void* object);
/*@}*/


/*@}*/

void ui_layout_init();
void ui_layout_save();


#endif
