/* Copyright (C) 2005-2006 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.14 $
 * $Date: 2006/09/26 16:33:41 $
 * $Author: sgop $
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifndef WIN32
#include <unistd.h>
#endif

#include <gtk/gtk.h>

#include "ui_support.h"
#include "ui_filetree.h"
#include "ui_library.h"
#include "ui_main.h"
#include "ui_support.h"
#include "ui_layout.h"
#include "ui_treeview.h"
#include "ui_style.h"
#include "lcore/preferences.h"
#include "lcore/library.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"

#ifndef S_ISREG
#ifdef WIN32
#define S_ISREG( m ) ((( m ) & _S_IFMT ) == _S_IFREG )
#endif
#endif


static tree_gui_t* LibGui = NULL;
static GtkEntry* SaveEntry = NULL;
static library_t* CurrentLib = NULL;
static GtkWidget* RefreshLabel = NULL;

static void on_delete_list(GtkMenuItem *menuitem) {
  GtkLabel *label;
  char *s1;

  label = GTK_LABEL(GTK_BIN(menuitem)->child);
  s1 = label->label;
  library_delete_on_disk(s1);
}

static void on_load_list(GtkBin* menuitem) {
  GtkLabel* label = GTK_LABEL(gtk_bin_get_child(menuitem));
  const char* text = gtk_label_get_text(label);

  pref_store("/core/lib/current", text);
}

void ui_lib_working(gboolean val)
{
    if (val)
    {
        ui_event_label_set_sensitive(RefreshLabel, FALSE);
        ui_file_tree_set_abort(FILE_TREE(CurrentLib), (GFunc)library_abort_refresh, NULL);
    }
    else
    {
        ui_event_label_set_sensitive(RefreshLabel, TRUE);
        ui_file_tree_set_abort(FILE_TREE(CurrentLib), NULL, NULL);
    }
}

static GtkWidget* ui_create_manage_popup(void) {
  GtkWidget* popup;
  GtkWidget* menu1 = NULL;
  GtkWidget* menu2 = NULL;
  GtkWidget* item;
  GtkWidget* item1;
  GtkWidget* item2;
  char* dirname;
  const char* entry;
  struct stat buf;
  GDir* dir;
  int cnt = 0;

  dirname = g_strdup_printf(SDIR("%s") "share", lopster_get_home());

  popup = gtk_menu_new();

  item1 = gtk_menu_item_new_with_label(_("Delete List"));
  gtk_widget_show(item1);
  gtk_container_add(GTK_CONTAINER(popup), item1);

  item = gtk_menu_item_new();
  gtk_widget_show(item);
  gtk_widget_set_sensitive(item, FALSE);
  gtk_container_add(GTK_CONTAINER(popup), item);

  item2 = gtk_menu_item_new_with_label(_("Load List"));
  gtk_widget_show(item2);
  gtk_container_add(GTK_CONTAINER(popup), item2);


  if ((dir = g_dir_open(dirname, 0, NULL)) == NULL) {
    g_warning("could not open dir [%s]", dirname);
  } else {
    while ((entry = g_dir_read_name(dir)) != NULL) {
      //      if (!strncmp(entry, ".", 1)) continue;
      //      if (!strncmp(entry, "..", 2)) continue;
      char* filename = l_printf(SDIR("%s") "%s", dirname, entry);
      stat(filename, &buf);
      if (S_ISREG(buf.st_mode)) {
        if (!cnt) {
          menu1 = gtk_menu_new();
          gtk_widget_show(menu1);
          gtk_menu_item_set_submenu(GTK_MENU_ITEM(item1), menu1);
          menu2 = gtk_menu_new();
          gtk_widget_show(menu2);
          gtk_menu_item_set_submenu(GTK_MENU_ITEM(item2), menu2);
        }
        item = gtk_menu_item_new_with_label(entry);
        gtk_widget_show(item);
        gtk_container_add(GTK_CONTAINER(menu1), item);
        g_signal_connect(G_OBJECT(item), "activate",
                         G_CALLBACK(on_delete_list), NULL);
        item = gtk_menu_item_new_with_label(entry);
        gtk_widget_show(item);
        gtk_container_add(GTK_CONTAINER(menu2), item);
        g_signal_connect(G_OBJECT(item), "activate",
                         G_CALLBACK(on_load_list), NULL);
        cnt++;
      }
    }
    g_dir_close(dir);
  }

  if (!cnt) {
    gtk_widget_set_sensitive(item1, FALSE);
    gtk_widget_set_sensitive(item2, FALSE);
    //    gtk_widget_set_sensitive(item3, FALSE);
  }

  g_free(dirname);

  return popup;
}

static void on_library_save(void)
{
    const char *text;
    
    text = gtk_entry_get_text(SaveEntry);
    if (*text == 0)
    {
        lopster_notify(LOPSTER_MESSAGE_INFO, _("Missing filename"),
                       _("Please enter a filename for the library!"));
        return;
    }
    library_save_as(text);
}

static void on_library_rebuild(void) {
  library_refresh(TRUE);
}


typedef struct {
  char* pref;
  guint32 range;
  guint32 value;
  int x_drag_start;
  int drag_range;
  GtkWidget* da;
  int timer;
  int sec;
  int min;
  int hour;
  int day;
  int year;
} ui_mark_t;

static void _mark_force_update(GtkWidget* da) {
  if (da->window) gdk_window_invalidate_rect(da->window, NULL, FALSE);
}

static void _ui_mark_set_val(ui_mark_t* data, int value) {
  int sec = value;
  int min = sec / 60;
  int hour = min / 60;
  int day = hour / 24;
  int year = day / 365;

  sec = sec % 60;
  min = min % 60;
  hour = hour % 24;
  day = day % 365;

  if (data->year)
    { if (data->year == year && data->day == day) return; }
  else if (data->day && !year)
    { if (data->day == day && data->hour == hour) return; }
  else if (data->hour && !year && !day)
    { if (data->hour == hour && data->min == min) return; }
  else if (data->min && !year && !day && !hour)
    { if (data->min == min && data->sec == sec) return; }
  else if (data->sec && !year && !day && !hour && !min)
    { if (data->sec == sec) return; }

  data->value = value;
  data->year = year;
  data->day = day;
  data->hour = hour;
  data->min = min;
  data->sec = sec;
  if (data->pref) pref_store(data->pref, data->value);
  if (data->da) _mark_force_update(data->da);
}

static int calc_x(long val, long max, int width) {
  if (val >= max) return width;
  return (guint64)val * (guint64)width  / (guint64)max;
}

static gboolean
on_mark_expose(GtkWidget* widget, GdkEventExpose* event, ui_mark_t* data) {
  int width = widget->allocation.width;
  int height = widget->allocation.height;
  int texth, posx;
  PangoLayout* layout;
    
  (void)event;

  layout = ui_get_text_layout(widget);
  ui_layout_get_ext(layout, "1 day 1 hour 1 minute", NULL, &texth);
  texth += 4;

  gdk_draw_rectangle
    (widget->window, ui_style_get_gc("bandwidth:color", "Base"),
     TRUE, 0, 0, width, height);
  gdk_draw_rectangle
    (widget->window, ui_style_get_gc("bandwidth:color", "Border"),
     FALSE, 0, 0, width-1, height-1);
  
  posx = calc_x(data->value, data->range, width-4);
  gdk_draw_rectangle
    (widget->window, ui_style_get_gc("bandwidth:color", "Bar"),
     TRUE, 2, 2, posx, height-4);

  {
    const char* str;
    int sec = data->sec;
    int min = data->min;
    int hour = data->hour;
    int day = data->day;
    int year = data->year;
    char* s_sec = l_printf(ngettext("%d second", "%d seconds", sec), sec);
    char* s_min = l_printf(ngettext("%d minute", "%d minutes", min), min);
    char* s_hour = l_printf(ngettext("%d hour", "%d hours", hour), hour);
    char* s_day = l_printf(ngettext("%d day", "%d days", day), day);
    char* s_year = l_printf(ngettext("%d year", "%d years", year), year);
    int x1, dx;

    if (year) {
      if (day)
        str = l_printf(_("Mark files that were modified the past %s %s"), s_year, s_day);
      else
        str = l_printf(_("Mark files that were modified the past %s"), s_year);
    } else if (day) {
      if (hour)
        str = l_printf(_("Mark files that were modified the past %s %s"), s_day, s_hour);
      else
        str = l_printf(_("Mark files that were modified the past %s"), s_day);
    } else if (hour) {
      if (min)
        str = l_printf(_("Mark files that were modified the past %s %s"), s_hour, s_min);
      else
        str = l_printf(_("Mark files that were modified the past %s"), s_hour);
    } else if (min) {
      if (sec)
        str = l_printf(_("Mark files that were modified the past %s %s"), s_min, s_sec);
      else
        str = l_printf(_("Mark files that were modified the past %s"), s_min);
    } else {
      str = l_printf(_("Mark files that were modified the past %s"), s_sec);
    }
    pango_layout_set_text(layout, str, -1);
    gdk_draw_layout(widget->window, ui_style_get_gc("bandwidth:color", "Text"),
                    6, (height-texth)/2, layout);
    
    
    sec = data->range / 30;
    min = sec / 60;
    hour = min / 60;
    day = hour / 24;
    year = day / 365;

    dx = 1;
    if (year >= 10) dx = 60*60*24*365*10;
    else if (year >= 2) dx = 60*60*24*365*2;
    else if (day >= 365) dx = 60*60*24*365;
    else if (day >= 94) dx = 60*60*24*365/4;
    else if (day >= 30) dx = 60*60*24*365/12;
    else if (day >= 7) dx = 60*60*24*7;
    else if (day >= 1) dx = 60*60*24;
    else if (hour >= 6) dx = 60*60*6;
    else if (hour >= 1) dx = 60*60;
    else if (min >= 15) dx = 60*15;
    else if (min >= 3) dx = 60*3;
    else if (min >= 1) dx = 60;
    else if (sec >= 15) dx = 15;
    else if (sec >= 3) dx = 3;
    else dx = 1;
  
    x1 = 0;
    while (x1 <= data->range) {
      posx = calc_x(x1, data->range, width-4);
      gdk_draw_line(widget->window, ui_style_get_gc("bandwidth:color", "Border"),
                    posx+2, 0, posx+2, 5);
      x1 += dx;
    }
  }


  return TRUE;
}

static gboolean
on_mark_pressed(GtkWidget* widget, GdkEventButton* event, ui_mark_t* data) {
  int width = widget->allocation.width - 4;
  int x = (int)event->x - 2;

  if (x < 0) x = 0;
  if (x > width) x = width;

  if (event->button == 1) {
    if (x == 0) _ui_mark_set_val(data, 0);
    else _ui_mark_set_val(data, ((guint64)data->range * (guint64)x -1) / (guint64)width +1);
  } else {
    data->x_drag_start = x;
    data->drag_range = data->range;
  }

  return TRUE;
}

static gboolean
on_mark_motion(GtkWidget* widget, GdkEventMotion* event, ui_mark_t* data) {
  int width = widget->allocation.width - 4;
  int x = (int) event->x - 2;

  if (x < 0) x = 0;
  if (x > width) x = width;

  if (event->state & GDK_BUTTON1_MASK) {
    if (x == 0) _ui_mark_set_val(data, 0);
    else _ui_mark_set_val(data, ((guint64)data->range * (guint64)x -1) / (guint64)width +1);
    _mark_force_update(data->da);
  } else if (data->x_drag_start > 0 && x > 0) {
    data->range = (guint64)data->drag_range * (guint64)data->x_drag_start / (guint64)x;
    if (data->range < 16) data->range = 16;
    else if (data->range > 1024*1024*1024) 
      data->range = 1024*1024*1024;
    _mark_force_update(data->da);
  }

  return TRUE;
}

static void pref_to_mark(pref_node_t* node, ui_mark_t* data) {
  _ui_mark_set_val(data, *((int*)node->value));
}

GtkWidget* ui_create_library_content()
{
    GtkWidget* vbox;
    GtkWidget* hbox;
    GtkBox* header;
    GtkWidget* hbox2;
    GtkWidget* button;
    GtkWidget* entry;
    GtkWidget* item;
    GtkWidget* tbox;
    tree_gui_t* data;
    GtkRequisition req;

    data = g_malloc0(sizeof(tree_gui_t));
    LibGui = data;
  

    tbox = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(tbox);

    // first header line
    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(tbox), hbox, FALSE, FALSE, 0);
    header = GTK_BOX(hbox);

    // separator and controls
    vbox = gtk_vbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(tbox), vbox, FALSE, FALSE, 0);

    item = gtk_hseparator_new();
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 5);

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
    ui_layout_monitor("core:lib:controls:vis", vbox);

    // fill first line
    item = gtk_image_new_from_stock(STOCK_LOPSTER_LIBRARY, GTK_ICON_SIZE_SMALL_TOOLBAR);
    gtk_widget_show(item);
    gtk_box_pack_start(header, item, FALSE, FALSE, 0);

    item = ui_create_header_label(_("Library"));
    gtk_widget_show(item);
    gtk_box_pack_start(header, item, FALSE, FALSE, 0);

    RefreshLabel = item = ui_create_event_label
        (_("rescan library"), G_CALLBACK(on_library_rebuild), NULL, TRUE);
    gtk_widget_show(item);
    gtk_box_pack_end(header, item, FALSE, FALSE, 0);
  
    item = ui_create_show_hide_label(_("show controls"), _("hide controls"), vbox);
    gtk_widget_show(item);
    gtk_box_pack_end(header, item, FALSE, FALSE, 0);


    {
        GtkWidget* da = gtk_drawing_area_new();
        ui_mark_t* data = g_malloc0(sizeof(*data));

        data->da = da;
        data->pref = g_strdup("/core/lib/mark_val");
        _ui_mark_set_val(data, pref_get_ref("/core/lib/mark_val", int));
        data->range = data->value*2;
        if (data->range < data->value) data->range = data->value;
        if (data->range == 0) data->range = 60*60*24*7;
    
        gtk_widget_set_events(da, GDK_BUTTON1_MOTION_MASK|GDK_BUTTON3_MOTION_MASK|GDK_BUTTON_PRESS_MASK);
        gtk_widget_show(da);
        gtk_box_pack_start(GTK_BOX(hbox), da, TRUE, TRUE, 0);

        g_signal_connect(G_OBJECT(da), "expose-event",
                         G_CALLBACK(on_mark_expose), data);
        g_signal_connect(G_OBJECT(da), "button-press-event",
                         G_CALLBACK(on_mark_pressed), data);
        g_signal_connect(G_OBJECT(da), "motion-notify-event",
                         G_CALLBACK(on_mark_motion), data);
        pref_observe(data->pref, (ObjectFunc)pref_to_mark, data);
    }  
  

    hbox2 = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox2);
    gtk_box_pack_start(GTK_BOX (hbox), hbox2, FALSE, FALSE, 0);
  
    button = ui_create_popup_button
        (gtk_label_new(_("Manage")), POS_LEFT_DOWN, (PopupFunction)ui_create_manage_popup, NULL);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox2), button, FALSE, FALSE, 0);
  
    entry = gtk_entry_new();
    gtk_widget_show(entry);
    gtk_box_pack_start(GTK_BOX (hbox2), entry, FALSE, FALSE, 0);
    gtk_widget_set_size_request(entry, 100, -1);
    SaveEntry = GTK_ENTRY(entry);
  
    gtk_widget_size_request(entry, &req);
    gtk_widget_set_size_request(hbox2, -1, req.height);
  
    
    item = gtk_button_new_from_stock(GTK_STOCK_SAVE);
    gtk_widget_set_size_request(item, -1, req.height);
    gtk_widget_show(item);
    g_signal_connect_swapped(G_OBJECT(item), "clicked",
                             G_CALLBACK(on_library_save), NULL);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);



    vbox = ui_filetree_create_page(data, GTK_BOX(tbox), header);
    gtk_box_set_spacing(GTK_BOX(vbox), 5);
  
    ui_layout_monitor("core:lib:paned:pos", data->paned);
    ui_layout_monitor("core:lib:col:filename:size", data->col_file[0]);
    ui_layout_monitor("core:lib:col:filename:vis", data->col_file[0]);
    ui_layout_monitor("core:lib:col:filesize:size", data->col_file[1]);
    ui_layout_monitor("core:lib:col:filesize:vis", data->col_file[1]);
    ui_layout_monitor("core:lib:col:meta:size", data->col_file[2]);
    ui_layout_monitor("core:lib:col:meta:vis", data->col_file[2]);

    gtk_widget_show(vbox);
    return vbox;
}

static void ui_lib_destroy(library_t* lib)
{
    if (LibGui->tree == FILE_TREE(lib))
    {
        ui_filetree_assign_new_tree(LibGui, NULL);
        CurrentLib = NULL;
    }
}

void ui_lib_new(library_t* lib)
{
    file_tree_t* tree = FILE_TREE(lib);
    
    if (!LibGui) return;
    
    if (CurrentLib == lib) return;
    if (!lib) return;
    
    CurrentLib = lib;
    if (object_get_data(OBJ(lib), "FileTreePage")) return;   // already shown
    
    ui_filetree_assign_new_tree(LibGui, tree);
    
    // additional lib signals
    {
        action_item_t* actions =
            action_item_search("Library folder actions");
        ui_tv_handle_button(GTK_TREE_VIEW(LibGui->folder_view),
                            1, &actions, lib);
    }
    {
        action_item_t* actions =
            action_item_search("Library file actions");
        ui_tv_handle_button(GTK_TREE_VIEW(LibGui->file_view),
                            1, &actions, lib);
    }
    
    gtk_entry_set_text(SaveEntry, OBJ(tree)->name);
    object_notify_d(OBJ(lib), (ObjectFunc)ui_lib_destroy, NULL);
    
    return;
}

