/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.29 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdlib.h>
#include "lcore/l_time.h"

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "ui_support.h"
#include "ui_search.h"
#include "ui_library.h"
#include "ui_network.h"
#include "ui_filetree.h"
#include "ui_support.h"
#include "ui_chat.h"
#include "ui_buddy.h"
#include "ui_main.h"
#include "ui_style.h"
#include "ui_transfer.h"
#include "ui_upload.h"
#include "ui_download.h"
#include "ui_layout.h"
#include "ui_preferences.h"
#include "ui_init.h"
#include "ui_notebook.h"
#include "ui_treeview.h"
#include "ui_about.h"
#include "lcore/ui.h"
#include "lcore/l_base.h"
#include "lcore/plugin.h"
#include "lcore/library.h"
#include "lcore/search.h"
#include "lcore/mtypes.h"
#include "lcore/buddy.h"
#include "lcore/preferences.h"
#include "lcore/l_base.h"
#include "lcore/thread.h"
#include "lcore/action_item.h"
#include "lcore/version.h"
#include "lcore/l_limit.h"
#include "lcore/string_list.h"
#include "lcore/l_i18n.h"
#ifdef WIN32
#include "lgtk2corealias.h"
#endif

#define UI_BOOT_TIME    0
#define UI_UP_TIME      1
#define UI_CURRENT_TIME 2

const char* ThePluginName = N_("Gtk+-2.0 - The GIMP Toolkit");

typedef struct {
  GtkWidget* win;
  GtkLabel* header;
  GtkBox* box;
} ui_thread_t;

typedef struct {
  char* name;
  guint32 range;
  GtkWidget* widget;
  volume_t* vol;
  rate_t* rate;
  int x_drag_start;
  int drag_range;
} ui_bw_t;


static const char* TabNames[TAB_NUMBER] = {
  N_("Networks"),
  N_("Search"),
  N_("Transfers"),
  N_("Buddies"),
  N_("Chat"),
  N_("Library"),
};

const char* MediaStock[MEDIA_ANY+1] = {
  STOCK_LOPSTER_MTYPE_OTHER,
  STOCK_LOPSTER_MTYPE_AUDIO,
  STOCK_LOPSTER_MTYPE_VIDEO,
  STOCK_LOPSTER_MTYPE_APP,
  STOCK_LOPSTER_MTYPE_IMAGE,
  STOCK_LOPSTER_MTYPE_DOC,
  STOCK_LOPSTER_MTYPE_FOLDER, //FIXME
  STOCK_LOPSTER_MTYPE_ALL,
};

static const char* TabIcon[TAB_NUMBER] = {
  STOCK_LOPSTER_NETWORK,

  STOCK_LOPSTER_SEARCH,
  STOCK_LOPSTER_TRANSFER,

  STOCK_LOPSTER_HOTLIST,
  STOCK_LOPSTER_CHAT,

  STOCK_LOPSTER_LIBRARY,
};

static ui_nb_t* MainNotebook = NULL;
static GtkWidget* MainWin = NULL;
static int CurrentIndex = -1;

static GList* Threads = NULL;
static ui_thread_t* ThreadGui = NULL;

static GtkButton* TimeButton = NULL;
static GtkLabel* TimeLabel = NULL;
static int TimeFormat = 1;



static void ui_enter_main_loop(void);
static void ui_leave_main_loop(void);
static void ui_thread_new(thread_t* thread);

GtkWindow* ui_main_get_win(void)
{
    return GTK_WINDOW(MainWin);
}

static void ui_string_list_remove(GList* items, const char* name) {
  GList* dlist;
  for (dlist = items; dlist; dlist = dlist->next) {
    const char* item = dlist->data;
    string_list_remove(name, item);
  }
}

static void ui_user_info(int type, const char* header, const char* text)
{
    GtkWidget* win;
    GtkMessageType t;
    const char* title;
    
    if (!MainWin) return;
    
    t =
        (type==LOPSTER_MESSAGE_INFO)    ?GTK_MESSAGE_INFO    :
        (type==LOPSTER_MESSAGE_WARNING) ?GTK_MESSAGE_WARNING :
        (type==LOPSTER_MESSAGE_QUESTION)?GTK_MESSAGE_QUESTION:
        (type==LOPSTER_MESSAGE_ERROR)   ?GTK_MESSAGE_ERROR   :
        GTK_MESSAGE_INFO;
    title =
        (type==LOPSTER_MESSAGE_INFO)    ?_("Information"):
        (type==LOPSTER_MESSAGE_WARNING) ?_("Warning")    :
        (type==LOPSTER_MESSAGE_QUESTION)?_("Question")   :
        (type==LOPSTER_MESSAGE_ERROR)   ?_("Error")      :
        _("Information");
    
    win = gtk_message_dialog_new
        (GTK_WINDOW(MainWin),
         GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT,
         t, GTK_BUTTONS_CLOSE, header);
    gtk_message_dialog_format_secondary_markup
        (GTK_MESSAGE_DIALOG(win), text);
    gtk_window_set_title(GTK_WINDOW(win), title);

    g_signal_connect_swapped(win, "response", G_CALLBACK(gtk_widget_destroy), win);
    gtk_widget_show_all(win);
}


void ui_init(ui_interface_t* iface)
{
    iface->enter_main_loop = ui_enter_main_loop;
    iface->leave_main_loop = ui_leave_main_loop;
    iface->download_show_details = ui_download_show_details;
    iface->file_tree_update_label = ui_file_tree_update_label;
    iface->file_tree_set_progress = ui_file_tree_set_progress;
    iface->file_tree_set_progress_name = ui_file_tree_set_progress_name;
    iface->lib_working = ui_lib_working;
    iface->chat_switch_page = ui_chat_switch_page;
    iface->chat_page_printable = ui_chat_page_printable;
    iface->chat_create_simple_page = ui_chat_create_simple_page;
    iface->chat_print = ui_chat_print;
    iface->threads_show = ui_threads_show;

    iface->idle_timeout_add = g_idle_add;
    iface->idle_timeout_remove = g_source_remove;
    iface->timeout_add = g_timeout_add;
    iface->timeout_remove = g_source_remove;

    object_register_instance("Download", (ObjectFunc)ui_download_new, TRUE);
    object_register_instance("Source", (ObjectFunc)ui_source_new, TRUE);
    object_register_instance("Library", (ObjectFunc)ui_lib_new, TRUE);
    object_register_instance("SearchPattern", (ObjectFunc)ui_search_pattern_show, FALSE);
    object_register_instance("Search", (ObjectFunc)ui_search_add_page, TRUE);
    object_register_instance("SearchResult", (ObjectFunc)ui_search_result_show, TRUE);
    object_register_instance("Upload", (ObjectFunc)ui_upload_show, TRUE);
    object_register_instance("Thread", (ObjectFunc)ui_thread_new, TRUE);
    object_register_instance("Buddy", (ObjectFunc)ui_buddy_new, TRUE);
    object_register_instance("Browse", (ObjectFunc)ui_browse_show, TRUE);
    object_register_instance("Browse", (ObjectFunc)ui_browse_link_new, TRUE);
    
    //  gtk_set_locale();
    gtk_init(lopster_get_argc(), lopster_get_argv());
    
    lopster_notify_register((NotifyFunc)ui_user_info);
    
    ui_styles_init();
    ui_layout_init();
    
    // library layout
    ui_layout_register_paned("core:lib:paned:pos");
    ui_layout_register_col("core:lib:col:filename");
    ui_layout_register_col("core:lib:col:filesize");
    ui_layout_register_col("core:lib:col:meta");
    ui_layout_register_vis("core:lib:controls:vis");
    
    // layout for browses
    ui_layout_register_paned("core:browse:paned:pos");
    ui_layout_register_col("core:browse:col:filename");
    ui_layout_register_col("core:browse:col:filesize");
    ui_layout_register_col("core:browse:col:meta");
    
    // search page layout
    ui_layout_register_paned("core:search:paned:pos");
    ui_layout_register_col("core:search:col:filename");
    ui_layout_register_col("core:search:col:filesize");
    ui_layout_register_col("core:search:col:sources");
    ui_layout_register_col("core:search:col:meta");
    ui_layout_register_col("core:search:col:searches");
    ui_layout_register_col("core:search:col:results");
    
    // transfer page layout
    ui_layout_register_paned("core:transfer:paned:pos");
    // upload layout
    ui_layout_register_vis("core:upload:bandwidth:vis");
    ui_layout_register_col("core:upload:col:filename");
    ui_layout_register_col("core:upload:col:size");
    ui_layout_register_col("core:upload:col:destination");
    ui_layout_register_col("core:upload:col:progress");
    ui_layout_register_col("core:upload:col:rate");
    ui_layout_register_col("core:upload:col:eta");
    // download layout
    ui_layout_register_vis("core:download:bandwidth:vis");
    ui_layout_register_col("core:download:col:filename");
    ui_layout_register_col("core:download:col:status");
    ui_layout_register_col("core:download:col:source");
    ui_layout_register_col("core:download:col:progress");
    ui_layout_register_col("core:download:col:rate");
    ui_layout_register_col("core:download:col:eta");

    ui_layout_register_col("core:source:col:filename");
    ui_layout_register_col("core:source:col:source");
    
    // main window position and size
    ui_layout_register_win("core:main:win");
    
    // format of the time display in the top right corner
    ui_layout_register_int("core:main:time:format");
    
    // layout of the buddy lists
    ui_layout_register_paned("core:buddy:paned:pos");
    ui_layout_register_exp("core:buddy:online:exp");
    ui_layout_register_exp("core:buddy:offline:exp");
    ui_layout_register_exp("core:buddy:browses:exp");
    ui_layout_register_vis("core:buddy:sidebar:vis");
    
    ui_style_register(N_("General style settings"), "bandwidth:color",
                      N_("Color of the bandwidth bar"),
                      N_("Background"), STYLE_COLOR, FALSE, 0xdff5, 0xb08d, 0xb08d,
                      N_("Border"),     STYLE_COLOR, FALSE, 0x9999, 0x9999, 0x9999,
                      N_("Base"),       STYLE_COLOR, TRUE,  0xffff, 0xffff, 0xffff,
                      N_("Bar"),        STYLE_COLOR, TRUE,  0x965f, 0xa55a, 0x9607,
                      N_("Text"),       STYLE_COLOR, TRUE,  0x0000, 0x0000, 0x0000,
                      NULL);
    ui_style_register(N_("General style settings"), "filetree:recent",
                      N_("Recent files in the library"), 
                      N_("Text"),       STYLE_COLOR, TRUE,  0xffff, 0x44e2, 0x0000,
                      N_("Background"), STYLE_COLOR, FALSE, 0xcccc, 0xcccc, 0xcccc,
                      N_("Font"),       STYLE_FONT,  FALSE, NULL,
                      NULL);

    ui_style_register(N_("General style settings"), "search:available",
                      N_("Search results that are available for download"),
                      N_("Text"),       STYLE_COLOR, TRUE,  0xc896, 0x49a3, 0x3f40,
                      N_("Background"), STYLE_COLOR, FALSE, 0xcccc, 0xcccc, 0xcccc,
                      N_("Font"),       STYLE_FONT,  FALSE, NULL,
                      NULL);
    ui_style_register(N_("General style settings"), "network:entry:online",
                      N_("Online networks in the a server list"),
                      N_("Text"),       STYLE_COLOR, TRUE,  0x3caf, 0xaaaa, 0x0000,
                      N_("Background"), STYLE_COLOR, FALSE, 0xcccc, 0xcccc, 0xcccc,
                      N_("Font"),       STYLE_FONT,  FALSE, NULL,
                      NULL);
    ui_style_register(N_("General style settings"), "transfer:bar",
                      N_("Colors of the upload and download progress bars"),
                      N_("Background"), STYLE_COLOR, TRUE,  0xe739, 0xe739, 0xe739,
                      N_("Border"),     STYLE_COLOR, TRUE,  0xcf5e, 0x4125, 0x4125,
                      N_("Finished"),   STYLE_COLOR, TRUE,  0xef00, 0xc800, 0x6500,
                      N_("In-progress"),STYLE_COLOR, TRUE,  0xe400, 0xd900, 0xb400,
                      N_("Percentage"), STYLE_COLOR, TRUE,  0xcf5e, 0x4125, 0x4125,
                      NULL);

    {
        action_item_t* item = action_item_root("String list actions");
        action_item_simple(N_("Remove item"), item, 0,
                           (ActionItemFunc)ui_string_list_remove);
    }
}
  
static GtkIconFactory* ui_main_get_icon_factory(void) {
  static GtkIconFactory* Factory = NULL;
  if (!Factory) {
    Factory = gtk_icon_factory_new();
    gtk_icon_factory_add_default(Factory);
  }
  return Factory;
}

static void ui_main_add_stock_icon(const char* name, const char* file) {
  GtkIconFactory* factory = ui_main_get_icon_factory();
  GtkIconSet* set = gtk_icon_set_new_from_pixbuf(create_pixbuf(file));
  gtk_icon_factory_add(factory, name, set);
}

/** Get the stock icon of a plugin
 * @param plugin the plugin
 * @return the stock icon, or NULL if none found
 */
const char* ui_main_get_plugin_stock_from_plugin(plugin_t* plugin) {
  return plugin?plugin->filename:NULL;
}

/** Get the stock icon of a plugin
 * @param id the ID of the plugin
 * @return the stock icon, or NULL if none found
 */
const char* ui_main_get_plugin_stock(int id) {
  plugin_t* plugin;

  if (id == -1) return STOCK_LOPSTER_LIBRARY;
  if (id == -2) return STOCK_LOPSTER_SEARCH;  // FIXME

  plugin = plugin_get(id);
  return plugin?plugin->filename:NULL;
}
static void ui_main_add_plugin_icon(plugin_t* plugin) {
  if (!plugin->filename || !plugin->info->icon) return;
  ui_main_add_stock_icon(plugin->filename, plugin->info->icon);
}

static void setup_stock_items(void) {
  GList* dlist;

  ui_main_add_stock_icon(STOCK_LOPSTER_NETWORK,   "server.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_CHAT,      "chat.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_SEARCH,    "search.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_LIBRARY,   "library.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_HOTLIST,   "hotlist.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_DOWNLOAD,  "download.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_UPLOAD,    "upload.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_TRANSFER,  "transfer.png");

  ui_main_add_stock_icon(STOCK_LOPSTER_CLOSE,     "close.png");

  ui_main_add_stock_icon(STOCK_LOPSTER_MEDIA_PREV,  "media_prev.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MEDIA_PAUSE, "media_pause.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MEDIA_PLAY,  "media_play.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MEDIA_STOP,  "media_stop.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MEDIA_NEXT,  "media_next.png");

  ui_main_add_stock_icon(STOCK_LOPSTER_MTYPE_OTHER, "mtype_other.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MTYPE_AUDIO, "mtype_audio.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MTYPE_VIDEO, "mtype_video.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MTYPE_DOC,   "mtype_document.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MTYPE_APP,   "mtype_application.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MTYPE_IMAGE, "mtype_image.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MTYPE_FOLDER,"mtype_folder.png");
  ui_main_add_stock_icon(STOCK_LOPSTER_MTYPE_ALL,   "mtype_all.png");

  ui_main_add_stock_icon(STOCK_LOPSTER_ICON, "lopster_icon.png");

  for (dlist = plugin_get_list(); dlist; dlist = dlist->next) {
    plugin_t* plugin = dlist->data;
    ui_main_add_plugin_icon(plugin);
  }
}


static void ui_time_periodic_update() {
  char* string;
#ifdef WIN32
  time_t wtime;
#endif    
  //  if (!TimeButton) return;
  if (!TimeLabel) return;
  
  switch (TimeFormat) {
  case UI_BOOT_TIME: {
    time_t t = BootTime();
    string = l_printf(_("Booted: %s"), ctime(&t));
    string[strlen(string)-1] = 0;
    break;
  }
  case UI_UP_TIME: {
    int sec = CurrentTime - BootTime();
    int min = sec / 60;
    int hour = min / 60;
    int day = hour / 24;
    sec = sec % 60;
    min = min % 60;
    hour = hour % 24;
    if (day)
      string = l_printf(_("Uptime: %d:%02d:%02d:%02d"), day, hour, min, sec);
    else
      string = l_printf(_("Uptime: %02d:%02d:%02d"), hour, min, sec);
    break;
  }
  case UI_CURRENT_TIME: {
#ifdef WIN32
    wtime = CurrentTime;
    string = l_printf("%s", ctime(&wtime));
#else    
    string = l_printf("%s", ctime(&CurrentTime));
#endif
    string[strlen(string)-1] = 0;
    break;
  }
  default:
    string = l_strdup("");
    break;
  }
  //  gtk_button_set_label(TimeButton, string);
  gtk_label_set_text(TimeLabel, string);
}

static gboolean ui_periodic_update(void)
{
    ui_time_periodic_update();
    ui_upload_periodic_update();
    ui_download_periodic_update();
    return TRUE;
}


static void ui_create_main_win(void);
void ui_run(void) {
  setup_stock_items();
  
  ui_create_main_win();

  ui_pref_init();
  
  ui_chat_init();
  
  // now show global content
  ui_lib_new(library_get_current());

  ui_init_init();

  lopster_call_periodic(1000, (CallFunc)ui_periodic_update, NULL);
}

static void ui_enter_main_loop(void) {
  ui_pref_post_init();
  ui_styles_load();
  gtk_main();
}

static void ui_leave_main_loop(void) {
  gtk_main_quit();
}

void ui_done(void) {
  ui_layout_save();
}


static gboolean win_delete_event(void) {
  lopster_termination_request(LOPSTER_EXIT_DEFAULT);
  return TRUE;
}

/* Another callback */
static void win_destroy(void) {
}

static void on_tab_changed(GtkWidget* child) {
  int index = gtk_notebook_page_num(MainNotebook->notebook, child);
  if (index < 0) return;
  CurrentIndex = index;
}


static GtkWidget* ui_create_main_tab(int no) {
  GtkWidget* hbox;
  GtkWidget* item;
  
  hbox = gtk_hbox_new(FALSE, 3);
  
  item = gtk_image_new_from_stock(TabIcon[no], GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  
/*   item = gtk_label_new(l_printf("<span weight=\"bold\" size=\"larger\">%s</span>", _(TabNames[no]))); */
/*   item = gtk_label_new(l_printf("<span size=\"larger\">%s</span>", _(TabNames[no]))); */
  item = gtk_label_new(l_printf("<b>%s</b>", _(TabNames[no])));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  
  gtk_widget_show_all(hbox);

  return hbox;
}


static void ui_add_main_content(int index) {
  GtkWidget* child = NULL;
  GtkWidget* button;

  switch (index) {
  case TAB_NET:
    child = ui_create_network_content();
    break;
  case TAB_CHAT:
    child = ui_create_chat_content();
    break;
  case TAB_SEARCH:
    child = ui_create_search_content();
    break;
  case TAB_LIB:
    child = ui_create_library_content();
    break;
  case TAB_HOTLIST:
    child = ui_create_buddy_content();
    break;
  case TAB_TRANSFER:
    child = ui_create_transfer_content();
    break;
  default:
    g_assert(FALSE);
    break;
  }

  gtk_container_set_border_width(GTK_CONTAINER(child), 5);
  button = ui_create_main_tab(index);
  ui_notebook_add_page(MainNotebook, child, button, child);
}

static void on_menu_threads()
{
    ui_threads_show(_("List of active threads"));
}

static void on_menu_about()
{
    // insert translator here
    const char* authors[] = {
        "Markus Lausser <sgop@users.sourceforge.net>",
        NULL
    };
    ui_about_show
        (GTK_WINDOW(MainWin), PACKAGE_NAME, plugin_get_version_string(NULL),
         "Copyright \302\251 2003-2007 Markus Lausser",
         "GPL (GNU General Public License)\n\nsee http://www.gnu.org/licenses/gpl.html",
         "http://lopster.sourceforge.net", authors,
         _("Lopster is a file-sharing tool which allows to join numerous networks."),
         "lopster.png");
}

static void on_menu_help()
{
    ui_about_help();
}

static void on_menu_quit()
{
    lopster_termination_request(LOPSTER_EXIT_DEFAULT);
}


static GtkWidget* on_menu() {
  GtkWidget* menu;
  GtkWidget* item;
  
  menu = gtk_menu_new();

  item = gtk_menu_item_new_with_label(_("Preferences..."));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(menu), item);
  g_signal_connect(G_OBJECT(item), "activate",
                   G_CALLBACK(ui_create_preferences), NULL);

  item = gtk_menu_item_new_with_label(_("Styles..."));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(menu), item);
  g_signal_connect(G_OBJECT(item), "activate",
                   G_CALLBACK(ui_create_style_editor), NULL);

  item = gtk_menu_item_new_with_label(_("Threads..."));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(menu), item);
  g_signal_connect(G_OBJECT(item), "activate",
                   G_CALLBACK(on_menu_threads), NULL);
  
  item = gtk_menu_item_new();
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(menu), item);
  
  item = gtk_menu_item_new_with_label(_("Help..."));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(menu), item);
  g_signal_connect(G_OBJECT(item), "activate",
                   G_CALLBACK(on_menu_help), NULL);
  
  item = gtk_menu_item_new_with_label(_("About..."));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(menu), item);
  g_signal_connect(G_OBJECT(item), "activate",
                   G_CALLBACK(on_menu_about), NULL);

  item = gtk_menu_item_new();
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(menu), item);
  
  item = gtk_menu_item_new_with_label(_("Quit"));
  gtk_widget_show(item);
  gtk_container_add(GTK_CONTAINER(menu), item);
  g_signal_connect(G_OBJECT(item), "activate",
                   G_CALLBACK(on_menu_quit), NULL);
  

  return menu;
}

static void ui_add_menu()
{
    GtkWidget* button;
    
    button = ui_create_popup_button
        (gtk_label_new(_("Menu")), POS_LEFT_DOWN, (PopupFunction)on_menu, NULL);
    
    gtk_box_pack_start(MainNotebook->tab_box, button, FALSE, FALSE, 0);
    gtk_box_reorder_child(MainNotebook->tab_box, button, 0);
    gtk_widget_show(button);
}

static void on_top_right(void) {
  if (TimeFormat >= UI_CURRENT_TIME) TimeFormat = 0;
  else TimeFormat++;
}

static gint win_key_pressed(GtkWindow* win, GdkEventKey* event) {
  static gboolean full = FALSE;

	if ((event->state & GDK_CONTROL_MASK) || (event->state & GDK_MOD1_MASK))
		return FALSE;

	switch (event->keyval) {
	case GDK_F11:
    if (full) gtk_window_unfullscreen(win);
    else gtk_window_fullscreen(win);
    full = !full;
    gtk_window_present(win);
		return TRUE;
	default:
		break;
	}

	return FALSE;
}


static void ui_create_main_win(void) {
  GtkWidget *window;
  GtkWidget *vbox;
  ui_nb_t* notebook;
  GtkWidget* item;
  GtkWidget* label;
  int i1;

  MainWin = window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  //  gtk_widget_set_events (window, GDK_EXPOSURE_MASK | GDK_BUTTON_MOTION_MASK | GDK_BUTTON_PRESS_MASK | GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
  gtk_window_set_title(GTK_WINDOW(MainWin), FULL_VERSION);
  gtk_window_set_role(GTK_WINDOW(window), "Main window");
  gtk_window_set_default_icon(create_pixbuf("lopster_icon.png"));
  //  gtk_window_fullscreen(GTK_WINDOW(window));

  ui_styles_set_main(MainWin);

  gtk_window_set_default_size(GTK_WINDOW(window), 800, 500);
  gtk_window_set_resizable(GTK_WINDOW(window), TRUE);

  g_signal_connect(G_OBJECT(window), "delete_event",
                   G_CALLBACK(win_delete_event), NULL);
  
  g_signal_connect(G_OBJECT(window), "destroy",
                   G_CALLBACK(win_destroy), NULL);

  g_signal_connect(G_OBJECT(window), "key_press_event",
                   G_CALLBACK(win_key_pressed), NULL);

  
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox);
  gtk_container_add(GTK_CONTAINER(window), vbox);
  
  // notebook
  MainNotebook = notebook = ui_create_notebook
    ((TabChangedFunc)on_tab_changed, GTK_ORIENTATION_HORIZONTAL, FALSE);
  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(notebook->tab_box), FALSE, FALSE, 0);
  item = gtk_hseparator_new();
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(notebook->notebook), TRUE, TRUE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(notebook->tab_box), 5);
  //  gtk_notebook_set_show_border(MainNotebook, FALSE);
  
  // now add pages to the main notebook
  for (i1 = 0; i1 < TAB_NUMBER; i1++) {
    ui_add_main_content(i1);
  }

  item = gtk_button_new();
  TimeButton = GTK_BUTTON(item);
  gtk_widget_show(item);
  gtk_box_pack_end(MainNotebook->tab_box, item, FALSE, FALSE, 0);
  gtk_button_set_relief(TimeButton, GTK_RELIEF_NONE);
  label = gtk_label_new("");
  TimeLabel = GTK_LABEL(label);
  gtk_widget_show(label);
  gtk_container_add(GTK_CONTAINER(item), label);

  g_signal_connect(G_OBJECT(item), "clicked",
		   G_CALLBACK(on_top_right), NULL);
  ui_layout_monitor("core:main:time:format", &TimeFormat);
  
  ui_add_menu();

  // status bar
  /*
  child = ui_create_status(window);
  gtk_box_pack_start(GTK_BOX(vbox), child, FALSE, FALSE, 0);
  */

  ui_layout_monitor("core:main:win:size", window);
  ui_layout_monitor("core:main:win:pos", window);

  gtk_widget_show(window);
}

GtkWidget* ui_box_add_label(GtkBox* box, const char* text,
                            unsigned padding) {
  GtkWidget* label;

  label = gtk_label_new(text);
  gtk_widget_show(label);
  gtk_box_pack_start(box, label, FALSE, FALSE, padding);

  return label;
}

gint sort_file_by_size(GtkTreeModel* model,
                       GtkTreeIter* a, GtkTreeIter* b) {
  file_t* r1;
  file_t* r2;
  
  gtk_tree_model_get (model, a, 0, &r1, -1);
  gtk_tree_model_get (model, b, 0, &r2, -1);
  if (r1->size < r2->size) return -1;
  if (r1->size > r2->size) return 1;
  return 0;
}

static void
popup_position(GtkMenu* menu, gint* x, gint* y,
	       gboolean* push_in, GtkWidget* widget) {
  GtkRequisition requisition;
  gint screen_width;
  gint screen_height;
  int mode;
  GdkScreen *screen;

  (void)push_in;
  mode = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(widget), "mode"));

  // maybe replace with gtk function
  gdk_window_get_origin(GTK_WIDGET(widget)->window, x, y);

  *x += widget->allocation.x;
  *y += widget->allocation.y;
  gtk_widget_size_request(GTK_WIDGET(menu), &requisition);
  screen = gtk_widget_get_screen(widget);
  screen_width = gdk_screen_get_width(screen);
  screen_height = gdk_screen_get_height(screen);

  switch (mode) {
  case POS_LEFT_UP:
    *y -= requisition.height;
    break;
  case POS_LEFT_DOWN:
    *y += widget->allocation.height;
    break;
  case POS_RIGHT_UP:
    *y -= requisition.height;
    *x += widget->allocation.width-requisition.width;
    break;
  case POS_RIGHT_DOWN:
    *y += widget->allocation.height;
    *x += widget->allocation.width-requisition.width;
    break;
  }
  *x = CLAMP (*x, 0, MAX (0, screen_width - requisition.width));
  *y = CLAMP (*y, 0, MAX (0, screen_height - requisition.height));

}

static void on_popup_button_clicked(GtkButton* button, PopupFunction func) {
  GtkWidget *pop;
  gpointer data;

  data = g_object_get_data(G_OBJECT(button), "data");
  pop = func(data);
  if (!pop) return;

  gtk_menu_popup(GTK_MENU(pop), NULL, NULL,
		 (GtkMenuPositionFunc)popup_position,
		 button, 1, 0);
}

GtkWidget*
ui_create_popup_button(GtkWidget* label, int arrow, PopupFunction func, gpointer data)
{
    GtkWidget* button;
    GtkWidget* pixmap;
    GtkWidget* hbox;
    
    g_assert(func);
    
    button = gtk_button_new();
    g_object_set_data(G_OBJECT(button), "data", data?data:button);
    g_object_set_data(G_OBJECT(button), "mode", GINT_TO_POINTER(arrow));
    //  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
    
    {
        GtkRcStyle* rcstyle = gtk_rc_style_new();
        rcstyle->xthickness = rcstyle->ythickness = 1;
        gtk_widget_modify_style(button, rcstyle);
        g_object_unref(rcstyle);
        gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
        g_object_set(G_OBJECT(button),
                     "can-focus", FALSE, "can-default", FALSE, NULL);
        gtk_widget_set_name(button, "CloseButton");
    }
    
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox);
    gtk_container_add(GTK_CONTAINER(button), hbox);
    
    
    if (arrow & 0x10)
        pixmap = gtk_image_new_from_stock(GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_MENU);
    else
        pixmap = gtk_image_new_from_stock(GTK_STOCK_GO_UP, GTK_ICON_SIZE_MENU);
    gtk_widget_show(pixmap);
    
    if (label) gtk_widget_show(label);
    
    if (arrow & 0x01)
    {
        if (label) gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 4);
        gtk_box_pack_start(GTK_BOX(hbox), pixmap, FALSE, FALSE, 0);
    }
    else
    {
        gtk_box_pack_start(GTK_BOX(hbox), pixmap, FALSE, FALSE, 0);
        if (label) gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 4);
    }
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(on_popup_button_clicked), func);
    return button;
}

gboolean
ui_iter_is_parent_to(GtkTreeModel* model, GtkTreeIter* parent,
                     GtkTreeIter* node) {
  GtkTreeIter temp;
  GtkTreePath* path1;
  GtkTreePath* path2;
  gboolean result;
  
  if (!parent) return FALSE;
  gtk_tree_model_iter_parent(model, &temp, node);
  path1 = gtk_tree_model_get_path(model, parent);
  path2 = gtk_tree_model_get_path(model, &temp);

  if (gtk_tree_path_compare(path1, path2)) result = FALSE;
  else result = TRUE;

  gtk_tree_path_free(path1);
  gtk_tree_path_free(path2);
  
  return result;
}

/** Switch to a main page
 * @param number the number of the page
 */
void ui_switch_main(int number) {
  g_assert(number >= 0 && number < TAB_NUMBER);
  ui_notebook_switch_to_index(MainNotebook, number);
}

static void on_des_update(thread_t* thread, GtkLabel* label) {
  if (thread_get_status(thread) == THREAD_RUNNING)
    gtk_label_set_text(label, thread_get_description(thread));
  else
    gtk_label_set_text(label, "inactive");
}

static void on_progress_update(thread_t* thread, GtkProgressBar* bar) {
  if (thread_get_status(thread) == THREAD_RUNNING)
    gtk_progress_bar_set_fraction
      (bar, (double)thread_get_progress(thread)/100.);
  else
    gtk_progress_bar_set_fraction(bar, 0.0);
}

static void on_deactivate(thread_t* thread, GtkWidget* box) {
  (void)thread;
  gtk_widget_destroy(box);
  //  gtk_widget_set_sensitive(button, FALSE);
}

static void on_abort(GtkWidget* button, thread_t* thread) {
  gtk_widget_set_sensitive(button, FALSE);
  thread_interrupt(thread);
}

static void on_thread_destroy(thread_t* thread) {
  Threads = g_list_remove(Threads, thread);
}

static void on_label_destroy(GtkWidget* label, object_t* thread)
{
    object_unmonitor_d(thread, (ObjectFunc)on_des_update, label);
}

static void on_bar_destroy(GtkWidget* bar, object_t* thread)
{
    object_unmonitor_d(thread, (ObjectFunc)on_progress_update, bar);
}

static void on_box_destroy(GtkWidget* box, object_t* thread)
{
    object_unnotify_d(thread, (ObjectFunc)on_deactivate, box);
}

void ui_thread_show(thread_t* thread) {
  GtkWidget* vbox;
  GtkWidget* label;
  GtkWidget* hbox;
  GtkWidget* bar;
  GtkWidget* button;

  if (!ThreadGui) return;

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(ThreadGui->box, vbox, FALSE, FALSE, 0);

  label = gtk_label_new(l_printf("<b>%s</b>", thread_get_name(thread)));
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

  label = gtk_label_new(thread_get_description(thread));
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
  
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  
  bar = gtk_progress_bar_new();
  gtk_box_pack_start(GTK_BOX(hbox), bar, TRUE, TRUE, 0);
  
  button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    
  object_monitor_d(OBJ(thread), (ObjectFunc)on_des_update, label);
  g_signal_connect(G_OBJECT(label), "destroy",
		   G_CALLBACK(on_label_destroy), thread);

  object_monitor_d(OBJ(thread), (ObjectFunc)on_progress_update, bar);
  g_signal_connect(G_OBJECT(bar), "destroy",
		   G_CALLBACK(on_bar_destroy), thread);

  object_notify_d(OBJ(thread), (ObjectFunc)on_deactivate, vbox);
  g_signal_connect(G_OBJECT(vbox), "destroy",
                   G_CALLBACK(on_box_destroy), thread);
  g_signal_connect(G_OBJECT(button), "clicked",
                   G_CALLBACK(on_abort), thread);
  
  gtk_widget_show_all(vbox);
}

static void ui_thread_new(thread_t* thread)
{
    Threads = g_list_append(Threads, thread);
    object_notify_d(OBJ(thread), (ObjectFunc)on_thread_destroy, NULL);
    ui_thread_show(thread);
}


static void th_destroy_event(GtkWidget* win)
{
    (void)win;
    g_free(ThreadGui);
    ThreadGui = NULL;
}

void ui_threads_show(const char* message)
{
    GList* dlist;
    GtkWidget* win;
    GtkWidget* vbox;
    GtkWidget* label;
    GtkWidget* vbox2;
    GtkWidget* swindow;
    GtkWidget* hbox;
    GtkWidget* temp;
  
    if (ThreadGui)
    {
        ui_set_header_label(ThreadGui->header, message);
        if (ThreadGui->win) gtk_window_present(GTK_WINDOW(ThreadGui->win));
        return;
    }

    ThreadGui = g_malloc(sizeof(*ThreadGui));
    ThreadGui->win = win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
    if (ui_main_get_win())
    {
        gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
        gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
    }

    g_signal_connect(G_OBJECT(win), "destroy",
                     G_CALLBACK(th_destroy_event), NULL);

    gtk_window_set_title(GTK_WINDOW(win), _("Active threads"));
    gtk_window_set_role(GTK_WINDOW(win), "Threads");
    gtk_window_set_default_size(GTK_WINDOW(win), 400, 200);
    gtk_window_set_resizable(GTK_WINDOW(win), TRUE);
  
    vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
    gtk_container_add(GTK_CONTAINER(win), vbox);

    label = ui_create_header_label(message);
    ThreadGui->header = GTK_LABEL(label);
    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
  

    swindow = gtk_scrolled_window_new (NULL, NULL);
    gtk_box_pack_start(GTK_BOX(vbox), swindow, TRUE, TRUE, 0);
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(swindow), 
         GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type
        (GTK_SCROLLED_WINDOW(swindow), GTK_SHADOW_NONE);

    vbox2 = gtk_vbox_new(FALSE, 5);
    gtk_scrolled_window_add_with_viewport
        (GTK_SCROLLED_WINDOW(swindow), vbox2);
    gtk_viewport_set_shadow_type
        (GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(swindow))), GTK_SHADOW_NONE);

    ThreadGui->box = GTK_BOX(vbox2);

    for (dlist = Threads; dlist; dlist = dlist->next)
    {
        thread_t* thread = dlist->data;
        ui_thread_show(thread);
    }
  
    temp = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX(vbox), temp, FALSE, FALSE, 0);

    hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

    temp = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
    gtk_container_add(GTK_CONTAINER(hbox), temp);
    g_signal_connect_swapped(G_OBJECT(temp), "clicked",
                             G_CALLBACK(gtk_widget_destroy), win);

    gtk_widget_show_all(win);
}

static void on_item_add(GtkEntry* entry, const char* name) {
    const char* text;
  
    text = gtk_entry_get_text(entry);
    if (!text || *text == 0) return;
    string_list_add(name, text);
    gtk_entry_set_text(entry, "");
}

static gboolean
remove_cb(GtkTreeModel* model, GtkTreePath* path,
	  GtkTreeIter* iter, const char* item) {
  char* text;

  (void)path;

  gtk_tree_model_get(model, iter, 0, &text, -1);
  if (!strcasecmp(text, item)) {
    gtk_list_store_remove(GTK_LIST_STORE(model), iter);
    g_free(text);
    return TRUE;
  } else {
    g_free(text);
    return FALSE;
  }
}


static void on_item_removed(char* item, GtkTreeModel* store) {
  gtk_tree_model_foreach(store,
			 (GtkTreeModelForeachFunc)remove_cb, item);
}

static void on_item_added(const char* item, GtkListStore* store) {
  GtkTreeIter iter;
  gtk_list_store_append(store, &iter);
  gtk_list_store_set(store, &iter, 0, item, -1);
}

GtkWidget* ui_string_list_view(const char* name, const char* actions) {
  GtkListStore* store;
  GtkWidget* tv;
  GtkCellRenderer* renderer;
  GtkTreeViewColumn* column;
  GtkTreeSelection *select;
  const char* header = string_list_get_label(name);

  store = gtk_list_store_new(1, G_TYPE_STRING);
  string_list_monitor(name, (StringListFunc)on_item_added,
		      (StringListFunc)on_item_removed, store);
  gtk_tree_sortable_set_sort_column_id
    (GTK_TREE_SORTABLE(store), 0, GTK_SORT_ASCENDING);
  

  tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(tv), FALSE);
  
  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
  gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);

  renderer = gtk_cell_renderer_text_new();
  column = gtk_tree_view_column_new_with_attributes 
    (header, renderer, "text", 0, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);

  {
    action_item_t* act = action_item_search(actions);
    char* n = g_strdup(name);
    ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &act, n);
    g_signal_connect_swapped(G_OBJECT(tv), "destroy",
			     G_CALLBACK(g_free), n);
  }

  string_list_foreach(name, (StringListFunc)on_item_added, store);
  return tv;
}

GtkWidget* ui_string_list_show(GtkBox* box, const char* name,
                               const char* actions, gboolean allow_hide) {
  GtkWidget* hbox;
  GtkWidget* vbox;
  GtkWidget* vbox2;
  GtkWidget* label;
  GtkWidget* entry;
  GtkWidget* sw;
  GtkWidget* tv;
  const char* header = string_list_get_label(name);
  GtkRequisition req;
  
  if (!header) {
    g_log("GTK", G_LOG_LEVEL_WARNING,
          "cannot show string list '%s': missing header", name);
    return 0;
  }

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(box, vbox, TRUE, TRUE, 0);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  gtk_widget_show(hbox);

  vbox2 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(vbox), vbox2, TRUE, TRUE, 0);

  label = gtk_label_new(l_printf("<b>%s</b>", _(header)));
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
/*   label = ui_create_header_label(header); */
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  gtk_widget_size_request(label, &req);

  if (allow_hide) {
    label = ui_create_show_hide_label(_("show"), _("hide"), vbox2);
    gtk_widget_show(label);
    gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  } else {
    gtk_widget_show(vbox2);
  }

  entry = gtk_entry_new();
  gtk_widget_show(entry);
  gtk_widget_set_size_request(entry, 1, -1/*req.height*/);
  gtk_box_pack_start(GTK_BOX(vbox2), entry, FALSE, FALSE, 0);
  g_signal_connect(G_OBJECT(entry), "activate",
		   G_CALLBACK(on_item_add), (char*)name);


  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(sw);
  gtk_box_pack_start(GTK_BOX(vbox2), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type
    (GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);

  tv = ui_string_list_view(name, actions);
  gtk_container_add(GTK_CONTAINER(sw), tv);
  gtk_widget_show(tv);

  return vbox;
}

#define BLR 0
#define BOT 0

static int calc_x(guint32 val, guint32 max, int width) {
  if (val > max) return width;
  else return (int) (((double) val / (double) max) * (double) width);
}
static gboolean
on_bandwidth_expose(GtkWidget *widget, GdkEventExpose *event, ui_bw_t* data) {
  int width = widget->allocation.width;
  int height = widget->allocation.height;
  unsigned texth, dx, x1, posx;
  GdkGC* gc;
  PangoLayout* layout;
  unsigned limit;

  (void)event;

  layout = ui_get_text_layout(widget);
  ui_layout_get_ext(layout, "128.0 KiB/s", NULL, &texth);
  texth += 4;

  if ((signed)texth+15+BOT != height) {
    gtk_widget_set_size_request(widget, -1, texth+15+BOT);
    return TRUE;
  }

  if (ui_style_sets("bandwidth:color", "Background")) {
    gdk_draw_rectangle(widget->window, 
                       ui_style_get_gc("bandwidth:color", "Background"),
                       TRUE, 0, 0, width, height);
  }
  if (ui_style_sets("bandwidth:color", "Border")) {
    gdk_draw_rectangle(widget->window,
                       ui_style_get_gc("bandwidth:color", "Border"),
                       FALSE, 0, 0, width-1, height-1);
  }

  dx = 1;
  while (calc_x(dx, data->range, width-2*(2+BLR)) < 80) dx *=2;

  gc = ui_style_get_gc("bandwidth:color", "Text");
  x1 = 0;
  while (x1 <= data->range) {
    const char* str = print_speed(&x1);
    
    posx = calc_x(x1, data->range, width-2*(2+BLR));
    pango_layout_set_text(layout, str, -1);
    gdk_draw_layout(widget->window, gc, posx+BLR+6, 0, layout);

    gdk_draw_line(widget->window, gc, posx+BLR+2, 0, posx+BLR+2, height);
    x1 += dx;
  }
  
  if (ui_style_sets("bandwidth:color", "Base")) {
    gdk_draw_rectangle(widget->window,
                       ui_style_get_gc("bandwidth:color", "Base"),
                       TRUE, BLR, texth, width-2*BLR, height-texth-BOT);
    if (ui_style_sets("bandwidth:color", "Border")) {
      gdk_draw_rectangle(widget->window,
                         ui_style_get_gc("bandwidth:color", "Border"),
                         FALSE, BLR, texth, width-2*BLR-1, height-texth-BOT-1);
    } else if (ui_style_sets("bandwidth:color", "Text") &&
               ui_style_sets("bandwidth:color", "Base")) {
      gdk_draw_rectangle(widget->window,
                         ui_style_get_gc("bandwidth:color", "Text"),
                         FALSE, BLR, texth, width-2*BLR-1, height-texth-BOT-1);
    }
  }

  x1 = calc_x(rate_get_rate(data->rate), data->range, width - 2*(2+BLR));
  gdk_draw_rectangle(widget->window,
                     ui_style_get_gc("bandwidth:color", "Bar"),
                     TRUE, BLR+2, texth+2, x1, height-texth-BOT-4);
  

  limit = limit_get_limit(LIMIT(data->vol));
  if (limit <= data->range) {
    x1 = calc_x(limit, data->range, width - 2*(2+BLR));
    gtk_paint_arrow(widget->style, widget->window,
                    GTK_STATE_NORMAL, GTK_SHADOW_OUT,
                    NULL, widget, "arrow", GTK_ARROW_UP, TRUE,
                    2+BLR + x1 - 6, texth, 13, height - texth);
  }

  return TRUE;
}

static void ui_bandwidth_force_update(void* dummy, ui_bw_t* data) {
  GtkWidget* da = data->widget;
  if (da && da->window) 
    gdk_window_invalidate_rect(da->window, NULL, FALSE);
  (void)dummy;
}

static gboolean
on_bandwidth_pressed(GtkWidget* widget, GdkEventButton* event, ui_bw_t* data) {
  int width = widget->allocation.width - 2*(2+BLR);
  int x = (int)event->x - (2+BLR);
  int limit;

  if (x < 0) x = 0;
  if (x > width) x = width;

  if (event->button == 1) {
    if (x == 0) limit = 0;
    else limit = ((guint64)data->range * (guint64)x -1) / (guint64)width +1;
    limit_set_limit(LIMIT(data->vol), limit);
  } else {
    data->x_drag_start = x>0?x:1;   // must be > 0 !
    data->drag_range = data->range;
    ui_bandwidth_force_update(NULL, data);
  }

  return TRUE;
}

static gboolean
on_bandwidth_motion(GtkWidget* widget, GdkEventMotion* event, ui_bw_t* data) {
  int width = widget->allocation.width - 2*(2+BLR);
  int x = (int) event->x - (2+BLR);
  int limit;

  if (x < 0) x = 0;
  if (x > width) x = width;

  if (event->state & GDK_BUTTON1_MASK) {
    if (x == 0) limit = 0;
    else limit = ((guint64)data->range * (guint64)x -1) / (guint64)width +1;
    limit_set_limit(LIMIT(data->vol), limit);
  } else if (x > 0) {
    guint64 res = (guint64)data->drag_range * (guint64)data->x_drag_start / (guint64)x;
    if (res < 16) res = 16;
    else if (res > (1<<30)) res = (1<<30);
    data->range = res;
    ui_bandwidth_force_update(NULL, data);
  }

  return TRUE;
}

static void on_bandwidth_destroy(GtkWidget* da, ui_bw_t* data) {
  (void)da;
  object_unmonitor_d(OBJ(data->vol), (ObjectFunc)ui_bandwidth_force_update, data);
  ui_style_unobserve("bandwidth:color", (StyleFunc)ui_bandwidth_force_update, data);
  ui_layout_unmonitor(data->name, data);

  g_free(data->name);
  g_free(data);
}

static char* ui_bw_get_range(ui_bw_t* data) {
  return g_strdup_printf("%u", data->range);
}
static void ui_bw_set_range(ui_bw_t* data, const char* val) {
  if (!val) return;
  data->range = atoi(val);
  if (data->widget) ui_bandwidth_force_update(NULL, data);
}

GtkWidget* ui_create_bandwidth_bar(const char* name, volume_t* volume, rate_t* rate) {
  GtkWidget* da;
  ui_bw_t* data;

  data = g_malloc0(sizeof(*data));
  data->name = g_strdup(name);
  data->range = 1024*128;  // default value
  data->vol = volume;
  data->rate = rate;
  
  da = gtk_drawing_area_new();
  //  gtk_widget_set_size_request(da, -1, 35);
  gtk_widget_set_events(da, GDK_BUTTON1_MOTION_MASK|GDK_BUTTON3_MOTION_MASK|GDK_BUTTON_PRESS_MASK);
/*   gtk_widget_show(da); */
  data->widget = da;
  
  ui_layout_register(name, (LayoutGetFunc)ui_bw_get_range,
                     (LayoutSetFunc)ui_bw_set_range);
  
  g_signal_connect(G_OBJECT(da), "expose-event",
                   G_CALLBACK(on_bandwidth_expose), data);
  g_signal_connect(G_OBJECT(da), "button-press-event",
                   G_CALLBACK(on_bandwidth_pressed), data);
  g_signal_connect(G_OBJECT(da), "motion-notify-event",
                   G_CALLBACK(on_bandwidth_motion), data);
  g_signal_connect(G_OBJECT(da), "destroy",
                   G_CALLBACK(on_bandwidth_destroy), data);
  

  ui_style_observe("bandwidth:color",
                   (StyleFunc)ui_bandwidth_force_update, data);
  object_monitor_d(OBJ(volume),
                   (ObjectFunc)ui_bandwidth_force_update, data);
  ui_layout_monitor(name, data);

  return da;
}

static void ui_charset_append(GtkTreeStore* store, GtkTreeIter* parent,
                              const char* charset, const char* label)
{
    GtkTreeIter iter;
    gtk_tree_store_append(store, &iter, parent);
    gtk_tree_store_set(store, &iter, 0, label, 1, charset, 2, TRUE, -1);
}

static GtkTreeStore* ui_get_charset_store() {
  static GtkTreeStore* Store = NULL;

  if (!Store) {
    GtkTreeIter parent;

    Store = gtk_tree_store_new
      (3,
       G_TYPE_STRING,  // label,
       G_TYPE_STRING,  // charset
       G_TYPE_BOOLEAN  // selectable.
       );

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("European languages"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "ASCII", NULL);
    ui_charset_append(Store, &parent, "ISO-8859-1", _("(Western Europe)"));
    ui_charset_append(Store, &parent, "ISO-8859-2", _("(Central/Eastern Europe)"));
    ui_charset_append(Store, &parent, "ISO-8859-3", _("(South Europe)"));
    ui_charset_append(Store, &parent, "ISO-8859-4", _("(North Europe)"));
    ui_charset_append(Store, &parent, "ISO-8859-5", _("(Cyrillic)"));
    ui_charset_append(Store, &parent, "ISO-8859-7", _("(Greek)"));
    ui_charset_append(Store, &parent, "ISO-8859-9", _("(Turkish)"));
    ui_charset_append(Store, &parent, "ISO-8859-10", _("(Nordic)"));
    ui_charset_append(Store, &parent, "ISO-8859-13", _("(Baltic)"));
    ui_charset_append(Store, &parent, "ISO-8859-14", _("(Celtic)"));
    ui_charset_append(Store, &parent, "ISO-8859-15", _("(Western Europe)"));
    ui_charset_append(Store, &parent, "ISO-8859-16", _("(South-Eastern Europe)"));
    ui_charset_append(Store, &parent, "KOI8-R", _("(Cyrillic)"));
    ui_charset_append(Store, &parent, "KOI8-U", _("(Cyrillic)"));
/*     ui_charset_append(Store, &parent, NULL, "KOI8-RU"); */
    ui_charset_append(Store, &parent, "CP1250", _("(MS Central/Eastern Europe)"));
    ui_charset_append(Store, &parent, "CP1251", _("(MS Cyrillic)"));
    ui_charset_append(Store, &parent, "CP1252", _("(MS Western Europe)"));
    ui_charset_append(Store, &parent, "CP1253", _("(MS Greek)"));
    ui_charset_append(Store, &parent, "CP1254", _("(MS Turkish)"));
    ui_charset_append(Store, &parent, "CP1257", _("(MS Baltic)"));
/*     ui_charset_append(Store, &parent, NULL, "CP850"); */
/*     ui_charset_append(Store, &parent, NULL, "CP866"); */
    ui_charset_append(Store, &parent, "MacRoman", NULL);
    ui_charset_append(Store, &parent, "MacCentralEurope", NULL);
    ui_charset_append(Store, &parent, "MacIceland", NULL);
    ui_charset_append(Store, &parent, "MacCroatian", NULL);
    ui_charset_append(Store, &parent, "MacRomania", NULL);
    ui_charset_append(Store, &parent, "MacCyrillic", NULL);
    ui_charset_append(Store, &parent, "MacUkraine", NULL);
    ui_charset_append(Store, &parent, "MacGreek", NULL);
    ui_charset_append(Store, &parent, "MacTurkish", NULL);
    ui_charset_append(Store, &parent, "Macintosh", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Semitic languages"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "ISO-8859-6", _("(Arabic)"));
    ui_charset_append(Store, &parent, "ISO-8859-8", _("(Hebrew)"));
    ui_charset_append(Store, &parent, "CP1255", _("(MS Hebrew)"));
    ui_charset_append(Store, &parent, "CP1256", _("(MS Arabic)"));
/*     ui_charset_append(Store, &parent, NULL, "CP862"); */
    ui_charset_append(Store, &parent, "MacHebrew", NULL);
    ui_charset_append(Store, &parent, "MacArabic", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Japanese"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "EUC-JP", NULL);
    ui_charset_append(Store, &parent, "SHIFT_JIS", NULL);
    ui_charset_append(Store, &parent, "CP932", NULL);
    ui_charset_append(Store, &parent, "ISO-2022-JP", NULL);
    ui_charset_append(Store, &parent, "ISO-2022-JP-2", NULL);
    ui_charset_append(Store, &parent, "ISO-2022-JP-1", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Chinese"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "EUC-CN", NULL);
    ui_charset_append(Store, &parent, "HZ", NULL);
    ui_charset_append(Store, &parent, "GBK", NULL);
    ui_charset_append(Store, &parent, "GB18030", NULL);
    ui_charset_append(Store, &parent, "EUC-TW", NULL);
    ui_charset_append(Store, &parent, "BIG5", NULL);
/*     ui_charset_append(Store, &parent, NULL, "CP950"); */
    ui_charset_append(Store, &parent, "BIG5-HKSCS", NULL);
    ui_charset_append(Store, &parent, "ISO-2022-CN", NULL);
    ui_charset_append(Store, &parent, "ISO-2022-CN-EXT", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Korean"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "EUC-KR", NULL);
/*     ui_charset_append(Store, &parent, NULL, "CP949"); */
    ui_charset_append(Store, &parent, "ISO-2022-KR", NULL);
    ui_charset_append(Store, &parent, "JOHAB", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Armenian"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "ARMSCII-8", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Georgian"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "Georgian-Academy", NULL);
    ui_charset_append(Store, &parent, "Georgian-PS", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Tajik"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "KOI8-T", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Thai"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "TIS-620", NULL);
    ui_charset_append(Store, &parent, "ISO-8859-11", NULL);
    ui_charset_append(Store, &parent, "CP874", NULL);
    ui_charset_append(Store, &parent, "MacThai", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Laotian"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "MuleLao-1", NULL);
    ui_charset_append(Store, &parent, "CP1133", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Vietnamese"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "VISCII", NULL);
    ui_charset_append(Store, &parent, "TCVN", NULL);
    ui_charset_append(Store, &parent, "CP1258", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Platform specifics"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "HP-ROMAN8", NULL);
    ui_charset_append(Store, &parent, "NEXTSTEP", NULL);

    gtk_tree_store_append(Store, &parent, NULL);
    gtk_tree_store_set(Store, &parent, 0, _("Full Unicode"), 2, FALSE, -1);
    ui_charset_append(Store, &parent, "UTF-8", NULL);
    ui_charset_append(Store, &parent, "UCS-2", NULL);
    ui_charset_append(Store, &parent, "UCS-2BE", NULL);
    ui_charset_append(Store, &parent, "UCS-2LE", NULL);
    ui_charset_append(Store, &parent, "UCS-4", NULL);
    ui_charset_append(Store, &parent, "UCS-4BE", NULL);
    ui_charset_append(Store, &parent, "UCS-4LE", NULL);
    ui_charset_append(Store, &parent, "UTF-16", NULL);
    ui_charset_append(Store, &parent, "UTF-16BE", NULL);
    ui_charset_append(Store, &parent, "UTF-16LE", NULL);
    ui_charset_append(Store, &parent, "UTF-32", NULL);
    ui_charset_append(Store, &parent, "UTF-32BE", NULL);
    ui_charset_append(Store, &parent, "UTF-32LE", NULL);
    ui_charset_append(Store, &parent, "UTF-7", NULL);
    ui_charset_append(Store, &parent, "C99", NULL);
    ui_charset_append(Store, &parent, "JAVA", NULL);
  }

  return Store;
}

const char* ui_charset_control_get_charset(GtkWidget* combo) {
  const char* charset = gtk_entry_get_text(GTK_ENTRY(GTK_BIN(combo)->child));
  return charset;
/*   GtkTreeIter iter; */

/*   if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combo), &iter)) { */
/*     GtkTreeModel* model = gtk_combo_box_get_model(GTK_COMBO_BOX(combo)); */
/*     char* charset; */

/*     gtk_tree_model_get(model, &iter, 1, &charset, -1); */
/*     return charset; */
/*   } */
/*   return NULL; */
}

void ui_charset_control_set_charset(GtkWidget* combo, const char* charset) {
  gtk_entry_set_text(GTK_ENTRY(GTK_BIN(combo)->child), charset?charset:"");
}

static void on_charset_changed(GtkComboBox* combo, CharsetFunc func) {
  const char* charset = ui_charset_control_get_charset(GTK_WIDGET(combo));
  void* data = g_object_get_data(G_OBJECT(combo), "data");
  func(charset, data);
}

GtkWidget* ui_create_charset_control(const char* charset, CharsetFunc func, void* data) {
  GtkWidget* combo;
  GtkCellRenderer* rend;

  combo = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(ui_get_charset_store()), 1);

  rend = gtk_cell_renderer_text_new();
  gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), rend, FALSE);
  gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo), rend,
                                 "text", 0, "sensitive", 2, NULL);
  
  ui_charset_control_set_charset(combo, charset);

  if (func) {
    g_object_set_data(G_OBJECT(combo), "data", data);
    g_signal_connect(G_OBJECT(combo), "changed",
                     G_CALLBACK(on_charset_changed), func);
  }

  return combo;
}

#ifdef WIN32
#define _UI_MAIN_C_
#include "lgtk2corealiasdef.c"
#endif
