/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.8 $
 * $Date: 2007/11/21 20:15:25 $
 * $Author: sgop $
 */

#ifndef _UI_MAIN_H
#define _UI_MAIN_H

#include "lcore/files.h"
#include "lcore/action_item.h"
#include "lcore/l_socket.h"
#include "lcore/plugin.h"
#include "lcore/mtypes.h"
#include "lcore/thread.h"

/** @name Popup positions
 * @ingroup UserInterface
 */
/*@{*/
#define POS_NONE       0x100 ///< unknown
#define POS_LEFT_UP    0x000 ///< popup left up
#define POS_LEFT_DOWN  0x010 ///< popup left down
#define POS_RIGHT_UP   0x001 ///< popup right up
#define POS_RIGHT_DOWN 0x011 ///< popup right down
/*@}*/

/** @name Tab index
 * @ingroup UserInterface
 */
/*@{*/
#define TAB_NET       0  ///< network page
#define TAB_SEARCH    1  ///< search page
#define TAB_TRANSFER  2  ///< transfer page
#define TAB_HOTLIST   3  ///< buddy page
#define TAB_CHAT      4  ///< chat page
#define TAB_LIB       5  ///< library page
#define TAB_NUMBER    6  ///< number of main pages
/*@}*/

extern const char* MediaStock[MEDIA_ANY+1];

typedef void (*CharsetFunc)(const char* charset, void* data);

/** @name General functions
 * @ingroup UserInterface
 */
/*@{*/

#define DCLICK(event) \
  (event->type == GDK_2BUTTON_PRESS && event->button == 1)

typedef GtkWidget* (*PopupFunction)(gpointer data);
typedef void (*ColorFunc)(gpointer data, GdkColor* color);

const char* ui_main_get_plugin_stock_from_plugin(plugin_t* plugin);
const char* ui_main_get_plugin_stock(int id);

GtkWindow* ui_main_get_win(void);
void ui_switch_main(int number);
GtkWidget* ui_box_add_label(GtkBox* box, const char* text, unsigned padding);
gint sort_file_by_size(GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b);


GtkWidget*
ui_create_popup_button(GtkWidget* label, int arrow,
                       PopupFunction func, gpointer data);
gboolean
ui_iter_is_parent_to(GtkTreeModel* model, GtkTreeIter* parent, GtkTreeIter* node);

void ui_threads_show(const char* message);

GtkWidget* ui_string_list_show(GtkBox* box, const char* name,
                               const char* actions, gboolean allow_hide);
GtkWidget* ui_create_bandwidth_bar(const char* name, volume_t* volume, rate_t* rate);
GtkWidget* ui_string_list_view(const char* name, const char* actions);

GtkWidget* ui_create_charset_control(const char* charset, CharsetFunc func, void* data);
const char* ui_charset_control_get_charset(GtkWidget* combo);
void ui_charset_control_set_charset(GtkWidget* combo, const char* charset);

#ifdef WIN32
/* Export functions for dll */
void ui_init();
void ui_run(void);
void ui_done(void);
void ui_enter_main_loop(void);
void ui_leave_main_loop(void);
void ui_thread_new(thread_t* thread);
void ui_threads_show(const char* message);
#endif

/*@}*/

#endif
