/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.17 $
 * $Date: 2007/11/21 20:15:25 $
 * $Author: sgop $
 */

#include <string.h>

#include <gtk/gtk.h>
#include "lcore/preferences.h"
#include "lcore/plugin.h"
#include "lcore/l_i18n.h"
#include "ui_support.h"
#include "ui_style.h"
#include "ui_main.h"
#include "ui_network.h"
#ifdef WIN32
#include "lgtk2corealias.h"
#endif

static GtkNotebook* Notebook = NULL;
static GtkBox* NetBar = NULL;
static GtkBox* NetToolBox = NULL;
static GtkIconFactory* Factory = NULL;
static GtkWidget* OverviewLabel = NULL;
static GtkTable* MainTable = NULL;
static int SummaryCols = 4;
static GtkLabel* HeaderLabel = NULL;

/* static void on_connect(void) { */
/*   plugin_connect_change(-1, CONNECT_START); */
/* } */
/* static void on_disconnect(void) { */
/*   plugin_connect_change(-1, CONNECT_DISCONNECT); */
/* } */

static void on_do_auto(plugin_t* plugin)
{
    if (plugin && !(plugin->online->status & ONLINE_STATUS_ONLINE))
        plugin_connect_change(plugin->ID, CONNECT_START);
}
static void on_do_discon(plugin_t* plugin)
{
    if (plugin) plugin_connect_change(plugin->ID, CONNECT_STOP);
}

typedef struct {
  GtkLabel* users;
  GtkLabel* files;
  GtkLabel* content;
  GtkLabel* status;
  GtkWidget* do_auto;
  GtkWidget* do_discon;
} ui_pstatus_t;

static ui_pstatus_t* PluginStatus = NULL;

const char* _ui_get_number_str(int users) {
  if (users < 1000) 
    return l_printf("%d", users);
  else if (users < 1000000)
    return l_printf(_("%.1f thousand"), (double)users/1000.);
  else if (users < 1000000000)
    return l_printf(_("%.1f million"), (double)users/1000000.);
  else
    return l_printf(_("%.1f billion"), (double)users/1000000000.);
}

static void ui_online_status_changed(online_status_t* status, ui_pstatus_t* ui) {
  gboolean online = (status->status & ONLINE_STATUS_ONLINE);
  file_size_t size = status->gigs;
  
  gtk_label_set_text(ui->status, online?_("online"):_("offline"));
  gtk_widget_set_state(GTK_WIDGET(ui->status), online?GTK_STATE_ACTIVE:GTK_STATE_NORMAL);
  gtk_label_set_text(ui->users, (status->users && online)?_ui_get_number_str(status->users):_("unknown"));
  gtk_label_set_text(ui->files, (status->files && online)?_ui_get_number_str(status->files):_("unknown"));
  size *= 1024*1024*1024;
  gtk_label_set_text(ui->content, (status->gigs && online)?print_filesize(&size):_("unknown"));
  ui_event_label_set_sensitive(ui->do_auto, !online);
  ui_event_label_set_sensitive(ui->do_discon, online);
  
}


static void on_select_page(GtkWidget* child)
{
    int index = gtk_notebook_page_num(Notebook, child);
    plugin_t* plugin = g_object_get_data(G_OBJECT(child), "plugin");
    
    if (plugin)
    {
        gtk_label_set_markup
            (HeaderLabel,
             l_printf("<span size=\"larger\">%s</span>", plugin->info->name));
        gtk_widget_show(OverviewLabel);
    }
    else
    {
        gtk_label_set_markup
            (HeaderLabel,
             l_printf("<span size=\"larger\">%s</span>", _("Networks")));
        gtk_widget_hide(OverviewLabel);
    }

    if (index < 0) return;
    gtk_notebook_set_current_page(Notebook, index);
}

static void ui_network_add_plugin(int num, GtkWidget* widget) {
  plugin_t* plugin = plugin_get(num);
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* table;
  const char* stock;
  int row2 = 0;
  GtkWidget* item;

  static unsigned row=0, col=0;

  if (!PluginStatus) {
    PluginStatus = g_malloc0(sizeof(*PluginStatus)*plugin_get_number());
  }

  vbox = gtk_vbox_new(FALSE, 5);
  gtk_widget_show(vbox);
  gtk_table_attach(MainTable, vbox, col, col+1, row, row+1,
                   GTK_FILL, GTK_FILL, 0, 0);
  col++; if (col == SummaryCols) { col = 0; row++; }
    
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  stock = ui_main_get_plugin_stock_from_plugin(plugin);
  if (stock) {
    item = gtk_image_new_from_stock(stock, GTK_ICON_SIZE_BUTTON);
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  }
  item = ui_create_header_label(plugin_get_name(plugin));
/*     item = ui_create_header_box(plugin_get_name(plugin)); */
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  item = ui_create_event_label(_("details"), G_CALLBACK(on_select_page), widget, TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

    
  item = gtk_hseparator_new();
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  table = gtk_table_new(1, 2, FALSE);
  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
  gtk_table_set_row_spacings(GTK_TABLE(table), 2);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);
  
  item = gtk_label_new(_("<b>Status:</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_misc_set_alignment(GTK_MISC(item), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), item, 0, 1, row2, row2+1,
                   GTK_FILL, GTK_FILL, 0, 0);

  item = gtk_label_new(_("offline"));
  gtk_widget_set_name(item, "OnlineLabel");
  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), item, 1, 2, row2, row2+1,
                   GTK_FILL, GTK_FILL, 0, 0);
  PluginStatus[num].status = GTK_LABEL(item);
  row2++;

  item = gtk_label_new(_("<b>Users:</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_misc_set_alignment(GTK_MISC(item), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), item, 0, 1, row2, row2+1,
                   GTK_FILL, GTK_FILL, 0, 0);

  item = gtk_label_new(_("unknown"));
  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), item, 1, 2, row2, row2+1,
                   GTK_FILL, GTK_FILL, 0, 0);
  PluginStatus[num].users = GTK_LABEL(item);
  row2++;

  item = gtk_label_new(_("<b>Files:</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_misc_set_alignment(GTK_MISC(item), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), item, 0, 1, row2, row2+1,
                   GTK_FILL, GTK_FILL, 0, 0);

  item = gtk_label_new(_("unknown"));
  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), item, 1, 2, row2, row2+1,
                   GTK_FILL, GTK_FILL, 0, 0);
  PluginStatus[num].files = GTK_LABEL(item);
  row2++;

  item = gtk_label_new(_("<b>Content:</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_misc_set_alignment(GTK_MISC(item), 1, 0.5);
  gtk_table_attach(GTK_TABLE(table), item, 0, 1, row2, row2+1,
                   GTK_FILL, GTK_FILL, 0, 0);

  item = gtk_label_new(_("unknown"));
  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), item, 1, 2, row2, row2+1,
                   GTK_FILL, GTK_FILL, 0, 0);
  PluginStatus[num].content = GTK_LABEL(item);
  row2++;

  gtk_widget_show_all(table);


  item = gtk_hseparator_new();
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  PluginStatus[num].do_auto = item = ui_create_event_label
      (_("Start"), (GCallback)on_do_auto, plugin, TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    
  PluginStatus[num].do_discon = item = ui_create_event_label
      (_("Stop"), (GCallback)on_do_discon, plugin, TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start/*end*/(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  ui_event_label_set_sensitive(item, FALSE);
    

  object_monitor_d(OBJ(plugin->online),
                   (ObjectFunc)ui_online_status_changed, PluginStatus+num);
    
}

static void ui_network_add_summary_page() {
  GtkWidget* maint;
/*   GtkWidget* hbox; */
  
  SummaryCols = plugin_get_number();
  if (SummaryCols > 4) SummaryCols = 4;

  maint = gtk_table_new(1, SummaryCols, TRUE);
  MainTable = GTK_TABLE(maint);
  gtk_widget_show(maint);
  gtk_table_set_row_spacings(GTK_TABLE(maint), 20);
  gtk_table_set_col_spacings(GTK_TABLE(maint), 30);

/*   hbox = gtk_hbox_new(FALSE, 0); */
/*   gtk_widget_show(hbox); */
/*   gtk_box_pack_start(GTK_BOX(hbox), maint, FALSE, FALSE, 0); */
  
  ui_add_network_subpage(-1, ui_create_scrolled_area(maint, FALSE));
/*   ui_add_network_subpage(-1, maint); */
  gtk_container_set_border_width(GTK_CONTAINER(maint), 20);
}


GtkWidget* ui_create_network_content() {
  GtkWidget* notebook;
  GtkWidget* hbox;
  GtkWidget* vbox0;
  GtkWidget* vbox;
  GtkWidget* item;

  vbox0 = gtk_vbox_new(FALSE, 5);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox0), hbox, FALSE, FALSE, 0);

  item = gtk_image_new_from_stock(STOCK_LOPSTER_NETWORK, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  item = ui_create_header_label(_("Networks"));
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  HeaderLabel = GTK_LABEL(item);

  NetBar = GTK_BOX(gtk_hbox_new(FALSE, 10));
  gtk_widget_show(GTK_WIDGET(NetBar));
  gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(NetBar), FALSE, FALSE, 10);

  item = gtk_hseparator_new();
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(vbox0), item, FALSE, FALSE, 0);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox0), hbox, TRUE, TRUE, 0);

  {
      vbox = gtk_vbox_new(FALSE, 0);
      gtk_widget_show(vbox);
      gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
      NetToolBox = GTK_BOX(vbox);
  }

  notebook = gtk_notebook_new();
  gtk_widget_show(notebook);
  Notebook = GTK_NOTEBOOK(notebook);
  gtk_notebook_set_scrollable (Notebook, TRUE);
  gtk_notebook_set_tab_pos (Notebook, GTK_POS_LEFT);
  gtk_notebook_set_show_tabs(Notebook, FALSE);
  gtk_notebook_set_show_border(Notebook, FALSE);
  gtk_box_pack_start(GTK_BOX(hbox), notebook, TRUE, TRUE, 0);
  
  ui_network_add_summary_page();

  // the icon factory for the protocol icons
  Factory = gtk_icon_factory_new();
  gtk_icon_factory_add_default(Factory);

  gtk_widget_show(vbox0);
  return vbox0;
}

/** Add a new subpage in the main network page.
 * @param ID the plugin ID
 * @param pixmap the pixmap nearby the toggle button
 * @param widget the subpage.
 */
void ui_add_network_subpage(int ID, GtkWidget* widget)
{
    plugin_t* plugin;
    GtkWidget* item;
    
    plugin = plugin_get(ID);
    g_object_set_data(G_OBJECT(widget), "plugin", plugin);
    
    gtk_container_set_border_width(GTK_CONTAINER(widget), 0);
    
    if (plugin)
    {
        ui_network_add_plugin(ID, widget);
    }
    else
    {
        OverviewLabel = item = ui_create_event_label(_("Back to Overview"), G_CALLBACK(on_select_page), widget, TRUE);
        gtk_widget_hide(item);
        gtk_box_pack_start(NetBar, item, FALSE, FALSE, 0);
    }
    
    gtk_notebook_append_page(Notebook, widget, NULL);
}

#ifdef WIN32
#define _UI_NETWORK_C_
#include "lgtk2corealiasdef.c"
#endif
