/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2006/07/22 17:11:58 $
 * $Author: sid4u $
 */

#include <stdio.h>

#include <sys/types.h>
#include <sys/stat.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include <string.h>

#include <gtk/gtk.h>

#include "ui_notebook.h"
#include "gtkhtabs.h"

typedef struct {
  GtkWidget* main;
  GtkWidget* button;
  ui_nb_t* book;
  void* data;
} ui_page_t;

static ui_page_t* ui_notebook_page_by_data(ui_nb_t* nb, void* data) {
  GList* dlist;
  for (dlist = nb->pages; dlist; dlist = dlist->next) {
    ui_page_t* page = dlist->data;
    if (page->data == data) return page;
  }
  return NULL;
}

static ui_page_t* ui_notebook_page_by_main(ui_nb_t* nb, GtkWidget* widget) {
  GList* dlist;
  for (dlist = nb->pages; dlist; dlist = dlist->next) {
    ui_page_t* page = dlist->data;
    if (page->main == widget) return page;
  }
  return NULL;
}

static void
on_notebook_switch_page(GtkNotebook* notebook, GtkNotebookPage* p,
                        gint page_num, ui_nb_t* nb) {
  ui_page_t* page;
  GtkWidget* widget;

  (void)p;
  widget = gtk_notebook_get_nth_page(notebook, page_num);
  page = ui_notebook_page_by_main(nb, widget);
  if (!page) return;
  if (nb->change) nb->change(page->data);
  if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(page->button)))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(page->button), TRUE);
}

ui_nb_t* ui_create_notebook(TabChangedFunc change,
                            GtkOrientation orientation,
                            gboolean extended) {
  GtkWidget* notebook;
  GtkWidget* box;
  ui_nb_t* nb = g_malloc0(sizeof(*nb));
  
  notebook = gtk_notebook_new();
  gtk_widget_show(notebook);
  gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
  nb->notebook = GTK_NOTEBOOK(notebook);
  gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);

  g_signal_connect(G_OBJECT(notebook), "switch_page",
                   G_CALLBACK(on_notebook_switch_page), nb);
  
  nb->orient = orientation;
  if (nb->orient == GTK_ORIENTATION_HORIZONTAL) {
    if (extended) box = gtk_htabs_new(FALSE, 0);
    else box = gtk_hbox_new(FALSE, 0);
  } else
    box = gtk_vbox_new(FALSE, 0);

  gtk_widget_show(box);
  nb->tab_box = GTK_BOX(box);
  
  nb->change = change;

  return nb;
}

static void on_tab_toggled(GtkToggleButton* button, ui_page_t* p) {
  int index;

  if (gtk_toggle_button_get_active(button)) {
    index = gtk_notebook_page_num(p->book->notebook, p->main);
    if (index < 0) return;
    gtk_notebook_set_current_page(p->book->notebook, index);
  }
}

void ui_notebook_add_page(ui_nb_t* nb, GtkWidget* page,
                          GtkWidget* label, void* data) {
  GtkWidget* button;
  ui_page_t* p = g_malloc(sizeof(*p));

  p->main = page;
  p->data = data;
  p->book = nb;
  p->button = button = gtk_radio_button_new(nb->group);
  nb->group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
  
  gtk_widget_show(label);
  gtk_container_add(GTK_CONTAINER(button), label);

  nb->pages = g_list_append(nb->pages, p);

  gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(button), FALSE);
  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
  //  gtk_widget_set_size_request(button, 10, 20);
  gtk_widget_show(button);
  gtk_box_pack_start(nb->tab_box, button, FALSE, FALSE, 0);
  g_signal_connect(G_OBJECT(button), "toggled",
                   G_CALLBACK(on_tab_toggled), p);
  
  gtk_notebook_append_page(nb->notebook, page, NULL);

}

void ui_notebook_remove_page(ui_nb_t* nb, void* data) {
  ui_page_t* page;
  int index;

  page = ui_notebook_page_by_data(nb, data);
  if (!page) return;

  gtk_widget_destroy(page->button);

  index = gtk_notebook_page_num(nb->notebook, page->main);
  if (index != -1)
    gtk_notebook_remove_page(nb->notebook, index);

  nb->pages = g_list_remove(nb->pages, page);
  g_free(page);
  if (nb->pages) {
    page = nb->pages->data;
    nb->group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(page->button));
  } else {
    nb->group = NULL;
    if (nb->change) nb->change(NULL);
  }

}

void ui_notebook_switch_to_data(ui_nb_t* nb, void* data) {
  ui_page_t* page = ui_notebook_page_by_data(nb, data);
  int index;

  if (!page) {
    g_log("UI_NB", G_LOG_LEVEL_WARNING,
          "could not switch to page: not found");
    return;
  }

  index = gtk_notebook_page_num(nb->notebook, page->main);
  if (index < 0) return;
  gtk_notebook_set_current_page(nb->notebook, index);
}

void ui_notebook_switch_to_child(ui_nb_t* nb, GtkWidget* child) {
  int index = gtk_notebook_page_num(nb->notebook, child);
  if (index < 0) {
    g_log("UI_NB", G_LOG_LEVEL_WARNING,
          "could not switch to page: not found");
    return;
  }
  gtk_notebook_set_current_page(nb->notebook, index);
}

void ui_notebook_switch_to_index(ui_nb_t* nb, unsigned index) {
  gtk_notebook_set_current_page(nb->notebook, index);
}
