/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.8 $
 * $Date: 2007/04/29 09:21:47 $
 * $Author: sgop $
 */

#ifndef _UI_PREFERENCES_H
#define _UI_PREFERENCES_H

#include <gtk/gtk.h>

/** @defgroup UIPrefModule Integrate into the preferences dialog
 * @ingroup UserInterface
 */
/*@{*/

typedef struct _ui_pref_page_t ui_pref_page_t;

/** Function to create a section in a preferences page. Sections
 * usually are created with ui_create_section() or ui_pref_bool_section()
 * @return the section widget
 */
typedef GtkWidget* (*CreateSectionFunc)(gboolean* expand);
/** Function to apply or cancel a setting. 
 * @param name the name of the setting
 * @param data the user data pointer for the setting
 * @see ui_pref_set_funcs()
 */
typedef void (*UIPrefFunc)(const char* name, gpointer data);


/** @name Registering preferences pages. */
/*@{*/
ui_pref_page_t* ui_pref_register_leaf(const char* label, gboolean scrolled, const char* des,
                                      CreateSectionFunc first_func, ...);
ui_pref_page_t* ui_pref_register_page(const char* label, gboolean scrolled, const char* des,
                                      ui_pref_page_t* first_child, ...);
void ui_pref_hook_plugin(int ID, ui_pref_page_t* page);
/*@}*/

/** @name Connection between setting and GUI */
/*@{*/
void ui_pref_set_funcs(const char* name, UIPrefFunc apply, UIPrefFunc cancel, gpointer data);
void ui_pref_set_dirty(const char* name);
void ui_pref_cancel   (const char* name);
/*@}*/

/** @name Predefined wrappers.
 * Using one of these is a very simply way to create a GUI for the
 * preferences. Not need to mess around with ui_pref_set_funcs()
 */
/*@{*/
GtkWidget* ui_pref_string_entry(const char* name, const char* info);
GtkWidget* ui_pref_string_entry_inv(const char* name, const char* info);
GtkWidget* ui_pref_int_spin(const char* name, int min, int max);
GtkWidget* ui_pref_int_combo(const char* name, ...);
GtkWidget* ui_pref_charset_combo(const char* name);
GtkWidget* ui_pref_int_flags(const char* name, gboolean sync, int pos, ...);
GtkWidget* ui_pref_bool_label(const char* name, const char* tlabel, 
                              const char* flabel, GtkWidget* ref, gboolean swap);
GtkWidget* ui_pref_bool_section(const char* header, const char* help, GtkWidget** hbox,
                                const char* pref, const char* active,
                                const char* inactive, gboolean swap);
GtkWidget* ui_pref_bool_check(const char* name, const char* tlabel,
                              GtkWidget* ref, gboolean swap);
/*@}*/


/*@}*/

GtkWidget* ui_create_preferences(void);
void ui_pref_init(void);
void ui_pref_post_init(void);



#endif
