/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.12 $
 * $Date: 2007/04/29 09:21:47 $
 * $Author: sgop $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <string.h>

#include <gtk/gtk.h>

#include "lcore/l_base.h"
#include "lcore/load.h"
#include "lcore/utils.h"
#include "lcore/l_i18n.h"
#include "ui_main.h"
#include "ui_layout.h"
#include "ui_support.h"
#include "ui_statistic.h"
#include "ui_style.h"

#define TC    0
#define CL    1
#define BL    2
#define BC    3
#define TR    4
#define CR    5
#define BR    6
#define CC    7
#define TL    8
#define AREAS 9

typedef struct {
    GtkWidget* main;
    GtkWidget* area[AREAS];
    gboolean mouse_in;
    GList* loads;
    int interval;
    gboolean stretch;
    time_t next_update;
    double max;
    int items;
    int index;
} ui_load_t;

typedef struct {
    GtkToggleButton* check;
    GtkColorButton* color;
    load_t* load;
} ui_sl_t;

typedef struct {
    GtkWidget* win;
    ui_load_t* load;
    ui_sl_t* loads;
    int size;
    GtkEntry* range;
    GtkToggleButton* stretch;
} ui_setup_t;

#define STRETCH   1.2

#define B_LEFT    70
#define B_RIGHT   10
#define B_TOP     20
#define B_BOTTOM  15

static GList* Loads = NULL;

static void ui_statistic_force_update(void);

gboolean load_equal(load_t* a, load_t* b)
{
    if (a == b) return TRUE;
    else if (!a || !b) return FALSE;
    else if (a->in == b->in && !strcmp(a->name, b->name)) return TRUE;
    else return FALSE;
}


const char* load_get_name(load_t* load)
{
    return l_printf("load:%s:%s", load->in?"in":"out", load->name);
}

static char* ui_graph_get_load(ui_load_t* load)
{
    GList* dlist;
    char* res = l_strdup("");
  
    for (dlist = load->loads; dlist; dlist = dlist->next)
    {
        load_t* load = dlist->data;
        res = l_printf("%s%s%d:%s", res, *res==0?"":" ",
                       load->in?1:0, load->name);
    }
    return res;
}

static void ui_graph_set_load(ui_load_t* load, const char* data)
{
    char* temp;
    char* item;
    char* pos;
    load_t* l;
    
    g_list_free(load->loads);
    load->loads = NULL;
    if (!data) return;
    
    temp = l_strdup(data);
    item = arg(temp, 0);
    while (item)
    {
        pos = strchr(item, ':');
        if (pos)
        {
            *pos++ = 0;
            l = load_get(pos, atoi(item)?TRUE:FALSE);
            load->loads = g_list_append(load->loads, l);
        }
        item = arg(NULL, 0);
    }
}

void ui_layout_register_graph(const char* name)
{
    ui_layout_register(name,
                       (LayoutGetFunc)ui_graph_get_load,
                       (LayoutSetFunc)ui_graph_set_load);
    ui_layout_register_bool(l_printf("%s:stretch", name));
    ui_layout_register_int(l_printf("%s:interval", name));
}


void ui_statistic_init()
{
    GList* dlist;
    
    ui_style_register(N_("General style settings"), "load:color",
                      N_("Colors for bandwidth load graphs"),
                      N_("Background"), STYLE_COLOR, FALSE, 0xe2e2, 0xe2e2, 0xe2e2,
                      N_("Border"), STYLE_COLOR, FALSE, 0xe2e2, 0xe2e2, 0xe2e2,
                      N_("Graph border"), STYLE_COLOR, TRUE, 0xc51e, 0xc353, 0xbed8,
                      N_("Graph area"), STYLE_COLOR, TRUE, 0xf332, 0xf332, 0xf332,
                      N_("Text"), STYLE_COLOR, TRUE, 0xb18c, 0xb18c, 0xb18c,
                      NULL);
    
    ui_style_observe("load:color", (StyleFunc)ui_statistic_force_update, NULL);
    
    for (dlist = loads_get(); dlist; dlist = dlist->next)
    {
        load_t* load = dlist->data;
        const char* name = load_get_name(load);
        if (load->in)
        {
            //      const char* des = l_printf("Graph for incoming %s", load->name);
            ui_style_register(NULL, name, NULL,
                              N_("Graph"), STYLE_COLOR, TRUE, 10000, 10000, 50000,
                              NULL);
        }
        else
        {
            //      const char* des = l_printf("Graph for outgoing %s", load->name);
            ui_style_register(NULL, name, NULL, 
                              N_("Graph"), STYLE_COLOR, TRUE, 50000, 10000, 10000,
                              NULL);
        }
        ui_style_observe(name, (StyleFunc)ui_statistic_force_update, NULL);
    }
}

static void ui_statistic_force_update(void)
{
    GList* dlist;
    int i1;
    
    for (dlist = Loads; dlist; dlist = dlist->next)
    {
        ui_load_t* load = dlist->data;
        for (i1 = 0; i1 < AREAS; i1++)
        {
            GtkWidget* area = load->area[i1];
            GdkRegion* region;
            
            if (!area->window) continue;
            region = 
                gdk_drawable_get_visible_region((GdkDrawable*)(area->window));
            gdk_window_invalidate_region(area->window, region, FALSE);
            gdk_region_destroy(region);
        }
    }
}

static gboolean
area_expose_tc(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    GList* dlist;
    int width = widget->allocation.width;
    int height = widget->allocation.height;
    int posx = 5;
    
    (void)event;
    
    if (ui_style_sets("load:color", "Background"))
    {
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Background"), TRUE, 
                           0, 0, width, height);
    }
    if (ui_style_sets("load:color", "Border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      0, 0, width-1, 0);
    }
    if (ui_style_sets("load:color", "Graph border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Graph border"),
                      0, height-1, width-1, height-1);
    }
    
    for (dlist = uiload->loads; dlist; dlist = dlist->next)
    {
        int dx;
        load_t* load = dlist->data;
        GdkGC* gc;
        PangoLayout* layout = ui_get_text_layout(widget);
        
        gc = ui_style_get_gc(load_get_name(load), "Graph");
        if (!gc) continue;
        
        ui_layout_get_ext
            (layout, l_printf("%s:%s", _(load->name), load->in?_("in"):_("out")),
             &dx, NULL);
        gdk_draw_layout(window, gc, posx, 2, layout);
        posx += dx+5;
    }
    
    return TRUE;
}

static gboolean
area_expose_cc(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    int width = widget->allocation.width;
    int height = widget->allocation.height;
    GList* dlist;
    time_t period = 0;
    int index;
    int items;
    int x_old, y_old;
    int pos;
    double max;
    int i1;
    int next_update = 60;

    (void)event;

    if (ui_style_sets("load:color", "Graph area"))
    {
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Graph area"), TRUE, 
                           0, 0, width, height);
    }
  
    for (dlist = uiload->loads; dlist; dlist = dlist->next)
    {
        load_t* load = dlist->data;
        time_t p;
        if (load->first)
        {
            p = CurrentTime - load->first;
            if (p > period) period = p;
        }
    }
    if (!period) return TRUE;

    // use given interval if set, otherwise keep showing all
    if (uiload->interval) period = uiload->interval;
    // search for the appropriate array index
    index = 0;
    while (width * LoadSteps[index] < period)
    {
        if (index == BAND_SIZE-1) break;
        index++;
    }
    // if we stretch figure out how many values we need to cover
    // the time period, otherwise make every column a new value
    if (uiload->stretch)
        items = (period-1) / LoadSteps[index] + 1;
    else
        items = width;

    uiload->items = items;
    uiload->index = index;

    max = uiload->max*STRETCH;
    uiload->max = 0.;
    for (dlist = uiload->loads; dlist; dlist = dlist->next)
    {
        load_t* load = dlist->data;
        GdkGC* gc = ui_style_get_gc(load_get_name(load), "Graph");
        double value;

        if (!gc) continue;
        if (load->timestamp < CurrentTime) load_step(load);

        x_old = 0;
        y_old = 0;

        for (i1 = 0; i1 < items+1; i1++)
        {
            int h, w = width * i1 / items;

            pos = load->pos[index] - 1 - i1;
            while (pos < 0) pos += BAND_WIDTH;
            value = load->val[index][pos];
            if (value > uiload->max) uiload->max = value;
            h = (int)((double)height*value/max);

            if (i1 > 0) {
                gdk_draw_line(window, gc, width-x_old, height-y_old, width-w, height-h);
//                gdk_draw_rectangle(window, gc, TRUE, width-w, height-y_old, w-x_old, y_old);
            }

            x_old = w;
            y_old = h;
        }
    
    }

    // detect when we need to update again.
    for (dlist = uiload->loads; dlist; dlist = dlist->next)
    {
        load_t* load = dlist->data;
        int next;
    
        next = LoadSteps[index] - load->tno[index];
        if (next < next_update) next_update = next;
    }
    uiload->next_update = CurrentTime + next_update;

    return TRUE;
}

static gboolean
area_expose_cc_old(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    int width = widget->allocation.width;
    int height = widget->allocation.height;
    GList* dlist;
    time_t period = 0;
    int index;
    int items;
    int x_old;
    int pos;
    double max;
    int i1;
    int next_update = 60;

    (void)event;

    if (ui_style_sets("load:color", "Graph area"))
    {
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Graph area"), TRUE, 
                           0, 0, width, height);
    }
  
    for (dlist = uiload->loads; dlist; dlist = dlist->next)
    {
        load_t* load = dlist->data;
        time_t p;
        if (load->first)
        {
            p = CurrentTime - load->first;
            if (p > period) period = p;
        }
    }
    if (!period) return TRUE;

    // use given interval if set, otherwise keep showing all
    if (uiload->interval) period = uiload->interval;
    // search for the appropriate array index
    index = 0;
    while (width * LoadSteps[index] < period)
    {
        if (index == BAND_SIZE-1) break;
        index++;
    }
    // if we stretch figure out how many values we need to cover
    // the time period, otherwise make every column a new value
    if (uiload->stretch)
        items = (period-1) / LoadSteps[index] + 1;
    else
        items = width;

    uiload->items = items;
    uiload->index = index;

    x_old = 0;
    max = uiload->max*STRETCH;
    uiload->max = 0.;
    //  printf("max is %f\n", max);
    for (i1 = 0; i1 < items; i1++)
    {
        int w = width * (i1+1) / items;
        int h;
        if (w == x_old) continue;
        for (dlist = uiload->loads; dlist; dlist = dlist->next)
        {
            double value;
            load_t* load = dlist->data;
            GdkGC* gc;

            gc = ui_style_get_gc(load_get_name(load), "Graph");
            if (!gc) continue;

            // fill unfilled values
            if (load->timestamp < CurrentTime) load_step(load);

            pos = load->pos[index] - 1 - i1;
            while (pos < 0) pos += BAND_WIDTH;
            value = load->val[index][pos];
            //      printf(" - %03d / %03d : %02d / %3.3f\n", x_old, w, pos, value);
            if (value > uiload->max) uiload->max = value;
            h = (int)((double)height*value/max);
            gdk_draw_rectangle(window, gc, TRUE, width-w, height-h, w-x_old, h);
        }
    
        x_old = w;
    }

    // detect when we need to update again.
    for (dlist = uiload->loads; dlist; dlist = dlist->next)
    {
        load_t* load = dlist->data;
        int next;
    
        next = LoadSteps[index] - load->tno[index];
        if (next < next_update) next_update = next;
    }
    uiload->next_update = CurrentTime + next_update;

    return TRUE;
}

static gboolean
area_expose_bc(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    int width = widget->allocation.width;
    int height = widget->allocation.height;

    (void)event;
    if (ui_style_sets("load:color", "Background"))
    {
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Background"), TRUE, 
                           0, 0, width, height);
    }
    if (ui_style_sets("load:color", "Border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      0, height-1, width-1, height-1);
    }
    if (ui_style_sets("load:color", "Graph border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Graph border"),
                      0, 0, width-1, 0);
    }

    if (uiload->items > 0)
    {
        int step = 1;
        int w, i1;
        const char* str;
        PangoLayout* layout = ui_get_text_layout(widget);

        while (width * step / uiload->items < 60) step *= 2;
        i1 = step;
        while (i1 < uiload->items)
        {
            w = i1 * LoadSteps[uiload->index];
            str = print_duration_unit(&w);
            w = width * i1 / uiload->items;
            gdk_draw_line(window, ui_style_get_gc("load:color", "Text"),
                          width-w, 0, width-w, 10);

            pango_layout_set_text(layout, str, -1);
            gdk_draw_layout(window, ui_style_get_gc("load:color", "Text"),
                            width-w+3, 0, layout);
            i1 += step;
        }
    }

    return TRUE;
}

static gboolean
area_expose_cl(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    int width = widget->allocation.width;
    int height = widget->allocation.height;
    int size = 0;
/*   GtkRequisition req; */

    (void)event;

    if (ui_style_sets("load:color", "Background"))
    {
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Background"), TRUE, 
                           0, 0, width, height);
    }
    if (ui_style_sets("load:color", "Border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      0, 0, 0, height-1);
    }
    if (uiload->max >= 8)
    {
        double max = uiload->max * STRETCH;
        int unit = 1;
        int val;
        int h;
        int dx;
        const char* str;
        PangoLayout* layout = ui_get_text_layout(widget);
    
        while (height * unit < 25 * max) unit *= 2;

        val = 0;
        while (1)
        {
            file_size_t v;
            val += unit;
            v = val;
            h = (int)((double)height*val/max);
            if (height < h) break;
            str = print_filesize(&v); //FIXME: print speed

            pango_layout_set_text(layout, str, -1);
            ui_layout_get_ext(layout, str, &dx, NULL);
            if (dx+4 > size) size = dx+4;
      
            gdk_draw_layout(window, ui_style_get_gc("load:color", "Text"),
                            width-dx-2, height-h, layout);
            gdk_draw_line(window, ui_style_get_gc("load:color", "Text"),
                          width-11, height-1 - h,
                          width-1, height-1 - h);
        }
    }

    if (ui_style_sets("load:color", "Graph border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Graph border"),
                      width-1, 0, width-1, height-1);
    }

/*   gtk_widget_size_request(widget, &req); */
/*   if (size && req.width != size) */
/*     gtk_widget_set_size_request(widget, size, -1); */

    return TRUE;
}

static gboolean
area_expose_bl(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    int width = widget->allocation.width;
    int height = widget->allocation.height;

    (void)event;
    (void)uiload;

    if (ui_style_sets("load:color", "Background"))
    {
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Background"), TRUE, 
                           0, 0, width, height);
    }
    if (ui_style_sets("load:color", "Border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      0, 0, 0, height-1);
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      0, height-1, width-1, height-1);
    }

    return TRUE;
}


static gboolean
area_expose_tr(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    int width = widget->allocation.width;
    int height = widget->allocation.height;

    (void)event;
    (void)uiload;

    if (ui_style_sets("load:color", "Background"))
    {
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Background"), TRUE, 
                           0, 0, width, height);
    }
    if (ui_style_sets("load:color", "Border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      0, 0, width-1, 0);
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      width-1, 0, width-1, height-1);
    }

    return TRUE;
}

static gboolean
area_expose_tl(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    int width = widget->allocation.width;
    int height = widget->allocation.height;

    (void)event;
    if (uiload->mouse_in)
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Text"), TRUE, 
                           0, 0, width, height);
    else
    {
        if (ui_style_sets("load:color", "Background"))
        {
            gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Background"), TRUE, 
                               0, 0, width, height);
        }
    }
    if (ui_style_sets("load:color", "Border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      0, 0, width-1, 0);
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      0, 0, 0, height-1);
    }

    {
        PangoLayout* layout = ui_get_text_layout(widget);
        GdkGC* gc;
        int dx;

        if (uiload->mouse_in)
            gc = ui_style_get_gc("load:color", "Background");
        else
            gc = ui_style_get_gc("load:color", "Text");

        ui_layout_get_ext(layout, _("setup"), &dx, NULL);
        gdk_draw_layout(window, gc,(width-dx)/2, 2, layout);
    }

    return TRUE;
}

static gboolean
area_expose_cr(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    int width = widget->allocation.width;
    int height = widget->allocation.height;

    (void)event;
    (void)uiload;

    if (ui_style_sets("load:color", "Background"))
    {
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Background"), TRUE, 
                           0, 0, width, height);
    }
    if (ui_style_sets("load:color", "Border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      width-1, 0, width-1, height-1);
    }
    if (ui_style_sets("load:color", "Graph border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Graph border"),
                      0, 0, 0, height-1);
    }

    return TRUE;
}

static gboolean
area_expose_br(GtkWidget *widget, GdkEventExpose *event, ui_load_t* uiload)
{
    GdkWindow* window = widget->window;
    int width = widget->allocation.width;
    int height = widget->allocation.height;

    (void)event;
    (void)uiload;

    if (ui_style_sets("load:color", "Background"))
    {
        gdk_draw_rectangle(window, ui_style_get_gc("load:color", "Background"), TRUE, 
                           0, 0, width, height);
    }
    if (ui_style_sets("load:color", "Border"))
    {
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      width-1, 0, width-1, height-1);
        gdk_draw_line(window, ui_style_get_gc("load:color", "Border"),
                      0, height-1, width-1, height-1);
    }

    return TRUE;
}



static void on_cancel_setup(ui_setup_t* us)
{
    if (us->win) gtk_widget_destroy(us->win);
    g_free(us->loads);
    g_free(us);
}

static void on_apply_setup(ui_setup_t* us)
{
    ui_load_t* ul = us->load;
    int i1;

    g_list_free(ul->loads);
    ul->loads = NULL;

    ul->stretch = gtk_toggle_button_get_active(us->stretch);
    ul->interval = extract_duration(gtk_entry_get_text(us->range));

    for (i1 = 0; i1 < us->size; i1++)
    {
        ui_sl_t* sl = &(us->loads[i1]);
        GdkColor color;

        if (gtk_toggle_button_get_active(sl->check))
            ul->loads = g_list_append(ul->loads, sl->load);
        gtk_color_button_get_color(sl->color, &color);
        ui_style_set(load_get_name(sl->load),
                     "Graph", color.red, color.green, color.blue,
                     NULL);
    }

    on_cancel_setup(us);
}

static void on_setup(ui_load_t* uiload)
{
    GtkWidget* window;
    GtkWidget* vbox;
    GtkWidget* table;
    GtkWidget* hbox;
    GtkWidget* label;
    GtkWidget* check;
    GtkWidget* button;
    GtkWidget* entry;
    int i1;
    int row;
    GList* dlist;
    ui_setup_t* us;

    us = g_malloc0(sizeof(*us));
    us->load = uiload;

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    us->win = window;
    gtk_window_set_title(GTK_WINDOW(window), _("Setup graph"));
    gtk_window_set_type_hint(GTK_WINDOW(window), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER_ON_PARENT);
    if (ui_main_get_win())
    {
        gtk_window_set_transient_for(GTK_WINDOW(window), ui_main_get_win());
        gtk_window_set_destroy_with_parent(GTK_WINDOW(window), TRUE);
    }
    g_signal_connect_swapped(G_OBJECT(window), "delete_event",
                             G_CALLBACK(on_cancel_setup), us);

    vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
    gtk_container_add(GTK_CONTAINER(window), vbox);

    table = gtk_table_new(4, 3, FALSE);
    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);

    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    gtk_table_set_row_spacing(GTK_TABLE(table), 0, 5);
    gtk_table_set_row_spacing(GTK_TABLE(table), 1, 10);
    gtk_table_set_row_spacing(GTK_TABLE(table), 2, 5);

    label = ui_create_header_label(_("General display"));
    gtk_table_attach(GTK_TABLE(table), label, 0, 3, 0, 1,
                     GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

    label = gtk_label_new("");
    gtk_widget_set_size_request(label, 20, -1);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
                     GTK_FILL,GTK_FILL,0,0);


    hbox = gtk_hbox_new(FALSE, 5);
    gtk_table_attach(GTK_TABLE(table), hbox, 1, 3, 1, 2,
                     GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

    label = gtk_label_new(_("Display range (0=everything)"));
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    entry = gtk_entry_new();
    us->range = GTK_ENTRY(entry);
    gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
    i1 = uiload->interval;
    gtk_entry_set_text(GTK_ENTRY(entry), print_seconds_exact(&i1));
    gtk_widget_set_size_request(entry, 50, -1);

    check = gtk_check_button_new_with_label(_("stretch graph"));
    us->stretch = GTK_TOGGLE_BUTTON(check);
    gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);
    if (uiload->stretch)
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), TRUE);

    label = ui_create_header_label(_("Loads to show"));
    gtk_table_attach(GTK_TABLE(table), label, 0, 3, 2, 3,
                     GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

    row = 3;
    dlist = loads_get();
    us->size = g_list_length(dlist);
    us->loads = g_malloc0(sizeof(ui_sl_t)*us->size);
    for (i1 = 0; dlist; dlist = dlist->next)
    {
        load_t* load = dlist->data;
        GdkColor* color;
  
        if (!load->in) continue;

        // incoming
        us->loads[i1].load = load;
        hbox = gtk_hbox_new(FALSE, 5);
        gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, row, row+1,
                         GTK_FILL,GTK_FILL,0,0);
    
        check = gtk_check_button_new_with_label(l_printf(_("Incoming %s"), _(load->name)));
        us->loads[i1].check = GTK_TOGGLE_BUTTON(check);
        gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);

        color = ui_style_get_color(load_get_name(load), "Graph");
        if (g_list_find(uiload->loads, load))
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), TRUE);

        button = gtk_color_button_new_with_color(color);
        us->loads[i1].color = GTK_COLOR_BUTTON(button);
        gtk_color_button_set_title
            (GTK_COLOR_BUTTON(button), l_printf(_("Incoming %s"), _(load->name)));
        gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
        i1++;

        // outgoing
        load = load_get(load->name, FALSE);
        us->loads[i1].load = load;
        hbox = gtk_hbox_new(FALSE, 5);
        gtk_table_attach(GTK_TABLE(table), hbox, 2, 3, row, row+1,
                         GTK_FILL,GTK_FILL,0,0);
        row++;
    
        check = gtk_check_button_new_with_label(l_printf(_("Outgoing %s"), _(load->name)));
        us->loads[i1].check = GTK_TOGGLE_BUTTON(check);
        gtk_box_pack_start(GTK_BOX(hbox), check, TRUE, TRUE, 0);

        color = ui_style_get_color(load_get_name(load), "Graph");
        if (g_list_find(uiload->loads, load))
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), TRUE);

        button = gtk_color_button_new_with_color(color);
        us->loads[i1].color = GTK_COLOR_BUTTON(button);
        gtk_color_button_set_title
            (GTK_COLOR_BUTTON(button), l_printf(_("Outgoing %s"), _(load->name)));
        gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
        i1++;
    }

    ///////////

        hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
        gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

        button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
        gtk_container_add(GTK_CONTAINER(hbox), button);
        g_signal_connect_swapped(G_OBJECT(button), "clicked",
                                 G_CALLBACK(on_cancel_setup), us);

        button = gtk_button_new_from_stock(GTK_STOCK_OK);
        gtk_container_add(GTK_CONTAINER(hbox), button);
        g_signal_connect_swapped(G_OBJECT(button), "clicked",
                                 G_CALLBACK(on_apply_setup), us);
  
        gtk_widget_show_all(window);

}

static gboolean on_release_tl(ui_load_t* load)
{
    if (load->mouse_in) on_setup(load);
    return TRUE;
}

static gboolean
on_enter_tl(GtkWidget* area, GdkEventCrossing *event, ui_load_t* load)
{
    (void)event;
    load->mouse_in = TRUE;
    gdk_window_invalidate_rect(area->window, NULL, FALSE);
    return TRUE;
}
static gboolean
on_leave_tl(GtkWidget* area, GdkEventCrossing *event, ui_load_t* load)
{
    (void)event;
    load->mouse_in = FALSE;
    gdk_window_invalidate_rect(area->window, NULL, FALSE);
    return TRUE;
}

static ui_load_t* ui_load_new(void)
{
    GtkWidget* da;
    GtkWidget* table;
    GtkWidget* frame;
    ui_load_t* load = g_malloc0(sizeof(*load));

    load->interval = 120*60;
    load->stretch = FALSE;

    load->loads = NULL;
    load->max = 1024.;  // initial value
    Loads = g_list_append(Loads, load);

    frame = gtk_frame_new(NULL);
    gtk_widget_show(frame);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    load->main = frame;

    table = gtk_table_new(3, 3, FALSE);
    gtk_widget_show(table);
    gtk_table_set_col_spacings(GTK_TABLE(table), 0);
    gtk_table_set_row_spacings(GTK_TABLE(table), 0);
    gtk_container_add(GTK_CONTAINER(frame), table);

    da = gtk_drawing_area_new();
    gtk_widget_set_size_request(da, B_RIGHT, B_TOP);
    gtk_widget_show(da);
    gtk_table_attach(GTK_TABLE(table), da, 2, 3, 0, 1,
                     GTK_FILL, GTK_FILL, 0, 0);
    gtk_widget_set_events(da, GDK_EXPOSURE_MASK);
    load->area[TR] = da;
    g_signal_connect(G_OBJECT(da), "expose-event",
                     G_CALLBACK(area_expose_tr), load);

    da = gtk_drawing_area_new();
    gtk_widget_set_size_request(da, B_RIGHT, -1);
    gtk_widget_show(da);
    gtk_table_attach(GTK_TABLE(table), da, 2, 3, 1, 2,
                     GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
    gtk_widget_set_events(da, GDK_EXPOSURE_MASK);
    load->area[CR] = da;
    g_signal_connect(G_OBJECT(da), "expose-event",
                     G_CALLBACK(area_expose_cr), load);

    da = gtk_drawing_area_new();
    gtk_widget_set_size_request(da, B_RIGHT, B_BOTTOM);
    gtk_widget_show(da);
    gtk_table_attach(GTK_TABLE(table), da, 2, 3, 2, 3,
                     GTK_FILL, GTK_FILL, 0, 0);
    gtk_widget_set_events(da, GDK_EXPOSURE_MASK);
    load->area[BR] = da;
    g_signal_connect(G_OBJECT(da), "expose-event",
                     G_CALLBACK(area_expose_br), load);


    da = gtk_drawing_area_new();
    gtk_widget_set_size_request(da, -1, B_TOP);
    gtk_widget_show(da);
    gtk_table_attach(GTK_TABLE(table), da, 1, 2, 0, 1,
                     GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_widget_set_events(da, GDK_EXPOSURE_MASK);
    load->area[TC] = da;
    g_signal_connect(G_OBJECT(da), "expose-event",
                     G_CALLBACK(area_expose_tc), load);

    da = gtk_drawing_area_new();
    gtk_widget_set_size_request(da, B_LEFT, -1);
    gtk_widget_show(da);
    gtk_table_attach(GTK_TABLE(table), da, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL|GTK_EXPAND, 0, 0);
    gtk_widget_set_events(da, GDK_EXPOSURE_MASK);
    load->area[CL] = da;
    g_signal_connect(G_OBJECT (da), "expose-event",
                     G_CALLBACK(area_expose_cl), load);

    da = gtk_drawing_area_new();
    gtk_widget_show(da);
    gtk_table_attach(GTK_TABLE(table), da, 1, 2, 1, 2,
                     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
    gtk_widget_set_events(da, GDK_EXPOSURE_MASK);
    load->area[CC] = da;
    g_signal_connect(G_OBJECT (da), "expose-event",
                     G_CALLBACK(area_expose_cc), load);

    da = gtk_drawing_area_new();
    gtk_widget_set_size_request(da, B_LEFT, B_BOTTOM);
    gtk_widget_show(da);
    gtk_table_attach(GTK_TABLE(table), da, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 0, 0);
    gtk_widget_set_events(da, GDK_EXPOSURE_MASK);
    load->area[BL] = da;
    g_signal_connect(G_OBJECT(da), "expose-event",
                     G_CALLBACK(area_expose_bl), load);

    da = gtk_drawing_area_new();
    gtk_widget_set_size_request(da, -1, B_BOTTOM);
    gtk_widget_show(da);
    gtk_table_attach(GTK_TABLE(table), da, 1, 2, 2, 3,
                     GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_widget_set_events(da, GDK_EXPOSURE_MASK);
    g_signal_connect(G_OBJECT(da), "expose-event",
                     G_CALLBACK(area_expose_bc), load);
    load->area[BC] = da;

    da = gtk_drawing_area_new();
    gtk_widget_set_size_request(da, B_LEFT, B_TOP);
    gtk_widget_show(da);
    gtk_table_attach(GTK_TABLE(table), da, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 0, 0);
    gtk_widget_set_events
        (da, GDK_EXPOSURE_MASK|
         GDK_BUTTON_PRESS_MASK|GDK_BUTTON_RELEASE_MASK|
         GDK_LEAVE_NOTIFY_MASK|GDK_ENTER_NOTIFY_MASK
            );
    g_signal_connect(G_OBJECT(da), "expose-event",
                     G_CALLBACK(area_expose_tl), load);
    g_signal_connect_swapped(G_OBJECT(da), "button-release-event",
                             G_CALLBACK(on_release_tl), load);
    g_signal_connect(G_OBJECT(da), "enter-notify-event",
                     G_CALLBACK(on_enter_tl), load);
    g_signal_connect(G_OBJECT(da), "leave-notify-event",
                     G_CALLBACK(on_leave_tl), load);
    load->area[TL] = da;


    return load;
}

GtkWidget* ui_create_statistic_content()
{
    GtkWidget* table;
    ui_load_t* load;

    //////////////
    table = gtk_table_new(3, 2, TRUE);
    gtk_widget_show(table);
    gtk_container_set_border_width(GTK_CONTAINER(table), 3);
    gtk_table_set_col_spacings(GTK_TABLE(table), 3);
    gtk_table_set_row_spacings(GTK_TABLE(table), 3);
  
    // area 1
    load = ui_load_new();
    gtk_table_attach(GTK_TABLE(table), load->main, 0, 1, 0, 1,
                     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
    ui_layout_monitor("core:stat:graph1", load);
    ui_layout_monitor("core:stat:graph1:stretch", &(load->stretch));
    ui_layout_monitor("core:stat:graph1:interval", &(load->interval));
  
    // area 2
    load = ui_load_new();
    gtk_table_attach(GTK_TABLE(table), load->main, 1, 2, 0, 1,
                     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
    ui_layout_monitor("core:stat:graph2", load);
    ui_layout_monitor("core:stat:graph2:stretch", &(load->stretch));
    ui_layout_monitor("core:stat:graph2:interval", &(load->interval));
  
    // area 3
    load = ui_load_new();
    gtk_table_attach(GTK_TABLE(table), load->main, 1, 2, 1, 2,
                     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
    ui_layout_monitor("core:stat:graph3", load);
    ui_layout_monitor("core:stat:graph3:stretch", &(load->stretch));
    ui_layout_monitor("core:stat:graph3:interval", &(load->interval));
  
    // area 4
    load = ui_load_new();
    gtk_table_attach(GTK_TABLE(table), load->main, 1, 2, 2, 3,
                     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
    ui_layout_monitor("core:stat:graph4", load);
    ui_layout_monitor("core:stat:graph4:stretch", &(load->stretch));
    ui_layout_monitor("core:stat:graph4:interval", &(load->interval));
  
    ////// TEMPARARY until we have other content to display
    // area 5
    load = ui_load_new();
    gtk_table_attach(GTK_TABLE(table), load->main, 0, 1, 1, 2,
                     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
    ui_layout_monitor("core:stat:graph5", load);
    ui_layout_monitor("core:stat:graph5:stretch", &(load->stretch));
    ui_layout_monitor("core:stat:graph5:interval", &(load->interval));
  
    // area 6
    load = ui_load_new();
    gtk_table_attach(GTK_TABLE(table), load->main, 0, 1, 2, 3,
                     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
    ui_layout_monitor("core:stat:graph6", load);
    ui_layout_monitor("core:stat:graph6:stretch", &(load->stretch));
    ui_layout_monitor("core:stat:graph6:interval", &(load->interval));
  
    return table;
}

void ui_statistic_periodic_update()
{
    GList* dlist;
    int i1;

    for (dlist = Loads; dlist; dlist = dlist->next)
    {
        ui_load_t* load = dlist->data;
        for (i1 = 0; i1 < AREAS; i1++)
        {
            GtkWidget* area = load->area[i1];
            if (i1 != CC && i1 != CL && i1 != BC) continue;
            if (area->window && (!load->next_update || load->next_update <= CurrentTime))
            {
                GdkRegion* region = 
                    gdk_drawable_get_visible_region((GdkDrawable*)(area->window));
                gdk_window_invalidate_region(area->window, region, FALSE);
                gdk_region_destroy(region);
            }
        }		  
    }
}
