/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _UI_STYLE_H_
#define _UI_STYLE_H_

/** @defgroup UIStyleModule Define your own styles
 * @ingroup UserInterface
 */
/*@{*/

/** @name Style types */
/*@{*/
#define STYLE_NONE       0 ///< no style
#define STYLE_COLOR      1 ///< color
#define STYLE_FONT       2 ///< font name
#define STYLE_UNDERLINE  3 ///< underlined?
/*@}*/

/** Observer function for a style.
 * @param name name of the style.
 * @param user data.
 */
typedef void (*StyleFunc)(const char* name, gpointer data);

void ui_styles_init();
void ui_styles_set_main(GtkWidget* widget);

void ui_style_register (const char* group, const char* name, const char* des,
                        const char* first_name, ...);
void ui_style_observe  (const char* name, StyleFunc func, gpointer data);
void ui_style_unobserve(const char* name, StyleFunc func, gpointer data);

void ui_style_set(const char* name, const char* first_item, ...);
void ui_style_unset(const char* name, const char* item);
gboolean ui_style_sets(const char* name, const char* item);

GdkGC* ui_style_get_gc(const char* name, const char* color);
GdkColor* ui_style_get_color(const char* name, const char* color);
const char* ui_style_get_font(const char* name, const char* font);
gboolean ui_style_get_underline(const char* name, const char* underline);

void ui_create_style_editor(void);
void ui_styles_load(void);

/*@}*/

#endif
