/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.12 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#ifndef _UI_SUPPORT_H_
#define _UI_SUPPORT_H_

#include <gtk/gtk.h>

#include "lcore/files.h"
#include "lcore/user.h"

/** @defgroup UISupportModule Functions which can save lots of coding
 * @ingroup UserInterface
 */
/*@{*/


/** @name Special lopster sock items */
/*@{*/
#define STOCK_LOPSTER_NETWORK     "lopster-server"    ///< the network tab icon
#define STOCK_LOPSTER_CHAT        "lopster-chat"      ///< the chat tab icon
#define STOCK_LOPSTER_SEARCH      "lopster-search"    ///< the search tab icon
#define STOCK_LOPSTER_LIBRARY     "lopster-library"   ///< the library tab icon
#define STOCK_LOPSTER_HOTLIST     "lopster-hotlist"   ///< the buddy tab icon
#define STOCK_LOPSTER_TRANSFER    "lopster-transfer"  ///< the transfer tab icon
#define STOCK_LOPSTER_UPLOAD      "lopster-upload"    ///< the upload icon
#define STOCK_LOPSTER_DOWNLOAD    "lopster-download"  ///< the download icon
#define STOCK_LOPSTER_STATISTIC   "lopster-statistic" ///< the statistics tab icon
#define STOCK_LOPSTER_SETTINGS    "lopster-settings"  ///< the settings tab icon (unused atm)

#define STOCK_LOPSTER_CLOSE       "lopster-close"     ///< a stock icon for close buttons

#define STOCK_LOPSTER_MEDIA_PREV  "lopster-prev"      ///< media previous
#define STOCK_LOPSTER_MEDIA_PAUSE "lopster-pause"     ///< media pause
#define STOCK_LOPSTER_MEDIA_PLAY  "lopster-play"      ///< media play
#define STOCK_LOPSTER_MEDIA_STOP  "lopster-stop"      ///< media stop
#define STOCK_LOPSTER_MEDIA_NEXT  "lopster-next"      ///< media next

#define STOCK_LOPSTER_MTYPE_OTHER   "lopster-mtype-other"  ///< unknown type
#define STOCK_LOPSTER_MTYPE_AUDIO   "lopster-mtype-audio"  ///< audio
#define STOCK_LOPSTER_MTYPE_VIDEO   "lopster-mtype-video"  ///< video
#define STOCK_LOPSTER_MTYPE_APP     "lopster-mtype-app"    ///< application
#define STOCK_LOPSTER_MTYPE_IMAGE   "lopster-mtype-image"  ///< image
#define STOCK_LOPSTER_MTYPE_DOC     "lopster-mtype-doc"    ///< documents
#define STOCK_LOPSTER_MTYPE_FOLDER  "lopster-mtype-folder" ///< folders
#define STOCK_LOPSTER_MTYPE_ALL     "lopster-mtype-all"    ///< all mediatypes

#define STOCK_LOPSTER_ICON          "lopster-icon"         ///< lopster icon
/*@}*/


/** @name Special labels */
/*@{*/
GtkWidget* ui_create_header_label(const char* text);
GtkWidget* ui_create_header_box(const char* text);
void ui_set_header_label(GtkLabel* label, const char* text);
GtkWidget* ui_create_event_label(const char* text, GCallback callback, gpointer user_data, gboolean swapped);
void ui_event_label_set_sensitive(GtkWidget* ebox, gboolean set);
void ui_event_label_set_text(GtkWidget* label, const char* text);
GtkWidget* ui_create_show_hide_label(const char* show_text, const char* hide_text, GtkWidget* widget);
GtkWidget* ui_create_url_label(const char* text, const char* url);
/*@}*/

/** @name Special buttons */
/*@{*/
GtkWidget* ui_create_tool_button(const char* stock, const char* tip);
GtkWidget* ui_create_close_button(const char* tip);
GtkWidget* ui_create_date_button(void);
/*@}*/

/** @name Special entries */
/*@{*/
GtkWidget* ui_entry_new(const char* text);
GtkWidget* ui_entry_new_default(const char* text, const char* def);
GtkWidget* ui_entry_new_default_pass(const char* text, const char* def);
void ui_entry_set_default(GtkEntry* entry, const char* default_text);
const char* ui_entry_get_text(GtkEntry* entry);
void ui_entry_set_text(GtkEntry* entry, const char* text);
/*@}*/

/** @name Layout widgets */
/*@{*/
GtkWidget* ui_create_bbox(gint spacing, gint layout);
GtkWidget* ui_create_headed_item(const char* header, GtkWidget* item);
GtkWidget* ui_create_advanced_headed_item(const char* header, GtkWidget* item, GtkWidget** hbox);
GtkWidget* ui_create_functional_headed_item(const char* header, GtkWidget* item, const char* action, GCallback func);
GtkWidget* ui_create_section(const char* header, const char* help, GtkWidget** hbox, GtkWidget** header_hbox);
GtkWidget* ui_create_expander_section(const char* header, const char* help, GtkWidget** hbox);
GtkWidget* ui_create_scrolled_area(GtkWidget* content, gboolean s);
/*@}*/

/** @name Table packer */
/*@{*/
void ui_table_add_item (GtkTable* table, int* line, GtkWidget* item1, GtkWidget* item2);
void ui_table_add_line (GtkTable* table, int* line, const char* l, const char* text);
void ui_table_add_sep  (GtkTable* table, int* line);
void ui_table_add_attr (GtkTable* table, int* line, attribute_t* attr);
void ui_table_add_attrc(GtkTable* table, int* line, attr_container_t* cont, const char* domain);
void ui_table_add_file (GtkTable* table, int* line, file_t* file);
void ui_table_add_user (GtkTable* table, int* line, user_t* user);
/*@}*/

/** @name Utility functions */
/*@{*/
GtkWidget* create_pixmap(const gchar* filename);
GdkPixbuf* create_pixbuf(const gchar *filename); //remove from ui_support, is base
PangoLayout* ui_get_text_layout(GtkWidget* widget);
void ui_layout_get_ext(PangoLayout* layout, const char* text, int* width, int* height);
char* ui_select_file(const char* title, const char* current, GtkWindow* parent);
char* ui_select_folder(const char* title, const char* current, GtkWindow* parent);
GtkWidget* ui_create_window(const char* title, unsigned width, unsigned height);
/*@}*/



/*@}*/

#endif
