/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.14 $
 * $Date: 2007/11/21 20:15:25 $
 * $Author: sgop $
 */

#include <string.h>

#include <gtk/gtk.h>

#include "ui_main.h"
#include "ui_support.h"
#include "ui_treeview.h"
#include "ui_style.h"
#include "ui_tooltips.h"
#include "ui_layout.h"
#include "renderer_upload.h"
#include "lcore/upload.h"
#include "lcore/l_i18n.h"

static GtkListStore* UploadStore  = NULL;
static GtkTreeView*  UploadView   = NULL;
static GList* Uploads = NULL;
static rate_t* TheRate = NULL;
static volume_t* TheVolume = NULL;

static void ui_style_changed(const char* name, GObject* rend) {
  (void)name;
  g_object_set(rend,
               "background-color", ui_style_get_color("transfer:bar", "Background"),
               "border-color",   ui_style_get_color("transfer:bar", "Border"),
               "active-color",   ui_style_get_color("transfer:bar", "In-progress"),
               "finished-color", ui_style_get_color("transfer:bar", "Finished"),
               "progress-color", ui_style_get_color("transfer:bar", "Percentage"),
               NULL);
}

static GtkWidget* on_upload_tip(upload_t* upload, unsigned depth) {
  GtkWidget* table;
  int line = 0;
  char* str;

  (void)depth;

  table = gtk_table_new(1, 2, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);

  ui_table_add_file(GTK_TABLE(table), &line, upload->file);
  ui_table_add_sep(GTK_TABLE(table), &line);
  ui_table_add_user(GTK_TABLE(table), &line, upload->user);
  ui_table_add_sep(GTK_TABLE(table), &line);
  
  str = l_printf("%s", ctime(&upload->request_time));
  str[strlen(str)-1] = 0;
  ui_table_add_line(GTK_TABLE(table), &line, _("Requested"), str);
  
  ui_table_add_line(GTK_TABLE(table), &line, _("Priority"),
                    l_printf("%.2f", upload->priority));

  gtk_widget_show(table);
  return table;
}

static void on_limit_changed(limit_t* limit, GtkLabel* label) {
  int l = limit_get_limit(limit);
  gtk_label_set_text(label, print_speed(&l));
}

static void on_band_changed(limit_t* limit, GtkLabel* label) {
  int r = rate_get_rate(TheRate);
  gtk_label_set_text(label, print_speed(&r));
  (void)limit;
}

static void on_load_changed(volume_t* volume, GtkLabel* label) {
  file_size_t size = volume_get_sum(volume);
  gtk_label_set_text(label, print_filesize(&size));
}

GtkWidget* ui_create_upload_content(void) {
  GtkWidget* swindow;
  GtkWidget* vbox0;
  GtkWidget* hbox;
  GtkListStore* store;
  GtkWidget* tv;
  GtkWidget* da;
  GtkTreeViewColumn* column;
  GtkTreeSelection* select;
  GtkCellRenderer* rend;
  GtkWidget* item;
  rate_t* rate = TheRate = uploads_get_rate();
  volume_t* volume = TheVolume = volume_find("Upload Bandwidth Limit");

  //////////////
  vbox0 = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox0);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox0), hbox, FALSE, FALSE, 2);

  item = gtk_image_new_from_stock(STOCK_LOPSTER_UPLOAD, GTK_ICON_SIZE_SMALL_TOOLBAR);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  item = ui_create_header_label(_("Uploads"));
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  {
    GtkWidget* hbox2 = gtk_hbox_new(FALSE, 10);
    gtk_widget_show(hbox2);
    gtk_box_pack_start(GTK_BOX(hbox), hbox2, TRUE, FALSE, 5);
  
    item = gtk_label_new(_("<b>Bandwidth:</b>"));
    gtk_widget_show(item);
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

    item = gtk_label_new("");
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
    object_monitor_d(OBJ(volume), (ObjectFunc)on_band_changed, item);

    item = gtk_label_new(_("<b>Limit:</b>"));
    gtk_widget_show(item);
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

    item = gtk_label_new("");
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
    object_monitor_d(OBJ(volume), (ObjectFunc)on_limit_changed, item);
    
    item = gtk_label_new(_("<b>Uploaded:</b>"));
    gtk_widget_show(item);
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

    item = gtk_label_new("");
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
    object_monitor_d(OBJ(volume), (ObjectFunc)on_load_changed, item);

    object_set_dirty(OBJ(volume));
  }  

  swindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start(GTK_BOX(vbox0), swindow, TRUE, TRUE, 0);
  gtk_widget_show(swindow);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swindow), 
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(swindow),
                                      GTK_SHADOW_IN);

  store = gtk_list_store_new(9,
			     G_TYPE_POINTER,  // data pointer
			     G_TYPE_STRING,   // filename
			     G_TYPE_STRING,   // size
			     G_TYPE_STRING,   // status
			     G_TYPE_POINTER,  // progress
			     G_TYPE_STRING,   // rate
			     G_TYPE_STRING,   // time
			     G_TYPE_STRING,   // user
			     G_TYPE_STRING);  // plugin pix
  gtk_tree_sortable_set_sort_func
    (GTK_TREE_SORTABLE(store),
     2, (GtkTreeIterCompareFunc)sort_file_by_size, store, NULL);

  tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL (store));
  gtk_widget_show(tv);
  UploadStore = GTK_LIST_STORE(store);
  UploadView = GTK_TREE_VIEW(tv);
  gtk_container_add(GTK_CONTAINER(swindow), tv);

  gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(tv), TRUE);
  g_object_set(G_OBJECT(tv), "fixed-height-mode", TRUE, NULL);
  //  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW (tv), FALSE);
  select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
  gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
  gtk_fancytips_new(GTK_TREE_VIEW(tv), (FancyTipsFunc)on_upload_tip,
                    GTK_FANCY_POS_MOUSE);

  {
    action_item_t* actions =
      action_item_search("Upload actions");
    ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &actions, NULL);
  }

  column = ui_tv_column_new
    (_("Filename"), gtk_cell_renderer_text_new(), "text", 1, TRUE, NULL);
  ui_tv_config_default_col(column, 1, "core:upload:col:filename", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  column = ui_tv_column_new
    (_("Size"), gtk_cell_renderer_text_new(), "text", 2, TRUE, NULL);
  ui_tv_config_default_col(column, 2, "core:upload:col:size", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

/*   column = ui_tv_column_new */
/*     ("Status", gtk_cell_renderer_text_new(), "text", 3, TRUE, NULL); */
/*   ui_tv_config_default_col(column, 3, "core:upload:col:status", FALSE); */
/*   gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column); */

  column = ui_tv_column_new
    (_("Destination"),
     gtk_cell_renderer_pixbuf_new(), "stock-id", 8, FALSE,
     gtk_cell_renderer_text_new(), "text", 7, TRUE, NULL);
  ui_tv_config_default_col(column, 7, "core:upload:col:destination", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  rend = gtk_cell_renderer_upload_new();
  ui_style_changed(NULL, G_OBJECT(rend));
  ui_style_observe("transfer:bar", (StyleFunc)ui_style_changed, rend);

  column = ui_tv_column_new
    (_("Progress"), rend, "upload", 4, TRUE, NULL);
  ui_tv_config_default_col(column, 4, "core:upload:col:progress", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  column = ui_tv_column_new
    (_("Rate"), gtk_cell_renderer_text_new(), "text", 5, TRUE, NULL);
  ui_tv_config_default_col(column, 5, "core:upload:col:rate", FALSE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  column = ui_tv_column_new
    (_("ETA"), gtk_cell_renderer_text_new(), "text", 6, TRUE, NULL);
  ui_tv_config_default_col(column, 6, "core:upload:col:eta", TRUE);
  gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);

  
  da = ui_create_bandwidth_bar("core:band:upload", volume, rate);
  gtk_box_pack_start(GTK_BOX(vbox0), da, FALSE, FALSE, 0);
  ui_layout_monitor("core:upload:bandwidth:vis", da);


  item = ui_create_show_hide_label(_("show bandwidth"), _("hide bandwidth"), da);
  gtk_widget_show(item);
  gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  return vbox0;
}

static void ui_upload_hide(upload_t* upload)
{
    GtkTreeIter* iter = object_get_data(OBJ(upload), "UploadEntry");
    if (!iter) return;
    
    Uploads = g_list_remove(Uploads, upload);
    gtk_list_store_remove(UploadStore, iter);
    object_free_data_common(OBJ(upload), "UploadEntry");
}

static void ui_upload_update(upload_t* upload)
{
    GtkTreeIter* iter = object_get_data(OBJ(upload), "UploadEntry");
    file_t* file;

    if (!iter) return;
    
    file = upload->file;
    if (upload->cur_segment)
    {
        segment_t* seg = upload->cur_segment;
        int rate = socket_get_rate(SOCK(upload));
        int left = rate>0?(seg->stop-seg->start-seg->size)/rate:0;
        gtk_list_store_set(UploadStore, iter,
                           4, upload,
                           5, print_speed(&rate),
                           6, print_duration_unit(&left),
                           -1);
    }
    else
    {
        gtk_list_store_set(UploadStore, iter,
                           5, NULL,
                           6, NULL,
                           -1);
    }
}

void ui_upload_show(upload_t* upload)
{
    GtkTreeIter* iter;
    file_t* file;
    
    if (object_get_data(OBJ(upload), "UploadEntry")) return;
    
    object_set_data(OBJ(upload), "UploadEntry", iter = g_malloc(sizeof(*iter)));
    gtk_list_store_append(UploadStore, iter);
    Uploads = g_list_append(Uploads, upload);

    object_notify_d(OBJ(upload), (ObjectFunc)ui_upload_hide, NULL);
    
    file = upload->file;
    gtk_list_store_set(UploadStore, iter,
                       0, upload,
                       1, file->filename,
                       2, print_filesize(&(file->size)),
                       7, upload->user->print(upload->user),
                       8, ui_main_get_plugin_stock(upload->user->ID),
                       -1);
    ui_upload_update(upload);
}

void ui_upload_periodic_update(void)
{
    GList* dlist;
    
    for (dlist = Uploads; dlist; dlist = dlist->next)
    {
        upload_t* upload = dlist->data;
        if (upload->cur_segment) ui_upload_update(upload);
    }
}
