
/* Copyright (C) 2005-2006 sgop@users.sourceforge.net 
* This is free software distributed under the terms of the
* GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.8 $
 * $Date: 2007/05/23 20:21:53 $
 * $Author: sid4u $
 */

#ifndef _LOPSTER_BASE_H_
#define _LOPSTER_BASE_H_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "l_timer.h"
#include "module.h"
#include "win32_defs.h"

G_BEGIN_DECLS

/** \defgroup BasicFramework Basic framework of lopster */
/*@{*/
/*@}*/

/** \defgroup FileSharingFramework Filesharing with lopster */
/*@{*/
/*@}*/

/** \defgroup LopsterLibs Lopster utilities */
/*@{*/
/*@}*/

/** \defgroup LopsterBase Basic program handling
 * @ingroup BasicFramework
 */
/*@{*/

/** @name Line speed defintions */
/*@{*/
#define LINE_OTHER     0  ///< unknown or other linespeed
#define LINE_MODEM     1  ///< analog modem
#define LINE_ISDN      2  ///< Integrated Services Digital Network
#define LINE_DSL       3  ///< Digital Subscriber Line
#define LINE_CABLE     4  ///< Cable
#define LINE_SATELLITE 5  ///< Satellite
#define LINE_POWER     6  ///< Power line
#define LINE_T1        7  ///< T1 (1.544 Mbit/s)
#define LINE_T3        8  ///< T3
/*@}*/

/** @name Types of message notifications */
/*@{*/
#define LOPSTER_MESSAGE_INFO      0  ///< Information
#define LOPSTER_MESSAGE_WARNING   1  ///< Warning
#define LOPSTER_MESSAGE_QUESTION  2  ///< Question
#define LOPSTER_MESSAGE_ERROR     3  ///< Error
/** @see lopster_notify() */
/*@}*/

/** @name Application status */
/*@{*/
#define LOPSTER_STARTING      0  /**< lopster is starting up */
#define LOPSTER_RUNNING       1  /**< lopster is running */
#define LOPSTER_EXIT_SAFE     2  /**< exit lopster safely, modules may continue as long as they think is necessary */
#define LOPSTER_EXIT_SOON     3  /**< exit lopster soon, modules should only finish critical stuff */
#define LOPSTER_EXIT_NOW      4  /**< exit lopster now, modules are not allowed to finish time consuming things */
#define LOPSTER_EXIT_DEFAULT  LOPSTER_EXIT_SOON
/* @see lopster_get_mode(), lopster_termination_request(), ExitFunc */
/*@}*/


typedef gboolean (*ExitFunc)(int mode);
typedef gboolean (*CallFunc)(void* data);
typedef void (*NotifyFunc)(int type, const char* header, const char* text);

module_t* base_module_instance();

time_t BootTime();

void lopster_pixmap_dir_add(const char* dir);

const char* lopster_pixmap_find(const char* name);

void lopster_call_at_exit(void (*func)());

const char* lopster_get_home();

void lopster_notify(int type, const char* header, const char* text);

void lopster_notify_register(NotifyFunc func);

int lopster_get_mode(void);

void lopster_exit_task_register(ExitFunc func);

void lopster_exit_task_terminated(void);

void lopster_termination_request(int mode);

/*@}*/

/* Private function, do not use from any plugin */
void lopster_set_home(const char* home);
void lopster_set_argcv(int* argc, char*** argv);
int* lopster_get_argc();
char*** lopster_get_argv();


#ifndef HAVE_STRCASESTR
const char* strcasestr(const char* str, const char* sub);
#endif

G_END_DECLS

#endif
