/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _LISTEN_H_
#define _LISTEN_H_

#include "tcp_socket.h"
#include "module.h"

G_BEGIN_DECLS

/** @defgroup ListenModule TCP listen socket class
 * @ingroup SocketClasses
 */
/*@{*/

/** @name Listen socket status */
/*@{*/
#define ST_CANT_LISTEN      100   ///< failed to listen
/*@}*/

typedef struct _listen_t listen_t;

/** Accept function.
 * @param fd the file descriptor of the incoming connection.
 * @param ip the incoming ip
 * @param port the incoming port
 * @return a new tcp socket.
 */
typedef tcp_socket_t* (*ListenAcceptFunc)(int fd, ip_t ip, port_t port);
/** Port change function.
 * @param listen the listen socket
 */
typedef void (*ListenChangeFunc)(listen_t* listen);

#define LISTEN(s) ((socket_t*)(s))
/** The tcp listen class */
struct _listen_t {
  socket_t _socket;          ///< the base class
  ListenAcceptFunc accept;
  ListenChangeFunc change;
  gboolean activated;
  char* pref;
  port_t port;
  int max_con;
};

module_t* listen_module_instance();

listen_t* listen_port_new(const char* pref, const char* name,
                          ListenAcceptFunc accept, ListenChangeFunc change);
void listen_port_start_listening(listen_t* listen, int max_con);
void listen_port_stop_listening(listen_t* l);

port_t listen_port_get_port(listen_t* list);
gboolean listen_port_end(listen_t* listen, const char* message);
void listen_port_destroy(listen_t* listen);
void listen_port_change_port(listen_t* l, port_t port);

void listen_init_accepted(tcp_socket_t* socket, int fd, ip_t ip, port_t port);

/*@}*/

G_END_DECLS
#endif
