/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.3 $
 * $Date: 2007/09/30 09:20:33 $
 * $Author: sid4u $
 */

static fd_set ReadFds;
static fd_set WriteFds;
static int lastMaxFd = 0;
static int MaxFd = 0;

static void backend_set_read(socket_t* s) {
  FD_SET(s->priv->fd, &ReadFds);
}

static void backend_clear_read(socket_t* s) {
  FD_CLR(s->priv->fd, &ReadFds);
}

static void backend_set_write(socket_t* s) {
  FD_SET(s->priv->fd, &WriteFds);
}

static void backend_clear_write(socket_t* s) {
  FD_CLR(s->priv->fd, &WriteFds);
}


static void backend_add_socket(socket_t* s) {
  if (s->priv->fd > MaxFd) 
  {
      lastMaxFd = MaxFd;
      MaxFd = s->priv->fd;
  }
}

static void backend_remove_socket(socket_t* s) {
}

// TODO : handle error on select( error_fds ??)
static gboolean poll_wait_and_dispatch(int* ms) {
  fd_set read_fds;
  fd_set write_fds;
  struct timeval to;
  int ready;
  GList *dlist;

  read_fds = ReadFds;
  write_fds = WriteFds;

  to.tv_sec = *ms/1000;
  to.tv_usec = (*ms%1000)*1000;
  ready = select(MaxFd+1, &read_fds, &write_fds, 0, &to); 
  if (ready == -1) {

    g_log("SOCK", G_LOG_LEVEL_WARNING,
          "select(2) failed due to %s (%d)", g_strerror(errno), errno);
    return TRUE;
  }

  // now dispatch
  for (dlist = LSocketModule->sockets; dlist; dlist = dlist->next) {
    socket_t* s = dlist->data;
    if (s->priv->fd == -1) continue;
    
    if (FD_ISSET(s->priv->fd, &read_fds)) {  
      s->priv->in_func(s);
    }
    //FIXME: we need additional checks to catch side effects (see glib backend)
    if (FD_ISSET(s->priv->fd, &write_fds)) {
      s->priv->out_func(s);
     }
  }

  return TRUE;
}


static void backend_init() {
  static int timeout = 100;

  FD_ZERO(&ReadFds);
  FD_ZERO(&WriteFds);
  g_idle_add((GSourceFunc)poll_wait_and_dispatch, &timeout);

}


