/* Copyright (C) 2006 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2006/07/22 17:11:46 $
 * $Author: sid4u $
 */

typedef struct {
    GIOChannel* channel;
    guint reader;
    guint writer;
} socket_info_t;

static unsigned InfoAllocated = 0;
socket_info_t* Info = NULL;


static void backend_add_socket(socket_t* s)
{
    while (s->priv->fd >= (int)InfoAllocated)
    {
        unsigned num = InfoAllocated?InfoAllocated*2:16;
        Info = g_realloc(Info, sizeof(*Info)*num);
        memset(Info+InfoAllocated, 0, sizeof(*Info)*(num-InfoAllocated));
        InfoAllocated = num;
    }
    g_assert(!Info[s->priv->fd].channel);
#ifdef WIN32
    Info[s->priv->fd].channel = g_io_channel_win32_new_socket(s->priv->fd);
#else
    Info[s->priv->fd].channel = g_io_channel_unix_new(s->priv->fd);
#endif
}

static void backend_remove_socket(socket_t* s)
{
    g_assert(s->priv->fd != -1);
    g_assert(s->priv->fd < (int)InfoAllocated);
    g_assert(Info[s->priv->fd].channel);
    g_io_channel_unref(Info[s->priv->fd].channel);
    Info[s->priv->fd].channel = NULL;
}

#define LOPSTER_READ      (G_IO_IN | G_IO_HUP | G_IO_ERR)
#define LOPSTER_WRITE     (G_IO_OUT | G_IO_HUP | G_IO_ERR)

static gboolean _do_read(GIOChannel* channel, GIOCondition condition, socket_t* s)
{
    (void)channel;
    if (condition & (G_IO_HUP | G_IO_ERR | G_IO_PRI | G_IO_NVAL))
    {
        socket_done(s, ST_ERROR, "socket error");
    }
    else if (condition & G_IO_IN)
    {
        s->priv->in_func(s);
    }
    return TRUE;
}

static gboolean _do_write(GIOChannel* channel, GIOCondition condition, socket_t* s)
{
    (void)channel;
    if (condition & (G_IO_HUP | G_IO_ERR | G_IO_PRI | G_IO_NVAL))
    {
        socket_done(s, ST_ERROR, "socket error");
    }
    else if (condition & G_IO_OUT)
    {
        s->priv->out_func(s);
    }
    return TRUE;
}

static void backend_set_read(socket_t* s)
{
    g_assert(s->priv->fd != -1);
    g_assert(s->priv->fd < (int)InfoAllocated);
    g_assert(Info[s->priv->fd].channel);
    if (!Info[s->priv->fd].reader)
    {
        Info[s->priv->fd].reader =
            g_io_add_watch(Info[s->priv->fd].channel,
                           LOPSTER_READ, (GIOFunc)_do_read, s);
    }
}

static void backend_clear_read(socket_t* s)
{
    g_assert(s->priv->fd != -1);
    g_assert(s->priv->fd < (int)InfoAllocated);
    g_assert(Info[s->priv->fd].channel);
    if (Info[s->priv->fd].reader)
    {
        g_source_remove(Info[s->priv->fd].reader);
        Info[s->priv->fd].reader = 0;
    }
}

static void backend_set_write(socket_t* s)
{
    g_assert(s->priv->fd != -1);
    g_assert(s->priv->fd < (int)InfoAllocated);
    g_assert(Info[s->priv->fd].channel);
    if (!Info[s->priv->fd].writer)
    {
        Info[s->priv->fd].writer =
            g_io_add_watch(Info[s->priv->fd].channel,
                           LOPSTER_WRITE, (GIOFunc)_do_write, s);
    }
}

static void backend_clear_write(socket_t* s)
{
    g_assert(s->priv->fd != -1);
    g_assert(s->priv->fd < (int)InfoAllocated);
    g_assert(Info[s->priv->fd].channel);
    if (Info[s->priv->fd].writer)
    {
        g_source_remove(Info[s->priv->fd].writer);
        Info[s->priv->fd].writer = 0;
    }
}

static void backend_init()
{
}
