/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2006/07/22 17:11:46 $
 * $Author: sid4u $
 */

typedef struct {
  GSource source;
  GPollFD pollfd;
  int watch;
  socket_t* socket;
} poll_t;

static gboolean poll_prepare(GSource* source, gint* timeout);
static gboolean poll_check(GSource* source);
static gboolean poll_dispatch(GSource* source, GSourceFunc callback, gpointer data);
static void  poll_finalize(GSource* source);

GSourceFuncs poll_funcs = {
  poll_prepare,
  poll_check,
  poll_dispatch,
  poll_finalize,
  NULL,
  NULL
};

static poll_t** Polls = NULL;
static unsigned PollsAllocated = 0;
static socket_t* Dispatch = NULL;


static gboolean poll_prepare(GSource* source, gint* timeout) {
  (void)source;
  *timeout = -1;
  return FALSE;
}

static gboolean poll_check(GSource* source) {
  poll_t* poll = (poll_t*)source;
  return (poll->pollfd.revents & poll->pollfd.events);
}

static gboolean poll_dispatch(GSource* source, GSourceFunc callback, gpointer data)
{
    poll_t* poll = (poll_t*)source;
    gboolean res;

/*   g_assert(!LSocketModule->dispatch); */
//FIXME: we need a stack here, because g_main_loop might be called recursively

    Dispatch = poll->socket;
    res = (*callback)(source);
    Dispatch = NULL;
    
    return res;
}

static void poll_finalize(GSource* source)
{
    (void)source;
}


static poll_t* poll_find_by_socket(socket_t* s) {
  if (s->priv->fd == -1) return NULL;
  g_assert(s->priv->fd < (int)PollsAllocated);
  g_assert(Polls[s->priv->fd]->socket == s);
  return Polls[s->priv->fd];
}

static gboolean poll_invoke(poll_t* p);

static void backend_add_socket(socket_t* s) {
    while (s->priv->fd >= (int)PollsAllocated)
    {
        unsigned num = PollsAllocated?PollsAllocated*2:16;
        Polls = g_realloc(Polls, sizeof(poll_t*)*num);
        memset(Polls+PollsAllocated, 0, sizeof(poll_t*)*(num-PollsAllocated));
        PollsAllocated = num;
    }
    g_assert(!Polls[s->priv->fd]);

    {
    GSource* source = g_source_new(&poll_funcs, sizeof(poll_t));
    poll_t* p = (poll_t*)source;

    // TEST: i think the next line avoids the sources getting blocked
    // while a sub-loop is running (when a dialog is opened for example).
    g_source_set_can_recurse(source, TRUE);

    p->pollfd.fd = s->priv->fd;
    p->pollfd.events = 0;
    p->socket = s;
    
    g_source_add_poll(source, &p->pollfd);
    g_source_set_callback(source, (GSourceFunc)poll_invoke, NULL, NULL);
    
    p->watch = g_source_attach(source, NULL);
    g_source_unref(source);
    
    Polls[s->priv->fd] = p;
    }
}

static void backend_remove_socket(socket_t* s) {
  poll_t* p = poll_find_by_socket(s);
  g_assert(p);
  Polls[p->pollfd.fd] = NULL;
  g_source_remove(p->watch);
  // next line makes sure that the dispatcher does not process this
  // socket any further.
  if (Dispatch == s) Dispatch = NULL;
}

#define LOPSTER_READ      (G_IO_IN | G_IO_HUP | G_IO_ERR)
#define LOPSTER_WRITE     (G_IO_OUT | G_IO_HUP | G_IO_ERR)

static gboolean poll_invoke(poll_t* p)
{
    GIOCondition condition = (p->pollfd.revents & p->pollfd.events);
    int cond = 0;

    // this must be the currently dispatching socket.
    g_assert(Dispatch == p->socket);
    // LSocketModule->dispatch is set to NULL if the socket should be
    // destroyed within this function call, so always check this var before
    // accessing the socket.
  
    if (condition & G_IO_IN)
        cond |= SOCK_READ;
    if (condition & G_IO_OUT)
        cond |= SOCK_WRITE;
    if (condition & (G_IO_HUP | G_IO_ERR | G_IO_PRI | G_IO_NVAL))
        cond |= SOCK_ERROR;
    
    if (cond & SOCK_ERROR)
    {
        g_assert(Dispatch);
        socket_done(p->socket, ST_ERROR, "socket error");
    }
    else
    {
        if ((cond & SOCK_WRITE) && Dispatch     // writeable and valid?
            && (p->pollfd.revents & G_IO_OUT)   // still on write?
           ) {
            p->socket->priv->out_func(p->socket);
        }
        if ((cond & SOCK_READ) && Dispatch      // readable and valid?
            && (p->pollfd.revents & G_IO_IN)    // still on read?
           ) {
            p->socket->priv->in_func(p->socket);
        }
    }
    return TRUE;
}


static void backend_set_read(socket_t* s) {
  poll_t* p = poll_find_by_socket(s);
  if (p && s->priv->in_func) {
    p->pollfd.events |= LOPSTER_READ;
  }
}

static void backend_clear_read(socket_t* s) {
  poll_t* p = poll_find_by_socket(s);
  if (!p) return;
  p->pollfd.events &= ~LOPSTER_READ;
}

static void backend_set_write(socket_t* s) {
  poll_t* p = poll_find_by_socket(s);
  if (p && s->priv->in_func) {
    p->pollfd.events |= LOPSTER_WRITE;
  }
}

static void backend_clear_write(socket_t* s) {
  poll_t* p = poll_find_by_socket(s);
  if (!p) return;
  p->pollfd.events &= ~LOPSTER_WRITE;
}

static void backend_init() {
}

