/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2007/05/23 20:21:53 $
 * $Author: sid4u $
 */

#include <string.h>

#include "legacy.h"
#include "utils.h"
#include "mtypes.h"

#ifdef WIN32
#include "lcorealias.h"
#endif

typedef struct {
  char* name;
  char* value;
} legacy_pref_t;

typedef struct {
  module_t _module;
  GList* prefs;
  char** paths;
} legacy_module_t;

static legacy_module_t* LegacyModule = NULL;


static void legacy_import_prefs() {
  char rc_line[2048];
  char* filename =
    l_printf(SDIR("%s") SDIR(".lopster") "lopsterrc", g_get_home_dir());
  FILE* file;

  file = fopen(filename, "r");
  if (!file) return;

  while (mfgets(rc_line, sizeof(rc_line), file)) {
    char* pos;
    legacy_pref_t* pref;

    pos = strchr(rc_line, '=');
    if (!pos) continue;
    
    *pos = 0;
    pref = g_malloc(sizeof(*pref));
    pref->name = g_strdup(rc_line);
    pref->value = g_strdup(pos+1);
    LegacyModule->prefs = g_list_append(LegacyModule->prefs, pref);
  }
  fclose(file);
}

static void legacy_import_paths() {
  char* fname = l_printf(SDIR("%s") SDIR(".lopster") "media.list", g_get_home_dir());
  char line[2048];
  FILE* file;
  int mtype;
  char* token;
  char* name;
  char* temp;
  gboolean suffix;

  file = fopen(fname, "r");
  if (!file) return;

  LegacyModule->paths = g_malloc0(sizeof(char*) * MEDIA_ANY);

  mtype = MEDIA_INVALID;
  suffix = FALSE;
  while (mfgets(line, sizeof(line), file)) {
    token = arg(line, 0);
    if (!token || *token == '#') continue;
    if (!strcmp(token, "MEDIA")) {
      if (mtype != MEDIA_INVALID) continue;  // bad line
      name = arg(NULL, 0);
      temp = arg(NULL, 0);
      if (!temp || *temp != '{' || temp[1] != 0) continue; // bad line
      if (!strcmp(name, "mp3")) mtype = MEDIA_AUDIO;
      else mtype = media2int(name);
    } else if (!strcmp(token, "OPEN")) {
      ;
    } else if (mtype == MEDIA_INVALID) {
      continue;  // bad line
    } else if (!strcmp(token, "DOWNLOAD")) {
      if (LegacyModule->paths[mtype]) continue; //already have path.
      name = arg(NULL, 1);
      if (!name) continue;  // bad line
      LegacyModule->paths[mtype] = g_strdup(name);
    } else if (!strcmp(token, "}")) {
      if (suffix) suffix = FALSE;   // we are closing a suffix
      else mtype = MEDIA_INVALID;   // we are closing a mtype
    } else if (mtype == MEDIA_UNKNOWN || mtype == MEDIA_FOLDER) {
      continue;  // bad line
    } else if (!strcmp(token, "SUFFIX")) {
      if (suffix) continue;  // bad line
      name = arg(NULL, 0);
      temp = arg(NULL, 0);
      if (!temp || *temp != '{' || temp[1] != 0) continue;  // bad line
      suffix = TRUE;
    }
  }

  // ignore unclosed tags

  fclose(file);

}




/** Check whether the preferences from a lopster1 installation were
 * found.
 * @return TRUE if there is a lopster 1 installation, FALSE otherwise
 */
gboolean legacy_pref_available() {
  g_assert(LegacyModule);
  return (LegacyModule->prefs != NULL);
}

/** Get a setting from the lopster1 installation.
 * @param name the name of the value
 * @return the value, or NULL if not found
 */
const char* legacy_pref_get_value(const char* name) {
  GList* dlist;
  g_assert(LegacyModule);
  for (dlist = LegacyModule->prefs; dlist; dlist = dlist->next) {
    legacy_pref_t* pref = dlist->data;
    if (!strcmp(pref->name, name)) return pref->value;
  }
  return NULL;
}

/** Iterate through all lopster1 settings with a certain name.
 * @param name the name of the value
 * @param func the function to call on every matching value.
 */
void legacy_pref_foreach(const char* name, Lopster1Foreach func) {
  GList* dlist;
  g_assert(LegacyModule);
  for (dlist = LegacyModule->prefs; dlist; dlist = dlist->next) {
    legacy_pref_t* pref = dlist->data;
    if (!strcmp(pref->name, name)) func(pref->value);
  }
}


/** Check whether the download paths from a lopster1 installation were
 * found.
 * @return TRUE if there is a lopster 1 installation, FALSE otherwise
 */
gboolean legacy_paths_available() {
  g_assert(LegacyModule);
  return (LegacyModule->paths != NULL);
}

/** Get the download path for a media type.
 * @param media the media type.
 * @return the download path, or NULL if none given.
 */
const char* legacy_path_get(int media) {
  g_assert(LegacyModule);
  if (media < MEDIA_UNKNOWN || media >= MEDIA_ANY) return NULL;
  if (!LegacyModule->paths) return NULL;
  return LegacyModule->paths[media];
}


static void legacy_init() {
}

static void legacy_run() {
  legacy_import_prefs();
  legacy_import_paths();
}

static void legacy_exit() {
  GList* dlist;

  for (dlist = LegacyModule->prefs; dlist; dlist = dlist->next) {
    legacy_pref_t* pref = dlist->data;
    g_free(pref->name);
    g_free(pref->value);
    g_free(pref);
  }
  g_list_free(LegacyModule->prefs);

  if (LegacyModule->paths) {
    int i1;
    for (i1 = 0; i1 < MEDIA_ANY; i1++) g_free(LegacyModule->paths[i1]);
    g_free(LegacyModule->paths);
  }

  LegacyModule = NULL;
}

static void pref_module_init(legacy_module_t* module) {
  module_init(MODULE(module), "Legacy Module",
              (ModuleFunc)legacy_init, (ModuleFunc)legacy_run, (ModuleFunc)legacy_exit);
  
  module->prefs = NULL;
  module->paths = NULL;

  module_require(mtype_module_instance);
  //  module_require(MODULE(module), base_module_instance);
}

/** Module to access a lopster1 configuration.
 * @return the legacy module
 * @see module_require()
 */
module_t* legacy_module_instance() {
  if (!LegacyModule) {
    LegacyModule = g_malloc0(sizeof(*LegacyModule));
    pref_module_init(LegacyModule);
  }
  return MODULE(LegacyModule);
}

#ifdef WIN32
# define _LEGACY_C_
# include "lcorealiasdef.c"
#endif
