/* Copyright (C) 2005-2006 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.8 $
 * $Date: 2006/07/22 17:11:46 $
 * $Author: sid4u $
 */

#ifndef _LIB_H_
#define _LIB_H_

#include "file_tree.h"
#include "search.h"
#include "thread.h"
#include "module.h"

G_BEGIN_DECLS

/** \defgroup LibraryModule The library
 * @ingroup FileSharingFramework
 */
/*@{*/

#define LIB_DELETE    0x08
#define LIB_REMOVE    0x10
#define LIB_CHANGED   0x20
#define LIB_NEW       0x40

typedef struct {
  file_tree_t _tree;
  int mark;
  GList* shared_folder;
} library_t;
#define LIB_TREE(arg) ((library_t*)(arg))

typedef struct {
  file_node_t _node;
  time_t mod_time;
} lib_node_t;
#define LIB_NODE(arg) ((lib_node_t*)(arg))

typedef void (*LibNodeFunc)(file_t* file);
typedef void (*FileProcessFunc)(thread_t*, lib_node_t*);



/** @name Setting up a library */
/*@{*/
module_t* library_module_instance();
void library_register_actions(LibNodeFunc shared, LibNodeFunc unshared, LibNodeFunc changed);
void library_register_process_func(const char* name, FileProcessFunc func);
/*@}*/

/** @name Utilities */
/*@{*/
library_t* library_get_current(void);
file_node_t* library_foreach_shared(FileNodeTraverse func, void* data);
file_t* library_winname_requested(const char* winname);
/*@}*/

/** @name Utilities, not used by plugins */
/*@{*/
void library_add_shared(library_t* lib, const char* folder);
void library_clear_shared(library_t* lib);
void library_save_as(const char* name);
void library_delete_on_disk(const char *name);
void library_refresh(gboolean rescan);
void library_abort_refresh(void);
/*@}*/



/*@}*/

G_END_DECLS
#endif
