/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.10 $
 * $Date: 2007/11/21 20:18:24 $
 * $Author: sgop $
 */

#ifndef _LOAD_H_
#define _LOAD_H_

#include <glib.h>

#include "module.h"
#ifdef WIN32
#include "win32_defs.h"
#endif

G_BEGIN_DECLS


#define BAND_SIZE             6
#define BAND_WIDTH            2048

#ifdef WIN32
EXPORT_CORE const int LoadSteps[BAND_SIZE];
#else
extern const int LoadSteps[BAND_SIZE];
#endif
/** @defgroup LoadModule Class to keep track of the transfer load
 * @ingroup SocketUtils

 * This module is used to keep track of the complete transfer load
 * during a long period of time. This mainly is used to do statistics
 * on different classes of transfers like server/client transfers,
 * file up/downloads, etc.

 * All a plugin normally uses is the function socket_add_load() and
 * eventually load_register() to create your own load object.
 */
/*@{*/

/** Private structure */
typedef struct {
  char* name;
  double** val;
  int* pos;
  int* tno;
  gboolean in;
  time_t timestamp;
  time_t first;
} load_t;

module_t* load_module_instance();

void load_step(load_t* load);
void load_add(load_t* load, int val);
load_t* load_find(const char* name, gboolean in);
load_t* load_get(const char* name, gboolean in);
void load_register(const char* name);
GList* loads_get(void);

/* @} */

G_END_DECLS
#endif
