/* Copyright (C) 2005-2007 The Lopster Team
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.9 $
 * $Date: 2007/10/11 11:57:38 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <glib.h>

#include "plugin.h"
#include "module.h"
#include "l_base.h"
#include "ui.h"
#include "lopster.h"
#include "preferences.h"

#include "web_network.h"
#include "g_commands.h"

#ifdef WIN32
#include "lcorealias.h"
#endif

typedef struct {
  module_t _module;
} lopster_module_t;


static const char* Gui = NULL;
static const char* Home = NULL;

#if GLIB_CHECK_VERSION(2,6,0)
static GOptionEntry Options[] = {
  { "config-dir", 'c', 0, G_OPTION_ARG_STRING, &Home, "Override the standard configuration directory", "dir" },
  { "user-interface", 'i', 0, G_OPTION_ARG_STRING, &Gui, "Override the gui settings", "ui" },
  { NULL, 0, 0, 0, NULL, NULL, NULL }
};
#endif

static lopster_module_t* LopsterModule = NULL;


static void no_log(void) { }

static void lopster_init()
{
    lopster_call_at_exit(UI_leave_main_loop);
}

static void lopster_run()
{
    UI_enter_main_loop();
}

static void lopster_exit()
{
    LopsterModule = NULL;
}

static void lopster_module_init(lopster_module_t* module,
                                int* argc, char*** argv) {
  
#if GLIB_CHECK_VERSION(2,6,0)
  {
    GError* error = NULL;
    GOptionContext* context = g_option_context_new("- Lopster filesharing");
    g_option_context_set_ignore_unknown_options(context, FALSE);
    g_option_context_set_help_enabled(context, TRUE);
    
    g_option_context_add_main_entries(context, Options, NULL);
    if (!g_option_context_parse(context, argc, argv, &error)) {
      g_log("CORE", G_LOG_LEVEL_WARNING, error->message);
      g_option_context_free(context);
      exit(0);
    }
    g_option_context_free(context);
    g_log_set_default_handler((GLogFunc)std_log, NULL);
  }
#endif

  // set the values that havent been set with the command line args.
  if (Home) lopster_set_home(Home);
  else lopster_set_home(l_printf(SDIR("%s") ".2lopster", g_get_home_dir()));

  lopster_set_argcv(argc, argv);

  g_log_set_handler("OBJECT", G_LOG_LEVEL_INFO, (GLogFunc)no_log, NULL);
  //  g_log_set_handler(NULL, G_LOG_LEVEL_INFO, (GLogFunc)NULL, NULL);

  module_init(MODULE(module), "Lopster Module",
              (ModuleFunc)lopster_init,
              (ModuleFunc)lopster_run,
              (ModuleFunc)lopster_exit);

  if (Gui) ui_set_name(Gui, FALSE);
  // we of course need the base module
  module_require(base_module_instance);
  // we need a timing module
  module_require(timer_module_instance);
  // we need some kind of ui.
  module_require(ui_module_instance);
  // we like to load plugins
  module_require(plugin_module_instance);
  // enable the web interface support
  module_require(web_module_instance);
  // remove to disable global chat commands
  module_require(g_command_module_instance);

}

void lopster_module_instance(int* argc, char*** argv) {
  if (!LopsterModule) {
    LopsterModule = g_malloc0(sizeof(*LopsterModule));
    lopster_module_init(LopsterModule, argc, argv);
  }
}

#ifdef WIN32
# define _LOPSTER_C_
# include "lcorealiasdef.c"
#endif
