/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.5 $
 * $Date: 2006/07/22 17:11:47 $
 * $Author: sid4u $
 */

#ifndef _MODULE_H_
#define _MODULE_H_

#include <glib.h>

#include "object.h"

G_BEGIN_DECLS

/** @defgroup ModulesModule Module dependencies
 * In many cases you need to require a module if you like to use the
 * classes that are implemented in there. The reason for that is, that
 * the modules can be initialized and destructed in the correct order,
 * if the dependencies are known. To require a module simply use
 * module_require() when your own module is instantiated.
 *
 * @ingroup BasicFramework
 * @{
 */

typedef struct _module_priv_t module_priv_t;

/** The module class */
typedef struct {
  object_t _object;     ///< the base class
  module_priv_t* priv;  ///< private members
} module_t;

typedef void (*ModuleFunc)(module_t*);
typedef module_t* (*ModuleInstanceFunc)(void);

#define MODULE(arg) ((module_t*)(arg))


/* #define EXPORT_MODULE_H(instance)               \ */
/*   ModuleInstanceFunc instance(void); */

/* #define EXPORT_MODULE(instance)                         \ */
/*   static module_t* create_ ## instance (void);          \ */
/*                                                         \ */
/*   ModuleInstanceFunc instance(void) {                   \ */
/*     return (ModuleInstanceFunc)(create_ ## instance);   \ */
/*   }                                                     \ */
/*                                                         \ */
/*   static module_t* create_ ## instance (void) */


void module_init(module_t* module, const char* name,
                 ModuleFunc init, ModuleFunc run, ModuleFunc done);
void module_require(ModuleInstanceFunc func);
gboolean module_running(module_t* module);

/*@}*/

void module_destroy(module_t* module);
void module_require_direct(module_t* dep, ModuleFunc init);
void modules_initialize();
void modules_run();
void modules_shutdown();

G_END_DECLS
#endif
