/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.11 $
 * $Date: 2007/05/23 20:21:53 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <libxml/parser.h>

#if defined HAVE_LIBGVFS
# include "libgnomevfs/gnome-vfs.h"
# include "libgnomevfs/gnome-vfs-mime-handlers.h"
# include "libgnomevfs/gnome-vfs-mime.h"
#elif defined HAVE_XDGMIME
# define XDG_PREFIX l_mime
# include "xdgmime/xdgmime.h"
#endif

#include "l_i18n.h"
#include "xml_handler.h"
#include "l_base.h"
#include "mtypes.h"
#include "utils.h"
#include "exec.h"
#include "preferences.h"
#ifdef WIN32
# include "lcorealias.h"
#endif

static mtype_t MType[MEDIA_ANY] = {
    { NULL, NULL, 0 },
    { NULL, NULL, 1 },
    { NULL, NULL, 2 },
    { NULL, NULL, 3 },
    { NULL, NULL, 4 },
    { NULL, NULL, 5 },
    { NULL, NULL, 6 },
};

typedef struct {
    module_t _module;
} mtype_module_t;


typedef struct {
    char* command;
    char* name;
    gboolean allow_multiple;
} l_appl_t;

struct _appl_t {
#ifdef HAVE_LIBGVFS
    GnomeVFSMimeApplication* gapp;
#endif  
    l_appl_t* app;
    unsigned ref;
};

static mtype_module_t* MTypeModule = NULL;


void mtype_clear_all();


const char* app_get_command(appl_t* appl)
{
    return appl->app?appl->app->command:NULL;
}

const char* app_get_name(appl_t* appl)
{
#ifdef HAVE_LIBGVFS
    if (appl->gapp) return appl->gapp->name;
    //return gnome_vfs_mime_application_get_name(appl->gapp)
#endif
    return appl->app?appl->app->name:NULL;
}

gboolean app_get_multiple(appl_t* appl)
{
#ifdef HAVE_LIBGVFS
    if (appl->gapp) return appl->gapp->can_open_multiple_files;
#endif
    return appl->app->allow_multiple;
}

void app_set_multiple(appl_t* appl, gboolean multiple)
{
    if (appl->app)
    {
        appl->app->allow_multiple = multiple;
    }
}

appl_t* app_new(const char* command, const char* label)
{
    appl_t* app = g_malloc0(sizeof(*app));
    
    g_assert(command);
    app->app = g_malloc0(sizeof(*app->app));
    app->app->command = g_strdup(command);
    app->app->name = g_strdup(label);
    app->app->allow_multiple = TRUE;
    app->ref = 1;
    return app;
}

gboolean app_equal(appl_t* app1, appl_t* app2)
{
#ifdef HAVE_LIBGVFS
    if (app1->gapp && app2->gapp) 
        return (strcmp(app1->gapp->id, app2->gapp->id) == 0);
    //    return gnome_vfs_mime_application_equal(app1->gapp, app2->gapp);
#endif
    if (app1->app && app2->app)
        return !strcmp(app1->app->command, app2->app->command);
    
    return FALSE;
}

static void app_destroy(appl_t* app)
{
#ifdef HAVE_LIBGVFS
    if (app->gapp) gnome_vfs_mime_application_free(app->gapp);
#endif
    if (app->app)
    {
        g_free(app->app->command);
        g_free(app->app->name);
        g_free(app->app);
    }
    g_free(app);
}

void app_ref(appl_t* app)
{
    app->ref++;
}

void app_unref(appl_t* app)
{
    g_assert(app->ref);
    app->ref--;
    if (!app->ref) app_destroy(app);
}



static void mtypes_load()
{
    mtype_clear_all();
    lopster_xml_load("core:mtypes");
}

void mtype_save()
{
    lopster_xml_save("core:mtypes", 0);
}



const char* int2media(int no, int mode)
{
    switch (no) {
    case MEDIA_INVALID:     return N_("invalid");
    case MEDIA_AUDIO:       return N_("audio files");
    case MEDIA_VIDEO:       return N_("video files");
    case MEDIA_APPLICATION: return N_("applications");
    case MEDIA_IMAGE:       return N_("images");
    case MEDIA_DOCUMENT:    return N_("documents");
    case MEDIA_FOLDER:      return N_("folders");
    case MEDIA_ANY:         return N_("all files");
    default:
        if (mode == 0)      return N_("other files");
        else                return N_("unknown");
    };
}

int media2int(const char* media)
{
    if (!media) return MEDIA_INVALID;
    
    if (!strncmp(media, "folde", 5)) return MEDIA_FOLDER;
    if (!strncmp(media, "audio", 5)) return MEDIA_AUDIO;
    if (!strncmp(media, "music", 5)) return MEDIA_AUDIO;
    if (!strncmp(media, "video", 5)) return MEDIA_VIDEO;
    if (!strncmp(media, "appli", 5)) return MEDIA_APPLICATION;
    if (!strncmp(media, "image", 5)) return MEDIA_IMAGE;
    if (!strncmp(media, "docum", 5)) return MEDIA_DOCUMENT;
    if (!strncmp(media, "text", 4)) return MEDIA_DOCUMENT;
    if (!strncmp(media, "all", 3)) return MEDIA_ANY;
    return MEDIA_UNKNOWN;
}

mtype_t* mtype_find(int no)
{
    if (no < 0 || no >= MEDIA_ANY) return 0;
    return &(MType[no]);
}

static char* xml_get_text(xmlDocPtr xfile, xmlNodePtr xnode)
{
    char* result;
    xmlChar* attr;
    
    attr = xmlNodeListGetString(xfile, xnode->xmlChildrenNode, 1);
    result = g_strdup(attr);
    xmlFree(attr);
    return result;
}

static appl_t* xml_get_application(xmlDocPtr xfile, xmlNodePtr xnode)
{
    xmlChar* attr1 = xmlGetProp(xnode, "Description");
    xmlChar* attr2 = xmlGetProp(xnode, "AllowMultiple");
    xmlChar* attr3 = xmlNodeListGetString(xfile, xnode->xmlChildrenNode, 1);
    appl_t* result = NULL;
    
    if (attr3)
    {
        result = app_new(attr3, attr1);
        if (attr2) app_set_multiple(result, !strcasecmp(attr2, "yes"));
        else app_set_multiple(result, FALSE);
    }
    if (attr1) xmlFree(attr1);
    if (attr2) xmlFree(attr2);
    if (attr3) xmlFree(attr3);
    
    return result;
}

static void xml_get_media(xmlDocPtr xfile, xmlNodePtr xnode)
{
    xmlNodePtr xsub;
    mtype_t* mtype;
    char* text;
    appl_t* appl;
    xmlChar* attr;

    attr = xmlGetProp(xnode, "Name");
    if (!attr)
    {
        g_log("MTYPE", G_LOG_LEVEL_WARNING,
              "media type does not have a name");
        return;
    }
    mtype = &MType[media2int(attr)];
    xmlFree(attr);

    xsub = xnode->xmlChildrenNode;
    while (xsub != NULL)
    {
        if (!strcmp(xsub->name, "Download"))
        {
            text = xml_get_text(xfile, xsub);
            if (mtype->download)
            {
                g_log("MTYPE", G_LOG_LEVEL_WARNING, 
                      "duplicate 'Download' node in type '%s'",
                      int2media(mtype->type, -1));
                g_free(text);
            }
            else
            {
                mtype->download = text;
            }
        }
        else if (!strcmp(xsub->name, "Application"))
        {
            appl = xml_get_application(xfile, xsub);
            if (appl) mtype->app = g_list_append(mtype->app, appl);
        }
        xsub = xsub->next;
    }
}

static void xml_load_media(xmlDocPtr xfile, xmlNodePtr xroot)
{
    xmlNodePtr xsub = xroot->xmlChildrenNode;
    while (xsub != NULL)
    {
        if (!strcmp(xsub->name, "Media"))
        {
            xml_get_media(xfile, xsub);
        }
        xsub = xsub->next;
    }
}

static void xml_save_app(xmlNodePtr xnode, appl_t* app)
{
    xmlNodePtr xsub;
    
    xsub = xmlNewTextChild(xnode, NULL, "Application", app_get_command(app));
    if (app_get_name(app)) xmlNewProp(xsub, "Description", app_get_name(app));
    xmlNewProp(xsub, "AllowMultiple", app_get_multiple(app)?"yes":"no");
}

static void xml_save_media(xmlNodePtr xnode, mtype_t* mtype)
{
    xmlNodePtr xsub;
    GList* dlist;
    
    xsub = xmlNewChild(xnode, NULL, "Media", NULL);
    xmlNewProp(xsub, "Name", int2media(mtype->type, 2));
    if (mtype->download)
        xmlNewTextChild(xsub, NULL, "Download", mtype->download);
    
    for (dlist = mtype->app; dlist; dlist = dlist->next)
    {
        appl_t* app = dlist->data;
        xml_save_app(xsub, app);
    }
}

static void xml_save_medias(xmlDocPtr xfile, xmlNodePtr xroot)
{
    int i1;
    (void)xfile;

    for (i1 = 0; i1 < MEDIA_ANY; i1++)
    {
        mtype_t* mtype = &MType[i1];
        xml_save_media(xroot, mtype);
    }
}

char* mtype_get_mime_type(const char* name, int* media)
{
    char* mime = NULL;
    
    if (media) *media = MEDIA_UNKNOWN;
    if (!name) return NULL;
    
#if defined HAVE_LIBGVFS
/*   mime = gnome_vfs_get_mime_type(name); */
    mime = g_strdup(gnome_vfs_mime_type_from_name(name));
#elif defined HAVE_XDGMIME
    {
        const char* temp = l_mime_get_mime_type_for_file(name);
        if (temp != l_mime_type_unknown) mime = g_strdup(temp);
    }
#endif
    
    if (mime)
    {
        char* sep = strchr(mime, '/');
        
        if (sep && media)
        {
            *sep = 0;
            *media = media2int(mime);
            *sep = '/';
        }
    }
    
    return mime;
}

const char* mtype_get_folder(int type)
{
    const char* res;
    
    if (type < 0 || type >= MEDIA_ANY) res = NULL;
    else res = MType[type].download;
    if (!res || *res == 0) res = MType[MEDIA_UNKNOWN].download;
    if (!res || *res == 0) res = g_get_home_dir();
    return res;
}

void mtype_set_folder(int type, const char* path)
{
    if (type < 0 || type >= MEDIA_ANY) return;
    g_free(MType[type].download);
    if (path && *path) MType[type].download = g_strdup(path);
    else MType[type].download = NULL;
}

void mtype_free(mtype_t* mtype)
{
    g_assert(mtype);
    
    g_free(mtype->download);
	 
    g_list_foreach(mtype->app, (GFunc)app_unref, NULL);
    g_list_free(mtype->app);
    
    mtype->download = NULL;
    mtype->app = NULL;
}

void mtype_clear_all()
{
    int i1;
    for (i1 = 0; i1 < MEDIA_ANY; i1++) mtype_free(&MType[i1]);
}



#if defined HAVE_LIBGVFS

typedef struct {
    char* mime;
    GList* apps;
    time_t stamp;
} mime_app_t;

static GHashTable* CachedMimeApps = NULL;

static void _mime_apps_destroy(mime_app_t* mapp)
{
    g_free(mapp->mime);
    g_list_foreach(mapp->apps, (GFunc)app_unref, NULL);
    g_list_free(mapp->apps);
    g_free(mapp);
}

static mime_app_t* _mime_apps_new(const char* mime)
{
    mime_app_t* mapp = g_malloc0(sizeof(*mapp));
    mapp->mime = g_strdup(mime);
    mapp->apps = NULL;
    mapp->stamp = 0;
    return mapp;
}

static void _mime_apps_refresh(mime_app_t* mapp)
{
    GList* apps = gnome_vfs_mime_get_all_applications(mapp->mime);
    GList* dlist;
    
    g_list_foreach(mapp->apps, (GFunc)app_unref, NULL);
    g_list_free(mapp->apps);
    mapp->apps = NULL;
    
    for (dlist = apps; dlist; dlist = dlist->next)
    {
        GnomeVFSMimeApplication* gapp = dlist->data;
        appl_t* app = g_malloc0(sizeof(*app));
        app->gapp = gapp;
        mapp->apps = g_list_append(mapp->apps, app);
        app_ref(app);
    }
    mapp->stamp = CurrentTime;
}

static GList* _mime_apps_get(const char* mime)
{
    mime_app_t* mapp = NULL;
    
    if (!mime) return NULL;
    
    if (!CachedMimeApps)
    {
        CachedMimeApps =
            g_hash_table_new_full
            ((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal,
             (GDestroyNotify)NULL, (GDestroyNotify)_mime_apps_destroy);
    }
    else
    {
        mapp = g_hash_table_lookup(CachedMimeApps, mime);
    }
  
    if (!mapp)
    {
        mapp = _mime_apps_new(mime);
        g_hash_table_insert(CachedMimeApps, mapp->mime, mapp);
    }
  
    if (CurrentTime - mapp->stamp > 60) _mime_apps_refresh(mapp);
    
    return mapp->apps;
}

#endif


/** Get lopster's internal applications and the applications defined by the
 * desktop (if desktop is supported)
 */
GList* mtype_get_applications(char* mime, int type, GList** desktop_apps)
{
    *desktop_apps = NULL;
    if (type < 0 || type >= MEDIA_ANY) return NULL;
    
#if defined HAVE_LIBGVFS
    *desktop_apps = _mime_apps_get(mime);
#endif
    
    return MType[type].app;
}

static void _mtype_start_error(appl_t* app, unsigned num)
{
    lopster_notify(LOPSTER_MESSAGE_ERROR, "Error",
                   l_printf(_("Failed to open %u items with program <i>%s</i>"),
                            num, app_get_name(app)));
}

static void on_open_end(exec_t* exec, void* data)
{
    if (exec->status != EXEC_SUCCESS)
    {
        int num = GPOINTER_TO_INT(data);
        lopster_notify(LOPSTER_MESSAGE_ERROR, "Error",
                       l_printf(_("Failed to open %u items"), num));
    }
}

void mtype_run_application(appl_t* app, GList* items)
{
    
#if defined HAVE_LIBGVFS
    if (app->gapp)
    {
        GList* dlist;
        GList* copy = NULL;
        // older gnome-vfs crash if we don't pass uris
        for (dlist = items; dlist; dlist = dlist->next)
        {
            char* item = dlist->data;
            copy = g_list_append(copy, g_strdup_printf("file://%s", item));
        }
        if (gnome_vfs_mime_application_launch(app->gapp, copy) != GNOME_VFS_OK)
            _mtype_start_error(app, g_list_length(items));
        g_list_foreach(copy, (GFunc)g_free, NULL);
        g_list_free(copy);
        return;
    }
#endif

    if (app->app)
    {
        char** argv;
        char** argv2;
        int cnt, cnt2, replaces;
        
        if (!g_shell_parse_argv(app->app->command, NULL, &argv, NULL))
        {
            g_log("MTYPE", G_LOG_LEVEL_WARNING, "could not parse command");
            _mtype_start_error(app, g_list_length(items));
            return;
        }
        
        // count the arguments
        for (cnt = 0; argv[cnt] != 0; cnt++);
        argv2 = g_malloc(sizeof(*argv2)*(cnt+g_list_length(items)+1));
        
        // copy the arguments and replaces %f with folder/filename
        replaces = 0;
        cnt2 = 0;
        for (cnt = 0; argv[cnt] != 0; cnt++)
        {
            if (!strcmp("%f", argv[cnt]) || !strcmp("\"%f\"", argv[cnt]))
            {
                if (items)
                {
                    char* item = items->data;
                    argv2[cnt2++] = g_strdup(item);
                    items = items->next;
                    replaces++;
                }
            }
            else if (!strcmp("%F", argv[cnt]) || !strcmp("\"%F\"", argv[cnt]))
            {
                while (items)
                {
                    char* item = items->data;
                    argv2[cnt2++] = g_strdup(item);
                    items = items->next;
                    replaces++;
                }
            }
            else
            {
                argv2[cnt2++] = g_strdup(argv[cnt]);
            }
        }
        
        // if there was no replacement, append all items
        if (!replaces)
        {
            while (items)
            {
                char* item = items->data;
                argv2[cnt2++] = g_strdup(item);
                items = items->next;
                replaces++;
            }
        }
        argv2[cnt2++] = NULL;
        g_strfreev(argv);
        
        exec_start(exec_new(argv2), (ExecEndFunc)on_open_end, GINT_TO_POINTER(replaces));
        return;
    }
    
    g_assert(FALSE);
}

void mtype_browse(const char* address)
{
    const char* browser = pref_get_ptr("/core/desktop/browser", char);
    GList* items = NULL;
    appl_t* app;
    
    if (browser && *browser)
    {
        app = app_new(browser, "Browser");
        if (!strncmp(address, "http://", 7))
            items = g_list_append(items, g_strdup(address));
        else
            items = g_list_append(items, g_strdup_printf("http://%s", address));
        mtype_run_application(app, items);
        g_list_foreach(items, (GFunc)g_free, NULL);
        g_list_free(items);
        app_unref(app);
    }
    else
    {
        lopster_notify(LOPSTER_MESSAGE_WARNING, _("Unable to open URL"),
                       _("Please setup a browser in your preferences."));
    }
}

void mtype_mail(const char* address)
{
    const char* email = pref_get_ptr("/core/desktop/email", char);
    GList* items = NULL;
    appl_t* app;
    
    if (email && *email)
    {
        app = app_new(email, "EMail");
        
        if (!strncmp(address, "mailto:", 7))
            items = g_list_append(items, g_strdup(address));
        else
            items = g_list_append(items, g_strdup_printf("mailto:%s", address));
        mtype_run_application(app, items);
        g_list_foreach(items, (GFunc)g_free, NULL);
        g_list_free(items);
        app_unref(app);
    }
    else
    {
        lopster_notify(LOPSTER_MESSAGE_WARNING, _("Unable open mail client"),
                       _("Please specify an E-Mail client in your preferences."));
    }
}


static void mtypes_init()
{
#if defined HAVE_LIBGVFS
    gnome_vfs_init();
#endif
    lopster_xml_register_home("core:mtypes", "media.xml",
                              "MediaTypes", xml_load_media, xml_save_medias);
    mtypes_load();
}

static void mtypes_run()
{
}

static void mtypes_exit()
{
    mtype_clear_all();
}

static void mtype_module_init(mtype_module_t* module)
{
    module_init(MODULE(module), "MType Module",
                (ModuleFunc)mtypes_init, (ModuleFunc)mtypes_run,
                (ModuleFunc)mtypes_exit);
    module_require(xml_module_instance);
    module_require(base_module_instance);
}


module_t* mtype_module_instance()
{
    if (!MTypeModule)
    {
        MTypeModule = g_malloc0(sizeof(*MTypeModule));
        mtype_module_init(MTypeModule);
    }
    return MODULE(MTypeModule);
}

#ifdef WIN32
# define _MTYPES_C_
# include "lcorealiasdef.c"
#endif
