
/* Copyright (C) 2005-2006 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.9 $
 * $Date: 2006/09/26 16:41:38 $
 * $Author: sgop $
 */

#ifndef _OBJECT_H_
#define _OBJECT_H_

#include <glib.h>
#include "utils.h"

G_BEGIN_DECLS

#define OBJ(obj) ((object_t*)(obj))

/** \defgroup ObjectModule The base object class
 * @ingroup BasicFramework
 * @{
 */

typedef struct _object_priv_t object_priv_t;
typedef struct _object_t object_t;


/** The function used for callbacks.
 * @see object_notify(), object_monitor()
 */
typedef void (*ObjectFunc)(object_t*, gpointer);

/** The object class.*/
struct _object_t {
    char* name;           ///< the name of the object (allocated string)
    ObjectFunc destroy;   ///< the destroy function (see object_kill())
    object_priv_t* priv;  ///< private fields
};


void object_init          (object_t* object, const char* name, gboolean sync);
void object_destroy       (object_t* object);
void object_copy          (object_t* dest, object_t* src);
void object_set_sync      (object_t* object, gboolean sync);

void object_notify_d      (object_t* object, ObjectFunc func, gpointer data);
void object_unnotify_d    (object_t* object, ObjectFunc func, gpointer data);
void object_notify_o      (object_t* object, ObjectFunc func, object_t* observer);
void object_unnotify_o    (object_t* object, ObjectFunc func, object_t* observer);

void object_monitor_d     (object_t* object, ObjectFunc func, gpointer data);
void object_unmonitor_d   (object_t* object, ObjectFunc func, gpointer data);
void object_monitor_o     (object_t* object, ObjectFunc func, object_t* observer);
void object_unmonitor_o   (object_t* object, ObjectFunc func, object_t* observer);

gboolean object_monitor_find(object_t* object, ObjectFunc func, gpointer data);

void* object_get_data     (object_t* object, const char* name);
void object_set_data      (object_t* object, const char* name, void* data);
void object_free_data     (object_t* object, const char* name, FreeFunc func);
void object_free_data_common(object_t* object, const char* name);
void object_set_dirty     (object_t* object);
void object_kill          (object_t* object);
void object_kill_full     (object_t* object, void* data);
void object_set_name      (object_t* object, const char* name);

void object_set_ptr(object_t** object);
void object_unset_ptr(object_t** object);

void object_add_type(object_t* object, const char* type);
const char* object_get_type(object_t* object);
void object_make_public(object_t* object, void* data);
void object_register_instance(const char* type, ObjectFunc func, gboolean inherited);

/*@}*/

G_END_DECLS

#endif
