/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.7 $
 * $Date: 2006/07/22 17:11:47 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef HAVE_OGG

#include <sys/types.h>
#include <string.h>
#include <stdio.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include <fcntl.h>
#include <sys/stat.h>
#include <ogg/ogg.h>

#include "library.h"
#include "files.h"
#include "l_io.h"

#ifdef WIN32
#include "win32_defs.h"
#endif

#define CHUNKSIZE 4096

static int read32bits(ogg_packet *op, int off) {
  unsigned char *buf = op->packet;
  return (buf[off])|(buf[off+1]<<8)|(buf[off+2]<<16)|(buf[off+3]<<24);
}

/* Try to find the final page. Can be slightly off (but not enough to cause
 * problems) for non-standard page lengths */
static int get_final_page(FILE* fd, ogg_sync_state *oy, int len, ogg_page *og)
{
    int size = 0;
    unsigned char *buffer;
    int found=0;
    int seekp;
    int tlen=len;
    
    while(1)
    {
        len -= CHUNKSIZE;
        size += CHUNKSIZE;
        if (FSEEK(fd, len, SEEK_SET) != 0) return -1;
        
        ogg_sync_reset(oy);
        buffer = ogg_sync_buffer(oy, size);

        if (fread(buffer, size, 1, fd) != 1) return -1;
        
        ogg_sync_wrote(oy, size);
        found = ogg_sync_pageseek(oy, og);
        
        seekp = len;
        while (found && seekp <= len+CHUNKSIZE && seekp-found < tlen)
        {
            seekp -= found;
            FSEEK(fd, seekp, SEEK_SET); /* Seek to where we were told it is. */
            
            ogg_sync_reset(oy);
            buffer = ogg_sync_buffer(oy, size);
            if (fread(buffer, size, 1, fd) != 1) return -1;

            ogg_sync_wrote(oy, size);
            
            found = ogg_sync_pageseek(oy, og);
            if (found > 0) return 0;
        }
    }
    
    return -1;
}

// shamelessly stolen from gnapster (Good work, Jasta) by the_turner
void file_parse_ogg(thread_t* thread, lib_node_t* node)
{
    FILE* fd;
    file_t* file;
    ogg_sync_state oy;
    ogg_stream_state os;
    ogg_page og;
    ogg_packet op;
  
    unsigned char *buffer;
    int serial;
    int streaminit=0;
    int length;
    int bitrate;
    int frequency;
    int duration;
    char* temp;
    attribute_t* attr;
  
    file = FILE_NODE(node)->file;
    temp = get_suffix(file->filename);
    if (!temp || strcasecmp(temp, "ogg")) return;

    attr = attr_find(ATTRC(file), "file", "Bitrate");
    if (attr && attr->refreshed >= node->mod_time) return;

    ogg_sync_init(&oy);
    buffer = ogg_sync_buffer(&oy, CHUNKSIZE);
  
    temp = auto_text(locale_from_utf8(file->localname));
    if (!temp) return;

    fd = fopen(temp, "rb");
    if (!fd) return;

    FSEEK(fd, 0, SEEK_END);
    file->size = FTELL(fd);
    FSEEK(fd, 0, SEEK_SET);

    if (fread(buffer, CHUNKSIZE, 1, fd) != 1) goto failure;
    ogg_sync_wrote(&oy, CHUNKSIZE);
  
    if (ogg_sync_pageout(&oy, &og) != 1)
    {
        fprintf(stderr, "WARNING: Could not do ogg_sync_pageout ");
        goto failure;
    }
  
    serial = ogg_page_serialno(&og);
    ogg_stream_init(&os, serial);
    streaminit = 1;
    if (ogg_stream_pagein(&os, &og) < 0) goto failure;
    if (ogg_stream_packetout(&os, &op) != 1) goto failure;
    if (read32bits(&op,7) != 0) goto failure;
  
    frequency = read32bits(&op, 12);
  
    ogg_stream_clear(&os); /* We're done with that, now. */
    streaminit=0;
  
    if (get_final_page(fd, &oy, file->size, &og) != 0) goto failure;
  
    length = ogg_page_granulepos(&og);
  
    if (frequency <= 0) goto failure;
    duration = (time_t)(length/frequency);
    if (duration <= 0) goto failure;
    bitrate = (int)(file->size/duration*8/1000);
  
    ogg_sync_clear(&oy);

    // lock thread as we do changes in the library
    thread_lock(thread);
    attr_store(ATTRC(file), "file", "Bitrate", bitrate);
    attr_store(ATTRC(file), "file", "Frequency", frequency);
    attr_store(ATTRC(file), "file", "Duration", duration);
    thread_unlock(thread);

    goto close;
  
  failure:
    if (streaminit) ogg_stream_clear(&os);
    ogg_sync_clear(&oy);
  close:
    fclose(fd);
}

#endif /* HAVE_OGG */
