/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2007/11/21 20:18:24 $
 * $Author: sgop $
 */

#include <string.h>

#include <glib.h>

#include "open_actions.h"
#include "file_tree.h"

static void on_open_file(GList* files, appl_t* appl) {
  GList* Items = NULL;
  GList* dlist;
  
  for (dlist = files; dlist; dlist = dlist->next) {
    file_t* file = dlist->data;
    Items = g_list_append(Items, file->localname);
  }
  mtype_run_application(appl, Items);
  g_list_free(Items);
}

static gboolean on_check_file_open(file_t* file, appl_t* appl) {
  GList* lapps;
  GList* gapps;
  GList* dlist;

  lapps = mtype_get_applications(file->mimetype, file->media, &gapps);

  // we need to use app_equal() because mtype_get_application()
  // might give us freshly allocated applications
  for (dlist = lapps; dlist; dlist = dlist->next) {
    appl_t* a = dlist->data;
    if (app_equal(appl, a)) return TRUE;
  }
  for (dlist = gapps; dlist; dlist = dlist->next) {
    appl_t* a = dlist->data;
    if (app_equal(appl, a)) return TRUE;
  }
  return FALSE;
}

static action_item_t* on_file_open_menu(GList* selection) {
  GList* dlist;
  static GList* Lapps = NULL;
  static GList* Gapps = NULL;
  static action_item_t* Result = NULL;

  if (Result) {
    action_item_destroy(Result);
    Result = NULL;
  }
  g_list_foreach(Lapps, (GFunc)app_unref, NULL);
  g_list_free(Lapps);
  Lapps = NULL;
  g_list_foreach(Gapps, (GFunc)app_unref, NULL);
  g_list_free(Gapps);
  Gapps = NULL;

  Result = action_item_root("File open actions");

  // FIXME: speedup, dont need to get apps for every item, just for every mimetype
  for (dlist = selection; dlist; dlist = dlist->next) {
    file_t* file = dlist->data;
    GList* lapps;
    GList* gapps;
    GList* it;

    lapps = mtype_get_applications(file->mimetype, file->media, &gapps);

    for (it = lapps; it; it = it->next) {
      appl_t* gapp = it->data;
      GList* dlist2;
      for (dlist2 = Lapps; dlist2; dlist2 = dlist2->next) {
        appl_t* appl = dlist2->data;
        if (appl == gapp) break;
      }
      if (!dlist2) Lapps = g_list_append(Lapps, gapp);
    }
    for (it = gapps; it; it = it->next) {
      appl_t* gapp = it->data;
      GList* dlist2;
      for (dlist2 = Gapps; dlist2; dlist2 = dlist2->next) {
        appl_t* appl = dlist2->data;
        if (appl == gapp) break;
      }
      if (!dlist2) Gapps = g_list_append(Gapps, gapp);
    }
  }
  for (dlist = Lapps; dlist; dlist = dlist->next) {
    appl_t* appl = dlist->data;
    action_item_t* item =
      action_item_new(app_get_name(appl), Result,
                      app_get_multiple(appl)?AI_LOCAL_DATA:(AI_LOCAL_DATA|AI_RESTRICT_1),
                      (ActionItemFunc)on_open_file, (ActionItemCheck)on_check_file_open);
    item->user_data = appl;
    app_ref(appl);
  }
  if (Lapps && Gapps) action_item_sep(Result);
  for (dlist = Gapps; dlist; dlist = dlist->next) {
    appl_t* appl = dlist->data;
    action_item_t* item =
      action_item_new(app_get_name(appl), Result,
                      app_get_multiple(appl)?AI_LOCAL_DATA:(AI_LOCAL_DATA|AI_RESTRICT_1),
                      (ActionItemFunc)on_open_file, (ActionItemCheck)on_check_file_open);
    item->user_data = appl;
    app_ref(appl);
  }

  return Result;
}

static void on_open_folder(GList* folders, appl_t* appl) {
  static GList* Items = NULL;
  GList* dlist;

  if (Items) {
    g_list_foreach(Items, (GFunc)g_free, NULL);
    g_list_free(Items);
    Items = NULL;
  }
  for (dlist = folders; dlist; dlist = dlist->next) {
    file_node_t* node = dlist->data;
    //    char* folder = from_utf8(file_node_get_folder(node, NULL));
    char* folder = file_node_get_folder(node, NULL);
    Items = g_list_append(Items, g_strdup(folder));
  }
  mtype_run_application(appl, Items);
}

static gboolean on_check_folder_open(file_node_t* node, appl_t* appl) {
  GList* lapps;
  GList* gapps;
  GList* dlist;
  gboolean res = FALSE;

  (void)node;

  lapps = mtype_get_applications("inode/directory", MEDIA_FOLDER, &gapps);

  for (dlist = lapps; dlist; dlist = dlist->next) {
    appl_t* a = dlist->data;
    if (app_equal(appl, a)) return TRUE;
  }
  for (dlist = gapps; dlist; dlist = dlist->next) {
    appl_t* a = dlist->data;
    if (app_equal(appl, a)) return TRUE;
  }

  return res;
}

static action_item_t* on_folder_open_menu(GList* selection) {
  GList* dlist;
  GList* lapps = NULL;
  GList* gapps = NULL;
  static GList* Lapps = NULL;
  static GList* Gapps = NULL;
  static action_item_t* Result = NULL;

  (void)selection;
  if (Result) {
    action_item_destroy(Result);
    Result = NULL;
  }
  g_list_foreach(Lapps, (GFunc)app_unref, NULL);
  g_list_free(Lapps);
  Lapps = NULL;
  g_list_foreach(Gapps, (GFunc)app_unref, NULL);
  g_list_free(Gapps);
  Gapps = NULL;
  
  Result = action_item_root("Folder open actions");

  lapps = mtype_get_applications(NULL, MEDIA_FOLDER, &gapps);

  for (dlist = lapps; dlist; dlist = dlist->next) {
    appl_t* appl = dlist->data;
    action_item_t* item =
      action_item_new(app_get_name(appl), Result,
                      app_get_multiple(appl)?AI_LOCAL_DATA:(AI_LOCAL_DATA|AI_RESTRICT_1),
                      (ActionItemFunc)on_open_folder,
                      (ActionItemCheck)on_check_folder_open);
    item->user_data = appl;
    Lapps = g_list_append(Lapps, appl);
    app_ref(appl);
  }
  if (lapps && gapps) action_item_sep(Result);
  for (dlist = gapps; dlist; dlist = dlist->next) {
    appl_t* appl = dlist->data;
    action_item_t* item =
      action_item_new(app_get_name(appl), Result,
                      app_get_multiple(appl)?AI_LOCAL_DATA:(AI_LOCAL_DATA|AI_RESTRICT_1),
                      (ActionItemFunc)on_open_folder,
                      (ActionItemCheck)on_check_folder_open);
    item->user_data = appl;
    Gapps = g_list_append(Gapps, appl);
    app_ref(appl);
  }

  return Result;
}

action_item_t*
action_item_open_file(const char* name, action_item_t* parent,
                      ActionObjectConvert convert)
{
    action_item_t* result = action_item_sub
        (name, parent, (ActionItemLink)on_file_open_menu, FALSE);
    result->convert = convert;
    return result;
}

action_item_t*
action_item_open_folder(const char* name, action_item_t* parent,
                        ActionObjectConvert convert)
{
    action_item_t* result = action_item_sub
        (name, parent, (ActionItemLink)on_folder_open_menu, FALSE);
    result->convert = convert;
    return result;
}
