/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#include <string.h>
#include <stdlib.h>

#include "ports.h"
#include "preferences.h"

typedef struct {
  module_t _module;
  GSList* allowed;
} port_module_t;

static port_module_t* PortModule = NULL;

static void ports_convert(GSList* list) {
  GSList* dlist;
  char* sep;
  port_t p1, p2, temp;
  port_t* res;

  for (dlist = list; dlist; dlist = dlist->next) {
    char* ports = dlist->data;
    if (!ports) continue;
    sep = strchr(ports, '-');
    if (sep) {
      *sep = 0;
      p1 = atoi(ports);
      p2 = atoi(sep + 1);
      if (p1 > p2) {
	temp = p1;
	p1 = p2;
	p2 = temp;
      }
    } else {
      p1 = p2 = atoi(ports);
    }
    g_free(ports);
    
    dlist->data = g_malloc(sizeof(port_t)*2);
    res = dlist->data;
    res[0] = p1;
    res[1] = p2;
  }
}

static int ports_compare_func(gconstpointer a, gconstpointer b) {
  const port_t* p1 = (port_t*)a;
  const port_t* p2 = (port_t*)b;

  if (!p2) return (p1 != NULL);
  if (!p1) return -1;

  if (p1[0] < p2[0]) return -1;
  if (p1[0] > p2[0]) return 1;
  return 0;
}

static GSList* ports_simplify(GSList** list) {
  GSList* dlist;
  port_t* res;
  port_t* res2 = NULL;

  *list = g_slist_sort(*list, (GCompareFunc)ports_compare_func);

  if (!(*list)) return NULL;
  dlist = *list;
  res = dlist->data;
  dlist = dlist->next;
  while (dlist) {
    res2 = dlist->data;
    dlist = dlist->next;
    if (res[1] + 1 >= res2[0]) {
      if (res2[1] > res[1]) res[1] = res2[1];
      *list = g_slist_remove(*list, res2);
      g_free(res2);
    } else {
      res = res2;
    }
  }
  return *list;
}

static void ports_setup(const char *string) {
  GSList* temp_list = NULL;
  char* temp;

  if (PortModule->allowed) {
    g_slist_foreach(PortModule->allowed, (GFunc)g_free, NULL);
    g_slist_free(PortModule->allowed);
    PortModule->allowed = NULL;
  }
  if (!string || !(*string)) return;

  temp = g_strdup(string);
  make_list_from_string(&temp_list, temp, ",");
  g_free(temp);
  ports_convert(temp_list);
  ports_simplify(&temp_list);

  PortModule->allowed = temp_list;
  for (temp_list = PortModule->allowed; temp_list; temp_list = temp_list->next) {
    port_t* res = temp_list->data;
    g_log("PORT", G_LOG_LEVEL_DEBUG,
	  "added allowed port range %5d - %5d", res[0], res[1]);
  }
}

gboolean port_is_allowed(port_t port) {
  GSList *dlist;
  g_assert(PortModule);
  for (dlist = PortModule->allowed; dlist; dlist = dlist->next) {
    port_t* res = dlist->data;
    if (port >= res[0] && port <= res[1]) return TRUE;
  }
  return FALSE;
}

port_t port_get_next(port_t last) {
  GSList *dlist;
  port_t* res;
  port_t new_port = last + 1;

  g_assert(PortModule);
  if (!PortModule->allowed) return 0;
  for (dlist = PortModule->allowed; dlist; dlist = dlist->next) {
    res = dlist->data;
    if (res[1] < new_port) continue;
    break;
  }
  if (!dlist) {			// last port too high, getting first allowed port
    res = PortModule->allowed->data;
    return res[0];
  }
  // ok, segment found that may includes last+1, check min value
  // new_port <= res[1]
  if (new_port < res[0]) return res[0];
  else return new_port;
}



static void ports_init() {
}
static void ports_run() {
  ports_setup(pref_get_ptr("/core/listen/range", char));
}
static void ports_exit() {
  g_slist_foreach(PortModule->allowed, (GFunc)g_free, NULL);
  g_slist_free(PortModule->allowed);
  PortModule = NULL;
}

static void port_module_init(port_module_t* module) {
  module_init(MODULE(module), "Port Module",
	      (ModuleFunc)ports_init, (ModuleFunc)ports_run, (ModuleFunc)ports_exit);

  module->allowed = NULL;
  
  module_require(pref_module_instance);
}


module_t* port_module_instance() {
  if (!PortModule) {
    PortModule = g_malloc0(sizeof(*PortModule));
    port_module_init(PortModule);
  }
  return MODULE(PortModule);
}
