/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.9 $
 * $Date: 2007/08/24 14:15:06 $
 * $Author: sid4u $
 */

#ifndef _PREF_H_
#define _PREF_H_

#include "attribute.h"
#include "object.h"
#include "module.h"

#include "win32_defs.h"
G_BEGIN_DECLS

/** @defgroup PrefModuleValues Lopster settings defined by the core application
 * @ingroup PrefModule
 * @{
 * @}
 */

/** @defgroup PrefModule Lopster preferences
 * Preferences values are separated by a slash '/' and must
 * start with one. To add your own values you have to use
 * pref_register() or one of its predefined wrappers.
 * @ingroup BasicFramework
 */
/*@{*/


typedef struct _pref_node_t pref_node_t;
typedef struct _constraint_t constraint_t;

/** Function which is called for every constraint when a value changes.
 * @param node the value that has been changed.
 * @param con the constraint that shoudl be applied.
 */
typedef gboolean (*PrefFunc)(pref_node_t* node, constraint_t* con);

/** The preferences node. You should not access this class directly,
 * instead use pref_get_ptr() and pref_get_ref().
 */
struct _pref_node_t {
  object_t _object;       ///< the base class
  GList* children;
  pref_node_t* parent;
  void* value;            ///< member pointing to memory containg the value
  gboolean save;
  FreeFunc free;
  StoreFunc store;
  StoreStrFunc store_str;
  XMLPrintFunc xml_print;
  GList* constraint;
};

#define CONSTRAINT(arg) ((constraint_t*)(arg))

/** A constraint that can be applied to a preferences value.
 * @see pref_add_constraint()
 */
struct _constraint_t {
  PrefFunc func;     ///< the constraint function
};

#ifdef WIN32
EXPORT_CORE pref_node_t* __node__;
#else
extern pref_node_t* __node__;
#endif

/** @name Registering settings */
/*@{*/
module_t* pref_module_instance(void);
void pref_register(const char* name, gboolean save,
                   XMLPrintFunc xml_print,
                   FreeFunc free,
                   StoreFunc store,
                   StoreStrFunc store_str, ...);
void pref_register_int(const char* name, gboolean save, int value);
void pref_register_string(const char* name, gboolean save, const char* value);
void pref_register_bool(const char* name, gboolean save, gboolean value);
void pref_register_ip(const char* name, gboolean save, ip_t value);
/*@}*/

/** @name Accessing settings */
/*@{*/
pref_node_t* pref_search(const char* name, gboolean only_leaf);

/** Gets the value and casts it to the type pointer.
 * @param name the name of the value.
 * @param type the type to cast to
 * @return the settings value casted to a pointer
 */
#define pref_get_ptr(name, type) \
  (type*)(((__node__=pref_search(name,TRUE))&&__node__&&__node__->value)?__node__->value:NULL)
/** Gets the value and casts it to the type.
 * @param name the name of the value.
 * @param type the type to cast to
 * @return the settings value casted to the correct type
 */
#define pref_get_ref(name, type) \
  (((__node__=pref_search(name,TRUE))&&__node__->value)?*((type*)(__node__->value)):(type)0)

void pref_store(const char* value, ...);
void pref_store_str(const char* name, const char* value);
/*@}*/


void pref_observe(const char* name, ObjectFunc func, void* data);
void pref_unobserve(const char* name, ObjectFunc func, void* data);

/** @name Adding constraints */
/*@{*/
void pref_add_constraint(const char* name, constraint_t* constraint);
void pref_add_constraint_min(const char* name, int max);
void pref_add_constraint_max(const char* name, int max);
void pref_add_constraint_minmax(const char* name, int min, int max);
void pref_add_constraint_replace(const char* name, char from, char to);
void pref_add_constraint_port(const char* name);
/*@}*/

/*@}*/

void pref_foreach(ObjectFunc func, void* data);
const char* pref_node_fullname(pref_node_t* node);

G_END_DECLS
#endif
