/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.9 $
 * $Date: 2007/09/19 14:32:59 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QtGui>
#include <cmath>

#include "lcore/l_limit.h"
#include "lcore/l_i18n.h"

#include "ui_style.hpp"
#include "ui_main.hpp"
#include "BandwidthBar.hpp"


using namespace std;

enum {
  COLOR_BACK_GROUND,
  COLOR_BORDER,
  COLOR_BASE,
  COLOR_BAR,
  COLOR_TEXT
};

BandwidthBar::BandwidthBar(QWidget *parent)
    : QWidget(parent)
{ 

    LOPSTER_APP_GET_STYLE(this->style_core);
    this->brange = 1024*128;  // default value
    this->bvolume = NULL;
    this->brate = NULL;
    this->mouse_pressed = false;
    this->setMouseTracking(true);

    QFontMetrics fm = fontMetrics();
    this->grid_height = fm.height();
    setMinimumSize(QSize(400, this->grid_height + 20 ));
    // These colors will be updated when the style is changed
    ucolor[COLOR_BACK_GROUND] = QColor(0xdf, 0xb0, 0xb0);
    ucolor[COLOR_BORDER] = QColor(0x99, 0x99, 0x99);
    ucolor[COLOR_BASE] = QColor(0xff, 0xff, 0xff);
    ucolor[COLOR_BAR] = QColor(0x96, 0xa5, 0x96);
    ucolor[COLOR_TEXT] = QColor(0x00, 0x00, 0x00);   
}

void BandwidthBar::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);
    if (!this->bvolume || !this->brate){return;}
    draw(&painter);
}

QSize BandwidthBar::minimumSizeHint() const
{ 
    return QSize(400, this->grid_height + 20 );
}  

#define BLR 0
#define BOT 0

void BandwidthBar::mousePressEvent(QMouseEvent *event)
{
    int width = this->width() - 2*(2+BLR);
    int x = (int)event->x() - (2+BLR);
    int y = event->y();
    int limit;

    mouse_pressed = true;
    if (x < 0) x = 0;
    if (x > width) x = width;

    if ( y > this->grid_height ) {
        setCursor(Qt::ArrowCursor);
        if (x == 0) limit = 0;
        else limit = ((quint64)this->brange * (quint64)x -1) / (quint64)width +1;
        limit_set_limit(LIMIT(this->bvolume), limit);
        update();
    } else {
        setCursor(Qt::ClosedHandCursor);
        this->x_drag_start = x>0?x:1;   // must be > 0 !
        this->drag_range = this->brange;
        update();
    }
}

void BandwidthBar::mouseReleaseEvent(QMouseEvent *event)
{
    int y = event->y();
    mouse_pressed = false;
    if (y > this->grid_height ) {
        setCursor(Qt::ArrowCursor);
    } else {
        setCursor(Qt::OpenHandCursor);
    }
    update();
}

void BandwidthBar::mouseMoveEvent(QMouseEvent *event)
{
    int width = this->width() - 2*(2+BLR);
    int x = (int) event->x() - (2+BLR);
    int y = event->y();
    int limit;

    if (x < 0) x = 0;
    if (x > width) x = width;

    if (y > this->grid_height ) {
        setCursor(Qt::ArrowCursor);
        if (mouse_pressed)
        {
            if (x == 0) limit = 0;
            else limit = ((quint64)this->brange * (quint64)x -1) / (quint64)width +1;
            limit_set_limit(LIMIT(this->bvolume), limit);
        }
        update();
    } else if (x > 0) {
        if (mouse_pressed)
        {
            setCursor(Qt::ClosedHandCursor);
            quint64 res = (quint64)this->drag_range * (quint64)this->x_drag_start / (quint64)x;
            if (res < 16) res = 16;
            else if (res > (1<<30)) res = (1<<30);
            this->brange = res;
        }
        else {
            setCursor(Qt::OpenHandCursor);
        }
        update();
    }
    else {
        if (mouse_pressed)
        {
            setCursor(Qt::ClosedHandCursor);
        }
        else {
            setCursor(Qt::OpenHandCursor);
        }
        update();
    }
}

void BandwidthBar::ui_bandwidth_force_update(void* dummy, BandwidthBar *bar)
{
    Q_ASSERT(bar);
    bar->update();
}

void BandwidthBar::setVolume(volume_t* volume)
{
    this->bvolume = volume;
    object_monitor_d(OBJ(this->bvolume),
                   (ObjectFunc)ui_bandwidth_force_update, this);
}

unsigned int BandwidthBar::calc_x(quint32 val, quint32 max, int width) {
    if (val > max) return width;
    else return (int) (((double) val / (double) max) * (double) width);
}

void BandwidthBar::draw(QPainter *painter)
{
    int width = this->width();
    int height = this->height();
    unsigned texth, dx, posx;
    int x1; // need to be signed, as it may go out of range (ie. -3 )
    unsigned limit;
    QFontMetrics fm = fontMetrics();       
    QPen thickPen(palette().foreground(), 1.5);
    QPen thinPen(palette().foreground(), 0.5);
    
    texth = fm.height() + 4;
    dx = 1;
    while (calc_x(dx, this->brange, width-2*(2+BLR)) < 80) dx *=2;

    x1 = 0;
    while (x1 <= this->brange) {
        QString str = qfu(print_speed((int*)&x1));
        posx = calc_x(x1, this->brange, width-2*(2+BLR));
        QFontMetrics fm = fontMetrics();       
        painter->drawText(posx+BLR+6,0, fm.width("9999999 KB/s"), fm.height(),Qt::AlignCenter, str); 
        painter->drawLine(QLineF(QPointF(posx+BLR+2, 0), QPointF(posx+BLR+2, height))); 
        x1 += dx;
    }
  
    if (this->style_core->ui_style_sets("bandwidth:color", qtr("Base"))) {
        painter->setPen(ucolor[COLOR_BASE]);
        painter->setBrush(ucolor[COLOR_BASE]);
        painter->drawRect(QRectF(BLR, texth, width-2*BLR, height-texth-BOT));
        if (this->style_core->ui_style_sets("bandwidth:color", qtr("Border"))) {
            painter->setPen(ucolor[COLOR_BORDER]);
            painter->drawRect(BLR+1, texth, width-2*BLR-2, height-texth-BOT-1);
        } else if (this->style_core->ui_style_sets("bandwidth:color", qtr("Text")) &&
               this->style_core->ui_style_sets("bandwidth:color", qtr("Base"))) {
            painter->setPen(ucolor[COLOR_TEXT]);
            painter->drawRect(BLR+1, texth, width-2*BLR-2, height-texth-BOT-1);
        }
    }

   
    // FIXME : would a gradient be much nicer for the rate ? 
    x1 = calc_x(rate_get_rate(this->brate), this->brange, width - 2*(2+BLR));
    if (x1 > 0)
    {
        painter->setPen(QPen(ucolor[COLOR_BAR], 0.5));
        painter->setBrush(ucolor[COLOR_BAR]);
        painter->drawRect(QRectF(BLR+2, texth+2,  x1, height-texth-BOT-4));
    }

    limit = limit_get_limit(LIMIT(this->bvolume));
    if (limit <= this->brange) {
        x1 = calc_x(limit, this->brange, width - 2*(2+BLR));
        QPolygonF polygon;
        polygon <<  QPointF( 2+BLR + x1 - 6 + 7 , texth + 5 ) 
                 << QPointF( 2+BLR + x1 - 6,  height - 5 )
                 << QPointF( 2+BLR + x1 - 6 + 14, height - 5);
        painter->setPen(thinPen);
        painter->setBrush(palette().foreground());
        painter->drawPolygon(polygon);

    }
}


RateProgressWidget::RateProgressWidget(QWidget *parent)
    : QWidget(parent)
{ 
    this->bvolume = NULL;
    this->brate = NULL;
    this->brange = 1024*128;  // default value
}

void RateProgressWidget::paintEvent(QPaintEvent * /* event */)
{
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing, true);
    if (!this->bvolume || !this->brate){return;}
    draw(&painter);
}

unsigned int RateProgressWidget::calc_x(quint32 val, quint32 max, int width) {
    if (val > max) return width;
    else return (int) (((double) val / (double) max) * (double) width);
}


void RateProgressWidget::draw(QPainter *painter)
{
    int width = this->width();
    int height = this->height();

    int x1 = calc_x(rate_get_rate(this->brate), this->brange, width - 2 - 2);
    if (x1 > 0)
    {
        painter->setPen(QPen( QColor(0x96, 0xa5, 0x96), 0.5));
        painter->setBrush(QColor(0x96, 0xa5, 0x96));
        painter->drawRect(QRectF(2, 2, x1-2, height-2));
    }
}


QSize RateProgressWidget::minimumSizeHint() const
{ 
    return QSize(183, 24);
}  

void RateProgressWidget::ui_bandwidth_force_update(void* dummy, RateProgressWidget *widget)
{
    Q_ASSERT(widget);
    widget->update();
}

void RateProgressWidget::setVolume(volume_t* volume)
{
    this->bvolume = volume;
    object_monitor_d(OBJ(this->bvolume),
                   (ObjectFunc)ui_bandwidth_force_update, this);
}

BandwidthProgress::BandwidthProgress(const QString& text, QWidget *parent)
    : QWidget(parent)
{
    QHBoxLayout *layout = new QHBoxLayout(this);
    layout->setSpacing(2);
    layout->setContentsMargins(1, 1, 1, 1);
    label = new QLabel(this);
    label->setText(text);
    layout->addWidget(label);
    pbar = new RateProgressWidget(this);
    layout->addWidget(pbar); 
}

QSize BandwidthProgress::minimumSizeHint() const
{ 
    return QSize(183, 24);
}  
