/****************************************************************************
**
** Copyright (c) 2007 Trolltech ASA <info@trolltech.com>
**
** Use, modification and distribution is allowed without limitation,
** warranty, liability or support of any kind.
**
** Version 1.1
****************************************************************************/

/*
 * 20070803 - Sid4U : modified to behave as the gtk2 hyperlink buttons from sgop
 *                    In my opinion look much better
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "commandlink.hpp"
#include <qstylepainter.h>
#include <qstyleoption.h>
#include <qfont.h>


CommandLink::CommandLink(QWidget *parent)
: QPushButton(parent)
{
    setAttribute(Qt::WA_Hover);
}

QSize CommandLink::sizeHint() const
{
    QSize size = QPushButton::sizeHint();
    QFont buttonFont = font();
    buttonFont.setPointSize(11);
    QFontMetrics fm(buttonFont);
    int titleWidth = fm.width(text());
    size.setWidth(qMax(size.width(), titleWidth + 32));
    size.setHeight(size.height() + 44);
    return size;
}

static QColor mixColors(const QColor &a, const QColor &b, int value)
{
    QColor tmp = a;
    tmp.setRed((tmp.red() * value) / 255 + (b.red() * (255 - value)) / 255);
    tmp.setGreen((tmp.green() * value) / 255 + (b.green() * (255 - value)) / 255);
    tmp.setBlue((tmp.blue() * value) / 255 + (b.blue() * (255 - value)) / 255);
    return tmp;
}

void CommandLink::paintEvent(QPaintEvent *)
{
    QStylePainter p(this);
    QStyleOptionButton option;
    initStyleOption(&option);
    //Enable command link appearence on Vista
    option.features |= QStyleOptionButton::CommandLinkButton; 
    option.text = QString();
    option.icon = QIcon();

    QIcon glyph = style()->standardIcon(QStyle::SP_CommandLink);
    QSize glyphSize = glyph.actualSize(QSize(32, 32));
    int hOffset = isDown() ? style()->pixelMetric(QStyle::PM_ButtonShiftHorizontal) : 0;
    int vOffset = isDown() ? style()->pixelMetric(QStyle::PM_ButtonShiftVertical) : 0;

    int topMargin = 10 + hOffset;
    int leftMargin = 7 + vOffset;
    int textOffset = glyphSize.width() + leftMargin + 6;
    int rightMargin = 4;
    int bottomMargin = 4;
    float descriptionSize = 9.0;
    QColor textColor = Qt::black;

    //This is a hack to detect if we are indeed running Vista style themed and not in classic
    bool usingVistaStyle = style()->inherits("QWindowsVistaStyle") 
        && !style()->pixelMetric(QStyle::PM_ButtonShiftHorizontal);
    float titleSize = usingVistaStyle ? descriptionSize + 3.0 : descriptionSize;

    //Draw glyph
    p.drawControl(QStyle::CE_PushButton, option);
    p.drawPixmap(leftMargin, topMargin, glyph.pixmap(glyphSize, QIcon::Normal, QIcon::Off));

    //Draw title    
    QFont buttonFont = font();
    if (usingVistaStyle) {
        textColor = QColor(21, 28, 85);
        buttonFont.setFamily("Segoe UI");
        if (underMouse() && !isDown())
            textColor = QColor(7, 64, 229);
        //A simple text color transition
        currentColor = mixColors(textColor, currentColor, 60); 
        option.palette.setColor(QPalette::ButtonText, currentColor);
    } else {
        buttonFont.setBold(true);
    }
    buttonFont.setPointSizeF(titleSize);
    p.setFont(buttonFont);
    int textflags = Qt::TextShowMnemonic;
    if (!style()->styleHint(QStyle::SH_UnderlineShortcut, &option, this))
        textflags |= Qt::TextHideMnemonic;
    p.drawItemText(rect().adjusted(textOffset, topMargin, -rightMargin, 0), 
        textflags, option.palette, isEnabled(), text(), QPalette::ButtonText);

    //Draw description
    buttonFont.setBold(false);
    textflags |= Qt::TextWordWrap | Qt::ElideRight;
    buttonFont.setPointSizeF(descriptionSize);
    p.setFont(buttonFont);
    p.drawItemText(rect().adjusted(textOffset, topMargin + 6 + p.fontMetrics().height(), 
        -rightMargin, -bottomMargin), textflags,
        option.palette, isEnabled(), description(), QPalette::ButtonText);
}
