/*****************************************************************************
 * qvlcframe.hpp : A few helpers
 *****************************************************************************
 * Copyright (C) 2006-2007 the VideoLAN team
 * $Id: CustomWidgets.hpp,v 1.1 2007/09/19 14:32:14 sid4u Exp $
 *
 * Authors: Clment Stenac <zorglub@videolan.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
 *****************************************************************************/

#ifndef _CUSTOMWIDGETS_H_
#define _CUSTOMWIDGETS_H_

#include <QWidget>
#include <QSplitter>
#include <QDialog>
#include <QSpacerItem>
#include <QHBoxLayout>
#include <QApplication>
#include <QSettings>
#include <QMainWindow>
#include <QPlastiqueStyle>
#include <QPushButton>
#include <QKeyEvent>

class LWidget : public QWidget
{
public:
    LWidget(QWidget *parent=0) : QWidget( parent )
    {};
    virtual ~LWidget()  {};

    virtual void loadCoreData( )=0;
protected:
    void readSettings( QString name, QSize defSize )
    {
        QSettings settings( "LopsterP2P", "Lopster2" ); // FIXME
        settings.beginGroup( name );
        resize( settings.value( "size", defSize ).toSize() );
        move( settings.value( "pos", QPoint( 0,0 ) ).toPoint() );
        settings.endGroup();
    }
    void writeSettings( QString name )
    {
        QSettings settings( "LopsterP2P", "Lopster2" ); // FIXME
        settings.beginGroup( name );
        settings.setValue ("size", size() );
        settings.setValue( "pos", pos() );
        settings.endGroup();
    }
};


class LSplitter : public QSplitter
{
public:
    LSplitter(QWidget *parent=0) : QSplitter( parent ) {
        this->name = QString("BasicSplitter");
    };
    ~LSplitter()  {
        writeSettings();
    };

    void readSettings( QString name )
    {
        this->name = name;
        QSettings settings( "LopsterP2P", "Lopster2" ); // FIXME
        settings.beginGroup( "splitter" );
        restoreState(settings.value(name).toByteArray()); 
        settings.endGroup();
    }

    void writeSettings()
    {
        QSettings settings( "LopsterP2P", "Lopster2" ); // FIXME
        settings.beginGroup( "splitter" );
        settings.setValue( name, saveState());
        settings.endGroup();
    }
private:
    QString name;
};

class LDialog : public QDialog
{
public:
    LDialog(QWidget* parent=0) : QDialog( parent )
    {}
    virtual ~LDialog() {};
    void toggleVisible()
    {
        if( isVisible() ) hide();
        else show();
    }

protected:
    virtual void cancel()
    {
        hide();
    }
    virtual void close()
    {
        hide();
    }
    virtual void keyPressEvent( QKeyEvent *keyEvent )
    {
        if( keyEvent->key() == Qt::Key_Escape )
        {
            cancel();
        }
        else if( keyEvent->key() == Qt::Key_Return )
        {
             close();
        }
    }
};

class LMainWindow : public QMainWindow
{
public:
    LMainWindow(QWidget* parent=0) : QMainWindow(parent)
    {}
    virtual ~LMainWindow() {};
    void toggleVisible()
    {
        if( isVisible() ) hide();
        else show();
    }
protected:
    QSize mainSize;

    void readSettings( QString name, QSize defSize )
    {
        QSettings settings( "LopsterP2P", "Lopster2" ); // FIXME
        settings.beginGroup( name );
        QSize s =  settings.value( "size", defSize ).toSize() ;
        // fprintf( stderr, "%i %i ", s.width(), s.height() ); // this line crash on win32
        move( settings.value( "pos", QPoint( 0,0 ) ).toPoint() );
        settings.endGroup();
    }
    void readSettings( QString name )
    {
        QSettings settings( "LopsterP2P", "Lopster2" ); // FIXME
        settings.beginGroup( name );
        mainSize = settings.value( "size", QSize( 0,0 ) ).toSize();
        settings.endGroup();
    }
    void writeSettings( QString name )
    {
        QSettings settings( "LopsterP2P", "Lopster2" ); // FIXME
        settings.beginGroup( name );
        settings.setValue ("size", size() );
        settings.setValue( "pos", pos() );
        settings.endGroup();
    }
};

#endif
