/*
 * WengoPhone, a voice over Internet phone
 * Copyright (C) 2004-2007  Wengo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


  /*
  * This nice expanding widget is slightly modified version of WengoPhone ExpandButton, but adding a bit of 
  * more fancy icons, and label
  */


#include <QtGui/QStyleOption>
#include <QtGui/QStylePainter>
#include <QtGui/QHBoxLayout>
#include <QtGui/QToolButton>
#include <QtGui/QLabel>
#include <QMouseEvent>

#include "ExpandWidget.hpp"

ExpandWidget::ExpandWidget(const QString &res, QWidget* parent)
    : QFrame(parent) , _expandDirection(ED_Down) 
{
    setObjectName("ExpandWidget");
    QHBoxLayout *layout = new QHBoxLayout(this);

    expand = new QLabel(this);
	// Set icon, so that QToolButton::sizeHint().width() is correct
	QPixmap pix(":/expand_button/right.png");
	expand->setPixmap(pix);
    QPixmap ico_pix(res); 
    icon = new QLabel(this);
    icon->setPixmap(ico_pix);

    label = new QLabel(this);

	layout->addWidget(expand);
	layout->addWidget(label);
	layout->addItem(new QSpacerItem(expand->sizeHint().width(),
                expand->sizeHint().height(),QSizePolicy::Expanding,QSizePolicy::Fixed) );
	layout->addWidget(icon);

    toggle = false;
    setMouseTracking(true);
	setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
}

void ExpandWidget::on_toggle(bool toggle)
{
    setExpandButtonPixmap(false);
    emit expand_toggled(toggle);
}

void ExpandWidget::setToggle(bool toggled)
{   
    toggle = toggled;
    setExpandButtonPixmap(false);    
}

void ExpandWidget::setText(const QString &text)
{
    label->setText(text);
}

QSize ExpandWidget::sizeHint() const {
	QSize size = expand->sizeHint();
	size.setHeight(fontMetrics().height());
	return size;
}

void ExpandWidget::setExpandDirection(ExpandDirection direction) {
	_expandDirection = direction;
    setExpandButtonPixmap(false);
}


ExpandWidget::ExpandDirection ExpandWidget::expandDirection() const {
	return _expandDirection;
}

void ExpandWidget::setExpandButtonPixmap(bool hover)
{
	QString pixmapName;
	if (toggle) {
		if (_expandDirection == ED_Down) {
			pixmapName = "down";
		} else {
			pixmapName = "up";
		}
	} else {
		pixmapName = "right";
	}
	if (hover) {
		pixmapName += "_hover";
	}
	QPixmap pixmap(QString(":/expand_button/%1.png").arg(pixmapName));
    expand->setPixmap(pixmap);
}

void ExpandWidget::mouseMoveEvent(QMouseEvent *event)
{
    setExpandButtonPixmap(true);
    QFrame::mouseMoveEvent(event);
}

void ExpandWidget::leaveEvent(QEvent *event)
{
    setExpandButtonPixmap(false);
    QFrame::leaveEvent(event);
}

void ExpandWidget::mouseReleaseEvent(QMouseEvent *event)
{
    toggle = !toggle;
    on_toggle(toggle);
    QFrame::mouseReleaseEvent(event);
}
