/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.8 $
 * $Date: 2007/11/12 14:41:16 $
 * $Author: sid4u $
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "lcore/l_i18n.h"
#include "lcore/l_base.h"
#include "ui_main.hpp"
#include "CheckBoxDirModel.hpp"
#include "LibraryPref.hpp"



LibraryPref::LibraryPref(QWidget *parent)
                : QWidget(parent)
{

  	setupUi(this);
    QStringList empty;
    model = new CheckBoxDirModel(empty,QDir::AllDirs | QDir::NoDotAndDotDot); 
    dir_view->setModel(model);
    int column_count = model->columnCount();
    for (int i=1;i<column_count;i++)
    {
        dir_view->hideColumn(i); 
    }
    lib_updated = false;

    dir_view->show();
    connect(model, SIGNAL(selectionChanged()), this, SLOT(library_updated()));
}

void LibraryPref::share_init_from_library(library_t* lib)
{
    GList* dlist;
    this->lib = lib;
    if (lib) {
        for (dlist = lib->shared_folder; dlist; dlist = dlist->next) {
            char* folder = (char*) dlist->data;
            folder_vector.push_back( qfu("%1").arg(folder).toStdString() );
        }
        model->setSelection(folder_vector);
    }
}

void LibraryPref::share_update_library()
{

}

void LibraryPref::library_updated()
{
    lib_updated = true;
}

void LibraryPref::share_on_library_apply() {
    if (!lib_updated) return;
    
    // TODO : enchance this - Check the removed folders, and the new ones, instead of refreshing all

    library_t* lib = library_get_current();
    if (!lib) return;
    library_clear_shared(lib);

    model->fillFileVectors(dir_view->rootIndex());
    vector <string> folder_vector = model->getFolderVector();
    for( unsigned int i = 0; i < folder_vector.size(); ++i )
    {
        QString path = QString::fromStdString( folder_vector[i] );
        // Check if dir exist
        QDir dir(path);
        if (!dir.exists())
        {
            lopster_notify(LOPSTER_MESSAGE_WARNING, _("Directory does not exist"),
                     l_printf(_("The directory <b>%s</b> does not exist and "
                                "therefore cannot be used for sharing."), qtu(path)));
            continue;
        }
        library_add_shared(lib, qtu(QDir::toNativeSeparators(path)));
    }
    library_refresh(TRUE);
    model->clearVectors();
}

void LibraryPref::share_on_library_cancel() {
    library_t* lib = library_get_current();
    share_init_from_library(lib);
    model->clearVectors();
}
