/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.9 $
 * $Date: 2007/09/11 07:32:11 $
 * $Author: sid4u $
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QVBoxLayout>
#include "ModuleWidget.hpp"



/*
 * Very usefull class to handle html box like widgets 
 * Inspired from Joomla XHTML modules with label and body
 * Skinnable in style sheet
 */

ModuleLabel::ModuleLabel(QWidget *parent)
    : QFrame(parent)
{
    hlayout = new QHBoxLayout(this);
    m_icon = new QLabel(this);
    
    //FIXME: make maybe the size of icon changeable in pref
    QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(m_icon->sizePolicy().hasHeightForWidth());
    m_icon->setSizePolicy(sizePolicy);
    m_icon->setMinimumSize(QSize(25, 25));
    m_icon->setMaximumSize(QSize(25, 25));


    m_label = new QLabel(this);
	m_label->setObjectName("SearchModuleTitle");

    QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
    sizePolicy1.setHorizontalStretch(0);
    sizePolicy1.setVerticalStretch(0);
    sizePolicy1.setHeightForWidth(m_label->sizePolicy().hasHeightForWidth());
    m_label->setSizePolicy(sizePolicy1);


	hlayout->addWidget(m_icon);
	hlayout->addWidget(m_label);
	setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
}

QSize ModuleLabel::minimumSizeHint() const
{ 
    return QSize(0,15);
}

void ModuleLabel::set_text(const QString &title)
{
	m_label->setText(title);
}

void ModuleLabel::set_icon(const QString &res)
{
    QPixmap pix; 
    pix.load(res);
    m_icon->setMinimumSize(QSize(24,24));
    m_icon->setMaximumSize(QSize(24,24));
    m_icon->setPixmap(pix);
    hlayout->invalidate();
}


void ModuleLabel::hide_icon(bool value)
{
	if (value) {
		m_icon->setVisible(false);
		m_icon->setMinimumSize(QSize(0,0));
		m_icon->setMaximumSize(QSize(0,0));
	}
	else {
		m_icon->setVisible(true);
		m_icon->setMinimumSize(QSize(20,20));
		m_icon->setMaximumSize(QSize(20,20));
	}
    hlayout->invalidate();
}

ModuleBody::ModuleBody(QWidget *parent)
    : QFrame(parent)
{
	setFrameStyle(QFrame::StyledPanel | QFrame::Raised);
}

ModuleWidget::ModuleWidget(QWidget *parent)
    : QWidget(parent)
{     
    QVBoxLayout *layout = new QVBoxLayout(this);
	m_label = new ModuleLabel(this);
    QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Minimum);
    sizePolicy.setHorizontalStretch(0);
    sizePolicy.setVerticalStretch(0);
    sizePolicy.setHeightForWidth(m_label->sizePolicy().hasHeightForWidth());
    m_label->setSizePolicy(sizePolicy);

    m_body = new ModuleBody(this);
    QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Expanding);
    sizePolicy1.setHorizontalStretch(0);
    sizePolicy1.setVerticalStretch(0);
    sizePolicy1.setHeightForWidth(m_body->sizePolicy().hasHeightForWidth());
    m_body->setSizePolicy(sizePolicy1);

	body_layout = new QVBoxLayout(m_body);
    body_layout->setSpacing(0);
    body_layout->setMargin(0);

	layout->addWidget(m_label);
    layout->addWidget(m_body);
    layout->setSpacing(0);
    layout->setMargin(0);

	hide_icon(true);
}

void ModuleWidget::set_title(const QString &title)
{
    m_label->set_text(title);
}

void ModuleWidget::set_widget(QWidget *widget)
{
    widget->setParent(m_body);
    body_layout->addWidget(widget);
}

ModuleWidget *ModuleWidget::CreateSearchModuleWidget(QWidget *parent)
{
    ModuleWidget *widget = new ModuleWidget(parent);
    widget->get_label()->setObjectName("SearchModuleLabel");
    widget->get_body()->setObjectName("SearchModuleBody");
    return widget;
}

void ModuleWidget::set_icon(const QString &res)
{
	m_label->set_icon(res);
}

void ModuleWidget::hide_icon(bool hide)
{
	m_label->hide_icon(hide);
}

ModuleWidget *ModuleWidget::CreateStandardModule(QWidget *parent)
{
    ModuleWidget *widget = new ModuleWidget(parent);
    widget->get_label()->setObjectName("ChatModuleLabel");
    widget->get_body()->setObjectName("ChatModuleBody");
    return widget;
}
