/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.21 $
 * $Date: 2007/11/13 13:24:24 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "lcore/download.h"
#include "lcore/search.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "ui_main.hpp"
#include "ui_utils.hpp"
#include "ui_style.hpp"
#include "download_view.hpp"



enum {
  COLOR_BACK_GROUND,
  COLOR_BORDER,
  COLOR_ACTIVE,
  COLOR_FINISHED,
  COLOR_PROGRESS,
  COLOR_SIZE,
};

#define FIND_DOWNLOAD_VIEW(v) \
            v =  qobject_cast<DownloadView *> (findWidgetByName(QString("download_view"))); \
            Q_ASSERT(v); 

#define FIND_DOWNLOAD_TREE_VIEW(t) \
            t =  qobject_cast<DownloadTreeView *> (findWidgetByName(QString("download_tree_view"))); \
            Q_ASSERT(t); 

QList<download_t*> Downloads_list;
QList <download_t*> queued_downloads;
volume_t* TheDownloadVolume;
rate_t* TheDownloadRate;

class DownloadSourceItem : public QStandardItem
{
public:
    DownloadSourceItem(const QString &text)
        : QStandardItem(text) {
        m_download = NULL;
        m_source = NULL;
    }
    ~DownloadSourceItem() {
        if (m_download)
        {
            object_free_data(OBJ(m_download), "DownloadEntry", NULL);   
        }
        if (m_source)
        {
            object_free_data(OBJ(m_source), "SourceEntry", NULL); 
        }

    }
    void set_download(download_t* download) {
        this->m_download = download;
        this->setData(QVariant::fromValue((void*)download));
        object_set_data(OBJ(download), "DownloadEntry", this);
    }
    download_t* download() {return m_download;}
    void set_source(source_t* source) {
        this->m_source = source;
        this->setData(QVariant::fromValue((void*)source));
        object_set_data(OBJ(source), "SourceEntry", this);
    }
    source_t* source() {return m_source;}
private:
    download_t* m_download;
    source_t* m_source;
};

class DownloadItemModel : public QStandardItemModel
{
public:
    DownloadItemModel(QObject *parent=0);
    void download_node_add(download_t* download);
    void download_node_update(download_t* download);
    void download_node_delete(download_t* download);

    void source_node_add(source_t* source);
    void source_node_update(source_t* source);
    void source_node_delete(source_t* source);
};


DownloadItemModel::DownloadItemModel(QObject *parent)
    : QStandardItemModel(0,1,parent)
{
    QStringList headers;
    headers << qtr("Filename") << qtr("Status") << qtr("Source")
            << qtr("Progress") << qtr("Rate") << qtr("ETA");
    setHorizontalHeaderLabels(headers);  
}

void DownloadItemModel::download_node_add(download_t* download)
{
    QList<QStandardItem *> items;

    DownloadSourceItem *filename = static_cast<DownloadSourceItem *> 
                                (object_get_data(OBJ(download), "DownloadEntry"));
    if (filename) return;

    file_t* file;
    file = FILE(download);

    filename = new DownloadSourceItem(qfu(file->filename));
    QStandardItem *status = new QStandardItem();
    QStandardItem *source = new QStandardItem();
    QStandardItem *progress = new QStandardItem();
    QStandardItem *rate = new QStandardItem();
    QStandardItem *ETA = new QStandardItem();

    filename->set_download(download);
    
    items.append(filename);
    items.append(status);
    items.append(source);
    items.append(progress);
    items.append(rate);
    items.append(ETA);

    appendRow(items);
    download_node_update(download);
}
void DownloadItemModel::download_node_update(download_t* download)
{
    DownloadSourceItem  *filename;
    QStandardItem *status;
    QStandardItem *source;
    QStandardItem *progress;
    QStandardItem *rate;
    QStandardItem *ETA;

    filename = static_cast< DownloadSourceItem *>
                    (object_get_data(OBJ(download), "DownloadEntry"));
    if (!filename) return;

    QModelIndex model_index = indexFromItem(filename);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    
    
    // We have the row, let get all the items and update them
    status = item(row, 1); 
    source = item(row, 2); 
    progress = item(row, 3); 
    rate = item(row, 4); 
    ETA = item(row, 5); 

    file_t* file = FILE(download);
    file_size_t todo = file->size - download_get_progress(download);
     
    if (download->no_active)
    {
        int r = download_get_rate(download);
        int l = r>0?todo/r:0;
        status->setText(qtr("%1 to go").arg(print_filesize(&todo)));
        source->setText(qtr("%1/%2 active").arg(download->no_active).arg(download->no_source)); 
        progress->setText(QString::number(qrand ())); // FIXME
        rate->setText(qtr(print_speed(&r)));
        ETA->setText(qtr(print_duration_unit(&l)));
    }
    else
    {
        status->setText(qtr("%1 to go").arg(print_filesize(&todo)));
        source->setText(qtr("%1 sources").arg(download->no_source)); 
        rate->setText(QString(""));
        ETA->setText(QString(""));
    }
}
void DownloadItemModel::download_node_delete(download_t* download)
{
    DownloadSourceItem  *filename = static_cast<DownloadSourceItem *>
                        (object_get_data(OBJ(download), "DownloadEntry"));
    if (!filename) return;

    QModelIndex model_index = indexFromItem(filename);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    removeRow(row);  
}

void DownloadItemModel::source_node_add(source_t* source)
{
    QList<QStandardItem *> items;
    DownloadSourceItem *filename;
    DownloadSourceItem *parent;
    
    if (!source->download) return;
    parent = static_cast<DownloadSourceItem *> 
                    (object_get_data(OBJ(source->download), "DownloadEntry"));
    if (!parent) return;
    
    if (object_get_data(OBJ(source), "SourceEntry")) return;

    filename = new DownloadSourceItem(qfu(source->file->filename));
    QStandardItem *status = new QStandardItem();
    QStandardItem *user = 
        new QStandardItem(  PICON(source->user->ID, LOPSTER_TREEVIEW_ICON),
                            qfu(source->user->print(source->user))
                         );
    QStandardItem *progress = new QStandardItem();
    QStandardItem *rate = new QStandardItem();
    QStandardItem *ETA = new QStandardItem();
       
    filename->set_source(source);
    
    items.append(filename);
    items.append(status);
    items.append(user);
    items.append(progress);
    items.append(rate);
    items.append(ETA);

    parent->appendRow(items);
    source_node_update(source);

}
void DownloadItemModel::source_node_update(source_t* source)
{
    DownloadSourceItem *filename = static_cast<DownloadSourceItem *> 
                                (object_get_data(OBJ(source), "SourceEntry"));
    QStandardItem *status;
    QStandardItem *user;
    QStandardItem *progress;
    QStandardItem *rate;
    QStandardItem *ETA;
    
    if (!filename) return;


    QModelIndex model_index = indexFromItem(filename);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    
    
    // We have the row, let get all the items and update them
    status = item(row, 1); 
    user = item(row, 2); 
    progress = item(row, 3); 
    rate = item(row, 4); 
    ETA = item(row, 5); 

    QString str_status;
    if (source->scheduled)
    {
        str_status = qtr("queued (locally)");
    }
    else if (source->status == SOURCE_REMOTE)
    {
        if (source->remote)
            str_status = qtr("queued (%1)").arg(source->remote);
        else
            str_status = qtr("queued (remotely)");
    }
    else if (source->status == SOURCE_STARTED)
    {
        str_status = qtr("requested");
    }
    
    if (source->cur_segment)
    {
        segment_t* seg = source->cur_segment;
        int r = socket_get_rate(SOCK(source));
        file_size_t s = seg->stop - seg->start - seg->size;
        int l = r>0?s/r:0;
        
        status->setText(!str_status.isEmpty()?str_status:qtr("%1 to go").arg(print_filesize(&s)));
        progress->setText(QString::number(qrand ())); // FIXME
        rate->setText(qtr(print_speed(&r)));
        ETA->setText(qtr(print_duration_unit(&l)));

    }
    else
    {
        status->setText(QString(""));
        rate->setText(QString(""));
        ETA->setText(QString(""));
    }
}
void DownloadItemModel::source_node_delete(source_t* source)
{
    DownloadSourceItem  *filename = static_cast<DownloadSourceItem *>
                        (object_get_data(OBJ(source), "SourceEntry"));
    if (!filename) return;

    QModelIndex model_index = indexFromItem(filename);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    removeRow(row);  
}

DownloadView::DownloadView(QWidget *parent ) : 
    LWidget(parent)
{
   	setupUi(this);
    QPixmap plib; 
    plib.load(":/download.png");
    plib = plib.scaled(LOPSTER_LABEL_ICON,Qt::KeepAspectRatio);
    label_down_pix->setPixmap(plib);
    push_clear_down->setIcon(QIcon(QString::fromUtf8(":/reload.png")));

    QFontMetrics fm = fontMetrics();
    label_d_bandwidth->setFixedSize(QSize(fm.width(qtr("999999999999 KB/s")), fm.lineSpacing()));
    label_d_downloaded->setFixedSize(QSize(fm.width(qtr("999999999999 KB/s")), fm.lineSpacing()));
    label_d_limit->setFixedSize(QSize(fm.width(qtr("999999999999 KB/s")), fm.lineSpacing()));    
    // FIXME : fix the max range to max download defined in preference
    update = new DownloadInfoLabels;
    update->bandwidth = label_d_bandwidth;
    update->uploaded = label_d_downloaded;
    update->limit = label_d_limit;
    download_tree_view->set_update_labels(update);
}

DownloadView::~DownloadView()
{
	delete update;
}

void DownloadView::loadCoreData( )
{
    TheDownloadVolume = volume_find("Download Bandwidth Limit");
    TheDownloadRate = downloads_get_rate();
    download_tree_view->ui_download_set_monitors( );
    connect(push_clear_down,SIGNAL(clicked()),this, SLOT(clear_download())); 
}

void DownloadView::clear_download()
{
    download_clear_finished();
}

void DownloadView::ui_download_show_details(download_t* download) {
    DownloadView *down_view;
    FIND_DOWNLOAD_VIEW(down_view);
    DownloadDetails *details = new DownloadDetails(download, down_view);
    details->exec();
}


DownloadTreeView::DownloadTreeView(QWidget *parent)
    : UiTreeView(parent)
{
    setItemDelegate(new DownloadTreeViewDelegate(parent, this));
    setSelectionBehavior(QAbstractItemView::SelectRows);

    ui_treeview_set_model(new DownloadItemModel(this));

    action_item_t* actions[2];
    actions[0] =  action_item_search("Download actions");
    actions[1] =  action_item_search("Source actions");
    ui_tv_handle_button(2, actions, NULL);

}

void DownloadTreeView::ui_download_set_monitors( )
{
    object_monitor_d(OBJ(TheDownloadVolume), (ObjectFunc)s_on_band_changed, update->bandwidth);
    object_monitor_d(OBJ(TheDownloadVolume), (ObjectFunc)s_on_limit_changed, update->limit);
    object_monitor_d(OBJ(TheDownloadVolume), (ObjectFunc)s_on_load_changed, update->uploaded);
    object_set_dirty(OBJ(TheDownloadVolume));
}
/*
static GtkWidget* on_download_tip(void* object, unsigned depth) {
  GtkWidget* table;
  int line = 0;

  table = gtk_table_new(1, 2, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);

  if (depth == 1) {
    download_t* download = object;
    char* str;
    
    ui_table_add_file(GTK_TABLE(table), &line, FILE(download));
    ui_table_add_sep(GTK_TABLE(table), &line);
    
    str = make_string_from_list(download->search_keys, " ");
    ui_table_add_line(GTK_TABLE(table), &line, _("Search keys"), str);
    g_free(str);
    
    ui_table_add_line(GTK_TABLE(table), &line, _("Sources"),
                      l_printf("%d", download->no_source));
  } else if (depth == 2) {
    source_t* source = object;
 
    ui_table_add_file(GTK_TABLE(table), &line, source->file);
    ui_table_add_sep(GTK_TABLE(table), &line);
    ui_table_add_user(GTK_TABLE(table), &line, source->user);

    // DEBUG
    ui_table_add_sep(GTK_TABLE(table), &line);
    ui_table_add_line(GTK_TABLE(table), &line, _("Status"),
                      l_printf("%d", source->status));
    ui_table_add_line(GTK_TABLE(table), &line, _("Scheduled"),
                      source->scheduled?_("yes"):_("no"));
    ui_table_add_line(GTK_TABLE(table), &line, _("Timer"),
                      l_printf("%d", source->timeout));
    // DEBUG END
  }

  gtk_widget_show(table);
  return table;
}

*/

void DownloadTreeView::s_on_limit_changed(limit_t* limit, QLabel *label) {
    int l = limit_get_limit(limit);
    label->setText(qtr("%1").arg(print_speed(&l)));
}

void DownloadTreeView::s_on_band_changed(limit_t* limit, QLabel *label) {
    int r = rate_get_rate(TheDownloadRate);
    label->setText(QString("%1").arg(print_speed(&r)));
    (void)limit;
}

void DownloadTreeView::s_on_load_changed(volume_t* volume, QLabel *label) {
    file_size_t size = volume_get_sum(volume);
    label->setText(QString("%1").arg(print_filesize(&size)));
}

void DownloadTreeView::ui_download_hide(download_t* download)
{
    DownloadTreeView *tree;
    FIND_DOWNLOAD_TREE_VIEW(tree);

    static_cast<DownloadItemModel *>(tree->model())->download_node_delete(download);
}

void DownloadTreeView::ui_download_update(download_t* download)
{
    DownloadTreeView *tree;
    FIND_DOWNLOAD_TREE_VIEW(tree);
    static_cast<DownloadItemModel *>(tree->model())->download_node_update(download);
}

void DownloadTreeView::ui_download_new(download_t* download)
{  
  
    DownloadTreeView *tree;
    FIND_DOWNLOAD_TREE_VIEW(tree);

    Downloads_list.append(download);

    static_cast<DownloadItemModel *>(tree->model())->download_node_add(download);

    object_notify_d(OBJ(download), (ObjectFunc)ui_download_hide, NULL);
    object_monitor_d(OBJ(download), (ObjectFunc)ui_download_update, NULL);
}

void DownloadTreeView::ui_source_hide(source_t* source)
{
    DownloadTreeView *tree;
    FIND_DOWNLOAD_TREE_VIEW(tree);
    static_cast<DownloadItemModel *>(tree->model())->source_node_delete(source);
}

void DownloadTreeView::ui_source_update(source_t* source)
{
    DownloadTreeView *tree;
    FIND_DOWNLOAD_TREE_VIEW(tree);
    static_cast<DownloadItemModel *>(tree->model())->source_node_update(source);
}

void DownloadTreeView::ui_source_new(source_t* source)
{
    DownloadTreeView *tree;
    FIND_DOWNLOAD_TREE_VIEW(tree);

    if (!source->download) return;   

    static_cast<DownloadItemModel *>(tree->model())->source_node_add(source);
    
    object_notify_d(OBJ(source), (ObjectFunc)ui_source_hide, NULL);
    object_monitor_d(OBJ(source), (ObjectFunc)ui_source_update, NULL);   
}

void DownloadTreeView::ui_download_periodic_update(void)
{
    GList* dlist;
    DownloadTreeView *tree;
    FIND_DOWNLOAD_TREE_VIEW(tree);

    for (int i = 0; i < Downloads_list.size() ; i++ )
    {
        download_t* download = Downloads_list.at(i);
        if (download->segments_used) {
            static_cast<DownloadItemModel *>(tree->model())->download_node_update(download);
        }
        for (dlist = download->sources; dlist; dlist = dlist->next) {
            source_t* source = (source_t*) dlist->data;
            if (source->used_segments) {
                static_cast<DownloadItemModel *>(tree->model())->source_node_update(source);
            }
        } 
    }
}

DownloadDetails::DownloadDetails(download_t* download, QWidget *parent)
                : QDialog(parent)
{
    setupUi(this);
    setAttribute( Qt::WA_DeleteOnClose );
    this->download = download;
    object_notify_d(OBJ(download), (ObjectFunc)s_on_download_killed, this);

    line_down_finename->setText(qfu(FILE(download)->localname)); 
    default_search_keys();

    connect(tool_down_change_file, SIGNAL(clicked()), this, SLOT(change_file()));
    connect(tool_down_default_search, SIGNAL(clicked()), this, SLOT(default_search_keys()));
    connect(push_apply_download, SIGNAL(clicked()), this, SLOT(details_apply()));
    connect(push_cancel_download, SIGNAL(clicked()), this, SLOT(details_cancel()));
}

void DownloadDetails::default_search_keys()
{
    char* str = make_string_from_list(download->search_keys, " ");
    line_down_search_keys->setText(qfu("%1").arg(str));
}

void DownloadDetails::change_file()
{
    // FIXME : gtk2 ui doesn't do much either after changing the file
    // maybe not implemented yet
    QString filename = QFileDialog::getOpenFileName(this,
                        tr("Choose file"));
    if (!filename.isEmpty())
    {
        line_down_finename->setText(filename); 
    }
}

void DownloadDetails::s_on_download_killed(download_t* download, DownloadDetails *dialog)
{
    object_unnotify_d(OBJ(download), (ObjectFunc)s_on_download_killed, dialog);
    dialog->close();
}

void DownloadDetails::details_cancel( )
{
    object_unnotify_d(OBJ(download), (ObjectFunc)s_on_download_killed, this);
    close();
}

// Double check here as something need to be applyed and it's not from the gtk2 plugin
void DownloadDetails::details_apply( )
{
    GSList* list = NULL;
    
    make_list_from_string(&list, qtu(line_down_search_keys->text()), " ");
    if (!list)
    {
        lopster_notify(LOPSTER_MESSAGE_INFO, _("Invalid search keys"),
                       _("Please specify search keys"));
        return;
    }
    g_slist_foreach(download->search_keys, (GFunc)g_free, NULL);
    g_slist_free(download->search_keys);
    download->search_keys = list;
    
    details_cancel( );
}

DownloadTreeViewDelegate::DownloadTreeViewDelegate(QWidget *parent, DownloadTreeView *tree_view) 
                        : QItemDelegate(parent) 
{
    this->tree_view = tree_view;

    // These colors will be updated when the style is changed
    ucolor[COLOR_BACK_GROUND] = QColor(0xe7 /*39*/, 0xe7/*39*/, 0xe7/*39*/);
    ucolor[COLOR_BORDER] = QColor(0xcf/*5e*/, 0x41/*25*/, 0x41/*25*/);
    ucolor[COLOR_FINISHED] = QColor(0xef/*00*/, 0xc8/*00*/, 0x65/*00*/);
    ucolor[COLOR_ACTIVE] = QColor(0xe4/*00*/, 0xd9/*00*/, 0xb4/*00*/);
    ucolor[COLOR_PROGRESS] = QColor(0xcf/*5e*/, 0x41/*25*/, 0x41/*25*/);
 
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);    
    connect(style_core,SIGNAL(styledUpdated(const QString &)), SLOT(style_changed(const QString &)));
}

void DownloadTreeViewDelegate::style_changed(const QString &style)
{
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);   


    if ( style_core->ui_style_sets(style, qtr("Background"))) {
        ucolor[COLOR_BACK_GROUND] = style_core->ui_style_get_color(style, qtr("Background") );
    } else {
        ucolor[COLOR_BACK_GROUND] = QColor();
    }

    if ( style_core->ui_style_sets(style, qtr("Border"))) {
        ucolor[COLOR_BORDER] = style_core->ui_style_get_color(style, qtr("Border") );
    } else {
        ucolor[COLOR_BORDER] = QColor();
    }

    if ( style_core->ui_style_sets(style, qtr("Finished"))) {
        ucolor[COLOR_FINISHED] = style_core->ui_style_get_color(style, qtr("Finished") );
    } else {
        ucolor[COLOR_FINISHED] = QColor();
    }

    if ( style_core->ui_style_sets(style, qtr("In-progress"))) {
        ucolor[COLOR_ACTIVE] = style_core->ui_style_get_color(style, qtr("In-progress") );
    } else {
        ucolor[COLOR_ACTIVE] = QColor();
    }

    if ( style_core->ui_style_sets(style, qtr("Percentage"))) {
        ucolor[COLOR_PROGRESS] = style_core->ui_style_get_color(style, qtr("Percentage") );
    } else {
        ucolor[COLOR_PROGRESS] = QColor();
    }
}

void DownloadTreeViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
                  const QModelIndex &index ) const
{
   
    if (index.column() != 3) {
        QItemDelegate::paint(painter, option, index);
        return;
    }

    file_size_t prog, todo;
    int x, w, h, x2;

    int row = index.row();

   
    DownloadSourceItem *cd = (DownloadSourceItem *)static_cast<DownloadItemModel *>(tree_view->model())-> item(0,row);
    if (!cd)
    {
        Q_ASSERT(false); // Should never happen :)
    }
    todo = prog = 0;
    QRect rect = option.rect;

    w = rect.width();
    h = rect.height();
  
    if (cd->download() || 
       (cd->source() && cd->source()->cur_segment)) {
        // the background
       if (ucolor[COLOR_BACK_GROUND].isValid()) {
          painter->setPen(ucolor[COLOR_BACK_GROUND]);
          painter->setBrush(ucolor[COLOR_BACK_GROUND]);
          painter->drawRect(QRectF(rect.x() ,rect.y() , w, h));
       }
       if (ucolor[COLOR_BORDER].isValid()) {
          painter->setPen(ucolor[COLOR_BORDER]);
          painter->drawRect(rect.x() ,rect.y() , w-1, h-1);
       }
    }

    w -= 2;
    h -= 2;
    if (cd->download()) {
        GList* dlist;
        download_t* download = (download_t*) cd->download();
        file_size_t size = FILE(download)->size;
        
        todo = size;

        for (dlist = segments_get_list(download->segments_done); dlist; dlist = dlist->next) {
          segment_t* segment = (segment_t*) dlist->data;
          prog += (segment->stop-segment->start);
          if (ucolor[COLOR_FINISHED].isValid()) {
            x = segment->start*w/size;
            x2 = (segment->stop-1)*w/size +1;
            
            painter->setPen(ucolor[COLOR_BACK_GROUND]);
            painter->setBrush(ucolor[COLOR_BACK_GROUND]);
            painter->drawRect(QRectF(rect.x() + x+1,rect.y() , x2-x, h));
          }
        }


    
        for (dlist = download->segments_used; dlist; dlist = dlist->next) {
          dl_segment_t* s = (dl_segment_t*) dlist->data;
          segment_t* segment = s->segment;
          
          x = segment->start*w/size;
          if (ucolor[COLOR_ACTIVE].isValid()) {
            if (segment->start + segment->size < segment->stop) {
                x2 = (segment->stop-1)*w/size +1;
                painter->setPen(ucolor[COLOR_ACTIVE]);
                painter->setBrush(ucolor[COLOR_ACTIVE]);
                painter->drawRect(QRectF(rect.x() + x+1,rect.y()+1 , x2-x, h));
            }
          }
            if (segment->size > 0) {
                prog += segment->size;
                if (ucolor[COLOR_FINISHED].isValid()) {
                 x2 = (segment->start+segment->size-1)*w/size +1;
                 painter->setPen(ucolor[COLOR_FINISHED]);
                 painter->setBrush(ucolor[COLOR_FINISHED]);
                 painter->drawRect(QRectF(rect.x() + x+1,rect.y()+1 , x2-x, h));
                }
            }
        }
    }    
    else if (cd->source()) {
        GList* dlist;
        source_t* source = (source_t*) cd->source();
        download_t* download = source->download;
        file_size_t size = FILE(download)->size;

        for (dlist = source->used_segments; dlist; dlist = dlist->next) {
          segment_t* segment = (segment_t*) dlist->data;
          
          x = segment->start*w/size;
          if (ucolor[COLOR_ACTIVE].isValid()) { 
            if (segment->start + segment->size < segment->stop) {
              x2 = (segment->stop-1)*w/size +1;
              painter->setPen(ucolor[COLOR_ACTIVE]);
              painter->setBrush(ucolor[COLOR_ACTIVE]);
              painter->drawRect(QRectF(rect.x() + x+1,rect.y()+1 , x2-x, h));
            }
          } 
          if (ucolor[COLOR_FINISHED].isValid()) { 
            if (segment->size > 0) {
              x2 = (segment->start+segment->size-1)*w/size +1;
              painter->setPen(ucolor[COLOR_FINISHED]);
              painter->setBrush(ucolor[COLOR_FINISHED]);
              painter->drawRect(QRectF(rect.x() + x+1,rect.y()+1 , x2-x, h));
            }
          }
        }


        if (source->cur_segment) {
          segment_t* segment = source->cur_segment;
          todo = segment->stop - segment->start;
          prog = segment->size;

          x = segment->start*w/size;
          if (ucolor[COLOR_ACTIVE].isValid()) {
            if (segment->start + segment->size < segment->stop) {
              x2 = (segment->stop-1)*w/size +1;

              painter->setPen(ucolor[COLOR_ACTIVE]);
              painter->setBrush(ucolor[COLOR_ACTIVE]);
              painter->drawRect(QRectF(rect.x() + x+1,rect.y()+1 , x2-x, h));
            }
          }
          if (ucolor[COLOR_FINISHED].isValid()) {
            if (segment->size > 0) {
              x2 = (segment->start+segment->size-1)*w/size +1;

              painter->setPen(ucolor[COLOR_FINISHED]);
              painter->setBrush(ucolor[COLOR_FINISHED]);
              painter->drawRect(QRectF(rect.x() + x+1,rect.y()+1 , x2-x, h));
            }
          }
        }
     }

     if (ucolor[COLOR_PROGRESS].isValid()) {
        if (prog > 0 && todo > 0) {
          x = 0;
          x2 = (prog-1)*w / todo +1;
              
          painter->setPen(ucolor[COLOR_PROGRESS]);
          painter->setBrush(ucolor[COLOR_PROGRESS]);
          painter->drawRect(QRectF(rect.x() + x+1,rect.y()+1 , x2-x, h/4));
        }
     }
}
