#ifndef _DOWNLOADVIEW_H_
#define _DOWNLOADVIEW_H_


#include <QtGui>
#include "lcore/download.h"
#include "CustomWidgets.hpp"
#include "ui_treeview.hpp"
#include "widgets/DownloadDetailsDialog.h"


class DownloadInfoLabels
{
public:
    QLabel *bandwidth;
    QLabel *uploaded;
    QLabel *limit;
};

class DownloadTreeView : public UiTreeView
{
   Q_OBJECT
public:
    DownloadTreeView(QWidget *parent = 0);
    
    void ui_download_set_monitors( );

    static void ui_download_update(download_t* download);
    static void ui_download_hide(download_t* download);
    static void ui_download_new(download_t* download);
        
    static  void ui_source_update(source_t* source);
    static void ui_source_hide(source_t* source);
    static void ui_source_show(source_t* source);
    static void ui_source_new(source_t* source);
    static void ui_download_periodic_update(void);

    void set_update_labels(DownloadInfoLabels *update) { this->update = update;} 

    static void s_on_limit_changed(limit_t* limit, QLabel *label);
    static void s_on_band_changed(limit_t* limit, QLabel *label);
    static void s_on_load_changed(volume_t* volume, QLabel *label);
private:
    DownloadInfoLabels *update; 
};

class DownloadTreeViewDelegate : public QItemDelegate
{
   Q_OBJECT
public:
    DownloadTreeViewDelegate(QWidget *parent, DownloadTreeView *tree_view);

    void paint(QPainter *painter, const QStyleOptionViewItem &option,
                      const QModelIndex &index ) const;

private slots:
    void style_changed(const QString &style);

private:
    DownloadTreeView * tree_view;
    QColor ucolor[5]; //back,border,active,finished,progress

};


#include "widgets/DownloadViewWidget.h"

class DownloadView :  public LWidget, private Ui::DownloadViewWidget
{
    Q_OBJECT
public: 
    DownloadView(QWidget *parent = 0);
    ~DownloadView();
    void loadCoreData( );
    static void ui_download_show_details(download_t* download);
private slots:
    void clear_download();
private:
	DownloadInfoLabels *update;
};

class DownloadDetails :  public QDialog, private Ui::DownloadDetailsDialog
{
    Q_OBJECT
public: 
    DownloadDetails(download_t* download, QWidget *parent = 0);
    static void s_on_download_killed(download_t* download, DownloadDetails *dialog);
private slots:
    void default_search_keys();
    void change_file();
    void details_apply( );
    void details_cancel( );
private:
    download_t* download;
};

#endif //_DOWNLOADVIEW_H_
