/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.18 $
 * $Date: 2007/09/19 14:32:13 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QtGui>
#include <QObject>

#include "lcore/plugin.h"
#include "lcore/l_i18n.h"

#include "ui_utils.hpp"
#include "ui_browse.hpp"
#include "ui_main.hpp"
#include "ui_buddies.hpp"

#define FIND_BUDDIES_TREE(t) \
            t =  qobject_cast<BuddiesTree *> (findWidgetByName(QString("buddies_tree"))); \
            Q_ASSERT(t); 

QHash<int, BuddyFuncs*> users_funcs;

BuddyItem::BuddyItem(buddy_t *buddy, BuddiesTree* tree)
        : BuddyTreeItem(tree)
{
    this->buddy = buddy;
    this->context = NULL;
}

BuddyItem::~BuddyItem( )
{
    object_free_data(OBJ(buddy), "BuddyEntry", NULL);
}

UserItem::UserItem(user_t *user, BuddyItem* item)
                : BuddyTreeItem(item)
{
    this->user = user;
    // Updated by plugin
    this->context = NULL;
}

UserItem::~UserItem( )
{
    char* object_name = g_strdup_printf("%d_UserEntry",user->ID);
    object_free_data(OBJ(user), object_name, NULL);
}


BasicUserEditorWidget::BasicUserEditorWidget(QWidget *parent)
        : AbstractUserEditorWidget(parent)
{
    QVBoxLayout *vboxLayout = new QVBoxLayout(this);
    group_plugin = new QGroupBox(this);
    QVBoxLayout *group_layout = new QVBoxLayout(group_plugin);
    QLabel *label = new QLabel(group_plugin);
    label->setText(qtr("monitor nicks (space separated list)"));
    group_layout->addWidget(label);
    line_users = new QLineEdit(group_plugin);
    group_layout->addWidget(line_users);
    vboxLayout->addWidget(group_plugin);      
}

BuddiesTree::BuddiesTree(QWidget* parent)
                : QTreeWidget(parent)
{
    header()->hide();
    //FIXME : set one selectable at the time
}

BuddiesTree::~BuddiesTree()
{
}

void BuddiesTree::ui_buddy_populate_buddy_menu(QMenu *menu)
{
    QAction *edit_buddy = menu->addAction(qtr("Edit buddy"));
    QAction *delete_buddy = menu->addAction(qtr("Delete buddy"));   
    menu->addSeparator();

    connect(edit_buddy,   SIGNAL(triggered()), this, SLOT(ui_buddy_edit_buddy()));
    connect(delete_buddy, SIGNAL(triggered()), this, SLOT(ui_buddy_delete_buddy()));
}

void BuddiesTree::ui_buddy_populate_top_menu(QMenu *menu)
{
    QAction *new_buddy = menu->addAction(qtr("Add buddy"));
    menu->addSeparator();

    connect(new_buddy,    SIGNAL(triggered()), this, SLOT(ui_buddy_create_buddy()));
}

// Normally only on selectable at the time
void BuddiesTree::contextMenuEvent(QContextMenuEvent *e)
{
    QMenu *menu = new QMenu(this);
    ui_buddy_populate_top_menu(menu);
    QList<QTreeWidgetItem*> buddy_list = selectedItems();
    if (!buddy_list.isEmpty())
    {
        BuddyTreeItem* buddy_item = static_cast<BuddyTreeItem*>(buddy_list.takeFirst());
        QMenu *buddy_menu = buddy_item->context;
        // Merge top and that one from plugin
        if (buddy_menu)
        {
            QAction *action= menu->addMenu(buddy_menu);
            action->setText(qtr("Actions"));
        } //  buddy one is null as it is set here
        else {
            ui_buddy_populate_buddy_menu(menu);
        }
    }
    menu->exec(e->globalPos());
}

void BuddiesTree::ui_buddy_create_buddy()
{
    BuddyEditor *editor = new BuddyEditor(NULL,true,this);
    editor->show();          
}

// Only one at the time
void BuddiesTree::ui_buddy_edit_buddy( )
{
    QList<QTreeWidgetItem* > buddy_list = selectedItems();
    if (!buddy_list.isEmpty())
    {
        BuddyItem* buddy_item = static_cast<BuddyItem*> (buddy_list.first());
        BuddyEditor *editor = new BuddyEditor(buddy_item->buddy, false,this);
        editor->show();
    }     
}

void BuddiesTree::ui_buddy_delete_buddy( )
{
    QList<QTreeWidgetItem* > buddy_list = selectedItems();
    if (buddy_list.isEmpty())
        return;

    QString message = qtr("Are you sure you wan to delete the following buddy ? \n\n");
    if (!buddy_list.isEmpty())
    {
        BuddyItem* buddy_item = static_cast<BuddyItem*> (buddy_list.first());
        message += qfu("\t %1 \n").arg(OBJ(buddy_item->buddy)->name) ;
    }     
    int ret = QMessageBox::warning(this, qtr("Delete buddy ?"),
               message, QMessageBox::Ok | QMessageBox::Cancel);
    if (ret == QMessageBox::Ok)
    {
        BuddyItem* buddy_item = static_cast<BuddyItem*> (buddy_list.first());
        object_kill(OBJ(buddy_item->buddy));
    }
}

BuddiesTree* BuddiesTree::getBuddiesTree() {
    return qobject_cast<BuddiesTree *> (findWidgetByName(QString("buddies_tree")));
}

void BuddiesTree::ui_buddies_apply_user_func(buddy_t* buddy, UserForeach func, QTreeWidgetItem *item)
{
    QHashIterator<int, BuddyFuncs*> i(users_funcs);
    while (i.hasNext()) {
        i.next();
        int ID = i.key();
        if (!buddy_has_user(buddy, ID)) continue;
        buddy_foreach_user(buddy, ID, (UserForeach)func, item);
    } 
}

void BuddiesTree::ui_buddy_user_hide(user_t* user)
{
    Q_ASSERT(user);
    char* object_name = g_strdup_printf("%d_UserEntry",user->ID);
    UserItem* item = static_cast<UserItem *> (object_get_data(OBJ(user), object_name));
    if (!item) return;
    delete item;
}

void BuddiesTree::ui_buddy_hide(buddy_t* buddy)
{
    Q_ASSERT(buddy);
    BuddyItem* item = static_cast<BuddyItem *> (object_get_data(OBJ(buddy), "BuddyEntry"));
    if (!item) return;   
    //WILL delete children automaticallly (users)
    delete item;
}

gboolean BuddiesTree::ui_buddy_update_user(user_t *user, BuddyItem *parent)
{
    BuddyFuncs *funcs = users_funcs[user->ID];
   
    UserItem *item;
    char* object_name = g_strdup_printf("%d_UserEntry",user->ID);
    item = static_cast<UserItem *> (object_get_data(OBJ(user), object_name));
    if (!item) return ui_buddy_show_user(user,parent);
    funcs->ui_buddy_plugin_user_update(user,item);
    return FALSE;
}

gboolean BuddiesTree::ui_buddy_show_user(user_t *user, BuddyItem *parent)
{
    object_notify_d(OBJ(user), (ObjectFunc)ui_buddy_user_hide, NULL);
//    object_monitor_d(OBJ(user), (ObjectFunc)ui_buddy_user_update, NULL);
    UserItem *item;   
    item = new UserItem(user,parent);  
    item->setText(0,qfu("%1").arg(user->print(user)));
    char* object_name = g_strdup_printf("%d_UserEntry",user->ID);
    object_set_data(OBJ(user), object_name, item);
    ui_buddy_update_user(user, parent);
    return FALSE;
}

void BuddiesTree::ui_buddy_update(buddy_t* buddy)
{
    BuddyItem* item;
    Q_ASSERT(buddy);   
    item = static_cast<BuddyItem *> (object_get_data(OBJ(buddy), "BuddyEntry"));
    if (!item) return;   
    item->setText(0,qfu("%1").arg(OBJ(buddy)->name));
    if (buddy->online)
        item->setIcon(0, LICON(":/hotlist.png", LOPSTER_TREEVIEW_ICON));
    else
        item->setIcon(0, LICONG(":/hotlist.png", LOPSTER_TREEVIEW_ICON));
    // TODO: re-display details if shown
    // if (ui_info_search(buddy)) ui_buddy_show_details(buddy, FALSE);
    
    ui_buddies_apply_user_func(buddy, (UserForeach)ui_buddy_update_user, item);
}

void BuddiesTree::s_ui_buddy_new(buddy_t* buddy)
{ 
    Q_ASSERT(buddy);
    
    // object_notify_d(OBJ(buddy), (ObjectFunc)s_ui_buddy_hide_info, NULL);
    object_notify_d(OBJ(buddy), (ObjectFunc)ui_buddy_hide, NULL);
    object_monitor_d(OBJ(buddy), (ObjectFunc)ui_buddy_update, NULL);
   
    if (object_get_data(OBJ(buddy), "BuddyEntry")) return;  
    BuddyItem *item = new BuddyItem(buddy,getBuddiesTree());
    if (buddy->online)
        item->setIcon(0, LICON(":/hotlist.png", LOPSTER_TREEVIEW_ICON));
    else
        item->setIcon(0, LICONG(":/hotlist.png", LOPSTER_TREEVIEW_ICON));
    object_set_data(OBJ(buddy), "BuddyEntry", item);
    ui_buddies_apply_user_func(buddy, (UserForeach)ui_buddy_show_user, item);
    ui_buddy_update(buddy);  
}


BuddiesPage* BuddiesPage::getBuddiesPage( ) {
    return qobject_cast <BuddiesPage *> (findWidgetByName("buddies_page"));
}

BuddiesPage::BuddiesPage(QWidget* parent)
        : LWidget(parent)
{
    QVBoxLayout *layout = new QVBoxLayout(this);

    LSplitter *splitter = new LSplitter(this);

    tab_buddies = new LTabWidget(splitter);
    buddies_tree = new BuddiesTree(splitter);
    buddies_tree->setObjectName("buddies_tree");

    splitter->addWidget(buddies_tree);
	splitter->addWidget(tab_buddies);
    layout->addWidget(splitter);

    splitter->readSettings(QString("BuddiesSplitter"));
    connect(tab_buddies,SIGNAL(close_tab(int)),this, SLOT(ui_buddy_close_browse(int)));
}

BuddiesPage::~BuddiesPage()
{
}

void BuddiesPage::ui_buddy_close_browse(int index)
{
    BrowseWidget *widget = qobject_cast<BrowseWidget *>(tab_buddies->widget(index));
    Q_ASSERT(widget);
    tab_buddies->removeTab(index);
    delete widget;
}   

void BuddiesPage::s_ui_browse_show(browse_t* browse)
{
    BuddiesPage *bpage = BuddiesPage::getBuddiesPage( );
    BrowseWidget *widget = new BrowseWidget(browse, bpage->tab_buddies);
    // FIXME : Handle own tab bar supporting font format
    bpage->tab_buddies->addTab(widget,
                            PICON(browse->user?browse->user->ID:-1, LOPSTER_TAB_ICON), 
                            qfu("Browsing %1").arg(OBJ(browse)->name)); 
}

void BuddiesPage::ui_buddy_register_funcs(int ID, BuddyFuncs *funcs)
{
    BuddiesPage * page = BuddiesPage::getBuddiesPage( );      
    Q_ASSERT(funcs);
    g_log("QT4_BUDDY", G_LOG_LEVEL_DEBUG,
        "registering buddy funcs for '%s'", plugin_get_name(plugin_get(ID)));
    users_funcs[ID] = funcs;
}


void BuddiesPage::s_ui_browse_link_new(browse_t* browse)
{
    // TODO : Update first the tree widget to handle blinkin icons
    //        then , add a special icon to the user icon item corresponding 
    //        to the browse (to show that there is a pending browse)
    //        unlink it on deleting browse
}

BuddyEditor::BuddyEditor(buddy_t* buddy, bool create, QWidget *parent)
    : QDialog(parent)
{
    setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose);

    this->buddy = buddy;
    this->create = create;

    if (!create)
    {
        Q_ASSERT(buddy);
        setWindowTitle(qtr("Edit buddy"));
        line_buddy->setText(QString("%1").arg(OBJ(buddy)->name));
        push_buddy_ok->setText(qtr("Apply"));
        connect(push_buddy_ok, SIGNAL(clicked()), this, SLOT(ui_buddy_editor_edit()));
    }
    else {
        setWindowTitle(qtr("Add new buddy"));
        push_buddy_ok->setText(qtr("Create"));
        connect(push_buddy_ok, SIGNAL(clicked()), this, SLOT(ui_buddy_editor_create()));
    }
    QVBoxLayout *layout = new QVBoxLayout(buddy_plugins);   
    QHashIterator<int, BuddyFuncs*> i(users_funcs);
    while (i.hasNext()) {
        i.next();
        BuddyFuncs *funcs = i.value();
        AbstractUserEditorWidget *widget = funcs->ui_buddy_plugin_edit_user();
        if (widget)
        {
            widget->setParent(buddy_plugins);
            widget->ui_buddy_user_populate(buddy);
            layout->addWidget(widget);
        }
        buddy_edit_widgets.append(widget);
    }
    connect(push_buddy_cancel, SIGNAL(clicked()), this, SLOT(ui_buddy_editor_close()));           
}

void BuddyEditor::ui_buddy_editor_close()
{
    done(QDialog::Accepted);
}

void BuddyEditor::ui_buddy_editor_edit()
{
    Q_ASSERT(buddy);
    if (!line_buddy->text().isEmpty())
    {
        object_set_name(OBJ(buddy), qtu(line_buddy->text()));
    }
    if (!line_buddy_description->text().isEmpty())
    {
        buddy_set_description(buddy, qtu(line_buddy_description->text()));
    }

    for (int i=0; i < buddy_edit_widgets.size() ; i++ )
    {
        AbstractUserEditorWidget* user_editor = buddy_edit_widgets.at(i);
        user_editor->ui_buddy_user_apply(buddy);
    }
    
    buddy_propagate_update(buddy);
    done(QDialog::Accepted);
}

void BuddyEditor::ui_buddy_editor_create()
{
    if (!line_buddy->text().isEmpty())
    {
        buddy = buddy_new(qtu(line_buddy->text())); 
    }
    else {
        // FIXME
        QMessageBox::warning(this, qtr("Error, mandatory field"),
                       qtr("You need to fill at least the buddy name"), QMessageBox::Ok );
        return;
    }
    Q_ASSERT(buddy);
    if (!line_buddy_description->text().isEmpty())
    {
        buddy_set_description(buddy, qtu(line_buddy_description->text()));
    }
    else {
        buddy_set_description(buddy, NULL);
    }

    for (int i=0; i < buddy_edit_widgets.size() ; i++ )
    {
        AbstractUserEditorWidget* user_editor = buddy_edit_widgets.at(i);
        user_editor->ui_buddy_user_apply(buddy);
    }
    buddy_propagate_update(buddy);
    done(QDialog::Accepted);
}
