#ifndef _UI_BUDDIES_H
#define _UI_BUDDIES_H

#include <QMainWindow>
#include <QTreeWidget>
#include "lcore/buddy.h"
#include "lcore/user.h"
#include "lcore/browse.h"
#include "CustomWidgets.hpp"
#include "LTabWidget.hpp"
#include "ui_utils.hpp"
#include "widgets/BuddyEditorDialog.h"


class BuddiesTree;

class BuddyTreeItem : public QTreeWidgetItem
{
public:
    BuddyTreeItem(QTreeWidget *tree) : QTreeWidgetItem(tree) {}
    BuddyTreeItem(QTreeWidgetItem *item) : QTreeWidgetItem(item) {}
    QMenu *context;
};

class BuddyItem : public BuddyTreeItem
{
public:
    BuddyItem(buddy_t *buddy, BuddiesTree *tree);
    ~BuddyItem( );
    buddy_t *buddy;
};

class QT4_UI_EXPORT UserItem : public BuddyTreeItem
{
public:
    UserItem(user_t* user, BuddyItem *buddy_item);
    ~UserItem( );
    user_t *user;
};

/* generic way for plugin to display the list of users 
*/
class QT4_UI_EXPORT AbstractUserEditorWidget : public QWidget
{
public:
    AbstractUserEditorWidget(QWidget *parent) : QWidget(parent) {};
    virtual void ui_buddy_user_populate(buddy_t *buddy)=0;
    virtual void ui_buddy_user_apply(buddy_t *buddy)=0;
};

/* Implementation of AbstractUserEditorWidget :
 * QLineEdit, with user names separated by spaces
 * It is used currently in Nap, DC, SLSK, but each plugin can create their own widget
 * It was defined here so they can just use it
 */
class QT4_UI_EXPORT BasicUserEditorWidget : public AbstractUserEditorWidget
{
public:
    BasicUserEditorWidget(QWidget *parent=0);
    virtual void ui_buddy_user_populate(buddy_t *buddy)=0;
    virtual void ui_buddy_user_apply(buddy_t *buddy)=0;
    QLineEdit *line_users;
    QGroupBox *group_plugin;
};


class QT4_UI_EXPORT BuddyFuncs 
{
public:    
    // Plugin should update item text, icons, status, and important, context menu
    virtual void ui_buddy_plugin_user_update(user_t* user, UserItem *item)=0;
    // Plugin can send either a subclass of BasicUserEditorWidget, or create their own
    virtual AbstractUserEditorWidget *ui_buddy_plugin_edit_user()=0;
};

class QT4_UI_EXPORT BuddiesTree : public QTreeWidget
{
    Q_OBJECT
public:
    BuddiesTree(QWidget *parent=0);
	~BuddiesTree( );

    static void s_ui_buddy_new(buddy_t* buddy);
    static void ui_buddy_update(buddy_t* buddy);    
    static void ui_buddy_user_hide(user_t* user);
    static gboolean ui_buddy_show_user(user_t *user, BuddyItem *parent);
    static gboolean ui_buddy_update_user(user_t *user, BuddyItem *parent);
    static void ui_buddy_hide(buddy_t* buddy);
    static void ui_buddies_apply_user_func(buddy_t* buddy, UserForeach func, QTreeWidgetItem *item);
    static BuddiesTree* getBuddiesTree( );
    void contextMenuEvent(QContextMenuEvent *e);
    QMenu *buddy_menu;
private:
    void ui_buddy_populate_buddy_menu(QMenu *menu);
    void ui_buddy_populate_top_menu(QMenu *menu);
private slots:
    void ui_buddy_create_buddy( );
    void ui_buddy_edit_buddy( );
    void ui_buddy_delete_buddy( );
};

class QT4_UI_EXPORT BuddiesPage : public LWidget
{
   Q_OBJECT
public:
    BuddiesPage(QWidget *parent = 0);
    ~BuddiesPage();   
    void loadCoreData(){} ;
    static void ui_buddy_register_funcs(int ID, BuddyFuncs *funcs);
    static void s_ui_browse_show(browse_t* browse);
    static void s_ui_browse_link_new(browse_t* browse);
    static BuddiesPage* getBuddiesPage( );
private slots:
    void ui_buddy_close_browse(int);
private:
    LTabWidget *tab_buddies;
    BuddiesTree *buddies_tree;
};

class QT4_UI_EXPORT BuddyEditor : public QDialog, private Ui::BuddyEditorDialog
{
   Q_OBJECT
public:
    BuddyEditor(buddy_t* buddy=0, bool create = true, QWidget *parent = 0);
private slots:
    void ui_buddy_editor_close();
    void ui_buddy_editor_create();
    void ui_buddy_editor_edit();
private:
    buddy_t* buddy;
    bool create;
    QList <AbstractUserEditorWidget*> buddy_edit_widgets;
};

#endif //_UI_BUDDIES_H
