/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.20 $
 * $Date: 2007/09/19 14:32:13 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QObject>
#include "ui_main.hpp"
#include "ui_style.hpp"
#include "ui_utils.hpp"
#include "ModuleWidget.hpp"
#include "ui_chat.hpp"

static QHash <QString, PrintFunc > PrintFuncList;
static QHash <QString, QTextCharFormat> FormatTag;
static QHash <QString, QTextCharFormat> FormatStyle;
static QHash <QString, QTextCharFormat> NickTag;
static ChatGui* current_gui;
static int current_gui_index;

static QString SPrefix ;
static QString CPrefix ;
static QString PPrefix ;
static QString JPrefix ;
static QString TPrefix ;

static bool show_hour = true;
static bool show_min = true;
static bool show_sec = true;

// This page use another way of handling gui_data, testing if it works better
// as it uses the pointer address to retrieve the data as text
// and free itself automatically with g_free

#define FIND_CHAT_GUI(p,d) \
            char* name = (char*) object_get_data(OBJ(p), "ChatGui"); \
            d =  qobject_cast<ChatGui *> (findWidgetByName(QString("%1").arg(name)));

#define FIND_CHAT_PAGE(p) \
            p =  qobject_cast<ChatPage *> (findWidgetByName(QString("chat_page"))); \
            Q_ASSERT(p); 


ChatLineEntry::ChatLineEntry(QWidget *parent) : QLineEdit(parent)
{
    monitor_changes = true;
    connect(this,SIGNAL(returnPressed()),this,SLOT(on_return()));
    connect(this,SIGNAL(textChanged(const QString&)),this,SLOT(on_text_changed(const QString&)));
}

void ChatLineEntry::keyPressEvent(QKeyEvent *event)
{
    char *ctext;
    chat_page_t* cur;
    cur = chat_page_get_current();

    if (event->key() == Qt::Key_Tab )  
    {	
        int pos;
        QString entry_text = text();
        char* t;
        pos = cursorPosition();
        // FIXME : for right to left languages
        entry_text = entry_text.left(pos);
        t = g_strdup(qtu(entry_text));
        ctext = command_expand(cur, t);
        g_free(t);
        if (ctext) {
            QString old_text = entry_text.right(entry_text.size() - pos);
            del();
            pos = 0;
            insert(qfu(ctext));
            setCursorPosition(pos);
            g_free(ctext);
        }
    } 
    else if (event->key() == Qt::Key_Up|| event->key() == Qt::Key_Down) 
    {
        const char* t;
        if (event->key() == Qt::Key_Up) 
	        t = command_history_up();
        else if (event->key() == Qt::Key_Down)
	        t = command_history_down();
        else 
	        t = NULL;
        if (t) {
            QString hist_text = qfu(t);
	        monitor_changes = false;
            setText(hist_text);
            setCursorPosition(hist_text.size()-1);
            monitor_changes = true;
        } else {
	        QApplication::beep();
        }
    }
    else 
    {
      QLineEdit::keyPressEvent(event);
    }
}

void ChatLineEntry::on_return( )
{
    QString ctext;
    ctext = text();
    if (ctext.isEmpty())
    {
        return;
    }   
    command_handle(chat_page_get_current(), qtu(ctext));
    setText("");
}

void ChatLineEntry::on_text_changed(const QString &text)
{
  if (monitor_changes)
    command_history_update(qtu(text));
}

QTextCharFormat ChatPage::ui_chat_get_format_from_tag(QString tag)
{
    return FormatTag[tag];
}

ChatGui::ChatGui(QWidget *parent)
    : QWidget(parent)
{
    setupUi(this);
}

void ChatGui::print_ts(QTextCursor cursor) {
#ifdef WIN32
    time_t wtime;
    struct tm* ltime;
#else
    struct tm* ltime = localtime(&CurrentTime);
#endif
#ifdef WIN32
    wtime = CurrentTime;
    ltime = localtime(&wtime);
#endif
    //FIXME
    
    QTextCharFormat format = FormatTag["c_timestamp"];
    if (show_hour)
    {
        QString res = QString("%1:").arg(ltime->tm_hour,2,10,QChar('0'));
        cursor.insertText(res, format);        
    }
    if (show_min)
    {
        QString res = QString("%1:").arg(ltime->tm_min,2,10,QChar('0'));
        cursor.insertText(res, format);   
    }
    if (show_sec)
    {
        QString res = QString("%1:").arg(ltime->tm_sec,2,10,QChar('0'));
        cursor.insertText(res, format);   
    }
}

void ChatGui::ui_chat_print_client(QTextCursor cursor, va_list ap) {
    (void)ap;
    // prints timestamp and client prefix
    // no args   
    print_ts(cursor);

    if (!CPrefix.isEmpty()) {
        QTextCharFormat format = FormatTag["c_cprefix"];
        cursor.insertText(CPrefix, format);   
    }
}

void ChatGui::ui_chat_print_server(QTextCursor cursor, va_list ap) {
    (void)ap;
    // prints timestamp and server prefix
    // no args
    print_ts(cursor);
    if (!SPrefix.isEmpty()) {
        QTextCharFormat format = FormatTag["c_sprefix"];
        cursor.insertText(SPrefix, format);   
    }
}

void ChatGui::ui_chat_print_join(QTextCursor cursor, va_list ap) {
    (void)ap;
    // prints timestamp and join prefix
    // no args
    print_ts(cursor);
    if (!JPrefix.isEmpty()) {
        QTextCharFormat format = FormatTag["c_jprefix"];
        cursor.insertText(JPrefix,format);
    }
}
void
ChatGui::ui_chat_print_part(QTextCursor cursor, va_list ap) {
    (void)ap;
    // prints timestamp and part prefix
    // no args
    print_ts(cursor);
    if (!PPrefix.isEmpty()) {
        QTextCharFormat format = FormatTag["c_pprefix"];
        cursor.insertText(PPrefix,format);
    }
}
void
ChatGui::ui_chat_print_other(QTextCursor cursor, va_list ap) {
    (void)ap;
    // prints timestamp and server prefix
    // no args
    print_ts(cursor);
    if (!TPrefix.isEmpty()) {
        QTextCharFormat format = FormatTag["c_tprefix"];
        cursor.insertText(TPrefix,format);   
    }
}
void
ChatGui::ui_chat_print_proto(QTextCursor cursor, va_list ap) {
    char* text;

    // prints [string]
    text = (char*) va_arg(ap, void*);

    QTextCharFormat format = FormatTag["c_bracket"];
    cursor.insertText( "[",format);   
    
    format = FormatTag["c_text"];
    cursor.insertText(qfu("%1").arg(text), format);   
    
    format = FormatTag["c_bracket"];
    cursor.insertText("]", format); 
}

//FIXME
void
ChatGui::ui_chat_print_nl(QTextCursor cursor, va_list ap) {
    (void)ap;
    // prints new line
    // no args 
    current_gui->cursors[current_gui_index] = 
        current_gui->ui_chat_create_cursor_from_view(current_gui_index);
}

QColor ChatGui::_get_nick_color(const char *nick, int mode) {
    uint hash = g_str_hash(l_printf("%sabcdefg", nick?nick:""));
    QColor fore;
    char* hex1 = "01234567";
    char* hex2 = "456789ab";
    char* hex3 = "89abcdef";
    char* hex;
    char ncolor[8] = "#000000";
    int i1;

    if (mode == 1)      hex = hex1;
    else if (mode == 2) hex = hex2;
    else if (mode == 3) hex = hex3;
    else                hex = hex2;

    for (i1 = 1; i1 < 7; i1++) {
        ncolor[i1] = hex[hash%8];
        hash >>= 3;
    }
    fore.setNamedColor(QString("%1").arg(ncolor));
    return fore;
}

void ChatGui::_set_colored(QTextCharFormat format, const char* tag_name, int* setting) {
    if (!strncmp(tag_name, "n_", 2)) {
        if (*setting) {
            QColor color = _get_nick_color(tag_name+2, *setting);
            format.setForeground(QBrush(color));
        } else {
            format.setForeground(QBrush(Qt::black));        
        }
    }
}
QTextCharFormat ChatGui::_get_nick_tag( const char* nick) {
    QString tag_name = qfu("n_%1").arg(nick);
    QTextCharFormat format = NickTag[tag_name];
    if (!format.isValid()) {
        int setting = pref_get_ref("/core/chat/colored_nicks", int);   
        _set_colored(format, nick, &setting);   
    }
    NickTag[tag_name] = format;
    return format;
}

void ChatGui::ui_chat_print_nick_text(QTextCursor cursor, va_list ap) {
    char* nick = va_arg(ap, char*);
    char* text = va_arg(ap, char*);
    // prints text in nick color, nick is not printed
    // char* nick, char* text
    QTextCharFormat format = _get_nick_tag( nick);
    cursor.insertText(qfu("%1").arg(text), format);  
}

void ChatGui::ui_chat_print_nick(QTextCursor cursor, va_list ap) {
    char* nick;
    char* text;

    // prints nick in nick color, text is 2 bytes with brackets
    // char* nick, char* text
    nick = va_arg(ap, char*);
    text = va_arg(ap, char*);

    if (text && text[0]) {
        QTextCharFormat format = FormatTag["c_bracket"];
        cursor.insertText(qfu("%1").arg(text[0]), format);   
    }

    QTextCharFormat format = _get_nick_tag(nick);
    cursor.insertText(qfu("%1").arg(nick), format);  

    if (text && text[1]) {
        QTextCharFormat format = FormatTag["c_bracket"];
        cursor.insertText(qfu("%1").arg(text+1), format);   
    }
}

void ChatGui::ui_chat_print_url(QTextCursor cursor, va_list ap) {
    char* text;
    // prints an url
    text = va_arg(ap, char*);  // FIXME
    QTextCharFormat format = FormatTag["c_url"];
    cursor.insertText( qfu("%1").arg(text), format);   
}

QTextCursor ChatGui::ui_chat_create_cursor_from_view( int index ) {
    // Cursor is invalid, we need to add a table cell 1 1
    // Get the cell cursor and update the hash
    QTextTableFormat tableFormat;
    tableFormat.setBorder(0);
    QTextCursor cr(current_gui->view[index]->textCursor());
    cr.movePosition(QTextCursor::End);
    QTextTable *table = cr.insertTable(1, 1, tableFormat);
    QTextCursor cursor = table->cellAt(0, 0).firstCursorPosition();
    current_gui->cursors[index] = cursor;
    return cursor;
}


ChatPage::ChatPage(QWidget* parent)
    : LWidget(parent)
{
	setupUi(this);
    chat_layout = new QVBoxLayout(chats_holder);
    chat_tabs = NULL;
}

ChatPage::~ChatPage()
{
}

void ChatPage::loadCoreData()
{
    ui_chat_init();
}

void ChatPage::ui_chat_register_print_func(const QString name, PrintFunc func) { 
    PrintFunc f2;
    f2 = PrintFuncList[name];
    if (f2) {
        g_log("UI_CHAT", G_LOG_LEVEL_WARNING,
          "print function '%s' already defined, overriding", qtu(name));
    }
    PrintFuncList[name] = func;
}

void ChatPage::on_timestamp_changed(pref_node_t* node ) {
    if (node && node->value)
    {
        int* value = ( int*) node->value;
        show_hour = (*value == TIME_HOUR_MIN || *value == TIME_HOUR_MIN_SEC);
        show_min = (*value == TIME_HOUR_MIN || *value == TIME_HOUR_MIN_SEC);
        show_sec = (*value == TIME_HOUR_MIN_SEC);
    }
}

void ChatPage::ui_chat_define_timestamp(char* pref) {
    on_timestamp_changed(pref_search(pref, TRUE));
    pref_observe(pref, (ObjectFunc)on_timestamp_changed, NULL); 
}


void ChatPage::ui_chat_define_default_text_color( QTextCharFormat format) {
    format.setForeground(QBrush(QColor(Qt::black)));       
}

void ChatPage::ui_chat_define_default_background_color(QTextCharFormat format ) {
    format.setBackground(QBrush(QColor(Qt::white))); 
}

void ChatPage::ui_chat_define_default_font_format(QTextCharFormat format) {
    format.setFont(QFont()); 
}

void ChatPage::ui_chat_define_default_underline_format(QTextCharFormat format ) {
    format.setFontUnderline(false); 
}


// called each time there is a change in the style , connected to style change emiter
void ChatPage::ui_chat_update_style(const QString style) {
    StyleCore * style_core;
    LOPSTER_APP_GET_STYLE(style_core);
    QTextCharFormat format;  

    if ( style_core->ui_style_sets(style, qtr("Text"))) {
        format.setForeground(QBrush(style_core->ui_style_get_color(style, qtr("Text"))));   
    } else {
        ui_chat_define_default_text_color(format );
    }

    if ( style_core->ui_style_sets(style, qtr("Background"))) {
        format.setBackground(QBrush(style_core->ui_style_get_color(style,  qtr("Background"))));   
    } else {
        ui_chat_define_default_background_color(format);
    }

    if ( style_core->ui_style_sets(style, qtr("Font"))) {
        format.setFont(style_core->ui_style_get_font(style, qtr("Font")));   
    } else {
        ui_chat_define_default_font_format(format);
    }

    if ( style_core->ui_style_sets(style, qtr("Underline"))) {
        format.setBackground(QBrush(style_core->ui_style_get_underline(style, qtr("Underline"))));   
    } else {
        ui_chat_define_default_underline_format(format);
    }

    FormatStyle[style] = format;
}

/*
 * We need to create a QTextCharFormat for each cell that is going to be displayed
 * As the core chat system relys on tags, we'll handle each tag to a QTextCharFormat
 * in a Hash, so when printing, we'll check for the tag, in specific print function first
 * If there are no function available for printing
 * we assume it is text to be printed according to a QTextFormat associated to the current tag
 * 
 */
void ChatPage::ui_chat_register_tag_from_style(const QString name, const QString style) {
    ui_chat_update_style(style);
    FormatTag[name] = FormatStyle[style];
}

void ChatPage::ui_chat_init() {
	
    ui_chat_register_tag_from_style("c_text",      "chat:text");
    ui_chat_register_tag_from_style("c_timestamp", "chat:timestamp");
    ui_chat_register_tag_from_style("c_client",    "chat:client");
    ui_chat_register_tag_from_style("c_server",    "chat:server");
    ui_chat_register_tag_from_style("c_join",      "chat:join");
    ui_chat_register_tag_from_style("c_part",      "chat:part");
    ui_chat_register_tag_from_style("c_sprefix",   "chat:sprefix");
    ui_chat_register_tag_from_style("c_cprefix",   "chat:cprefix");
    ui_chat_register_tag_from_style("c_tprefix",   "chat:tprefix");
    ui_chat_register_tag_from_style("c_jprefix",   "chat:jprefix");
    ui_chat_register_tag_from_style("c_pprefix",   "chat:pprefix");
    ui_chat_register_tag_from_style("c_bracket",   "chat:bracket");
    /*NickTag =*/ ui_chat_register_tag_from_style("c_nick",      "chat:nick");
    ui_chat_register_tag_from_style("c_priv_msg",  "chat:private");
    ui_chat_register_tag_from_style("c_hilight",   "chat:hilight");
   /*UrlTag = */ ui_chat_register_tag_from_style("c_url",       "chat:url");
	//	g_signal_connect(G_OBJECT(UrlTag), "event", G_CALLBACK(on_url_event), NULL);

  //  ui_chat_define_timestamp("/core/chat/timestamp");
    // finally the print functions, which is the frontend to the user
    ui_chat_register_print_func("client",   (PrintFunc) ChatGui::ui_chat_print_client);
    ui_chat_register_print_func("server",   (PrintFunc) ChatGui::ui_chat_print_server);
    ui_chat_register_print_func("join",     (PrintFunc) ChatGui::ui_chat_print_join);
    ui_chat_register_print_func("part",     (PrintFunc) ChatGui::ui_chat_print_part);
    ui_chat_register_print_func("proto",    (PrintFunc) ChatGui::ui_chat_print_proto);
    ui_chat_register_print_func("stamp",    (PrintFunc) ChatGui::ui_chat_print_other);
    ui_chat_register_print_func("nl",       (PrintFunc) ChatGui::ui_chat_print_nl);
    ui_chat_register_print_func("nick_text",(PrintFunc) ChatGui::ui_chat_print_nick_text);
    ui_chat_register_print_func("nick",     (PrintFunc) ChatGui::ui_chat_print_nick);
    ui_chat_register_print_func("url",      (PrintFunc) ChatGui::ui_chat_print_url);
    SPrefix = qfu("%1").arg(pref_get_ptr("/core/chat/prefix/server", char));
   /* pref_observe("/core/chat/prefix/server",
                 (ObjectFunc)on_prefix_changed, &SPrefix);*/
    CPrefix = qfu("%1").arg(pref_get_ptr("/core/chat/prefix/client", char));
  /*  pref_observe("/core/chat/prefix/client",
                 (ObjectFunc)on_prefix_changed, &CPrefix);*/
    JPrefix = qfu("%1").arg(pref_get_ptr("/core/chat/prefix/join", char));
   /* pref_observe("/core/chat/prefix/join",
                 (ObjectFunc)on_prefix_changed, &JPrefix);*/
    PPrefix = qfu("%1").arg(pref_get_ptr("/core/chat/prefix/part", char));
  /*  pref_observe("/core/chat/prefix/part",
                 (ObjectFunc)on_prefix_changed, &PPrefix);*/
	TPrefix = qfu("%1").arg(pref_get_ptr("/core/chat/prefix/text", char));
  /*  pref_observe("/core/chat/prefix/text",
                 (ObjectFunc)on_prefix_changed, &TPrefix);
    pref_observe("/core/chat/colored_nicks",
                 (ObjectFunc)on_colored_changed, NULL);*/
}



void ChatPage::ui_chat_page_add(chat_page_t* page, ChatGui* data) {

    if (!page) return;
    ChatPage *chat_page;
    FIND_CHAT_PAGE(chat_page);

    // Create the tab system first if it does not exist
    if (!chat_page->chat_tabs)
    {
        chat_page->chat_tabs = new LTabWidget(chat_page->chats_holder, QTabWidget::South);
        chat_page->chat_layout->addWidget(chat_page->chat_tabs);
        connect(chat_page->chat_tabs, SIGNAL(close_tab(int)),
                    chat_page, SLOT(ui_chat_page_close_page(int)));
        connect(chat_page->chat_tabs, SIGNAL(currentChanged(int)),
                    chat_page, SLOT(ui_chat_current_page_changed(int)));       
    }
    
    if (page->ID >= 0) {
        plugin_t* plugin = plugin_get(page->ID);
        QString title = qfu("<b>%1 - %2</b>").arg(plugin_get_name(plugin)).arg(page->title);
        data->ui_chat_gui_set_icon(qfu(":/%1").arg(plugin->info->icon));
        data->ui_chat_gui_set_title(title);
    } else {
        data->ui_chat_gui_set_icon(QString("")); // no icon
        data->ui_chat_gui_set_title( qfu("<b>%1</b>").arg(page->title));
    }

    QIcon icon;
    if (page->ID >= 0) {
        icon = PICON(page->ID, LOPSTER_TAB_ICON);
    } else {
        icon = LICON(":/lopster_icon.png", LOPSTER_TAB_ICON);
    }

    int index = chat_page->chat_tabs->addTab(data,icon,qfu("%1").arg(page->label));
    chat_page->chat_tabs->ltab_set_object(data, OBJ(page));
    object_monitor_d(OBJ(page), (ObjectFunc)ui_chat_page_update, NULL);
    return;
}

void ChatPage::ui_chat_page_close_page(int index)
{
    object_t* obj = chat_tabs->ltab_get_object(chat_tabs->widget(index));
    chat_page_close(PAGE(obj));
}

void ChatPage::ui_chat_current_page_changed(int index)
{
    object_t* obj = chat_tabs->ltab_get_object(chat_tabs->widget(index));
    chat_page_set_current(PAGE(obj));
}

void ChatPage::s_ui_chat_page_tab_remove(ChatGui *gui)
{
    ChatPage *chat_page;
    FIND_CHAT_PAGE(chat_page);
    if (gui)
    {
        chat_page->chat_tabs->removeTab(chat_page->chat_tabs->indexOf(gui));
    }
}

void ChatPage::s_ui_chat_page_remove(chat_page_t* page)
{
    ChatGui *data ;
    char* name = (char*) object_get_data(OBJ(page), "ChatGui");
    if (name)
    {
        data = qobject_cast<ChatGui *> (findWidgetByName(QString("%1").arg(name)));
    }
    else {
        return;
    }
    ChatPage *chat_page;
    FIND_CHAT_PAGE(chat_page);
    chat_page->chat_tabs->removeTab(chat_page->chat_tabs->indexOf(data));
    delete data;
    object_free_data_common(OBJ(page), "ChatGui");
}

void ChatPage::ui_chat_page_update(chat_page_t* page)
{
    ChatGui *data;
    FIND_CHAT_GUI(page, data);
    if (!data) return;
/*
    // update label
    if (!data->label.isEmpty() &&
        ( QString("%1").arg(page->label) != data->label))
    {
        data->label =  QString("%1").arg(page->label) ;
        //FIXME : update the label in the tab
    }


    // update title
    if (page->title)
    {
        char* temp;

        if (page->ID >= 0)
            temp = auto_text(g_markup_printf_escaped
                             ("<b>%s - %s</b>", plugin_get_name(plugin_get(page->ID)), page->title));
        else
            temp = auto_text(g_markup_printf_escaped("<b>%s</b>", page->title));
        

        if (strcmp(temp, gtk_label_get_label(data->title)))
        {
            gtk_label_set_markup(data->title, temp);
        }

        if ((GTK_WIDGET_SENSITIVE(data->title)!=FALSE) != (page->active!=FALSE))
        {
            gtk_widget_set_sensitive(GTK_WIDGET(data->title), page->active);
        }
    }

    */
}

void ChatPage::ui_chat_create_simple_page(chat_page_t* page) {
    // No need to set parent here, as it will be deleted when closing in tab
    ChatGui* tab = new ChatGui();
    char* object_name;
    STORE_OBJECT("ChatGui",object_name,OBJ(page),tab);
    
    QWidget *body = tab->ui_chat_gui_get_body();
    QVBoxLayout *vboxLayout = new QVBoxLayout(body);
    vboxLayout->setSpacing(0);
    vboxLayout->setMargin(0);

   //  ModuleWidget *moduleWidget = ModuleWidget::CreateStandardModule(tab);
   //  QWidget *moduleWidgetContents = new QWidget(moduleWidget->get_body());
   //  QVBoxLayout *vboxLayout2 = new QVBoxLayout(moduleWidgetContents);
   //  vboxLayout2->setSpacing(1);
   //  vboxLayout2->setMargin(1);
   //  moduleWidget->set_widget(moduleWidgetContents);
   //  vboxLayout1->addWidget(moduleWidget);
    tab->no_views = 1;
    tab->view[0] = new QTextEdit(body); // FIXME : can be a text browser here ? for better quality od display ?
    tab->view[0]->setAcceptRichText(true);
    tab->view[0]->setReadOnly(true);
    
    // To update title
    // tab->module_widget = moduleWidget;
    // vboxLayout2->addWidget(tab->view[0] );
    vboxLayout->addWidget(tab->view[0] );
    
    ui_chat_page_add(page, tab);
    object_notify_d(OBJ(page), (ObjectFunc)s_ui_chat_page_remove, NULL);
    return;
}

void ChatPage::ui_chat_print(chat_page_t* page, int index, va_list ap)
{
    ChatGui* data;
    char* tag;
    int padding;
    char* text;
    bool at_bottom = true;
         
    if (!page) return;
    FIND_CHAT_GUI(page, data);

    if (!data || index < 0 || index >= data->no_views) return;
    current_gui = data;
    current_gui_index = index;
    QTextCursor cursor = data->cursors[index];
    if (!cursor.currentFrame())
    {
        cursor = data->ui_chat_create_cursor_from_view( index );
    }

    while ((tag = va_arg(ap, char*)) != NULL)
    {
        PrintFunc pfunc;
        if (tag[0] != '+')
        {
            g_log("UI_CHAT", G_LOG_LEVEL_WARNING,
                  "ignoring chat tag [%s]", tag);
            continue;
        }
        tag++;
        padding = *tag++;
               
        if (padding & PAD_LEFT)
        {
            cursor.insertText(" ");
        }

        QString qtag = QString("%1").arg(tag);
        
        pfunc = PrintFuncList[qtag];

        
        if (pfunc)       
        {
            pfunc(cursor, ap);
        }
        else {
            // no func found, assume string
            text = va_arg(ap, char*);
            if (text && *text)
            {
                unsigned len = strlen(text);
                while (len && text[len-1] == '\n') len--;
                QString qtext = QString().fromUtf8(text,len);
                QTextCharFormat format = FormatTag[qtag];
                cursor.insertText(qtext, format);
            }
        }
        
        if (padding & PAD_RIGHT)
        {
            cursor.insertText(" ");
        }
        
        // hilight some of the inserted text if requested so.
        if (padding & PAD_HILIGHT)
        {
            //TODO : should we send a message on the tray icon also ?
         //   ui_chat_buffer_check_for_high(buffer, &citer, &iter);
        }
            //TODO : me wonders if hyperlinks are not detected automatically
       // ui_chat_buffer_check_for_url(buffer, &citer, &iter);        
    }
}

gboolean ChatPage::ui_chat_page_printable(chat_page_t* page)
{
    ChatGui *data;
    FIND_CHAT_GUI(page, data);
    return (data && data->no_views > 0 );
}

void ChatPage::ui_chat_switch_page(chat_page_t* page) {
    ChatGui *data;
    ChatPage *chat_page;
    FIND_CHAT_GUI(page, data);
    Q_ASSERT(data);
    FIND_CHAT_PAGE(chat_page);
    
    chat_page->chat_tabs->setCurrentIndex(chat_page->chat_tabs->indexOf(data));

    LopsterApp *l_app = qobject_cast<LopsterApp *>(QCoreApplication::instance());
    l_app->ui_app_get_main_window()->ui_main_swith_to_page(QT_CHAT_PAGE);
}
