#ifndef _UI_CHAT_H
#define _UI_CHAT_H

#include <QObject>
#include <QtGui> 
#include <QKeyEvent>
#include <QHash>
#include "lcore/l_time.h"
#include "lcore/chat.h"
#include "lcore/preferences.h"
#include "lcore/commands.h"
#include "lcore/l_base.h"
#include "lcore/utils.h"
#include "lcore/plugin.h"
#include "lcore/string_list.h"
#include "lcore/mtypes.h"
#include "lcore/l_i18n.h"
#include "LTabWidget.hpp"
#include "CustomWidgets.hpp"
#include "ui_utils.hpp"


typedef void (*PrintFunc)(const QTextCursor, va_list);

class ChatLineEntry : public QLineEdit
{
    Q_OBJECT
public:
    ChatLineEntry(QWidget *parent=0);
private slots:
    void on_return( );
    void on_text_changed(const QString &text);
protected:
    void keyPressEvent ( QKeyEvent * event );
private:
    bool monitor_changes;
};

#include "widgets/ChatPage.h"
#include "widgets/ChatGui.h"
class ModuleWidget;
// Don't want to have so many text view and ugly display.
// create more pages if more text view is needed
#define MAX_TEXT_VIEW   5
class QT4_UI_EXPORT ChatGui : public QWidget, private Ui::ChatGui
{
    Q_OBJECT
public:
    ChatGui(QWidget *parent=0);
    QWidget *ui_chat_gui_get_body() {
        return widget_body;
    }
    void ui_chat_gui_set_icon(const QString &res) { 
        QPixmap pix; pix.load(res);
        this->label_icon->setPixmap(pix);
        this->label_icon->setText(QString(""));
    }
    void ui_chat_gui_set_title(const QString &title) {
        this->label_title->setText(title);
    }
    // declare all static printing functions
    static void print_ts(QTextCursor cursor);
    static void ui_chat_print_client(QTextCursor cursor, va_list ap);
    static void ui_chat_print_server(QTextCursor cursor, va_list ap);
    static void ui_chat_print_join(QTextCursor cursor, va_list ap);
    static void ui_chat_print_part(QTextCursor cursor, va_list ap);
    static void ui_chat_print_other(QTextCursor cursor, va_list ap);
    static void ui_chat_print_proto(QTextCursor cursor, va_list ap);
    static void ui_chat_print_nl(QTextCursor cursor, va_list ap);
    static QColor _get_nick_color(const char *nick, int mode);
    static void _set_colored(QTextCharFormat format, const char* tag_name, int* setting);
    static QTextCharFormat _get_nick_tag( const char* nick);
    static void ui_chat_print_nick_text(QTextCursor cursor, va_list ap);
    static void ui_chat_print_nick(QTextCursor cursor, va_list ap);
    static void ui_chat_print_url(QTextCursor cursor, va_list ap);

    QTextCursor ui_chat_create_cursor_from_view( int index );
    QTextEdit *view[MAX_TEXT_VIEW]; 
    QHash <int, QTextCursor> cursors;
    int no_views;  
};

class QT4_UI_EXPORT ChatPage : public LWidget, private Ui::ChatPage
{
   Q_OBJECT
public:
    ChatPage(QWidget *parent = 0);
    ~ChatPage();
    void loadCoreData();
    void ui_chat_define_default_text_color( QTextCharFormat format);
    void ui_chat_define_default_background_color(QTextCharFormat format );
    void ui_chat_define_default_font_format(QTextCharFormat format);
    void ui_chat_define_default_underline_format(QTextCharFormat format );
    void ui_chat_update_style(const QString style);
    static void on_timestamp_changed(pref_node_t* node );
    void ui_chat_define_timestamp(char* pref);
    static void ui_chat_register_print_func(const QString name, PrintFunc func);
    void ui_chat_register_tag_from_style(const QString name, const QString style);
    void ui_chat_init();

    static void ui_chat_create_simple_page(chat_page_t* page);
    static void ui_chat_page_update(chat_page_t* page);
    static void ui_chat_print(chat_page_t* page, int index, va_list ap);
    static gboolean ui_chat_page_printable(chat_page_t* page);
    static void ui_chat_switch_page(chat_page_t* page);
    static void ui_chat_page_add(chat_page_t* page, ChatGui* data);
    static void s_ui_chat_page_tab_remove(ChatGui *gui);
    static void s_ui_chat_page_remove(chat_page_t* page);

    static QTextCharFormat ui_chat_get_format_from_tag(QString tag);
private slots:
    void ui_chat_page_close_page(int);
    void ui_chat_current_page_changed(int);
private:
    QVBoxLayout *chat_layout;
    LTabWidget  *chat_tabs;
    QToolButton *close_button;
    int current_tab;
};

#endif //_UI_CHAT_H
