/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.19 $
 * $Date: 2007/11/13 09:47:16 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QSortFilterProxyModel>

#include "lcore/l_i18n.h"
#include "ui_main.hpp"
#include "ui_support.hpp"
#include "ui_filetree.hpp"

static QIcon share_none;
static QIcon share_part;
static QIcon share_all;

#define FIND_FILETREE(t,qt) \
            char* tname = (char*) object_get_data(OBJ(t), "FileTreeWidget"); \
            qt =  qobject_cast<FileTreeWidget *> (findWidgetByName(QString("%1").arg(tname))); 


class NodeItem : public QStandardItem
{
public:
    NodeItem(const QString &text)
        : QStandardItem(text) {}
    NodeItem(const QIcon &icon, const QString &text)
        : QStandardItem(icon,text) {}
    ~NodeItem() {
        object_free_data(OBJ(m_node), "FileNodeEntry", NULL);    
    }
    void set_node(file_node_t* node) {
        this->m_node = node;
        this->setData(QVariant::fromValue((void*)node));
        object_set_data(OBJ(node), "FileNodeEntry", this);
    }
    file_node_t* node() {
        return this->m_node;
    }
    void set_tree(file_tree_t* tree) {
        this->m_tree = tree;
    }
    file_tree_t* tree() {
        return this->m_tree;
    }
private:
    file_node_t* m_node;
    file_tree_t* m_tree;
};

FoldersModelItem::FoldersModelItem(QObject *parent)
    : QStandardItemModel(0,1,parent)
{
    setHeaderData(0, Qt::Horizontal, qtr("Folder"));    
}

void FoldersModelItem::folders_node_add(file_node_t* node, file_tree_t* tree)
{
    if (static_cast<NodeItem *>(object_get_data(OBJ(node), "FileNodeEntry")))
    {
        return;
    }
    
    NodeItem *parent;
    NodeItem *folder;

    QIcon icon = FileTreeWidget::ui_node_get_pixbuf(tree, node);

    if (!node->parent) parent = 0;
    else if (!(parent = static_cast<NodeItem *> 
            (object_get_data(OBJ(node->parent), "FileNodeEntry")))) return;

    folder = new NodeItem(icon,qfu(OBJ(node)->name));
    
    folder->set_node(node);
    folder->set_tree(tree);

    if (!parent) {
        appendRow(folder);
    }
    else {
        parent->appendRow(folder);
    }
    folders_node_update(node, tree);
}

void FoldersModelItem::folders_node_update(file_node_t* node, file_tree_t* tree)
{
    NodeItem *folder;
    folder = static_cast<NodeItem *> (object_get_data(OBJ(node), "FileNodeEntry"));
    if (!folder)
    {
        return;
    }

    // Set default data (for right action menu)
    // folder->setData(QVariant::fromValue((void*)node));

    QIcon icon = FileTreeWidget::ui_node_get_pixbuf(tree, node);
    folder->setText(qfu(OBJ(node)->name));
    folder->setIcon(icon);

    // TODO : Background color
    /* gtk_tree_store_set(data->folder_tree, iter,
       2, pix,
       3, (node->fci[tree->show_media][2]>0)?TRUE:FALSE,
      -1); */
}

void FoldersModelItem::folders_node_delete(file_node_t* node, file_tree_t* tree)
{
    NodeItem *folder;

    folder = static_cast< NodeItem *>(object_get_data(OBJ(node), "FileNodeEntry"));
    if (!folder) return;

    QModelIndex model_index = indexFromItem(folder);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    removeRow(row);  
}

void FoldersModelItem::folders_node_clear()
{
    removeRows(1,rowCount()); 
}


FilesModelItem::FilesModelItem(QObject *parent)
    : QStandardItemModel(0,3,parent)
{
    QStringList headers;
    headers << qtr("Filename") << qtr("Filesize") << qtr("Meta");
    setHorizontalHeaderLabels(headers); 
}

void FilesModelItem::files_node_add(file_node_t* node, file_tree_t* tree)
{
    QList<QStandardItem *> items;

    if (static_cast<NodeItem *>(object_get_data(OBJ(node), "FileNodeEntry")))
    {
        return;
    }
    NodeItem *file;
    QStandardItem *size;
    QStandardItem *meta;

    QIcon icon = FileTreeWidget::ui_node_get_pixbuf(tree, node);

    file = new NodeItem(icon,qfu(OBJ(node)->name));
    size = new QStandardItem(qfu(print_filesize(&(node->file->size))));
    meta = new QStandardItem(qfu(attr_print_all(ATTRC(node->file))));
    
    file->set_node(node);

    items.append(file);
    items.append(size);
    items.append(meta);
    // Add the items
    appendRow(items);

    files_node_update(node,tree);
}

void FilesModelItem::files_node_update(file_node_t* node, file_tree_t* tree)
{
    NodeItem *file;
    QStandardItem *size;
    QStandardItem *meta;
    file = static_cast<NodeItem *> (object_get_data(OBJ(node), "FileNodeEntry"));
    if (!file)
        return;

    QModelIndex model_index = indexFromItem(file);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    
    
    // We have the row, let get all the items and update them
    size = item(row, 1); 
    meta = item(row, 2); 

    QIcon icon = FileTreeWidget::ui_node_get_pixbuf(tree, node);
    file->setIcon(icon);
    file->setText(qfu(OBJ(node)->name));
    size->setText(qfu(print_filesize(&(node->file->size))));
    meta->setText(qfu(attr_print_all(ATTRC(node->file))));
}

void FilesModelItem::files_node_delete(file_node_t* node, file_tree_t* tree)
{
    NodeItem *file;

    file = static_cast< NodeItem *>(object_get_data(OBJ(node), "FileNodeEntry"));
    if (!file) return;

    QModelIndex model_index = indexFromItem(file);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    removeRow(row);  
}

void FilesModelItem::files_node_clear()
{
    removeRows(0,rowCount()); 
}

void FolderListTree::initTree(QSortFilterProxyModel *filter)
{
    setModel(filter);
    folders_model = new FoldersModelItem(this);
    // setSourceModel
    filter->setSourceModel(folders_model);
    setSortingEnabled(true);
    sortByColumn(0, Qt::AscendingOrder);
    ui_treeview_set_model(folders_model,false); // for internal popup menu    
    connect(this, SIGNAL(clicked(const QModelIndex &)),
            this, SLOT(folder_tree_selected(const QModelIndex &)));
}

void FolderListTree::clearTree()
{
    folders_model->folders_node_clear();
}

void FolderListTree::folder_tree_selected(const QModelIndex &index)
{
    if (!index.isValid()) return;
    QSortFilterProxyModel *proxy_model = static_cast< QSortFilterProxyModel *>(model());
   
    NodeItem *item = static_cast< NodeItem * >
                    (folders_model->itemFromIndex(proxy_model->mapToSource(index)));
    if (!item) return;
    file_tree_t* tree = item->tree();
    file_node_t* node = item->node();

    if (node && tree) {
        FileTreeWidget* tree_widget; 
        FIND_FILETREE(tree,tree_widget);
        tree_widget->ui_filetree_hide_childs(tree);
        tree_widget->ui_filetree_show_childs(tree, node);
    }
}

void FileListTree::initTree(QSortFilterProxyModel *filter)
{
    setModel(filter);
    files_model = new FilesModelItem(this);   
    filter->setSourceModel(files_model);
    setSortingEnabled(true);
    sortByColumn(1, Qt::AscendingOrder);
    ui_treeview_set_model(files_model,false);
}


void FileListTree::clearTree()
{
    files_model->files_node_clear();
}


FileTreeWidget::FileTreeWidget(QWidget* parent)
    : QWidget(parent)
{
	setupUi(this);

    current_tree = 0;
    current_node = 0;

    // Init the Trees
    proxy_folders =  new QSortFilterProxyModel(this);
    proxy_files = new QSortFilterProxyModel(this);
    proxy_folders->setDynamicSortFilter(true);
    proxy_files->setDynamicSortFilter(true);

    folders_tree->initTree(proxy_folders);
    files_tree->initTree(proxy_files);

    // FIXME : change those ugly arrow to nicer ones
    tool_menu1->setIcon(QIcon(QString::fromUtf8(":/downarrow.png")));
    tool_menu2->setIcon(QIcon(QString::fromUtf8(":/downarrow.png")));

    //TODO : handle save layout on the splitter
    splitter_filetree->setStretchFactor(1, 2);

    connect(push_cancel, SIGNAL(clicked()),this,SLOT(filetree_cancel_scanning()));
    connect(line_search, SIGNAL(textChanged(QString)),this,SLOT(text_filter_changed(QString)));
}

FileTreeWidget::~FileTreeWidget()
{
    folders_tree->clearTree();
    files_tree->clearTree(); 
}

const QString FileTreeWidget::MNames(file_tree_t* tree) {
  const char* fill;
  int num = tree->fci[tree->show_media][tree->show_time];

  if (tree->show_time == 0) fill = _(" unshared");
  else if (tree->show_time == 1) fill = _(" shared");
  else if (tree->show_time == 2) fill = _(" new");
  else fill = "";

  if (tree->show_time == 0) {
    switch (tree->show_media) {
    case MEDIA_FOLDER:
      return qtr(ngettext("%1 unshared folder", "%1 unshared folders", num)).arg(num);
    case MEDIA_AUDIO:
      return qtr(ngettext("%1 unshared audio file", "%1 unshared audio files", num)).arg(num);
    case MEDIA_VIDEO:
      return qtr(ngettext("%1 unshared video file", "%1 unshared video files", num)).arg(num);
    case MEDIA_APPLICATION:
      return qtr(ngettext("%1 unshared application", "%1 unshared applications", num)).arg(num);
    case MEDIA_IMAGE:
      return qfu(ngettext("%1 unshared image", "%1 unshared images", num)).arg(num);
    case MEDIA_DOCUMENT:
      return qtr(ngettext("%1 unshared document", "%1 unshared documents", num)).arg(num);
    case MEDIA_UNKNOWN:
      return qtr(ngettext("%1 unshared other file", "%1 unshared other files", num)).arg(num);
    default:
      return qtr(ngettext("%1 unshared file", "%1 unshared files", num)).arg(num);
    };
  } else if (tree->show_time == 1) {
    switch (tree->show_media) {
    case MEDIA_FOLDER:
      return qtr(ngettext("%1 shared folder", "%1 shared folders", num)).arg(num);
    case MEDIA_AUDIO:
      return qtr(ngettext("%1 shared audio file", "%1 shared audio files", num)).arg(num);
    case MEDIA_VIDEO:
      return qtr(ngettext("%1 shared video file", "%1 shared video files", num)).arg(num);
    case MEDIA_APPLICATION:
      return qtr(ngettext("%1 shared application", "%1 shared applications", num)).arg(num);
    case MEDIA_IMAGE:
      return QString(ngettext("%1 shared image", "%1 shared images", num)).arg(num);
    case MEDIA_DOCUMENT:
      return qtr(ngettext("%1 shared document", "%1 shared documents", num)).arg(num);
    case MEDIA_UNKNOWN:
      return qtr(ngettext("%1 shared other file", "%1 shared other files", num)).arg(num);
    default:
      return qtr(ngettext("%1 shared file", "%1 shared files", num)).arg(num);
    };
  } else if (tree->show_time == 2) {
    switch (tree->show_media) {
    case MEDIA_FOLDER:
      return qtr(ngettext("%1 new folder", "%1 new folders", num)).arg(num);
    case MEDIA_AUDIO:
      return qtr(ngettext("%1 new audio file", "%1 new audio files", num)).arg(num);
    case MEDIA_VIDEO:
      return qtr(ngettext("%1 new video file", "%1 new video files", num)).arg(num);
    case MEDIA_APPLICATION:
      return qtr(ngettext("%1 new application", "%1 new applications", num)).arg(num);
    case MEDIA_IMAGE:
      return qtr(ngettext("%1 new image", "%1 new images", num)).arg(num);
    case MEDIA_DOCUMENT:
      return qtr(ngettext("%1 new document", "%1 new documents", num)).arg(num);
    case MEDIA_UNKNOWN:
      return qtr(ngettext("%1 new other file", "%1 new other files", num)).arg(num);
    default:
      return qtr(ngettext("%1 new file", "%1 new files", num)).arg(num);
    };
  } else {
    switch (tree->show_media) {
    case MEDIA_FOLDER:
      return qtr(ngettext("%1 folder", "%1 folders", num)).arg(num);
    case MEDIA_AUDIO:
      return qtr(ngettext("%1 audio file", "%1 audio files", num)).arg(num);
    case MEDIA_VIDEO:
      return qtr(ngettext("%1 video file", "%1 video files", num)).arg(num);
    case MEDIA_APPLICATION:
      return qtr(ngettext("%1 application", "%1 applications", num)).arg(num);
    case MEDIA_IMAGE:
      return qtr(ngettext("%1 image", "%1 images", num)).arg(num);
    case MEDIA_DOCUMENT:
      return qtr(ngettext("%1 document", "%1 documents", num)).arg(num);
    case MEDIA_UNKNOWN:
      return qtr(ngettext("%1 other file", "%1 other files", num)).arg(num);
    default:
      return qtr(ngettext("%1 file", "%1 files", num)).arg(num);
    };
  }
}

const QString FileTreeWidget::TNames(int no) {
    switch (no) {
        case 0:  return qtr("Show unshared");
        case 1:  return qtr("Show shared");
        case 2:  return qtr("Show new");
        default: return qtr("Show all");
    };
}

void FileTreeWidget::set_time(QAction *action) {
    int num = action->data().toInt();
    current_tree->show_time = num;
    object_set_dirty(OBJ(current_tree));
}

QMenu *FileTreeWidget::ui_file_tree_menu1(QWidget *parent) {
    QMenu *popup;
    QAction *item;
    int i2;
    file_tree_t* tree = current_tree;

    popup = new QMenu(parent);

    for (i2 = 0; i2 <= 3; i2++) {
        const QString m = TNames(i2);
        QString l =  qtr(ngettext("%1 item", "%1 items", tree->fci[tree->show_media][i2]));

        if (i2 == 3) {
          popup->addSeparator();
        }

        QString sl = l.arg(tree->fci[tree->show_media][i2]);
        item =  new QAction(QString("%1 (%2)").arg(m, sl), parent);
        item->setData(QVariant(i2));
        popup->addAction(item);
        if (tree->fci[tree->show_media][i2] == 0)
            item->setEnabled(false);       
    }
    connect(popup, SIGNAL(triggered(QAction *)), this, SLOT(set_time(QAction *)));
    return popup;
}

void FileTreeWidget::set_mime(QAction *action) {
    int num = action->data().toInt();
    current_tree->show_media = num;
    object_set_dirty(OBJ(current_tree));
}

QMenu *FileTreeWidget::ui_file_tree_menu2(QWidget *parent) {
    QMenu *popup;
    QAction *item;
    int i1;
    file_tree_t* tree = current_tree;

    popup = new QMenu(parent);

    for (i1 = 0; i1 <= MEDIA_ANY; i1++) {
        const QString m = qtr(int2media(i1, 1));
        QString l = qtr(ngettext("%1 item", "%1 items", tree->fci[i1][tree->show_time]));

        if (i1 == MEDIA_ANY) {
            popup->addSeparator();
        }

        QString sl = l.arg(l, tree->fci[i1][tree->show_time]);
        item = new QAction(QIcon(UiSupport::GetMediaStock(i1)), QString("%1 (%2)").arg(m, sl), parent); 
        item->setData(QVariant(i1));
        popup->addAction(item);
        if (tree->fci[i1][tree->show_time] == 0)
            item->setEnabled(false);
    }
    connect(popup, SIGNAL(triggered(QAction *)), this, SLOT(set_mime(QAction *)));

    return popup;
}

/** gets pixmap for node */
QIcon FileTreeWidget::ui_node_get_pixbuf(file_tree_t* tree, file_node_t* node ) {
    QIcon pix;
    if (share_none.isNull()) share_none = QIcon(":/share_none.png");
    if (share_part.isNull()) share_part = QIcon(":/share_part.png");
    if (share_all.isNull())  share_all  = QIcon(":/share_all.png"); 

    if (node->fci[tree->show_media][1] == 0) {
        pix = share_none;
    } else if (tree->show_time == 0) {
        pix = share_none;
    } else if (tree->show_time == 1) {
        pix = share_all;
    } else if ( node->fci[tree->show_media][1] <
                node->fci[tree->show_media][3]) {
        pix = share_part;
    } else {
        pix = share_all;
    }
    return pix;
}

void FileTreeWidget::text_filter_changed(QString text) {
     // FIXME : add Checkbox for sensitive/insensitive search
    QRegExp reg_exp(text, Qt::CaseInsensitive, QRegExp::RegExp);
    proxy_folders->setFilterRegExp(reg_exp);
    proxy_files->setFilterRegExp(reg_exp);    
}

void FileTreeWidget::ui_filetree_hide_folder(file_tree_t* tree)
{
    folders_tree->folders_model->folders_node_clear();
}


void FileTreeWidget::ui_filetree_hide_childs(file_tree_t* tree)
{
    files_tree->clearTree();
    current_node = NULL;
}

void FileTreeWidget::ui_filetree_show_childs(file_tree_t* tree, file_node_t* node)
{
    file_node_t* child;
    
    if ( !node) return;

    child = node->child;
    while (child)
    {
        if (child->file && child->fci[tree->show_media][tree->show_time]) {
            ui_filetree_node_show(child, tree);
        }
        child = child->next;
    }
    current_node = node;
}

void FileTreeWidget::ui_filetree_node_update(file_node_t* node, file_tree_t* tree)
{    
    node->flags &= ~FN_FLAG_UPDATE;       
    if (node->file)
    {
        files_tree->files_model->files_node_update(node,tree);
    }
    else
    {
        folders_tree->folders_model->folders_node_update(node,tree);
    }
}

void FileTreeWidget::ui_filetree_node_show(file_node_t* node, file_tree_t* tree) { 
    if (node->file)
    {
        files_tree->files_model->files_node_add(node,tree);  
    }
    else if (!node->file)
    {
        folders_tree->folders_model->folders_node_add(node,tree);         
    }
    object_notify_d(OBJ(node), (ObjectFunc)s_ui_filetree_node_hide, tree);
}

void FileTreeWidget::s_ui_filetree_node_hide(file_node_t* node, file_tree_t* tree)
{
    FileTreeWidget* tree_widget = NULL; 
    FIND_FILETREE(tree,tree_widget);
    if (tree_widget)
    {
        tree_widget->ui_filetree_node_hide(node, tree);
    }
}

void FileTreeWidget::ui_filetree_node_hide(file_node_t* node, file_tree_t* tree)
{
    object_unnotify_d(OBJ(node), (ObjectFunc)s_ui_filetree_node_hide, tree);
    
    // and hide the node
    if (node->file && files_tree )
    {
        files_tree->files_model->files_node_delete(node,tree);  
    }
    else if (!node->file  && folders_tree )
    {
        folders_tree->folders_model->folders_node_delete(node,tree);
    }
}


void FileTreeWidget::filetree_cancel_scanning()
{
    if (abort_func)
    {
        abort_func( abort_data );
    }
}

void FileTreeWidget::ui_file_tree_show_bottom(int timeout)
{
}

void FileTreeWidget::s_ui_file_tree_update_label(file_tree_t* tree, const char* label)
{
    FileTreeWidget *tree_widget ; 
    FIND_FILETREE(tree,tree_widget); 
    if (!tree_widget) return;
    tree_widget->label_filetree_scanning->setText(label?qfu(label):"");
    tree_widget->ui_file_tree_show_bottom(30);
}

void FileTreeWidget::s_ui_file_tree_set_progress(file_tree_t* tree, double progress)
{
    FileTreeWidget *tree_widget ; 
    FIND_FILETREE(tree,tree_widget); 
    if (!tree_widget) return;
    double percentage = progress * 100.;
    int per = (int) percentage;
    tree_widget->progress_scan->setValue(per>100?100:per);   
    tree_widget->ui_file_tree_show_bottom( 10);
}

void FileTreeWidget::s_ui_file_tree_set_progress_name(file_tree_t* tree, const char* name)
{
    FileTreeWidget *tree_widget ; 
    FIND_FILETREE(tree,tree_widget); 
    if (!tree_widget) return;
    /*tree_widget->progress_filetree_scanning->setText(name?qfu(name):"");   */
    tree_widget->ui_file_tree_show_bottom(30);
    
}

void FileTreeWidget::s_ui_file_tree_set_abort(file_tree_t* tree, file_tree_func func, void* user_data)
{
    // connect from constructor the cancel button 
    // to a cancel function that will check for an abort_func and abord_data
    // will call in consequence

    FileTreeWidget *tree_widget ; 
    FIND_FILETREE(tree,tree_widget); 
    if (!tree_widget) return;

    tree_widget->abort_func = func;
    tree_widget->abort_data = user_data;
    
    if (!tree_widget->abort_func)
    {
        tree_widget->push_cancel->setEnabled(false);
    }
    else
    {
        tree_widget->push_cancel->setEnabled(true);
    }
}

void FileTreeWidget::node_vis_action(file_tree_t* tree, file_node_t* node)
{
    file_node_t* child;
    int sm = tree->show_media;
    int st = tree->show_time;
    bool hide = false;

    if (node)
    {
        if (object_get_data(OBJ(node), "FileNodeEntry"))
        {
            if (!node->file && !node->fci[sm][st])
                hide = true;
            else if (node->file && !node->fci[sm][st] && current_node == node->parent)
                hide = true;
            else 
            if (node->flags & FN_FLAG_UPDATE) {
                ui_filetree_node_update(node, tree);
            }
        }
        else
        {            
            if (!node->file && node->fci[sm][st]) {
                ui_filetree_node_show(node, tree);
            }
            else if (node->file && node->fci[sm][st] && current_node == node->parent) {
                ui_filetree_node_show(node, tree);
            }

        }
        child = node->child;
    }
    else
    {
        child = tree->files;
    }
    
    while (child)
    {
        file_node_t* temp = child->next;
        node_vis_action(tree, child);
        child = temp;
    }

    if (hide) ui_filetree_node_hide(node, tree);
}

void FileTreeWidget::s_ui_filetree_update(file_tree_t* tree)
{
    FileTreeWidget *tree_widget ; 
    FIND_FILETREE(tree,tree_widget); 
    tree_widget->ui_filetree_update(tree);
}

void FileTreeWidget::ui_filetree_update(file_tree_t* tree)
{
    node_vis_action(tree, NULL);
   
    if (label_summary)
    {
        const QString m = MNames(current_tree);
        // display summary for library/browses, example: 5 items - 4.3 MiB
        QString str = QString("<b>%1 - %2</b>").arg(m).arg(
                             print_filesize(&(current_tree->fsi[current_tree->show_media][current_tree->show_time])));
        label_summary->setText(str);
    }
}

void FileTreeWidget::ui_filetree_hide(file_tree_t* tree )
{
    ui_filetree_hide_folder(tree);
    ui_filetree_hide_childs(tree);

    object_free_data(OBJ(tree), "FileTreeWidget", NULL);
    current_node = NULL;
    object_unmonitor_d(OBJ(tree), (ObjectFunc)s_ui_filetree_update, NULL);
}

void FileTreeWidget::ui_filetree_show( file_tree_t* tree )
{
    ui_filetree_update(tree);
    object_monitor_d(OBJ(tree), (ObjectFunc)s_ui_filetree_update, NULL);
}

void FileTreeWidget::ui_filetree_expand(file_tree_t* tree, file_node_t* node)
{
    /*    if (node)
    {
    GtkTreeIter* iter = object_get_data(OBJ(node), "FileNodeEntry");
        GtkTreePath* path;
        
        if (!iter) return;
        path = gtk_tree_model_get_path(GTK_TREE_MODEL(data->folder_tree), iter);
        if (path)
        {
            gtk_tree_view_expand_row(data->folder_view, path, TRUE);
            gtk_tree_path_free(path);
        }  
    }
    else
    {
        if (folders_tree->topLevelItem(0))
        {
            folders_tree->expandItem(folders_tree->topLevelItem(0));
        }

    }
   */
}

void FileTreeWidget::ui_filetree_assign_new_tree(file_tree_t* tree)
{
    if (current_tree == tree) return;
    if (current_tree) {
        ui_filetree_hide(current_tree);
    }
    if (tree)
    {
        char* object_name;
        STORE_OBJECT("FileTreeWidget",object_name,OBJ(tree),this);
        current_tree = tree;
        tool_menu1->setMenu(ui_file_tree_menu1(tool_menu1));
        tool_menu2->setMenu(ui_file_tree_menu2(tool_menu2));
        ui_filetree_show(tree);
        ui_filetree_expand(tree, NULL);
    }
}

void FileTreeWidget::ui_assign_folders_actions(unsigned num_actions, action_item_t** actions, void* user_data)
{
    folders_tree->ui_tv_handle_button(num_actions, actions, user_data);
}

void FileTreeWidget::ui_assign_files_actions(unsigned num_actions, action_item_t** actions, void* user_data)
{
    files_tree->ui_tv_handle_button(num_actions, actions, user_data);
}
