#ifndef _UI_FILETREE_H_
#define _UI_FILETREE_H_

#include <QStandardItemModel>
#include "lcore/file_tree.h"
#include "ui_treeview.hpp"

class QSortFilterProxyModel;

class FoldersModelItem : public QStandardItemModel
{
public:
    FoldersModelItem(QObject *parent=0);
    void folders_node_add(file_node_t* node, file_tree_t* tree);
    void folders_node_update(file_node_t* node, file_tree_t* tree);
    void folders_node_delete(file_node_t* node, file_tree_t* tree);
    void folders_node_clear();
};

class FilesModelItem : public QStandardItemModel
{
public:
    FilesModelItem(QObject *parent=0);
    void files_node_add(file_node_t* node, file_tree_t* tree);
    void files_node_update(file_node_t* node, file_tree_t* tree);
    void files_node_delete(file_node_t* node, file_tree_t* tree);
    void files_node_clear();
};

class FolderListTree : public UiTreeView
{
    Q_OBJECT
public:
    FolderListTree(QWidget *parent) : UiTreeView(parent) {}
    void initTree(QSortFilterProxyModel *filter);
    void clearTree();
    FoldersModelItem *folders_model;
private slots:
    void folder_tree_selected(const QModelIndex &);
};

class FileListTree : public UiTreeView
{
public:
    FileListTree(QWidget *parent) : UiTreeView(parent) {}
    void initTree(QSortFilterProxyModel *filter);
    void clearTree();
    FilesModelItem *files_model;
};

#include "widgets/FileTreeWidget.h"

typedef void (*file_tree_func)(void* data);


class FileTreeWidget : public QWidget, private Ui::FileTreeWidget
{
    Q_OBJECT
public:
    FileTreeWidget( QWidget *parent = 0);
    ~FileTreeWidget();
    void ui_filetree_assign_new_tree( file_tree_t* tree);
    void ui_filetree_hide(file_tree_t* tree);
    void ui_filetree_show(file_tree_t* tree);
    void ui_filetree_expand(file_tree_t* tree, file_node_t* node);
    void ui_filetree_update(file_tree_t* tree);
    static void s_ui_filetree_update(file_tree_t* tree);
    void node_vis_action(file_tree_t* tree, file_node_t* node);
    void ui_filetree_node_hide(file_node_t* node, file_tree_t* tree);
    static void s_ui_filetree_node_hide(file_node_t* node, file_tree_t* tree);
    void ui_filetree_node_show(file_node_t* node, file_tree_t* tree);
    void ui_filetree_node_update(file_node_t* node, file_tree_t* tree);
    void ui_filetree_show_childs(file_tree_t* tree, file_node_t* node);
    /*  void ui_unset_items(NodeTreeItem* item );
    void ui_unset(NodeTreeItem* item ); */
    //void ui_unset_folders(NodeTreeItem* item );
    static QIcon ui_node_get_pixbuf(file_tree_t* tree, file_node_t* node );
    void ui_filetree_hide_folder(file_tree_t* tree);
    void ui_filetree_hide_childs(file_tree_t* tree);
    void ui_assign_folders_actions(unsigned num_actions, action_item_t** actions, void* user_data);
    void ui_assign_files_actions(unsigned num_actions, action_item_t** actions, void* user_data);
    void ui_file_tree_show_bottom(int timeout);
    
    const QString TNames(int no);
    QMenu *ui_file_tree_menu1(QWidget *parent);
    QMenu *ui_file_tree_menu2(QWidget *parent);

    static void s_ui_file_tree_update_label(file_tree_t* tree, const char* label);
    static void s_ui_file_tree_set_progress(file_tree_t* tree, double progress);   
    static void s_ui_file_tree_set_progress_name(file_tree_t* tree, const char* name);
    static void s_ui_file_tree_set_abort(file_tree_t* tree, file_tree_func func, void* user_data);

    const QString MNames(file_tree_t* tree);

  //  static file_node_t* search_cb(file_tree_t* tree, file_node_t* node, FileTreeWidget *data);

    QString search_token;
    int search_timeout;
    file_tree_t* current_tree;
    file_node_t* current_node;   
    file_tree_func abort_func;
    void* abort_data;
private slots:
  /*  void search_files(); */
    void text_filter_changed(QString);
    void set_mime(QAction *action);
    void set_time(QAction *action);
    void filetree_cancel_scanning();
private:

    QSortFilterProxyModel *proxy_folders;
    QSortFilterProxyModel *proxy_files;
};

#endif // _UI_FILETREE_H_
