/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.14 $
 * $Date: 2007/11/05 14:32:02 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QtGui>
#include <QObject>

#include "lcore/l_base.h"
#include "lcore/xml_handler.h"
#include "lcore/files.h"
#include "lcore/mtypes.h"
#include "lcore/file_tree.h"
#include "lcore/utils.h"
#include "lcore/preferences.h"
#include "lcore/attribute.h"
#include "lcore/search.h"
#include "lcore/plugin.h"
#include "lcore/action_item.h"
#include "lcore/thread.h"
#include "lcore/ui.h"
#include "lcore/exec.h"
#include "lcore/l_i18n.h"
#include "lcore/open_actions.h"


#include "ui_main.hpp"
#include "ui_style.hpp"
#include "ui_utils.hpp"
#include "ui_filetree.hpp"
#include "ui_library.hpp"

LibraryPage::LibraryPage(QWidget* parent)
    : LWidget(parent)
{
	setupUi(this);
    ready = false;
    current_lib = NULL;
    QPixmap plib; 
    plib.load(":/library.png");
    plib = plib.scaled(LOPSTER_LABEL_ICON,Qt::KeepAspectRatio);
    label_libpix->setPixmap(plib);
    
    lib_manager = new QMenu();

    ui_lib_create_manage_popup( );
    tool_rescan_lib->setIcon(QIcon(QString::fromUtf8(":/reload.png")));
    tool_save_lib->setIcon(QIcon(QString::fromUtf8(":/save.png")));
    connect(tool_rescan_lib, SIGNAL(clicked()), this, SLOT(ui_lib_on_library_rebuild()));
    connect(tool_save_lib, SIGNAL(clicked()), this, SLOT(ui_lib_on_library_save()));
    //connect(push_lib_addfolder, SIGNAL(clicked()), this, SLOT(on_add_folder()));
}

LibraryPage::~LibraryPage()
{
    if (lib_manager)
    {
        delete lib_manager;
    }
    if (current_lib)
    {
        ui_lib_destroy(current_lib);
    }
}

void LibraryPage::loadCoreData()
{
    ready = true;
    if (lib_to_load)
    {
        ui_lib_update(lib_to_load);
    }
}

void LibraryPage::s_ui_lib_working(gboolean val)
{
    LibraryPage *lib_page = getLibraryPage();
    if (val)
    {
        lib_page->tool_rescan_lib->setEnabled(false);
        lib_page->label_rescan_lib->setText(qtr("(re)scanning ..."));
        lib_page->tree_widget->s_ui_file_tree_set_abort(
                        FILE_TREE(lib_page->current_lib), (file_tree_func)library_abort_refresh, NULL);
    }
    else
    {
        lib_page->tool_rescan_lib->setEnabled(true);
        lib_page->label_rescan_lib->setText(qtr("Re(scan) library"));
        lib_page->tree_widget->s_ui_file_tree_set_abort(
                        FILE_TREE(lib_page->current_lib), NULL, NULL);
    }
}

void LibraryPage::ui_lib_on_library_save()
{
    // FIXME: check wether QMenu manage button has this new filename as action, else add it
    QString lib_name;
    
    lib_name = line_lib_name->text();
    if (lib_name.isEmpty() )
    {
        lopster_notify(LOPSTER_MESSAGE_INFO, _("Missing filename"),
                       _("Please enter a filename for the library!"));
        return;
    }
    library_save_as(qtu(lib_name));
    ui_lib_create_manage_popup();
}

void LibraryPage::ui_lib_on_library_rebuild() {
    library_refresh(TRUE);
}

void LibraryPage::ui_lib_on_add_folder( ){
    QString dir = QFileDialog::getExistingDirectory(this, tr("Share this folder"),
                                                 QDir::homePath(),
                                                 QFileDialog::ShowDirsOnly
                                                 | QFileDialog::DontResolveSymlinks);
    if (!dir.isEmpty())
    {
        // lopster handle automatically native sep, and
        // QFileDialog::getExistingDirectory uses "/" as sep
        library_add_shared(current_lib, qtu(QDir::toNativeSeparators(dir)));
        library_refresh(TRUE);
    }
}


void LibraryPage::ui_lib_on_delete_list(QAction *action){
    char *s1;
    s1 = qtu(action->text());
    library_delete_on_disk(s1);
    ui_lib_create_manage_popup();
}

void LibraryPage::ui_lib_on_load_list(QAction *action){
    const char* text = qtu(action->text());
    pref_store("/core/lib/current", text);
}

int LibraryPage::ui_lib_populate_load_delete_popup(QMenu *item_load, QMenu *item_delete)
{
    int cnt = 0;
    QDir dir;
    QString dirname = qfu("%1").arg(lopster_get_home()) + 
                            QString(QDir::separator()) + QString("share");
    dir.setPath(dirname);

    item_delete->clear(); 
    item_load->clear();
    if (!dir.exists(dirname)) {
        qWarning("could not open dir [%s]", qtu(dirname));
    } else {
        QStringList list = dir.entryList(QDir::Files);
        for (int i = 0; i < list.size(); ++i) {
            cnt++;
            QString lib_name = list.at(i);
            item_delete->addAction(lib_name);
            item_load->addAction(lib_name);    
        }
    }
    return cnt;
}

void LibraryPage::ui_lib_create_manage_popup( ) {
    const char* entry;
    if (lib_manager)
    {
        lib_manager->clear();
    }   
    QMenu *menu_delete = lib_manager->addMenu(qtr("Delete List"));
    QMenu *menu_load = lib_manager->addMenu(qtr("Load List"));
    int cnt = ui_lib_populate_load_delete_popup(menu_load,menu_delete);
    if (!cnt) {
        menu_delete->setEnabled(false);
        menu_load->setEnabled(false);
    } else {
        connect(menu_delete,SIGNAL(triggered(QAction *)), this, SLOT(ui_lib_on_delete_list(QAction *)));
        connect(menu_load,SIGNAL(triggered(QAction *)), this, SLOT(ui_lib_on_load_list(QAction *)));
    }     
    push_manage_lib->setMenu(lib_manager);
}

void LibraryPage::ui_lib_update(library_t* lib)
{
    if (!ready)
    {
        ui_lib_set_library_to_load(lib);
    }
    file_tree_t* tree = FILE_TREE(lib);
    if (current_lib == lib) return;   
    if (!lib) return;
    current_lib = lib;
    if (object_get_data(OBJ(lib), "FileTreeWidget")) return;   // already shown
    tree_widget->ui_filetree_assign_new_tree( tree);
    // additional lib signals
    action_item_t* actions = action_item_search("Library folder actions");
    tree_widget->ui_assign_folders_actions( 1, &actions, lib);
    action_item_t* actions2 = action_item_search("Library file actions");
    tree_widget->ui_assign_files_actions(1, &actions2, lib);    
    line_lib_name->setText(qfu(OBJ(tree)->name));
}

void LibraryPage::ui_lib_destroy(library_t* lib)
{
    tree_widget->ui_filetree_assign_new_tree(NULL);
}

LibraryPage* LibraryPage::getLibraryPage()
{
    return qobject_cast <LibraryPage *> (findWidgetByName("library_page"));
}

void LibraryPage::s_ui_lib_destroy(library_t* lib)
{
    LibraryPage *lib_page = NULL;
    lib_page = getLibraryPage();
    if (lib_page)
    {
        lib_page->ui_lib_destroy(lib);
    }
}

void LibraryPage::s_ui_lib_new(library_t* lib)
{   
    LibraryPage *lib_page = getLibraryPage(); 
    if (!lib_page) 
    {
        return; 
    }
    lib_page->ui_lib_update(lib);
    object_notify_d(OBJ(lib), (ObjectFunc)s_ui_lib_destroy, NULL);
}
