/* Copyright (C) 2007 The Lopster Team.
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.29 $
 * $Date: 2007/10/11 12:01:13 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QApplication>
#include <QtGui>

#include "lcore/l_base.h"
#include "lcore/plugin.h"
#include "lcore/library.h"
#include "lcore/search.h"
#include "lcore/buddy.h"
#include "lcore/preferences.h"
#include "lcore/l_base.h"
#include "lcore/thread.h"
#include "lcore/action_item.h"
#include "lcore/version.h"
#include "lcore/l_limit.h"
#include "lcore/string_list.h"
#include "lcore/l_i18n.h"

#include "ui_main.hpp"
#include "ui_network.hpp"
#include "ui_search.hpp"
#include "ui_transfer.hpp"
#include "ui_buddies.hpp"
#include "ui_chat.hpp"
#include "ui_library.hpp"
#include "ui_timer.hpp"
#include "ui_preferences.hpp"
#include "ui_style.hpp"
#include "upload_view.hpp"
#include "CustomWidgets.hpp"
#include "BandwidthBar.hpp"

extern "C"
{
    const char* ThePluginName = N_("Qt4\302\256 - Development Framework by TrollTech\302\256");
}

QHash <QAction *, QWidget *> actions_pages;

UiMain::UiMain(QWidget *parent)
    : LMainWindow(parent)
{
    setupUi(this);

    NetworkPage* network_page = new NetworkPage(mainPage);
    network_page->setObjectName(QString::fromUtf8("network_page"));
    vboxLayout1->addWidget(network_page);
    pages[QT_NETWORK_PAGE] = network_page;
    actions[QT_NETWORK_PAGE] = actionNetworks;
    actions_pages[actionNetworks] = network_page;
    last_visible = network_page;
    last_action = actionNetworks;

    SearchPage* search_page = new SearchPage(mainPage);
    search_page->setObjectName(QString::fromUtf8("search_page"));
    vboxLayout1->addWidget(search_page);
    pages[QT_SEARCH_PAGE] = search_page;
    actions[QT_SEARCH_PAGE] = actionSearch;
    actions_pages[actionSearch] = search_page;
    search_page->setVisible(false);

    TransferPage* transfer_page = new TransferPage(mainPage);
    transfer_page->setObjectName(QString::fromUtf8("transfer_page"));
    vboxLayout1->addWidget(transfer_page);
    pages[QT_TRANSFER_PAGE] = transfer_page;
    actions[QT_TRANSFER_PAGE] = actionTransfer;
    actions_pages[actionTransfer] = transfer_page;
    transfer_page->setVisible(false);

    BuddiesPage* buddies_page = new BuddiesPage(mainPage);
    buddies_page->setObjectName(QString::fromUtf8("buddies_page"));
    vboxLayout1->addWidget(buddies_page);
    pages[QT_BUDDIES_PAGE] = buddies_page;
    actions[QT_BUDDIES_PAGE] = actionBuddies;
    actions_pages[actionBuddies] = buddies_page;
    buddies_page->setVisible(false);

    ChatPage* chat_page = new ChatPage(mainPage);
    chat_page->setObjectName(QString::fromUtf8("chat_page"));
    vboxLayout1->addWidget(chat_page);
    pages[QT_CHAT_PAGE] = chat_page;
    actions[QT_CHAT_PAGE] = actionChat;
    actions_pages[actionChat] = chat_page;
    chat_page->setVisible(false);

    LibraryPage* library_page = new LibraryPage(mainPage);
    library_page->setObjectName(QString::fromUtf8("library_page"));
    vboxLayout1->addWidget(library_page);
    pages[QT_LIBRARY_PAGE] = library_page;
    actions[QT_LIBRARY_PAGE] = actionLibrary;
    actions_pages[actionLibrary] = library_page;
    library_page->setVisible(false); 

    dprogress = new BandwidthProgress(qtr("Down"));
    uprogress = new BandwidthProgress(qtr("Up"));

    statusBar()->addPermanentWidget(uprogress); 
    statusBar()->addPermanentWidget(dprogress); 

    // Update toolbar icons
    actionNetworks->setIcon(LopsterApp::ui_app_get_res_icon(":/server.png", LOPSTER_TOOLBAR_ICON));
    actionSearch->setIcon(LopsterApp::ui_app_get_res_icon(":/search.png", LOPSTER_TOOLBAR_ICON));
    actionTransfer->setIcon(LopsterApp::ui_app_get_res_icon(":/transfer.png", LOPSTER_TOOLBAR_ICON));
    actionBuddies->setIcon(LopsterApp::ui_app_get_res_icon(":/hotlist.png", LOPSTER_TOOLBAR_ICON));
    actionChat->setIcon(LopsterApp::ui_app_get_res_icon(":/chat.png", LOPSTER_TOOLBAR_ICON));
    actionLibrary->setIcon(LopsterApp::ui_app_get_res_icon(":/library.png", LOPSTER_TOOLBAR_ICON));
    actionPreferences->setIcon(LopsterApp::ui_app_get_res_icon(":/mtype_application.png", LOPSTER_TOOLBAR_ICON));
    

}

UiMain::~UiMain()
{
    writeSettings( "Main" );
}

void UiMain::run()
{
    for (int i=0;i<6;i++) // except preferences
    {
        pages[i]->loadCoreData();
    }


    dprogress->pbar->setVolume(volume_find("Download Bandwidth Limit"));
    dprogress->pbar->setRate(downloads_get_rate());

    connect(toolBar, SIGNAL(actionTriggered(QAction*)), this, SLOT(ui_main_toolbar_clicked(QAction*)));

    if( QSystemTrayIcon::isSystemTrayAvailable() /*&& TODO: Add a setting for systray in pref */) {
        ui_main_create_systray();
    }
    
    // Create timer
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(ui_main_update_timer()));
    timer->start(1000);

    readSettings( "Main", QSize( 600,700 ) );

    show();
}

void UiMain::ui_user_info(int type, const char* header, const char* text)
{
    QMessageBox::Icon icon;
    QString title;
       
    icon =
        (type==LOPSTER_MESSAGE_INFO)    ? QMessageBox::Information:
        (type==LOPSTER_MESSAGE_WARNING) ? QMessageBox::Warning :
        (type==LOPSTER_MESSAGE_QUESTION)? QMessageBox::Question:
        (type==LOPSTER_MESSAGE_ERROR)   ? QMessageBox::Critical:
        QMessageBox::Information;
    title =
        (type==LOPSTER_MESSAGE_INFO)    ?qtr("Information"):
        (type==LOPSTER_MESSAGE_WARNING) ?qtr("Warning")    :
        (type==LOPSTER_MESSAGE_QUESTION)?qtr("Question")   :
        (type==LOPSTER_MESSAGE_ERROR)   ?qtr("Error")      :
        qtr("Information");
    
    QString body = qfu(header) + QString("\r\n") + qfu(text);
    QMessageBox box(icon , title , body, QMessageBox::Close);
    box.exec();
}


void UiMain::ui_main_quit()
{
    lopster_termination_request(LOPSTER_EXIT_DEFAULT);
}

void UiMain::ui_main_toggle_visible()
{
    if( isVisible() ) hide();
    else show();
}

void UiMain::ui_main_toggle_update_systray_menu()
{
    ui_main_create_systray_menu( );
    ui_main_toggle_visible();
}

void UiMain::ui_main_create_systray_menu( )
{
    systray_menu->clear();

    if( !this->isVisible() )
    {
        systray_menu->addAction( qtr("Hide Interface"), this,
                SLOT( ui_main_toggle_update_systray_menu() ) );
    }
    else
    {
        systray_menu->addAction( qtr("Show Interface"), this,
                SLOT( ui_main_toggle_update_systray_menu() ) );
    }
    systray_menu->addAction(qtr("&Quit") , this,
                SLOT( ui_main_quit() ) );

    systray->setContextMenu( systray_menu );
}

void UiMain::ui_main_create_systray() {
    QIcon icon =  QIcon( QPixmap( ":/lopster_icon.png" ) );
    systray = new QSystemTrayIcon( icon, this );
    systray_menu = new QMenu( qtr( "Lopster II" ), this ); //FIXME 
    systray_menu->setIcon( icon );
    systray->setToolTip( qtr( "Lopster II - Multinetwork P2P" )); //FIXME 
    ui_main_create_systray_menu( );
    systray->show();
    CONNECT( systray, activated(  QSystemTrayIcon::ActivationReason ),
            this, ui_main_handle_systray_event( QSystemTrayIcon::ActivationReason ) );
}

void UiMain::ui_main_handle_systray_event( QSystemTrayIcon::ActivationReason reason )
{
    switch( reason )
    {
        case QSystemTrayIcon::Trigger:
            this->ui_main_toggle_visible(); break;
        case QSystemTrayIcon::MiddleClick:
            systray->showMessage( qtr( "Lopster II" ),
                    qtr( "Control menu" ),
                    QSystemTrayIcon::Information, 4000 );
            break;
    }
}

void UiMain::ui_main_display_page(QWidget *page, QAction *action) 
{
    if (page != last_visible)
    {
        last_visible->setVisible(false);
        last_action->setChecked(false);
        page->setVisible(true);
        action->setChecked(true);
        last_visible = page;
        last_action = action;
    }
    else {
        // Same page visited , just make sure the toobar button stay pushed
        action->setChecked(true);
    }
}

void UiMain::ui_main_swith_to_page(int index)
{
    QWidget *page = pages[index];
    QAction *action = actions[index];
    ui_main_display_page(page, action);
}

void UiMain::ui_main_toolbar_clicked(QAction* action)
{
    if (action == actionPreferences)
    {
        PreferencesDialog* preferences_dialog = new PreferencesDialog(this); 
        preferences_dialog->setObjectName(QString::fromUtf8("preferences_dialog"));
        preferences_dialog->ui_preferences_show();
        return;
    }
    QWidget *page = actions_pages[action];   
    ui_main_display_page(page, action);
}

void UiMain::ui_main_update_timer()
{
  /*  if (pages[QT_TRANSFER_PAGE]->isVisible()) */
  /*  { */
  /*      UploadTreeView::s_ui_upload_periodic_update(); */
  /*      DownloadTreeView::s_ui_download_periodic_update(); */
  /*  } */
    //TODO : make an update of the download and upload pages (even hidden) every 2 mn ?    
}

static void ui_string_list_remove(GList* items, const char* name) {
    GList* dlist;
    for (dlist = items; dlist; dlist = dlist->next) {
        const char* item = (char*) dlist->data;
        string_list_remove(name, item);
    }
}

extern "C" QT4_UI_EXPORT void ui_init(ui_interface_t* iface)
{ 
    initLopsterResource();

    iface->enter_main_loop = LopsterApp::ui_app_enter_main_loop;
    iface->leave_main_loop = LopsterApp::ui_app_leave_main_loop;

    iface->download_show_details = DownloadView::ui_download_show_details;
    iface->file_tree_update_label = FileTreeWidget::s_ui_file_tree_update_label;
    iface->file_tree_set_progress = FileTreeWidget::s_ui_file_tree_set_progress;
    iface->file_tree_set_progress_name = FileTreeWidget::s_ui_file_tree_set_progress_name;
    iface->lib_working = LibraryPage::s_ui_lib_working;   
    iface->chat_switch_page = ChatPage::ui_chat_switch_page;
    iface->chat_page_printable = ChatPage::ui_chat_page_printable;
    iface->chat_create_simple_page = ChatPage::ui_chat_create_simple_page;
    iface->chat_print = ChatPage::ui_chat_print;    
    iface->idle_timeout_add = LopsterUiTimer::qt_idle_add;
    iface->timeout_add = LopsterUiTimer::qt_timeout_add;
    iface->timeout_remove = LopsterUiTimer::qt_timeout_remove;

    LopsterApp *lapp = new LopsterApp(*lopster_get_argc(), *lopster_get_argv(), true );
    lapp->setWindowIcon( QIcon( QPixmap(":/lopster_icon.png") ) );
    
    TimerEventFilter *timerFilter = new TimerEventFilter(lapp);
    lapp->installEventFilter(timerFilter);

    lapp->ui_app_set_current_thead(QThread::currentThread());

    object_register_instance("Download", (ObjectFunc)DownloadTreeView::ui_download_new, TRUE);
    object_register_instance("Source", (ObjectFunc)DownloadTreeView::ui_source_new, TRUE);
    object_register_instance("Library", (ObjectFunc)LibraryPage::s_ui_lib_new, TRUE);
//    object_register_instance("SearchPattern", (ObjectFunc)ui_search_pattern_show, FALSE);
    object_register_instance("Search", (ObjectFunc)SearchPage::s_ui_search_add_page, TRUE);
    object_register_instance("SearchResult", (ObjectFunc)TreeResult::s_tree_search_result_show, TRUE);
    object_register_instance("Upload", (ObjectFunc)UploadTreeView::ui_upload_show, TRUE);
//     object_register_instance("Thread", (ObjectFunc)ui_thread_new, TRUE);
    object_register_instance("Buddy", (ObjectFunc)BuddiesTree::s_ui_buddy_new, TRUE);
    object_register_instance("Browse", (ObjectFunc)BuddiesPage::s_ui_browse_show, TRUE);
    object_register_instance("Browse", (ObjectFunc)BuddiesPage::s_ui_browse_link_new, TRUE);

    lopster_notify_register((NotifyFunc)UiMain::ui_user_info);   
    lapp->ui_app_register_styles(); // register all style for lopster
    
    {
        action_item_t* item = action_item_root("String list actions");
        action_item_simple(N_("Remove item"), item, 0,
                           (ActionItemFunc)ui_string_list_remove);
    }

    lapp->setQuitOnLastWindowClosed( false );
    
    UiMain *main_win = new UiMain();    
    lapp->ui_app_set_main_window(main_win);
}

extern "C" QT4_UI_EXPORT void ui_run(void) {
    LopsterApp *l_app = qobject_cast<LopsterApp *>(QCoreApplication::instance());
    l_app->ui_app_get_main_window()->run();
} 

extern "C" QT4_UI_EXPORT void ui_done(void) {
    LopsterApp *l_app = qobject_cast<LopsterApp *>(QCoreApplication::instance());
        
    LopsterUiTimer::qt_timers_cleanup();
    delete l_app->ui_app_get_main_window();

    delete l_app;
}

LopsterApp::LopsterApp(int &argc, char **argv, bool GUIenabled)
            : QApplication(argc,argv,GUIenabled) 
{
}

LopsterApp::~LopsterApp( )
{
}

void LopsterApp::ui_app_set_main_window(UiMain *main) 
{
    this->main_window = main;
    connect(this,SIGNAL(lastWindowClosed()),this->main_window, SLOT(ui_main_quit()));
}

QIcon LopsterApp::ui_app_plugin_get_icon(int id, const QSize &size, bool grey) {
    plugin_t* plugin = plugin_get(id);
    QIcon res;
    if (plugin)
    {
        return ui_app_get_res_icon(qfu(":/%1").arg(plugin->info->icon), size, grey);
    }   
    return res;
}

QIcon LopsterApp::ui_app_get_res_icon(const QString &res, const QSize &size, bool grey)
{
    QIcon orig;
    QIcon result;
    orig = QIcon(res);
    if (orig.isNull())
    {
        return orig;
    }
    
    QIcon::Mode mode = grey? (QIcon::Disabled) : (QIcon::Normal) ;
    result =  QIcon( orig ).pixmap(size, mode);
    return result;
}

void LopsterApp::on_register_temp_lists(const char* name) {
    char* name2 = l_printf("temp:%s", name);
    const char* label = string_list_get_label(name);
    string_list_register(name2, label?label:name, string_list_get_allow_ws(name));
    string_list_set_save(name2, FALSE);
}

void LopsterApp::ui_pref_post_init() {
  /*  int i1; */

  /*  for (i1 = -1; i1 < plugin_get_number(); i1++) {*/
  /*     plugin_t* plugin = plugin_get(i1); */
  /*      char* pref = l_printf("/version/%s", plugin_get_short_name(plugin)); */
  /*      ui_pref_set_funcs(pref, NULL, NULL, NULL); */
  /*  } */

  /*  ui_pref_set_funcs("core:known:plugins", NULL, NULL, NULL); */
  /*  ui_pref_set_funcs("/core/lib/mark_val", NULL, NULL, NULL); */

    string_lists_foreach((StringListFunc)on_register_temp_lists, NULL);
  
    // some special apply function which set a complex structure
    // instead of simple pref items
  /* ui_pref_set_funcs("MediaTypes", (UIPrefFunc)on_media_apply,*/
  /*                  (   UIPrefFunc)on_media_cancel, NULL); */
  
  /* ui_pref_set_funcs("/core/lib/current", (UIPrefFunc)on_library_apply, */
  /*                      (UIPrefFunc)on_library_cancel, NULL); */
}

void LopsterApp::ui_app_enter_main_loop(void) {
    // ui_pref_post_init();  
    LopsterApp *l_app = qobject_cast<LopsterApp *>(QCoreApplication::instance()); 
    l_app->ui_app_get_style_core()->ui_styles_load();
    l_app->ui_pref_post_init();
    l_app->exec();   
}

void LopsterApp::ui_app_leave_main_loop(void) {
    LopsterApp *l_app = qobject_cast<LopsterApp *>(QCoreApplication::instance()); 
    l_app->quit();
}


void LopsterApp::ui_app_register_styles()
{
    style_core = new StyleCore(this);
    style_core->ui_styles_init();
    style_core->ui_style_register(qtr("General style settings"), 
                      QString("bandwidth:color"),
                      qtr("Color of the bandwidth bar"),
                      qtr("Background"), false,  QColor(0xdf, 0xb0, 0xb0),
                      qtr("Border"),     false,  QColor(0x99, 0x99, 0x99),
                      qtr("Base"),       true,   QColor(0xff, 0xff, 0xff),
                      qtr("Bar"),        true,   QColor(0x96, 0xa5, 0x96),
                      qtr("Text"),       true,   QColor(0x00, 0x00, 0x00));

    style_core->ui_style_register(qtr("General style settings"), 
                      QString("transfer:bar"),
                      qtr("Colors of the upload and download progress bars"),
                      qtr("Background"), false,  QColor(0xe7, 0xe7, 0xe7),
                      qtr("Border"),     false,  QColor(0xcf, 0x41, 0x41),
                      qtr("Finished"),   true,   QColor(0xef, 0xc8, 0x65),
                      qtr("In-progress"),true,   QColor(0xe4, 0xd9, 0xb4),
                      qtr("Percentage"), true,   QColor(0xcf, 0x41, 0x41));

    style_core->ui_style_register(qtr("General style settings"), 
                      QString("filetree:recent"),
                      qtr("Recent files in the library"),
                      qtr("Text"),          true,  QColor(0xff, 0x44, 0x00),
                      qtr("Background"),    false, QColor(0xcc, 0xcc, 0xcc),
                      qtr("Font"),          false, QFont());

    style_core->ui_style_register(qtr("General style settings"), 
                      QString("network:entry:online"),
                      qtr("Online networks in the a server list"),
                      qtr("Text"),          true,  QColor(0x3c, 0xaa, 0x00),
                      qtr("Background"),    false, QColor(0xcc, 0xcc, 0xcc),
                      qtr("Font"),          false, QFont());

    style_core->ui_style_register(qtr("General style settings"), 
                      QString("search:available"),
                      qtr("Search results that are available for download"),
                      qtr("Text"),          true,  QColor(0xc8, 0x49, 0x3f),
                      qtr("Background"),    false, QColor(0xcc, 0xcc, 0xcc),
                      qtr("Font"),          false, QFont());

    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:timestamp"),
                      qtr("Chat timestamps"),
                      qtr("Text"), TRUE, QColor(0xde, 0x90, 0x7d),
                      qtr("Background"), TRUE, QColor(0xfd, 0xef, 0xe7),
                      qtr("Font"), FALSE, QFont(),
                      qtr("Underline"), FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:text"),
                      qtr("Normal chat text"),
                      qtr("Text"),  TRUE, QColor(0x75, 0x75, 0x75),
                      qtr("Background"),  FALSE, QColor(0x00, 0x00, 0x00),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:private"),
                      qtr("Private chat text"),
                      qtr("Text"),  FALSE, QColor(0x6b, 0x6b, 0x6b),
                      qtr("Background"),  TRUE, QColor(0xff, 0xbe, 0x9e),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:nick"),
                      qtr("Style for nicknames"),
                      qtr("Text"),  TRUE, QColor(0x00, 0x55, 0xaa),
                      qtr("Background"),  FALSE, QColor(0x00, 0x00, 0x00),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:client"),
                      qtr("Client messages"),
                      qtr("Text"),  TRUE, QColor(0x3d, 0x75, 0x73),
                      qtr("Background"),  FALSE, QColor(0x00, 0x00, 0x00),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:server"),
                      qtr("Server messages"),
                      qtr("Text"),  TRUE, QColor(0xa7, 0x37, 0x34),
                      qtr("Background"),  FALSE, QColor(0x00, 0x00, 0x00),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:cprefix"),
                      qtr("Prefix for client messages"),
                      qtr("Text"),  TRUE, QColor(0x4b, 0x7f, 0xcd),
                      qtr("Background"),  TRUE, QColor(0x4b, 0x7f, 0xcd),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:sprefix"),
                      qtr("Prefix for server messages"),
                      qtr("Text"),  TRUE, QColor(0xf2, 0x47, 0x2c),
                      qtr("Background"),  TRUE, QColor(0xf2, 0x47, 0x2c),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:jprefix"),
                      qtr("Prefix for join messages"),
                      qtr("Text"),  TRUE, QColor(0x43, 0xa8, 0x9c),
                      qtr("Background"),  TRUE, QColor(0x43, 0xa8, 0x9c),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:pprefix"),
                      qtr("Prefix for part messages"),
                      qtr("Text"),  TRUE, QColor(0x43, 0xa8, 0x9c),
                      qtr("Background"),  TRUE, QColor(0x43, 0xa8, 0x9c),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:tprefix"),
                      qtr("Prefix for normal chat lines"),
                      qtr("Text"),  TRUE, QColor(0xfd, 0xef, 0xe7),
                      qtr("Background"),  TRUE, QColor(0xfd, 0xef, 0xe7),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:url"),
                      qtr("Urls in chat messages"),
                      qtr("Text"),  TRUE, QColor(0x00, 0x7a, 0xb6),
                      qtr("Background"),  FALSE, QColor(0x00, 0x00, 0x00),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  TRUE, TRUE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:bracket"),
                      qtr("Style of chat brackets"),
                      qtr("Text"),  TRUE, QColor(0xe9, 0x50, 0x00),
                      qtr("Background"),  FALSE, QColor(0x00, 0x00, 0x00),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:join"),
                      qtr("Channel join messages"),
                      qtr("Text"),  TRUE, QColor(0x43, 0xa8, 0x9c),
                      qtr("Background"),  FALSE, QColor(0x00, 0x00, 0x00),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:part"),
                      qtr("Channel part messages"),
                      qtr("Text"),  TRUE, QColor(0x43, 0xa8, 0x9c),
                      qtr("Background"),  FALSE, QColor(0x00, 0x00, 0x00),
                      qtr("Font"),  FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:hilight"),
                      qtr("Hilighted text"),
                      qtr("Text"),  TRUE, QColor(0x00, 0x00, 0x00),
                      qtr("Background"),  FALSE, QColor(0x00, 0x00, 0x00),
                      qtr("Font"),  TRUE, QFont("Luxi Mono Bold 10"),
                      qtr("Underline"),  FALSE, FALSE);
    style_core->ui_style_register(qtr("Style settings for chat pages"), 
                      QString("chat:user:opped"),
                      qtr("Users who have operator status"),
                      qtr("Text"),        TRUE,  QColor(0x3c, 0xaa, 0x00),
                      qtr("Background"),  FALSE, QColor(0xcc, 0xcc, 0xcc),
                      qtr("Font"),         FALSE, QFont(),
                      qtr("Underline"),  FALSE, FALSE);

}
