/* Copyright (C) 2007 Lopster Team. This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2007/09/19 14:32:14 $
 * $Author: sid4u $
 */

#ifndef _UI_MAIN_H
#define _UI_MAIN_H


#include <QSystemTrayIcon>
#include "lcore/ui.h"
#include "CustomWidgets.hpp"
#include "ui_utils.hpp"
#include "widgets/LopsterMainWindow.h"

#define qfl( i ) QString::fromLatin1( i ) 
#define qfu( i ) QString::fromUtf8( i ) 
#define qtr( i ) QString::fromUtf8( _( i )) 
#define qtu( i ) i.toUtf8().data() 
#define qta( i ) i.toAscii().data() 
#define qtl( i ) i.toLatin1().data() 

#define LICON(a, b)     LopsterApp::ui_app_get_res_icon(a,b)
#define LICONG(a, b)    LopsterApp::ui_app_get_res_icon(a,b,true)
#define PICON(i, s)     LopsterApp::ui_app_plugin_get_icon(i,s)
#define PICONG(i, s)    LopsterApp::ui_app_plugin_get_icon(i,s,true)

#define CONNECT( a, b, c, d ) connect( a, SIGNAL( b ), c, SLOT(d) )

#define LOPSTER_TOOLBAR_ICON        QSize(48,48)
#define LOPSTER_TAB_ICON            QSize(38,38)
#define LOPSTER_TREEVIEW_ICON       QSize(16,16)
#define LOPSTER_PUSHBUTTON_ICON     QSize(22,22)
#define LOPSTER_TOOL_ICON           QSize(18,18)
#define LOPSTER_LABEL_ICON          QSize(24,24)

inline void initLopsterResource() {Q_INIT_RESOURCE( lopster2 ); }

enum {
	QT_NETWORK_PAGE=0,
	QT_SEARCH_PAGE,
	QT_TRANSFER_PAGE,
	QT_BUDDIES_PAGE,
	QT_CHAT_PAGE,
	QT_LIBRARY_PAGE,
    QT_PREFERENCES_DIALOG
};

enum {
    INTERFACE_INIT,
    INTERFACE_RUN,
    INTERFACE_EXIT
};

class StyleCore;
class BandwidthProgress;
class UiMain : public LMainWindow, private Ui::LopsterMainWindow
{
   Q_OBJECT
public:
    UiMain(QWidget *parent = 0);
    ~UiMain();
    void run();
    void ui_main_swith_to_page(int index);
    static void ui_user_info(int type, const char* header, const char* text);
public:
    LWidget *pages[7];
    QAction *actions[7];
private:
    void ui_main_create_systray_menu();
    void ui_main_create_systray();
    void ui_main_toggle_visible();
    void ui_main_display_page(QWidget *page, QAction *);
private slots:
    void ui_main_toolbar_clicked(QAction* action);
    void ui_main_toggle_update_systray_menu();
    void ui_main_handle_systray_event( QSystemTrayIcon::ActivationReason );
    void ui_main_update_timer();
    void ui_main_quit();
private:
    QSystemTrayIcon *systray;
    QMenu *systray_menu;
    QWidget *last_visible;
    QAction *last_action;

    BandwidthProgress *dprogress;
    BandwidthProgress *uprogress;

};

class LopsterApp : public QApplication
{
    Q_OBJECT
public:
    LopsterApp(int &argc, char **argv, bool GUIenabled);
    ~LopsterApp( );
    void ui_app_register_styles();
    void ui_pref_post_init();
    static void on_register_temp_lists(const char* name);
    static void ui_app_enter_main_loop(void);
    static void ui_app_leave_main_loop(void);

    // extra function , maybe to put in ui_support
    static QIcon ui_app_plugin_get_icon(int id, const QSize &size, bool grey=false);
    static QIcon ui_app_get_res_icon(const QString &res, const QSize &size, bool grey=false);
    
    void ui_app_set_current_thead(QThread *thread) {this->current_thread = thread;}
    QThread *ui_app_get_current_thead(){return this->current_thread;}

    void ui_app_set_main_window(UiMain *main);// {this->main_window = main;}
    UiMain *ui_app_get_main_window() {return this->main_window;}

    StyleCore *ui_app_get_style_core() {return this->style_core;}

   // static int ui_main_get_interface_state(UiMain *main) {this->main_window = main;}
private:
    QThread *current_thread;
    StyleCore *style_core;
    UiMain *main_window;
};

#define LOPSTER_APP_GET_STYLE(s) \
                s = qobject_cast<LopsterApp *> \
                            (QCoreApplication::instance())->ui_app_get_style_core()

extern "C" QT4_UI_EXPORT void ui_init(ui_interface_t* iface);
extern "C" QT4_UI_EXPORT void ui_run(void);
extern "C" QT4_UI_EXPORT void ui_done(void);

#endif
