/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.17 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QtGui>
#include <QObject>

#include "lcore/plugin.h"
#include "lcore/l_i18n.h"

#include "ui_main.hpp"
#include "ui_utils.hpp"
#include "ui_style.hpp"
#include "ui_network.hpp"

#define FIND_NETWORK_PAGE(p) \
            p =  qobject_cast<NetworkPage *> (findWidgetByName(QString("network_page"))); \
            Q_ASSERT(p); 

#define MAX_SUMMARY_PER_LINE    4

PluginSummary::PluginSummary(QWidget* parent, int plugin_id)
    : QWidget(parent)
{
	setupUi(this);
    m_plugin = plugin_get(plugin_id);
    if (m_plugin)
    {
        pluginIcon->setPixmap(QPixmap(QString("pixmaps/%1").arg(QString(m_plugin->info->icon))));
        plugin_name->setText(qfu(plugin_get_name(m_plugin)));
    }
    start_connect->setText(qtr("Start"));
    stop_connect->setText(qtr("Stop"));
    stop_connect->setEnabled(false);
    connect(start_connect, SIGNAL(linkActivated(const QString&)), this, SLOT(do_auto()));  
    connect(stop_connect,  SIGNAL(linkActivated(const QString&)), this, SLOT(do_discon()));      
}

QString PluginSummary::ui_net_summary_status_to_number(int value) {
    if (value < 1000) 
        return qfu("%1").arg(value);
    else if (value < 1000000)
        return qtr("%1.1f thousand").arg((double)value/1000.);
    else if (value < 1000000000)
        return qtr("%1.1f million").arg((double)value/1000000.);
    else
        return qtr("%1.1f billion").arg((double)value/1000000000.);
}

void PluginSummary::ui_net_summary_update_status(online_status_t* status)
{
    bool online = (status->status & ONLINE_STATUS_ONLINE);
//     bool auto_connect = (status->status & ONLINE_STATUS_AUTO);
    file_size_t size = status->gigs;

    plugin_status->setText(online?qtr("online"):qtr("offline"));
//     plugin_auto->setText(auto_connect?qtr("yes"):qtr("no"));
    plugin_users->setText((status->users && online)?
                ui_net_summary_status_to_number(status->users):qtr("unknown"));
    plugin_files->setText((status->files && online)?
                ui_net_summary_status_to_number(status->files):qtr("unknown"));
    size *= 1024*1024*1024;
    plugin_content->setText((status->gigs && online)?
                qfu(print_filesize(&size)):qtr("unknown"));
    
    start_connect->setEnabled(online?false:true);
    stop_connect->setEnabled(online?true:false);
}


void PluginSummary::do_auto() {
    if (m_plugin) {
        plugin_connect_change(m_plugin->ID, CONNECT_START);
    }
}
void PluginSummary::do_discon() {
    if (m_plugin) {
        plugin_connect_change(m_plugin->ID, CONNECT_STOP);
    }
}

NetworkPage::NetworkPage(QWidget* parent)
    : LWidget(parent)
{
	setupUi(this);
    main_layout = new QVBoxLayout(overview);
    summary_cnt = 0;
    current_widget = new QWidget(overview);
    main_layout->addWidget(current_widget);
    main_layout->addItem(new QSpacerItem(10,10,QSizePolicy::Minimum, QSizePolicy::Expanding));
    current_layout = new QHBoxLayout(current_widget);
    current_layout->addItem(new QSpacerItem(10,10,QSizePolicy::Expanding, QSizePolicy::Minimum));
}

NetworkPage::~NetworkPage()
{
    int cnt = tab_networks->count();
    if (cnt)
    {
        QWidget *w = tab_networks->widget(0);
        delete w;
    }
}

void NetworkPage::ui_network_add_summary(int plugin_id)
{
    if (summary_cnt == MAX_SUMMARY_PER_LINE)
    {
        summary_cnt = 0;
        current_widget = new QWidget(overview);
        main_layout->addWidget(current_widget);
        current_layout = new QHBoxLayout(current_widget);
        current_layout->addItem(new QSpacerItem(10,10, QSizePolicy::Expanding, QSizePolicy::Minimum));
    }
    PluginSummary *summary = new PluginSummary( current_widget, plugin_id );
    current_layout->insertWidget(summary_cnt,summary);
    summary_cnt++;
    plugin_t* plugin = plugin_get(plugin_id);
    object_monitor_d(OBJ(plugin->online),
                   (ObjectFunc)s_ui_network_online_status_changed, summary);
}

void NetworkPage::ui_network_add_page(int plugin_id, QWidget* pluginwidget )
{
    NetworkPage *page;
    FIND_NETWORK_PAGE(page);
    plugin_t* plugin = plugin_get(plugin_id);
    page->tab_networks->addTab(pluginwidget, PICON(plugin_id, LOPSTER_TAB_ICON),  qfu(plugin_get_name(plugin)));
    page->ui_network_add_summary(plugin_id);
}

void NetworkPage::ui_network_remove_page(QWidget* widget)
{
    NetworkPage *page;
    FIND_NETWORK_PAGE(page);
    if (page->tab_networks->indexOf(widget) != -1)
    {
        page->tab_networks->removeTab(page->tab_networks->indexOf(widget));
    }
}

void NetworkPage::s_ui_network_online_status_changed(online_status_t* status, 
                                                        PluginSummary* summary)
{
    summary->ui_net_summary_update_status(status);
}
