/* Copyright (C) 2007 The Lopster Team
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.20 $
 * $Date: 2007/11/13 10:23:22 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "ui_main.hpp"
#include "ui_preferences.hpp"
#include "ui_utils.hpp"
#include "widgets/PrefClient_BasicClientSetupWidget.h"
#include "widgets/PrefFilesharing_MyIdentityWidget.h"
#include "widgets/PrefFilesharing_TransferWidget.h"
#include "widgets/PrefClient_PathApplicationWidget.h"
#include "widgets/PrefClient_ChatFunctionnalityWidget.h"
#include "widgets/PrefClient_ChatAppearanceWidget.h"

QList <PrefUpdater *> applys_cancels;
QList <PluginPageInfo *> plugin_lists;

DropButtonPrefUpdater::DropButtonPrefUpdater(char* preference, QPushButton *button, 
											 QList <DropButtonValue *> items) :
        PrefUpdater(preference), m_button(button) {           
    if (preference)
    {
        m_val = pref_get_ref(m_preference, int);
        m_drop_list = items;
        m_menu = create_menu(-1);
        button->setMenu(m_menu);
        connect(m_menu, SIGNAL(triggered(QAction*)), this, SLOT(updateFlag(QAction*)));
    }

}

void DropButtonPrefUpdater::apply_pref() { 
    pref_store(m_preference, m_val);
}

void DropButtonPrefUpdater::cancel_pref() {
    m_val = pref_get_ref(m_preference, int);
}


void DropButtonPrefUpdater::updateFlag(QAction *action) {
    DropButtonValue *value = m_items[action];
    if (value)
    {
          m_val ^= value->flag;
    }
}

QMenu * DropButtonPrefUpdater::create_menu(/*QList <DropButtonValue *> items, */int no_loop) {
    QMenu *menu = new QMenu(m_button);
   
    while (!m_drop_list.empty() && no_loop !=0 )
    {
        DropButtonValue *value = m_drop_list.takeFirst();
        if (value->child > 0)
        {
            QMenu *child = create_menu(value->child);
            child->setTitle(value->item);
            menu->addMenu(child);
        }
        else if (value->sep)
        {
            menu->addSeparator();
        }
        else {
            QAction *action = new QAction(value->item, m_button);
            action->setCheckable(true);
            if (m_val & value->flag)
            {
                action->setChecked(true);
            }
            menu->addAction(action);
            m_actions.append(action);
            m_items[action] = value;
            no_loop--;
        }
    }
    return menu;
}

void DropButtonPrefUpdater::updateMenu( ) {
    for (int i=0;i<m_actions.size();i++)
    {
        QAction *action = m_actions.at(i);
        DropButtonValue *value = m_items[action];
        if (value)
        {
            if (m_val & value->flag)
            {
                action->setChecked(true);
            }
        }
    }
}

StringListPrefUpdater::StringListPrefUpdater(char* preference, char* name, 
                                             const QString &title, QLineEdit *entry,
                                            StringListWidget *widget) :
			LineEditPrefUpdater(preference, entry), string_list(name), str_widget(widget) 
{
    tmp_string_list = g_strdup_printf("temp:%s", string_list);
    string_list_copy_items(tmp_string_list, string_list);
    str_widget->setup(title, qfu(string_list_get_label(string_list)),tmp_string_list, "String list actions");
}

StringListPrefUpdater::~StringListPrefUpdater()
{
    g_free(tmp_string_list);
}

void StringListPrefUpdater::apply_pref() 
{
    if (m_preference)
    {
        pref_store_str(m_preference, qtu(m_line->text()));
    }
    string_list_copy_items(string_list, tmp_string_list);
}
void StringListPrefUpdater::cancel_pref() 
{
    if (m_preference)
    {
        pref_node_t* node = pref_search(m_preference, TRUE);
        m_line->setText( qfu("%1").arg(node->xml_print(node->value))) ;
    }
    string_list_copy_items(tmp_string_list, string_list);
}

PreferencesDialog::PreferencesDialog(QWidget* parent)
    : QDialog(parent)
{
	setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose);

    pageno = 0;

    preferences_tree->setHeaderLabel(qtr("Preferences"));
    preferences_tree->setColumnCount(1); 

    root_item = new PrefTreeWidgetItem(preferences_tree);
    root_item->setText(0,qtr("Lopster Preferences"));
	root_item->setIcon(0, LICON(":/lopster_icon.png", LOPSTER_TREEVIEW_ICON));
    connect(ok_prefs, SIGNAL(clicked()), this, SLOT(ui_preferences_ok_all()));
    connect(apply_prefs, SIGNAL(clicked()), this, SLOT(ui_preferences_apply_all()));
    connect(cancel_prefs, SIGNAL(clicked()), this, SLOT(ui_preferences_cancel_all()));
}

PreferencesDialog::~PreferencesDialog()
{
    for (int i=0;i<applys_cancels.size();i++ )
    {
        PrefUpdater *pref = applys_cancels.at(i);
        delete pref;
    }
    applys_cancels.clear();
}

void PreferencesDialog::ui_preferences_id_page( )
{
    QWidget *widget = new QWidget(preferences_stacked);
    Ui::myidentity myidentity;
    myidentity.setupUi(widget);  preferences_stacked->addWidget(widget);

    // user infos
    LineEditPrefUpdater *default_username =    
                new LineEditPrefUpdater("/core/user/nick", myidentity.line_default_username);
    LineEditPrefUpdater *default_passwd =     
                new LineEditPrefUpdater("/core/user/pass", myidentity.line_default_passwd);
    TextEditPrefUpdater *user_description =     
                new TextEditPrefUpdater("/core/user/description", myidentity.text_user_description);
    ImageButtonPrefUpdater *user_picture =     
                new ImageButtonPrefUpdater("/core/user/picture", myidentity.tool_user_picture);

    // Line type and speed

    QStringList lines_list;
    lines_list << qtr("Not sure") << qtr("Analog modem") << qtr("ISDN") << qtr("DSL")
                 << qtr("Cable") << qtr("Satellite") << qtr("Power line")
                   << qtr("T1")  << qtr("T3+");
    ComboPrefUpdater *line_type = new ComboPrefUpdater("/core/line/type",
                                        myidentity.combo_line_type, lines_list);
    SpinBoxPrefUpdater *max_upload =     
                new SpinBoxPrefUpdater("/core/line/speed", 
                            myidentity.spin_upload_max);

    // static ip
    bool val = pref_get_ref("/core/local/force_ip", bool);
    myidentity.group_force_ip->setChecked(val);
    GroupCheckPrefUpdater *force_ip_enabled = 
                new GroupCheckPrefUpdater("/core/local/force_ip", myidentity.group_force_ip);
    LineEditPrefUpdater *local_ip =     
                new LineEditPrefUpdater("/core/local/ip", myidentity.line_local_ip);

    applys_cancels.append(default_username);
    applys_cancels.append(default_passwd);
    applys_cancels.append(user_description);
    applys_cancels.append(user_picture);

    applys_cancels.append(line_type);
    applys_cancels.append(max_upload);

    applys_cancels.append(force_ip_enabled);
    applys_cancels.append(local_ip);
}

void  PreferencesDialog::ui_preferences_share_page( )
{
    LibraryPref *widget = new LibraryPref(preferences_stacked); preferences_stacked->addWidget(widget);
    SharedFileTreePrefUpdater *shared_widget = 
                new SharedFileTreePrefUpdater("/core/lib/current", widget);

    applys_cancels.append(shared_widget);
}

void  PreferencesDialog::ui_preferences_transfer_page( )
{
    QWidget *widget = new QWidget(preferences_stacked); preferences_stacked->addWidget(widget);
    Ui::transfer transfer;
    transfer.setupUi(widget);

    bool val = pref_get_ref("/core/listen/firewalled", bool);
    transfer.group_data_ports->setChecked(val);
    // Data ports
    GroupCheckPrefUpdater *data_ports = 
                new GroupCheckPrefUpdater("/core/listen/firewalled", transfer.group_data_ports);
    LineEditPrefUpdater *port_range =    
                new LineEditPrefUpdater("/core/listen/range", transfer.line_port_range);
    
    // Download
    SpinBoxPrefUpdater *down_overall =  
                new SpinBoxPrefUpdater("/core/download/limit", transfer.spin_down_overall);
    SpinBoxPrefUpdater *down_peruser =  
                new SpinBoxPrefUpdater("/core/download/limit_per_user", transfer.spin_down_peruser);
    
    SpinBoxPrefUpdater *down_bandwidth =
                new SpinBoxPrefUpdater("/core/download/bandlimit", 
                            transfer.spin_download);

    // Upload
    SpinBoxPrefUpdater *up_overall =  
                new SpinBoxPrefUpdater("/core/upload/limit", transfer.spin_up_overall);
    SpinBoxPrefUpdater *up_peruser =  
                new SpinBoxPrefUpdater("/core/upload/limit_per_user", transfer.spin_up_peruser);
    SpinBoxPrefUpdater *up_bandwidth =     
                new SpinBoxPrefUpdater("/core/upload/bandlimit", 
                            transfer.spin_upload);
   
   // Incomplete folder
   FolderLinePrefUpdater *folder_incomplete = 
                new FolderLinePrefUpdater("/core/path/incomplete",
                            transfer.line_path_incomplete,  transfer.tool_path_incomplete);

   applys_cancels.append(data_ports);
   applys_cancels.append(port_range);  

   applys_cancels.append(down_overall);
   applys_cancels.append(down_peruser);   
   applys_cancels.append(down_bandwidth);

   applys_cancels.append(up_overall);
   applys_cancels.append(up_peruser);   
   applys_cancels.append(up_bandwidth);

   applys_cancels.append(folder_incomplete);
}


void PreferencesDialog::ui_preferences_basic_page( )
{   
    QWidget *widget = new QWidget(preferences_stacked);  preferences_stacked->addWidget(widget);
    Ui::basic_client_setup basic_client;
    basic_client.setupUi(widget);

    // basic_client.combo_interface_name->setText(qfu("%1").arg(ui_get_list(NULL)?ui_get_list(NULL)->name:""));
    // Create the preferences cancel/applyers
    UiComboPrefUpdater *ui_interface = new UiComboPrefUpdater(basic_client.combo_interface_name);

    bool val = pref_get_ref("/core/web/active", bool);
    basic_client.group_web->setChecked(val);
    GroupCheckPrefUpdater *web_enabled = 
                new GroupCheckPrefUpdater("/core/web/active", basic_client.group_web);
    LineEditPrefUpdater *web_login =    
                new LineEditPrefUpdater("/core/web/username", basic_client.line_web_login);
    LineEditPrefUpdater *web_pass =     
                new LineEditPrefUpdater("/core/web/password", basic_client.line_web_passwd);
    SpinBoxPrefUpdater *web_port =  
                new SpinBoxPrefUpdater("/core/web/port", basic_client.spin_web_port);
    LineEditPrefUpdater *web_res =  
                new LineEditPrefUpdater("/core/web/resource_path", basic_client.line_web_res);

    val = pref_get_ref("/core/http/proxy/enabled", bool);
    basic_client.group_proxy->setChecked(val);
    GroupCheckPrefUpdater *proxy_enabled = 
                new GroupCheckPrefUpdater("/core/http/proxy/enabled", basic_client.group_proxy);
    LineEditPrefUpdater *proxy_host =  
                new LineEditPrefUpdater("/core/http/proxy/host", basic_client.line_proxy_addr);
     
    // FIXME : Fix that when proxy functionnality will be available
    // LineEditPrefUpdater *proxy_login =    
    //             new LineEditPrefUpdater("/core/web/username", basic_client.line_proxy_login);
    // LineEditPrefUpdater *proxy_pass =     
    //             new LineEditPrefUpdater("/core/web/password", basic_client.line_proxy_passwd);
    
    SpinBoxPrefUpdater *proxy_port =  
                new SpinBoxPrefUpdater("/core/http/proxy/port", basic_client.spin_proxy_port);
    applys_cancels.append(ui_interface);
    
    applys_cancels.append(web_enabled);
    applys_cancels.append(web_login);
    applys_cancels.append(web_pass);
    applys_cancels.append(web_port);
    applys_cancels.append(web_res);

    applys_cancels.append(proxy_enabled);
    applys_cancels.append(proxy_host);
    applys_cancels.append(proxy_port);
}

void PreferencesDialog::ui_preferences_pathapp_page()
{
    QWidget *widget = new QWidget(preferences_stacked); preferences_stacked->addWidget(widget);
    Ui::path_application path_app;
    path_app.setupUi(widget);

    MediaLinePrefUpdater *unknown = new MediaLinePrefUpdater(NULL, path_app.line_unknown_path, 
                                         path_app.tool_unknown_path, MEDIA_UNKNOWN);
    MediaLinePrefUpdater *audio = new MediaLinePrefUpdater(NULL, path_app.line_audio_path, 
                                         path_app.tool_audio_path, MEDIA_AUDIO);
    MediaLinePrefUpdater *video = new MediaLinePrefUpdater(NULL, path_app.line_video_path, 
                                         path_app.tool_video_path, MEDIA_VIDEO);
    MediaLinePrefUpdater *app = new MediaLinePrefUpdater(NULL, path_app.line_app_path, 
                                         path_app.tool_app_path, MEDIA_APPLICATION);
    MediaLinePrefUpdater *image = new MediaLinePrefUpdater(NULL, path_app.line_image_path, 
                                         path_app.tool_image_path, MEDIA_IMAGE);
    MediaLinePrefUpdater *txt = new MediaLinePrefUpdater(NULL, path_app.line_txt_path, 
                                         path_app.tool_txt_path, MEDIA_DOCUMENT);
    applys_cancels.append(unknown);
    applys_cancels.append(audio);
    applys_cancels.append(video);
    applys_cancels.append(app);
    applys_cancels.append(image);
    applys_cancels.append(txt);
}

void PreferencesDialog::ui_preferences_chatfunc_page()
{
    QWidget *widget = new QWidget(preferences_stacked); preferences_stacked->addWidget(widget);
    Ui::chatfunctionnality chatfunc;
    chatfunc.setupUi(widget);


    QStringList lines_list;
    lines_list << qtr("don't rejoin, but leave page open and deactivated")
                 << qtr("if part was not requested (kicked, connection lost, etc.)");
    ComboPrefUpdater *auto_chan = new ComboPrefUpdater("/core/chat/auto_rejoin",
                                        chatfunc.combo_auto_chan, lines_list);

    QList <DropButtonValue *> items;
    items.append(new DropButtonValue(0,qtr("server messages"), HIDE_SERVER,widget));
    items.append(new DropButtonValue(0,qtr("global messages"), HIDE_GLOBAL,widget));
    items.append(new DropButtonValue(0,qtr("channel parts"), HIDE_PARTS,widget));
    items.append(new DropButtonValue(0,qtr("channel joins"), HIDE_JOINS,widget));
    items.append(new DropButtonValue(0,qtr("messages of the day"), HIDE_MOTD,widget));

    DropButtonPrefUpdater *not_show = new DropButtonPrefUpdater("/core/chat/hide",
                                    chatfunc.push_not_show, items);

    QList <DropButtonValue *> items2;
    items2.append(new DropButtonValue(0,qtr("private messages"), PIPE_PRIVATE,widget));
    items2.append(new DropButtonValue(0,qtr("global messages"), PIPE_GLOBAL,widget));
    items2.append(new DropButtonValue(0,qtr("operator messages"), PIPE_OPERATOR,widget));
    items2.append(new DropButtonValue(0,qtr("channel operator messages"), PIPE_CHAN_OP,widget));

    DropButtonPrefUpdater *sep_win = new DropButtonPrefUpdater("/core/chat/pipe",
                                    chatfunc.push_separate_win, items2);

    bool val = pref_get_ref("/core/user/away", bool);
    chatfunc.group_away->setChecked(val);
    GroupCheckPrefUpdater *away_enabled = 
                new GroupCheckPrefUpdater("/core/user/away", chatfunc.group_away);
    StringListPrefUpdater *away_msg = new StringListPrefUpdater("/core/user/away_message",
                                    "core:user:away", qtr("away message"), 
                                    chatfunc.widget_str_away->get_line(), chatfunc.widget_str_away);

    applys_cancels.append(auto_chan);
    applys_cancels.append(not_show);
    applys_cancels.append(sep_win);
    applys_cancels.append(away_enabled);
    applys_cancels.append(away_msg);
}

void PreferencesDialog::ui_preferences_chatappear_page()
{
    QWidget *widget = new QWidget(preferences_stacked); preferences_stacked->addWidget(widget);
    Ui::chatappearance chatappear;
    chatappear.setupUi(widget);

    QStringList timestamp_list;
    timestamp_list << qtr("no timestamps")
                 << qtr("hour:minute")
                 << qtr("hour:minute:second");
    ComboPrefUpdater *timestamp = new ComboPrefUpdater("/core/chat/timestamp",
                                        chatappear.combo_timestamp_format, timestamp_list);

    QStringList colored_list;
    colored_list << qtr("default color")
                 << qtr("dark colors")
                 << qtr("medium colors")
                 << qtr("light colors");
    ComboPrefUpdater *colored_nick = new ComboPrefUpdater("/core/chat/colored_nicks",
                                        chatappear.combo_colored_nick, colored_list);

    LineEditPrefUpdater *server_msg =  
                new LineEditPrefUpdater("/core/chat/prefix/server", chatappear.line_msg_server);
    LineEditPrefUpdater *client_msg =  
                new LineEditPrefUpdater("/core/chat/prefix/client", chatappear.line_msg_client);
    LineEditPrefUpdater *join_msg =  
                new LineEditPrefUpdater("/core/chat/prefix/join", chatappear.line_msg_join);
    LineEditPrefUpdater *part_msg =  
                new LineEditPrefUpdater("/core/chat/prefix/part", chatappear.line_msg_part);
    LineEditPrefUpdater *other_msg =  
                new LineEditPrefUpdater("/core/chat/prefix/text", chatappear.line_msg_other);
    
    StringListPrefUpdater *highlight_msg = new StringListPrefUpdater(NULL,
                                    "core:chat:hilight", qtr("highlight lines that contain one of these terms"), 
                                    chatappear.widget_str_high->get_line(), chatappear.widget_str_high);

    applys_cancels.append(timestamp);
    applys_cancels.append(colored_nick);
    applys_cancels.append(server_msg);
    applys_cancels.append(client_msg);
    applys_cancels.append(join_msg);
    applys_cancels.append(part_msg);
    applys_cancels.append(other_msg);
    applys_cancels.append(highlight_msg);
}

void PreferencesDialog::ui_preferences_style_page()
{
    StyleWidget *styles = new StyleWidget(preferences_stacked);
    styles->ui_style_generate_styles();
    preferences_stacked->addWidget(styles);
    // Just get the style widget initied from main and wrap it to an updater
    StylePrefUpdater *style_pref = new StylePrefUpdater(styles);
    applys_cancels.append(style_pref);
}

void PreferencesDialog::ui_preferences_apply_cancel_add(PrefUpdater *updater)
{
    applys_cancels.append(updater);
}

void PreferencesDialog::ui_preferences_init_plugins()
{
    for (int i=0;i<plugin_lists.size();i++ )
    {
        PluginPageInfo *info = plugin_lists.at(i);
        QWidget *page = info->create_func();
        page->setParent(preferences_stacked);
        preferences_stacked->addWidget(page);
        PrefTreeWidgetItem *plugin = new PrefTreeWidgetItem(plugins_item);
        plugin->setText(0,info->label);
		plugin->setIcon(0, PICON(info->id, LOPSTER_TREEVIEW_ICON));
        plugin->set_page(++pageno);
    }
}


void PreferencesDialog::ui_preferences_show()
{
    ui_preferences_init();
    this->exec(); // modal during change of preferences
}

void PreferencesDialog::ui_preferences_init()
{   

    PrefTreeWidgetItem *filesharing = new PrefTreeWidgetItem(root_item);
    filesharing->setText(0,qtr("Filesharing"));
	filesharing->setIcon(0, LICON(":/filesharing.png", LOPSTER_TREEVIEW_ICON));
    // Filesharing - My Identity
    PrefTreeWidgetItem *identity = new PrefTreeWidgetItem(filesharing);
    identity->setText(0,qtr("My identity"));
	identity->setIcon(0, LICON(":/personal.png", LOPSTER_TREEVIEW_ICON));

	identity->set_page(++pageno);
    ui_preferences_id_page( );   
    // Filesharing - Shared Files
    PrefTreeWidgetItem *shared_files = new PrefTreeWidgetItem(filesharing);
    shared_files->setText(0,qtr("Shared files"));
	shared_files->setIcon(0, LICON(":/shared_files.png", LOPSTER_TREEVIEW_ICON));
    shared_files->set_page(++pageno);
    ui_preferences_share_page( );
    // Filesharing - Transfers
    PrefTreeWidgetItem *transfers = new PrefTreeWidgetItem(filesharing);
    transfers->setText(0,qtr("Transfers"));
	transfers->setIcon(0, LICON(":/transfer.png", LOPSTER_TREEVIEW_ICON));
	transfers->set_page(++pageno);
    ui_preferences_transfer_page( );


    PrefTreeWidgetItem *client = new PrefTreeWidgetItem(root_item);
    client->setText(0,qtr("Client"));
	client->setIcon(0, LICON(":/client.png", LOPSTER_TREEVIEW_ICON));
    // Client, basic client setup
    PrefTreeWidgetItem *basicclient = new PrefTreeWidgetItem(client);
    basicclient->setText(0,qtr("Basic client setup"));
	basicclient->setIcon(0, LICON(":/basic.png", LOPSTER_TREEVIEW_ICON));
    basicclient->set_page(++pageno);
    ui_preferences_basic_page( );
    // Client, path and applications
    PrefTreeWidgetItem *pathsapp = new PrefTreeWidgetItem(client);
    pathsapp->setText(0,qtr("Paths/Applications"));
	pathsapp->setIcon(0, LICON(":/path.png", LOPSTER_TREEVIEW_ICON));
    pathsapp->set_page(++pageno);
    ui_preferences_pathapp_page( );
    // Client, chat functionnality
    PrefTreeWidgetItem *chatfunc = new PrefTreeWidgetItem(client);
    chatfunc->setText(0,qtr("Chat functionnality"));
	chatfunc->setIcon(0, LICON(":/chat_func.png", LOPSTER_TREEVIEW_ICON));
    chatfunc->set_page(++pageno);
    ui_preferences_chatfunc_page( );
    // Client, chat appearance
    PrefTreeWidgetItem *chatappear = new PrefTreeWidgetItem(client);
    chatappear->setText(0,qtr("Chat appearance"));
	chatappear->setIcon(0, LICON(":/chat_appear.png", LOPSTER_TREEVIEW_ICON));
    chatappear->set_page(++pageno);
    ui_preferences_chatappear_page( );

    plugins_item = new PrefTreeWidgetItem(root_item);
    plugins_item->setText(0,qtr("All Plugins"));
	plugins_item->setIcon(0, LICON(":/plugins.png", LOPSTER_TREEVIEW_ICON));
    ui_preferences_init_plugins();

    PrefTreeWidgetItem *intf = new PrefTreeWidgetItem(root_item);
    intf->setText(0,qtr("Interface"));
	intf->setIcon(0, LICON(":/interface.png", LOPSTER_TREEVIEW_ICON));
    // Interface - style
    PrefTreeWidgetItem *intf_style = new PrefTreeWidgetItem(intf);
    intf_style->setText(0,qtr("Style"));
	intf_style->setIcon(0, LICON(":/style.png", LOPSTER_TREEVIEW_ICON));
    intf_style->set_page(++pageno);
    ui_preferences_style_page( );

    preferences_tree->expandItem(root_item);
    preferences_tree->expandItem(filesharing);
    preferences_tree->expandItem(client);
    preferences_tree->expandItem(plugins_item);
    preferences_tree->expandItem(intf);

    preferences_stacked->setCurrentIndex(0);

    connect(preferences_tree, SIGNAL(itemClicked(QTreeWidgetItem *,  int )),
            this, SLOT(ui_preferences_display_page(QTreeWidgetItem *)));
    connect(preferences_tree, SIGNAL(itemPressed(QTreeWidgetItem *,  int )),
            this, SLOT(ui_preferences_display_page(QTreeWidgetItem *)));
    connect(preferences_tree, SIGNAL(itemActivated(QTreeWidgetItem *,  int )),
            this, SLOT(ui_preferences_display_page(QTreeWidgetItem *)));

}

void PreferencesDialog::ui_preferences_display_page(QTreeWidgetItem *item)
{
    PrefTreeWidgetItem *pitem = static_cast<PrefTreeWidgetItem *>(item);
    preferences_stacked->setCurrentIndex(pitem->page());
}

void PreferencesDialog::ui_preferences_ok_all()
{
    ui_preferences_apply_all();
    done(QDialog::Accepted);
}

void PreferencesDialog::ui_preferences_apply_all()
{
    for (int i=0;i<applys_cancels.size();i++ )
    {
        PrefUpdater *pref = applys_cancels.at(i);
        pref->apply_pref();
    }
}

void PreferencesDialog::ui_preferences_cancel_all()
{
    for (int i=0;i<applys_cancels.size();i++ )
    {
        PrefUpdater *pref = applys_cancels.at(i);
        pref->cancel_pref();
    }
    done(QDialog::Rejected);
}


void PreferencesDialog::ui_preferences_plugin_add_page(int plugin_id, QString label, 
                                        QString desc, 
										CreatePrefPageFunc f1)
{
    //FIXME : update incon and description
    plugin_lists.append(new PluginPageInfo(plugin_id, label, desc, f1));
}
