/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.25 $
 * $Date: 2007/11/12 14:41:53 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "lcore/attribute.h"
#include "lcore/plugin.h"
#include "lcore/utils.h"
#include "lcore/search.h"
#include "lcore/files.h"
#include "lcore/mtypes.h"
#include "lcore/search.h"
#include "lcore/download.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"

#include "ui_main.hpp"
#include "ui_style.hpp"
#include "ui_search.hpp"
#include "LTabWidget.hpp"
#include "ExpandWidget.hpp"
#include "ui_support.hpp"
#include "ui_utils.hpp"

SearchBaseParameters::SearchBaseParameters(QWidget *parent)
    : QFrame(parent)
{
    setupUi(this);
    setObjectName("SearchBaseParameters"); 
    UiSupport::ui_support_populate_combo_with_types(combo_type);
    // TODO : select by default the last chosen file type

    // edit signal
    connect(push_search,   SIGNAL(clicked()), this, SLOT(search_clicked()));
    connect(combo_include, SIGNAL(editTextChanged(QString)), this, SLOT(include_text_changed()));
    connect(combo_exclude, SIGNAL(currentIndexChanged(int)), this, SLOT(include_text_changed()));
    connect(combo_include, SIGNAL(currentIndexChanged(QString)), this, SLOT(include_text_changed()));
    connect(combo_exclude, SIGNAL(activated(int)), this, SLOT(search_clicked()));
    connect(combo_include, SIGNAL(activated(int)), this, SLOT(search_clicked()));

    combo_include->setAutoFillBackground(true);
    incOrigPalette = combo_include->palette();
}
void SearchBaseParameters::include_text_changed()
{
    ui_search_unhighlight_mandatory_field();
}
void SearchBaseParameters::search_clicked()
{    
    if (combo_include->currentText().isEmpty())
    {
        ui_search_highlight_mandatory_field();
        return;
    }
    emit start_search();

}
void SearchBaseParameters::ui_search_highlight_mandatory_field()
{
    QPalette incHighPalette(incOrigPalette); 
    incHighPalette.setColor(QPalette::Base,Qt::yellow);
    combo_include->setPalette(incHighPalette);
}
void SearchBaseParameters::ui_search_unhighlight_mandatory_field()
{
    combo_include->setPalette(incOrigPalette);
}

SearchAdvancedParameters::SearchAdvancedParameters(QWidget *parent)
    : QFrame(parent)
{
    setupUi(this);
    setObjectName("SearchAdvancedParameters");
}

void SearchAdvancedParameters::add_widget(QWidget *widget)
{
    vboxLayout->addWidget(widget);
}


SearchPage::SearchPage(QWidget* parent)
    : LWidget(parent)
{
	QVBoxLayout *vboxLayout = new QVBoxLayout(this);

	LSplitter *splitter = new LSplitter(this);

    tab_results = new LTabWidget(splitter);

    QFrame *parameters = new QFrame(splitter);
    parameters->setObjectName("SearchSidePanel");
	QVBoxLayout *param_layout = new QVBoxLayout(parameters);  

    expand_param = new ExpandWidget(":/search_icon.png",parameters);
	expand_param->setExpandDirection(ExpandWidget::ED_Down);
	expand_param->setText(qtr("Keywords"));
	expand_param->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    expand_param->setToggle(true);
    bp = new SearchBaseParameters(parameters);
    
    expand_adv = new ExpandWidget(":/search_advanced.png",parameters);
	expand_adv->setExpandDirection(ExpandWidget::ED_Down);
	expand_adv->setText(qtr("Advanced settings"));
	expand_adv->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
    ap  = new SearchAdvancedParameters(parameters); 

    param_layout->addWidget(expand_param);
    param_layout->addWidget(bp);
    param_layout->addWidget(expand_adv);
	param_layout->addWidget(ap);
    param_layout->addItem(new QSpacerItem(10,30,QSizePolicy::Expanding,QSizePolicy::Expanding));

    splitter->addWidget(parameters);
	splitter->addWidget(tab_results);
    vboxLayout->addWidget(splitter);

	splitter->setStretchFactor(1,3); 

    ap->setVisible(false);
   
    // base
    ui_search_register_option(new SearchTerm(bp->get_include(), this));
    ui_search_register_option(new SearchRealm(bp->get_realm(), this));
    ui_search_register_option(new SearchExclude(bp->get_exclude(), this));
    
    // TODO : add checkbox to select on which network to search.

    // advanced
    ui_search_register_option(new SearchSize(ap->get_size_min(), ap->get_size_max(), this));
    ui_search_register_option(new SearchBitrate(ap->get_bitrate_min(), ap->get_bitrate_max(), this));
    ui_search_register_option(new SearchFrequency(ap->get_frequency_min(), ap->get_frequency_max(), this));
    ui_search_register_option(new SearchDuration(ap->get_duration_min(), ap->get_duration_max(), this)); 
    
    // Load layout 
    splitter->readSettings(QString("SearchSplitter"));

    connect(tab_results,SIGNAL(close_tab(int)),this, SLOT(ui_search_close_page(int)));
    connect(expand_param,SIGNAL(expand_toggled(bool)),bp,SLOT(setVisible(bool)));
    connect(expand_adv,SIGNAL(expand_toggled(bool)),ap,SLOT(setVisible(bool)));
    connect(bp, SIGNAL(start_search()), this, SLOT(ui_search_start()));
}

SearchPage::~SearchPage()
{
}

SearchPage* SearchPage::getSearchPage( ) 
{
    return qobject_cast <SearchPage *> (findWidgetByName("search_page"));
}

void SearchPage::ui_search_remove(search_t* search)
{
    int index;
    char* s_name = (char*) object_get_data(OBJ(search), "TreeResult");
    TreeResult *tree = qobject_cast<TreeResult *> (findWidgetByName(QString("%1").arg(s_name)));
    Q_ASSERT(tree);

    SearchPage* search_page = SearchPage::getSearchPage( );
    
    if (tree)
    {
        index = search_page->tab_results->indexOf(tree);
        if (index != -1) {
            search_page->tab_results->removeTab(index);
        } 
        delete tree;
    }
}

void SearchPage::ui_search_add_page(search_t* search)
{
    if (!search) return;
    if (object_get_data(OBJ(search), "TreeResult")) return;
    
    TreeResult *tree = new TreeResult(search,tab_results); 

    int index = tab_results->addTab(tree, qfu(OBJ(search)->name));
    tab_results->ltab_set_object(tree, OBJ(search));
    object_notify_d(OBJ(search), (ObjectFunc)ui_search_remove, NULL);
}



void SearchPage::s_ui_search_add_page(search_t* search)
{
    SearchPage* search_page = SearchPage::getSearchPage( );
    search_page->ui_search_add_page(search);
}

void SearchPage::ui_search_close_page(int page)
{
    object_t* obj = tab_results->ltab_get_object(tab_results->widget(page));
    Q_ASSERT(obj);
    object_kill(obj);
}

search_pattern_t* SearchPage::ui_search_generate_pattern_from_options( ) {
    search_pattern_t* pattern = NULL;
    SearchOption* option;

    pattern = search_pattern_new();

    for (int i= 0; i < search_options.size() ; i++) {
        option = search_options.at(i);
        option->search_option_get_value(pattern);
    }
    char* str = make_string_from_list(pattern->include, " ");
    object_set_name(OBJ(pattern), str);
    g_free(str);
    return pattern;
}


void SearchPage::ui_search_start() 
{    
    // include data has been added , now fire the search
    search_pattern_t* pattern = ui_search_generate_pattern_from_options();
    if (pattern)
    {
        search_new_from_pattern(pattern, -1);
        object_kill(OBJ(pattern));
    }
}

void SearchPage::ui_search_register_option(SearchOption *option,int plugin_id) 
{
    if (search_options.contains(option)) {
        return;
    }

    search_options.append(option);
    if (plugin_id != -1 && option->widget)
    {
        // meaning parent of option is null need to set it
        option->setParent(this);
    }

    if (option->widget)
    {
        // plugin_t* plugin = plugin_get(plugin_id);
        // FIXME : add plugin icon next to widget
        ap->add_widget(option->widget);
    }
}

void SearchPage::s_ui_search_register_option(
                        SearchOption *option,
                        int plugin_id) {

    SearchPage* search_page = SearchPage::getSearchPage( );
    search_page->ui_search_register_option(option, plugin_id);
}

/********** TERMS ********/
SearchTerm::SearchTerm(QComboBox *widget, QObject *parent)
    : SearchOption(parent)
{
    search = widget;
}

int SearchTerm::search_option_set_value(search_pattern_t* pattern)
{
    (void)pattern;
    return 1;
}

int SearchTerm::search_option_get_value(search_pattern_t* pattern)
{
    if (!pattern) return 1; 

    g_slist_foreach(pattern->include, (GFunc)g_free, NULL);
    g_slist_free(pattern->include);
    pattern->include = NULL;
    g_slist_foreach(pattern->exclude, (GFunc)g_free, NULL);
    g_slist_free(pattern->exclude);
    pattern->exclude = NULL;

    QString text = search->currentText();
    search_string_tokenize(l_strdup(qtu(text)), &pattern->include, &pattern->exclude);
    return 1;
}

/********** REALM ********/
SearchRealm::SearchRealm(QComboBox *widget, QObject *parent)
    : SearchOption(parent)
{
    realm = widget;
}

int SearchRealm::search_option_set_value(search_pattern_t* pattern) {
    if (!pattern) return 1;
    realm->setCurrentIndex(pattern->media);
    return 1;
}

int SearchRealm::search_option_get_value(search_pattern_t* pattern) {
    if (!pattern) return 1;
    pattern->media = realm->currentIndex();
    return 1;
}

/********** EXCLUDE ********/
SearchExclude::SearchExclude(QComboBox *widget, QObject *parent)
    : SearchOption(parent)
{
    exclude = widget;
}

int SearchExclude::search_option_set_value(search_pattern_t* pattern) {
    if (!pattern) return 1;
    if (pattern->exclude) {
        QString str = qfu("%1").arg(make_string_from_list(pattern->exclude, " "));
        exclude->setItemText(exclude->currentIndex(), str);
        return 1;
    } else {
        exclude->setItemText(exclude->currentIndex(), QString(""));
        return 0;
    }
}

int SearchExclude::search_option_get_value(search_pattern_t* pattern) {
    if (!pattern) return 1; 
    QString text = exclude->currentText();
    if (!text.isEmpty()) {
        search_string_tokenize(l_strdup(qtu(text)), &pattern->exclude, NULL);
        return 1;
    } else {
        return 0;
    }
}


// TODO : Set a mask for the following options, as only the needed types of value can be inputed
/********** SIZE ********/
SearchSize::SearchSize(QLineEdit *min, QLineEdit *max, QObject *parent)
    : SearchMinMax(min,max, parent)
{
}

int SearchSize::search_option_set_value(search_pattern_t* pattern) {
    attr_container_t* ac;
    attribute_t* attr;
    if (!pattern) return 1; 
    ac = ATTRC(pattern);
    attr = attr_find(ac, "pattern", "Size");
    if (attr) {
        min->setText(attr_get_refi(attr, file_size_t, 0)?
                    QString("%1").arg(print_bytes_exact(attr_get_ptr(attr, file_size_t))):"");

        max->setText(attr_get_refi(attr, file_size_t, 1)?
                    QString("%1").arg(print_bytes_exact(attr_get_ptr(attr, file_size_t)+1)):"");
        return 1;
    } else {
        return 0;
    }
}

int SearchSize::search_option_get_value(search_pattern_t* pattern) {
    file_size_t smin, smax;
    smin = smax = 0;
    if (!pattern) return 1;
    QString stmin = min->text();
    QString stmax = max->text();
    if (!stmin.isEmpty())
    {
        smin = extract_bytes(qta(stmin));
    }
    if (!stmax.isEmpty())
    {
        smax = extract_bytes(qta(stmax));
    }
    if (smin || smax) {
        attr_store(ATTRC(pattern), "pattern", "Size", smin, smax);
        return 1;
    } else {
        return 0;
    }
}

/********** DURATION ********/
SearchDuration::SearchDuration(QLineEdit *min, QLineEdit *max, QObject *parent)
    : SearchMinMax(min,max, parent)
{
}

int SearchDuration::search_option_set_value(search_pattern_t* pattern) {
    attr_container_t* ac;
    attribute_t* attr;

    if (!pattern) return 1;

    ac = ATTRC(pattern);
    attr = attr_find(ac, "pattern", "Duration");
    if (attr) {
        min->setText(attr_get_refi(attr, int, 0)?
                    QString("%1").arg(print_seconds_exact(attr_get_ptr(attr, int))):"");

        max->setText(attr_get_refi(attr, int, 1)?
                    QString("%1").arg(print_seconds_exact(attr_get_ptr(attr, int)+1)):"");
        return 1;
    } else {
        return 0;
    }
}

int SearchDuration::search_option_get_value(search_pattern_t* pattern) {
    int imin, imax;
    imin = imax = 0;
    if (!pattern) return 1; 

    QString stmin = min->text();
    QString stmax = max->text();
    if (!stmin.isEmpty())
    {
        imin = extract_duration(qta(stmin));
    }
    if (!stmax.isEmpty())
    {
        imax = extract_duration(qta(stmax));
    }
    if (imax || imax) {
        attr_store(ATTRC(pattern), "pattern", "Duration", imin, imax);
        return 1;
    } else {
        return 0;
    }
}

/********** BITRATE ********/
SearchBitrate::SearchBitrate(QLineEdit *min, QLineEdit *max, QObject *parent)
    : SearchMinMax(min,max, parent)
{
}

int SearchBitrate::search_option_set_value(search_pattern_t* pattern) {
    attr_container_t* ac;
    attribute_t* attr;
    char text[100];

    if (!pattern) return 1;

    ac = ATTRC(pattern);
    attr = attr_find(ac, "pattern", "Bitrate");
    if (attr) {
        attr_get_refi(attr, int, 0)?sprintf(text, "%d", ((int*)(attr->value))[0]):(text[0] = 0);
        min->setText(QString("%1").arg(text));
        attr_get_refi(attr, int, 1)?sprintf(text, "%d", ((int*)(attr->value))[1]):(text[0] = 0);
        max->setText(QString("%1").arg(text));
        return 1;
    } else {
        return 0;
    }
}
int SearchBitrate::search_option_get_value(search_pattern_t* pattern) {
    int imin, imax;
    imin = imax = 0;

    if (!pattern) return 1;

    QString stmin = min->text();
    QString stmax = max->text();
  
    //FIXME: force entry of only digits here and double check
    if (!stmin.isEmpty())
    {
        imin = atoi(qta(stmin));
    }
    if (!stmax.isEmpty())
    {
        imax = atoi(qta(stmax));
    }

    if (imin || imax) {
        attr_store(ATTRC(pattern), "pattern", "Bitrate", imin, imax);
        return 1;
    } else {
        return 0;
    }
}

/********** FREQUENCY ********/
SearchFrequency::SearchFrequency(QLineEdit *min, QLineEdit *max, QObject *parent)
    : SearchMinMax(min,max, parent)
{
}

int SearchFrequency::search_option_set_value(search_pattern_t* pattern) {
    attr_container_t* ac;
    attribute_t* attr;
    char text[100];

    if (!pattern) return 1;

    ac = ATTRC(pattern);
    attr = attr_find(ac, "pattern", "Frequency");
    if (attr) {
        ((int*)(attr->value))[0]?sprintf(text, "%d", ((int*)(attr->value))[0]):(text[0] = 0);
        min->setText(QString("%1").arg(text));
        ((int*)(attr->value))[1]?sprintf(text, "%d", ((int*)(attr->value))[1]):(text[0] = 0);
        max->setText(QString("%1").arg(text));
        return 1;
    } else {
        return 0;
    }
}

int SearchFrequency::search_option_get_value(search_pattern_t* pattern) {
    int imin, imax;
    imin = imax = 0;

    if (!pattern) return 1;

    QString stmin = min->text();
    QString stmax = max->text();
  
    //FIXME: force entry of only digits here and double check
    if (!stmin.isEmpty())
    {
        imin = atoi(qta(stmin));
    }
    if (!stmax.isEmpty())
    {
        imax = atoi(qta(stmax));
    }
    if (imin || imax) {
        attr_store(ATTRC(pattern), "pattern", "Frequency", imin, imax);
        return 1;
    } else {
        return 0;
    }
}

class TreeResultModelItem : public QStandardItemModel
{
public:
    TreeResultModelItem(QObject *parent=0);
    void search_item_add(search_result_t* result);
};

TreeResultModelItem::TreeResultModelItem(QObject *parent)
    : QStandardItemModel(0, 4, parent) 
{
    QStringList headers;
    headers << qtr("Filename") << qtr("Filesize") << qtr("Source") << qtr("Meta");
    setHorizontalHeaderLabels(headers); 
}

void TreeResultModelItem::search_item_add(search_result_t* result)
{
    QList<QStandardItem *> items;

    QStandardItem *parent = 0;
    QStandardItem *filename;
    QStandardItem *filesize;
    QStandardItem *source;
    QStandardItem *meta;

    if (!result) return;

    if (object_get_data(OBJ(result), "TreeResultItem")) return;    
    
    filename = new QStandardItem(PICON(result->user->ID, LOPSTER_TREEVIEW_ICON), qfu(FILE(result)->filename)); 
    filesize = new QStandardItem(qfu(print_filesize(&(FILE(result)->size))));
    source = new QStandardItem(qfu(result->user->print(result->user)));
    meta = new QStandardItem(qfu(attr_print_all(ATTRC(result))));

    items.append(filename);
    items.append(filesize);
    items.append(source);
    items.append(meta);
    
    filename->setData(QVariant::fromValue((void*)result));

    // Add the items
    if (result->master) {
        parent = static_cast< QStandardItem *>(object_get_data(OBJ(result), "TreeResultItem"));
    }
    if (parent) {
        parent->appendRow(items);
    }
    else {
        appendRow(items);
    }
    object_set_data(OBJ(result), "TreeResultItem", filename);
}

TreeResult::TreeResult(search_t* s, QWidget *parent)
        : UiTreeView(parent)
{  
    search = s;
    char* object_name;
    STORE_OBJECT("TreeResult",object_name,OBJ(search),this);

    ui_treeview_set_model(new TreeResultModelItem(this));

    action_item_t* actions[1];
    actions[0] = action_item_search("Search result actions");    
    ui_tv_handle_button(1, actions, search);
}

TreeResult::~TreeResult()
{
    object_free_data_common(OBJ(search), "TreeResult");
}

void TreeResult::tree_search_result_hide(search_result_t* result, search_t* search)
{
    // Item will be deleted automatically anyway, just free the OBJ
    object_free_data(OBJ(result),"TreeResultItem",NULL);
}

void TreeResult::s_tree_search_result_show(search_result_t* result, search_t* search)
{
  //  TreeResultItem *item;    
    Q_ASSERT(search && result);

    char* r_name = (char*) object_get_data(OBJ(search), "TreeResult");
    TreeResult *tree =  qobject_cast<TreeResult *> (findWidgetByName(QString("%1").arg(r_name)));
    Q_ASSERT(tree);

    static_cast<TreeResultModelItem *>(tree->model())->search_item_add(result);
    object_notify_d(OBJ(result), (ObjectFunc)tree_search_result_hide, search);       
}
