#ifndef _UI_SEARCH_H
#define _UI_SEARCH_H

#include "lcore/search.h"
#include "CustomWidgets.hpp"
#include "ui_treeview.hpp"
#include "ui_utils.hpp"
#include "widgets/SearchParametersBaseWidget.h"
#include "widgets/SearchParametersAdvancedWidget.h"

class QT4_UI_EXPORT SearchOption : public QObject
{
public:
    SearchOption(QObject *parent=0) 
            : QObject(parent), widget(0) {}
    virtual int search_option_get_value(search_pattern_t*)=0;
    virtual int search_option_set_value(search_pattern_t*)=0;
    QWidget *widget; // NULL, except if plugin add their own
};

class SearchTerm : public SearchOption
{
public:
    SearchTerm(QComboBox *widget, QObject *parent);    
    int search_option_get_value(search_pattern_t*);
    int search_option_set_value(search_pattern_t*);
    QComboBox *search; // FIXME : will have to change to custom widget for better history handling
};

class SearchRealm : public SearchOption
{
public:
    SearchRealm(QComboBox *widget, QObject *parent);    
    int search_option_get_value(search_pattern_t*);
    int search_option_set_value(search_pattern_t*);
    QComboBox *realm;
};

class SearchExclude : public SearchOption
{
public:
    SearchExclude(QComboBox *widget, QObject *parent);    
    int search_option_get_value(search_pattern_t*);
    int search_option_set_value(search_pattern_t*);
    QComboBox *exclude;
};

class SearchMinMax : public SearchOption
{
public:
    SearchMinMax(QLineEdit *mi, QLineEdit *ma, QObject *parent)
        : SearchOption(parent)
    {   min= mi; max = ma; }
    virtual int search_option_get_value(search_pattern_t*)=0;
    virtual int search_option_set_value(search_pattern_t*)=0;
    QLineEdit *min;
    QLineEdit *max;
};

class SearchSize : public SearchMinMax
{
public:
    SearchSize(QLineEdit *min, QLineEdit *max, QObject *parent);    
    int search_option_get_value(search_pattern_t*);
    int search_option_set_value(search_pattern_t*);
};

class SearchDuration : public SearchMinMax
{
public:
    SearchDuration(QLineEdit *min, QLineEdit *max, QObject *parent);    
    int search_option_get_value(search_pattern_t*);
    int search_option_set_value(search_pattern_t*);
};

class SearchBitrate : public SearchMinMax
{
public:
    SearchBitrate(QLineEdit *min, QLineEdit *max, QObject *parent);    
    int search_option_get_value(search_pattern_t*);
    int search_option_set_value(search_pattern_t*);
};

class SearchFrequency : public SearchMinMax
{
public:
    SearchFrequency(QLineEdit *min, QLineEdit *max, QObject *parent);    
    int search_option_get_value(search_pattern_t*);
    int search_option_set_value(search_pattern_t*);
};

class SearchBaseParameters : public QFrame, private Ui::SearchParametersBaseWidget
{
    Q_OBJECT
public:
    SearchBaseParameters(QWidget *parent = 0);   
    QComboBox *get_include()     {return combo_include;}
    QComboBox *get_exclude()     {return combo_exclude;}
    QComboBox *get_realm()       {return combo_type;}
private slots:
    void include_text_changed();
    void search_clicked();
signals:
    void start_search();
private:
    void ui_search_highlight_mandatory_field();
    void ui_search_unhighlight_mandatory_field();
private:
    QPalette incOrigPalette;    
};

class SearchAdvancedParameters : public QFrame, private Ui::SearchParametersAdvancedWidget
{
    Q_OBJECT
public:
    SearchAdvancedParameters(QWidget *parent=0);    
    QLineEdit *get_size_min()           {return line_size_min;}
    QLineEdit *get_size_max()           {return line_size_max;}
    QLineEdit *get_bitrate_min()        {return line_bitrate_min;}
    QLineEdit *get_bitrate_max()        {return line_bitrate_max;}
    QLineEdit *get_frequency_min()      {return line_frequency_min;}
    QLineEdit *get_frequency_max()      {return line_frequency_max;}
    QLineEdit *get_duration_min()       {return line_duration_min;}
    QLineEdit *get_duration_max()       {return line_duration_max;}
    void add_widget(QWidget *widget);
};

class TreeResult : public UiTreeView
{
    Q_OBJECT
public:
    TreeResult(search_t* s, QWidget *parent=0);
    ~TreeResult();
    static void s_tree_search_result_show(search_result_t* result, search_t* search);
    static void tree_search_result_hide(search_result_t* result, search_t* search);
private:
    search_t* search;
};

class LTabWidget;
class ExpandWidget;
class QT4_UI_EXPORT SearchPage : public LWidget
{
   Q_OBJECT
public:
    SearchPage(QWidget *parent = 0);
    ~SearchPage(); 
    static SearchPage* getSearchPage( );
    void loadCoreData( ) {}
    static void s_ui_search_add_page(search_t* search);
    static void s_ui_search_register_option(SearchOption *option, int plugin_id);
    static void ui_search_remove(search_t* search);

    void ui_search_add_page(search_t* search);
    void ui_search_register_option(SearchOption *option, int plugin_id = -1);
	LTabWidget *tab_results;
private:
	search_pattern_t* ui_search_generate_pattern_from_options( );
private slots:
    void ui_search_close_page(int page);
    void ui_search_start();
private:
    SearchBaseParameters *bp;
    SearchAdvancedParameters *ap;
    ExpandWidget *expand_param;
    ExpandWidget *expand_adv;
    QList <SearchOption*> search_options;
};

#endif //_UI_SEARCH_H
