/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.17 $
 * $Date: 2007/11/13 12:27:10 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QCheckBox>
#include <QFontComboBox>
#include <QFile>
#include "ui_main.hpp"
#include "ui_utils.hpp"
#include "ui_support.hpp"
#include "ui_style.hpp"


// Manage better the QFont (toString(), and QColor (intensity))

StyleWidget::StyleWidget(QWidget* parent)
    : QWidget(parent)
{
	setupUi(this);

    // By default it's not visible
    // action from main menu will make it visible
    //    setVisible(false);
    // display the list of registred styles

    QVBoxLayout *layout = new QVBoxLayout(styles_area);
    scroll_area = new QScrollArea(styles_area);
    layout->addWidget(scroll_area);

    styles_list = new QWidget(this); 
    styles_list_layout = new QVBoxLayout(styles_list);
    styles_list_layout->setSpacing(1);
    styles_list_layout->setMargin(1);
    styles_list_layout->setObjectName("styles_list_layout");
}


void StyleWidget::ui_style_generate_styles( )
{
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);  
    QList <QString> groups = style_core->ui_styles_get_group_list();
    for (int i = 0; i < groups.size() ; i++) {
        QString group_name = groups[i];

        // Create layout, and add header label current_edit_group
        // label = ui_create_header_box(_(EditGroup));
        
        
        QFrame *frame = new QFrame(styles_list);
        QSizePolicy sizePolicy3(static_cast<QSizePolicy::Policy>(5), static_cast<QSizePolicy::Policy>(0));
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(frame->sizePolicy().hasHeightForWidth());
        frame->setSizePolicy(sizePolicy3);
        frame->setFrameShape(QFrame::Box);
        frame->setFrameShadow(QFrame::Raised);
    
      //  QWidget *widget = new QWidget(styles_list);
        QVBoxLayout *vboxLayout = new QVBoxLayout(frame);

        QLabel *group = new QLabel(frame);
        QFont font;
        font.setPointSize(10);
        font.setBold(true);
        font.setWeight(75);
        group->setFont(font);
        group->setText(group_name);
        vboxLayout->addWidget(group);

        QList <StyleDef*> ordered = style_core->ui_styles_get_styles_ordered();
        for (int j=0;j<ordered.size();j++) {
              ui_style_show_style_items(frame, ordered[j], group_name, vboxLayout);
        }
        vboxLayout->setSpacing(1);
        vboxLayout->setMargin(1);
        styles_list_layout->addWidget(frame);
    }
    scroll_area->setWidget(styles_list);
    scroll_area->setWidgetResizable(true);
}


void StyleWidget::ui_style_show_style_items(QFrame *parent, StyleDef* style, 
                    const QString &current_group, QVBoxLayout *vboxLayout )
{
    if (style->des.isEmpty() || style->group.isEmpty()) return;
    if (style->group !=  current_group) return;
    
    QVBoxLayout *styles_layout = new QVBoxLayout();
    
    QHBoxLayout *title_layout = new QHBoxLayout();
    QSpacerItem *spacerItem = new QSpacerItem(10, 5, QSizePolicy::Fixed, QSizePolicy::Minimum);
    QLabel *stylename = new QLabel(parent); 
    stylename->setText(style->des); 

    title_layout->addItem(spacerItem);
    title_layout->addWidget(stylename);
   
    QHBoxLayout *items_layout = new QHBoxLayout();
    QSpacerItem *item_spacer = new QSpacerItem(20, 10, QSizePolicy::Fixed, QSizePolicy::Minimum);
    items_layout->addItem(item_spacer);

    for (int i = 0; i <  style->items.size() ; i++)
    {
        StyleItem *item = style->items[i];     
        
        ItemEditor* ie = new ItemEditor(this);
        ie->style = style;
        ie->item = item;
        
        QVBoxLayout *item_layout = new QVBoxLayout();
        QCheckBox *itemcheck = new QCheckBox(parent);
        itemcheck->setChecked(item->active);
        itemcheck->setText(item->name);
        item_layout->addWidget(itemcheck);

        ie->checked = itemcheck;

        if (item->type == STYLE_COLOR)
        {
            StyleColor *temp = qobject_cast <StyleColor *>(item);
            ColorButton *colorvalue = new ColorButton(temp->color, parent);
            item_layout->addWidget(colorvalue); 
            ie->data = colorvalue;
        }
        else if (item->type == STYLE_FONT)
        {
            StyleFont *temp = qobject_cast <StyleFont *>(item);
            QFontComboBox *fontvalue = new QFontComboBox(parent);
            item_layout->addWidget(fontvalue); 
            ie->data = fontvalue;
        }
        else if (item->type == STYLE_UNDERLINE)
        {
            StyleUnderline *temp = qobject_cast <StyleUnderline *>(item);
            QCheckBox *underlinevalue = new QCheckBox(parent);
            underlinevalue->setChecked(temp->underline);
            item_layout->addWidget(underlinevalue); 
            ie->data = underlinevalue;
        }
       // QSpacerItem *spacer = new QSpacerItem(16, 20);
      //  vboxLayout2->addItem(spacer); 

        items_layout->addLayout(item_layout);
      //  QSpacerItem *spacer = new QSpacerItem(5, 20,QSizePolicy::Fixed, QSizePolicy::Expanding);
      //  items_layout->addItem(spacer); 
        Items.append(ie);
   }
   QSpacerItem *end_spacer = new QSpacerItem(10, 10, QSizePolicy::Expanding, QSizePolicy::Minimum);
   items_layout->addItem(end_spacer);

   styles_layout->addLayout(title_layout);
   styles_layout->addLayout(items_layout);
   
   // Now add it the the parent layout
   vboxLayout->addLayout(styles_layout);
}

void StyleWidget::ui_style_apply_changes( ) {
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);   

    for (int i = 0; i < Items.size(); i++) {
        ItemEditor* ie = Items[i];
        StyleDef* style = ie->style;
        StyleItem* item = ie->item;

        if (ie->checked->isChecked()) {
            if (item->type == STYLE_COLOR) {
                StyleColor *temp = qobject_cast<StyleColor *>(item);
                QColor color;
                ColorButton *colorvalue = qobject_cast<ColorButton *>(ie->data);
                color = QColor(colorvalue->objectName());
                style_core->ui_style_set_color(style,temp,color);
            } else if (item->type == STYLE_FONT) {
                StyleFont *temp = qobject_cast<StyleFont *>(item);
                QFont font;
                QFontComboBox *fontvalue = qobject_cast<QFontComboBox *>(ie->data);
                font = fontvalue->currentFont();
                style_core->ui_style_set_font(style,temp,font);
            } else if (item->type == STYLE_UNDERLINE) {
                StyleUnderline *temp = qobject_cast<StyleUnderline *>(item);
                bool underline;
                QCheckBox *underlinevalue = qobject_cast<QCheckBox *>(ie->data);
                underline = underlinevalue->isChecked();
                style_core->ui_style_set_underline(style,temp,underline);
            }
            item->active = true;
        } else {
             style_core->ui_style_unset(style->name, item->name);
        }
    }
    style_core->ui_styles_save();
    /* {
        const char* text;
        text = gtk_label_get_text(StyleLabel);
        if (text && *text == 0) text = NULL;
        if (lopster_xml_change("ui:core:styles", text, TRUE)) ui_styles_set();
        text = gtk_label_get_text(RcLabel);
        if (RcFile != text && (!RcFile || !text || strcmp(RcFile, text)))
          ui_set_rc_file(text);
    } */
}

StyleCore::StyleCore(QObject *parent)
    : QObject(parent)
{
}

StyleCore::~StyleCore( )
{
    //FIXME : normally when deleting the core these should be deleted automatically
   /* foreach (StyleDef *value, Styles)
        delete value;
    StylesOrdered.clear(); */
}

void StyleCore::ui_style_set_underline(StyleDef* style, StyleUnderline *item, bool underline) {
    if (item->underline != underline)
    {
        item->underline = underline;
        emit styledUpdated(style->name);
    }    
}

void StyleCore::ui_style_set_font(StyleDef* style, StyleFont *item, const QFont &font) {
    if (item->font != font)
    {
        item->font = font;
        emit styledUpdated(style->name);
    }    
}

void StyleCore::ui_style_set_color(StyleDef* style, StyleColor *item, const QColor &color) {
    if (item->color != color)
    {
        item->color = color;
        emit styledUpdated(style->name);
    }    
}


StyleItem* StyleCore::ui_style_find_item(StyleDef* style, const QString &itemname) {
    if (style && !itemname.isEmpty()) {
        for (int i = 0; i < style->items.size(); i++) {
            StyleItem* item = style->items[i];
            if (item->name == itemname ) return item;
        }
    }
    return NULL;
}


QColor StyleCore::ui_style_get_color(const QString &stylename, const QString &itemname)
{
    StyleDef * style = Styles[stylename];
    if (!style)
    {
        return QColor();
    }
    for (int i = 0; i < style->items.size(); i++) {
      StyleItem* item = style->items[i];
      if (item->name == itemname ) {
        if (item->type != STYLE_COLOR)
        {
            return QColor();
        }
        else {
            StyleColor* temp = (StyleColor*) item;
            return temp->color;
        }
      }
    }
    return QColor();
}

const QFont StyleCore::ui_style_get_font(const QString &stylename, const QString &itemname)
{
    StyleDef * style = Styles[stylename];
    if (!style)
    {
        return QFont();
    }
    for (int i = 0; i < style->items.size(); i++) {
      StyleItem* item = style->items[i];
      if (item->name == itemname ) {
        if (item->type != STYLE_FONT)
        {
            return QFont();
        }
        else {
            StyleFont* temp = (StyleFont*) item;
            return temp->font;
        }
      }
    }
    return QFont();
}

bool StyleCore::ui_style_get_underline(const QString &stylename, const QString &itemname)
{
    StyleDef * style = Styles[stylename];
    if (!style)
    {
        return false;
    }
    for (int i = 0; i < style->items.size(); i++) {
      StyleItem* item = style->items[i];
      if (item->name == itemname ) {
        if (item->type != STYLE_UNDERLINE)
        {
            return false;
        }
        else {
            StyleUnderline* temp = qobject_cast<StyleUnderline *>(item);
            return temp->underline;
        }
      }
    }
    return false;
}

void StyleCore::ui_style_set_group(StyleDef* style, const QString &group) {
    if (group.isEmpty()) return;
    if (! style->group.isEmpty()) {
      StylesOrdered.removeAt(StylesOrdered.indexOf(style));
    }
    style->group = QString(group);
    StylesOrdered.append(style);

    if (StyleGroups.contains(group)) return;
    StyleGroups.append(QString(group));
}

bool StyleCore::ui_style_sets(const QString &name, const QString &item) {
    StyleDef *style = Styles[name];
    StyleItem *i;

    if (!style) {
        return false;
    }
    i = ui_style_find_item(style, item);
    return (i && i->active); 
}

void StyleCore::ui_style_unset(const QString &name, const QString &item) {
    StyleDef *style = Styles[name];
    StyleItem *i;

    if (!style) {
        return;
    }
    i = ui_style_find_item(style, item);
    if (!i) {
        return;
    }
    if (i->active) {
        i->active = false;
        emit styledUpdated(style->name);
        ui_styles_save();
    }
}

void StyleCore::ui_style_register(const QString &group, const QString &name, const QString &des, 
                    const QString &back_item,   bool bset,  const QColor &bcolor,
                    const QString &border_item, bool boset, const QColor &bocolor,
                    const QString &base_item,   bool baset, const QColor &bacolor,
                    const QString &bar_item,    bool barset,const QColor &barcolor,
                    const QString &text_item,   bool tset,  const QColor &tcolor) {

    StyleDef* style = Styles[name];
    if (!style) {
        style = new StyleDef(this,name);
        style->des = des;
        Styles[name] = style;   
    }

    if (!ui_style_find_item(style, back_item)) {
        StyleColor* item = new StyleColor(style, back_item, bset, STYLE_COLOR, bcolor); 
        style->items.append(item);       
    }
    if (!ui_style_find_item(style, border_item)) {
        StyleColor* item = new StyleColor(style, border_item, boset, STYLE_COLOR, bocolor); 
        style->items.append(item);
    }
    if (!ui_style_find_item(style, base_item)) {
        StyleColor* item = new StyleColor(style, base_item, baset, STYLE_COLOR, bacolor); 
        style->items.append(item);       
    }
    if (!ui_style_find_item(style, bar_item)) {
        StyleColor* item = new StyleColor(style, bar_item, barset, STYLE_COLOR, barcolor); 
        style->items.append(item);       
    }
    if (!ui_style_find_item(style, text_item)) {
        StyleColor* item = new StyleColor(style, text_item, tset, STYLE_COLOR, tcolor); 
        style->items.append(item);       
    }

    emit styledUpdated(style->name);
    ui_style_set_group(style, group);
}

void StyleCore::ui_style_register(const QString &group, const QString &name, const QString &des, 
                    const QString &text_item, bool tset, const QColor &tcolor,
                    const QString &back_item, bool bset, const QColor &bcolor,
                    const QString &font_item, bool fset, const QFont &font,
                    const QString &underline_item, bool uset, bool underline) {

    StyleDef* style = Styles[name];
    if (!style) {
        style = new StyleDef(this,name );
        style->des = des;
        Styles[name] = style;   
    }

    if (!ui_style_find_item(style, text_item)) {
        StyleColor* item = new StyleColor(style, text_item, tset, STYLE_COLOR, tcolor); 
        style->items.append(item);       
    }
    if (!ui_style_find_item(style, back_item)) {
        StyleColor* item = new StyleColor(style, back_item, bset, STYLE_COLOR, bcolor); 
        style->items.append(item);       
    }
    if (!ui_style_find_item(style, font_item)) {
        StyleFont* item = new StyleFont(style, font_item, fset, STYLE_FONT, font); 
        style->items.append(item);       
    } 
    if (!ui_style_find_item(style, underline_item)) {
        StyleUnderline* item = new StyleUnderline(style, underline_item, uset, STYLE_UNDERLINE, underline); 
        style->items.append(item);       
    }

    emit styledUpdated(style->name);
    ui_style_set_group(style, group);
}

void StyleCore::ui_style_register(const QString &group, const QString &name, const QString &des, 
                    const QString &text_item, bool tset, const QColor &tcolor,
                    const QString &back_item, bool bset, const QColor &bcolor,
                    const QString &font_item, bool fset, const QFont &font) {

    StyleDef* style = Styles[name];
    if (!style) {
        style = new StyleDef(this,name );
        style->des = des;
        Styles[name] = style;   
    }

    if (!ui_style_find_item(style, text_item)) {
        StyleColor* item = new StyleColor(style, text_item, tset, STYLE_COLOR, tcolor); 
        style->items.append(item);       
    }
    if (!ui_style_find_item(style, back_item)) {
        StyleColor* item = new StyleColor(style, back_item, bset, STYLE_COLOR, bcolor); 
        style->items.append(item);       
    }
    if (!ui_style_find_item(style, font_item)) {
        StyleFont* item = new StyleFont(style, font_item, fset, STYLE_FONT, font); 
        style->items.append(item);       
    } 

    emit styledUpdated(style->name);
    ui_style_set_group(style, group);
}

void StyleCore::ui_style_register_color(const QString &group, const QString &name, const QString &des,
                       const QString &first_name, bool set, const  QColor &color) {
    StyleDef* style;

    style = Styles[name];

    if (style) {
        if (ui_style_find_item(style, first_name)) {
            return;
        }
    } else {
        style = new StyleDef(this, name );
        style->des = des;
        Styles[name] = style;   
    }

    StyleItem* item = new StyleColor(style, first_name, set, STYLE_COLOR, color); 
    style->items.append(item);
    emit styledUpdated(style->name);
    ui_style_set_group(style, group);
}

void StyleCore::ui_style_register_font(const QString &group, const QString &name, const QString &des,
                       const QString &first_name, bool set,  const QFont &font) {
    StyleDef* style;

    style = Styles[name];

    if (style) {
        if (ui_style_find_item(style, first_name)) {
            return;
        }
    } else {
        style = new StyleDef(this, name);
        style->des = des;
        Styles[name] = style;   
    }

    StyleFont* item = new StyleFont(style, first_name, set, STYLE_FONT, font); 
    style->items.append(item);
    emit styledUpdated(style->name);
    ui_style_set_group(style, group);
}

void StyleCore::ui_style_register_underline(const QString &group, const QString &name, const QString &des,
                       const QString &first_name, bool set,  bool underlined) {
    StyleDef* style;

    style = Styles[name];

    if (style) {
        if (ui_style_find_item(style, first_name)) {
            return;
        }
    } else {
        style = new StyleDef(this, name);
        style->des = des;
        Styles[name] = style;   
    }

    StyleUnderline* item = new StyleUnderline(style, first_name, set, STYLE_UNDERLINE,  underlined); 
    style->items.append(item);
    emit styledUpdated(style->name);
    ui_style_set_group(style, group);
}


void StyleCore::xml_get_style(xmlNodePtr xnode) {
    xmlNodePtr xsub;
    xmlChar* name;
    StyleDef* style;

    name = xmlGetProp(xnode, ( xmlChar*) "name");
    if (!name) return;
    style = Styles[qfu((char*) name)];
    if (!style) {
        g_log("QT4_STYLE", G_LOG_LEVEL_WARNING,
                    "style '%s' not found", name); 
        return;
    }
    xsub = xnode->xmlChildrenNode;

    while (xsub != NULL) {
        int r=0,g=0,b=0;
    
        if (!strcmp((char*) xsub->name, "Color")) {
            xmlChar* attr = xmlGetProp(xsub, ( xmlChar*)"name");
            xmlChar* attr2 = xmlGetProp(xsub, ( xmlChar*)"active");
            StyleColor *item = qobject_cast<StyleColor *>(ui_style_find_item(style,qfu((char*)attr)));
            if (attr) xmlFree(attr);
            if (item) {
                attr = xmlGetProp(xsub, ( xmlChar*)"red");
                if (attr) {
                    r = atoi((char*) attr);
                    xmlFree(attr);
                }
                attr = xmlGetProp(xsub, ( xmlChar*)"green");
                if (attr ) {
                    g = atoi((char*)attr);
                    xmlFree(attr);
                } 
                attr = xmlGetProp(xsub, ( xmlChar*)"blue");
                if (attr) {
                    b = atoi((char*)attr);
                    xmlFree(attr);
                } 
                item->color.setRgb(r,g,b);
                item->active = (attr2 && (!strcasecmp((char*)attr2, "yes") || !strcmp((char*)attr2, "1")));
            }
            if (attr2) xmlFree(attr2);
        } 
        else if (!strcmp((char*) xsub->name, "Font")) {
            xmlChar* attr = xmlGetProp(xsub, ( xmlChar*)"name");
            xmlChar* attr2 = xmlGetProp(xsub,( xmlChar*) "active");
            StyleFont *item = qobject_cast<StyleFont *>(ui_style_find_item(style, qfu((char*)attr)));
            if (attr) xmlFree(attr);
            if (item) {
                attr = xmlGetProp(xsub, ( xmlChar*)"font");
                if (attr) {
                    item->font = QFont(qfl((char*)attr));
                    xmlFree(attr);
                }
                item->active = (attr2 && (!strcasecmp((char*)attr2, "yes") || !strcmp((char*)attr2, "1")));
            }
            if (attr2) xmlFree(attr2);
        } 
        else if (!strcmp((char*)xsub->name, "Underline")) {
            xmlChar* attr = xmlGetProp(xsub, ( xmlChar*)"name");
            xmlChar* attr2 = xmlGetProp(xsub, ( xmlChar*)"active");
            StyleUnderline *item = qobject_cast<StyleUnderline *>(ui_style_find_item(style, qfu((char*)attr)));
            if (item) {
                if (attr) xmlFree(attr);
                attr = xmlGetProp(xsub, ( xmlChar*)"underline");
                item->underline = (attr && (!strcasecmp((char*)attr, "yes") || !strcmp((char*)attr, "1")));
                item->active = (attr2 && (!strcasecmp((char*)attr2, "yes") || !strcmp((char*)attr2, "1")));
            }
            if (attr) xmlFree(attr);
            if (attr2) xmlFree(attr2);
        } else if (strcmp((char*)xsub->name, "text")) {
            g_log("QT4_STYLE", G_LOG_LEVEL_WARNING,
                "unknown node in xml file: '%s'", xsub->name);
        }
        xsub = xsub->next;
    }
    xmlFree(name);
    emit styledUpdated(style->name);
}

void StyleCore::xml_load_styles(xmlDocPtr xfile, xmlNodePtr xroot) {
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);
    xmlNodePtr xsub = xroot->xmlChildrenNode;
    (void)xfile;
    while (xsub != NULL) {
        if (!strcmp((char*)xsub->name, "Style")) {
            style_core->xml_get_style(xsub);
        } else if (strcmp((char*)xsub->name, "text")) {
            g_log("QT4_STYLE", G_LOG_LEVEL_WARNING,
                "unknown node in xml file: '%s'", xsub->name);
        }
        xsub = xsub->next;
    }
}


void StyleCore::xml_save_style_item(StyleItem* item, xmlNodePtr xnode) {
    xmlNodePtr xsub;

    if (item->type == STYLE_COLOR) {
        StyleColor* temp = (StyleColor*) item;
        xsub = xmlNewChild(xnode, NULL, (xmlChar*) "Color", NULL);
        xmlNewProp(xsub, (xmlChar*) "name", (xmlChar*)  qtl(temp->name));
        xmlNewProp(xsub, (xmlChar*) "active", item->active? (xmlChar*) "yes": (xmlChar*) "no");
        xmlNewProp(xsub, (xmlChar*) "red",  (xmlChar*) qtl(QString("%1").arg(temp->color.red())));
        xmlNewProp(xsub, (xmlChar*) "green",  (xmlChar*) qtl(QString("%1").arg(temp->color.green())));
        xmlNewProp(xsub, (xmlChar*) "blue",  (xmlChar*) qtl(QString("%1").arg(temp->color.blue())));
    } else if (item->type == STYLE_FONT) {
        StyleFont* temp = (StyleFont*) item;
        xsub = xmlNewChild(xnode, NULL, (xmlChar*) "Font", NULL);
        xmlNewProp(xsub, (xmlChar*)"name", (xmlChar*) qtl(temp->name));
        xmlNewProp(xsub, (xmlChar*)"active", temp->active?(xmlChar*)"yes":(xmlChar*)"no");
        xmlNewProp(xsub, (xmlChar*) "font", (xmlChar*)  qtl(temp->font.toString()) );
    } else if (item->type == STYLE_UNDERLINE) {
        StyleUnderline* temp = (StyleUnderline*) item;
        xsub = xmlNewChild(xnode, NULL, (xmlChar*)"Underline", NULL);
        xmlNewProp(xsub, (xmlChar*)"name", (xmlChar*) qtl(temp->name));
        xmlNewProp(xsub, (xmlChar*)"active", temp->active?(xmlChar*)"yes":(xmlChar*)"no");
        xmlNewProp(xsub, (xmlChar*)"underline", temp->underline?(xmlChar*)"yes":(xmlChar*)"no");
    }
}

void StyleCore::xml_save_style( StyleDef* style, xmlNodePtr xnode) {
    xmlNodePtr xsub = xmlNewChild(xnode, NULL, (xmlChar*)"Style", NULL);
    xmlNewProp(xsub, (xmlChar*)"name", (xmlChar*)qtu(style->name));
    for (int i = 0 ;i < style->items.size() ; i++)
    {
        xml_save_style_item(style->items[i], xsub);
    }

    //  (void)key;
}

void StyleCore::xml_save_styles(xmlDocPtr xfile, xmlNodePtr xroot) {
    (void)xfile;
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);
    foreach(StyleDef* style, style_core->Styles )
        style_core->xml_save_style(style,xroot);
}

void StyleCore::xml_load_settings(xmlDocPtr xfile, xmlNodePtr xroot) {
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);

    xmlNodePtr xsub = xroot->xmlChildrenNode;
    xmlChar* attr;

    while (xsub != NULL) {
        if (!strcmp((char*) xsub->name, "StyleFile")) {
            attr = xmlNodeListGetString(xfile, xsub->xmlChildrenNode, 1);
            if (attr && *attr) lopster_xml_change("ui:core:styles", (char*) attr, TRUE);
            if (attr) xmlFree(attr);
        } else if (!strcmp((char*)xsub->name, "rc_file")) {
            attr = xmlNodeListGetString(xfile, xsub->xmlChildrenNode, 1);
            if (attr && *attr) style_core->rc_file = qfu((char*)attr);
            if (attr) xmlFree(attr);
        }
        xsub = xsub->next;
    }
}

void StyleCore::xml_save_settings(xmlDocPtr xfile, xmlNodePtr xroot) {
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);
    const char* filename = lopster_xml_find("ui:core:styles");
    (void)xfile;
    if (filename)
        xmlNewTextChild(xroot, NULL, (xmlChar*)"StyleFile", (xmlChar*)filename);
    if (!style_core->rc_file.isEmpty())
        xmlNewTextChild(xroot, NULL, (xmlChar*)"rc_file", (xmlChar*)qtu(style_core->rc_file));
}

void StyleCore::ui_style_parse_qss( ) {
    QFile file(rc_file);
    if (!file.exists())
    {
        //FIXME : message box
        return;
    }
    file.open(QFile::ReadOnly);
    QString styleSheet = QLatin1String(file.readAll());
    qApp->setStyleSheet(styleSheet);
}

void StyleCore::ui_styles_save(void) {
    if (!m_loaded) return;
    lopster_xml_save("ui:core:styles", 5000);
}

void StyleCore::ui_styles_load(void) {
    m_loaded = false;
    lopster_xml_load("ui:core:styles");
    m_loaded = true;
}


void StyleCore::ui_styles_init() {
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);   
    lopster_xml_register_home("ui:core:settings", SDIR("qt4") "settings.xml",
                "Settings", StyleCore::xml_load_settings, StyleCore::xml_save_settings);
    lopster_xml_register_home("ui:core:styles", SDIR("qt4") "styles.xml",
                "Styles", StyleCore::xml_load_styles, StyleCore::xml_save_styles);
    if (!lopster_xml_load("ui:core:settings")) {
        style_core->rc_file = QString("lopster2.qss");
    }
    style_core->ui_style_parse_qss();
}
