/* Copyright (C) 2007 Lopster Team
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _UI_STYLE_H_
#define _UI_STYLE_H_

#include <QHash>
#include <QList>
#include <QCheckBox>
#include <QScrollArea>
#include "lcore/xml_handler.h"
#include "lcore/utils.h"
#include "lcore/object.h"
#include "lcore/l_i18n.h"
#include "ui_utils.hpp"
#include "widgets/StyleWidget.h"


#define STYLE_NONE       0 ///< no style
#define STYLE_COLOR      1 ///< color
#define STYLE_FONT       2 ///< font name
#define STYLE_UNDERLINE  3 ///< underlined?


typedef void (*StyleFunc)(const char* name, gpointer data);

class StyleItem  : public QObject
{
    Q_OBJECT
public:
    StyleItem(QObject *parent, QString name, bool active ,int type)
        : QObject(parent)
    {
        this->name = name;
        this->active = active;
        this->type = type;
    }
    QString name;
    bool active;
    int type;
};

class StyleColor : public StyleItem
{
    Q_OBJECT
public:
    StyleColor(QObject *parent, QString name, bool active ,int type , QColor color ) 
        : StyleItem(parent,name,active,type) 
    {
        this->color = color ;
    }
    QColor color;
};

class StyleFont : public StyleItem
{
    Q_OBJECT
public:
    StyleFont(QObject *parent, QString name, bool active,int type, QFont font ) 
        : StyleItem(parent,name,active ,type) 
    {
        this->font = font;
    }
    QFont font;
};

class StyleUnderline : public StyleItem
{
    Q_OBJECT
public:
    StyleUnderline(QObject *parent, QString name, bool active,int type,  bool underline ) 
        : StyleItem(parent,name,active,type) 
    {
        this->underline = underline ;
    }
    bool underline;
};

class StyleDef : public QObject
{
    Q_OBJECT
public:
    StyleDef(QObject *parent, QString name )
        : QObject(parent) 
    {
        this->name = name;
    }

    StyleDef(QObject *parent, QString name, QString group, QString des)
        : QObject(parent) 
    {
        this->name = name; 
        this->group = group; 
        this->des = des; 
    }
    QString name;
    QString group;
    QString des;
    QList <StyleItem*> items;
};

class ItemEditor : public QObject
{
    Q_OBJECT
public:
    ItemEditor(QObject *parent) : QObject(parent){}
    StyleDef *style;
    StyleItem *item;
    QCheckBox *checked;
    QWidget *data;
};

class QT4_UI_EXPORT StyleCore : public QObject
{
    Q_OBJECT
public:
    StyleCore(QObject *parent=0);
    ~StyleCore( );

    void ui_style_set_underline(StyleDef* style, StyleUnderline *item, bool underline);
    void ui_style_set_font(StyleDef* style, StyleFont *item, const QFont &font);
    void ui_style_set_color(StyleDef* style, StyleColor *item, const QColor &color);
    
    StyleItem* ui_style_find_item(StyleDef* style, const QString &name);

    void ui_style_set_group(StyleDef* style, const QString &group);

    void ui_style_register(const QString &group, const QString &name, const QString &des, 
                    const QString &back_item,   bool bset,   const QColor &bcolor,
                    const QString &border_item, bool boset,  const QColor &bocolor,
                    const QString &base_item,   bool baset,  const QColor &bacolor,
                    const QString &bar_item,    bool barset, const QColor &barcolor,
                    const QString &text_item,   bool tset,   const QColor &tcolor);
    
    void ui_style_register(const QString &group, const QString &name, const QString &des, 
                    const QString &text_item, bool tset, const QColor &tcolor,
                    const QString &back_item, bool bset, const QColor &bcolor,
                    const QString &font_item, bool fset, const QFont  &font,
                    const QString &underline_item, bool uset, bool underline);

    void ui_style_register(const QString &group, const QString &name, const QString &des, 
                    const QString &text_item, bool tset, const QColor &tcolor,
                    const QString &back_item, bool bset, const QColor &bcolor,
                    const QString &font_item, bool fset, const QFont &font);

    void ui_style_register_color(const QString &group, const QString &name, const QString &des, 
                            const QString &first_name, bool set, const QColor &color);
    void ui_style_register_font(const QString &group, const QString &name, const QString &des, 
                            const QString &first_name, bool set, const QFont &font);
    void ui_style_register_underline(const QString &group, const QString &name, const QString &des, 
                            const QString &first_name, bool set, bool underlined);

    bool ui_style_sets(const QString &name, const QString &item);
    void ui_style_unset(const QString &name, const QString &item);

    QColor ui_style_get_color(const QString &stylename, const QString &itemname);
    const QFont ui_style_get_font(const QString &stylename, const QString &itemname);
    bool ui_style_get_underline(const QString &stylename, const QString &itemname);

    void xml_get_style(xmlNodePtr xnode);
    void xml_save_style_item(StyleItem* item, xmlNodePtr xnode);
    void xml_save_style( StyleDef* style, xmlNodePtr xnode);

    static void xml_load_styles(xmlDocPtr xfile, xmlNodePtr xroot);
    static void xml_save_styles(xmlDocPtr xfile, xmlNodePtr xroot);   

    static void xml_load_settings(xmlDocPtr xfile, xmlNodePtr xroot);
    static void xml_save_settings(xmlDocPtr xfile, xmlNodePtr xroot);

    void ui_styles_save(void);
    void ui_styles_load(void);

    void ui_styles_init();
    
    void ui_style_parse_qss();

    QList <QString> ui_styles_get_group_list() { return StyleGroups;}
    QHash <QString, StyleDef*> ui_styles_get_styles() { return Styles;}
    QList <StyleDef*>  ui_styles_get_styles_ordered() { return StylesOrdered;}

signals:
    void styledUpdated(const QString &name );

public:
    QHash <QString, StyleDef*> Styles ;
    QList <StyleDef*> StylesOrdered ;
    QList <QString> StyleGroups;
    QString rc_file; 
private:
    bool m_loaded;
};

class StyleWidget : public QWidget, private Ui::StyleWidget
{
    Q_OBJECT
public:
    StyleWidget(QWidget *parent = 0 );
    void ui_style_generate_styles( );
    void ui_style_apply_changes( );
private:
    void ui_style_show_style_items(QFrame *parent, StyleDef* style, 
                        const QString &current_group, QVBoxLayout *vboxLayout );
private:
    QList <ItemEditor*> Items;
    QScrollArea *scroll_area;
    QWidget *styles_list;
    QVBoxLayout *styles_list_layout;
};

/*@}*/

#endif
