/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.18 $
 * $Date: 2007/11/19 15:39:30 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "lcore/l_i18n.h"
#include "lcore/string_list.h"
#include "ui_main.hpp"
#include "ui_support.hpp"


CharsetTree::CharsetTree(QWidget *parent) 
    : QWidget(parent)
{
    setWindowFlags(Qt::Tool);
    QStringList headers;
    headers << ("Charset") << ("Description");
    tree = new QTreeWidget(this);
    tree->setHeaderLabels(headers);    
    charset_init_tree();

    QVBoxLayout *layout = new QVBoxLayout;
    layout->addWidget(tree);
    setLayout(layout);

    connect(tree, SIGNAL(itemActivated(QTreeWidgetItem *,int)), this, SLOT(charset_item_selected(QTreeWidgetItem *,int)));
}


void CharsetTree::charset_init_tree() {

    QTreeWidgetItem *i1 = new QTreeWidgetItem(tree);
    i1->setText(0,"European languages");  
    append_charset(tree, i1, "ASCII", QString(""));
    append_charset(tree, i1, "ISO-8859-1", "(Western Europe)");
    append_charset(tree, i1, "ISO-8859-2", "(Central/Eastern Europe)");
    append_charset(tree, i1, "ISO-8859-3", "(South Europe)");
    append_charset(tree, i1, "ISO-8859-4", "(North Europe)");
    append_charset(tree, i1, "ISO-8859-5", "(Cyrillic)");
    append_charset(tree, i1, "ISO-8859-7", "(Greek)");
    append_charset(tree, i1, "ISO-8859-9", "(Turkish)");
    append_charset(tree, i1, "ISO-8859-10", "(Nordic)");
    append_charset(tree, i1, "ISO-8859-13", "(Baltic)");
    append_charset(tree, i1, "ISO-8859-14", "(Celtic)");
    append_charset(tree, i1, "ISO-8859-15", "(Western Europe)");
    append_charset(tree, i1, "ISO-8859-16", "(South-Eastern Europe)");
    append_charset(tree, i1, "KOI8-R", "(Cyrillic)");
    append_charset(tree, i1, "KOI8-U", "(Cyrillic)");
    /*     append_charset(tree, i1, NULL, "KOI8-RU"); */
    append_charset(tree, i1, "CP1250", "(MS Central/Eastern Europe)");
    append_charset(tree, i1, "CP1251", "(MS Cyrillic)");
    append_charset(tree, i1, "CP1252", "(MS Western Europe)");
    append_charset(tree, i1, "CP1253", "(MS Greek)");
    append_charset(tree, i1, "CP1254", "(MS Turkish)");
    append_charset(tree, i1, "CP1257", "(MS Baltic)");
    /*     append_charset(tree, i1, NULL, "CP850"); */
    /*     append_charset(tree, i1, NULL, "CP866"); */
    append_charset(tree, i1, "MacRoman", QString(""));
    append_charset(tree, i1, "MacCentralEurope", QString(""));
    append_charset(tree, i1, "MacIceland", QString(""));
    append_charset(tree, i1, "MacCroatian", QString(""));
    append_charset(tree, i1, "MacRomania", QString(""));
    append_charset(tree, i1, "MacCyrillic", QString(""));
    append_charset(tree, i1, "MacUkraine", QString(""));
    append_charset(tree, i1, "MacGreek", QString(""));
    append_charset(tree, i1, "MacTurkish", QString(""));
    append_charset(tree, i1, "Macintosh", QString(""));

    QTreeWidgetItem *i2 = new QTreeWidgetItem(tree);
    i2->setText(0,"Semitic languages");
    append_charset(tree, i2, "ISO-8859-6", "(Arabic)");
    append_charset(tree, i2, "ISO-8859-8", "(Hebrew)");
    append_charset(tree, i2, "CP1255", "(MS Hebrew)");
    append_charset(tree, i2, "CP1256", "(MS Arabic)");
    /*     append_charset(tree, i2, NULL, "CP862"); */
    append_charset(tree, i2, "MacHebrew", QString(""));
    append_charset(tree, i2, "MacArabic", QString(""));

    QTreeWidgetItem *i3 = new QTreeWidgetItem(tree);
    i3->setText(0,"Japanese");
    append_charset(tree, i3, "EUC-JP", QString(""));
    append_charset(tree, i3, "SHIFT_JIS", QString(""));
    append_charset(tree, i3, "CP932", QString(""));
    append_charset(tree, i3, "ISO-2022-JP", QString(""));
    append_charset(tree, i3, "ISO-2022-JP-2", QString(""));
    append_charset(tree, i3, "ISO-2022-JP-1", QString(""));

    QTreeWidgetItem *i4 = new QTreeWidgetItem(tree);
    i4->setText(0,"Chinese");
    append_charset(tree, i4, "EUC-CN", QString(""));
    append_charset(tree, i4, "HZ", QString(""));
    append_charset(tree, i4, "GBK", QString(""));
    append_charset(tree, i4, "GB18030", QString(""));
    append_charset(tree, i4, "EUC-TW", QString(""));
    append_charset(tree, i4, "BIG5", QString(""));
    /*     append_charset(tree, i4, NULL, "CP950"); */
    append_charset(tree, i4, "BIG5-HKSCS", QString(""));
    append_charset(tree, i4, "ISO-2022-CN", QString(""));
    append_charset(tree, i4, "ISO-2022-CN-EXT", QString(""));

    QTreeWidgetItem *i5 = new QTreeWidgetItem(tree);
    i5->setText(0,"Korean");
    append_charset(tree, i5, "EUC-KR", QString(""));
    /*     append_charset(tree, i5, NULL, "CP949"); */
    append_charset(tree, i5, "ISO-2022-KR", QString(""));
    append_charset(tree, i5, "JOHAB", QString(""));

    QTreeWidgetItem *i6 = new QTreeWidgetItem(tree);
    i6->setText(0,"Armenian");
    append_charset(tree, i6, "ARMSCII-8", QString(""));

    QTreeWidgetItem *i7 = new QTreeWidgetItem(tree);
    i7->setText(0,"Georgian");
    append_charset(tree, i7, "Georgian-Academy", QString(""));
    append_charset(tree, i7, "Georgian-PS", QString(""));

    QTreeWidgetItem *i8 = new QTreeWidgetItem(tree);
    i8->setText(0,"Tajik");
    append_charset(tree, i8, "KOI8-T", QString(""));

    QTreeWidgetItem *i9 = new QTreeWidgetItem(tree);
    i9->setText(0,"Thai");
    append_charset(tree, i9, "TIS-620", QString(""));
    append_charset(tree, i9, "ISO-8859-11", QString(""));
    append_charset(tree, i9, "CP874", QString(""));
    append_charset(tree, i9, "MacThai", QString(""));

    QTreeWidgetItem *i10 = new QTreeWidgetItem(tree);
    i10->setText(0,"Laotian");
    append_charset(tree, i10, "MuleLao-1", QString(""));
    append_charset(tree, i10, "CP1133", QString(""));

    QTreeWidgetItem *i11 = new QTreeWidgetItem(tree);
    i11->setText(0,"Vietnamese");
    append_charset(tree, i11, "VISCII", QString(""));
    append_charset(tree, i11, "TCVN", QString(""));
    append_charset(tree, i11, "CP1258", QString(""));

    QTreeWidgetItem *i12 = new QTreeWidgetItem(tree);
    i12->setText(0,"Platform specifics");
    append_charset(tree, i12, "HP-ROMAN8", QString(""));
    append_charset(tree, i12, "NEXTSTEP", QString(""));

    QTreeWidgetItem *i13 = new QTreeWidgetItem(tree);
    i13->setText(0,"Full Unicode");
    append_charset(tree, i13, "UTF-8", QString(""));
    append_charset(tree, i13, "UCS-2", QString(""));
    append_charset(tree, i13, "UCS-2BE", QString(""));
    append_charset(tree, i13, "UCS-2LE", QString(""));
    append_charset(tree, i13, "UCS-4", QString(""));
    append_charset(tree, i13, "UCS-4BE", QString(""));
    append_charset(tree, i13, "UCS-4LE", QString(""));
    append_charset(tree, i13, "UTF-16", QString(""));
    append_charset(tree, i13, "UTF-16BE", QString(""));
    append_charset(tree, i13, "UTF-16LE", QString(""));
    append_charset(tree, i13, "UTF-32", QString(""));
    append_charset(tree, i13, "UTF-32BE", QString(""));
    append_charset(tree, i13, "UTF-32LE", QString(""));
    append_charset(tree, i13, "UTF-7", QString(""));
    append_charset(tree, i13, "C99", QString(""));
    append_charset(tree, i13, "JAVA", QString(""));
}

void CharsetTree::append_charset(QTreeWidget *widget, QTreeWidgetItem *parent,
                              QString charset, QString label)
{
    QTreeWidgetItem *item;

    if (parent == NULL)
    {
        item = new QTreeWidgetItem(widget);
    }
    else {
        item = new QTreeWidgetItem(parent);
    }
    item->setText(0,charset);
    item->setText(1,label);
}

void CharsetTree::charset_item_selected(QTreeWidgetItem *item, int column)
{
    int index = tree->indexOfTopLevelItem(item);
    if (index != -1) {
        QMessageBox::warning(this, qtr("Choose a charset"),
                       qtr("Please select a charset, not a group"), QMessageBox::Ok );
    }
    else {
        emit charset_selected(item);    
    }
}

CharsetWidget::CharsetWidget(QWidget *parent) : 
    QWidget(parent)
{   
    QHBoxLayout *layout = new QHBoxLayout();

    //TODO : fill in the 3 or 4 latest charset selected (need to add it to pref)
    combo_charset = new QComboBox(this);    
    chooser = new QToolButton(this);

    popup = new CharsetTree(chooser);
    popup->setVisible(false);

    layout->addWidget(combo_charset);
    layout->addWidget(chooser);
    setLayout(layout);

    connect(chooser, SIGNAL(clicked()), this, SLOT(on_select_charset()));
    connect(popup, SIGNAL(charset_selected(QTreeWidgetItem *)), 
                        this, SLOT(charset_selected(QTreeWidgetItem *))); 
}

void CharsetWidget::on_select_charset()
{
    QPoint pos = combo_charset->mapToGlobal(QPoint(0, combo_charset->height()));
    popup->move(pos);
    popup->show();
}

void CharsetWidget::charset_selected(QTreeWidgetItem *item)
{
    popup->setVisible(false);
    charset_set_selection(item->text(0));
}

void CharsetWidget::charset_set_selection(QString charset)
{
    int index = combo_charset->findText(charset);
    if (index == -1)
    {
        combo_charset->addItem(charset);
        combo_charset->setCurrentIndex(combo_charset->count() - 1);
    }
    else {
        combo_charset->setCurrentIndex(index);
    }
}


QString CharsetWidget::charset_get_selection()
{
    return combo_charset->currentText();
}

const QString UiSupport::GetMediaStock(int id)
{
    return MediaIcons[id];
}

void UiSupport::ui_support_populate_combo_with_types(QComboBox *combo )
{
    int index = 0;
    for (int i = 0; i <= MEDIA_ANY; i++) {
        if (i == MEDIA_UNKNOWN || i == MEDIA_FOLDER) continue;
        QString name = QString("%1").arg(int2media(i, 0));
        QIcon icon = QIcon(MediaIcons[i]);
        combo->insertItem(index,icon,name);
        index++;
    }
}


ColorButton::ColorButton(QColor c, QWidget *parent) 
    : QToolButton(parent) 
{
    this->setObjectName(c.name());
    setIconColor(c);
    connect(this,SIGNAL(clicked()),this,SLOT(setColor()));
}

void ColorButton::setIconColor(QColor c)
{
    QPixmap pix(25, 25);
    pix.fill(c);
    this->setIcon(pix);
}

void ColorButton::setColor()
{
    QColor color = QColorDialog::getColor(Qt::green, this);
    if (color.isValid()) {
        this->setObjectName(color.name());
        setIconColor(color);
    }
}

class StringListModel : public QStandardItemModel
{
public:
    StringListModel(const QString &title, QObject *parent);
    void add_string(const char* item);
    void remove_string(const char* item);
};

StringListModel::StringListModel(const QString &title, QObject *parent)
    : QStandardItemModel(0,1,parent)
{
    setHeaderData(0, Qt::Horizontal, title);    
}

void StringListModel::add_string(const char* item)
{
    QStandardItem *str = new QStandardItem(qfu(item));
    str->setData(QVariant::fromValue((void*)item));
    appendRow(str);
}
void StringListModel::remove_string(const char* item)
{
    //TODO
}

StringListWidget::StringListWidget(QWidget *parent)
            : QWidget(parent)
{
    setupUi(this);
    connect(tool_add,SIGNAL(clicked()),this,SLOT(add_item()));
    connect(line_add,SIGNAL(returnPressed()),this,SLOT(add_item()));
    connect(tree_string_list,
            SIGNAL(clicked(const QModelIndex &)),
            this, SLOT(string_selected(const QModelIndex &)));
}

void StringListWidget::add_item()
{
    QString name = line_add->text();
    if (!name.isEmpty())
    {
        string_list_add(string_list, qtu(name));
    }
}

void StringListWidget::on_item_removed(char* item, UiTreeView *tree) {
    static_cast<StringListModel *>(tree->model())->remove_string(item);
}

void StringListWidget::on_item_added(const char* item, UiTreeView *tree) {
    static_cast<StringListModel *>(tree->model())->add_string(item);
}

void StringListWidget::string_selected(const QModelIndex &index)
{
    QStandardItem *item = static_cast<StringListModel *>(tree_string_list->model())->itemFromIndex(index);
    if (item)
    {
        line_add->setText(item->text());
    }
}

void StringListWidget::setup(const QString &title, const QString &title2, 
                                    char* str_name, const char* actions)
{
    string_list = str_name;
    if (title.isEmpty())
    {
        label_add->setText(qtr(string_list_get_label(str_name)));
    }
    else 
    {
        label_add->setText(title);
    }
    tree_string_list->ui_treeview_set_model(new StringListModel(title2,this));

    string_list_monitor(string_list, (StringListFunc)on_item_added,
		      (StringListFunc)on_item_removed, tree_string_list);

    action_item_t* act = action_item_search(actions);
    char* n = g_strdup(str_name);
    tree_string_list->ui_tv_handle_button(1, &act, n);
    
    const GList* items = string_list_get_items(string_list);

    string_list_foreach(string_list, (StringListFunc)on_item_added, tree_string_list);
}

QLineEdit *StringListWidget::get_line()
{
    return line_add;
}

void StringListWidget::set_header_visible(bool set)
{
    tree_string_list->header()->setVisible(set);
}
