/* Copyright (C) 2007 Lopster Team
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2007/11/19 15:39:30 $
 * $Author: sid4u $
 */

#ifndef _UI_SUPPORT_H_
#define _UI_SUPPORT_H_

#include <QtGui>
#include "lcore/files.h"
#include "lcore/user.h"
#include "lcore/mtypes.h"
#include "ui_utils.hpp"
#include "widgets/StringListManager.h"

/** @defgroup UISupportModule Functions which can save lots of coding
 * @ingroup UserInterface
 */
/*@{*/


#define ICON_LOPSTER_MTYPE_OTHER   ":/mtype_other.png"  ///< unknown type
#define ICON_LOPSTER_MTYPE_AUDIO   ":/mtype_audio.png"  ///< audio
#define ICON_LOPSTER_MTYPE_VIDEO   ":/mtype_video.png"  ///< video
#define ICON_LOPSTER_MTYPE_APP     ":/mtype_application.png"    ///< application
#define ICON_LOPSTER_MTYPE_IMAGE   ":/mtype_image.png"  ///< image
#define ICON_LOPSTER_MTYPE_DOC     ":/mtype_document.png"    ///< documents
#define ICON_LOPSTER_MTYPE_FOLDER  ":/mtype_folder.png" ///< folders
#define ICON_LOPSTER_MTYPE_ALL     ":/mtype_all.png"    ///< all mediatypes

class QT4_UI_EXPORT CharsetTree : public QWidget
{
    Q_OBJECT
public:
    CharsetTree(QWidget *parent=0);

    void charset_init_tree();
    void append_charset(QTreeWidget *widget, QTreeWidgetItem *parent,
                                            QString charset, QString label);  
private slots:
    void charset_item_selected(QTreeWidgetItem *item, int column);

signals:
    void charset_selected(QTreeWidgetItem *item);
private:
    QTreeWidget *tree;
};

class QT4_UI_EXPORT CharsetWidget : public QWidget
{
    Q_OBJECT
public:
    CharsetWidget(QWidget *parent=0);
    QString charset_get_selection();
    void charset_set_selection(QString charset);
private slots:
    void on_select_charset();
    void charset_selected(QTreeWidgetItem *item);
private:
    QComboBox *combo_charset;
    QToolButton *chooser;
    CharsetTree *popup;
};

const QString MediaIcons[MEDIA_ANY+1] = {
    ICON_LOPSTER_MTYPE_OTHER,
    ICON_LOPSTER_MTYPE_AUDIO,
    ICON_LOPSTER_MTYPE_VIDEO,
    ICON_LOPSTER_MTYPE_APP,
    ICON_LOPSTER_MTYPE_IMAGE,
    ICON_LOPSTER_MTYPE_DOC,
    ICON_LOPSTER_MTYPE_FOLDER, //FIXME
    ICON_LOPSTER_MTYPE_ALL,
};

class ColorButton : public QToolButton
{
    Q_OBJECT
public:
    ColorButton(QColor c, QWidget *parent);
    void setIconColor(QColor c);
private slots:
    void setColor( );
};

class QT4_UI_EXPORT StringListWidget : public QWidget, private Ui::StringListManager
{
    Q_OBJECT
public:
    StringListWidget(QWidget *parent=0);
    static void on_item_removed(char* item, UiTreeView *tree);
    static void on_item_added(const char* item, UiTreeView *tree);
    void setup(const QString &title, const QString &title2, 
                                    char* str_name, const char* actions);
    QLineEdit *get_line();
    void set_header_visible(bool set);
private slots:
    void add_item();
    void string_selected(const QModelIndex &);
private:
    char* string_list;
};

class UiSupport : public QObject
{
public:
    static const QString GetMediaStock(int id);
    static void ui_support_populate_combo_with_types(QComboBox *combo);
};


/*@}*/

#endif
