/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.7 $
 * $Date: 2007/09/11 07:32:11 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QHash>
#include <QThread>
#include "ui_main.hpp"
#include "ui_timer.hpp"


//TODO : clean and change some functions

static QHash<uint, LopsterUiTimer*> timer_handles;
static uint timer_next = 0;

LopsterUiTimer::LopsterUiTimer(guint interval, GSourceFunc function, gpointer data)
                : QTimer( )
{
    this->interval = interval;
    this->function = function;
    this->data = data; 
    setInterval(this->interval);
    connect(this, SIGNAL(timeout()), this, SLOT(OnTimerReached()));    
}

LopsterUiTimer::~LopsterUiTimer()
{

}

void LopsterUiTimer::OnTimerReached( )
{
    if (function(data) == false)
    {
        stop();        
    } 
    
}

void LopsterUiTimer::customEvent(QEvent* event)
{
    if (event->type() == ( (QEvent::Type)(QEvent::User+TIMER_EVENT_ID)) )
    {
        start();
    }
}

uint LopsterUiTimer::qt_timeout_add(guint interval, GSourceFunc function, gpointer data)
{
    LopsterApp *l_app = static_cast<LopsterApp *>(QCoreApplication::instance());
    if (!l_app)
    {
        qWarning("Timeout Add error - App is NULL");
        return 0;
    } 
    timer_next++;
    // QT doesn't allow other thread than QTread to start timer
    // lopster send some thread timer events from it's modules (ie library , object, xml)
    if (  QThread::currentThread () != l_app->ui_app_get_current_thead()) {
        TimerEvent *event = new TimerEvent(timer_next,interval, function,data);
        QCoreApplication::postEvent(qApp, event);        
    }
    else {
        LopsterUiTimer *timer = new LopsterUiTimer(interval, function, data);
        timer_handles.insert( timer_next, timer );
        timer->start();
    }
    return timer_next;
}

uint LopsterUiTimer::qt_idle_add(GSourceFunc function, gpointer data)
{
    LopsterApp *l_app = static_cast<LopsterApp *>(QCoreApplication::instance()); 
    if (!l_app)
    {
        qWarning("Timeout Idle Add error - App is NULL");
        return 0;
    }    
    timer_next++;
    // QT doesn't allow other thread than QTread to start idle timer
    //  same as upper
    if (  QThread::currentThread () != l_app->ui_app_get_current_thead()) {
        TimerEvent *event = new TimerEvent(timer_next,0, function,data);
        QCoreApplication::postEvent(qApp, event);
    }
    else {
        LopsterUiTimer *timer = new LopsterUiTimer(0, function, data); 
        timer_handles.insert( timer_next, timer ); //For idle
        timer->start();
    }
    return timer_next;
}

gboolean LopsterUiTimer::qt_timeout_remove(uint id)
{
    LopsterApp *l_app = static_cast<LopsterApp *>(QCoreApplication::instance());
    if (!l_app)
    {
        qWarning("Timeout remove error - App is NULL");
        return false;
    }
    LopsterUiTimer* timer = timer_handles[id];
    if (timer)
    {
        // QT doesn't allow other thread than QTread to stop timer
        // same as upper
        if (  QThread::currentThread () != l_app->ui_app_get_current_thead()) {
            TimerRemoveEvent *event = new TimerRemoveEvent(id);
            QCoreApplication::postEvent(qApp, event);
        }
        else {
            timer->stop();
            // delete timer;
        }
    }
    return TRUE;
}

bool TimerEventFilter::eventFilter(QObject *obj, QEvent *event)
{
    if (event->type() == ( (QEvent::Type)(QEvent::User+TIMER_EVENT_ID))) {
         TimerEvent *e = static_cast<TimerEvent *>(event);
         LopsterUiTimer *timer = new LopsterUiTimer(e->interval, e->function, e->data); 
         timer_handles.insert( e->id, timer ); //For idle
         timer->start();
         // delete e;
         return true;
    } 
    else if (event->type() == ( (QEvent::Type)(QEvent::User+TIMER_REMOVE_EVENT_ID))) {
         TimerRemoveEvent *e = static_cast<TimerRemoveEvent *>(event);
         LopsterUiTimer *timer = timer_handles[e->id];
         timer->stop();
         // delete e;
         return true;
    }
    else {
         // standard event processing
         return QObject::eventFilter(obj, event);
    }
}

void LopsterUiTimer::qt_timers_cleanup()
{
    foreach (LopsterUiTimer *value, timer_handles)
        delete value;
}
