/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.11 $
 * $Date: 2007/11/13 09:47:16 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QStandardItemModel>
#include <QSortFilterProxyModel>
#include "lcore/utils.h"
#include "lcore/l_i18n.h"
#include "ui_main.hpp"
#include "ui_treeview.hpp"


void UiAction::on_item_action() {
    Q_ASSERT(action);
    Q_ASSERT(action->action);

    if (action->flags & AI_GLOBAL) {
        action->action(NULL, action->user_data);
    } else {
        GList* s = UiTreeView::_select_items(tree_menu->selection, action);
        GList* objects = UiTreeView::_real_objects(action, s);
        action->action(objects, action->user_data);
    }
}

UiTreeView::UiTreeView(QWidget *parent) 
    : QTreeView(parent)
{
    handle_context = false;
    _filtered = false;
}

UiTreeView::~UiTreeView()
{
    g_free(this->actions);
}

void UiTreeView::ui_treeview_set_model(QStandardItemModel *model, bool set)
{
    tree_model = model;
    if (set)
    {
        setModel(model);
    }
    else {
        _filtered = true;
    }
}

// Just get the depth in the tree (shouldn't normally use too much of time)
int UiTreeView::ui_tree_view_get_item_depth(QStandardItem *item)
{
    QStandardItem *parent = item->parent();
    int level = 1;
    while (parent)
    {
        parent = parent->parent();
        level++;
    }
    return level;
}

void UiTreeView::_inherit_properties(action_item_t* dest, action_item_t* src) {
    if (!dest || !src || dest == src) return;
    if (!(dest->flags & AI_LOCAL_DATA)) dest->user_data = src->user_data;
    if (!dest->convert) dest->convert = src->convert;
}

GList* UiTreeView::_real_objects(action_item_t* item, GList* selection) {
    static GList* result = NULL;

    if (!item->convert) return selection;
    if (result) g_list_free(result);
    result = NULL;
    for (; selection; selection = selection->next) {
        void* object = selection->data;
        void* real_obj = item->convert(object, item->user_data);
        result = g_list_append(result, real_obj);
    }
    return result;
}

action_item_t* UiTreeView::_real_actions(action_item_t* actions, GList* selection) {
    action_item_t* new_actions;
    GList* s;

    // return self if this is no link
    if (!actions || !actions->link) return actions;

    // get the list of real object
    s = _real_objects(actions, selection);
    // get the linked actions
    new_actions = actions->link(s, actions->user_data);
    // inherit the user data + convert function
    _inherit_properties(new_actions, actions);

    return new_actions;
}

void UiTreeView::count_func(QStandardItem *item, int depth, GList** list) {
    void* object;

    if (depth == 0 || depth == ui_tree_view_get_item_depth(item)) {
        QVariant v = item->data();
        if ( !v.isValid())
        {
            return;
        }
        object = v.value<void*>();
        (*list) = g_list_append(*list, object);
    }
}

GList* UiTreeView::ui_get_depth_selection(int depth) {
    GList* result = NULL;
    for (int i=0; i<tree_selection.size() ; i++ )
    {
        QStandardItem *item = tree_selection.at(i);
        count_func(item,depth,&result);
    }
    return result;
}

GList* UiTreeView::_select_items(GList* sel, action_item_t* action) {
    static GList* result = NULL;
    GList* dlist;

    if (!action->check) return sel;
    if (result) {
        g_list_free(result);
        result = NULL;
    }
    for (dlist = sel; dlist; dlist = dlist->next) {
        void* real_obj = action->convert?
        action->convert(dlist->data, action->user_data):dlist->data;
        if (action->check(real_obj, action->user_data))
        result = g_list_append(result, dlist->data);
    }
    if ((action->flags & AI_TOGGLE) && result == NULL) {
        //   SelectionInverted = TRUE;
        return sel;
    } else {
        //  SelectionInverted = FALSE;
        return result;
    }
}


TreeMenu *UiTreeView::ui_create_view_popup_rec(action_item_t* actions,
                         GList* selection, unsigned num) {
    TreeMenu *menu;
    unsigned rnum;
    GList* dlist;
    bool last_item_was_separator = TRUE;  // dont start with seps

    if (!actions) return NULL;

    menu = new TreeMenu(this);
    for (dlist = actions->children; dlist; dlist = dlist->next) {
        QMenu *submenu;
        UiAction *item;

        action_item_t* child = (action_item_t*) dlist->data;
        action_item_t* real_child;

        // inherit the user data + convert function
        _inherit_properties(child, actions);
        // get real child in case of action link
        real_child = _real_actions(child, selection);

        if (real_child) {
            // get the number of valid items
            GList* s = _select_items(selection, real_child);
            rnum = g_list_length(s);
            // if no valid items, continue if HIDE_0 flag is set
            if (rnum == 0 && (real_child->flags & AI_HIDE_0)) continue;
        } else {
            rnum = 0;   // not used, avoid compiler warnings
        }

        // CREATE THE ITEM
        if (!real_child) {
            // link that cannot expand, set original name and deactivate
            Q_ASSERT(child->name); // a separator should never link
            item = new UiAction(qfu(child->name), menu);
            last_item_was_separator = false;
        } else if (!real_child->name) {
            // separator
            if (last_item_was_separator) continue;
            item =  new UiAction(menu);
            /*item->setSeparator(true);*/
            //item = gtk_menu_item_new();
            last_item_was_separator = true; 
        } else if (real_child->children) {
            // parent node, use the name of the original child
            item = new UiAction(qfu(child->name), menu); //gtk_menu_item_new_with_label(_(child->name));
            last_item_was_separator = false;
        } else {
            QString l;

            if (num > 1 && !(real_child->flags & AI_GLOBAL))
                l = qfu("[%1] %2%2").arg(rnum).arg(child->name).arg(
                        real_child->action?"":" (NI)");
            else
                l = qfu("%1%2").arg(child->name).arg(
                        real_child->action?"":" (NI)");

            if (real_child->flags & AI_TOGGLE) {
            item = new UiAction(l, menu); item->setCheckable(true); 
            // TODO : handle these kind of actions
            /*  if (real_child->flags & AI_RADIO)
            gtk_check_menu_item_set_draw_as_radio
            (GTK_CHECK_MENU_ITEM(item), TRUE);
            if (SelectionInverted || rnum < num)
            gtk_check_menu_item_set_active
            (GTK_CHECK_MENU_ITEM(item), TRUE);
            if (rnum < num)
            gtk_check_menu_item_set_inconsistent
            (GTK_CHECK_MENU_ITEM(item), TRUE);*/
            } else {
                item = new UiAction(qfu("%1").arg(l), menu); 
            }
            last_item_was_separator = false;
        }

        menu->addAction(item);

        // SETUP THE ITEM
        if (!real_child || !child->name || 
            (rnum == 0 && !(real_child->flags & AI_GLOBAL))) {
            item->setEnabled(false);
        } else if (rnum > 1 && (real_child->flags & AI_RESTRICT_1)) {
            item->setEnabled(false);
        } else if (real_child->action) {
            item->setEnabled(true);
            item->setLopsterAction(real_child);
            // item->setListData(selection);        
            connect(item,SIGNAL(triggered(bool)), item, SLOT(on_item_action())); 
        } else if (real_child->children) {
            submenu = ui_create_view_popup_rec(real_child, selection, num);
            if (submenu) {
                item->setMenu(submenu);
            }
            else {
                item->setEnabled(false);
            }
        } else {
            item->setEnabled(false);
        }
    }

    // FIXME:
    // if last item was separator, remove it...
    return menu;
}

TreeMenu *UiTreeView::ui_create_view_popup( ) {
    unsigned num_popup;
    int i1;
    GList* selection = NULL;
    TreeMenu *popup;

    if (num_actions == 0) return NULL;

    TreeMenu **spopup = new TreeMenu *[num_actions];

    num_popup = 0;
    for (i1 = num_actions-1; i1 >= 0; i1--) {
        unsigned cnt;
        action_item_t* a;

        if (actions[i1] && !(actions[i1]->flags & AI_LOCAL_DATA))
            actions[i1]->user_data = this->user_data;

        // select all nodes of a particular depth (or all if num_actions==1)
        selection = ui_get_depth_selection(num_actions==1?0:i1+1);
        cnt = g_list_length(selection);

        // at least show one popup, even if no item selected
        if (cnt > 0 || (i1 == 0 && num_popup == 0)) {
            a = _real_actions(actions[i1], selection);
            spopup[i1] = ui_create_view_popup_rec(a, selection, cnt);
        } else {
            spopup[i1] = 0;
        }
        if (spopup[i1]) {
            num_popup++;
            spopup[i1]->selection = selection;
        } else {
            g_list_free(selection);
        }
    }

    if (num_popup == 0) {
        popup = NULL;
    } else if (num_popup == 1) {
        popup = NULL;
        for (i1 = 0; i1 < (int)num_actions; i1++) {
            if (spopup[i1]) {
                popup = spopup[i1];
                break;
            }
        }
    } else { // FIXME : Not sure that would work
        popup = new TreeMenu(this);
        for (i1 = 0; i1 < (int)num_actions; i1++) {
            if (spopup[i1]) {
                QMenu *item = popup->addMenu(qfu(actions[i1]->name));
                item->addMenu(spopup[i1]);
            }
        }
    }
    return popup;
}

void UiTreeView::selectionChanged(const QItemSelection &selected, const QItemSelection &deselected)
{
    tree_selection.clear();
    QModelIndexList index_list = selected.indexes();
    
    int list_size = index_list.size();

    for (int i=0;i<list_size; i++)
    {
        QModelIndex index = index_list.at(i);
        if (!index.isValid())
        {
            continue;
        }
        if (index.column() != 0)
        {
            continue;
        }
        QStandardItem *item;
        if (_filtered)
        {
            //FIXME : shouldn't be forced like that here, instead pass an abstract proxy model
            QSortFilterProxyModel *proxy_model = static_cast< QSortFilterProxyModel *>(model());           
            item = static_cast< QStandardItem * >
                            (tree_model->itemFromIndex(proxy_model->mapToSource(index)));
        }
        else
        {
            item = tree_model->itemFromIndex(index);
        }
        if (item)
        {
            tree_selection.append(item);
        }
    }
    QAbstractItemView::selectionChanged(selected,deselected);
}

void UiTreeView::contextMenuEvent(QContextMenuEvent *e) {
    if (handle_context)
    {
        QMenu *pop = ui_create_view_popup();
        if (pop) {
            pop->exec(e->globalPos());
        }
    }
}

// Assign lopster core actions as popup actions on a TreeView
// This assume that the Model defined for that tree handles data in a proper way
void UiTreeView::ui_tv_handle_button(unsigned num_actions,
			 action_item_t** actions, void* user_data) {
    unsigned i1;
    this->actions = NULL;

    Q_ASSERT(num_actions == 0 || actions);
    
    if (num_actions > 0) {
        this->actions =
                (action_item_t**) g_malloc(sizeof(*this->actions) * num_actions);
        for (i1 = 0; i1 < num_actions; i1++) {
            this->actions[i1] = actions[i1];
        }
    }

    this->num_actions = num_actions;
    this->user_data =  user_data;
    this->handle_context = true;
}

