#ifndef _UI_TREEVIEW_H_
#define _UI_TREEVIEW_H_

#include <QTreeView>
#include <QMenu>
#include <QAction>
#include <QList>
#include <glib.h>
#include "lcore/action_item.h"
#include "ui_utils.hpp"

/****
    * QT4 version of Lopster Tree, that can handle Lopster default actions
    * It is a QTreeView which handle a QStandardItemModel as model
    * But each class that needs a UiTreeView should subclass a QStandardItemModel of their own
    * Add method to update (create, delete) data. and set it with ui_treeview_set_model
    */
class QStandardItemModel;
class QStandardItem;

class TreeMenu : public QMenu
{
public:
    TreeMenu(QWidget * parent) : QMenu(parent) {}
    TreeMenu() : QMenu() {}
    ~TreeMenu() { g_list_free(selection); }
    GList* selection;
};

class QT4_UI_EXPORT UiTreeView : public QTreeView
{
    Q_OBJECT
public:

    UiTreeView(QWidget *parent=0);
    ~UiTreeView();
    
    int ui_tree_view_get_item_depth(QStandardItem *item);

    static void _inherit_properties(action_item_t* dest, action_item_t* src);
    static GList* _real_objects(action_item_t* item, GList* selection);
    static action_item_t* _real_actions(action_item_t* actions, GList* selection);
    void count_func(QStandardItem *item, int depth , GList** list);
    GList* ui_get_depth_selection(int depth);
    static GList* _select_items(GList* sel, action_item_t* action);

    TreeMenu *ui_create_view_popup_rec(action_item_t* actions,
                         GList* selection, unsigned num);
    TreeMenu *ui_create_view_popup( );

    void ui_tv_handle_button(unsigned num_actions, action_item_t** actions, void* user_data);

    void ui_treeview_set_model(QStandardItemModel *model, bool set=true);
    void contextMenuEvent(QContextMenuEvent *e);
    void selectionChanged(const QItemSelection &selected, const QItemSelection &deselected);

private:      
    // Keep an up to date list of item selected
    QList<QStandardItem *> tree_selection;
    QStandardItemModel *tree_model;
    bool _filtered;
    int num_actions;
    void* user_data;
    bool handle_context;
    action_item_t** actions;
    int SelectionDepth ;
};

class  QT4_UI_EXPORT UiAction : public QAction
{
   Q_OBJECT
public:
    UiAction(TreeMenu *parent) 
       : QAction(parent), tree_menu(parent) 
    { 
       setSeparator(true);
    }
    UiAction(QString action, TreeMenu *parent) 
       : QAction(action, parent) , tree_menu(parent) 
    {
    }
    void setLopsterAction(action_item_t* action) {this->action = action;}
    // void setListData(GList* selection) {this->selection = selection;}
    // GList* selection;
    action_item_t* action;
    TreeMenu *tree_menu;
private slots:
     void on_item_action();
};

#endif // _UI_TREEVIEW_H_
