/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */

/* $Revision: 1.19 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QPainter>
#include <QPixmap>
#include "lcore/plugin.h"
#include "lcore/buffer.h"
#include "lcore/b_socket.h"
#include "ui_main.hpp"
#include "ui_utils.hpp"
#include "upload_view.hpp"
#include "ui_style.hpp"


enum {
  COLOR_BACK_GROUND,
  COLOR_BORDER,
  COLOR_ACTIVE,
  COLOR_FINISHED,
  COLOR_PROGRESS,
  COLOR_SIZE,
};

#define FIND_UPLOAD_VIEW(v) \
            v =  qobject_cast<UploadView *> (findWidgetByName(QString("upload_view"))); \
            Q_ASSERT(v); 

#define FIND_UPLOAD_TREE_VIEW(t) \
            t =  qobject_cast<UploadTreeView *> (findWidgetByName(QString("upload_tree_view"))); \
            Q_ASSERT(t); 

QList<upload_t*> Uploads_list;
volume_t* TheUploadVolume;
rate_t* TheUploadRate;


class UploadItem : public QStandardItem
{
public:
    UploadItem(const QString &text)
        : QStandardItem(text) {
        m_upload = NULL;
    }
    ~UploadItem() {
        if (m_upload)
        {
            object_free_data(OBJ(m_upload), "UploadEntry", NULL);   
        }

    }
    void set_upload(upload_t* upload) {
        this->m_upload = upload;
        this->setData(QVariant::fromValue((void*)upload));
        object_set_data(OBJ(upload), "UploadEntry", this);
    }
    upload_t* upload() {return m_upload;}
private:
    upload_t* m_upload;
};

class UploadItemModel : public QStandardItemModel
{
public:
    UploadItemModel(QObject *parent=0);
    void upload_node_add(upload_t* upload);
    void upload_node_update(upload_t* upload);
    void upload_node_delete(upload_t* upload);
};

UploadItemModel::UploadItemModel(QObject *parent)
    : QStandardItemModel(0,1,parent)
{
    QStringList headers;
    headers << qtr("File") << qtr("Size") << qtr("Source")
            << qtr("Progress") << qtr("Rate") << qtr("ETA");
    setHorizontalHeaderLabels(headers);  
}

void UploadItemModel::upload_node_add(upload_t* upload)
{
    QList<QStandardItem *> items;

    UploadItem *filename = static_cast<UploadItem *> 
                                (object_get_data(OBJ(upload), "UploadEntry"));
    if (filename) return;
    
    file_t* file = upload->file;       
    filename = new UploadItem(qfu(file->filename));
    QStandardItem *size = new QStandardItem(qfu(print_filesize(&(file->size))));
    QStandardItem *user = new QStandardItem(
                            PICON(upload->user->ID, LOPSTER_TREEVIEW_ICON),
                            upload->user->print(upload->user)
                            );
    QStandardItem *progress = new QStandardItem();
    QStandardItem *rate = new QStandardItem();
    QStandardItem *ETA = new QStandardItem();
     
    filename->set_upload(upload);
    
    items.append(filename);
    items.append(size);
    items.append(user);
    items.append(progress);
    items.append(rate);
    items.append(ETA);

    appendRow(items);
    upload_node_update(upload);
}
void UploadItemModel::upload_node_update(upload_t* upload)
{
    UploadItem  *filename;
    QStandardItem *size;
    QStandardItem *user;
    QStandardItem *progress;
    QStandardItem *rate;
    QStandardItem *ETA;

    filename = static_cast< UploadItem *>
                    (object_get_data(OBJ(upload), "UploadEntry"));
    if (!filename) return;

    QModelIndex model_index = indexFromItem(filename);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    
    
    // We have the row, let get all the items and update them
    size = item(row, 1); 
    user = item(row, 2); 
    progress = item(row, 3); 
    rate = item(row, 4); 
    ETA = item(row, 5); 

    file_t* file = upload->file;    
    if (upload->cur_segment)
    {
        segment_t* seg = upload->cur_segment;
        int r = socket_get_rate(SOCK(upload));
        int l = r>0?(seg->stop-seg->start-seg->size)/r:0;

        progress->setText(QString::number(qrand()));
        rate->setText(qfu("%1").arg(print_speed(&r)));
        ETA->setText(qfu(print_duration_unit(&l)));
    }
    else
    {
        progress->setText(QString("0"));
        rate->setText(QString(""));
        ETA->setText(QString(""));
    }
}
void UploadItemModel::upload_node_delete(upload_t* upload)
{
    UploadItem  *filename = static_cast<UploadItem *>
                        (object_get_data(OBJ(upload), "UploadEntry"));
    if (!filename) return;

    QModelIndex model_index = indexFromItem(filename);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    removeRow(row);  
}

UploadView::UploadView(QWidget *parent )
    :   LWidget(parent)
{
   	setupUi(this);
    QPixmap plib; 
    plib.load(":/upload.png");
    plib = plib.scaled(LOPSTER_LABEL_ICON,Qt::KeepAspectRatio);
    label_upload_pix->setPixmap(plib);
    QFontMetrics fm = fontMetrics();   
    label_bandwidth->setFixedSize(QSize(fm.width(qtr("999999999999 KB/s")), fm.lineSpacing()));
    label_uploaded->setFixedSize(QSize(fm.width(qtr("999999999999 KB/s")), fm.lineSpacing()));
    label_limit->setFixedSize(QSize(fm.width(qtr("999999999999 KB/s")), fm.lineSpacing()));
    update = new UploadInfoLabels;
    update->bandwidth = label_bandwidth;
    update->uploaded = label_uploaded;
    update->limit = label_limit;
    upload_tree_view->set_update_labels(update);
}

UploadView::~UploadView()
{
	delete update;
}

void UploadView::loadCoreData( )
{
    TheUploadVolume = volume_find("Upload Bandwidth Limit");
    TheUploadRate = uploads_get_rate();
    upload_tree_view->ui_upload_set_monitors( );
}

UploadTreeView::UploadTreeView(QWidget *parent)
    : UiTreeView(parent)
{
    setItemDelegate(new UploadTreeViewDelegate(parent, this));
    setSelectionBehavior(QAbstractItemView::SelectRows);
    setAlternatingRowColors(true);
    setRootIsDecorated(false);

    ui_treeview_set_model(new UploadItemModel(this));

    action_item_t* actions =
      action_item_search("Upload actions");
    ui_tv_handle_button( 1, &actions, NULL);
}

void UploadTreeView::ui_upload_set_monitors( )
{
    object_monitor_d(OBJ(TheUploadVolume), (ObjectFunc)s_on_band_changed, update->bandwidth);
    object_monitor_d(OBJ(TheUploadVolume), (ObjectFunc)s_on_limit_changed, update->limit);
    object_monitor_d(OBJ(TheUploadVolume), (ObjectFunc)s_on_load_changed, update->uploaded);
    object_set_dirty(OBJ(TheUploadVolume));
}


/*
static GtkWidget* on_upload_tip(upload_t* upload, unsigned depth) {
  GtkWidget* table;
  int line = 0;
  char* str;

  (void)depth;

  table = gtk_table_new(1, 2, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);

  ui_table_add_file(GTK_TABLE(table), &line, upload->file);
  ui_table_add_sep(GTK_TABLE(table), &line);
  ui_table_add_user(GTK_TABLE(table), &line, upload->user);
  ui_table_add_sep(GTK_TABLE(table), &line);
  
  str = l_printf("%s", ctime(&upload->request_time));
  str[strlen(str)-1] = 0;
  ui_table_add_line(GTK_TABLE(table), &line, _("Requested"), str);
  
  ui_table_add_line(GTK_TABLE(table), &line, _("Priority"),
                    l_printf("%.2f", upload->priority));

  gtk_widget_show(table);
  return table;
}

*/

void UploadTreeView::s_on_limit_changed(limit_t* limit, QLabel *label) {
    int l = limit_get_limit(limit);
    label->setText(qtr("%1").arg(print_speed(&l)));
}

void UploadTreeView::s_on_band_changed(limit_t* limit, QLabel *label) {
    int r = rate_get_rate(TheUploadRate);
    label->setText(qfu("%1").arg(print_speed(&r)));
    (void)limit;
}

void UploadTreeView::s_on_load_changed(volume_t* volume, QLabel *label) {
    file_size_t size = volume_get_sum(volume);
    label->setText(qfu("%1").arg(print_filesize(&size)));
}

void UploadTreeView::ui_upload_hide(upload_t* upload)
{
    UploadTreeView *tree;
    FIND_UPLOAD_TREE_VIEW(tree);
    static_cast<UploadItemModel *>(tree->model())->upload_node_delete(upload);
}

void UploadTreeView::ui_upload_update(upload_t* upload)
{
    UploadTreeView *tree;
    FIND_UPLOAD_TREE_VIEW(tree);
    static_cast<UploadItemModel *>(tree->model())->upload_node_update(upload);
}

void UploadTreeView::ui_upload_show(upload_t* upload)
{
    UploadTreeView *tree;
    FIND_UPLOAD_TREE_VIEW(tree);  
    Uploads_list.append(upload);
    static_cast<UploadItemModel *>(tree->model())->upload_node_add(upload);
    object_notify_d(OBJ(upload), (ObjectFunc)ui_upload_hide, NULL);
}

void UploadTreeView::ui_upload_periodic_update(void)
{
    UploadTreeView *tree;
    FIND_UPLOAD_TREE_VIEW(tree);
    for (int i = 0; i < Uploads_list.size() ; i++ )
    {
        upload_t* upload = Uploads_list.at(i);
        if (upload->cur_segment) {
            static_cast<UploadItemModel *>(tree->model())->upload_node_update(upload);
        }
    }
}


UploadTreeViewDelegate::UploadTreeViewDelegate(QWidget *parent, UploadTreeView *tree_view) 
                        : QItemDelegate(parent)
{
    this->tree_view = tree_view;

    // These colors will be updated when the style is changed
    ucolor[COLOR_BACK_GROUND] = QColor(0xe7 /*39*/, 0xe7/*39*/, 0xe7/*39*/);
    ucolor[COLOR_BORDER] = QColor(0xcf/*5e*/, 0x41/*25*/, 0x41/*25*/);
    ucolor[COLOR_FINISHED] = QColor(0xef/*00*/, 0xc8/*00*/, 0x65/*00*/);
    ucolor[COLOR_ACTIVE] = QColor(0xe4/*00*/, 0xd9/*00*/, 0xb4/*00*/);
    ucolor[COLOR_PROGRESS] = QColor(0xcf/*5e*/, 0x41/*25*/, 0x41/*25*/);
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);     
    connect(style_core,SIGNAL(styledUpdated(const QString &)), this, SLOT(style_changed(const QString &)));
}

void UploadTreeViewDelegate::style_changed(const QString &style)
{
    QColor color;
    StyleCore *style_core;
    LOPSTER_APP_GET_STYLE(style_core);   

    if ( style_core->ui_style_sets(style, qtr("Background"))) {
        ucolor[COLOR_BACK_GROUND] = style_core->ui_style_get_color(style, qtr("Background") );
    } else {
        ucolor[COLOR_BACK_GROUND] = QColor();
    }

    if ( style_core->ui_style_sets(style, qtr("Border"))) {
        ucolor[COLOR_BORDER] = style_core->ui_style_get_color(style, qtr("Border") );
    } else {
        ucolor[COLOR_BORDER] = QColor();
    }

    if ( style_core->ui_style_sets(style, qtr("Finished"))) {
        ucolor[COLOR_FINISHED] = style_core->ui_style_get_color(style, qtr("Finished") );
    } else {
        ucolor[COLOR_FINISHED] = QColor();
    }

    if ( style_core->ui_style_sets(style, qtr("In-progress"))) {
        ucolor[COLOR_ACTIVE] = style_core->ui_style_get_color(style, qtr("In-progress") );
    } else {
        ucolor[COLOR_ACTIVE] = QColor();
    }

    if ( style_core->ui_style_sets(style, qtr("Percentage"))) {
        ucolor[COLOR_PROGRESS] = style_core->ui_style_get_color(style, qtr("Percentage") );
    } else {
        ucolor[COLOR_PROGRESS] = QColor();
    }
}

void UploadTreeViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
                  const QModelIndex &index ) const
{
   
    if (index.column() != 3) {
        QItemDelegate::paint(painter, option, index);
        return;
    }
    int row = index.row();
    UploadItem *cu = (UploadItem *)static_cast<UploadItemModel *>(tree_view->model())-> item(0,row);
    if (!cu)
    {
        Q_ASSERT(false); // Should never happen :)
    }

    QRect rect = option.rect;

    if (cu->upload() && (cu->upload())->cur_segment) {
    GList* dlist;
    upload_t* upload = cu->upload(); 
    file_size_t size = upload->file->size;
    int x, w, h;
    int x2;
    segment_t* seg;
    unsigned buf = buffer_avail(b_socket_get_out_buffer(BSOCK(upload), 0));

    w = rect.width();
    h = rect.height();

    // the background

    if (ucolor[COLOR_BACK_GROUND].isValid()) {
        painter->setPen(ucolor[COLOR_BACK_GROUND]);
        painter->setBrush(ucolor[COLOR_BACK_GROUND]);
        painter->drawRect(QRectF( rect.x(), rect.y(),  w, h));
    }
    if (ucolor[COLOR_BORDER].isValid()) {
        painter->setPen(ucolor[COLOR_BORDER]);
        painter->setBrush(ucolor[COLOR_BORDER]);
        painter->drawRect(rect.x(), rect.y(),  w-1, h-1);
    }

    w -= 2;
    h -= 2;

    for (dlist = segments_get_list((cu->upload())->finished_segments); dlist; dlist = dlist->next) {
      segment_t* segment = (segment_t*) dlist->data;
      if (segment->size <= 0) continue;
      
      if (ucolor[COLOR_FINISHED].isValid()) {
        x = segment->start*w/size;
        x2 = (segment->stop-1)*w/size +1;       
        painter->setPen(ucolor[COLOR_BORDER]);
        painter->setBrush(ucolor[COLOR_BORDER]);
        painter->drawRect(QRectF( rect.x() + x+1, rect.y() + 1,  x2-x, h));

      }
    }
    for (dlist = segments_get_list(upload->segments); dlist; dlist = dlist->next) {
      segment_t* segment = (segment_t*) dlist->data;
      file_size_t ssize;

      if (segment == (cu->upload())->cur_segment) {
        ssize = segment->size - buf;
      } else {
        ssize = segment->size;
      }
      x = segment->start*w/size;
      if (ucolor[COLOR_ACTIVE].isValid()) {
        if (segment->start + ssize < segment->stop) {
          x2 = (segment->stop-1)*w/size +1;
          painter->setPen(ucolor[COLOR_ACTIVE]);
          painter->setBrush(ucolor[COLOR_ACTIVE]);
          painter->drawRect(QRectF( rect.x() + x+1, rect.y() + 1,  x2-x, h));

        }
      }
      if (ucolor[COLOR_FINISHED].isValid()) {
        if (ssize > 0) {
          x2 = (segment->start+ssize-1)*w/size +1;
          painter->setPen(ucolor[COLOR_FINISHED]);
          painter->setBrush(ucolor[COLOR_FINISHED]);
          painter->drawRect(QRectF( rect.x() + x+1, rect.y() + 1,  x2-x, h));

        }
      }
    }
    if (ucolor[COLOR_PROGRESS].isValid()) {
      x = 0;
      seg = (cu->upload())->cur_segment;
      if (seg->size-buf > 0) {
        x2 = (seg->size-buf-1)*w / (seg->stop - seg->start) +1;
        painter->setPen(ucolor[COLOR_PROGRESS]);
        painter->setBrush(ucolor[COLOR_PROGRESS]);
        painter->drawRect(QRectF( rect.x() + x+1, rect.y() + 1,  x2-x, h/4));

      }
    }
  }
}
