/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.8 $
 * $Date: 2006/07/22 17:11:47 $
 * $Author: sid4u $
 */

#ifndef _SEARCH_H_
#define _SEARCH_H_

/** \defgroup SearchModule The search framework
 * @ingroup FileSharingFramework
 */
/*@{*/

typedef struct _search_pattern_t search_pattern_t;
typedef struct _search_result_t search_result_t;
typedef struct _search_t search_t;

#include "attribute.h"
#include "files.h"
#include "download.h"
#include "storage.h"
#include "module.h"

G_BEGIN_DECLS

/** Function to handle searches.
 * @param search the search to do
 * @see search_register_action()
 */
typedef void (*SearchFunc)(search_t* search);

#define SEARCHP(arg) ((search_pattern_t*)(arg))
/** Search pattern define the settings a a search. */
struct _search_pattern_t {
  attr_container_t _ac;   ///< base class
  GSList* include;        ///< search terms
  GSList* exclude;        ///< exclude terms
  int media;
};

#define SEARCH(arg) ((search_t*)(arg))
/** The actual search */
struct _search_t {
  search_pattern_t _pattern;  ///< base class
  storage_t* results;         ///< the results
  download_t* download;       ///< the download, if it's no normal search
  int network;                // unused atm
  int* sources;               ///< the number of sources for the networks.
};

#define SRESULT(arg) ((search_result_t*)(arg))
/** A search results */
struct _search_result_t {
  file_t _file;             ///< base class
  int weight;               ///< the number of results this entry represents.
  gboolean available;       ///< TRUE if sources allow to download
  GList* children;          ///< child results
  search_result_t* master;  ///< parent result
  user_t* user;             ///< the user providing this file
};


/** @name Registration */
/*@{*/
module_t* search_module_instance();
void search_register_action(SearchFunc action);
void search_register_result_compare(GCompareFunc compare, gboolean slow);
/*@}*/

/** @name Utilities */
/*@{*/
unsigned search_string_tokenize(char* term, GSList** include, GSList** exclude);
unsigned search_file_tokenize(file_t* file, GSList** tokens);
/*@}*/

/** @name Search handling */
/*@{*/
gboolean search_check_file(search_t* search, file_t* file);
void search_add_result(search_t* search, file_t* file, user_t* user, int weight, gboolean available);
void search_set_continuous(search_t* search, int minutes, ObjectFunc func, void* data);
void search_foreach_continuous(ObjectFunc func, void* data);
gboolean search_pattern_check_file(search_pattern_t* pattern, file_t* file, gboolean check_inex);

search_pattern_t* search_pattern_new(void);
search_t* search_new_from_pattern(search_pattern_t* pattern, int network);
/*@}*/

/*@}*/

search_t* search_new_from_download(download_t* download);
GList* search_get_searches(void);
void search_pattern_remove(search_pattern_t* pattern);
void search_pattern_add(search_pattern_t* pattern);
search_pattern_t* search_pattern_find(search_pattern_t* pattern);

void search_refresh(search_t* search);
void search_foreach_result(search_t* search, StorageTraverseFunc func, gpointer data);
int search_get_number_results(search_t* search);
void search_patterns_foreach(ObjectFunc func, void* data);

G_END_DECLS
#endif
