/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2007/05/23 20:21:53 $
 * $Author: sid4u $
 */

#include <glib.h>

#include "storage.h"
#ifdef WIN32
#include "lcorealias.h"
#endif

struct _storage_t {
  GTree* tree;
  GList* compare;
  GDestroyNotify destroy;
};

static StorageTraverseFunc TraverseFunc = NULL;

static int storage_comp(void* o1, void* o2, storage_t* storage) {
  GList* dlist;

  for (dlist = storage->compare; dlist; dlist = dlist->next) {
    GCompareFunc func = dlist->data;
    int res = func(o1, o2);
    if (res) return res;
  }
  return 0;
}

storage_t* storage_new(GDestroyNotify destroy) {
  storage_t* storage = g_malloc0(sizeof(*storage));
  storage->destroy = destroy;
  storage->tree = g_tree_new_full
    ((GCompareDataFunc)storage_comp, storage, NULL, destroy);
  storage->compare = NULL;
  return storage;
}

void storage_add_compare_func(storage_t* storage,
                              GCompareFunc compare, gboolean slow) {
  g_assert(compare);
  if (slow) 
    storage->compare = g_list_append(storage->compare, compare);
  else
    storage->compare = g_list_prepend(storage->compare, compare);
}

void* storage_find(storage_t* storage, void* object) {
  return g_tree_lookup(storage->tree, object);
}

void storage_remove(storage_t* storage, void* object) {
  g_tree_remove(storage->tree, object);
}

void storage_steal(storage_t* storage, void* object) {
  g_tree_steal(storage->tree, object);
}

void storage_insert(storage_t* storage, void* object) {
  g_tree_insert(storage->tree, object, object);
}

int storage_count(storage_t* storage) {
  return g_tree_nnodes(storage->tree);
}
static gboolean on_storage_foreach(void* object, void* dummy, gpointer data) {
  (void)dummy;
  TraverseFunc(object, data);
  return FALSE;
}

void storage_foreach(storage_t* storage, StorageTraverseFunc func,
                     gpointer data) {
  TraverseFunc = func;
  g_tree_foreach(storage->tree, (GTraverseFunc)on_storage_foreach, data);
}

void storage_destroy(storage_t* storage) {
  g_tree_destroy(storage->tree);
  g_list_free(storage->compare);
  g_free(storage);
}

void storage_clear(storage_t* storage) {
  g_tree_destroy(storage->tree);
  storage->tree = g_tree_new_full
    ((GCompareDataFunc)storage_comp, storage, NULL, storage->destroy);
}

#ifdef WIN32
# define _STORAGE_C_
# include "lcorealiasdef.c"
#endif
