/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2006/07/22 17:11:47 $
 * $Author: sid4u $
 */

#ifndef _STRING_LIST_H_
#define _STRING_LIST_H_

#include "module.h"

G_BEGIN_DECLS

/** @defgroup StringListModule Save/loadable string lists 
 * String lists do not allow duplicate items. They can be set up case
 * sensitive and with the restriction of no whitespaces allowed.
 * @ingroup LopsterLibs
 */
/*@{*/

/** Function that operates on string list items.
 * @param item the item
 * @param data user data
 */
typedef void (*StringListFunc)(const char* item, void* data);


/** @name Setup string lists */
/*@{*/
module_t* string_list_module_instance();
void string_list_register(const char* name, const char* label, gboolean allow_ws);
void string_list_set_case(const char* name, gboolean csens);
void string_list_set_save(const char* name, gboolean save);
gboolean string_list_get_allow_ws(const char* name);
const char* string_list_get_label(const char* name);
void string_list_monitor(const char* name, StringListFunc added, StringListFunc removed, void* data);
/*@}*/


/** @name Access functions */
/*@{*/
gboolean string_list_check(const char* name, const char* item);
gboolean string_list_add(const char* name, const char* item);
gboolean string_list_remove(const char* name, const char* item);
void string_list_copy_items(const char* dest, const char* src);
const GList* string_list_get_items(const char* name);
void string_list_foreach(const char* name, StringListFunc func, void* data);
/*@}*/


/** @name Helpber functions */
/*@{*/
GList* string_list_contains(GList* sl, const char* item);
GSList* string_slist_contains(GSList* sl, const char* item);
/*@}*/


/*@}*/

void string_lists_foreach(StringListFunc func, void* data);
void string_lists_save_unregistered(gboolean save);

G_END_DECLS
#endif
