/* Copyright (C) 2005-2006 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.5 $
 * $Date: 2006/07/23 00:09:27 $
 * $Author: sgop $
 */

#ifndef _THREAD_H_
#define _THREAD_H_

#include "object.h"
#include "module.h"

G_BEGIN_DECLS

/** \defgroup ThreadModule Using threads
 * @ingroup BasicFramework
 */
/*@{*/

/** @name Status of a thread */
/*@{*/
#define THREAD_NOT_STARTED   0  ///< not started
#define THREAD_RUNNING       1  ///< calculation is running
#define THREAD_INTERRUPTED   2  ///< calculation was interrupted.
#define THREAD_FAILED        3  ///< calculation failed
#define THREAD_COMPLETED     4  ///< calculation completed successfully
/*@}*/

typedef struct _thread_t thread_t;
typedef struct _thread_priv_t thread_priv_t;

/** The thread main function
 * @param thread the thread object.
 * @return the exit status (FIXME: verify)
 */
typedef unsigned (*ThreadMainFunc)(thread_t* thread);

/** The thread sync function
 * @param thread the thread object.
 * @param data the user data
 */
typedef void (*ThreadSyncFunc)(thread_t* thread, void* data);

#define THREAD(arg) ((thread_t*)(arg))

/** The thread class.*/
struct _thread_t {
  object_t _object;     ///< the base class
  thread_priv_t* priv;  ///< private data
};

/** @name Thread control */
/*@{*/
module_t* thread_module_instance(void);
thread_t* thread_new(unsigned update);
void thread_init(thread_t* thread, const char* name, unsigned update);
void thread_interrupt(thread_t* thread);
void thread_start(thread_t* thread, ThreadMainFunc main);
void thread_lock(thread_t* thread);
void thread_unlock(thread_t* thread);
void thread_destroy(thread_t* thread);
void thread_sync(thread_t* thread, ThreadSyncFunc sync, void* data);
void thread_set_force_interrupt(thread_t* thread, gboolean value);
gboolean thread_interrupted(thread_t* thread);

void thread_set_description(thread_t* thread, const char* des);
void thread_set_progress(thread_t* thread, unsigned val);
const char* thread_get_description(thread_t* thread);
const char* thread_get_name(thread_t* thread);
unsigned thread_get_progress(thread_t* thread);
unsigned thread_get_status(thread_t* thread);
/*@}*/



/*@}*/

G_END_DECLS
#endif
