/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _UDP_SOCKET_H_
#define _UDP_SOCKET_H_

#include "module.h"
#include "b_socket.h"

G_BEGIN_DECLS

typedef struct _udp_socket_t udp_socket_t;

/** @defgroup USocketModule UDP socket class
 * @ingroup SocketClasses
 */
/*@{*/

#define USOCK(s) ((udp_socket_t*)(s))
/** The udp socket class. */
struct _udp_socket_t {
  b_socket_t _socket;   ///< the base class
};

module_t* usocket_module_instance();


gboolean udp_socket_end(udp_socket_t* sock, const char* message);
void udp_socket_destroy(udp_socket_t* sock);
udp_socket_t* udp_socket_new(const char* name, int timeout);
void udp_socket_init(udp_socket_t* socket, const char* name, int timeout);
void udp_socket_send_str_to(udp_socket_t* socket, const char* message,
                            ip_t ip, port_t port);
void udp_socket_send_bin_to(udp_socket_t* socket, const char* message, int len,
                            ip_t ip, port_t port);

/*@}*/

/** @defgroup UDPListenModule UDP listen socket class
 * @ingroup SocketClasses
 */
/*@{*/

typedef struct _udp_listen_t udp_listen_t;

/** The udp listen socket class. */
struct _udp_listen_t {
  udp_socket_t _socket;   ///< the base class
  port_t port;
  gboolean activated;
  char* pref;
};

udp_listen_t* udp_listen_new(const char* pref, const char* name);
void udp_listen_init(udp_listen_t* list, const char* pref, const char* name);
void udp_listen_destroy(udp_listen_t* l);
gboolean udp_listen_end(udp_listen_t* list, const char* message);
void udp_listen_start_listening(udp_listen_t* list);
void udp_listen_stop_listening(udp_listen_t* l);
port_t udp_listen_get_port(udp_listen_t* list);

G_END_DECLS
#endif
