/* Copyright (C) 2005-2006 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.15 $
 * $Date: 2007/11/21 20:18:24 $
 * $Author: sgop $
 */

#ifndef _UI_H_
#define _UI_H_

#include "download.h"
#include "library.h"
#include "chat.h"
#include "buddy.h"
#include "browse.h"
#include "search.h"
#include "upload.h"
#include "win32_defs.h"

G_BEGIN_DECLS

/** \defgroup UserInterface Gtk2 user interface */
/*@{*/

typedef struct
{
    void (*enter_main_loop)(void);
    void (*leave_main_loop)(void);
    
    void (*download_show_details)(download_t*);
    
    void (*file_tree_update_label)(file_tree_t*, const char*);
    void (*file_tree_set_progress)(file_tree_t*, double);
    void (*file_tree_set_progress_name)(file_tree_t*, const char*);
    void (*file_tree_set_abort)(file_tree_t*, GFunc, void*);
    
    void (*lib_working)(gboolean);
    
    void (*chat_switch_page)(chat_page_t*);
    gboolean (*chat_page_printable)(chat_page_t*);
    void (*chat_create_simple_page)(chat_page_t*);
    void (*chat_print)(chat_page_t*, int index, va_list);
    
    void (*threads_show)(const char* message);

    guint (*idle_timeout_add) (GSourceFunc, gpointer);
    guint (*timeout_add) (guint, GSourceFunc, gpointer);
    gboolean (*timeout_remove) (guint);
    gboolean (*idle_timeout_remove) (guint);

} ui_interface_t;

typedef struct {
    char* name;
    char* iname;  // internal name (parsed from filename)
    char* filename;
} ui_entry_t;


void UI_enter_main_loop(void);
void UI_leave_main_loop(void);
void UI_download_show_details(download_t*);
void UI_file_tree_update_label(file_tree_t*, const char*);
void UI_file_tree_set_progress(file_tree_t*, double);
void UI_file_tree_set_progress_name(file_tree_t*, const char*);
void UI_file_tree_set_abort(file_tree_t*, GFunc, void*);
void UI_lib_working(gboolean);
void UI_chat_switch_page(chat_page_t*);
gboolean UI_chat_page_printable(chat_page_t*);
void UI_chat_create_simple_page(chat_page_t*);
void UI_chat_print(chat_page_t*, int index, va_list);
void UI_threads_show(const char* message);

guint UI_idle_add(GSourceFunc function, gpointer data);
void UI_idle_remove(guint id);
guint UI_timeout_add(guint interval, GSourceFunc function, gpointer data);
void UI_timeout_remove(guint id);

module_t* ui_module_instance();

ui_entry_t* ui_get_list(GList** list);
void ui_set_name(const char* ui, gboolean save);
const char* ui_find_library_file(const char* network);



/*@}*/

G_END_DECLS
#endif
