/* Copyright (C) 2007 The Lopster Team
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2007/11/23 08:36:02 $
 * $Author: sid4u $
 */

#ifdef WIN32
# include <winsock2.h>
#else
# include <stdlib.h>
# include <arpa/inet.h>
# include <string.h>
#endif

#include "url.h"

#ifdef WIN32
#include "lcorealias.h"
#endif

void url_destroy(url_t* url)
{
    g_free(url->proto);
    g_free(url->host);
    g_free(url->doc);
    object_destroy(OBJ(url));
}

static void url_init(url_t* url, const char* proto, const char* host,
                     port_t port, const char* doc)
{
    object_init(OBJ(url), host, TRUE);
    object_add_type(OBJ(url), "Url");
    OBJ(url)->destroy = (ObjectFunc)url_destroy;
    url->proto = g_strdup(proto);
    url->host = g_strdup(host);
    url->ip = inet_addr(host);
    if (url->ip == INADDR_NONE) url->ip = 0;
    url->port = port;
    url->doc = g_strdup(doc);
}

url_t* url_new(const char* proto, const char* host, port_t port, const char* doc)
{
    url_t* url = g_malloc0(sizeof(*url));
    url_init(url, proto, host, port, doc);
    return url;
}

static port_t url_get_default_port(const char* proto)
{
    if (!strcmp(proto, "http")) return 80;
    else return 80;   //FIXME
}

url_t* url_parse(const char* url)
{
    char* temp = l_strdup(url);
    char* proto;
    char* address;
    char* doc;
    char* pos;
    port_t port;

    if (!temp) return NULL;
    pos = strchr(temp, ':');
    if (!pos) return NULL;
    proto = temp;
    *pos++ = 0;
    if (strncmp(pos, "//", 2)) return NULL;
    address = pos+2;
    pos = strchr(address, '/');
    if (!pos) doc = NULL;
    else { *pos = 0; doc = pos+1; }

    pos = strchr(address, ':');
    if (pos) { *pos++ = 0; port = atoi(pos); }
    else port = url_get_default_port(proto);

    return url_new(proto, address, port, doc);
}

gboolean url_equal(url_t* url1, url_t* url2)
{
    g_assert(url1);
    g_assert(url2);
    //FIXME: check for host != NULL
    if (strcmp(url1->host, url2->host)) return FALSE;
    if (url1->port != url2->port) return FALSE;
    if (url1->doc != NULL && url2->doc != NULL)
    {
        if (strcmp(url1->doc, url2->doc)) return FALSE;
    }
    else if (url1->doc != url2->doc) return FALSE;

    return TRUE;
}

char* url_format(url_t* url)
{
    gboolean def_port = (url->port == url_get_default_port(url->proto));

    if (def_port)
        return l_printf("%s://%s/%s", url->proto, url->host, 
                        url->doc?url->doc:"");
    else
        return l_printf("%s://%s:%u/%s", url->proto, url->host, 
                        url->port, url->doc?url->doc:"");
}

#ifdef WIN32
#define _URL_C_
#include "lcorealiasdef.c"
#endif

