/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.23 $
 * $Date: 2007/11/23 08:25:54 $
 * $Author: sid4u $
 */

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifndef WIN32
# include <unistd.h>
#endif
#ifndef WIN32
#include <netinet/in.h>
#include <arpa/inet.h>
#endif

#include <glib/gstdio.h>

#ifdef WIN32
# include "win32_defs.h"
#endif
#include "utils.h"
#include "l_base.h"

#ifdef WIN32
# include "lcorealias.h"
#endif

#define NBUF    100
static char SBuf[NBUF][2048];
static int BufPos = 0;
static char* CBuf[NBUF] = {
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
};
static int CBufPos = 0;
static FILE* WarningFile = NULL;
static FILE* LogFile = NULL;

char* ntoa(ip_t ip) {
  struct in_addr a;

  memset(&a, 0, sizeof(a));
  a.s_addr = ip;
  return (inet_ntoa(a));
}

#ifndef WIN32
extern time_t CurrentTime;
#endif

const char* print_timestamp(int* mode)
{
    struct tm *ltime;
    char* buf = get_text_buffer();
    
    buf[0] = 0;
    switch (*mode) {
    case TIME_HOUR_MIN:
    {
#ifdef WIN32
        time_t wtime = CurrentTime;
        ltime = localtime(&wtime);
#else
        ltime = localtime(&CurrentTime);
#endif
        sprintf(buf, "%02d:%02d", ltime->tm_hour, ltime->tm_min);
        break;
    }
    case TIME_HOUR_MIN_SEC:
    {
#ifdef WIN32
        time_t wtime = CurrentTime;
        ltime = localtime(&wtime);
#else
        ltime = localtime(&CurrentTime);
#endif
        sprintf(buf, "%02d:%02d:%02d",
                ltime->tm_hour, ltime->tm_min, ltime->tm_sec);
        break;
    }
    default:
        return NULL;
    }
    return buf;
}

static void print_raw_real(FILE* file, unsigned char* message, int length)
{
    int i1;
    
    for (i1 = 0; i1 < length; i1++)
    {
        if (message[i1] >= 32 && message[i1] < 127)
            fputc((int)(message[i1]), file);
        else if (message[i1] == '\r')
            fprintf(file, "[CR]");
        else if (message[i1] == '\n')
            fprintf(file, "[LF]");
        else
            fprintf(file, "[%02x]", message[i1]);
    }
}

void print_raw(const char* header, unsigned char* message, int length)
{
    if (header) fprintf(stderr, "%s [%d]:\n", header, length);
    fputc('[', stderr);
    print_raw_real(stderr, message, length);
    fputc(']', stderr);
    fputc('\n', stderr);
}

static FILE* _open_log_file(const char* name, const char* mode)
{
    FILE* lfile;

    if (directory_exists(lopster_get_home()))
        lfile = fopen(l_printf(SDIR("%s")"%s", lopster_get_home(), name), mode);
    else
        lfile = fopen(l_printf("Lopster2_%s", name), mode);

    if (lfile)
    {
        time_t t = CurrentTime;
        struct tm* ti = localtime(&t); //time(&t)
        fprintf(lfile, "---------------------------------------------------\n");
        fprintf(lfile, " Lopster Session [%02d:%02d:%02d]\n", ti->tm_hour, ti->tm_min, ti->tm_sec);
        fprintf(lfile, "---------------------------------------------------\n");
    }
    return lfile;
}

void std_log(const char* domain, GLogLevelFlags level, const char* text)
{
    const char* code = "";
/*     if (level > G_LOG_LEVEL_INFO) return; */

    if (!domain) domain = "";

    if (!LogFile) LogFile = _open_log_file("Full.log", "w");
    if (!WarningFile) WarningFile = _open_log_file("Warning.log", "a");
    
    if (LogFile)
    {
        if (level == G_LOG_LEVEL_WARNING)
            fprintf(LogFile, "*** %s *** : %s\n", domain, text);
        else
            fprintf(LogFile, "%-11s| %s\n", domain, text);
        fflush(LogFile);  // remove this, because slow?
    }

    // log any warning, criticals and errors
    if (level <= G_LOG_LEVEL_WARNING && WarningFile)
    {
        fprintf(WarningFile, "%-11s| %s\n", domain, text);
        fflush(WarningFile);
    }

#ifdef WIN32
    code = "";
#else
    switch (level)
    {
    case G_LOG_LEVEL_ERROR:
    case G_LOG_LEVEL_CRITICAL:
        code = "\033[1m\033[31m";
        break;
    case G_LOG_LEVEL_WARNING:
        code = "\033[1m\033[33m";
        break;
    case G_LOG_LEVEL_MESSAGE:
    case G_LOG_LEVEL_INFO:
        code = "\033[0m\033[39m";
        break;
    case G_LOG_LEVEL_DEBUG:
        code = "\033[37m";
        break;
    default:
        code = "";
        break;
    }
#endif
    fprintf(stdout, "%s%-11s| ", code, domain);
    print_raw_real(stdout, (unsigned char*)text, strlen(text));
    fprintf(stdout, "\033[0m\n");
}


gboolean pattern_match(const gchar *match_pattern, const gchar *match_string) {
  const gchar *pattern, *string;
  gchar ch;

  pattern = match_pattern;
  string = match_string;

  ch = *pattern++;
  while (ch) {
    switch (ch) {

    case '?':
      if (!*string) return FALSE;
      string = g_utf8_next_char(string);
      break;

    case '*':
      do {
        ch = *pattern++;
        if (ch == '?') {
          if (!*string) return FALSE;
          string = g_utf8_next_char(string);
        }
      } while (ch == '*' || ch == '?');
      if (!ch) return TRUE;
      do {
        while (ch != *string) {
          if (!*string) return FALSE;
          string = g_utf8_next_char(string);
        }
        string++;
        if (pattern_match(pattern, string))
          return TRUE;
      } while (*string);
      break;

    default:
      if (ch == *string) string++;
      else return FALSE;
      break;
    }

    ch = *pattern;
    pattern++;
  }

  return *string == 0;
}


void line_insert_lf(char *line, int pos)
{
    char *last_ws = line;
    char *temp = line;
    int length;
    
    if (!line || pos <= 0) return;
    length = strlen(line);
    if (length == 0) return;
    if (line[length - 1] == '\n')
    {
        line[length - 1] = 0;
        length--;
    }
    if (length <= pos) return;

    while (line)
    {
        while (temp - line <= pos)
        {
            last_ws = temp;
            temp = strpbrk(temp + 1, " \n\t");
            if (!temp) break;
            if (*temp == '\n') {
                last_ws = temp;
                break;
            }
        }
        if ((last_ws - line > 0) && (temp))
        {
            *last_ws = '\n';
            line = temp = last_ws + 1;
        }
        else
        {
            break;
        }
    }
}


int text_buffer_size() {
  return 2048;
}
char* get_text_buffer() {
  return SBuf[(++BufPos>=NBUF)?(BufPos=0):BufPos];
}

char* auto_text(char* text) {
  char** result = &CBuf[(++CBufPos>=NBUF)?(CBufPos=0):CBufPos];
  if (*result) g_free(*result);
  *result = text;
  return text;
}

char* l_printf(const char* fmt, ...) {
  va_list ap;
  char* result;
  g_assert(fmt);
  va_start(ap, fmt);
  result = g_strdup_vprintf(fmt, ap);
  va_end(ap);
  return auto_text(result);
}
char* l_strdup(const char* source) {
  g_assert(source);
  return auto_text(g_strdup(source));
}


/** Convert text to utf8 trying a number of charsets.
 * @param text the text to convert
 * @param len the length of the text, or -1 if text is 0-terminated.
 * @param[out] written the number of bytes that were written, may be NULL
 * @param[out] error the error, may be NULL
 * @param num the number of charsets to try.
 * @param ... num charsets
 * @return the (partially) converted message
 */
char* to_utf8(const char* text, gsize len, gsize* written, GError** error,
              unsigned num, ...) {
  GError* my_error = NULL;
  char* res = NULL;
  va_list ap;
  unsigned i1;

  if (error) *error = NULL;
  if (!text) return NULL;
  
  va_start(ap, num);
  for (i1 = 0; i1 < num; i1++) {
    const char* charset = va_arg(ap, char*);

    if (!charset || *charset == 0) continue;
    
    if (res) {
      g_free(res);
      res = NULL;
    }
    if (strcmp(charset, "locale"))
      res = g_convert(text, len, "UTF-8", charset, NULL, written, &my_error);
    else
      res = g_locale_to_utf8(text, len, NULL, written, &my_error);

    if (!my_error) {
      if (error && *error) {
        g_error_free(*error);
        *error = NULL;
      }
      break;
    }
    
    // save first error.
    if (error && !*error) *error = my_error;
    else g_error_free(my_error);
    my_error = NULL;
  }
  va_end(ap);

  return res;
}

/** Convert text from utf8 to a specific charset. If charset is NULL
 * or 'locale' then the message is converted from the current locale.
 * @param text the text to convert
 * @param len the length of the text, or -1 if text is 0-terminated.
 * @param[out] written the number of bytes that were written, may be NULL
 * @param[out] error the error, may be NULL
 * @param num the number of charsets to try.
 * @param ... num charsets
 * @return the (partially) converted message
 */
char* from_utf8(const char* text, gsize len, gsize* written, GError** error,
                unsigned num, ...) {
  GError* my_error = NULL;
  char* res = NULL;
  va_list ap;
  unsigned i1;

  if (error) *error = NULL;
  if (!text) return NULL;
  
  va_start(ap, num);
  for (i1 = 0; i1 < num; i1++) {
    const char* charset = va_arg(ap, char*);

    if (!charset || *charset == 0) continue;
    
    if (res) {
      g_free(res);
      res = NULL;
    }
    if (strcmp(charset, "locale")) {
      res = g_convert(text, len, charset, "UTF-8", NULL, written, &my_error);
/*       if (my_error) { */
/*         g_message("%d/%d could not convert %s from %s to %s", */
/*                   i1, num, text, "utf8", charset); */
/*       } */
    } else {
      res = g_locale_from_utf8(text, len, NULL, written, &my_error);
/*       if (my_error) { */
/*         g_message("%d/%d could not convert %s from %s to %s", */
/*                   i1, num, text, "locale", charset); */
/*       } */
    }

    if (!my_error) {
      if (error && *error) {
        g_error_free(*error);
        *error = NULL;
      }
      break;
    }
    
    // save first error.
    if (error && !*error) *error = my_error;
    else g_error_free(my_error);
    my_error = NULL;
  }
  va_end(ap);

  return res;
}

/** Convert to current locale.
 * @param text text to convert.
 * @return converted text.
 */
char* locale_from_utf8(const char* text) {
  return from_utf8(text, -1, NULL, NULL, 1, "locale");
}

/** Convert from current locale.
 * @param text text to convert.
 * @return converted text.
 */
char* locale_to_utf8(const char* text) {
  return to_utf8(text, -1, NULL, NULL, 1, "locale");
}

// dont pass str == NULL !

file_size_t extract_bytes(const char* string) {
  const char* pos;
  file_size_t result = 0;

  if (!string) return 0;
  if (!isdigit(*string)) return 0;

  pos = string;
  while (isdigit(*pos)) {
    result *= 10;
    result += *pos -'0';
    pos++;
  }

  if (*pos == ' ') pos++;
  switch (*pos) {
  case 't': 
  case 'T': 
    result *= 1024*1024;
    result *= 1024*1024;
    break;
  case 'g': 
  case 'G': 
    result *= 1024*1024*1024;
    break;
  case 'm': 
  case 'M': 
    result *= 1024*1024;
    break;
  case 'k': 
  case 'K': 
    result *= 1024;
    break;
  case 'b': 
  case 'B': 
  case ' ': 
  case 0: 
  default:
    break;
  }
  
  return result;
}

int extract_duration(const char* string) {
  const char* pos;
  unsigned int result = 0;

  if (!string) return 0;
  if (!isdigit(*string)) return 0;

  pos = string;
  while (isdigit(*pos)) {
    result *= 10;
    result += *pos -'0';
    pos++;
  }

  if (*pos == ' ') pos++;

  switch (*pos) {
  case 'h': 
  case 'H': 
    result *= 3600;
    break;
  case 'm': 
  case 'M': 
    result *= 60;
    break;
  case 's': 
  case 'S': 
  case ' ': 
  case 0: 
  default:
    break;
  }
  
  return result;
}

char* arg(char *data, int flags) {
  static char *string;
  char *pos;
  char *pos2;

  if (data) string = data;
  if (!string) return NULL;

  if (flags & 1) {
    pos = string;
    string = NULL;
    if (*pos == 0) return NULL;
    else return pos;
  }

  while (isspace(*string)) string++;

  if (*string == '"') {
    pos = pos2 = ++string;
    if (flags & 2) {
      for (;;) {
		char c = *string++;
		if (c == '\\')
			c = *string++;
		else if (c == '"')
			break;
		if (!c) {
			string = NULL;
			return NULL;
		}
		*pos2++ = c;
      }
      *pos2 = 0;
      if (isspace(*string)) string++;
      else string = NULL;
      return pos;
    } 
	
	else {
      int cnt = 1;
      char* last_pos = NULL;
      while (1) {
		pos2 = strchr(pos2, '"');
		if (!pos2) break;
		if (pos2[1] == 0 || isspace(pos2[1])) {
			cnt--;
			last_pos = pos2;
			if (cnt == 0) break;
		} 
		else if (pos2[1] == '"') {
			if (cnt > 1) cnt--;
			else cnt++;
		} 
		else {
			cnt++;
			pos2 = pos2+1;
		}
		pos2 = pos2+1;
      }
      if (!pos2) pos2 = last_pos;
      
      // end quote found?
      if (pos2) {
		*pos2 = 0;
		string = pos2 + 1;
		if (isspace(*string)) string++;
		return pos;
      } 
	  else {
		printf("** unterminated quoted string **\n");
		string = NULL;
		return NULL;
      }
    }
  } 
  
  else {
    pos = string;
    pos2 = strpbrk(string, " \n\r\t\v\f");
    if (pos2) {
      *pos2 = 0;
      string = pos2 + 1;
    } else {
      string = NULL;
    }
    if (*pos == 0) return NULL;
    else return pos;
  }

  return NULL;
}

char* arg2(char *data, int flags) {
  static char *string;
  char *pos;
  char *pos2;

  if (data) string = data;
  if (!string) return NULL;

  if (flags & 1) {
    pos = string;
    string = NULL;
    if (*pos == 0) return NULL;
    else return pos;
  }

  while (isspace(*string)) string++;

  if (*string == '"') {
    pos = pos2 = ++string;
    if (flags & 2) {
      for (;;) {
		char c = *string++;
		/*if (c == '\\')
			c = *string++; */
		if (c == '"')
			break;
		if (!c) {
			string = NULL;
			return NULL;
		}
		*pos2++ = c;
      }
      *pos2 = 0;
      if (isspace(*string)) string++;
      else string = NULL;
      return pos;
    } 
	
	else {
      int cnt = 1;
      char* last_pos = NULL;
      while (1) {
		pos2 = strchr(pos2, '"');
		if (!pos2) break;
		if (pos2[1] == 0 || isspace(pos2[1])) {
			cnt--;
			last_pos = pos2;
			if (cnt == 0) break;
		} 
		else if (pos2[1] == '"') {
			if (cnt > 1) cnt--;
			else cnt++;
		} 
		else {
			cnt++;
			pos2 = pos2+1;
		}
		pos2 = pos2+1;
      }
      if (!pos2) pos2 = last_pos;
      
      // end quote found?
      if (pos2) {
		*pos2 = 0;
		string = pos2 + 1;
		if (isspace(*string)) string++;
		return pos;
      } 
	  else {
		printf("** unterminated quoted string **\n");
		string = NULL;
		return NULL;
      }
    }
  } 
  
  else {
    pos = string;
    pos2 = strpbrk(string, " \n\r\t\v\f");
    if (pos2) {
      *pos2 = 0;
      string = pos2 + 1;
    } else {
      string = NULL;
    }
    if (*pos == 0) return NULL;
    else return pos;
  }

  return NULL;
}

char* extract_filename( char* data )
{
  char* pos;
  pos = strrchr(data, DIR_SEP);
  if (pos) return pos+1;
  return data;
}

void make_list_from_string(GSList** glist, const char *text, const char* sep) {
  char* pos2;
  char* temp = g_strdup(text);
  
  g_slist_foreach(*glist, (GFunc)g_free, NULL);
  g_slist_free(*glist);
  *glist = NULL;

  if (!text) return;

  pos2 = strtok(temp, sep);
  while (pos2) {
    if (*pos2) *glist = g_slist_append(*glist, g_strdup(pos2));
    pos2 = strtok(NULL, sep);
  }
  g_free(temp);
}

char* make_string_from_list(GSList* glist, const char* sep) {
  GSList* dlist;
  char* result;
  unsigned len;
  unsigned add = strlen(sep);

  if (glist == NULL) return g_strdup("");

  len = 0;
  for (dlist = glist; dlist; dlist = dlist->next) {
    char* text = dlist->data;
    len += strlen(text)+add;
  }
  len -= add;
  result = g_malloc0(len+1);
  len = 0;
  for (dlist = glist; dlist; dlist = dlist->next) {
    char* text = dlist->data;
    unsigned l = strlen(text);
    if (l && dlist != glist) {
      memcpy(result+len, sep, add);
      len += add;
    }
    memcpy(result+len, text, l);
    len += l;
  }

  return result;
}

gboolean directory_exists(const char *dir) {
  struct stat st;

  if (stat(dir, &st) < 0) return FALSE;
  else return TRUE;
}

int create_dir(char *dir) {
  char *pos;
  char *slash;

  if (!dir) return 0;
  pos = strrchr(dir, DIR_SEP);
  if (!pos) return 0;
  *pos = 0;
  if (directory_exists(dir)) {
    *pos = DIR_SEP;
    return 1;
  }
  *pos = DIR_SEP;
  
  slash = dir;
  while ((pos = strchr(slash + 1, DIR_SEP)) != NULL) {
    *pos = 0;
// skip C:, D: etc... (Example C:\Document And Setting\etc..)
// This would try to create C:
// Fixme: Maybe we should use glib create_dir api
#ifdef WIN32
    if ((strlen(dir)==2) && (strchr(dir,':'))) {
	    *pos = DIR_SEP;
	    slash = pos;
            continue;
    }
#endif
    if (!directory_exists(dir)) {
#if GLIB_CHECK_VERSION(2,6,0)
      if (g_mkdir(dir, 7 * 64 + 5 * 8 + 5))
#else
      if (mkdir(dir, 7 * 64 + 5 * 8 + 5))
#endif
        {
          g_log("UTILS", G_LOG_LEVEL_WARNING,
                "Could not create folder [%s]", dir);
          *pos = DIR_SEP;
          return 0;
        }
    }
    *pos = DIR_SEP;
    slash = pos;
  }
  return 1;
}

char* get_suffix(const char* filename) {
  char* pos;
  pos = strrchr(filename, '.');
  if (pos) return pos+1;
  else return NULL;
}

////////////////
// PACK functions
int pack_byte(char* buffer, char value) {
  *buffer = value;
  return 1;
}
int pack_u16(char* buffer, guint16 value) {
  guint16 val = GINT16_TO_LE(value);
  memcpy(buffer, &val, 2);
  return 2;
}
int pack_u16h(char* buffer, guint16 value) {
  guint16 val = htons(value);
  memcpy(buffer, &val, 2);
  return 2;
}
int pack_u32(char* buffer, guint32 value) {
  guint32 val = GINT32_TO_LE(value);
  memcpy(buffer, &val, 4);
  return 4;
}
int pack_u32h(char* buffer, guint32 value) {
  guint32 val = htonl(value);
  memcpy(buffer, &val, 4);
  return 4;
}
int pack_u32d(char* buffer, guint64 value) {
  int res = 0;

  //FIXME:
  guint32 v = (value&0xffffffff);
  res += pack_u32(buffer+res, v);
  res += pack_u32(buffer+res, 0);
  return res;
}
int pack_string_u8(char* buffer, const char* str)
{
    guint8 len = strlen(str);
    memcpy(buffer+1, str, len);
    *buffer = len;
    return len+1;
}
int pack_string_u16(char* buffer, const char* str) {
  guint16 len = strlen(str);
  memcpy(buffer+2, str, len);
  len = GINT16_TO_LE(len);
  memcpy(buffer, &len, 2);
  return GINT16_FROM_LE(len)+2;
}
int pack_string_u32(char* buffer, const char* str) {
  guint32 len = strlen(str);
  memcpy(buffer+4, str, len);
  len = GINT32_TO_LE(len);
  memcpy(buffer, &len, 4);
  return GINT32_FROM_LE(len)+4;
}
int pack_string_raw(char* buffer, const char* str, int len) {
  if (len < 0) len = strlen(str);
  memcpy(buffer, str, len);
  return len;
}

////////////////
// UNPACK functions
unsigned char unpack_byte(char** buffer, unsigned* len) {
  unsigned char result;
  if (*len < 1) return 0;
  result = (*buffer)[0];
  *buffer += 1;
  (*len) -= 1;
  return result;
}
guint16 unpack_u16(char** buffer, unsigned* len) {
  guint16 result;
  if (*len < 2) return 0;
  memcpy(&result, *buffer, 2);
  *buffer += 2;
  (*len) -= 2;
  return GINT16_FROM_LE(result);
}
guint16 unpack_u16h(char** buffer, unsigned* len) {
  guint16 result;
  if (*len < 2) return 0;
  memcpy(&result, *buffer, 2);
  *buffer += 2;
  (*len) -= 2;
  return htons(result);
}
guint32 unpack_u32(char** buffer, unsigned* len) {
  guint32 result;
  if (*len < 4) return 0;
  memcpy(&result, *buffer, 4);
  *buffer += 4;
  (*len) -= 4;
  return GINT32_FROM_LE(result);
}
guint32 unpack_u32h(char** buffer, unsigned* len) {
  guint32 result;
  if (*len < 4) return 0;
  memcpy(&result, *buffer, 4);
  *buffer += 4;
  (*len) -= 4;
  return htonl(result);
}
guint64 unpack_u32d(char** buffer, unsigned* len) {
  //FIXME:
  guint64 result;
  guint32 r1;
  guint32 r2;

  if (*len < 8) return 0;
  r1 = unpack_u32(buffer, len);
  r2 = unpack_u32(buffer, len);
  if (r2) printf("***********got %08x %08x\n", r1, r2);

  result = r1;
  return result;
}
char* unpack_string_u8(char** buffer, unsigned* size) {
  char* result;
  guint8 len;

  if (*size < 1) return NULL;
  memcpy(&len, *buffer, 1);

  if (*size < (unsigned)(len+1)) return NULL;
  memmove(*buffer, (*buffer)+1, len);

  (*buffer)[len] = 0;
  result = *buffer;

  *buffer += (len+1);
  (*size) -= (len+1);
  return result;
}
char* unpack_string_u16(char** buffer, unsigned* size) {
  char* result;
  guint16 len;

  if (*size < 2) return NULL;
  memcpy(&len, *buffer, 2);
  len = GINT16_FROM_LE(len);

  if (*size < (unsigned)len+2) return NULL;
  memmove(*buffer, (*buffer)+2, len);

  (*buffer)[len] = 0;
  result = *buffer;

  *buffer += (len+2);
  (*size) -= (len+2);
  return result;
}
char* unpack_string_u32(char** buffer, unsigned* size) {
  char* result;
  guint32 len;

  if (*size < 4) return NULL;
  memcpy(&len, *buffer, 4);
  len = GINT32_FROM_LE(len);

  if (*size < len+4) return NULL;
  memmove(*buffer, (*buffer)+4, len);

  (*buffer)[len] = 0;
  result = *buffer;

  *buffer += (len+4);
  (*size) -= (len+4);
  return result;
}

////////////////
// PRINT functions
const char* print_string(char* text) {
  return text;
}
const char* print_int(int* value) {
  char* result = get_text_buffer();
  sprintf(result, "%d", *value);
  return result;
}
const char* print_int_minmax(int* bytes) {
  char* str = get_text_buffer();
  sprintf(str, "%d-%d", bytes[0], bytes[1]);
  return str;
}
const char* print_filesize_minmax(file_size_t* bytes) {
  char* str = get_text_buffer();
  sprintf(str, FF"-"FF, bytes[0], bytes[1]);
  return str;
}
const char* print_bool(gboolean* value) {
  char* result = get_text_buffer();
  strcpy(result, (*value)?"yes":"no");
  return result;
}
const char* print_bitrate(int* value) {
  char* result = get_text_buffer();
  sprintf(result, "%d KiB/s", *value);
  return result;
}
const char* print_frequency(int* value) {
  char* result = get_text_buffer();
  sprintf(result, "%d Hz", *value);
  return result;
}
const char* print_vbr(int* value) {
  if (*value) {
    char* result = get_text_buffer();
    strcpy(result, "vbr");
    return result;
  } else return NULL;
}
const char* print_bytes_exact(file_size_t* bytes) {
  char* str = get_text_buffer();

  if ((*bytes) == 0)
    sprintf(str, "0B");
  else if ((*bytes)%(1024*1024*1024) == 0)
    sprintf(str, FF"G", (*bytes)/1024/1024/1024);
  else if ((*bytes)%(1024*1024) == 0)
    sprintf(str, FF"M", (*bytes)/1024/1024);
  else if ((*bytes)%1024 == 0)
    sprintf(str, FF"K", (*bytes)/1024);
  else
    sprintf(str, FF"B", (*bytes));
  return str;
}
const char* print_seconds_exact(int* seconds) {
  char* str = get_text_buffer();

  if ((*seconds) == 0)
    sprintf(str, "0s");
  else if ((*seconds)%(60*60) == 0)
    sprintf(str, "%dh", (*seconds)/60/60);
  else if ((*seconds)%60 == 0)
    sprintf(str, "%dm", (*seconds)/60);
  else
    sprintf(str, "%ds", (*seconds));
  return str;
}
const char* print_filesize(file_size_t* bytes) {
  char* str = get_text_buffer();
  double size;

  size = *bytes;
  if (size < 1024) {
    sprintf(str, "%.0f B", size);
    return str;
  }
  size /= 1024.;
  if (size < 1024) {
    sprintf(str, "%.2f KiB", size);
    return str;
  }
  if (size < 1024) {
    sprintf(str, "%.1f KiB", size);
    return str;
  }
  size /= 1024.;
  if (size < 128) {
    sprintf(str, "%.2f MiB", size);
    return str;
  }
  if (size < 1024) {
    sprintf(str, "%.1f MiB", size);
    return str;
  }
  size /= 1024.;
  if (size < 128) {
    sprintf(str, "%.2f GiB", size);
    return str;
  }
  if (size < 1024) {
    sprintf(str, "%.1f GiB", size);
    return str;
  }
  size /= 1024.;
  if (size < 128) {
    sprintf(str, "%.2f TiB", size);
    return str;
  }
  if (size < 1024) {
    sprintf(str, "%.1f TiB", size);
    return str;
  }
  size /= 1024.;
  if (size < 128) {
    sprintf(str, "%.2f PiB", size);
    return str;
  }
  if (size < 1024) {
    sprintf(str, "%.1f PiB", size);
    return str;
  }
  size /= 1024.;
  sprintf(str, "%.2f EiB", size);
  return str;
}

const char* print_duration(int* secs) {
  int min = (*secs) / 60;
  int sec = (*secs) % 60;
  int hour = min / 60;
  int day = hour / 24;
  char* str = get_text_buffer();
    
  min = min % 60;
    
  *str = 0;
  if (day) {
    sprintf(str, "%d:%02d:%02d:%02d", day, hour, min, sec);
  } else if (hour) {
    sprintf(str, "%02d:%02d:%02d", hour, min, sec);
  } else {
    sprintf(str, "%02d:%02d", min, sec);
  }
  return str;
}
const char* print_duration_unit(int* secs) {
  int day, hour, min, sec;
  const char* pfx;
  char* str = get_text_buffer();
    
  if ((*secs) < 0) {
    (*secs) = -(*secs);
    pfx = "-";
  } else {
    pfx = "";
  }
    
  sec = (*secs) % 60;
  min = (*secs) / 60;
  hour = min / 60;
  day = hour / 24;
  min = min % 60;
  hour = hour % 24;
    
  if (day) {
    sprintf(str, "%s%d:%02d d", pfx, day, hour);
  } else if (hour) {
    sprintf(str, "%s%02d:%02d h", pfx, hour, min);
  } else if (min) {
    sprintf(str, "%s%02d:%02d m", pfx, min, sec);
  } else {
    sprintf(str, "%s%02d s", pfx, sec);
  }
  return str;
}
const char* print_duration_unit_exact(int* secs) {
  int day, hour, min, sec;
  char* str = get_text_buffer();
    
  sec = (*secs) % 60;
  min = (*secs) / 60;
  hour = min / 60;
  day = hour / 24;
  min = min % 60;
  hour = hour % 24;
    
  if (day) {
    sprintf(str, "%dd %02dh %02dm %02ds", day, hour, min, sec);
  } else if (hour) {
    sprintf(str, "%dh %02dm %02ds", hour, min, sec);
  } else if (min) {
    sprintf(str, "%dm %02ds", min, sec);
  } else {
    sprintf(str, "%ds", sec);
  }
  return str;
}
const char* print_speed(int* bytes) {
  char* str = get_text_buffer();
    
  if ((*bytes) < 1024)
    sprintf(str, "%d B/s", (*bytes));
  else if ((*bytes) < 1024 * 128)
    sprintf(str, "%.2f KiB/s", (double) (*bytes) / 1024.0);
  else if ((*bytes) < 1024 * 1024)
    sprintf(str, "%.1f KiB/s", (double) (*bytes) / 1024.0);
  else
    sprintf(str, "%.1f MiB/s", (double) (*bytes) / 1024.0 / 1024.0);
    
  return str;
}
const char* print_ip(ip_t* value) {
  char* result = get_text_buffer();
  sprintf(result, "%s", ntoa(*value));
  return result;
}
const char* print_u32(guint32* value) {
  char* result = get_text_buffer();
  sprintf(result, "%u", *value);
  return result;
}
const char* print_hash(hash_t* hash) {
  unsigned int i1;
  char* result;
    
  result = g_malloc(hash->len*2+1);
    
  for (i1 = 0; i1 < hash->len; i1++) {
    sprintf(result+2*i1, "%02x", hash->hash[i1]);
  }
  result[hash->len*2] = 0;
  return auto_text(result);
}
const char* print_ptr(void* value) {
  char* result = get_text_buffer();
  sprintf(result, "%p", value);
  return result;
}

////////////////
// STORE functions
static char* store_string_real(char* old_value, const char* new_value) {
  char* result;
  if (old_value && !strcmp(old_value, new_value))
    return old_value;
  result = g_strdup(new_value);
  g_free(old_value);
  return result;
}
static int* store_int_real(int* old_value, int new_value) {
  int* result;
  if (old_value && *old_value == new_value) return old_value;
  result = g_malloc(sizeof(*result));
  *result = new_value;
  g_free(old_value);
  return result;
}
static ip_t* store_ip_real(ip_t* old_value, ip_t new_value) {
  ip_t* result;
  if (old_value && *old_value == new_value) return old_value;
  result = g_malloc(sizeof(*result));
  *result = new_value;
  g_free(old_value);
  return result;
}
static int* store_int_minmax_real(int* old_value, int min, int max) {
  int* result;
  if (old_value && old_value[0] == min && old_value[1] == max)
    return old_value;
  result = g_malloc(sizeof(*result)*2);
  result[0] = min;
  result[1] = max;
  g_free(old_value);
  return result;
}
static file_size_t*
store_filesize_minmax_real(file_size_t* old_value,
			   file_size_t min, file_size_t max) {
  file_size_t* result;
  if (old_value && old_value[0] == min && old_value[1] == max)
    return old_value;
  result = g_malloc(sizeof(*result)*2);
  result[0] = min;
  result[1] = max;
  g_free(old_value);
  return result;
}
static gboolean* store_bool_real(gboolean* old_value, gboolean new_value) {
  gboolean* result;
  if (old_value && *old_value == new_value) return old_value;
  result = g_malloc(sizeof(*result));
  *result = new_value;
  g_free(old_value);
  return result;
}
static hash_t* store_hash_real(hash_t* old_value, unsigned char* hash,
			       unsigned len) {
  hash_t* result;
  if (old_value && old_value->len == len && 
      !memcmp(old_value->hash, hash, len)) return old_value;
  result = g_malloc(sizeof(*result));
  result->len = len;
  result->hash = g_malloc(len);
  memcpy(result->hash, hash, len);
  free_hash(old_value);
  return result;
}
void* store_ptr_real(void* old_value, void* new_value) {
  (void)old_value;
  return new_value;
}

char* store_string(char* old_value, va_list ap) {
  char* new_value = va_arg(ap, char*);
  return store_string_real(old_value, new_value?new_value:"");
}
int* store_int(int* old_value, va_list ap) {
  int val = va_arg(ap, int);
  return store_int_real(old_value, val);
}
ip_t* store_ip(ip_t* old_value, va_list ap) {
  ip_t val = va_arg(ap, ip_t);
  return store_ip_real(old_value, val);
}
int* store_int_minmax(int* old_value, va_list ap) {
  int min = va_arg(ap, int);
  int max = va_arg(ap, int);
  return store_int_minmax_real(old_value, min, max);
}
file_size_t* store_filesize_minmax(file_size_t* old_value, va_list ap) {
  file_size_t min = va_arg(ap, file_size_t);
  file_size_t max = va_arg(ap, file_size_t);
  return store_filesize_minmax_real(old_value, min, max);
}
gboolean* store_bool(gboolean* old_value, va_list ap) {
  gboolean val = va_arg(ap, gboolean);
  return store_bool_real(old_value, val);
}
hash_t* store_hash(hash_t* old, va_list ap) {
  unsigned char* hash = va_arg(ap, char*);
  unsigned len = va_arg(ap, unsigned);
  return store_hash_real(old, hash, len);
}
void* store_ptr(void* old_value, va_list ap) {
  void* val =  va_arg(ap, void*);
  return store_ptr_real(old_value, val);
}


////////////////
// STORE from string functions
char* store_str_string(char* old_value, const char* new_value) {
  return store_string_real(old_value, new_value?new_value:"");
}
int* store_str_int(int* old_value, const char* new_value) {
  return store_int_real(old_value, new_value?atoi(new_value):0);
}
ip_t* store_str_ip(ip_t* old_value, const char* new_value) {
  return store_ip_real(old_value, new_value?inet_addr(new_value):0);
}
int* store_str_int_minmax(int* old_value, const char* new_value) {
  if (new_value) {
    char* temp = l_strdup(new_value);
    char* pos = strchr(temp, '-');
    int min, max;

    if (pos) *pos = 0;
    min = atoi(temp);
    max = pos?atoi(pos+1):0;
    return store_int_minmax_real(old_value, min, max);
  } else {
    return store_int_minmax_real(old_value, 0, 0);
  }
}
file_size_t* store_str_filesize_minmax(file_size_t* old_value, const char* new_value) {
  if (new_value) {
    char* temp = l_strdup(new_value);
    char* pos = strchr(temp, '-');
    file_size_t min, max;

    if (pos) *pos = 0;
    min = strtoll(temp, NULL, 10);
    max = pos?strtoll(pos+1, NULL, 10):0;
    return store_filesize_minmax_real(old_value, min, max);
  } else {
    return store_filesize_minmax_real(old_value, 0, 0);
  }
}
gboolean* store_str_bool(gboolean* old_value, const char* new_value) {
  if (new_value) {
    gboolean val = (strcasecmp(new_value, "no") && strcmp(new_value, "0"));
    return store_bool_real(old_value, val);
  } else {
    return store_bool_real(old_value, FALSE);
  }
}
hash_t* store_str_hash(hash_t* old, const unsigned char* str) {
  static char hex_tab[256] = {
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0,10,11,12,13,14,15, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
  };

  if (str) {
    unsigned len = strlen(str)/2;
    unsigned i1;
    char* temp = g_malloc(len);

    for (i1 = 0; i1 < len; i1++) {
      temp[i1] = hex_tab[str[i1*2]]*16 + hex_tab[str[i1*2+1]];
    }
    old = store_hash_real(old, temp, len);
    g_free(temp);
  } else {
    old = store_hash_real(old, "", 0);
  }
  return old;
}
void* store_str_ptr(void* old_value, const char* new_value) {
  (void)old_value;
  (void)new_value;
  // FIXME: we dont convert str_ptr -> real_ptr atm
  return NULL;
}


////////////////
// COMPARE functions
int compare_string(const char* val1, const char* val2) {
  if (!val2) return (val1!=0);
  if (!val1) return -1;
  return strcmp(val1, val2);
}
int compare_int(const int* val1, const int* val2) {
  if (!val2) return (val1!=0);
  if (!val1) return -1;
  return (*val1==*val2)?0:(*val1<*val2)?-1:1;
}
int compare_ip(const ip_t* val1, const ip_t* val2) {
  if (!val2) return (val1!=0);
  if (!val1) return -1;
  return (*val1==*val2)?0:(*val1<*val2)?-1:1;
}
int compare_int_minmax(const int* val1, const int* val2) {
  if (!val2) return (val1!=0);
  if (!val1) return -1;
  return (val1[0]!=val2[0] || val1[1]!=val2[1]);
}
int compare_filesize_minmax(const file_size_t* val1, const file_size_t* val2) {
  if (!val2) return (val1!=0);
  if (!val1) return -1;
  return (val1[0]!=val2[0] || val1[1]!=val2[1]);
}
int compare_bool(const gboolean* val1, const gboolean* val2) {
  if (!val2) return (val1!=0);
  if (!val1) return -1;
  return (!(*val1) && *val2)?-1:(*val1 && !(*val2))?1:0;
}
int compare_hash(const hash_t* h1, const hash_t* h2) {
  if (!h2) return (h1!=0);
  if (!h1) return -1;
  if (h1->len < h2->len) return -1;
  if (h1->len > h2->len) return 1;
  return memcmp(h1->hash, h2->hash, h1->len);
}
int compare_ptr(const void* p1, const void* p2) {
  return (p1 == p2);
}


////////////////
// COPY functions
char* copy_string(char* dest, char* source) {
  return store_string_real(dest, source);
}
int* copy_int(int* dest, int* source) {
  return store_int_real(dest, *source);
}
ip_t* copy_ip(ip_t* dest, ip_t* source) {
  return store_ip_real(dest, *source);
}
int* copy_int_minmax(int* dest, int* source) {
  return store_int_minmax_real(dest, source[0], source[1]);
}
file_size_t* copy_filesize_minmax(file_size_t* dest, file_size_t* source) {
  return store_filesize_minmax_real(dest, source[0], source[1]);
}
gboolean* copy_bool(gboolean* dest, gboolean* source) {
  return store_bool_real(dest, *source);
}
hash_t* copy_hash(hash_t* dest, hash_t* source) {
  return store_hash_real(dest, source->hash, source->len);
}
void* copy_ptr(void* dest, void* source) {
  return store_ptr_real(dest, source);
}


void free_hash(hash_t* hash) {
  if (!hash) return;
  g_free(hash->hash);
  g_free(hash);
}

unsigned char* base64_decode(const char* data, int* dst_len) {
  static const char base64[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  unsigned char *dst, *out;
  char *p;
  int i;
  unsigned char in[4];

  if (!data) return NULL;

  if ((out = dst = g_malloc(strlen(data)+1)) == NULL) return NULL;

  for (i=0, *dst_len=0; *data; data++) {
    if ((p = strchr(base64, *data)) == NULL) continue;

    in[i++] = (unsigned char)(p - base64);

    if (i == 4) {
      dst[0] = (in[0] << 2) | ((in[1] & 0x30) >> 4);
      dst[1] = ((in[1] & 0x0F) << 4) | ((in[2] & 0x3C) >> 2);
      dst[2] = ((in[2] & 0x03) << 6) | (in[3] & 0x3F);
      dst += 3;
      *dst_len += 3;
      i = 0;
    }
  }

  if (i >= 2) {
    dst[0] = (in[0] << 2) | ((in[1] & 0x30) >> 4);
    (*dst_len)++;
  }

  if (i == 3) {
    dst[1] = ((in[1] & 0x0F) << 4) | ((in[2] & 0x3C) >> 2);
    (*dst_len)++;
  }
  out[*dst_len] = 0;
  return out;
}

char* mfgets(char* buf, int size, FILE* file) {
  if (fgets(buf, size, file)) {
    int len = strlen(buf);
    if (len && buf[len-1] == '\n') {
      buf[len-1] = 0;
    } else if (!feof(file)) {
      buf[len-1] = 0;
      printf("Error: line '%s' too long\n", buf);
    }
    return buf;
  }
  return 0;
}

#ifdef WIN32
#ifndef HAVE_GETTIMEOFDAY
 int gettimeofday (struct timeval *tv, void* tz)
{
  union {
    __int64 ns100; //time since 1 Jan 1601 in 100ns units
    FILETIME ft;
  } now;

  GetSystemTimeAsFileTime (&now.ft);
  tv->tv_usec = (long) ((now.ns100 / 10LL) % 1000000LL);
  tv->tv_sec = (long) ((now.ns100 - 116444736000000000LL) / 10000000LL);
  return (0);
} 
# endif
#endif

#ifdef WIN32
# define _UTILS_C_
# include "lcorealiasdef.c"
#endif
