/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.11 $
 * $Date: 2007/11/23 12:42:44 $
 * $Author: sid4u $
 */

#ifndef _UTILS_H_
#define _UTILS_H_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <glib.h>
#include <stdarg.h>
#include <stdio.h>
#include "l_time.h"
#ifdef WIN32
#include "win32_defs.h"
#endif

G_BEGIN_DECLS

#define TIME_NONE          0
#define TIME_HOUR_MIN      1
#define TIME_HOUR_MIN_SEC  2

#define SDIR(name) name G_DIR_SEPARATOR_S
#define DIR_SEP G_DIR_SEPARATOR

typedef gint64 file_size_t;
typedef guint32 ip_t;
typedef guint16 port_t;

#define FF "%"G_GINT64_FORMAT  // format string for filesizes

#define IP_FROM_LE(arg) GUINT32_FROM_LE(arg)
#define IP_TO_LE(arg) GUINT32_TO_LE(arg)

/* #define UTF8_CR "\302\251"     // (C)  copyright */
/* #define UTF8_RT "\302\256"     //  R   registered trademark */
/* #define UTF8_TM "\342\204\242" // TM   trademark */

typedef struct {
  unsigned int len;
  unsigned char* hash;
} hash_t;


typedef const char* (*UIPrintFunc) (void*);
typedef const char* (*XMLPrintFunc)(void*);
typedef int         (*CompareFunc) (void*, void*);
typedef void*       (*CopyFunc)    (void*, void*);
typedef void        (*FreeFunc)    (void*);
typedef void*       (*StoreFunc)   (void*, va_list);
typedef void*       (*StoreStrFunc)(void*, const char*);

const char* print_timestamp(int* mode);
char* ntoa(ip_t ip);
void std_log(const char* domain, GLogLevelFlags level, const char* text);
void line_insert_lf(char *line, int pos);
gboolean pattern_match(const gchar* match_pattern, const gchar* match_string);

int text_buffer_size(void);
char* get_text_buffer(void);
char* auto_text(char* text);
char* l_printf(const char* fmt, ...);
char* l_strdup(const char* source);


char* to_utf8(const char* text, gsize len, gsize* written, GError** error, unsigned num, ...);
char* from_utf8(const char* text, gsize len, gsize* written, GError** error, unsigned num, ...);
char* locale_from_utf8(const char* text);
char* locale_to_utf8(const char* text);


file_size_t  extract_bytes   (const char* string);
int          extract_duration(const char* string);

char* arg(char *data, int flags);
char* arg2(char *data, int flags);

char* extract_filename (char* data);

void  make_list_from_string(GSList** glist, const char *text, const char *sep);
char* make_string_from_list(GSList* glist, const char *sep);

int create_dir(char *dir);
gboolean directory_exists(const char *dir);

void print_raw(const char* header, unsigned char* message, int length);
void play_file(const char* filename, const char* app);
char* get_suffix(const char* filename);

int pack_byte       (char* buffer, char value);
int pack_u16        (char* buffer, guint16 value);
int pack_u16h       (char* buffer, guint16 value);
int pack_u32        (char* buffer, guint32 value);
int pack_u32h       (char* buffer, guint32 value);
int pack_u32d       (char* buffer, guint64 value);
int pack_string_u8  (char* buffer, const char* str);
int pack_string_u16 (char* buffer, const char* str);
int pack_string_u32 (char* buffer, const char* str);
int pack_string_raw (char* buffer, const char* str, int len);

unsigned char unpack_byte(char** buffer, unsigned* len);
guint16 unpack_u16       (char** buffer, unsigned* len);
guint16 unpack_u16h      (char** buffer, unsigned* len);
guint32 unpack_u32       (char** buffer, unsigned* len);
guint32 unpack_u32h      (char** buffer, unsigned* len);
guint64 unpack_u32d      (char** buffer, unsigned* len);
char*   unpack_string_u8 (char** buffer, unsigned* size);
char*   unpack_string_u16(char** buffer, unsigned* size);
char*   unpack_string_u32(char** buffer, unsigned* size);

// printing functions
const char* print_string       (char* text);
const char* print_int          (int* value);
const char* print_int_minmax   (int* bytes);
const char* print_filesize_minmax(file_size_t* bytes);
const char* print_bool         (int* value);
const char* print_bitrate      (int* value);
// X Hz
const char* print_frequency    (int* value);
// vbr/(null)
const char* print_vbr          (int* value);
// 5 B/7 KB/...
const char* print_bytes_exact  (file_size_t* bytes);
const char* print_seconds_exact(int* seconds);
const char* print_filesize     (file_size_t* bytes);
const char* print_duration     (int* secs);
const char* print_duration_unit(int* secs);
const char* print_duration_unit_exact(int* secs);
const char* print_speed        (int* bytes);
const char* print_ip  (ip_t* value);
const char* print_hash(hash_t* hash);
const char* print_ptr (void* value);

char*     store_string(char*     old_value, va_list ap);
int*      store_int   (int*      old_value, va_list ap);
ip_t*     store_ip    (ip_t*     old_value, va_list ap);
int*      store_int_minmax(int*  old_value, va_list ap);
file_size_t* store_filesize_minmax(file_size_t* old_value, va_list ap);
gboolean* store_bool  (gboolean* old_value, va_list ap);
hash_t*   store_hash  (hash_t*   old,       va_list ap);
void*     store_ptr   (void*     old_value, va_list ap);

char*     store_str_string(char*     old_value, const char* new_value);
int*      store_str_int   (int*      old_value, const char* new_value);
int*      store_str_int_minmax(int*  old_value, const char* new_value);
file_size_t* store_str_filesize_minmax(file_size_t* old_value, const char* new_value);
gboolean* store_str_bool  (gboolean* old_value, const char* new_value);
ip_t*     store_str_ip(ip_t* old_value, const char* new_value);
hash_t*   store_str_hash(hash_t* old, const unsigned char* str);
void*     store_str_ptr(void* old_value, const char* new_value);

int compare_string    (const char* val1, const char* val2);
int compare_int       (const int*  val1, const int*  val2);
int compare_int_minmax(const int*  val1, const int*  val2);
int compare_filesize_minmax(const file_size_t* val1, const file_size_t* val2);
int compare_bool      (const gboolean* val1, const gboolean* val2);
int compare_ip  (const ip_t* val1, const ip_t* val2);
int compare_hash(const hash_t* h1, const hash_t* h2);
int compare_ptr (const void* p1, const void* p2);

char*     copy_string(char* dest, char* source);
int*      copy_int(int* dest, int* source);
int*      copy_int_minmax(int* dest, int* source);
file_size_t* copy_filesize_minmax(file_size_t* dest, file_size_t* source);
gboolean* copy_bool(gboolean* dest, gboolean* source);
ip_t*     copy_ip(ip_t* dest, ip_t* source);
hash_t*   copy_hash(hash_t* dest, hash_t* source);
void*     copy_ptr(void* dest, void* source);

void free_hash(hash_t* hash);

unsigned char* base64_decode(const char* data, int* dst_len);
char* mfgets(char *buf, int size, FILE *f);

#ifdef WIN32
#ifndef HAVE_GETTIMEOFDAY
int gettimeofday(struct timeval *tv, void* tz);
# endif
#endif

G_END_DECLS
#endif
