/* Copyright (C) 2006 Lopster Team
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */

/* $Revision: 1.3 $
 * $Date: 2006/07/30 18:39:15 $
 * $Author: sid4u $
 */

#include <string.h>
#include <glib.h>
#include "bt_bitfield.h"


// use *_free() instead of *_destroy if no deriving from object_t
void bt_bitfield_free(bt_bitfield_t* bfield)
{
    g_free(bfield->bits);
    g_free(bfield);
}
 
static void bt_bitfield_init(bt_bitfield_t* bfield, char* raw_bitfield, int btsize) {
    int i;
    char null_char[1] = {0};
    bfield->btsize = btsize; //strlen(raw_bitfield);
    bfield->bits = g_strndup(null_char,btsize);

    if (raw_bitfield != NULL)
    {
        for ( i = 0; i < btsize; i++) 
        {
            bfield->bits[i] = raw_bitfield[ btsize - i -1 ];
        }
    }
}

static bt_bitfield_t* bt_bitfield_new( char* raw_bitfield, int btsize )
{
    bt_bitfield_t* bfield ;
    bfield = g_malloc0(sizeof(*bfield));
    bt_bitfield_init(bfield, raw_bitfield, btsize); 
    return bfield;
}

void bt_bitfield_set( bt_bitfield_t*  bfield, int bit) 
{
    int byte = bit/8;
    int mask = 0x80 >> (bit %8);
    bfield->bits[byte] |= mask;
}

void bt_bitfield_clear(bt_bitfield_t*  bfield , int bit) 
{
    int byte = bit/8;
    int mask = 0x80 >> (bit %8);
    bfield->bits[byte] &= ~mask;
}

static void bt_bitfield_transform_binary_representation(bt_bitfield_t*  bfield, char* binary_bitfield )
{
    int i;
    int btsize;
    btsize = strlen(binary_bitfield);
    for ( i = 0 ;i < btsize; i++) 
    {
        if (binary_bitfield[i] == '1') {
            bt_bitfield_set(bfield,i);
        }
        else {
            bt_bitfield_clear(bfield,i);
        }
    }
}


// This is called to create a bitfield corresponding to the given raw_bitfield sent by peer
// and also we put a second parameter for a check that is the number pieces that torrent have
// Normally (nb_pieces / 8) + (nb_pieces % 8) = strlen(raw_bitfield)
bt_bitfield_t* bt_bitfield_setup_from_peer(int nb_pieces, char* raw_bitfield, int btsize )
{
    bt_bitfield_t* bfield;
    int check_size;
   // char* new_bitfield;
    //bitfield_size = strlen(raw_bitfield);
    g_log("BT_BFIELD", G_LOG_LEVEL_DEBUG,"bt_bitfield_setup_from_peer (%d), btsize(%d)", nb_pieces, btsize);
    check_size = nb_pieces / 8;
    if ((nb_pieces) % 8 > 0) check_size++;
    if ( check_size != btsize) {
        // not same size. peer sent us something wrong here
        g_log("BT_BFIELD", G_LOG_LEVEL_DEBUG,"bt_bitfield_setup_from_peer check_size (%d), btsize(%d)", check_size, btsize);
        return NULL;
    }
   // new_bitfield = g_strdup( raw_bitfield );
    // Matching pieces size. Let's create a bt_bitfield_t and send it back
    bfield = bt_bitfield_new( raw_bitfield , btsize);
    return bfield;
}

// If torrent was saved at some time and is starting back we send a binary bitfield in ascii
// corresponding to the pieces we have
// else it will still be a full string ascii filled with 0
bt_bitfield_t* bt_bitfield_setup_from_client(char* binary_bitfield)
{
    bt_bitfield_t* bfield;
/*     char* raw_bitfield; */
    int bitfield_size;
    bitfield_size = strlen(binary_bitfield);
    
    // bitfield_size % 8 should be 0
    
    if ( (bitfield_size % 8) != 0) return NULL;  

 /*  raw_bitfield = malloc( sizeof(char*)  * ((bitfield_size/8) + 1)  );
    for (i = 0; i < bitfield_size; i++) 
    {
        // fill with 00 to begin with
        raw_bitfield[i] = 0x00;
    } 
    raw_bitfield[bitfield_size/8] = '\0'; */
    g_log("BT_BFIELD", G_LOG_LEVEL_DEBUG,"bt_bitfield_setup_from_client (%d) ", bitfield_size);

    bfield = bt_bitfield_new( NULL, bitfield_size/8 );
    bt_bitfield_transform_binary_representation(bfield,binary_bitfield);
    return bfield;
}


gboolean bt_bitfield_has_piece( bt_bitfield_t*  bfield , int piece_index) 
{
    int byte = piece_index/8;
    int mask = 0x80 >> (piece_index %8);
    if ( (bfield->bits[byte] & mask) != 0 ) 
        return TRUE;
    else
        return FALSE;
}

gboolean bt_bitfield_empty( bt_bitfield_t*  bfield ) 
{
    int piece_index;
    int i;
    
    if (bfield == NULL)
        return TRUE;

    piece_index = bfield->btsize * 8;

    for (i = 0 ; i < piece_index ; i++) 
    {
        int byte = piece_index/8;
        int mask = 0x80 >> (piece_index %8);
        if ( (bfield->bits[byte] & mask) != 0 ) 
            return FALSE;
    }
    return TRUE;
}

gboolean bt_bitfield_has_all( bt_bitfield_t*  bfield ) 
{
    int piece_index;
    int i;
    
    if (bfield == NULL)
        return FALSE;

    piece_index = bfield->btsize * 8;

    for (i = 0 ; i < piece_index ; i++) 
    {
        int byte = i/8;
        int mask = 0x80 >> (piece_index %8);
        if ( (bfield->bits[byte] & mask) == 0 ) 
            return FALSE;
    }
    return TRUE;
}

//This function compares two bitfields, if one has a piece the other doesn't have, it returns TRUE.
gboolean bt_bitfield_compare(bt_bitfield_t*  bfield1, bt_bitfield_t*  bfield2 )
{


    int piece_index1;
    int piece_index2;
    int i;

    g_assert(bfield1);
    g_assert(bfield2);
    
    piece_index1 = bfield1->btsize * 8;
    piece_index2 = bfield2->btsize * 8;
    //g_log("BT_TRACKER", G_LOG_LEVEL_WARNING,"bt_bitfield_compare (%d)  (%d) ", piece_index1, piece_index2);

    if (piece_index1 != piece_index2) return FALSE;

    for (i = 0 ; i < piece_index1 ; i++) 
    {
        // This means : if peer has a piece we don't have
        if (bt_bitfield_has_piece(bfield2, i) && !bt_bitfield_has_piece(bfield1,i))
        {
            return TRUE;
        }
    }
    return FALSE;
}

/*  This function would update the first bitfield 
    (corresponding to our downloading torrent with only the rarest pieces )
*/
void bt_bitfield_update_rarest(bt_bitfield_t*  bfield1, bt_bitfield_t*  bfield2 )
{
    g_assert(bfield1);
    g_assert(bfield2);
    //TODO

}

gboolean bt_bitfield_complete(bt_bitfield_t*  bitfield)
{
    int piece_index;
    int i;
    piece_index = bitfield->btsize * 8;
    for (i = 0 ; i < piece_index ; i++) 
    {
        if (!bt_bitfield_has_piece(bitfield, i) )
        {
            return FALSE;
        }
    }
    return TRUE;
}
