/* Copyright (C) 2005-2006 (C) Lopster Team
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.3 $
 * $Date: 2006/07/30 18:39:15 $
 * $Author: sid4u $
 */

#include <string.h>

#include "lcore/utils.h"
#include "lcore/library.h"

#include "bt_torrent.h"
#include "bt_peer.h"
#include "bt_utils.h"
#include "bt.h"
#include "bt_peer_message.h"
#include "bt_peer_protocol.h"

#define BT_REC(level) \
 g_log("BT_PROTO", level, "R: %d [%s]", message->id, message->message?message->message:"NO MESSAGE")
#define HANDLER(f) static void f (bt_torrent_t* torrent, bt_peer_message_t* payload)
#define BT_CALLC(f) f(torrent, message )


// these messages are the one coming through our listening port.
// TODO : Handle these message accoriding to the situation (current peers downloading from etc)
HANDLER(handle_handshake)
{
    (void)torrent;
    (void)payload;
}

HANDLER(keep_alive)
{
    (void)torrent;
    (void)payload;
}

HANDLER(choke)
{
    (void)torrent;
    (void)payload;
}

HANDLER(unchoke)
{
    (void)torrent;
    (void)payload;
}

HANDLER(interested )
{
    (void)torrent;
    (void)payload;
}

HANDLER(not_interested )
{
    (void)torrent;
    (void)payload;
}

HANDLER(have)
{
    (void)torrent;
    (void)payload;
}

HANDLER(bitfield )
{
    (void)torrent;
    (void)payload;
}

HANDLER(request )
{
    (void)torrent;
    (void)payload;
}

HANDLER(piece )
{
    (void)torrent;
    (void)payload;
}

HANDLER(cancel )
{
    (void)torrent;
    (void)payload;
}
/*
HANDLER(port ) {
  
}
*/

static void bt_pack_command(buffer_t* buffer, guint32 id, va_list ap)
{
    char* pos = NULL;
    char* save_offset;
    int len;

    switch (id)
    {
    case BT_MESSAGE_HANDSHAKE:
    {
        // input: hash, peer_id 
        len = 68;
        pos = buffer_need_space(buffer, len);
        // saving offset to add back data to buffer (seems it won't work 
        save_offset = pos;
        pos += pack_string_u8(pos, "BitTorrent protocol");
        pos += pack_string_raw(pos, "\0\0\0\0\0\0\0\0", 8);
        pos += pack_string_raw(pos,va_arg(ap, char*), SHA_DIGEST_LENGTH);
        pos += pack_string_raw(pos, va_arg(ap, char*), 20);
        g_log("BT_PEER", G_LOG_LEVEL_DEBUG, "S: [handshake]");
        break;
    }
    case BT_MESSAGE_CHOKE:
    case BT_MESSAGE_UNCHOKE:
    case BT_MESSAGE_INTERESTED:
    case BT_MESSAGE_NOT_INTERESTED:
        // input: none
        len = 5;
        pos = buffer_need_space(buffer, len);
        save_offset = pos;
        pos += pack_u32h(pos, 1);
        pos += pack_byte(pos, (char)id);
        g_log("BT_PEER", G_LOG_LEVEL_DEBUG, "S: [%u]", id);
        break;
    case BT_MESSAGE_REQUEST:
    {
        // input: index, offset, size
        guint32 begin  = va_arg(ap, guint32);
        guint32 offset = va_arg(ap, guint32);
        guint32 size   = va_arg(ap, guint32);
        len = 17;
        pos = buffer_need_space(buffer, len);
        save_offset = pos;
        pos += pack_u32h(pos, 13);
        pos += pack_byte(pos, (char)id);
        pos += pack_u32h(pos, begin);
        pos += pack_u32h(pos, offset);
        pos += pack_u32h(pos, size);
        g_log("BT_PEER", G_LOG_LEVEL_DEBUG, "S: [%u] %u %u %u",
              id, begin, offset, size);
        break;
    }
    default:
        g_log("BT_PEER", G_LOG_LEVEL_WARNING,
              "unknown command to send: %u", id);
    }
    if (pos)
    {
        pos = save_offset;
        buffer_add_data(buffer, pos, len);
    }
}

void bt_peer_send(b_socket_t* sock, guint32 type, ...)
{
  
    if (sock/* && socket_connect(SOCK(sock))*/)
    {
        buffer_t* buffer = b_socket_get_out_buffer(sock, 1);
        va_list ap;
        va_start(ap, type);
        bt_pack_command(buffer, type, ap);
        va_end(ap);
        b_socket_start_sending(sock);
    }
}

// Bitfield Message -> ID : 5 , LEN = 1 + BITFIELD LENGTH
// Fixme : redo the function (with network byte ordering)
void bt_peer_send_bitfield( bt_peer_t* peer, bt_torrent_t* torrent )
{
 /*   bt_bitfield_t* bitfield;
    char* header;
    g_assert(peer);
    g_assert(torrent);
    
    bitfield = torrent->bitfield;
    g_assert(bitfield);
    header =  bt_peer_proto_create_message_header(bitfield->btsize + 1, 5);
    bt_send(BSOCK(peer),bitfield->btsize + 1 + 4 ,"%s%s",header,bitfield->bits); */
}


void bt_peer_send_have( int piece_index  )
{
    //TODO : To all connected peers, a have message will be sent when a piece is completed
}


void bt_handle_incoming_peer_message(bt_torrent_t* torrent, bt_peer_message_t* message)
{
    switch (message->id)
    {
    case BT_MESSAGE_KEEP_ALIVE:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(keep_alive);
        break;    
    case BT_MESSAGE_CHOKE:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(unchoke);
        break; 
    case BT_MESSAGE_UNCHOKE:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(unchoke);
        break; 
    case BT_MESSAGE_INTERESTED:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(interested);
        break; 
    case BT_MESSAGE_NOT_INTERESTED:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(not_interested);
        break; 
    case BT_MESSAGE_HAVE:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(have);
        break; 
    case BT_MESSAGE_BITFIELD:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(bitfield);
        break; 
    case BT_MESSAGE_REQUEST:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(request);
        break; 
    case BT_MESSAGE_PIECE:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(piece);
        break; 
    case BT_MESSAGE_CANCEL:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(cancel);
        break; 
/*    case BT_MESSAGE_PORT:
        BT_REC(G_LOG_LEVEL_DEBUG);
        BT_CALLC(port);
        break;  */
    }
}
