/* Copyright (C) 2005 sid4u@users.sourceforge.net, 2006 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2006/07/23 08:11:44 $
 * $Author: sid4u $
 */

#include <string.h>
#include <stdlib.h>

#include "lcore/attribute.h"
#include "bt_piece.h"

typedef struct {
    module_t _module;
} bt_pieces_module_t;

static bt_pieces_module_t* BtPiecesModule = NULL;


static const char* print_pieces(bt_pieces_t* pieces)
{
    unsigned int i1;
    char* header =
        l_printf("%u %u %u ", pieces->offset, pieces->pcnt, pieces->pbytes);
    unsigned len = strlen(header);
    char* result = g_malloc(len + pieces->hash_len*2 + 1);
    
    memcpy(result, header, len);
    
    for (i1 = 0; i1 < pieces->hash_len; i1++)
    {
        sprintf(result+len+2*i1, "%02x", pieces->hash[i1]);
    }
    result[len + pieces->hash_len*2] = 0;
    return auto_text(result);
}

static int compare_pieces(const bt_pieces_t* h1, const bt_pieces_t* h2)
{
    if (!h2) return (h1!=0);
    if (!h1) return -1;
    if (h1->hash_len < h2->hash_len) return -1;
    if (h1->hash_len > h2->hash_len) return 1;
    if (h1->pcnt < h2->pcnt) return -1;
    if (h1->pcnt > h2->pcnt) return 1;
    if (h1->pbytes < h2->pbytes) return -1;
    if (h1->pbytes > h2->pbytes) return 1;
    return memcmp(h1->hash, h2->hash, h1->hash_len);
}

static void free_pieces(bt_pieces_t* hash)
{
    if (!hash) return;
    g_free(hash->hash);
    g_free(hash);
}

static bt_pieces_t* store_pieces_real(
        bt_pieces_t* old_value, unsigned char* hash, unsigned len,
        unsigned bytes, unsigned cnt, int offset)
{
    bt_pieces_t* result;
    if (old_value && old_value->hash_len == len && 
        !memcmp(old_value->hash, hash, len))
    {
        old_value->pbytes = bytes;
        old_value->pcnt = cnt;
        old_value->offset = offset;
        return old_value;
    }
    result = g_malloc(sizeof(*result));
    result->hash_len = len;
    result->hash = g_malloc(len);
    memcpy(result->hash, hash, len);
    result->pbytes = bytes;
    result->pcnt = cnt;
    result->offset = offset;

    free_pieces(old_value);
    return result;
}

static bt_pieces_t* copy_pieces(bt_pieces_t* dest, bt_pieces_t* source)
{
    return store_pieces_real(dest, source->hash, source->hash_len,
                             source->pbytes, source->pcnt, source->offset);
}

static bt_pieces_t* store_pieces(bt_pieces_t* old, va_list ap)
{
    unsigned char* hash = va_arg(ap, char*);
    unsigned len = va_arg(ap, unsigned);
    unsigned bytes = va_arg(ap, unsigned);  // bytes a piece represents
    unsigned cnt = va_arg(ap, unsigned);  // number of pieces in hash
    int offset = va_arg(ap, int);  // file offset of the first piece
    return store_pieces_real(old, hash, len, bytes, cnt, offset);
}

static bt_pieces_t* store_str_pieces(bt_pieces_t* old, const unsigned char* str)
{
    static char hex_tab[256] = {
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0,10,11,12,13,14,15, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
        0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    };
    const unsigned char* pos;
    int offset;
    unsigned cnt;
    unsigned bytes;

    pos = strchr(str, ' ');
    if (!pos) return old;
    offset = atoi(pos);

    pos = strchr(pos+1, ' ');
    if (!pos) return old;
    cnt = atoi(pos);

    pos = strchr(pos+1, ' ');
    if (!pos) return old;
    bytes = atoi(pos);

    pos = strchr(pos+1, ' ');
    if (pos)
    {
        unsigned len = strlen(str)/2;
        unsigned i1;
        char* temp = g_malloc(len);
        
        for (i1 = 0; i1 < len; i1++) {
            temp[i1] = hex_tab[str[i1*2]]*16 + hex_tab[str[i1*2+1]];
        }
        old = store_pieces_real(old, temp, len, bytes, cnt, offset);
        g_free(temp);
    }
    else
    {
        old = store_pieces_real(old, "", 0, 0, 0, 0);
    }
    return old;
}

static void bt_pieces_mod_init()
{
    attr_register("file", "urn:sha1:pieces", ATTR_SKIP,
                  (UIPrintFunc)print_pieces,
                  (XMLPrintFunc)print_pieces,
                  (CompareFunc)compare_pieces,
                  (CopyFunc)copy_pieces,
                  (FreeFunc)free_pieces,
                  (StoreStrFunc)store_str_pieces,
                  (StoreFunc)store_pieces);
}

static void bt_pieces_mod_run()
{
}

static void bt_pieces_mod_exit()
{
    //FIXME: destroy all torrents.
}

static void bt_pieces_module_init(bt_pieces_module_t* module)
{
    module_init(MODULE(module), "Bt Pieces Module",
                (ModuleFunc)bt_pieces_mod_init,
                (ModuleFunc)bt_pieces_mod_run,
                (ModuleFunc)bt_pieces_mod_exit);

/*     module_require(bt_tracker_module_instance); */
/*     module_require(xml_module_instance); */
/*     module_require(tsocket_module_instance); */
/*     module_require(resolve_module_instance); */
/*     module_require(listen_module_instance); */
}


module_t* bt_pieces_module_instance()
{
    if (!BtPiecesModule)
    {
        BtPiecesModule = g_malloc0(sizeof(*BtPiecesModule));
        bt_pieces_module_init(BtPiecesModule);
    }
    return MODULE(BtPiecesModule);
}
