/* Copyright (C) 2005 The Lopster Team
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */

/* $Revision: 1.7 $
 * $Date: 2007/11/15 12:26:15 $
 * $Author: sid4u $
 */

#ifndef _BT_TORRENT_H_
#define _BT_TORRENT_H_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#include <openssl/sha.h>

#include "lcore/module.h"
#include "lcore/download.h"

#include "bt_bitfield.h"


G_BEGIN_DECLS
/*

Torrent structure (bencoded dictionnaries)

****************************Copied from bittorent.com************************************************

announce : The URL of the tracker. 

info : This maps to a dictionary, with keys described below ->

	-	The NAME key maps to a string which is the suggested name to save the file (or directory) as. It is purely advisory. 

	-	PIECE LENGTH maps to the number of bytes in each piece the file is split into. 
		For the purposes of transfer, files are split into fixed-size pieces which are all the same length 
		except for possibly the last one which may be truncated. Piece length is almost always a power of two, 
		most commonly 2 18 = 256 K (BitTorrent prior to version 3.2 uses 2 20 = 1 M as default). 

	-	PIECES maps to a string whose length is a multiple of 20. 
		It is to be subdivided into strings of length 20, each of which 
		is the SHA1 hash of the piece at the corresponding index. 

	-	There is also a key LENGTH or a key FILES , but not both or neither. 
		If LENGTH is present then the download represents a single file, 
		otherwise it represents a set of files which go in a directory structure. 

		-	In the single file case, LENGTH maps to the length of the file in bytes. 

	-	For the purposes of the other keys, the multi-file case is treated 
		as only having a single file by concatenating the files in the 
		order they appear in the files list. 
		The files list is the value files maps to, and is a list of 
		dictionaries containing the following keys: 

			length The length of the file, in bytes. 
			path A list of strings corresponding to subdirectory names,
			the last of which is the actual file name (a zero length list is an error case). 

			In the single file case, the name key is the name of a file, 
			in the muliple file case, it's the name of a directory. 
****************************Copied from bittorent.com************************************************
*/



enum {
    BT_TORRENT_STOPPED=10020,
    BT_TORRENT_PAUSED,
    BT_TORRENT_RUNNING,
};

typedef struct {
    object_t _object;
    //TODO: make it be a bt_file_t structure
    GList* files;  
    GList* downloads;
    GList* trackers; 
    GList* pieces; 
    GList* users; // List of bt_user_t 
    bt_bitfield_t* bitfield;
    int full_size;
    char* main_directory; // Can be empty by default, initialised also by default or updated by user on ok download torrent
    char* created_by;
    int creation_date;
    char info_hash[SHA_DIGEST_LENGTH];
    int status;
    int uploaded;
    int downloaded;
    int left;
    int no_pieces;
    file_size_t piece_length;
    gboolean seeding_mode;
    GList* peers; // List of connected and transferring peers for that particular torrent. needed for unchokes purposes
} bt_torrent_t;

#include "bt_users.h"

module_t* bt_torrent_module_instance();

bt_torrent_t* bt_torrent_new_from_file(const char* filename);
bt_user_t* bt_torrent_get_next_optimistic_user(bt_torrent_t* torrent);
download_t* bt_torrent_get_download_to_process(bt_torrent_t* torrent);

void bt_torrent_accept_file(bt_torrent_t* torrent, file_t* file);

G_END_DECLS

#endif //_BT_TORRENT_H_
