/* Copyright (C) 2005 sgop@users.sourceforge.net, sid4u@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.3 $
 * $Date: 2007/09/21 09:41:54 $
 * $Author: sid4u $
 */

#include "lcore/plugin.h"
#include "lcore/ui.h"
#include "lcore/dynamic.h"
#include "bt_ui.h"

static ui_bt_t BTUI;

typedef struct {
  module_t _module;
  void (*init)(ui_bt_t*);
  void (*run)(void);
} bt_ui_module_t;

static bt_ui_module_t* BtUIModule = NULL;

#define CHECK_SYMBOL(sym)                                               \
    if (!(sym)) {                                                       \
        g_log("BT", G_LOG_LEVEL_WARNING, "could not find symbol '%s'", #sym); \
    }

static void dummy_func() {}

void bt_ui_load_syms()
{
    const char* filename;
    GModule* handle;
    
    filename = ui_find_library_file("bt");
    if (!filename)
    {
        g_log("BT", G_LOG_LEVEL_DEBUG, "starting plugin without interface");
        return;
    }
    
    handle = lopster_dll_open(filename, G_MODULE_BIND_LAZY|G_MODULE_BIND_LOCAL);
    if (!handle)
    {
        g_log("BT", G_LOG_LEVEL_WARNING, "%s", g_module_error());
        g_log("BT", G_LOG_LEVEL_DEBUG, "starting plugin without interface");
        return;
    }
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_bt_init", (void**)&BtUIModule->init));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_bt_run", (void**)&BtUIModule->run));
}

static void bt_ui_init()
{
    BtUIModule->init(&BTUI);
}

static void bt_ui_run()
{
    BtUIModule->run();
}

static void bt_ui_module_init(bt_ui_module_t* module)
{
    module_init(MODULE(module), "Bt User Interface Module",
                (ModuleFunc)bt_ui_init, (ModuleFunc)bt_ui_run, NULL);
    
    module->init = dummy_func;
    module->run = dummy_func;
    
    module_require(ui_module_instance);
    module_require(dynamic_module_instance);
    
    bt_ui_load_syms();
}

module_t* bt_ui_module_instance()
{
    if (!BtUIModule)
    {
        BtUIModule = g_malloc0(sizeof(*BtUIModule));
        bt_ui_module_init(BtUIModule);
    }
    return MODULE(BtUIModule);
}

#undef CHECK_SYMBOL
