/* Copyright (C) 2005 (sid4u@users.sourceforge.net) 
 * mostly inspired from (sgop@users.sourceforge.net) work on napster/directconnect plugin
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2006/07/30 18:39:15 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>

#include "lcore/lopster.h"
#include "lcore/l_resolve.h"
#include "bt_users.h"
#include "bt_bitfield.h"
#include "bt.h"

typedef struct {
  module_t _module;
  GHashTable* users;
} bt_user_module_t;

static bt_user_module_t* BtUserModule = NULL;


static void bt_user_am_interested_torrent_free(bt_user_torrent* am_interested_torrent)
{
    if (am_interested_torrent->user_bitfield)
        bt_bitfield_free(am_interested_torrent->user_bitfield);
    g_free(am_interested_torrent);
}

static void bt_user_am_interested_torrent_init( bt_user_torrent* am_interested_torrent )
{
    am_interested_torrent->torrent = NULL;
    am_interested_torrent->user_bitfield = NULL;
}

static bt_user_torrent* bt_user_am_interested_torrent_new(void)
{
    bt_user_torrent* am_interested_torrent = g_malloc0(sizeof(*am_interested_torrent));
    bt_user_am_interested_torrent_init(am_interested_torrent);
    return am_interested_torrent;
}


static void bt_user_destroy(bt_user_t* user)
{
    // we could assert here if the user is not found in hash table.
    g_hash_table_remove(BtUserModule->users, OBJ(user)->name);
    g_free(user->client);
    if (user->rate) 
    {
        rate_destroy(user->rate);
        user->rate = NULL;
    }
    if (user->am_interested_torrent)
    {
        bt_user_am_interested_torrent_free(user->am_interested_torrent);
        user->am_interested_torrent = NULL;
    }
    user->interested_torrent_me = NULL;

    user_destroy(USER(user));
}
 
static void bt_user_monitor(bt_user_t* user) {
    // nothing to monitor
    (void)user;
}

static void bt_user_unmonitor(bt_user_t* user) {
    // nothing to unmonitor
    (void)user;
}

static void bt_user_resolver(bt_user_t* user, ip_t ip)
{
    if (ip == 0)
    {
        user->ip_resolved = FALSE;
    }
    else
    {
        user->ip_resolved = TRUE;
        user_set_ip(USER(user), ip);
    }
    user_unref(USER(user));
}

static const char* bt_get_client_from_id(const char* peer_id)
{
    (void)peer_id;
    //TODO: parser peer_id to get the client name, for the moment use unknown client, TEST ALSO IF peer_id is not NULL
    return BT_CLIENT_UNKNOWN;
}

// TODO: (ipv6?) format
static void bt_user_init(bt_user_t* user, const char* peer_id) { /*, const char* ip, int port) {*/
    user_init(USER(user), peer_id, BT_ID);
    object_add_type(OBJ(user), "BtUser");
    OBJ(user)->destroy = (ObjectFunc)bt_user_destroy;
    USER(user)->monitor = (UserMonitor)bt_user_monitor;
    USER(user)->unmonitor = (UserMonitor)bt_user_unmonitor;
    USER(user)->ip = 0;
    user->client = g_strdup(bt_get_client_from_id(peer_id));
    // Needed to update user rate for un/choking
    user->rate = rate_new(0);
    user->seeder = FALSE;
    user->is_snubbed = FALSE;
    user->is_optimistic_unchoke = FALSE;
    user->state = PEER_INCOMING;
    user->am_interested_torrent = NULL;
    user->interested_torrent_me = NULL;
}

static bt_user_t* bt_user_new(const char* peer_id){ /*,const char* ip, int port) { */
    bt_user_t* user = g_malloc0(sizeof(*user));
    bt_user_init(user, peer_id); //, ip, port);
    return user;
}

void bt_user_setup(bt_user_t* user, const char* ip, int port)
{
    user_set_port(USER(user), htons(port)); // htons
    // TODO: check also for ipv6 format
    resolve_address(ip, (call_func_t)bt_user_resolver, user_ref(USER(user)));
    // TODO : implement that peer_id parsing to update peer client (for information)
}

// Fixme : am_interested_torrent is bt_user_torrent
void bt_user_set_am_interested_torrent(bt_user_t* user, bt_torrent_t* torrent) 
{
    g_assert(user);
    g_assert(torrent);
    if ( user->am_interested_torrent == NULL)
    {
        user->am_interested_torrent =  bt_user_am_interested_torrent_new();

    }
    user->am_interested_torrent->torrent = torrent;

}

void bt_user_set_interested_torrent_me(bt_user_t* user, bt_torrent_t* torrent) 
{
    g_assert(user);
    g_assert(torrent);
    user->interested_torrent_me = torrent;
}

void bt_user_set_interested_torrent_bitfield(bt_user_t* user, bt_bitfield_t* user_torrent_bitfield)
{
    g_assert(user);
    g_assert(user->am_interested_torrent);
    user->am_interested_torrent->user_bitfield = user_torrent_bitfield;
}

// Fixme : am_interested_torrent is bt_user_torrent
bt_torrent_t*  bt_user_get_am_interested_torrent(bt_user_t* user) 
{
    g_assert(user);
    g_assert(user->am_interested_torrent);
    return  user->am_interested_torrent->torrent;
}

bt_torrent_t*  bt_user_get_interested_torrent_me(bt_user_t* user) 
{
    g_assert(user);
    return user->interested_torrent_me;
}

bt_bitfield_t* bt_user_get_am_interested_torrent_bitfield(bt_user_t* user) 
{
    g_assert(user);
    g_assert(user->am_interested_torrent);
    return user->am_interested_torrent->user_bitfield;
}


bt_user_t* bt_user_find(const char* peer_id)
{
    bt_user_t* user = g_hash_table_lookup(BtUserModule->users, peer_id);
    g_assert(BtUserModule);
    //FIXME: dont use binary peer id for name.
    if (!user)
    {
        user = bt_user_new(peer_id);
        g_hash_table_insert(BtUserModule->users, OBJ(user)->name, user);
    }
    return user;
}


static void bt_users_init()
{
    BtUserModule->users =
        g_hash_table_new_full((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal, NULL, NULL);
}
/*
static void bt_users_run()
{
}
*/
static void bt_users_exit()
{
    g_log("BT_USERS", G_LOG_LEVEL_DEBUG,
          "we have %u in cache consuming approx. %u bytes", 
          g_hash_table_size(BtUserModule->users),
          (int)(g_hash_table_size(BtUserModule->users)*sizeof(bt_user_t)));
    
    g_hash_table_destroy(BtUserModule->users);
    BtUserModule->users = NULL;
    BtUserModule = NULL;
}

static void bt_user_module_init(bt_user_module_t* module)
{
    module_init(MODULE(module), "Bittorrent User Module",
                (ModuleFunc)bt_users_init, NULL, (ModuleFunc)bt_users_exit);
    
    module->users = NULL;
    
    module_require(user_module_instance);
    module_require(resolve_module_instance);
}


module_t* bt_user_module_instance()
{
    if (!BtUserModule)
    {
        BtUserModule = g_malloc0(sizeof(*BtUserModule));
        bt_user_module_init(BtUserModule);
    }
    return MODULE(BtUserModule);
}

