/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.7 $
 * $Date: 2007/09/21 09:41:55 $
 * $Author: sid4u $
 */

#include <sys/stat.h>

#include <gtk/gtk.h>

#include "lcore/l_i18n.h"
#include "lcore/l_base.h"
#include "lcore/gtk2_ui/ui_network.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "bt.h"
#include "bt_torrent.h"
#include "bt_tracker.h"

typedef struct {
    GtkWidget* win;
    bt_torrent_t* torrent;
    GList* files;
} ui_bt_twin_t;

typedef struct {
    GtkWidget* vbox;
    GtkWidget* tbox;
} ui_bt_torrent_t;

typedef struct {
    GtkWidget* hbox;
} ui_bt_tracker_t;

static GtkBox* TorrentBox = NULL;
static GtkContainer* NewTorrentBox = NULL;


static void on_torrent_accept(ui_bt_twin_t* data)
{
    unsigned cnt = 0;
    GList* dlist;

    for (dlist = data->files; dlist; dlist = dlist->next)
    {
        GtkToggleButton* button = dlist->data;
        file_t* file = g_object_get_data(G_OBJECT(button), "file");
        if (file && gtk_toggle_button_get_active(button))
        {
            bt_torrent_accept_file(data->torrent, file);
            cnt++;
        }
    }

    if (cnt > 0)
    {
        object_make_public(OBJ(data->torrent), NULL);
    }
    else
    {
        object_kill(OBJ(data->torrent));
    }
    gtk_widget_destroy(data->win);
    g_list_free(data->files);
    g_free(data);
}

static void on_torrent_cancel(ui_bt_twin_t* data)
{
    g_message("on_torrent_cancel");
    object_kill(OBJ(data->torrent));
    gtk_widget_destroy(data->win);
    g_list_free(data->files);
    g_free(data);
}

static gboolean on_torrent_delete(ui_bt_twin_t* data)
{
    g_message("on_torrent_delete");
    on_torrent_cancel(data);
    return TRUE;
}

static void ui_bt_create_torrent_win(bt_torrent_t* torrent)
{
    GtkWidget* win;
    GtkWidget* vbox0;
    GtkWidget* vbox;
    GtkWidget* sw;
    GtkWidget* item;
    GtkWidget* hbox;
    GList* dlist;
    ui_bt_twin_t* data = g_malloc0(sizeof(*data));
    
    data->torrent = torrent;
    data->win = win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW(win), _("Add new torrent"));
    gtk_window_set_default_size(GTK_WINDOW(win), 500, 300);
/*     gtk_window_set_role(GTK_WINDOW(win), "Torrent"); */

    g_signal_connect_swapped(G_OBJECT(win), "delete_event",
                             G_CALLBACK(on_torrent_delete), data);

    vbox0 = gtk_vbox_new(FALSE, 10);
    gtk_container_set_border_width(GTK_CONTAINER(vbox0), 10);
    gtk_container_add(GTK_CONTAINER(win), vbox0);
    gtk_widget_show(vbox0);

/*     item = gtk_label_new(l_printf("<b>%s</b>", _("Add new torrent"))); */
/*     gtk_label_set_use_markup(GTK_LABEL(item), TRUE); */
/*     gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5); */
    item = ui_create_header_label(l_printf("<b>%s</b>", _("Add new torrent")));
    gtk_box_pack_start(GTK_BOX(vbox0), item, FALSE, FALSE, 0);
    gtk_widget_show(item);

    vbox = gtk_vbox_new(FALSE, 5);
    sw = ui_create_scrolled_area(vbox, TRUE);
    gtk_box_pack_start(GTK_BOX(vbox0), sw, TRUE, TRUE, 0);
    
    for (dlist = torrent->files; dlist; dlist = dlist->next)
    {
        file_t* file = dlist->data;
        
        hbox = gtk_hbox_new(FALSE, 10);
        gtk_widget_show(hbox);
        gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
        
        item = gtk_check_button_new_with_label(file->filename);
        gtk_widget_set_size_request(item, 100, -1);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(item), TRUE);
        gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 0);
        gtk_widget_show(item);

        g_object_set_data(G_OBJECT(item), "file", file);
        data->files = g_list_append(data->files, item);

        item = gtk_label_new(l_printf("<b>%s</b>", print_filesize(&file->size)));
        gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);
        gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
        gtk_widget_show(item);

    }

    hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
    gtk_box_pack_end(GTK_BOX(vbox0), hbox, FALSE, FALSE, 0);
    gtk_widget_show(hbox);

    item = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
    gtk_container_add(GTK_CONTAINER(hbox), item);
    g_signal_connect_swapped(G_OBJECT(item), "clicked",
                             G_CALLBACK(on_torrent_cancel), data);
    gtk_widget_show(item);

    item = gtk_button_new_from_stock(GTK_STOCK_ADD);
    gtk_container_add(GTK_CONTAINER(hbox), item);
    g_signal_connect_swapped(G_OBJECT(item), "clicked",
                             G_CALLBACK(on_torrent_accept), data);
    gtk_widget_show(item);


    gtk_widget_show(win);
}

static void ui_bt_add_torrent(const char* filename)
{
    bt_torrent_t* torrent = bt_torrent_new_from_file(filename);
    
    if (torrent) {
        // ui_bt_create_torrent_win(torrent);
        printf("**********TORRENT PARSED\n");
        g_free(torrent);
    }
    else {
        printf("***************TORRENT NOT PARSED\n");
    }
}

static void on_parse_torrent(GtkEntry* entry)
{
    const char* text = gtk_entry_get_text(entry);
    ui_bt_add_torrent(text);
}

static void ui_bt_create_net_page(void)
{
    GtkWidget* vbox0;
    GtkWidget* vbox;
    GtkWidget* hbox;
    GtkWidget* entry;
    GtkWidget* frame;
    GtkWidget* button;
    
    vbox0 = gtk_vbox_new(FALSE, 5);
    gtk_widget_show(vbox0);
    gtk_container_set_border_width(GTK_CONTAINER(vbox0), 5);
    
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox0), hbox, FALSE, FALSE, 0);
    
    entry = gtk_entry_new();
  /*   gtk_entry_set_text(GTK_ENTRY(entry), "D:\\PROJETS\\SOURCEFORGE\\collaboration\\TESTS\\test.zip.torrent"); */
  /*    gtk_entry_set_text(GTK_ENTRY(entry), "/home/lam/fc6-test1-binary-x86_64.torrent"); */

    gtk_widget_show(entry);
    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
    
    button = gtk_button_new_from_stock(GTK_STOCK_ADD);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    g_signal_connect_swapped(G_OBJECT(button), "clicked",
                             G_CALLBACK(on_parse_torrent), entry);
    
    frame = gtk_frame_new(NULL);
/*     gtk_widget_show(frame); */
    gtk_box_pack_start(GTK_BOX(vbox0), frame, FALSE, FALSE, 0);
    NewTorrentBox = GTK_CONTAINER(frame);
    
    vbox = gtk_vbox_new(FALSE, 10);
    gtk_widget_show(vbox);
    gtk_box_pack_start(GTK_BOX(vbox0), vbox, FALSE, FALSE, 0);
    TorrentBox = GTK_BOX(vbox);
    

    ui_add_network_subpage(BT_ID, vbox0);
}


void ui_bt_tracker_hide(bt_tracker_t* tracker)
{
    ui_bt_tracker_t* data = object_get_data(OBJ(tracker), "Tracker");

    if (data)
    {
        gtk_widget_destroy(data->hbox);
        object_free_data_common(OBJ(tracker), "Tracker");
    }
}

static void ui_bt_tracker_show(bt_tracker_t* tracker)
{
    if (!tracker->torrent || !object_get_data(OBJ(tracker->torrent), "Torrent"))
        return;

    if (!object_get_data(OBJ(tracker), "Tracker"))
    {
        ui_bt_torrent_t* tdata = object_get_data(OBJ(tracker->torrent), "Torrent");
        ui_bt_tracker_t* data;
        GtkWidget* box;
        GtkWidget* item;

        object_set_data(OBJ(tracker), "Tracker", data = g_malloc(sizeof(*data)));

        data->hbox = box = gtk_hbox_new(FALSE, 5);
        
/*         item = gtk_button_new_from_stock(GTK_STOCK_STOP); */
/*         gtk_widget_show(item); */
/*         gtk_box_pack_start(GTK_BOX(box), item, FALSE, FALSE, 0); */

/*         item = gtk_button_new_from_stock(GTK_STOCK_RESUME); */
/*         gtk_widget_show(item); */
/*         gtk_box_pack_start(GTK_BOX(box), item, FALSE, FALSE, 0); */

        item = gtk_label_new(OBJ(tracker)->name);
        gtk_widget_show(item);
        gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
        gtk_box_pack_start(GTK_BOX(box), item, FALSE, FALSE, 0);

        gtk_widget_show(box);
        gtk_box_pack_start(GTK_BOX(tdata->tbox), box, FALSE, FALSE, 0);

        object_notify_d(OBJ(tracker), (ObjectFunc)ui_bt_tracker_hide, NULL);
    }
}


static void ui_bt_torrent_hide(bt_torrent_t* torrent)
{
    ui_bt_torrent_t* data = object_get_data(OBJ(torrent), "Torrent");

    if (data)
    {
        gtk_widget_destroy(data->vbox);
        object_free_data_common(OBJ(torrent), "Torrent");
        g_list_foreach(torrent->trackers, (GFunc)object_free_data_common, "Tracker");
    }
}

static void ui_bt_torrent_show(bt_torrent_t* torrent)
{
    if (!object_get_data(OBJ(torrent), "Torrent"))
    {
        ui_bt_torrent_t* data;
        GtkWidget* box;
        GtkWidget* hbox;
        GtkWidget* vbox;
        GtkWidget* item;

        object_set_data(OBJ(torrent), "Torrent", data = g_malloc(sizeof(*data)));
        data->vbox = box = gtk_vbox_new(FALSE, 5);
        
        hbox = gtk_hbox_new(FALSE, 10);
        gtk_widget_show(hbox);
        gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
        
        item = gtk_hseparator_new();
        gtk_widget_show(item);
        gtk_box_pack_start(GTK_BOX(box), item, FALSE, FALSE, 0);

        vbox = gtk_vbox_new(FALSE, 0);
        gtk_widget_show(vbox);
        gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);

/*         if (torrent->status != BT_TORRENT_RUNNING) */
/*             item = gtk_button_new_from_stock(GTK_STOCK_MEDIA_PLAY); */
/*         else */
/*             item = gtk_button_new_from_stock(GTK_STOCK_MEDIA_STOP); */
/*         gtk_widget_show(item); */
/*         gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 0); */

        data->tbox = vbox = gtk_vbox_new(FALSE, 5);
        gtk_widget_show(vbox);
        gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
        
        gtk_widget_show(box);
        gtk_box_pack_start(TorrentBox, box, FALSE, FALSE, 0);

        g_list_foreach(torrent->trackers, (GFunc)ui_bt_tracker_show, NULL);
        
        object_notify_d(OBJ(torrent), (ObjectFunc)ui_bt_torrent_hide, NULL);
    }
}

void ui_bt_torrent_new(bt_torrent_t* torrent)
{
    ui_bt_torrent_show(torrent);
}

void ui_bt_net_init()
{
    ui_bt_create_net_page();
}

