/* Copyright (C) 2007   The Lopster Team
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. 
*/

/* $Revision: 1.5 $
 * $Date: 2007/11/19 11:12:55 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QPainter>
#include <QStandardItemModel>

#include "lcore/l_i18n.h"
#include "lcore/l_base.h"

#include "lcore/qt4_ui/ui_network.hpp"
#include "lcore/qt4_ui/ui_main.hpp"

#include "ui_bt_net.hpp"



class BtNetItem : public QStandardItem
{
public:
    BtNetItem(const QString &text)
        : QStandardItem(text) {}
    BtNetItem(const QIcon &icon, const QString &name)
        : QStandardItem(icon,name) {}
    ~BtNetItem() {
        object_free_data(OBJ(m_torrent), "TorrentEntry", NULL);    
    }
    void set_torrent(bt_torrent_t* torrent) {
        this->m_torrent = torrent;
        this->setData(QVariant::fromValue((void*)torrent));
        object_set_data(OBJ(torrent), "TorrentEntry", this);
    }
    bt_torrent_t* m_torrent;
};

class BtNetModel : public QStandardItemModel
{
public:
    BtNetModel(QWidget *parent=0);
    void bt_net_item_add(bt_torrent_t* torrent);
    void bt_net_item_update(bt_torrent_t* torrent);
    void bt_net_item_delete(bt_torrent_t* torrent);
};


BtNetModel::BtNetModel(QWidget *parent)
    : QStandardItemModel(0, 7 , parent) // FIXME : handle the color on running
{
    QStringList headers;
    headers << qtr("Name") << qtr("Size") << qtr("Downloaded") 
            << qtr("Done") << qtr("Seeds") << qtr("Peers") << qtr("Health");
    setHorizontalHeaderLabels(headers); 
}

void BtNetModel::bt_net_item_add(bt_torrent_t* torrent)
{
    QList<QStandardItem *> items;

    BtNetItem *name;
    QStandardItem *size;
    QStandardItem *downloaded;
    QStandardItem *done;
    QStandardItem *seeds;
    QStandardItem *peers;
    QStandardItem *health;

    if (!torrent) return;

    if (object_get_data(OBJ(torrent), "TorrentEntry")) return;    

    // if the torrent holds a unique file, use the unique file name , else use the main_directory value
    if (g_list_length(torrent->downloads) == 1)
    {
        download_t* download = (download_t*) torrent->downloads->data;
        file_t* file = FILE(download);
        name = new BtNetItem(/*TODO : add mime type icon */ qfu(file->filename));
        size = new QStandardItem(qfu(print_filesize(&file->size)));
    }
    else 
    {
        name = new BtNetItem(/*TODO : add folder icon */ qfu(torrent->main_directory));
        // TODO/FIXME : get the Sum of all filesize
        size = new QStandardItem(QString("0"));
    }

    // TODO : this is all tbd for the moment
    downloaded = new QStandardItem("");
    done = new QStandardItem("");
    seeds = new QStandardItem("");
    peers = new QStandardItem("");
    health = new QStandardItem("");

    name->set_torrent(torrent);

    items.append(name);
    items.append(size);
    items.append(downloaded);
    items.append(done);
    items.append(seeds);
    items.append(peers);
    items.append(health);
    appendRow(items);

    bt_net_item_update(torrent);
}

void BtNetModel::bt_net_item_update(bt_torrent_t* torrent)
{
    // TODO
}

void BtNetModel::bt_net_item_delete(bt_torrent_t* torrent)
{
    BtNetItem *name = static_cast< BtNetItem *>(object_get_data(OBJ(torrent), "TorrentEntry"));
    if (!name) return;

    QModelIndex model_index = indexFromItem(name);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    removeRow(row);   
}

UiBtNet::UiBtNet(QWidget *parent)
    : QWidget(parent)
{
    setupUi(this);
    torrent_model = new BtNetModel();
    tree_torrents->setModel(torrent_model);

    // TODO : manage tree advanced
    // Display information regarding Trackers, specifics torrent information etc

    connect(tool_add, SIGNAL(clicked()),this, SLOT(add_torrent()));

}

void UiBtNet::ui_bt_net_init()
{
    NetworkPage::ui_network_add_page(BT_ID, GetBtNet());
}

void UiBtNet::ui_bt_net_cleanup()
{
    UiBtNet* net = GetBtNet(); 
    NetworkPage::ui_network_remove_page(net);
    delete net;
}

UiBtNet *UiBtNet::GetBtNet()
{
   static UiBtNet* net = NULL;
   if (net == NULL)
   {
        net = new UiBtNet(); 
   }
   return net;
}

void UiBtNet::ui_bt_tracker_show(bt_tracker_t* tracker)
{

}

void UiBtNet::ui_bt_torrent_hide(bt_torrent_t* torrent)
{
    GetBtNet()->torrent_model->bt_net_item_delete(torrent);
}

void UiBtNet::ui_bt_torrent_update(bt_torrent_t* torrent)
{
    GetBtNet()->torrent_model->bt_net_item_update(torrent);
}

void UiBtNet::ui_bt_torrent_new(bt_torrent_t* torrent)
{
    GetBtNet()->torrent_model->bt_net_item_add(torrent);
    g_list_foreach(torrent->trackers, (GFunc)ui_bt_tracker_show, NULL);
    object_notify_d(OBJ(torrent), (ObjectFunc)ui_bt_torrent_hide, NULL);
}


void UiBtNet::ui_bt_create_torrent_win(bt_torrent_t* torrent)
{
    TorrentFileChooserDialog *torrent_dialog = new TorrentFileChooserDialog(torrent);
    torrent_dialog->show();
}

void UiBtNet::add_torrent()
{
    // Get the filename from QLineEdit 
    QString filename = combo_url->currentText();
    if (filename.isEmpty()) {
        qWarning("**********add_torrent - No Filename passed");
        return;
    }
    bt_torrent_t* torrent = bt_torrent_new_from_file(qtu(filename)); 
    if (torrent) {
        ui_bt_create_torrent_win(torrent);
        qWarning("**********TORRENT PARSED");
        g_free(torrent);
    }
    else {
        qWarning("**********TORRENT NOT PARSED");
    }
}

class FileTorrentItem : public QStandardItem
{
public:
    FileTorrentItem(const QString &text)
        : QStandardItem(text) {
        setCheckable(true); 
        setCheckState(Qt::Checked); // checked by default
    }
    FileTorrentItem(const QIcon &icon, const QString &name)
        : QStandardItem(icon,name) {
        setCheckable(true); 
        setCheckState(Qt::Checked); // checked by default
    }
    void set_file(file_t* file) {
        this->m_file = file;
        object_set_data(OBJ(file), "FileTorrentItem", this);
    }
    file_t* m_file;
};

class BtFileModel : public QStandardItemModel
{
public:
    BtFileModel(QWidget *parent=0);
    void bt_file_item_add(file_t* file);
};

BtFileModel::BtFileModel(QWidget *parent)
    : QStandardItemModel(0, 2 , parent) 
{
    QStringList headers;
    headers << qtr("Name") << qtr("Size");
    setHorizontalHeaderLabels(headers); 
}

void BtFileModel::bt_file_item_add(file_t* file)
{
    QList<QStandardItem *> items;
    
    FileTorrentItem *name = new FileTorrentItem(qfu(file->filename));
    QStandardItem *size = new QStandardItem(qfu(print_filesize(&file->size)));

    name->set_file(file);

    items.append(name);
    items.append(size);
    
    appendRow(items);
}

TorrentFileChooserDialog::TorrentFileChooserDialog(bt_torrent_t* torrent, QWidget *parent)
    : QDialog(parent), m_torrent(torrent)
{
    setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose);
    file_model = new BtFileModel(this);
    tree_files->setModel(file_model);

    // update name and total size labels

    // update the save to folder

    // update the space disk left label

    // List all the files
    GList *dlist;
    for (dlist = torrent->files; dlist; dlist = dlist->next)
    {
        file_t* file = (file_t*) dlist->data;        
        file_model->bt_file_item_add(file);
    }   

    // all checked by default
    check_all->setChecked(true);

    connect(push_ok, SIGNAL(clicked()), this, SLOT(validate_torrent()));
    connect(push_cancel, SIGNAL(clicked()), this, SLOT(cancel_torrent()));
    connect(check_all, SIGNAL(toggled(bool)), this, SLOT(check_all_toggled(bool)));
}

void TorrentFileChooserDialog::check_all_toggled(bool checked)
{
    GList *dlist;
    for (dlist = m_torrent->files; dlist; dlist = dlist->next)
    {
        file_t* file = (file_t*) dlist->data;        
        FileTorrentItem *item = static_cast< FileTorrentItem *>
                                        (object_get_data(OBJ(file), "FileTorrentItem"));
        if (!item) continue;
        if (checked)
        {
            item->setCheckState(Qt::Checked);
        }
        else 
        {
            item->setCheckState(Qt::Unchecked);
        }
    }
}

void TorrentFileChooserDialog::cancel_torrent()
{
    object_kill(OBJ(m_torrent));
    accept();
}

void TorrentFileChooserDialog::validate_torrent()
{
    unsigned cnt = 0;
    GList *dlist;
    for (dlist = m_torrent->files; dlist; dlist = dlist->next)
    {
        file_t* file = (file_t*) dlist->data;        
        FileTorrentItem *item = static_cast< FileTorrentItem *>
                                        (object_get_data(OBJ(file), "FileTorrentItem"));
        if (!item) continue;
        // Free data first
        object_free_data(OBJ(file), "FileTorrentItem", NULL); 
        if (item->checkState() == Qt::Checked)
        {
            bt_torrent_accept_file(m_torrent, file);         
            cnt++;
        }
    }   
    if (cnt > 0)
    {
        object_make_public(OBJ(m_torrent), NULL);
    }
    else
    {
        object_kill(OBJ(m_torrent));
    }
    accept();
}
