/* Copyright (C) 2005-2006 sgop@users.sourceforge.net.
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.7 $
 * $Date: 2007/06/02 18:41:25 $
 * $Author: sgop $
 */

#ifndef WIN32
#include <dlfcn.h>
#endif
#include <string.h>
#include <stdlib.h>

#include "lcore/plugin.h"
#include "lcore/version.h"
#include "lcore/utils.h"
#include "lcore/preferences.h"
#include "lcore/library.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "dc.h"
#include "dc_ui.h"
#include "dc_search.h"
#include "dc_commands.h"
#include "dc_users.h"
#include "dc_hotlist.h"
#include "dc_actions.h"
#include "dc_share.h"
#include "dc_search.h"
#include "dc_download.h"
#include "dc_browse.h"
#include "hash_tth.h"

#ifdef WIN32
#include "dcalias.h"
#endif


#ifndef WIN32
int DC_ID = -1;
#else
int dc_id = -1;
#endif

int dc_get_media(int media)
{
    switch (media) {
    case DC_MEDIA_ANY: return MEDIA_ANY;
    case DC_MEDIA_AUDIO: return MEDIA_AUDIO;
    case DC_MEDIA_COMPRESSED: return MEDIA_APPLICATION;
    case DC_MEDIA_TEXT: return MEDIA_DOCUMENT;
    case DC_MEDIA_APPLICATION: return MEDIA_APPLICATION;
    case DC_MEDIA_IMAGE: return MEDIA_IMAGE;
    case DC_MEDIA_VIDEO: return MEDIA_VIDEO;
    case DC_MEDIA_FOLDER: return MEDIA_FOLDER;
    case DC_MEDIA_TTH: return MEDIA_INVALID;
    default: return MEDIA_INVALID;
    }
}

int dc_set_media(int media)
{
    switch (media) {
    case MEDIA_ANY: return DC_MEDIA_ANY;
    case MEDIA_AUDIO: return DC_MEDIA_AUDIO;
        //  case MEDIA_COMPRESSED: return DC_MEDIA_COMPRESSED;
    case MEDIA_DOCUMENT: return DC_MEDIA_TEXT;
    case MEDIA_APPLICATION: return DC_MEDIA_APPLICATION;
    case MEDIA_IMAGE: return DC_MEDIA_IMAGE;
    case MEDIA_VIDEO: return DC_MEDIA_VIDEO;
    case MEDIA_FOLDER: return DC_MEDIA_FOLDER;
    default: return DC_MEDIA_ANY;
    }
}


/** Base32 encoding. Note: we do *no* padding, not even the 5-byte
 * blocks are completed. Thus, for decoding, we must assume that the
 * hash is 24 byte long. That's DirectConnect philosophy.  Actually
 * 24bytes+3bits are encoded, which results in a 39byte string.
 */
const char* print_hash_base32(hash_t* hash) {
  static char alpha[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567";
  unsigned bitlen;
  unsigned i1, index;
  char* result;
  char* pos;
  unsigned char c;

  bitlen = hash->len * 8;

  pos = result = g_malloc0(bitlen+4 / 5 +1);  // +1 for terminating \0
  for (i1 = 0, index = 0; i1 < hash->len; ) {
    // take 5 bits and encode them.
    if (index > 3) {
      // take first part of 5 bits
      c = (unsigned char)(hash->hash[i1] & (0xff >> index));
      index = (index + 5) % 8;
      c <<= index;
      i1++;
      // take second part, if there's another char
      if (i1 < hash->len)
        c |= hash->hash[i1] >> (8-index);
    } else {
      // take 5 bits at once
      c = (unsigned char)(hash->hash[i1] >> (8 - (index+5))) & 0x1f;
      index = (index + 5) % 8;  // shift bit offset
      if (index == 0) i1++;     // shift char offset if necessary.
    }
    
    g_assert(c < 32);
    *pos++ = alpha[c];
  }
  return auto_text(result);
}

static hash_t* store_hash_real(hash_t* old_value, unsigned char* hash,
                               unsigned len) {
  hash_t* result;
  if (old_value && old_value->len == len && 
      !memcmp(old_value->hash, hash, len)) return old_value;
  result = g_malloc(sizeof(*result));
  result->len = len;
  result->hash = g_malloc(len);
  memcpy(result->hash, hash, len);
  free_hash(old_value);
  return result;
}

/** Base32 decoding. We assume the encoded data to be 24 bytes, and we
 * currently do case-sensitive parsing (which base32 is *not*).
 */
hash_t* store_hash_base32(hash_t* old, const unsigned char* str)
{
    static char table[256] = {
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,26,27,28,29,30,31,-1,-1,-1,-1,-1,-1,-1,-1,
        -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,
        15,16,17,18,19,20,21,22,23,24,25,-1,-1,-1,-1,-1,
        -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,
        15,16,17,18,19,20,21,22,23,24,25,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
        -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,
    };
    
#define SIZE 24
    if (str)
    {
        char* temp = g_malloc0(SIZE);
        int i1, index, offset;
        for (i1 = 0, index = 0, offset = 0; str[i1]; i1++)
        {
            char c = table[str[i1]];
            if (c == -1) continue;
            
            if (index <= 3)
            {
                index = (index + 5) % 8;
                if (index == 0)
                {
                    temp[offset++] |= c;
                    if (offset == SIZE) break;
                }
                else
                {
                    temp[offset] |= c << (8 - index);
                }
            }
            else
            {
                index = (index + 5) % 8;
                temp[offset++] |= (c >> index);
                if (offset == SIZE) break;
                temp[offset] |= c << (8 - index);
            }
        }
        old = store_hash_real(old, temp, SIZE);
        g_free(temp);
    }
    else
    {
        old = store_hash_real(old, "", 0);
    }
    return old;
}


static void dc_init()
{
    attr_register("file", "urn:tree:tiger", ATTR_UNIMPORTANT,
                  (UIPrintFunc)print_hash_base32,
                  (XMLPrintFunc)print_hash_base32,
                  (CompareFunc)compare_hash,
                  (CopyFunc)copy_hash,
                  (FreeFunc)free_hash,
                  (StoreStrFunc)store_hash_base32,
                  (StoreFunc)store_hash);
    
    library_register_process_func("file_hash_tth", file_hash_tth);
    
    dc_actions_init();
}

static void dc_run()
{
}

static void dc_exit()
{
}

static void dc_plugin_start(plugin_t* plugin)
{
    plugin_init(plugin, "DirectConnect Module",
                (ModuleFunc)dc_init, (ModuleFunc)dc_run, (ModuleFunc)dc_exit);
    
#ifndef WIN32
  DC_ID = plugin->ID;
#else
  dc_id = plugin->ID;
#endif
    plugin->connect = dc_connect_change;
    
    module_require(pref_module_instance);
    module_require(base_module_instance);
    module_require(dc_ui_module_instance);
    module_require(dc_command_module_instance);
    module_require(dc_user_module_instance);
    module_require(dc_hot_module_instance);
    module_require(dc_hub_module_instance);
    module_require(dc_share_module_instance);
    module_require(dc_search_module_instance);
    module_require(dc_download_module_instance);
    module_require(dc_browse_module_instance);
}

void plugin_initialize(plugin_info_t* info)
{
    info->name       = g_strdup("DirectConnect");
    info->sname      = g_strdup("DC");
    info->copyright  = g_strdup("Copyright \302\251 2003-2006 Markus Lausser");
    info->if_version = IF_VERSION;
    info->pl_version = 0x00060300;  // 0.6.3-0
    info->icon       = g_strdup("dc_icon.png");
    info->license    = g_strdup("GPL (GNU General Public License)");
    info->website    = g_strdup("http://lopster.sourceforge.net");
    info->comment    = g_strdup(N_("Plugin which connects to the DirectConnect\342\204\242 from NeoModus\302\256 (http://www.neo-modus.com/)."));
    info->authors    = g_list_append
        (info->authors, g_strdup("Markus Lausser <sgop@users.sourceforge.net>"));
    info->start      = dc_plugin_start;
#ifdef PACKAGE_SOURCE_DIR
    lopster_pixmap_dir_add(SDIR(PACKAGE_SOURCE_DIR) SDIR("plugin") 
                           SDIR("directconnect") "pixmaps");
#endif
}

#ifdef WIN32
int dc_plugin_get_id( )
{
    return dc_id;
}
#endif


#define isEChar(c) (c == 0 || c==5 || c==124 || c==96 || c==126 || c==36)

static char* key_subst(const char* key, int len, int n) {
  char* temp = g_malloc0(len + n*10 + 1);
  int i, j=0;
  
  for (i = 0; i < len; i++) {
    if (isEChar(key[i])) {
      j += sprintf(temp+j, "/%%DCN%03d%%/", key[i]);
    } else {
      temp[j++] = key[i];
    }
  }
  temp[j] = 0;
  return temp;
}


char* dc_generate_key(unsigned char* lock) {
  unsigned char* temp;
  unsigned char* tmp;
  unsigned char v1;
  int extra=0;
  int i;
  int len;

  if (!lock || (len = strlen(lock)) < 3) return g_strdup("");
  
  temp = g_malloc0(sizeof(char)*len);
  
  for (i = 0; i < len; i++) {
    if (i == 0) v1 = (lock[i]^5);
    else v1 = (lock[i]^lock[i-1]);

    v1 = (((v1 >> 4) | (v1 << 4))&0xff);
    temp[i] = v1;
    if (i > 0 && isEChar(temp[i])) extra++;
  }
  temp[0] ^= temp[len-1];
        
  if (isEChar(temp[0])) extra++;
        
  tmp = key_subst(temp, len, extra);
  g_free(temp);
  return tmp;
}

/*
char* dc_generate_key(unsigned char *lock) {
  int count = 0, len = 0, offset = 0;
  char *key = 0, *tkey= 0;
  
  len = strlen(lock);
  
  key = g_malloc0(sizeof(char)*(len+1));

  while (lock[++count])
    key[count] = lock[count] ^ lock[count - 1];

  key[0] = lock[0] ^ lock[len - 1] ^ lock[len - 2] ^ 5;
  
  count = 0;
  while (key[count++])
    key[count - 1] = ((key[count - 1]  << 4)) | ((key[count - 1] >> 4));

  count = 0;

  tkey = g_malloc0(sizeof(char)*(len + 1));
  while(key[count++]) {
    switch(key[count - 1]) {
    case 0:
    case 5:
    case 36:
    case 96:
    case 124:
    case 126:
      tkey = g_realloc(tkey, sizeof(char)*(len + offset + 11));
      sprintf(&tkey[count - 1 + offset], "/%%DCN%.3d%%/", key[count - 1]);
      offset += 9;
      break;
    default:
      tkey[count - 1 + offset] = key[count - 1];
      break;
    }
    
  }
  tkey[len + offset] = 0;
  free(key);

  return tkey;
}
*/

#ifdef WIN32
#define _DC_C_
#include "dcaliasdef.c"
#endif

