/* Copyright (C) 2005-2006 sgop@users.sourceforge.net.
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#include "lcore/action_item.h"
#include "lcore/browse.h"
#include "lcore/l_i18n.h"

#include "dc_network.h"
#include "dc_chat.h"
#include "dc_browse.h"
#include "dc_hotlist.h"
#include "dc_upload.h"
#include "dc_download.h"
#include "dc.h"


static void on_hub_bookmark(GList* hubs) {
    GList* dlist;
    for (dlist = hubs; dlist; dlist = dlist->next) {
        dc_hub_t* hub = dlist->data;
        dc_hub_set_bookmark(hub, !(hub->flags & DC_HUB_BOOKMARK));
    }
}

static void on_hub_kill(GList* hubs) {
    GList* dlist;
    for (dlist = hubs; dlist; dlist = dlist->next) {
        dc_hub_t* hub = dlist->data;
        object_kill(OBJ(hub));
    }
}

static void on_hub_edit(GList* hubs) {
    GList* dlist;
    for (dlist = hubs; dlist; dlist = dlist->next) {
        dc_hub_t* hub = dlist->data;
        dc_hub_edit(hub);
    }
}

static void dc_open_chat(dc_hub_t* hub) {
    if (hub->status == DC_HUB_ONLINE && !dc_chat_active(hub))
        dc_cp_new_public(hub);
}

static void dc_close_chat(dc_hub_t* hub) {
    dc_cp_t* page = dc_chat_active(hub);
    if (page) chat_page_close(PAGE(page));
}

#define HUB_LIST_ACTION(func) static void                 \
    on_ ## func (GList* hubs) {                           \
        GList* dlist;																			\
        for (dlist = hubs; dlist; dlist = dlist->next) {	\
            dc_hub_t* hub = dlist->data;                  \
            func(hub);                                    \
        }																									\
    }

HUB_LIST_ACTION(dc_hub_connect);
HUB_LIST_ACTION(dc_hub_disconnect);
HUB_LIST_ACTION(dc_open_chat);
HUB_LIST_ACTION(dc_close_chat);



static gboolean on_hub_check_book(dc_hub_t* hub) {
    return (hub->flags & DC_HUB_BOOKMARK);
}

static void on_dc_chat(GList* users) {
    GList* dlist;
    for (dlist = users; dlist; dlist = dlist->next) {
        dc_user_t* du = dlist->data;
        if (du && du->hubs)
            dc_cp_new_private(DC_HUB(du->hubs->data), OBJ(du)->name);
    }
}

static gboolean on_dc_check_not_buddy(dc_user_t* du) {
    return (du && !dc_hot_in_hotlist(du));
}

static void on_dc_buddy(GList* users) {
    GList* dlist;
    for (dlist = users; dlist; dlist = dlist->next) {
        dc_user_t* du = dlist->data;
        if (du && !dc_hot_in_hotlist(du))
            dc_hot_new(du, NULL);
    }
}

static void on_dc_browse(GList* users) {
    GList* dlist;
    for (dlist = users; dlist; dlist = dlist->next) {
        dc_user_t* du = dlist->data;
        if (du && du->hubs) dc_browse_user(du, NULL);
    }
}



static action_item_t* dc_actions_user_menu(const char* name) {
    action_item_t* item;

    item = action_item_root(name);
    action_item_simple(N_("Open chat..."), item, 0,
                       (ActionItemFunc)on_dc_chat);
    action_item_simple(N_("Send message..."), item, AI_RESTRICT_1,
                       NULL);
    action_item_sep(item);
    /*
      action_item_simple("Whois user", item, 0,
      (ActionItemFunc)on_dc_whois);
    */
    action_item_new(N_("Add to buddies"), item, 0,
                    (ActionItemFunc)on_dc_buddy,
                    (ActionItemCheck)on_dc_check_not_buddy);
    action_item_simple(N_("Browse user"), item, 0,
                       (ActionItemFunc)on_dc_browse);
    return item;
}


static gboolean on_check_connect(dc_hub_t* hub) {
    return (hub->status == DC_HUB_OFFLINE);
}
static gboolean on_check_disconnect(dc_hub_t* hub) {
    return (hub->status != DC_HUB_OFFLINE);
}

static gboolean on_check_open(dc_hub_t* hub) {
    return (hub->status == DC_HUB_ONLINE && !dc_chat_active(hub));
}

static gboolean on_check_close(dc_hub_t* hub) {
    return (dc_chat_active(hub) != NULL);
}



static dc_user_t* dc_upload_to_du(dc_upload_t* upload) {
    return DC_USER(UPLOAD(upload)->user);
}

static dc_user_t* dc_source_to_du(dc_source_t* source) {
    return DC_USER(SOURCE(source)->user);
}

static dc_user_t* dc_result_to_du(search_result_t* result) {
    return DC_USER(result->user);
}

static dc_user_t* dc_browse_to_du(file_t* file, browse_t* browse) {
    (void)file;
    return DC_USER(browse->user);
}


void dc_actions_init() {
    action_item_t* item;

    //////////
    item = action_item_root("DirectConnect hub actions");
    action_item_new(N_("Connect"), item, 0,
                    (ActionItemFunc)on_dc_hub_connect,
                    (ActionItemCheck)on_check_connect);
    action_item_new(N_("Disconnect"), item, 0,
                    (ActionItemFunc)on_dc_hub_disconnect,
                    (ActionItemCheck)on_check_disconnect);
    action_item_sep(item);
    action_item_new(N_("Open public chat"), item, AI_HIDE_0,
                    (ActionItemFunc)on_dc_open_chat,
                    (ActionItemCheck)on_check_open);
    action_item_new(N_("Close public chat"), item, AI_HIDE_0,
                    (ActionItemFunc)on_dc_close_chat,
                    (ActionItemCheck)on_check_close);
/*     action_item_sep(item); */
/*     action_item_new(N_("Bookmarked"), item, AI_CHECK, */
/*                     (ActionItemFunc)on_hub_bookmark, */
/*                     (ActionItemCheck)on_hub_check_book); */
    action_item_sep(item);
    //FIXME: these should be part of UI only?
    action_item_simple(N_("Edit hub"), item, /*AI_HIDE_0|*/0,
                       (ActionItemFunc)on_hub_edit);
    action_item_sep(item);
/*     action_item_simple(N_("Remove hub"), item, 0, */
/*                        (ActionItemFunc)on_hub_kill); */
    action_item_new(N_("Remove hub"), item, /*AI_HIDE_0|AI_CHECK*/0,
                    (ActionItemFunc)on_hub_bookmark,
                    (ActionItemCheck)on_hub_check_book);


    item = dc_actions_user_menu("Online dc users actions");
    //  item->convert = (ActionObjectConvert)dc_cu_to_du;
    //  action_item_sep(item);
    //  action_item_simple("Refresh list", item, AI_GLOBAL, NULL);

    item = dc_actions_user_menu("Upload dc users actions");
    item->convert = (ActionObjectConvert)dc_upload_to_du;
    user_action_upload_register(DC_ID, item);

    item = dc_actions_user_menu("Source dc users actions");
    item->convert = (ActionObjectConvert)dc_source_to_du;
    user_action_source_register(DC_ID, item);

    item = dc_actions_user_menu("Result dc users actions");
    item->convert = (ActionObjectConvert)dc_result_to_du;
    user_action_result_register(DC_ID, item);

    item = dc_actions_user_menu("Browse dc users actions");
    item->convert = (ActionObjectConvert)dc_browse_to_du;
    user_action_browse_register(DC_ID, item);

}
