/* Copyright (C) 2005-2006 sgop@users.sourceforge.net.
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.1 $
 * $Date: 2006/02/04 16:41:04 $
 * $Author: sgop $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdlib.h>

#include <glib.h>

#include "lcore/utils.h"
#include "lcore/chat.h"
#include "lcore/commands.h"
#include "lcore/l_i18n.h"
#include "dc_chat.h"
#include "dc.h"
#include "dc_commands.h"
#include "dc_network.h"
#include "dc_users.h"
#include "dc_search.h"

typedef void (*DcCommandFunc)(chat_page_t*, dc_hub_t*, char*);
#define DC_COMM(f) static void f (chat_page_t* page, dc_hub_t* hub, char *data)

#define DC_COMMAND(c) ((dc_command_t*)(c))
typedef struct {
    command_t _command;
    DcCommandFunc handle;
} dc_command_t;

typedef struct {
    module_t _module;
} dc_command_module_t;

static dc_command_module_t* DcCommandModule = NULL;

#define CHECK_HUB COMMAND_ERROR(!hub, page, "No network specified")

static void dc_command_handle(chat_page_t* page, command_t* comm, char *data) {
    dc_cp_t* dcp;
    dc_hub_t* hub;
    dc_command_t* command = DC_COMMAND(comm);
  
    if (page && page->ID == DC_ID) dcp = DCP(page);
    else dcp = NULL;

    g_log("DC_COM", G_LOG_LEVEL_DEBUG,
          "handle: [%s] [%s]", command?COMMAND(command)->name:"(null)",
          data?data:"(null)");
    if (command) {
        hub = NULL;
        if (data && *data == ':' && data[1] != 0
            && data[1] != ' ') {
            data = arg(data+1, 0);
            hub = dc_get_hub(atoi(data));
            if (!hub) {
                command_error(page, _("No hub with such index"));
                return;
            }
            data = arg(NULL, 1);
        } else if (dcp) {
            hub = dcp->hub;
        }
        command->handle(page, hub, data);
    } else if (dcp) {
        // normal line, pass to chat page
        if (dcp->type == DC_PAGE_PRIVATE) {
            dc_send_private_message(dcp->hub, page, OBJ(page)->name, data);
        } else if (dcp->type == DC_PAGE_PUBLIC) {
            dc_send_public_message(dcp, data);
        }
    }
    return;
}
  
static void
dc_command_new(const char* name, const char* syntax,
               const char* des, DcCommandFunc func) {
    dc_command_t *comm;
  
    g_assert(name);

    comm = g_malloc0(sizeof(*comm));
    command_init(COMMAND(comm), DC_ID, name, syntax, des);
    comm->handle = func;
}

DC_COMM(dc_com_msg) {
    char* nick = arg(data, 0);
    char* message = arg(NULL, 1);

    CHECK_HUB;

    COMMAND_ERROR(!nick, page, _("Missing nickname!"));
    COMMAND_ERROR(!message, page, _("Missing message!"));
  
    dc_send_private_message(hub, page, nick, message);
}

DC_COMM(dc_com_me) {
    (void)hub;

    COMMAND_ERROR(!page || page->ID != DC_ID || !page->active ||
                  DCP(page)->type != DC_PAGE_PUBLIC, page,
                  _("This is not a valid directconnect channel"));
    if (!data) return;
    dc_send_public_message(DCP(page), l_printf("/me %s", data));
}


static void dc_commands_init() {
    command_handler_install(DC_ID, (CommandHandler)dc_command_handle);
}

static void dc_commands_run() {
    dc_command_new("msg", "<user> <text>",
                   "Send a private message <text> to <user>",
                   dc_com_msg);
    dc_command_new("me", "<message>",
                   "Send action description to current channel",
                   dc_com_me);
}

static void dc_commands_exit() {
    //FIXME: cleanup
}

static void dc_command_module_init(dc_command_module_t* module) {
    module_init(MODULE(module), "Dc Command Module",
                (ModuleFunc)dc_commands_init,
                (ModuleFunc)dc_commands_run,
                (ModuleFunc)dc_commands_exit);
  
    module_require(command_module_instance);
    module_require(chat_module_instance);
}


module_t* dc_command_module_instance() {
    if (!DcCommandModule) {
        DcCommandModule = g_malloc0(sizeof(*DcCommandModule));
        dc_command_module_init(DcCommandModule);
    }
    return MODULE(DcCommandModule);
}


#undef DC_COMMAND
#undef DC_COMM
#undef CHECK_HUB
