/* Copyright (C) 2005-2006 sgop@users.sourceforge.net.
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.1 $
 * $Date: 2006/02/04 16:41:04 $
 * $Author: sgop $
 */

#include <string.h>

#include "lcore/buddy.h"
#include "lcore/utils.h"
#include "lcore/ui.h"
#include "dc.h"
#include "dc_hotlist.h"
#include "dc_proto.h"
#include "dc_users.h"

typedef struct {
    module_t _module;
} dc_hot_module_t;

static dc_hot_module_t* DcHotModule = NULL;

static gboolean collect_users(dc_user_t* user, GSList** list) {
    *list = g_slist_append(*list, user);
    return FALSE;
}

static void dc_hot_change_user(buddy_t* buddy) {
    attribute_t* attr = attr_find(ATTRC(buddy), "buddy", "DcNick");
    const char* nicks = attr?attr_get_ptr(attr, char):NULL;
    GSList* old_users = NULL;
    GSList* new_users = NULL;
    GSList* dlist;

    g_assert(DcHotModule);
    buddy_foreach_user(buddy, DC_ID, (UserForeach)collect_users, &old_users);

    make_list_from_string(&new_users, nicks, " ");
    for (dlist = new_users; dlist; dlist = dlist->next) {
        const char* nick = dlist->data;
        dc_user_t* user = dc_user_find(nick);
        if (!g_slist_find(old_users, user)) {
            buddy_add_user(buddy, USER(user));
        } else {
            old_users = g_slist_remove(old_users, user);
        }
    }
    for (dlist = old_users; dlist; dlist = dlist->next) {
        user_t* user = dlist->data;
        buddy_remove_user(buddy, USER(user));
    }
    g_slist_free(old_users);
    g_slist_free(new_users);
}

void dc_hot_new(dc_user_t* user, const char* des) {
    buddy_t* buddy;
  
    g_assert(DcHotModule);
    buddy = buddy_find_from_user(USER(user));
    if (buddy) return;

    buddy = buddy_new(OBJ(user)->name);
    if (des && *des) buddy_set_description(buddy, des);
    attr_store(ATTRC(buddy), "buddy", "DcNick", OBJ(user)->name);
    dc_hot_change_user(buddy);
}

gboolean dc_hot_in_hotlist(dc_user_t* user) {
    g_assert(DcHotModule);
    return (buddy_find_from_user(USER(user)) != NULL);
}
  
static void dc_hots_init() {
    attr_register_str("buddy", "DcNick", ATTR_SKIP, NULL);
    buddy_register_update(dc_hot_change_user);
}

static void dc_hot_module_init(dc_hot_module_t* module) {
    module_init(MODULE(module), "Dc Hotlist Module",
                (ModuleFunc)dc_hots_init, NULL, NULL);

    module_require(buddy_module_instance);
    //  module_require(dc_net_module_instance);
    module_require(dc_user_module_instance);
}


module_t* dc_hot_module_instance() {
    if (!DcHotModule) {
        DcHotModule = g_malloc0(sizeof(*DcHotModule));
        dc_hot_module_init(DcHotModule);
    }
    return MODULE(DcHotModule);
}
