/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.11 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#ifndef WIN32
#include <netinet/in.h>
#endif
#include <string.h>

#include <libxml/parser.h>

#include <glib.h>

#include "lcore/preferences.h"
#include "lcore/buffer.h"
#include "lcore/l_http.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "lcore/udp_socket.h"
#include "lcore/utils.h"
#include "lcore/l_resolve.h"
#include "lcore/l_listen.h"
#include "lcore/library.h"
#include "lcore/upload.h"
#include "lcore/version.h"
#include "lcore/xml_handler.h"
#include "dc_chat.h"
#include "dc_network.h"
#include "dc_ui.h"
#include "dc_proto.h"
#include "dc.h"
#include "dc_peer.h"
#include "dc_hotlist.h"
#include "dc_search.h"

#ifdef WIN32
#include "dcalias.h"
#endif


typedef struct {
  module_t _module;

  GList* hubs;
  GList* hubs_online;
  GList* pending;
  int auto_cnt;
  int auto_timer;
  GList* weblists;
  gboolean save;

  udp_listen_t* udp_listen;
  listen_t* tcp_listen;
} dc_hub_module_t;

static dc_hub_module_t* DcHubModule = NULL;
 
char* dc_hub_to_utf8(dc_hub_t* hub, const char* data)
{
    char* res = to_utf8(data, -1, NULL, NULL, 3,
                        "utf-8", hub?hub->charset:NULL, "WINDOWS-1252");
    //FIXME:
    if (data && !res) res = g_strdup(data);
    return res;
}

static void dc_hubs_save()
{
    if (module_running(MODULE(DcHubModule)))
        lopster_xml_save("dc:hublist", 1000);
}

const char* DcSpeedFromType(int type)
{
    //FIXME: can we define out own strings??
    switch (type) {
    case LINE_MODEM:     return "56Kbps";
    case LINE_ISDN:      return "ISDN";
    case LINE_DSL:       return "DSL";
    case LINE_CABLE:
    case LINE_POWER:     return "Cable";
    case LINE_SATELLITE: return "Satellite";
    case LINE_T1:        return "LAN(T1)";
    case LINE_T3:        return "LAN(T3)";
    default:             return "DSL";
    }
}

void dc_hub_edit(dc_hub_t* hub)
{
    DCUI.hub_edit(hub);
}

static dc_hub_t* dc_hub_find(const char* addy, int port)
{
    GList* dlist;

    for (dlist = DcHubModule->hubs; dlist; dlist = dlist->next)
    {
        dc_hub_t* hub = dlist->data;
        if (TSOCK(hub)->port == htons(port) &&
            !strcmp(hub->address, addy)) return hub;
    }
    return NULL;
}

static void xml_save_hub(dc_hub_t* hub, xmlNodePtr node)
{
    xmlNodePtr xsub;

    xsub = xmlNewChild(node, NULL, "Hub", NULL);
    xmlNewProp(xsub, "Name", hub->name);
    xmlNewProp(xsub, "Address", hub->address);
    xmlNewProp(xsub, "Port", l_printf("%d", ntohs(TSOCK(hub)->port)));
    xmlNewProp(xsub, "Description", hub->description);
    xmlNewProp(xsub, "Users", l_printf("%d", hub->user_cnt));
    if (hub->h_nick)
        xmlNewProp(xsub, "Login", hub->h_nick);
    if (hub->h_pass)
        xmlNewProp(xsub, "Password", hub->h_pass);
    if (hub->country)
        xmlNewProp(xsub, "Country", hub->country);
    if (hub->charset)
        xmlNewProp(xsub, "Charset", hub->charset);
    if (hub->flags & DC_HUB_BOOKMARK)
        xmlNewProp(xsub, "Bookmark", "yes");
}



static void xml_save_weblist(url_t* serv, xmlNodePtr xroot)
{
    xmlNodePtr xsub = xmlNewChild(xroot, NULL, "Weblist", NULL);
    xmlNewProp(xsub, "Host", serv->host);
    xmlNewProp(xsub, "Doc", serv->doc);
}

static void xml_save_hubs(xmlDocPtr xfile, xmlNodePtr xroot)
{
    (void)xfile;
    g_list_foreach(DcHubModule->weblists, (GFunc)xml_save_weblist, xroot);
    g_list_foreach(DcHubModule->hubs, (GFunc)xml_save_hub, xroot);
}

/*
<Hub Name="« « Türkiye Bad BoyZ » »     -  We ride together, We die togethe"
     Address="Turkiye.BBoyZ.net:8080"
     Description=" [Bad BoyZ - BBoyZ.net][Türkiyes #1 Hub!!!][Türkiyenin #1 Hubý!!!]"
     Users="4142"
     Country="Turkey"
     Status="Online"
     Minshare="1073741824"
     Maxhubs="30"
     Maxusers="5075"
     Reliability="81"
     Rating="*****"
     Port="8080"/>
*/
static url_t* dc_weblist_add(const char* url);
static void xml_get_hubs(xmlDocPtr xdoc, xmlNodePtr xnode)
{
    xmlNodePtr xsub = xnode->xmlChildrenNode;

    (void)xdoc;
    while (xsub != NULL)
    {
        if (!strcmp(xsub->name, "Hub"))
        {
            xmlChar* name = xmlGetProp(xsub, "Name");
            xmlChar* addy = xmlGetProp(xsub, "Address");
            xmlChar* des = xmlGetProp(xsub, "Description");
            xmlChar* users = xmlGetProp(xsub, "Users");
            xmlChar* country = xmlGetProp(xsub, "Country");
            xmlChar* charset = xmlGetProp(xsub, "Charset");
            xmlChar* port = xmlGetProp(xsub, "Port");
            char* rport = addy?strchr(addy, ':'):NULL;

            if (rport) *rport++ = 0; // cut the port from the addy
            if (port) rport = port;   // use attribute if available
            if (!rport) rport = "411"; // we can still use default DC port anyway

            if (name && addy && users && rport)  // rport never is NULL, but better leave check here.
            {
                dc_hub_t* hub;
                gboolean new_hub = FALSE;
        
                dc_unescape(name);

                hub = dc_hub_find(addy, atoi(rport));
                if (!hub)
                {
                    hub = dc_hub_new(name, addy, atoi(rport));
                    new_hub = TRUE;
                }
                else
                {
                    g_free(hub->address);
                    hub->address = g_strdup(addy);
                    TSOCK(hub)->port = htons((port_t)atoi(rport));
                }
                g_free(hub->description);
                hub->description = dc_unescape(g_strdup(des));
                g_free(hub->country);
                hub->country = g_strdup(country);
                hub->charset = g_strdup(charset);
                hub->user_cnt = atoi(users);
                
                if (new_hub)
                    object_make_public(OBJ(hub), NULL);
                else
                    dc_hub_set_name(hub, name);
            }
            if (name) xmlFree(name);
            if (addy) xmlFree(addy);
            if (des) xmlFree(des);
            if (users) xmlFree(users);
            if (country) xmlFree(country);
            if (charset) xmlFree(charset);
            if (port) xmlFree(port);
        }
        xsub = xsub->next;
    }
}

static void xml_get_hubs_local(xmlDocPtr xdoc, xmlNodePtr xnode)
{
    xmlNodePtr xsub = xnode->xmlChildrenNode;

    (void)xdoc;
    while (xsub != NULL)
    {
        if (!strcmp(xsub->name, "Hub"))
        {
            xmlChar* name = xmlGetProp(xsub, "Name");
            xmlChar* addy = xmlGetProp(xsub, "Address");
            xmlChar* port = xmlGetProp(xsub, "Port");
            xmlChar* des = xmlGetProp(xsub, "Description");
            xmlChar* users = xmlGetProp(xsub, "Users");
            xmlChar* country = xmlGetProp(xsub, "Country");
            xmlChar* charset = xmlGetProp(xsub, "Charset");
            xmlChar* book = xmlGetProp(xsub, "Bookmark");
            xmlChar* login = xmlGetProp(xsub, "Login");
            xmlChar* pass = xmlGetProp(xsub, "Password");

            if (name && addy && port)
            {
                dc_hub_t* hub;
                hub = dc_hub_new(name, addy, atoi(port));
                hub->description = g_strdup(des);
                hub->country = g_strdup(country);
                if (charset) hub->charset = g_strdup(charset);
               hub->user_cnt = atoi(users);
                if (book && !strcmp(book, "yes")) hub->flags |= DC_HUB_BOOKMARK;
                if (login) hub->h_nick = g_strdup(login);
                if (pass) hub->h_pass = g_strdup(pass);
                object_make_public(OBJ(hub), NULL);
            }
            if (name) xmlFree(name);
            if (addy) xmlFree(addy);
            if (port) xmlFree(port);
            if (des) xmlFree(des);
            if (users) xmlFree(users);
            if (country) xmlFree(country);
            if (charset) xmlFree(charset);
            if (book) xmlFree(book);
            if (login) xmlFree(login);
        }
        else if (!strcmp(xsub->name, "Weblist"))
        {
            xmlChar* attr1;
            attr1 = xmlGetProp(xsub, "Url");
            if (attr1)
            {
                dc_weblist_add(attr1);
                xmlFree(attr1);
            }
        }
        xsub = xsub->next;
    }
}

static void dc_hubs_load()
{
    DcHubModule->save = FALSE;

    lopster_xml_load("dc:hublist");

    if (!DcHubModule->weblists)
    {
        //FIXME: the following does not work anymore.
        dc_weblist_add("http://www.hublist.org/PublicHubList.xml.bz2");
        //FIXME: add some more.
    }

    DcHubModule->save = TRUE;
}


static void xml_get_hublist(xmlDocPtr xdoc)
{
    xmlNodePtr xroot;
    xmlNodePtr xsub;

    xroot = xmlDocGetRootElement(xdoc);
    if (xroot == NULL) {
        g_log("DC_HUB", G_LOG_LEVEL_WARNING, 
              "no root element in hub list");
        xmlFreeDoc(xdoc);
        return;
    }
    if (strcmp(xroot->name, "Hublist")) {
        g_log("DC_HUB", G_LOG_LEVEL_DEBUG, "Root xml element != HubList");
        xmlFreeDoc(xdoc);
        return;
    }
    xsub = xroot->xmlChildrenNode;
    while (xsub != NULL) {
        if (!strcmp(xsub->name, "Hubs")) {
            xml_get_hubs(xdoc, xsub);
        }
        xsub = xsub->next;
    }
}


// correct invalid chars of corrupt window-1252 buffers.
static void _dc_web_buffer_correct(buffer_t* buffer, unsigned len)
{
    unsigned char* pos = (unsigned char*)buffer_read_pos(buffer);
    unsigned i1;
    unsigned cnt[6] = {0,0,0,0,0,0};

    for (i1 = 0; i1 < len-12; i1++)
    {
        if (pos[i1] == '\n') return;
        if (!strncasecmp(pos+i1, "windows-1252", 12)) break;
    }
    for (i1 = 0; i1 < len; i1++)
    {
        if (pos[i1] == 0x81)
        {
            pos[i1] = ' ';
            cnt[0]++;
        }
        else if (pos[i1] == 0x8d)
        {
            pos[i1] = ' ';
            cnt[1]++;
        }
        else if (pos[i1] == 0x8f)
        {
            pos[i1] = ' ';
            cnt[2]++;
        }
        else if (pos[i1] == 0x90)
        {
            pos[i1] = ' ';
            cnt[3]++;
        }
        else if (pos[i1] == 0x9d)
        {
            pos[i1] = ' ';
            cnt[4]++;
        }
        else if (pos[i1] < 0x20 && pos[i1] != 0x09 &&
                 pos[i1] != 0x0a && pos[i1] != 0x0d)
        {
            pos[i1] = ' ';
            cnt[5]++;
        }
    }
    if (cnt[0] > 0)
    {
        g_log("DC_HUB", G_LOG_LEVEL_DEBUG,
              "correct %u characters (0x81) in corrupt xml hub list", cnt[0]);
    }
    if (cnt[1] > 0)
    {
        g_log("DC_HUB", G_LOG_LEVEL_DEBUG,
              "correct %u characters (0x8d) in corrupt xml hub list", cnt[1]);
    }
    if (cnt[2] > 0)
    {
        g_log("DC_HUB", G_LOG_LEVEL_DEBUG,
              "correct %u characters (0x8f) in corrupt xml hub list", cnt[2]);
    }
    if (cnt[3] > 0)
    {
        g_log("DC_HUB", G_LOG_LEVEL_DEBUG,
              "correct %u characters (0x90) in corrupt xml hub list", cnt[3]);
    }
    if (cnt[4] > 0)
    {
        g_log("DC_HUB", G_LOG_LEVEL_DEBUG,
              "correct %u characters (0x9d) in corrupt xml hub list", cnt[4]);
    }
    if (cnt[5] > 0)
    {
        g_log("DC_HUB", G_LOG_LEVEL_DEBUG,
              "correct %u characters (< 0x20) in corrupt xml hub list", cnt[5]);
    }
}

static void dc_web_process_body(http_t* http)
{
    buffer_t* buffer = b_socket_get_in_buffer(BSOCK(http));
    char* pos = buffer_read_pos(buffer);
    xmlDocPtr xdoc;

    if (http->content_type != CONTENT_BZIP2 &&
        pos[0] == 'B' && pos[1] == 'Z')
    {
        http_decompress_bz(http);
    }
  
    buffer = b_socket_get_in_buffer(BSOCK(http));
    _dc_web_buffer_correct(buffer, http->content_length);
    xdoc = xmlParseMemory(buffer_read_pos(buffer), http->content_length);
    if (!xdoc)
    {
        dc_write_share_buffer("webhubs", buffer);
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Parse error"),
                       _("An error occurred while parsing the hub list."));
        return;
    }

    xml_get_hublist(xdoc);

    xmlFreeDoc(xdoc);

    dc_hubs_save();
}

static gboolean dc_web_end(http_t* http, const char* message)
{

    if (socket_get_status(SOCK(http)) == ST_INPUT_COMPLETE)
    {
        dc_web_process_body(http);
    }
    else
    {
        g_log("DC_HUB", G_LOG_LEVEL_DEBUG,
              "Getting server from direct connect failed");
    }
    http->keep_alive = FALSE;
    return http_end(http, message);
}

static url_t* dc_weblist_find(url_t* url)
{
    GList* dlist;

    for (dlist = DcHubModule->weblists; dlist; dlist = dlist->next)
    {
        url_t* web = dlist->data;
        if (url_equal(url, web)) return web;
    }
    return NULL;
}

static void on_weblist_destroy(url_t* url)
{
    DcHubModule->weblists = g_list_remove(DcHubModule->weblists, url);
    dc_hubs_save();
}

static url_t* dc_weblist_new(url_t* url)
{
    object_add_type(OBJ(url), "DcWebList");
    DcHubModule->weblists = g_list_append(DcHubModule->weblists, url);
    object_notify_d(OBJ(url), (ObjectFunc)on_weblist_destroy, NULL);
    dc_hubs_save();
    object_make_public(OBJ(url), NULL);
    return url;
}

static url_t* dc_weblist_add(const char* url)
{
    url_t* web = url_parse(url);
    url_t* web2;

    if (!web) return NULL;

    web2 = dc_weblist_find(web);
    if (web2)
    {
        object_kill(OBJ(web));
        return web2;
    }

    return dc_weblist_new(web);
}

void dc_weblist_get_list(const char* url)
{
    url_t* web;

    g_assert(DcHubModule);
    g_log("DC_HUB", G_LOG_LEVEL_DEBUG, "getting hub list");

    web = dc_weblist_add(url);
    if (!web)
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid address"),
                       _("Cannot use that address to get the hub list."));
    }
    else
    {
        http_t* http = http_new_from_url(web);
        if (!http)
        {
            g_log("DC_HUB", G_LOG_LEVEL_WARNING,
                  "Could not create http request to direct connect");
            return;
        }
  
        // set the function that handles the received data
        socket_set_end(SOCK(http), (SockEndFunc)dc_web_end);
        http_get(http);
    }
}

static void on_search_destroyed(search_t* search, dc_hub_t* hub);
static void dc_hub_reset(dc_hub_t* hub)
{
    hub->bytes = 0;
    dc_user_left_hub(hub);
    //  dc_hub_channels_clear(hub);

    if (hub->search_timer)
    {
        lopster_timeout_remove(hub->search_timer);
        hub->search_timer = 0;
    }
    if (hub->queued_searches)
    {
        GList* dlist;
        for (dlist = hub->queued_searches; dlist; dlist = dlist->next)
        {
            search_t* search = dlist->data;
            object_unnotify_o(OBJ(search), (ObjectFunc)on_search_destroyed, OBJ(hub));
        }
        g_list_free(hub->queued_searches);
        hub->queued_searches = NULL;
    }
}

static void dc_hub_update_status()
{
    int status = ONLINE_STATUS_IDLE;
    if (DcHubModule->hubs_online) status |=  ONLINE_STATUS_ONLINE;
    plugin_set_online_status(DC_ID, status);
}

static void dc_hub_update_stats()
{
    unsigned users = 0;
    file_size_t bytes = 0;
    GList* dlist;

    for (dlist = DcHubModule->hubs_online; dlist; dlist = dlist->next)
    {
        dc_hub_t* hub = dlist->data;
        users += dc_hub_get_user_cnt(hub);
        bytes += hub->bytes;
        object_set_dirty(OBJ(hub)); // this is not really the right place to do.
    }
    plugin_set_online_stats(DC_ID, users, 0, bytes/1024/1024/1024);
}

static void
dc_hub_set_status(dc_hub_t* hub, int status, const char* message)
{
    if (status == DC_HUB_ONLINE && hub->status != DC_HUB_ONLINE)
    {
        DcHubModule->hubs_online = g_list_append(DcHubModule->hubs_online, hub);
        dc_hub_update_stats();
        dc_hub_update_status();
    }
    else if (hub->status == DC_HUB_ONLINE && status != DC_HUB_ONLINE)
    {
        DcHubModule->hubs_online = g_list_remove(DcHubModule->hubs_online, hub);
        dc_hub_update_stats();
        dc_hub_update_status();
    }

    hub->status = status;
    DCUI.hub_set_status(hub, message);

    if (status == DC_HUB_OFFLINE)
    {
        socket_set_timeout(SOCK(hub), 0);
    }
    else if (status == DC_HUB_CONNECTING)
    {
        socket_set_timeout(SOCK(hub), 30);
    }
    else
    {
        socket_set_timeout(SOCK(hub), 600);
    }

    object_set_dirty(OBJ(hub));
}

static void dc_hub_add_auto(dc_hub_t* hub);
static gboolean relogin_timer(dc_hub_t* hub)
{
    hub->relogin = 0;
    dc_hub_set_status(hub, DC_HUB_OFFLINE, _("Ready to connect"));
    dc_hub_add_auto(hub);
    return FALSE;
}

static void dc_hub_set_auto(dc_hub_t* hub, gboolean set)
{
    if ((hub->flags & DC_HUB_AUTO) && !set)
    {
        DcHubModule->auto_cnt--;
        hub->flags &= ~DC_HUB_AUTO;
    }
    else if (!(hub->flags & DC_HUB_AUTO) && set)
    {
        DcHubModule->auto_cnt++;
        hub->flags |= DC_HUB_AUTO;
    }
    dc_hub_update_status();
}

static gboolean dc_hub_end(dc_hub_t* hub, const char* message)
{
    tcp_socket_end(TSOCK(hub), message);

    dc_hub_reset(hub);
  
    switch (socket_get_status(SOCK(hub)))
    {
    case S_HUB_USER_DISCONNECT:
        dc_hub_set_auto(hub, FALSE);
        break;
    }
  
    if ((hub->flags & DC_HUB_AUTO) && !hub->relogin)
        hub->relogin = lopster_timeout_add(60000, (GSourceFunc)relogin_timer, hub);
    dc_hub_set_status(hub, hub->relogin?DC_HUB_WAITING:DC_HUB_OFFLINE, message);
    socket_set_status(SOCK(hub), ST_UNKNOWN);

    return FALSE;
}

static const char* dc_hub_consume_input(dc_hub_t* hub)
{
    buffer_t* buffer = b_socket_get_in_buffer(BSOCK(hub));
    dc_message_t* result;

    if (!buffer) return NULL;

    while ((result = dc_consume_message(buffer)) != NULL)
    {
        const char* message = dc_handle_message(hub, result);
        dc_message_destroy(result);
        if (message) return message;
    }
    buffer_consume(buffer, 0);
    return NULL;
}

static void dc_hub_destroy(dc_hub_t* hub)
{
    dc_hub_disconnect(hub);
    g_free(hub->name);
    g_free(hub->short_name);
    g_free(hub->address);
    g_free(hub->description);
    g_free(hub->h_nick);
    g_free(hub->h_pass);
    g_free(hub->c_nick);
    g_free(hub->country);
    g_free(hub->charset);

    tcp_socket_destroy(TSOCK(hub));

    DcHubModule->hubs = g_list_remove(DcHubModule->hubs, hub);

    dc_hubs_save();
}

void dc_hub_set_name(dc_hub_t* hub, const char* name)
{
    int len;

    if (!hub || !name) return;

    // FIXME: make dynamic encoding.
    g_free(hub->name);
    hub->name = g_strdup(name);

    // strip brackets, since the GUI puts their own round the name.
    g_free(hub->short_name);
    len = strlen(hub->name);
    if (hub->name[0] == '[' && hub->name[len-1] == ']')
    {
        len -= 2;
        hub->short_name = g_malloc(len + 1);
        memcpy(hub->short_name, hub->name+1, len);
        hub->short_name[len] = 0;
    }
    else
    {
        hub->short_name = g_strdup(hub->name);
    }
    len = g_utf8_strlen(hub->short_name, len);
    if (len >= 18)
    {
        char* ptr = g_utf8_offset_to_pointer(hub->short_name, 15);
        strcpy(ptr, "...");
    }
    object_set_dirty(OBJ(hub));
    dc_hubs_save();
}

void dc_hub_set_bookmark(dc_hub_t* hub, gboolean set)
{
    if (set) hub->flags |= DC_HUB_BOOKMARK;
    else hub->flags &= ~DC_HUB_BOOKMARK;
    object_set_dirty(OBJ(hub));
    dc_hubs_save();
}

static void dc_hub_init(dc_hub_t* hub, const char* name,
                        const char* address, int port)
{
    tcp_socket_init(TSOCK(hub), name, 0);
    object_add_type(OBJ(hub), "DcHub");
    OBJ(hub)->destroy = (ObjectFunc)dc_hub_destroy;
    socket_set_end(SOCK(hub), (SockEndFunc)dc_hub_end);
    b_socket_set_consume(BSOCK(hub), (BConsumeInputFunc)dc_hub_consume_input);

    TSOCK(hub)->port = htons((port_t)port);

    hub->address = g_strdup(address);
    hub->description = NULL;
    hub->users = NULL;
    hub->user_cnt = 0;
    hub->bytes = 0;
    hub->status = DC_HUB_OFFLINE;
    hub->flags = 0;
    hub->last_search = 0;
    hub->search_interval = 60;  // 60 seconds min
    hub->queued_searches = NULL;
    hub->search_timer = 0;

    dc_hub_set_name(hub, name);
  
    socket_add_load(SOCK(hub), "Server");
}

dc_hub_t* dc_hub_new(const char* name, const char* address, int port)
{
    dc_hub_t* hub = g_malloc0(sizeof(*hub));
    dc_hub_init(hub, name, address, port);

    DcHubModule->hubs = g_list_append(DcHubModule->hubs, hub);

    dc_hubs_save();

    return hub;
}

static void dc_hub_connect_real(dc_hub_t* hub)
{
    socket_t* socket = SOCK(hub);
    char* nick;

    if (hub->status != DC_HUB_OFFLINE) return;

    nick = g_strdup(dc_hub_get_nick(hub));
    if (!nick) return;
  
    g_free(hub->c_nick);
    hub->c_nick = nick;
    // replace invalid characters.
    while (*nick)
    {
        if (*nick == ' ' || *nick == '$' || *nick == '|') *nick = '_';
        nick++;
    }

    dc_hub_set_status(hub, DC_HUB_CONNECTING, _("Connecting..."));

    if (!socket_activate(socket)) return;

    hub->flags &= ~DC_HUB_OPERATOR;
    b_socket_start_receiving(BSOCK(hub), 1024000);
}

void dc_hub_resolver(dc_hub_t* hub, ip_t ip)
{
    // check whether hub is still valid
    if (!g_list_find(DcHubModule->hubs, hub)) return;

    TSOCK(hub)->ip = ip;
    if (ip == 0)
    {
        dc_hub_set_status(hub, DC_HUB_OFFLINE,
                          _("Could not resolve address"));
    }
    else
    {
        // now ready to connect
        dc_hub_set_status(hub, DC_HUB_OFFLINE,
                          _("Successfully resolved address"));
        dc_hub_connect_real(hub);
    }
}

void dc_hub_disconnect(dc_hub_t* hub)
{
    if (hub->relogin)
    {
        lopster_timeout_remove(hub->relogin);
        hub->relogin = 0;
    }
    dc_hub_set_auto(hub, FALSE);

    if (hub->status >= DC_HUB_CONNECTING)
        socket_done(SOCK(hub), S_HUB_USER_DISCONNECT, "User disconnected");
    else if (hub->status == DC_HUB_WAITING)
        dc_hub_set_status(hub, DC_HUB_OFFLINE, "User disconnected");
    
}

void dc_hub_connect(dc_hub_t* hub)
{
    if (!hub) return;

    if (hub->status >= DC_HUB_CONNECTING) return;
    dc_hub_set_auto(hub, TRUE);

    //  if (TSOCK(hub)->ip == 0) {
    resolve_address(hub->address, (call_func_t)dc_hub_resolver, hub);
    //  } else {
    //    dc_hub_connect_real(hub);
    //  }
}

port_t dc_tcp_listen_port()
{
    return listen_port_get_port(DcHubModule->tcp_listen);
}

port_t dc_udp_listen_port()
{
    return udp_listen_get_port(DcHubModule->udp_listen);
}

gboolean dc_in_passive_mode()
{
    return (dc_tcp_listen_port()==0 || dc_udp_listen_port()==0);
}

static void dc_hub_send_search_real(dc_hub_t* hub, search_t* search)
{
    char* req = dc_search_create_request(search);

    if (!req) return;

    if (dc_in_passive_mode())
    {
        dc_hub_send(hub, FALSE,	"$Search Hub:%s %s", hub->c_nick, req);
    }
    else
    {
        char* req = dc_search_create_request(search);
        dc_hub_send(hub, FALSE, "$Search %s:%d %s",
                    ntoa(get_local_ip(SOCK(hub))), dc_udp_listen_port(), req);
    }
    g_free(req);
    hub->last_search = CurrentTime;
}

static void dc_hub_create_search_timer(dc_hub_t* hub);
static gboolean on_pop_search(dc_hub_t* hub)
{
    lopster_timeout_remove(hub->search_timer);
    hub->search_timer = 0;

    if (hub->queued_searches)
    {
        search_t* search = hub->queued_searches->data;
        hub->queued_searches = g_list_delete_link(hub->queued_searches, hub->queued_searches);
        dc_hub_send_search_real(hub, search);
        dc_hub_create_search_timer(hub);
    }
    return TRUE;
}

static void dc_hub_create_search_timer(dc_hub_t* hub)
{
    if (!hub->search_timer)
    {
        int t = hub->last_search + hub->search_interval - CurrentTime;
        if (t < 0) t = 0;
        hub->search_timer = lopster_timeout_add(t*1000, (GSourceFunc)on_pop_search, hub);
    }
}

static void on_search_destroyed(search_t* search, dc_hub_t* hub)
{
    hub->queued_searches = g_list_remove(hub->queued_searches, search);
}

void dc_hub_send_search(dc_hub_t* hub, search_t* search)
{
    if (!hub) {
        g_list_foreach(DcHubModule->hubs, (GFunc)dc_hub_send_search, search);
        return;
    }
    else if (hub->status == DC_HUB_ONLINE)
    {
        if (hub->queued_searches || 
            CurrentTime < hub->last_search + hub->search_interval)
        {
            // queue search
            hub->queued_searches = g_list_append(hub->queued_searches, search);
            // remove from queued list if search gets destroyed
            object_notify_o(OBJ(search), (ObjectFunc)on_search_destroyed, OBJ(hub));
            // activate timer (if not activated yet)
            dc_hub_create_search_timer(hub);
        }
        else
        {
            // perform search now.
            dc_hub_send_search_real(hub, search);
        }
    }
}

void dc_hub_send_myinfo(dc_hub_t* hub)
{
    file_tree_t* lib = FILE_TREE(library_get_current());
    int speed = pref_get_ref("/core/line/type", int);
    gboolean afk = pref_get_ref("/core/user/away", gboolean);

    dc_hub_send
        (hub, FALSE,
         "$MyINFO $ALL %s <%s V:%s,M:%c,H:%d/%d/%d,S:%d>$ $%s%c$anon@anon.com$"FF"$",
         hub->c_nick,
         "LII", PURE_VERSION, //client, version
/*      "++", "0.674", */
         dc_in_passive_mode()?'P':'A',
         1, 0, 0, //counts (normal,registered,op) //FIXME
         uploads_slots_number(),       //slots
         DcSpeedFromType(speed), afk?2:1,
         lib?lib->fsi[MEDIA_ANY][1]:0);
    // additional option: M:min_upload_speed
}

/* void dc_hub_send_all(const char* type, const char* message) { */
/*   GList* dlist = NULL; */

/*   g_assert(type); */

/*   dlist = DcHubModule->hubs; */

/*   while (dlist) { */
/*     dc_hub_t* hub = dlist->data; */
/*     if (hub->status != DC_HUB_OFFLINE) */
/*       dc_hub_send(hub, FALSE, "%s %s", type, message); */

/*     dlist = dlist->next; */
/*   } */
/* } */

void dc_hub_send(dc_hub_t* hub, gboolean convert, const char* fmt, ...)
{
    buffer_t* buffer;
    g_assert(DcHubModule);

    if (hub->status >= DC_HUB_CONNECTING)
    {
        va_list ap;
        char* message;

        va_start(ap, fmt);
        message = g_strdup_vprintf(fmt, ap);
        va_end(ap);
    
        buffer = b_socket_get_out_buffer(BSOCK(hub), 1024);
        if (convert)
            dc_pack_command(buffer, "DC_PROtO", auto_text
                            (from_utf8(message, -1, NULL, NULL, 3,
                                       hub->charset, "windows-1252", "utf-8")));
        else
            dc_pack_command(buffer, "DC_PROTO", message);
        b_socket_start_sending(BSOCK(hub));

        g_free(message);
    }
}

const char* dc_hub_get_nick(dc_hub_t* hub)
{
    char* nick = NULL;

    if (hub)
        nick = hub->h_nick;
    if (!nick || *nick == 0)
        nick = pref_get_ptr("/dc/username", char);
    if (!nick || *nick == 0)
        nick = pref_get_ptr("/core/user/nick", char);
    if (!nick || *nick == 0) 
        nick = "unknown";
    return nick;
}

const char* dc_hub_get_pass(dc_hub_t* hub)
{
    char* pass = NULL;
  
    if (hub)
        pass = hub->h_pass;
    if (!pass || *pass == 0) 
        pass = pref_get_ptr("/dc/password", char);
    if (!pass || *pass == 0)
        pass = pref_get_ptr("/core/user/pass", char);
    if (!pass || *pass == 0)
        pass = "unknown";
    return pass;
}

void dc_hub_success(dc_hub_t* hub)
{
    // dont answer the hello message more than once.
    if (hub->status == DC_HUB_ONLINE) return;

    dc_hub_set_status(hub, DC_HUB_ONLINE, _("Online"));
    if (pref_get_ref("/dc/chat/autojoin", gboolean) && !dc_chat_active(hub))
        dc_cp_new_public(hub);
    dc_hub_send(hub, FALSE, "$Version 1,0091");
    dc_hub_send_myinfo(hub);
    dc_hub_send(hub, FALSE, "$GetNickList");
}

void dc_hub_go(dc_hub_t* hub)
{
    dc_search_hub_entered(hub);
}

static const char* dc_read_input(udp_listen_t* listen)
{
    buffer_t* buffer = b_socket_get_in_buffer(BSOCK(listen));
    dc_message_t* result;
    ip_t ip;

    if (!buffer) return NULL;

    while ((result = dc_consume_peer_message(buffer, &ip)) != NULL)
    {
        const char* message = dc_handle_udp_message(USOCK(listen), result, ip);
        g_free(result);
        if (message) return message;
    }
    buffer_consume(buffer, 0);
    return NULL;
}

dc_hub_t* dc_get_hub(int index)
{
    return g_list_nth_data(DcHubModule->hubs_online, index-1);
}

unsigned dc_hub_get_user_cnt(dc_hub_t* hub)
{
    if (hub->users) return g_hash_table_size(hub->users);
    else return hub->user_cnt;
}


static dc_peer_t* dc_accept_con(int fd, ip_t ip, port_t port)
{
    dc_peer_t* peer = dc_peer_new();
    peer->flags |= DC_PEER_INCOMING;
    listen_init_accepted(TSOCK(peer), fd, ip, port);
    return peer;
}

static void dc_start_listen_service()
{
    DcHubModule->tcp_listen =
        listen_port_new("/dc/listen/tcp_port", "DC TCP listen",
                        (ListenAcceptFunc)dc_accept_con,
                        (ListenChangeFunc)NULL);
    object_set_ptr((object_t**)&DcHubModule->tcp_listen);
    listen_port_start_listening(DcHubModule->tcp_listen, 10);

    DcHubModule->udp_listen =
        udp_listen_new("/dc/listen/udp_port", "DC UDP listen");
    object_set_ptr((object_t**)&DcHubModule->udp_listen);
    b_socket_set_consume(BSOCK(DcHubModule->udp_listen),
                         (BConsumeInputFunc)dc_read_input);
    udp_listen_start_listening(DcHubModule->udp_listen);
}

static void on_hub_destroyed(dc_hub_t* hub)
{
    DcHubModule->pending = g_list_remove(DcHubModule->pending, hub);
}

static gboolean dc_hub_start_next()
{
    dc_hub_t* hub;

    if (!DcHubModule->pending)
    {
        DcHubModule->auto_timer = 0;
        return FALSE;
    }
    hub = DcHubModule->pending->data;
    DcHubModule->pending = g_list_remove(DcHubModule->pending, hub);
    object_unnotify_d(OBJ(hub), (ObjectFunc)on_hub_destroyed, NULL);

    dc_hub_connect(hub);
    return TRUE;
}

static void dc_hub_add_auto(dc_hub_t* hub)
{
    DcHubModule->pending = g_list_append(DcHubModule->pending, hub);
    object_notify_d(OBJ(hub), (ObjectFunc)on_hub_destroyed, NULL);

    dc_hub_set_auto(hub, TRUE);

    if (!DcHubModule->auto_timer)
    {
        DcHubModule->auto_timer =
            lopster_timeout_add(1000, (GSourceFunc)dc_hub_start_next, NULL);
    }
}

static void dc_start_connect()
{
    GList* dlist;
  
    g_assert(DcHubModule);
    for (dlist = DcHubModule->hubs; dlist; dlist = dlist->next)
    {
        dc_hub_t* hub = dlist->data;
        if (hub->flags & DC_HUB_AUTO) continue;
        if (hub->flags & DC_HUB_BOOKMARK) dc_hub_add_auto(hub);
    }
}

static void dc_stop_connect(void)
{
    g_list_foreach(DcHubModule->pending, (GFunc)on_hub_destroyed, NULL);
    g_list_free(DcHubModule->pending);
    DcHubModule->pending = NULL;
    g_list_foreach(DcHubModule->hubs, (GFunc)dc_hub_disconnect, NULL);
}
  
void dc_connect_change(unsigned status)
{
    switch (status) {
    case CONNECT_START:
        dc_start_connect();
        break;
    case CONNECT_STOP:
        dc_stop_connect();
        break;
    }
}


static void dc_hubs_init()
{
    lopster_xml_register_home("dc:hublist", SDIR("dc") "hub_list.xml",
                              "Hublist", xml_get_hubs_local, xml_save_hubs);

    pref_register_string("/dc/username", TRUE, NULL);
    pref_add_constraint_replace("/dc/username", ' ', '_');

    pref_register_string("/dc/password", TRUE, NULL);
    pref_add_constraint_replace("/dc/password", ' ', '_');

    pref_register_int("/dc/listen/udp_port", TRUE, 9000);
    pref_add_constraint_port("/dc/listen/udp_port");

    pref_register_int("/dc/listen/tcp_port", TRUE, 4111);
    pref_add_constraint_port("/dc/listen/tcp_port");

    pref_register_bool("/dc/chat/autojoin", TRUE, FALSE);
}

static gboolean _dc_hub_update_stats()
{
    dc_hub_update_stats();
    return TRUE;
}

static void dc_hubs_run()
{
    dc_hubs_load();
    dc_start_listen_service();
    lopster_call_periodic(60000, (CallFunc)_dc_hub_update_stats, NULL);
}


static void dc_hubs_exit()
{
    //  dc_hubs_save_real();
}

static void dc_hub_module_init(dc_hub_module_t* module)
{
    module_init(MODULE(module), "Dc Hub Module",
                (ModuleFunc)dc_hubs_init,
                (ModuleFunc)dc_hubs_run,
                (ModuleFunc)dc_hubs_exit);

    module->hubs = NULL;
    module->hubs_online = NULL;
    module->tcp_listen = NULL;
    module->save = TRUE;
    module->weblists = NULL;

    module_require(xml_module_instance);
    module_require(base_module_instance);
    module_require(tsocket_module_instance);
    module_require(usocket_module_instance);
    module_require(dc_search_module_instance);
    module_require(dc_ui_module_instance);
    module_require(listen_module_instance);
    /*
      module_require(buddy_module_instance);
      module_require(library_module_instance);
      module_require(resolve_module_instance);
    */
    //FIXME: add more deps
}


module_t* dc_hub_module_instance()
{
    if (!DcHubModule)
    {
        DcHubModule = g_malloc0(sizeof(*DcHubModule));
        dc_hub_module_init(DcHubModule);
    }
    return MODULE(DcHubModule);
}

#ifdef WIN32
#define _DC_NETWORK_C_
#include "dcaliasdef.c"
#endif

