/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.3 $
 * $Date: 2007/05/23 20:42:28 $
 * $Author: sid4u $
 */

#ifndef _DC_NET_H_
#define _DC_NET_H_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "lcore/tcp_socket.h"
#include "lcore/module.h"
#include "lcore/search.h"
//#include "library.h"

G_BEGIN_DECLS

#define DC_HUB_OFFLINE    0
#define DC_HUB_WAITING    1
#define DC_HUB_CONNECTING 2
#define DC_HUB_ONLINE     3

#define DC_FEATURE_NOGETINFO 0x01
#define DC_FEATURE_NOHELLO   0x02
#define DC_FEATURE_USERIP2   0x04

#define DC_HUB_OPERATOR   0x01
#define DC_HUB_BOOKMARK   0x02
#define DC_HUB_AUTO       0x04

#define S_HUB_USER_DISCONNECT  300

#define DC_HUB(n) ((dc_hub_t*)(n))
typedef struct {
    tcp_socket_t _socket;
    char* name;
    char* short_name;
    char* address;
    char* description;
    char* h_nick;
    char* h_pass;
    char* c_nick;
    char* country;
    int status;
    
    char* charset;
    unsigned features;
    unsigned flags;
    int relogin;
    
    time_t last_search;
    unsigned search_interval;
    GList* queued_searches;
    int search_timer;

    GHashTable* users;
    unsigned user_cnt;
    file_size_t bytes;
} dc_hub_t;


module_t* dc_hub_module_instance();

void dc_weblist_get_list(const char* url);

char* dc_hub_to_utf8(dc_hub_t* hub, const char* data);

dc_hub_t* dc_hub_new(const char* name, const char* address, int port);
void dc_hub_edit(dc_hub_t* hub);

void dc_hub_connect   (dc_hub_t* hub);
void dc_hub_disconnect(dc_hub_t* hub);
void dc_hub_success   (dc_hub_t* hub);
void dc_hub_go        (dc_hub_t* hub);

void dc_hub_set_name(dc_hub_t* hub, const char* name);
void dc_hub_set_bookmark(dc_hub_t* hub, gboolean set);

void dc_hub_send(dc_hub_t* hub, gboolean convert, const char* fmt, ...);
/* void dc_hub_send_all(const char* type, const char* message); */
void dc_hub_send_search(dc_hub_t* hub, search_t* search);
void dc_hub_send_myinfo(dc_hub_t* hub);


const char* dc_hub_get_nick(dc_hub_t* hub);
const char* dc_hub_get_pass(dc_hub_t* hub);
unsigned dc_hub_get_user_cnt(dc_hub_t* hub);
dc_hub_t* dc_get_hub(int index);

gboolean dc_in_passive_mode();
port_t dc_tcp_listen_port();
port_t dc_udp_listen_port();

void dc_connect_change(unsigned status);

G_END_DECLS

#endif
