/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2007/11/25 22:19:58 $
 * $Author: sgop $
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "lcore/utils.h"
#include "lcore/mtypes.h"
#include "lcore/udp_socket.h"
#include "lcore/lopster.h"
#include "lcore/preferences.h"
#include "dc_proto.h"
#include "dc.h"
#include "dc_chat.h"
#include "dc_search.h"
#include "dc_users.h"
#include "dc_peer.h"

#define HANDLER(f) static void f (dc_hub_t* hub, char* data)
#define DC_CALL(f) f(hub, command->message)
#define DC_CALLC(f) f(hub, auto_text(dc_hub_to_utf8(hub, command->message)))

#define CHECK(expr, proto, message)                                     \
    if (!(expr)) {g_log(proto, G_LOG_LEVEL_WARNING, message); return;}

static ip_t UDP_IP = 0;

char* dc_unescape(char* text)
{
    char* text_end = text + (text?strlen(text):0);
    char* iter = text;
    char* res_iter = text;

    if (!text) return NULL;

    // we unescape a few more than dc++
    while (iter != text_end)
    {
        if (*iter == '&')
        {
            if (iter+5 <= text_end && !strncmp(iter, "&amp;", 5))
            {
                *res_iter++ = '&'; iter += 5;
            }
            else if (iter+4 <= text_end && !strncmp(iter, "&lt;", 4))
            {
                *res_iter++ = '<'; iter += 4;
            }
            else if (iter+4 <= text_end && !strncmp(iter, "&gt;", 4))
            {
                *res_iter++ = '>'; iter += 4;
            }
            else if (iter+6 <= text_end && !strncmp(iter, "&quot;", 6))
            {
                *res_iter++ = '"'; iter += 6;
            }
            else if (iter+6 <= text_end && !strncmp(iter, "&apos;", 6))
            {
                *res_iter++ = '\''; iter += 6;
            }
            else if (iter+6 <= text_end && !strncmp(iter, "&#124;", 6))
            {
                *res_iter++ = '|'; iter += 6;
            }
            else if (iter+5 <= text_end && !strncmp(iter, "&#36;", 5))
            {
                *res_iter++ = '$'; iter += 5;
            }
            else
            {
                *res_iter++ = *iter++;
            }
        }
        else
        {
            *res_iter++ = *iter++;
        }
    }
    *res_iter++ = 0;

    return text;
}

char* dc_escape(const char* text)
{
    int len = strlen(text);
    const char* p = text;
    const char* end = text + len;
    GString* str = g_string_sized_new(len);

    while (p != end)
    {
        const char* next = g_utf8_next_char(p);

        switch (*p)
        {
        case '&':  // other than dc++, we escape any '&'
            g_string_append(str, "&amp;");
            break;
        case '|':
            g_string_append(str, "&#124;");
            break;
        case '$':
            g_string_append(str, "&#36;");
            break;
/*     case '<': */
/*       g_string_append (str, "&lt;"); */
/*       break; */
/*     case '>': */
/*       g_string_append (str, "&gt;"); */
/*       break; */
/*     case '\'': */
/*       g_string_append (str, "&apos;"); */
/*       break; */
/*     case '"': */
/*       g_string_append (str, "&quot;"); */
/*       break; */
        default:
            g_string_append_len(str, p, next - p);
            break;
        }
        p = next;
    }

    return g_string_free(str, FALSE);
}

void dc_message_destroy(dc_message_t* message)
{
    g_free(message->alloc);
    g_free(message);
}

void dc_send(b_socket_t* sock, gboolean convert, const char* fmt, ...)
{
    va_list ap;
    char* message;
    buffer_t* buffer;

    va_start(ap, fmt);
    message = g_strdup_vprintf(fmt, ap);
    va_end(ap);

    buffer = b_socket_get_out_buffer(sock, 1024);
    if (convert)
        dc_pack_command(buffer, "DC_SeND", auto_text
                        (from_utf8(message, -1, NULL, NULL, 2, "windows-1252", "utf-8")));
    else
        dc_pack_command(buffer, "DC_SEND", message);
    b_socket_start_sending(sock);

    g_free(message);
}

void dc_pack_command(buffer_t* buffer, const char* proto, const char* message)
{
    gsize len;

    if (!message)
    {
        g_log("DC_PROTO", G_LOG_LEVEL_WARNING, "skip sending NULL-message");
        g_assert(FALSE);
        return;
    }
    len = strlen(message);

    //  if (strncmp(message, "$SR", 3)) {
    g_log(proto, G_LOG_LEVEL_DEBUG, "S: %s", message);
    //  }
  
    buffer_need_space(buffer, len+3);
    buffer_add_data(buffer, message, len);
    buffer_add_data(buffer, "|", 1);
    buffer_write_pos(buffer)[0] = 0;
}

dc_message_t* dc_consume_message(buffer_t* buffer)
{
    dc_message_t* result;
    guint16 len;
    char* data;
    char* pos;

    if (!buffer) return NULL;

    /*
      if (buffer_avail(buffer))
      g_log("*****", G_LOG_LEVEL_DEBUG,
      buffer_read_pos(buffer), buffer_avail(buffer));
    */
    while (TRUE)
    {
        if (buffer_avail(buffer) < 1) return NULL;
        data = buffer_read_pos(buffer);
        pos = strchr(data, '|');
        if (!pos) return NULL;

        if (pos == data)
        {
            g_log("DC_PROTO", G_LOG_LEVEL_DEBUG, "** skip empty message **");
            buffer_consume_virt(buffer, 1);
            continue;
        }

        len = pos-data;
        buffer_consume_virt(buffer, len + 1);
        *pos = 0;

        result = g_malloc0(sizeof(dc_message_t));
        result->message = data;
        result->alloc = NULL;
        result->len = len;
    
        if (*result->message == '$' || *result->message == '<')
        {
            result->type = arg(result->message, 0);
            result->message = arg(NULL, 1);
            result->len = result->message?strlen(result->message):0;
        }
        return result;
    }
}

dc_message_t* dc_consume_peer_message(buffer_t* buffer, ip_t* ip)
{
    dc_message_t* result;
    unsigned int avail;
    char* data;
    int hdr = sizeof(unsigned)+sizeof(ip_t);
    unsigned len;

    if (!buffer) return NULL;
  
    avail = buffer_avail(buffer);
    if (avail < hdr) return NULL;
    memcpy(&len, buffer_read_pos(buffer), sizeof(unsigned));
    if (avail < hdr+len)
    {
        g_log("DC_PROTO", G_LOG_LEVEL_WARNING, "message too short");
        buffer_clear(buffer);  // reset buffer
        return NULL;
    }
    buffer_consume_virt(buffer, sizeof(unsigned));

    memcpy(ip, buffer_read_pos(buffer), sizeof(ip_t));
    buffer_consume_virt(buffer, sizeof(ip_t));

    if (len==0) return NULL;

    data = buffer_read_pos(buffer);
    if (data[len-1] != '|')
    {
        // dont care if '|' is missing at the end
        char* buf;
        char temp = data[len];

        data[len] = 0;
        buf = l_strdup(data);
        data[len] = temp;
        data = buf;
        g_log("DC_PROTO", G_LOG_LEVEL_WARNING, "end of message != '|'");
    } else {
        data[len-1] = 0;
    }
    len--;

    /*
      if ((pos = strchr(data, '|')) != NULL) {
      g_log("DC_PROTO", G_LOG_LEVEL_WARNING, "more than one '|' in message");
      *pos = 0;  // truncate rest of message
      }
    */

    result = g_malloc0(sizeof(dc_message_t));
    result->message = data;
    result->len = len;

    if (*result->message == '$' || *result->message == '<')
    {
        result->type = arg(result->message, 0);
        result->message = arg(NULL, 1);
        result->len = result->message?strlen(result->message):0;
    }

    buffer_consume_virt(buffer, len + 1);

    return result;
}

HANDLER(handle_lock)
{
    char* lock;
    char* pk;
    char* key;

    CHECK(data, "DC_PROTO", "invalid $Lock");
  
    lock = data;
    pk = strstr(lock, " Pk=");
    if (pk)
    {
        *pk = 0;
        pk += 4;
    }

    if (!strncmp(lock, "EXTENDEDPROTOCOL", 16))
    {
        // just testing for now
        dc_hub_send(hub, FALSE, "$Supports NoHello NoGetINFO UserIP2 TTHSearch");
        //GetZBlock UserCommand 
    }

    key = dc_generate_key(lock);
    dc_hub_send(hub, FALSE, "$Key %s", key);
    g_free(key);

    dc_hub_send(hub, FALSE, "$ValidateNick %s", hub->c_nick);
}

HANDLER(handle_supports)
{
    char* feature = arg(data, 0);
    hub->features = 0;
    while (feature)
    {
        if (!strcmp(feature, "NoGetINFO"))
            hub->features |= DC_FEATURE_NOGETINFO;
        else if (!strcmp(feature, "NoHello"))
            hub->features |= DC_FEATURE_NOHELLO;
        else if (!strcmp(feature, "UserIP2"))
            hub->features |= DC_FEATURE_USERIP2;
        else
            g_log("DC_PROTO", G_LOG_LEVEL_WARNING,
                  "feature '%s' unknown", feature);
        feature = arg(NULL, 0);
    }
}

HANDLER(handle_hello)
{
    char* nick = arg(data, 0);

    CHECK(nick, "DC_PROTO", "invalid $Hello");
  
    if (hub->c_nick && !strcmp(hub->c_nick, nick))
    {
        dc_hub_success(hub);
/*     dc_hub_go(hub);  //done in handle_op_list() now */
    }
    else
    {
        dc_user_t* user = dc_user_find(nick);
        dc_user_is_online(user, hub);
        if (!(hub->features & DC_FEATURE_NOGETINFO))
        {
            dc_hub_send(hub, FALSE, "$GetINFO %s %s", nick, hub->c_nick);
        }
    }
  
}

HANDLER(handle_nick_list)
{
    char* scan;
    char* pos;
    const char* my_nick;

    CHECK(data, "DC_PROTO", "invalid $NickList");

    if (hub->features & DC_FEATURE_NOGETINFO)
    {
        my_nick = NULL;
    }
    else
    {
        my_nick = hub->c_nick;
    }

    scan = data;
    while ((pos = strchr(scan, '$')) != NULL)
    {
        dc_user_t* user;
        if (pos[1] == '$')
        {
            *pos = 0;
            if (my_nick)
                dc_hub_send(hub, FALSE, "$GetINFO %s %s", data, my_nick);
            user = dc_user_find(data);
            dc_user_is_online(user, hub);
            data = scan = pos+2;
        }
        else
        {
            scan = pos+1;
        }
    }
}

HANDLER(handle_op_list)
{
    char* scan;
    char* pos;

    CHECK(data, "DC_PROTO", "invalid $OpList");
    scan = data;
    while ((pos = strchr(scan, '$')) != NULL)
    {
        dc_user_t* user;
        if (pos[1] == '$')
        {
            *pos = 0;
            user = dc_user_find(data);
            // make sure this user is in the server list.
            dc_user_is_online(user, hub);
            if (!g_list_find(user->opped, hub))
                user->opped = g_list_append(user->opped, hub);
            data = scan = pos+2;
        }
        else
        {
            scan = pos+1;
        }
    }

    // hope we always get OpList after a complete user list.
    dc_hub_go(hub);
}

HANDLER(handle_myinfo)
{
    char* nick;
    char* pos;
    char* rest;
    dc_user_t* user;

    /*
      g_log("...", G_LOG_LEVEL_DEBUG,
      "we have MyINFO %u", hub->features & DC_FEATURE_NOHELLO);
    */
    //  (void)hub;

    nick = arg(data, 0);
    CHECK(nick && !strcmp(nick, "$ALL"), "DC_PROTO", "invalid $MyINFO 1");

    nick = arg(NULL, 0);
    rest = arg(NULL, 1);
    CHECK(nick && *nick != 0 && rest, "DC_PROTO", "invalid $MyINFO 2");

    user = dc_user_find(nick);
    //  user->refreshed = CurrentTime;
    pos = strchr(rest, '$');
    CHECK(pos && pos[1]!=0 && pos[2]=='$', "DC_PROTO", "invalid $MyINFO 3");
    // NOTE: pos[1] is ' ' or 'P'(passive?) or 'A'(active?)
    *pos = 0;

    g_free(user->description);
    user->description = dc_unescape(g_strdup(rest));
    rest = pos+3;

    pos = strchr(rest, '$');
    CHECK(pos, "DC_PROTO", "invalid $MyINFO 4");
    *pos = 0;

    /* Cut last byte for now, interpretation:
       1 = normal,
       2 and 3 = away,
       4 and 5 = server,
       6 and 7 = server away,
       8 and 9 = fireball,
       10 and 11 = fireball away
    */
    rest[strlen(rest)-1] = 0;
    g_free(user->linespeed);
    user->linespeed = g_strdup(rest);
    rest = pos+1;

    pos = strchr(rest, '$');
    CHECK(pos, "DC_PROTO", "invalid $MyINFO 5");
    *pos = 0;
    g_free(user->email);
    user->email = g_strdup(rest);
    rest = pos+1;

    pos = strchr(rest, '$');
    CHECK(pos, "DC_PROTO", "invalid $MyINFO 5");
    *pos = 0;
    dc_user_set_bytes(user, strtoull(rest, NULL, 10));

    // don't check this feature for now, DCH++ doesnt send NoHello
/*   if (hub->features & DC_FEATURE_NOHELLO) { */
    dc_user_is_online(user, hub);
/*   } */

}

HANDLER(handle_quit)
{
    char* nick = arg(data, 0);
    dc_user_t* user;

    CHECK(nick, "DC_PROTO", "invalid $Quit");
  
    user = dc_user_find(nick);
    dc_user_is_offline(user, hub);
}

HANDLER(handle_to)
{
    char* to;
    char* from;
    char* nick;
    char* message;
    
    // $To: <othernick> From: <nick> $<<nick>> ;<message>

    to = arg(data, 0);    // this should be us
    from = arg(NULL, 0);  // "From:"
    from = arg(NULL, 0);  // the sender
    nick = arg(NULL, 0);  // the nickname
    message = arg(NULL, 1);
    CHECK(nick, "DC_PROTO", "invalid $To");
  
    dc_private_message(hub, from, auto_text(dc_hub_to_utf8(hub, message)));
}

HANDLER(handle_search) {
    char* address1;
    char* address2;
    char* query;
    file_size_t size;
    gboolean is_max;
    gboolean is_res;
    char* pos;
    int media;

    (void)hub;
    address1 = arg(data, 0);
    query = arg(NULL, 1);

    CHECK(query && strlen(query) >= 4, "DC_PROTO", "invalid $Search 1"); 

    pos = strchr(address1, ':');
    if (pos)
    {
        *pos = 0;
        address2 = pos+1;
    }
    else
    {
        address2 = NULL;
    }  

    if (query[0] == 'F')
    {
        is_res = FALSE;
        is_max = FALSE;
    }
    else if (query[2] == 'T')
    {
        is_res = TRUE;
        is_max = TRUE;
    }
    else
    {
        is_res = TRUE;
        is_max = FALSE;
    }

    query += 4;
    pos = strchr(query, '?');
    CHECK(pos, "DC_PROTO", "invalid $Search 2");
    *pos = 0;
    size = strtoll(query, NULL, 10);
  
    query = pos+1;
    pos = strchr(query, '?');
    CHECK(pos, "DC_PROTO", "invalid $Search 3");
    *pos = 0;
  
    media = atoi(query);
    query = pos+1;

    //FIXME: to utf8
    dc_search_answer
        (hub, address1, address2, query, media,
         (is_res&&!is_max)?size:0, (is_res&&is_max)?size:0);
}

HANDLER(handle_connect_to_me)
{
    char* nick = arg(data, 0);
    char* iport = arg(NULL, 0);
    char* port;

    CHECK(iport, "DC_PROTO", "invalid $ConnectToMe (1)");

    //FIXME: we could verify that nick is our nick
    port = strchr(iport, ':');
    CHECK(port, "DC_PROTO", "invalid $ConnectToMe (2)");
    *port++ = 0;
  
    dc_peer_connect_to_user(hub, inet_addr(iport), htons(atoi(port)));
}

HANDLER(handle_search_result)
{
    char* nick;
    char* filename;
    char* filesize;
    char* stats;
    char* checksum;
    char* ip;
    char* rest;
    char* pos;
    file_t* file;
    dc_user_t* user;
    unsigned fs, as;

    nick = arg(data, 0);
    rest = arg(NULL, 1);
    CHECK(rest, "DC_PROTO", "invalid $SR 1");

    pos = strchr(rest, 0x05);
    CHECK(pos, "DC_PROTO", "invalid $SR 2");
    filename = rest;
    *pos++ = 0;

    filesize = arg(pos, 0);
    rest = arg(NULL, 1);
    CHECK(rest, "DC_PROTO", "invalid $SR 3");

    pos = strchr(rest, 0x05);
    CHECK(pos, "DC_PROTO", "invalid $SR 4");
    stats = rest;
    *pos++ = 0;
  
    checksum = pos;
    pos = strrchr(checksum, ' ');
    CHECK(pos, "DC_PROTO", "invalid $SR 5");
    *pos++ = 0;

    ip = pos;  // (ip:port)
    if (*ip == '(') ip++;
    pos = strchr(ip, ':');
    if (pos) *pos++ = 0;

    //FIXME: evaluate hub ip:port ?

    file = file_new();
    file_set_origname(file, g_strdup(filename));
    file_set_locname(file, dc_hub_to_utf8(hub, filename));
    file->size = strtoull(filesize, NULL, 10);
    if (!strncmp(checksum, "TTH:", 4))
    {
        attr_store_str(ATTRC(file), "file", "urn:tree:tiger", checksum+4);
    }

    fs = as = 0;
    sscanf(stats, "%u/%u", &fs, &as);

    user = dc_user_find(auto_text(dc_hub_to_utf8(hub, nick)));
    // if results comes from server, set the user online.
    // else set the ip of the user from UDP
    if (hub) dc_user_is_online(user, hub);
    else user_set_ip(user, UDP_IP);

    dc_search_insert_result(user, file, fs, as);
}

HANDLER(handle_rev_connect)
{
    char* nick = arg(data, 0);
    char* me = arg(NULL, 0);
    const char* real_me;
    port_t port;

    real_me = hub->c_nick;
    CHECK(me && real_me && !strcmp(real_me, me),
          "DC_PROTO", "invalid $RevConnectToMe: nicknames do not match");

    port = dc_tcp_listen_port();
    if (port)
    {
        dc_hub_send(hub, FALSE, "$ConnectToMe %s %s:%u",
                    nick, ntoa(get_local_ip(SOCK(hub))), port);
    }
    else
    {
        // both firewalled
        dc_hub_send(hub, FALSE, "$RevConnectToMe %s %s", me, nick);
    }
}

HANDLER(handle_emote)
{
    char* nick;
    char* message;

    nick = arg(data, 0); //skip "*"
    nick = arg(NULL, 0);
    message = arg(NULL, 1);

    CHECK(nick, "DC_PROTO", "invalid emotion");
    dc_emote_message(hub, nick, message);
}

HANDLER(handle_user_ip)
{
    char* nick = arg(data, 0);
    char* ip = arg(NULL, 0);

    CHECK(ip, "DC_PROTO", "invalid $UserIP");
/*   CHECK(hub->features & DC_FEATURE_USERIP2, "DC_PROTO", "invalid $UserIP"); */

    if (!strcmp(nick, hub->c_nick))
    {
        set_local_ip(inet_addr(ip));
    }
    else
    {
        // FIXME: maybe needed for admins
    }
}

HANDLER(handle_get_pass)
{
    dc_hub_send(hub, FALSE,
                "$MyPass %s", dc_hub_get_pass(hub));
}

static gboolean
dc_message_is_special_message(dc_hub_t* hub, const char* n, const char* message)
{
    int t;

    (void)n;  //FIXME: we should not accept this message from any user.

    if (!message)
    {
        return FALSE;
    }
    if (sscanf(message, "Minimum search interval is:%ds", &t) == 1)
    {
        hub->search_interval = t+2;  // don't take the limit too accurate
        g_log("DC_PROTO", G_LOG_LEVEL_DEBUG,
              "Search interval on '%s' now is %d seconds",
              hub->short_name, hub->search_interval);
        //FIXME: maybe remember nick (=server)
        return TRUE;
    }
    else 
    {
        return FALSE;
    }
}

const char* dc_handle_message(dc_hub_t* hub, dc_message_t* command)
{
    //FIXME: handle escaped chars.
    if (!command->type)
    {
        command->alloc = command->message = 
            dc_unescape(dc_hub_to_utf8(hub, command->message));
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        if (command->message && *command->message == '*')
        {
            DC_CALL(handle_emote);
        } 
        else
        {
            dc_motd_message(hub, command->message);
        }
        //    dc_server_message(hub, command->message);
    }
    else if (!strcmp(command->type, "<-MOTD->") ||
             !strcmp(command->type, "<MOTD>"))
    {
        command->alloc = command->message =
            dc_unescape(dc_hub_to_utf8(hub, command->message));
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        dc_motd_message(hub, command->message);
        /*
          } else if (!strcmp(command->type, "<-CORE->")) {
          DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
          dc_server_message(hub, command->message);
        */
    }
    else if (command->type[0] == '<')
    {
        char* temp = dc_hub_to_utf8(hub, command->type);
        char* nick = temp;

        if (nick && *nick && nick[1] && nick[2])
        {
            if (*nick == '<') nick++;
            if (nick[strlen(nick)-1] == '>') nick[strlen(nick)-1] = 0;
        }

        command->alloc = command->message =
            dc_unescape(dc_hub_to_utf8(hub, command->message));
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        // take message as server message if login not completed yet.
        if (hub->status != DC_HUB_ONLINE)
        {
            dc_server_message(hub, command->message);
        }
        else if (command->message && !strncmp(command->message, "/me ", 4))
        {
            dc_emote_message(hub, nick, command->message+4);
        } 
        else if (!dc_message_is_special_message(hub, nick, command->message))
        {
            dc_public_message(hub, nick, command->message);
        }
        g_free(temp);
    }
    else if (!strcmp(command->type, "$Lock"))
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_lock);
    }
    else if (!strcmp(command->type, "$Supports"))
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_supports);
    }
    else if (!strcmp(command->type, "$HubName"))
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        command->alloc = command->message = 
            dc_unescape(dc_hub_to_utf8(hub, command->message));  //really unescape?
        dc_hub_set_name(hub, command->message);
    }
    else if (!strcmp(command->type, "$Hello"))
    {
        command->alloc = command->message = dc_hub_to_utf8(hub, command->message);
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_hello);
    }
    else if (!strcmp(command->type, "$NickList"))
    {
        command->alloc = command->message = dc_hub_to_utf8(hub, command->message);
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_nick_list);
    }
    else if (!strcmp(command->type, "$MyINFO"))
    {
        command->alloc = command->message = dc_hub_to_utf8(hub, command->message);
/*     DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG); */
        DC_CALL(handle_myinfo);
    }
    else if (!strcmp(command->type, "$Quit"))
    {
        command->alloc = command->message = dc_hub_to_utf8(hub, command->message);
/*     DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG); */
        DC_CALL(handle_quit);
    }
    else if (!strcmp(command->type, "$OpList"))
    {
        command->alloc = command->message = dc_hub_to_utf8(hub, command->message);
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_op_list);
    }
    else if (!strcmp(command->type, "$To:"))
    {
        command->alloc = command->message =
            dc_unescape(dc_hub_to_utf8(hub, command->message));
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        //    print_raw("TO", command->message, strlen(command->message));
        DC_CALL(handle_to);
    }
    else if (!strcmp(command->type, "$Search"))
    {
        //FIXME: utf8?
        //    command->alloc = command->message = dc_hub_to_utf8(hub, command->message);
        //    DC_REC("DC_PROTO", G_LOG_LEVEL_INFO);
        DC_CALL(handle_search);
    }
    else if (!strcmp(command->type, "$ConnectToMe"))
    {
        command->alloc = command->message = dc_hub_to_utf8(hub, command->message);
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_connect_to_me);
    }
    else if (!strcmp(command->type, "$SR"))
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_search_result);
    }
    else if (!strcmp(command->type, "$RevConnectToMe"))
    {
        command->alloc = command->message = dc_hub_to_utf8(hub, command->message);
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_rev_connect);
    }
    else if (!strcmp(command->type, "$UserIP"))
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_user_ip);
    }
    else if (!strcmp(command->type, "$GetPass"))
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_get_pass);
    }
    else if (!strcmp(command->type, "$BadPass"))
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        return "bad password";
    }
    else if (!strcmp(command->type, "$ValidateDenide"))  // wrong spelling is correct
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        return "login denied";
    }
    else if (!strcmp(command->type, "$LogedIn"))
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG);
        // not necessarily op
        // hub->flags |= DC_HUB_OPERATOR;
    }
/*     else if (!strcmp(command->type, "$HubTopic")) */
/*     { */
/*         DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG); */
/*         DC_CALL(handle_gobal_message); */
/*     } */
/*     else if (!strcmp(command->type, "$MultiSearch")) */
/*     { */
/*         DC_REC("DC_PROTO", G_LOG_LEVEL_DEBUG); */
/*         DC_CALL(handle_search); */
/*     } */
    else
    {
        DC_REC("DC_PROTO", G_LOG_LEVEL_WARNING);
    }
    return NULL;
}

const char* dc_handle_udp_message(udp_socket_t* socket, dc_message_t* command, ip_t ip)
{
    dc_hub_t* hub = NULL;

    UDP_IP = ip;

    if (!command->type)
    {
        DC_REC("DC_UDP", G_LOG_LEVEL_WARNING);
    }
    else if (!strcmp(command->type, "$SR"))
    {
        DC_REC("DC_UDP", G_LOG_LEVEL_DEBUG);
        DC_CALL(handle_search_result);
    }
    else
    {
        DC_REC("DC_UDP", G_LOG_LEVEL_WARNING);
    }
    return NULL;
}

/*
  DC_PEER    | R:          $Key [ÑÀ° A Ñ±±ÀÀ0ap 1q¡/%DCN096%/]
  *** DC_PEER *** : R:   $GetListLen []
  DC_PEER    | R:          $Get [MyList.bz2$1]
  DC_PEER    | S:        $Error [File not found]
  SOCK       | ending 0x84a4a68 : (file no shared) status:7 fd:17
*/
#undef HANDLER
#undef DC_CALL
#undef CHECK
