/* Copyright (C) 2005-2006 sgop@users.sourceforge.net.
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.3 $
 * $Date: 2007/09/05 14:34:07 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#ifndef WIN32
#include <unistd.h>
#endif
#include <stdio.h>
#include <string.h>

#include <libxml/parser.h>
#include <glib.h>

#include "lcore/l_base.h"
#include "lcore/buffer.h"
#include "lcore/zbuffer.h"
#include "lcore/version.h"
#include "lcore/library.h"
#include "dc_share.h"


typedef struct {
    module_t _module;
    gboolean dirty;
    GHashTable* files;
} dc_share_module_t;

static dc_share_module_t* DcShareModule = NULL;

void dc_write_share_buffer(const char* name, buffer_t* buffer) {
    char* fname = l_printf(SDIR("%s") SDIR("dc") "%s",
                           lopster_get_home(), name);
    FILE* file = fopen(fname, "wb");

    if (!file) {
        g_log("DC_SHARE", G_LOG_LEVEL_WARNING,
              "could not save to file '%s'\n", fname);
        return;
    }
    if (fwrite(buffer->data, 1, buffer->size, file) != buffer->size) {
        g_log("DC_SHARE", G_LOG_LEVEL_WARNING,
              "failed to write file '%s'\n", fname);
    }
    fclose(file);
}

static void dc_save_node(xmlNodePtr xnode, file_node_t* node) {
    xmlNodePtr xsub;
    file_node_t* dlist;

    for (dlist = node; dlist; dlist = dlist->next) {
        if (dlist->fci[MEDIA_ANY][3] == 0) continue;
        if (dlist->child) {
            xsub = xmlNewChild(xnode, NULL, "Directory", NULL);
            xmlNewProp(xsub, "Name", OBJ(dlist)->name);
            dc_save_node(xsub, dlist->child);
        }
    }
    for (dlist = node; dlist; dlist = dlist->next) {
        if (dlist->fci[MEDIA_ANY][3] == 0) continue;
        if (dlist->file) {
            attribute_t* attr = attr_find(ATTRC(dlist->file), "file", "urn:tree:tiger");
            xsub = xmlNewChild(xnode, NULL, "File", NULL);
            xmlNewProp(xsub, "Name", dlist->file->filename);
            xmlNewProp(xsub, "Size", l_printf(FF, dlist->file->size));
            if (attr) xmlNewProp(xsub, "TTH", attr_xml_print(attr));
            //FIXME: add tth
        }
    }
}

static void dc_create_userlist_xml(file_tree_t* tree) {
    xmlDocPtr xdoc;
    xmlNodePtr xroot;
    int written;
    compress_t* deflate;
    buffer_t* XmlList;
    buffer_t* BzXmlList;

    xdoc = xmlNewDoc("1.0");
    xroot = xmlNewNode(NULL, "FileListing");
    xmlDocSetRootElement(xdoc, xroot);

    xmlNewProp(xroot, "Version", "1");
    xmlNewProp(xroot, "Generator", FULL_VERSION);
    if (tree && tree->files) {
        // if there is only one toplevel node, make it a base
        if (!tree->files->next) {
            xmlNewProp(xroot, "Base", OBJ(tree->files)->name);
            dc_save_node(xroot, tree->files->child);
        } else {
            dc_save_node(xroot, tree->files);
        }
    }
  
    XmlList = buffer_new(0);
    if (XmlList->data) xmlFree(XmlList->data);
    XmlList->data = NULL;
    XmlList->consumed = 0;
    XmlList->size = 0;
    XmlList->max = 0;
    xmlDocDumpFormatMemoryEnc(xdoc, (xmlChar**)(&(XmlList->data)),
                              &written, "utf-8", 1);
    if (written >= 0) XmlList->size = written;
    xmlFreeDoc(xdoc);

    dc_write_share_buffer("files.xml", XmlList);

    // bzip2
    BzXmlList = buffer_new(2048);

    deflate = compress_new(COMPRESS_BZIP2, TRUE, XmlList, BzXmlList);
    if (deflate) {
        compress_action(deflate, TRUE);
        compress_destroy(deflate);
    }

    dc_write_share_buffer("files.xml.bz2", BzXmlList);

    buffer_destroy(XmlList);
    buffer_destroy(BzXmlList);
  
}

static void dc_save_node_plain(buffer_t* buffer, file_node_t* node, int indent) {
    unsigned len;
    file_t* file;

    while (node) {
        if (node->fci[1][3] == 0) {
            node = node->next;
            continue;
        }
        file = node->file;
        len = strlen(OBJ(node)->name);
        buffer_need_space(buffer, len+100);

        memset(buffer_write_pos(buffer), '\x9', indent);
        buffer->size += indent;
        if (file) {
            buffer->size += 
                sprintf(buffer_write_pos(buffer),
                        "%s|"FF"\xd\xa", file->filename, file->size);
        } else if (node->child) {
            buffer->size += 
                sprintf(buffer_write_pos(buffer), "%s\xd\xa", OBJ(node)->name);
            dc_save_node_plain(buffer, node->child, indent+1);
        }
        node = node->next;
    }

}

static void dc_create_userlist_dc(file_tree_t* tree) {
    compress_t* deflate;
    buffer_t* DcList = buffer_new(2048);
    buffer_t* DcListPack;

    if (tree && tree->files)
        dc_save_node_plain(DcList, tree->files, 0);
  
    DcListPack = buffer_new(2048);

    deflate = compress_new(COMPRESS_HUFFMAN, TRUE, DcList, DcListPack);
    if (deflate) {
        compress_action(deflate, TRUE);
        compress_destroy(deflate);
    }
    DcList->consumed = 0; // compressor has virtually consumed

    dc_write_share_buffer("MyList.DcLst", DcListPack);
    DcListPack->size = 0;
  
    deflate = compress_new(COMPRESS_BZIP2, TRUE, DcList, DcListPack);
    if (deflate) {
        compress_action(deflate, TRUE);
        compress_destroy(deflate);
    }
    DcList->consumed = 0; // compressor has virtually consumed
  
    dc_write_share_buffer("MyList.bz2", DcListPack);
  
    buffer_destroy(DcList);
    buffer_destroy(DcListPack);
}

static void dc_create_userlists() {
    file_tree_t* tree = FILE_TREE(library_get_current());
    dc_create_userlist_dc(tree);
    dc_create_userlist_xml(tree);
    DcShareModule->dirty = FALSE;
}

file_t* dc_get_special_file(const char* name) {
    char* fname;
    struct stat stats;

    if (!name) return NULL;
    fname = g_strdup_printf
        (SDIR("%s") SDIR("dc") "%s", lopster_get_home(), name);
  
    if (DcShareModule->dirty) dc_create_userlists();

    if (stat(fname, &stats) < 0) {
        g_free(fname);
        return NULL;
    } else {
        file_t* file = file_new();
        file_set_origname(file, g_strdup(fname));
        file_set_locname(file, fname);
        file->size = stats.st_size;
        return file;
    }
}


static void _create_lists() {
    g_assert(DcShareModule);
    DcShareModule->dirty = TRUE;
}

static void dc_file_added(file_t* file)
{
    attribute_t* attr = attr_find(ATTRC(file), "file", "urn:tree:tiger");
    const char* str;

    if (!attr) return;
    str = attr_xml_print(attr);

    if (!DcShareModule->files)
    {
        DcShareModule->files =
            g_hash_table_new_full
            ((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal,
             (GDestroyNotify)g_free, (GDestroyNotify)NULL);
    }
    else if (g_hash_table_lookup(DcShareModule->files, str))
    {
        //ops, file with same hash??
        return;
    }
    g_hash_table_insert(DcShareModule->files, g_strdup(str), file);
    _create_lists();
}

static char* TheFileHash = NULL;
static void _find_file(char* key, file_t* file, file_t* which)
{
    if (file == which) TheFileHash = key;
}

static void dc_file_removed(file_t* file)
{
    if (!DcShareModule->files) return;

    if (!file)
    {
        g_hash_table_destroy(DcShareModule->files);
        DcShareModule->files = NULL;
    }
    else
    {
        TheFileHash = NULL;
        g_hash_table_foreach(DcShareModule->files, (GHFunc)_find_file, file);
        if (TheFileHash)
        {
            g_hash_table_remove(DcShareModule->files, TheFileHash);
            TheFileHash = NULL;
        }
    }
    _create_lists();
}

static void dc_file_changed(file_t* file)
{
    if (file)
    {
        dc_file_removed(file);
        dc_file_added(file);
    }
}

file_t* dc_file_find_by_tth(const char* tth)
{
    file_t* file;
    
    if (!DcShareModule->files) return NULL;
    file = g_hash_table_lookup(DcShareModule->files, tth);
    return file;
}

static void dc_shares_init()
{
    library_register_actions((LibNodeFunc)dc_file_added,
                             (LibNodeFunc)dc_file_removed,
                             (LibNodeFunc)dc_file_changed);
}

static void dc_shares_run() {
    dc_create_userlists();  //FIXME: remove later.
}
static void dc_shares_exit() {
}

static void dc_share_module_init(dc_share_module_t* module) {
    module_init(MODULE(module), "Dc Share Module",
                (ModuleFunc)dc_shares_init,
                (ModuleFunc)dc_shares_run,
                (ModuleFunc)dc_shares_exit);
  
    module->dirty = TRUE;
    module->files = NULL;

    //  module_require(MODULE(module), pref_module_instance);
    module_require(library_module_instance);
    module_require(base_module_instance);
    //  module_require(exec_module_instance);
}


module_t* dc_share_module_instance() {
    if (!DcShareModule) {
        DcShareModule = g_malloc0(sizeof(*DcShareModule));
        dc_share_module_init(DcShareModule);
    }
    return MODULE(DcShareModule);
}

