/* Copyright (C) 2005-2006 sgop@users.sourceforge.net.
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2007/09/05 14:34:07 $
 * $Author: sid4u $
 */

#ifndef WIN32
#include <dlfcn.h>
#endif

#include "lcore/plugin.h"
#include "lcore/dynamic.h"
#include "lcore/ui.h"

#include "dc_ui.h"

ui_dc_t DCUI;

typedef struct {
    module_t _module;
    void (*init)(ui_dc_t*);
    void (*run)(void);
    void (*exit)(void);
} dc_ui_module_t;

static dc_ui_module_t* DcUIModule = NULL;

#define CHECK_SYMBOL(sym)                                               \
    if (!(sym)) {                                                       \
        g_log("DC", G_LOG_LEVEL_WARNING, "could not find symbol '%s'", #sym); \
    }

static void dummy_func() {}

void dc_ui_load_syms()
{
    const char* filename;
    GModule* handle;

    DCUI.hub_edit = dummy_func;
    DCUI.hub_set_status = dummy_func;
    DCUI.user_appeared = dummy_func;
    DCUI.user_vanished = dummy_func;
  
    filename = ui_find_library_file("dc");
    if (!filename)
    {
        g_log("DC", G_LOG_LEVEL_INFO, "starting without interface (could not find lib)");
        return;
    }

    handle = lopster_dll_open(filename, G_MODULE_BIND_LAZY|G_MODULE_BIND_LOCAL);
    if (!handle)
    {
        g_log("DC", G_LOG_LEVEL_WARNING, "%s", g_module_error());
        g_log("DC", G_LOG_LEVEL_INFO, "starting plugin without interface (could not load lib)");
        return;
    }
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_dc_init", (void**)&DcUIModule->init));
    CHECK_SYMBOL(g_module_symbol(handle, "ui_dc_run", (void**)&DcUIModule->run));
    CHECK_SYMBOL(g_module_symbol(handle, "ui_dc_exit", (void**)&DcUIModule->exit));
}

void DCUI_hub_edit(dc_hub_t* hub) {
    if (DCUI.hub_edit) DCUI.hub_edit(hub);
}
void DCUI_hub_set_status(dc_hub_t* hub, const char* status) {
    if (DCUI.hub_set_status) DCUI.hub_set_status(hub, status);
}
void DCUI_user_appeared(dc_cp_t* page, dc_user_t* user) {
    if (DCUI.user_appeared) DCUI.user_appeared(page, user);
}
void DCUI_user_vanished(dc_cp_t* page, dc_user_t* user) {
    if (DCUI.user_vanished) DCUI.user_vanished(page, user);
}


static void dc_ui_init()
{
    DcUIModule->init(&DCUI);
}

static void dc_ui_run()
{
    DcUIModule->run();
}

static void dc_ui_exit()
{
    DcUIModule->exit();
}


static void dc_ui_module_init(dc_ui_module_t* module)
{
    module_init(MODULE(module), "Dc User Interface Module",
                (ModuleFunc)dc_ui_init, (ModuleFunc)dc_ui_run, dc_ui_exit);

    module->init = dummy_func;
    module->run = dummy_func;
    module->exit = dummy_func;

    module_require(ui_module_instance);
    module_require(dynamic_module_instance);

    dc_ui_load_syms();
}

module_t* dc_ui_module_instance()
{
    if (!DcUIModule)
    {
        DcUIModule = g_malloc0(sizeof(*DcUIModule));
        dc_ui_module_init(DcUIModule);
    }
    return MODULE(DcUIModule);
}
#undef CHECK_SYMBOL
