/* Copyright (C) 2005-2006 sgop@users.sourceforge.net.
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2006/05/07 16:51:57 $
 * $Author: sgop $
 */

#include <string.h>
#include <stdlib.h>

#include "lcore/file_segment.h"
#include "dc_upload.h"
#include "dc_proto.h"

static GList* Uploads = NULL;

#define HANDLER(f) static const char* f (dc_upload_t* upload, char* data)
#define DC_CALL(f) f(upload, command->message)

static void dc_upload_queue(dc_upload_t* upload, int pos, int slots) {
    //FIXME
    if (!socket_connected(SOCK(upload))) return;
    dc_send(BSOCK(upload), FALSE, "$MaxedOut");
}

static gboolean dc_upload_start(dc_upload_t* upload) {

    if (!socket_connected(SOCK(upload))) return FALSE;

    if (upload->get_block) {
        // prepare segment now and start sending
        const char* res = upload_prepare_segment(UPLOAD(upload));
        segment_t* seg = UPLOAD(upload)->cur_segment;
        if (!res) return FALSE;
        dc_send(BSOCK(upload), FALSE, "$Sending "FF, seg->stop - seg->start);
    } else {
        // wait for $Send, prepare segment later
        dc_send(BSOCK(upload), FALSE, "$FileLength "FF, UPLOAD(upload)->file->size);
    }
    return TRUE;
}

static gboolean dc_upload_end(dc_upload_t* upload, const char* message) {
    return upload_end(UPLOAD(upload), message);
}

static void dc_upload_destroy(dc_upload_t* upload) {
    Uploads = g_list_remove(Uploads, upload);
    upload_destroy(UPLOAD(upload));
}

static void dc_upload_init(dc_upload_t* upload, dc_user_t* user, file_t* file) {
    upload_init(UPLOAD(upload), file, USER(user));
    object_add_type(OBJ(upload), "DcUpload");
    OBJ(upload)->destroy = (ObjectFunc)dc_upload_destroy;
    socket_set_end(SOCK(upload), (SockEndFunc)dc_upload_end);

    UPLOAD(upload)->queue = (UploadQueueFunc)dc_upload_queue;
    UPLOAD(upload)->start = (UploadStartFunc)dc_upload_start;
    Uploads = g_list_append(Uploads, upload);
}

dc_upload_t* dc_upload_new(dc_user_t* user, file_t* file) {
    dc_upload_t* upload = g_malloc0(sizeof(*upload));
    dc_upload_init(upload, user, file);
    return upload;
}

void dc_upload_set_offset(dc_upload_t* upload, file_size_t offset, file_size_t stop) {
    if (stop == 0) {
        upload_add_segment(UPLOAD(upload), offset, UPLOAD(upload)->file->size, FALSE);
        upload->get_block = FALSE;
    } else {
        upload_add_segment(UPLOAD(upload), offset, stop, FALSE);
        upload->get_block = TRUE;
    }
}

HANDLER(handle_send) {
    const char* res = upload_prepare_segment(UPLOAD(upload));
    if (res) return res;
    b_socket_start_sending(BSOCK(upload));
    return NULL;
}

static const char*
dc_handle_upload_message(dc_upload_t* upload, dc_message_t* command) {

    if (!command->type) {
        DC_REC("DC_UPLOAD", G_LOG_LEVEL_WARNING);
    } else if (!strcmp(command->type, "$Send")) {
        DC_REC("DC_UPLOAD", G_LOG_LEVEL_DEBUG);
        return DC_CALL(handle_send);
    } else {
        DC_REC("DC_UPLOAD", G_LOG_LEVEL_WARNING);
    }
    return NULL;
}

static const char* dc_upload_consume(dc_upload_t* upload) {
    buffer_t* buffer = b_socket_get_in_buffer(BSOCK(upload));
    dc_message_t* result;

    if (!buffer) return NULL;

    while ((result = dc_consume_message(buffer)) != NULL) {
        const char* message = dc_handle_upload_message(upload, result);
        dc_message_destroy(result);
        if (message) return message;
    }
    buffer_consume(buffer, 0);
    return NULL;
}

const char* dc_upload_set_connection(dc_upload_t* upload, dc_peer_t* peer) {

    if (socket_connected(SOCK(upload))) return "upload already running";
  
    tcp_socket_move(TSOCK(upload), TSOCK(peer), TRUE);

    upload->features = peer->features;

    upload_renew(UPLOAD(upload));

    if (!upload->get_block) {
        // wait for $Send before sending.
        b_socket_set_consume(BSOCK(upload), (BConsumeInputFunc)dc_upload_consume);
        b_socket_start_receiving(BSOCK(upload), 10240);
    }

    return "dc_peer->dc_upload";
}

#undef HANDLER
#undef DC_CALL
