/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.5 $
 * $Date: 2007/04/29 09:27:54 $
 * $Author: sgop $
 */

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "lcore/chat.h"
#include "lcore/preferences.h"
#include "lcore/lopster.h"
#include "lcore/utils.h"
#include "lcore/l_i18n.h"
#include "lcore/action_item.h"
#include "lcore/gtk2_ui/ui_chat.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "lcore/gtk2_ui/ui_layout.h"
#include "lcore/gtk2_ui/ui_treeview.h"
#include "ui_dc_chat.h"
#include "dc_network.h"
#include "dc.h"
#include "dc_chat.h"
#include "dc_users.h"

typedef struct {
    chat_gui_t _data;
    GtkListStore* online_store;
    GtkTreeViewColumn* col[3];
    GtkWidget* users;
} dc_public_gui_t;

void ui_dc_chat_init() {}

//////////////////////////////////////////////////////////////

static void ui_dc_destroy_user_gui(dc_user_t* user, chat_page_t* page)
{
    char* id = l_printf("ChatUser_%x", page);
    GtkTreeIter* iter = object_get_data(OBJ(user), id);

    // gui data is not necessarily expected (for public pages it is actually)
    // so check iter before free, to avoid warning message of object class
    if (iter) object_free_data_common(OBJ(user), id);
}

static void ui_dc_public_remove(chat_page_t* page)
{
    dc_public_gui_t* ndata;
    chat_gui_t* data;

    ndata = object_get_data(OBJ(page), "ChatPage");
    data = (chat_gui_t*)ndata;
    if (!data) return;

    // we need to free gui data of users since they are destroyed after
    // this function is called (this function destroys the user's gui).
    // this is not really nice design, think about it
    g_list_foreach(DCP(page)->users, (GFunc)ui_dc_destroy_user_gui, page);

    ui_layout_unmonitor("dc:chat:col:user:size", ndata->col[0]);
    ui_layout_unmonitor("dc:chat:col:user:vis", ndata->col[0]);
    ui_layout_unmonitor("dc:chat:col:shared:size", ndata->col[1]);
    ui_layout_unmonitor("dc:chat:col:shared:vis", ndata->col[1]);
    ui_layout_unmonitor("dc:chat:col:speed:size", ndata->col[2]);
    ui_layout_unmonitor("dc:chat:col:speed:vis", ndata->col[2]);
    ui_layout_unmonitor("dc:chat:paned:pos", data->toplevel);
    ui_layout_unmonitor("dc:chat:users:vis", ndata->users);
    
    ui_chat_page_remove(page);
}

/* static void on_encoding_changed(const char* charset, dc_hub_t* hub) { */
/*   //FIXME:  dc_hub_set_encoding(hub); */
/*   g_free(hub->charset); */
/*   hub->charset = g_strdup(charset); */
/* } */

static gint sort_users_by_bytes(GtkTreeModel* model,
                                GtkTreeIter* a, GtkTreeIter* b)
{
    dc_user_t* u1;
    dc_user_t* u2;
  
    gtk_tree_model_get(model, a, 0, &u1, -1);
    gtk_tree_model_get(model, b, 0, &u2, -1);
  
    if (u1->bytes < u2->bytes) return -1;
    else if (u1->bytes > u2->bytes) return 1;
    else return 0;
}


static void ui_dc_create_public_page(dc_cp_t* page)
{
    GtkWidget* toplevel;
    GtkWidget* text;
    dc_hub_t* hub;
    dc_public_gui_t* ndata;
    chat_gui_t* data;
    GtkWidget* scrolledwindow;
    GtkWidget* vbox;
    GtkListStore* store;
    GtkWidget* tv;
    GtkTreeViewColumn* column;
    GtkTreeSelection *select;

    ndata = g_malloc0(sizeof(dc_public_gui_t));
    data = (chat_gui_t*)ndata;

    object_set_data(OBJ(page), "ChatPage", ndata);
    hub = page->hub;
  
    toplevel = gtk_hpaned_new();
    gtk_widget_show(toplevel);
    data->toplevel = toplevel;
    ui_layout_monitor("dc:chat:paned:pos", toplevel);

    data->no_buffers = 1;
    data->buffer = g_malloc0(sizeof(GtkTextBuffer)*data->no_buffers);
    data->view = g_malloc0(sizeof(GtkTextView)*data->no_buffers);

    vbox = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(vbox);
    gtk_paned_pack1(GTK_PANED(toplevel), vbox, TRUE, TRUE);
  
    //
    scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(scrolledwindow);
    gtk_box_pack_start(GTK_BOX(vbox), scrolledwindow, TRUE, TRUE, 0);
  
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(scrolledwindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type
        (GTK_SCROLLED_WINDOW (scrolledwindow), GTK_SHADOW_IN);

    data->buffer[0] = ui_chat_new_buffer();
    text = ui_chat_new_text_view(data->buffer[0]);
    gtk_widget_show(text);
    gtk_container_add(GTK_CONTAINER(scrolledwindow), text);
    data->view[0] = GTK_TEXT_VIEW(text);

    
    //////////
    // RIGHT USER LIST
    ndata->users = scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(scrolledwindow);
    ui_layout_monitor("dc:chat:users:vis", scrolledwindow);
    gtk_paned_pack2(GTK_PANED(toplevel), scrolledwindow, FALSE, TRUE);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW (scrolledwindow),
                                        GTK_SHADOW_IN);
    
    store = gtk_list_store_new(6,
                               G_TYPE_POINTER,  // 0 pointer to user object
                               G_TYPE_STRING,   // 1 user name
                               G_TYPE_STRING,   // 2 user shares
                               G_TYPE_STRING,   // 3 user speed
                               G_TYPE_OBJECT,   // 4 pixmap
                               G_TYPE_BOOLEAN   // 5 opped?
                              );
    ndata->online_store = store;

    gtk_tree_sortable_set_sort_func
        (GTK_TREE_SORTABLE(store),
         2, (GtkTreeIterCompareFunc)sort_users_by_bytes,
         NULL, NULL);
    
    tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    //  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
    gtk_container_add (GTK_CONTAINER (scrolledwindow), tv);
    gtk_widget_show(tv);
    gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tv), TRUE);
    //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
    
    ndata->col[0] = column = ui_tv_column_new
        (_("User"),
         gtk_cell_renderer_pixbuf_new(), "pixbuf", 4, FALSE,
         gtk_cell_renderer_text_new(), "text", 1, TRUE,
         NULL);
    ui_tv_config_default_col(column, 1, "dc:chat:col:user", FALSE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    
    ndata->col[1] = column = ui_tv_column_new
        (_("Shared"), gtk_cell_renderer_text_new(), "text", 2, TRUE, NULL);
    ui_tv_config_default_col(column, 2, "dc:chat:col:shared", FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
    
    ndata->col[2] = column = ui_tv_column_new
        (_("Speed"), gtk_cell_renderer_text_new(), "text", 3, TRUE, NULL);
    ui_tv_config_default_col(column, 3, "dc:chat:col:speed", TRUE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    
    select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
    {
        action_item_t* actions = 
            action_item_search("Online dc users actions");
        ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &actions, page);
    }
    
    ui_chat_page_add(PAGE(page),
                     NULL, NULL, 
                     ui_create_show_hide_label(_("show users"), _("hide users"), ndata->users),
/*                    ui_create_charset_control(hub->charset, (CharsetFunc)on_encoding_changed, hub), */
                     NULL);
    
    ui_tv_config_line_marker(GTK_TREE_VIEW(tv), "chat:user:opped", 5);

    object_notify_d(OBJ(page), (ObjectFunc)ui_dc_public_remove, NULL);
}


void ui_dc_create_page(dc_cp_t* page)
{
    if (page->type == DC_PAGE_PRIVATE)
    {
        ui_chat_create_simple_page(PAGE(page));
    }
    else if (page->type == DC_PAGE_PUBLIC)
    {
        ui_dc_create_public_page(page);
    }
    else
    {
        g_log("GTK_DC", G_LOG_LEVEL_WARNING,
              "unknown chat page found, cannot create ui");
    }
}


static void ui_dc_user_hide(dc_user_t* user, dc_cp_t* dcp)
{
    GtkTreeIter* iter = object_get_data(OBJ(user), l_printf("ChatUser_%x", dcp));
    
    if (iter)
    {
        dc_public_gui_t* data = object_get_data(OBJ(dcp), "ChatPage");

        if (data && data->online_store)
            gtk_list_store_remove(data->online_store, iter);

        object_unnotify_o(OBJ(user), (ObjectFunc)ui_dc_user_hide, OBJ(dcp));
        object_free_data_common(OBJ(user), l_printf("ChatUser_%x", dcp));
    }
}

static void ui_dc_user_update(dc_user_t* user, dc_cp_t* dcp)
{
    dc_public_gui_t* data = object_get_data(OBJ(dcp), "ChatPage");
    GtkTreeIter* iter = object_get_data(OBJ(user), l_printf("ChatUser_%x", dcp));

    if (!iter || !data || !data->online_store) return;
  
    gtk_list_store_set(data->online_store, iter,
                       0, user,
                       1, OBJ(user)->name,
                       2, print_filesize(&user->bytes),
                       3, user->linespeed,
                       5, g_list_find(user->opped, dcp->hub),
                       -1);
}

static void ui_dc_user_show(dc_user_t* user, dc_cp_t* page)
{
    dc_public_gui_t* data;
    GtkTreeIter* iter;
    
    if (object_get_data(OBJ(user), l_printf("ChatUser_%x", page))) return;

    data = object_get_data(OBJ(page), "ChatPage");
    if (!data || !data->online_store) return;
    
    object_set_data(OBJ(user),l_printf("ChatUser_%x", page), 
                    iter = g_malloc(sizeof(*iter)));
    gtk_list_store_append(data->online_store, iter);
    
    object_notify_o(OBJ(user), (ObjectFunc)ui_dc_user_hide, OBJ(page));
    
    ui_dc_user_update(user, page);
  /*
  object_monitor_add(OBJ(user), D_DC_CU,
		     (ObjectFunc)ui_dc_user_update, dcp);
  */
}

void ui_dc_user_appeared(dc_cp_t* dcp, dc_user_t* user)
{
    ui_dc_user_show(user, dcp);
}

void ui_dc_user_vanished(dc_cp_t* dcp, dc_user_t* user)
{
    ui_dc_user_hide(user, dcp);
}
