/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.1 $
 * $Date: 2006/02/04 16:41:04 $
 * $Author: sgop $
 */

#include <string.h>

#include <gtk/gtk.h>

#include "lcore/buddy.h"
#include "lcore/attribute.h"
#include "lcore/l_i18n.h"
#include "lcore/gtk2_ui/ui_buddy.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "dc.h"
#include "dc_hotlist.h"
#include "dc_chat.h"
#include "dc_users.h"
#include "dc_browse.h"

static GtkEntry* NickEntry = NULL;
static int Row = 0;

static GtkWidget* ui_dc_buddy_edit(buddy_t* buddy) {
  GtkWidget* table;
  GtkWidget* entry;
  GtkWidget* item;
  attribute_t* attr;

  table = gtk_table_new(1, 1, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);

  entry = gtk_entry_new();
  NickEntry = GTK_ENTRY(entry);
  attr = attr_find(ATTRC(buddy), "buddy", "DcNick");
  if (attr) gtk_entry_set_text(NickEntry, attr_get_ptr(attr, char));
  g_signal_connect(G_OBJECT(entry), "destroy",
		   G_CALLBACK(gtk_widget_destroyed), &NickEntry);

  item = ui_create_headed_item("monitor nicks (space separated list)", entry);
  gtk_table_attach(GTK_TABLE(table), item, 0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

  gtk_widget_show_all(table);
  return table;
}

static void ui_dc_buddy_apply(buddy_t* buddy) {
  const char* nick;

  if (!buddy || !NickEntry) return;

  nick = gtk_entry_get_text(NickEntry);

  if (nick && *nick) {
    attr_store(ATTRC(buddy), "buddy", "DcNick", nick);
  } else {
    attribute_t* attr = attr_find(ATTRC(buddy), "buddy", "DcNick");
    if (attr) attr_remove(ATTRC(buddy), attr);
  }
}

static void on_buddy_chat(GObject* item, dc_user_t* user) {
  dc_hub_t* hub = g_object_get_data(item, "hub");
  if (hub && user) dc_cp_new_private(hub, OBJ(user)->name);
}
/*
static gboolean
on_buddy_whois(GObject* box, GdkEventButton* event, char* nick) {
  dc_hub_t* hub;

  (void)event;
  hub = g_object_get_data(box, "network");
  g_assert(hub && nick);
  //  dc_whois_user(nick, hub, DC_WHOIS_WINDOW, FALSE);
  return TRUE;
}
*/
static void on_buddy_browse(GObject* item, dc_user_t* user) {
  dc_hub_t* hub = g_object_get_data(item, "hub");
  if (user) dc_browse_user(user, hub);
}

static GtkWidget* ui_create_chat_popup(dc_user_t* user) {
  GtkWidget* popup;
  GList* dlist;
  
  popup = gtk_menu_new();
  for (dlist = user->hubs; dlist; dlist = dlist->next) {
    dc_hub_t* hub = dlist->data;
    GtkWidget* item = gtk_menu_item_new_with_label(hub->short_name);
    
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup), item);

    g_object_set_data(G_OBJECT(item), "hub", hub);
    g_signal_connect(G_OBJECT(item), "activate",
		     G_CALLBACK(on_buddy_chat), user);
  }
  return popup;
}
static GtkWidget* ui_create_browse_popup(dc_user_t* user) {
  GtkWidget* popup;
  GList* dlist;
  
  popup = gtk_menu_new();
  for (dlist = user->hubs; dlist; dlist = dlist->next) {
    dc_hub_t* hub = dlist->data;
    GtkWidget* item = gtk_menu_item_new_with_label(hub->short_name);

    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup), item);

    g_object_set_data(G_OBJECT(item), "hub", hub);
    g_signal_connect(G_OBJECT(item), "activate",
		     G_CALLBACK(on_buddy_browse), user);
  }

  return popup;
}

static gboolean ui_dc_user_show(dc_user_t* user, GtkTable* table) {
  GtkWidget* hbox;
  GtkWidget* label;
  user_t* u = USER(user);

  label = gtk_label_new(u->print(u));
  gtk_widget_set_name(label, "OnlineLabel");
  if (u->online) gtk_widget_set_state(label, GTK_STATE_ACTIVE);
  else gtk_widget_set_state(label, GTK_STATE_NORMAL);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, Row, Row+1,
                   GTK_FILL, GTK_FILL, 0, 0);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, Row, Row+1,
                   GTK_FILL, GTK_FILL, 0, 0);
  label = gtk_label_new(u->online?_("is online"):_("is not active"));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
  
  if (u->online) {
    GtkWidget* button =
      ui_create_popup_button(gtk_label_new(_("chat on")), POS_LEFT_DOWN,
                             (PopupFunction)ui_create_chat_popup, user);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    /*    
    button =
      ui_create_popup_button(gtk_label_new("whois on"), POS_RIGHT_DOWN,
			     (PopupFunction)ui_create_whois_popup, user);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    */    
    button =
      ui_create_popup_button(gtk_label_new(_("browse on")), POS_LEFT_DOWN,
                             (PopupFunction)ui_create_browse_popup, user);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
  }
  Row++;
  return FALSE;
}

static GtkWidget* ui_dc_buddy_show(buddy_t* buddy) {
  GtkWidget* table;
  
  g_assert(buddy);
  if (!buddy_has_user(buddy, DC_ID)) return NULL;

  table = gtk_table_new(1, 2, FALSE);
  gtk_widget_show(table);
/*   gtk_container_set_border_width(GTK_CONTAINER(table), 5); */
  //  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table), 2);

  Row = 0;
  buddy_foreach_user(buddy, DC_ID, (UserForeach)ui_dc_user_show, table);
  
  return table;
}

void ui_dc_hotlist_init() {
  ui_buddy_register_funcs(DC_ID,
                          ui_dc_buddy_edit,
                          ui_dc_buddy_apply,
                          ui_dc_buddy_show
                          );
}
