/* Copyright (C) 2005-2006 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2006/05/07 16:51:57 $
 * $Author: sgop $
 */

#include <stdio.h>
#include <string.h>

#include <glib.h>

#include "lcore/files.h"
#include "lcore/library.h"
#include "lcore/l_io.h"
#include "tth.h"

#define BUF_SIZE        256*1024

void file_hash_tth(thread_t* thread, lib_node_t* node) {
  file_t* file = FILE_NODE(node)->file;
  FILE* fd;
  char buffer[BUF_SIZE];
  file_size_t size;
  file_size_t progress;
  tth_t tth;
  void* hash;
  char* temp;
  attribute_t* attr;

  file = FILE_NODE(node)->file;
  attr = attr_find(ATTRC(file), "file", "urn:tree:tiger");
  if (attr && attr->refreshed >= node->mod_time) return;

  temp = locale_from_utf8(file->localname);
  if (!temp) return;

  fd = fopen(temp, "rb");
  g_free(temp);

  if (!fd) return;

  thread_set_description(thread, "tiger tree hashing...");

  FSEEK(fd, 0, SEEK_END);
  size = ftell(fd);
  progress = 0;

  FSEEK(fd, 0, SEEK_SET);

  tth_init(&tth);

  while (progress < size) {
    size_t r = size - progress;
    int res;

    if (r > BUF_SIZE) r = BUF_SIZE;
    res = fread(buffer, r, 1, fd);
    if (res == 0) {
      g_warning("** error reading %d bytes from file", (int)r);
      fclose(fd);
      return;
    }
    
    tth_update(&tth, buffer, r);
    progress += r;
    
    if ((progress % (1024*1024)) == 0)
      thread_set_progress(thread, (double)progress/(double)size*100.);
    if (thread_interrupted(thread)) break;
  }
  fclose(fd);

  if (thread_interrupted(thread)) return;

  hash = tth_finalize(&tth);

  thread_lock(thread);
  attr_store(ATTRC(file), "file", "urn:tree:tiger", hash, TIGER_HASH_SIZE);
  thread_unlock(thread);

}
