/* Copyright (C) 2005 sgop@users.sourceforge.net
/* Copyright (C) 2007 Lopster Team. This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2007/11/14 12:42:47 $
 * $Author: sid4u $
 */

#include "lcore/qt4_ui/ModuleWidget.hpp"
#include "lcore/qt4_ui/ui_main.hpp"

#include "ui_dc_chat.hpp"

#define FIND_DC_CHAT_GUI(p,d) \
            char* name = (char*) object_get_data(OBJ(p), "DcChatGui"); \
            d =  qobject_cast<DcChatGui *> (findWidgetByName(QString("%1").arg(name))); \
            Q_ASSERT(d);

class DcUserItem : public QStandardItem
{
public:
    DcUserItem(const QString &text)
        : QStandardItem(text) {}
    ~DcUserItem() {
        object_free_data(OBJ(user), l_printf("ChatUser_%x", dcp), NULL);    
    }
    void set_user(dc_user_t* user, dc_cp_t* dcp) {
        this->user = user;
        this->dcp = dcp;
        this->setData(QVariant::fromValue((void*)user));
        object_set_data(OBJ(user), l_printf("ChatUser_%x", dcp), this);
    }
private:
    dc_user_t* user;
    dc_cp_t* dcp;
};

class DcChatUserItemModel : public QStandardItemModel
{
public:
    DcChatUserItemModel(QObject *parent=0);
    void dc_item_chat_user_add(dc_cp_t* dcp, dc_user_t* user);
    void dc_item_chat_user_update(dc_cp_t* dcp, dc_user_t* user);
    void dc_item_chat_user_delete(dc_cp_t* dcp, dc_user_t* user);
};

DcChatUserItemModel::DcChatUserItemModel(QObject *parent)
    : QStandardItemModel(0, 3, parent) 
{
    QStringList headers;
    headers << qtr("User") << qtr("Shared") << qtr("Speed");
    setHorizontalHeaderLabels(headers); 
}

void DcChatUserItemModel::dc_item_chat_user_delete(dc_cp_t* dcp, dc_user_t* dc_user)
{
    DcUserItem *user;

    user = static_cast< DcUserItem *>(object_get_data(OBJ(dc_user),  l_printf("ChatUser_%x", dcp)));
    if (!user) return;

    QModelIndex model_index = indexFromItem(user);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    removeRow(row);   
}

void DcChatUserItemModel::dc_item_chat_user_update(dc_cp_t* dcp, dc_user_t* dc_user)
{
    DcUserItem *user;
    QStandardItem *shared;
    QStandardItem *speed;

    user = static_cast< DcUserItem *>(object_get_data(OBJ(dc_user), l_printf("ChatUser_%x", dcp)));
    if (!user) return;

    QModelIndex model_index = indexFromItem(user);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    
    
    // We have the row, let get all the items and update them
    shared = item(row, 1); 
    speed = item(row, 2); 

    // update the items
    user->setText(qfu(OBJ(dc_user)->name));
    shared->setText(qfu("%1").arg( print_filesize(&dc_user->bytes)));
    speed->setText(qfu(dc_user->linespeed));
}

void DcChatUserItemModel::dc_item_chat_user_add(dc_cp_t* dcp, dc_user_t* dc_user)
{
    QList<QStandardItem *> items;

    DcUserItem *user;
    QStandardItem *shared;
    QStandardItem *speed;

    if (object_get_data(OBJ(dc_user), l_printf("ChatUser_%x", dcp))) return;
    
    user = new DcUserItem(qfu(OBJ(dc_user)->name)); 
    shared = new QStandardItem(qfu("%1").arg( print_filesize(&dc_user->bytes)));
    speed = new QStandardItem(qfu(dc_user->linespeed));
    
    items.append(user);
    items.append(shared);
    items.append(speed);

    // Add the items
    appendRow(items);
    dc_item_chat_user_update(dcp,dc_user);
}


DcChatUserTree::DcChatUserTree(QWidget *parent)
    : UiTreeView(parent)
{
    ui_treeview_set_model(new DcChatUserItemModel(this));
}

void DcChatUserTree::ui_dc_user_init(dc_cp_t* page)
{
    action_item_t* actions = 
        action_item_search("Online dc users actions");
    ui_tv_handle_button(1, &actions, page);
}

void DcChatUserTree::ui_dc_user_vanished(dc_cp_t* dcp, dc_user_t* user)
{
    DcChatUserTree *tree;   
    DcChatGui *ndata;
    
    if (!dcp || !user) return;
    FIND_DC_CHAT_GUI(dcp,ndata);
    if (!ndata || !ndata->user_tree()) return;

    tree = ndata->user_tree();
    static_cast<DcChatUserItemModel *>(tree->model())->dc_item_chat_user_delete(dcp, user);
}

void DcChatUserTree::ui_dc_user_update(dc_cp_t* dcp, dc_user_t* user)
{
    DcChatUserTree *tree;   
    DcChatGui *ndata;
    
    if (!dcp) return;
    FIND_DC_CHAT_GUI(dcp,ndata);

    if (!ndata || !ndata->user_tree()) return;
    
    tree = ndata->user_tree();
    static_cast<DcChatUserItemModel *>(tree->model())->dc_item_chat_user_update(dcp, user);
}

void DcChatUserTree::ui_dc_user_show(dc_cp_t* dcp, dc_user_t* user)
{
    DcChatUserTree *tree;   
    DcChatGui *ndata;
    FIND_DC_CHAT_GUI(dcp,ndata);
    if (!ndata) return;
  
    tree = ndata->user_tree();
    static_cast<DcChatUserItemModel *>(tree->model())->dc_item_chat_user_add(dcp, user);
}

void DcChatUserTree::ui_dc_user_appeared(dc_cp_t* dcp, dc_user_t* user)
{
    Q_ASSERT(dcp && user);
    object_notify_o(OBJ(dcp), (ObjectFunc)ui_dc_user_vanished, OBJ(user));   
    ui_dc_user_show(dcp, user);
}

DcChatGui::DcChatGui(dc_cp_t* page, QWidget *parent )
    : ChatGui(parent), m_page(page)
{
    // Need to fill in the inherited parent body
    body.setupUi(ui_chat_gui_get_body());
    this->no_views = 1;
    this->view[0] = body.text_dcchat;
    body.splitter_main->setStretchFactor(0,3);
    body.tree_users->ui_dc_user_init(page);
}

DcChatUserTree *DcChatGui::user_tree()
{
    return body.tree_users;
}

void UiDcChat::ui_dc_chat_init() {
}

void UiDcChat::ui_dc_public_remove(dc_cp_t* page)
{
    DcChatGui *ndata;
    FIND_DC_CHAT_GUI(page,ndata);
    if (!ndata) return;
    ChatPage::s_ui_chat_page_tab_remove(qobject_cast<ChatGui *>(ndata));
    delete ndata;
    object_free_data_common(OBJ(page), "DcChatGui");
}

void UiDcChat::ui_dc_create_page(dc_cp_t* page)
{
    if (page->type == DC_PAGE_PRIVATE)
    {
        ChatPage::ui_chat_create_simple_page(PAGE(page));
    }
    else if (page->type == DC_PAGE_PUBLIC)
    {
        DcChatGui *tab = new DcChatGui(page);
        char* object_name = g_strdup_printf("DcChatGui_%x",tab);
        tab->setObjectName(QString("%1").arg(object_name));
        object_set_data(OBJ(page), "DcChatGui", object_name);
        // Then add it to the main chat entry page
        ChatPage::ui_chat_page_add(PAGE(page), tab);
        object_notify_d(OBJ(page), (ObjectFunc)ui_dc_public_remove, NULL);
    }
    else
    {
        g_log("QT4_DC", G_LOG_LEVEL_WARNING,
              "unknown chat page found, cannot create ui");
    }

}

