/* Copyright (C) 2007   The Lopster Team
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. 
*/

/* $Revision: 1.5 $
 * $Date: 2007/11/14 12:42:47 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QPainter>
#include <QStandardItemModel>
#include <QItemDelegate>

#include "lcore/preferences.h"
#include "lcore/action_item.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"

#include "dc.h"

#include "lcore/qt4_ui/ui_network.hpp"
#include "lcore/qt4_ui/ui_main.hpp"

#include "ui_dc.hpp"
#include "ui_dc_net.hpp"

#define UI_GROUP_ALL    0
#define UI_GROUP_BOOK   1
#define UI_GROUP_ONLINE 2

QHash <QString, QIcon> HubsIcons;
/* QHash <QString, QPixmap> HubsPixmaps; */


HubSortFilterProxyModel::HubSortFilterProxyModel(QObject *parent)
    : QSortFilterProxyModel(parent)
{
}
/*
static gboolean _hub_filtered(dc_hub_t* hub)
{
    if (Filter.group == UI_GROUP_ONLINE && hub->status != DC_HUB_ONLINE)
    {
        return TRUE;
    }

    if (Filter.group == UI_GROUP_BOOK && !(hub->flags & DC_HUB_BOOKMARK))
    {
        return TRUE;
    }
    
    if (Filter.flag && Filter.flag != AnyCountry &&
        (!hub->country || strcmp(Filter.flag->country, hub->country)))
    {
        return TRUE;
    }
    
    if (Filter.text &&
        (!hub->name || !strcasestr(hub->name, Filter.text)) &&
        (!hub->description || !strcasestr(hub->description, Filter.text)))
    {
        return TRUE;
    }
    return FALSE;
}
*/

bool HubSortFilterProxyModel::filterAcceptsRow(int sourceRow,
                                    const QModelIndex &sourceParent) const
{
    QModelIndex index_name    = sourceModel()->index(sourceRow, 0, sourceParent);
    QModelIndex index_state   = sourceModel()->index(sourceRow, 3, sourceParent);
    QModelIndex index_country = sourceModel()->index(sourceRow, 4, sourceParent);

    return (sourceModel()->data(index_name).toString().contains(hub_name)
            && filter_check_state  (sourceModel()->data(index_state  ).toString())
            && filter_check_country(sourceModel()->data(index_country).toString()) );
}

bool HubSortFilterProxyModel::lessThan(const QModelIndex &left,
                                      const QModelIndex &right) const
{
    QVariant leftData = sourceModel()->data(left);
    QVariant rightData = sourceModel()->data(right);

    return QString::localeAwareCompare(leftData.toString(), rightData.toString()) < 0;
}
bool HubSortFilterProxyModel::filter_check_state(const QString &state) const
{
    return (hub_state.isEmpty() || (hub_state == state) );
}
bool HubSortFilterProxyModel::filter_check_country(const QString &country) const
{
    return (hub_country.isEmpty() || (hub_country == country ));
}

class DcItem : public QStandardItem
{
public:
    DcItem(const QString &text)
        : QStandardItem(text) {}
    DcItem(const QIcon &icon, const QString &name)
        : QStandardItem(icon,name) {}
    ~DcItem() {
        object_free_data(OBJ(m_hub), "HubEntry", NULL);    
    }
    void set_hub(dc_hub_t* hub) {
        this->m_hub = hub;
        this->setData(QVariant::fromValue((void*)hub));
        object_set_data(OBJ(hub), "HubEntry", this);
    }
    dc_hub_t* m_hub;
};

class DcItemModel : public QStandardItemModel
{
public:
    DcItemModel(QWidget *parent=0);
    void dc_item_hub_add(dc_hub_t* hub);
    void dc_item_hub_update(dc_hub_t* hub);
    void dc_item_hub_delete(dc_hub_t* hub);
};
/*
class DcTreeDelegate : public QItemDelegate
{
public:
    DcTreeDelegate(QWidget *parent, QAbstractProxyModel *model);
    void paint(QPainter *painter, const QStyleOptionViewItem &option,
                      const QModelIndex &index ) const;
private:
    QAbstractProxyModel *m_model;
};

#define ICON_WIDTH  30
#define ICON_HEIGHT 20

DcTreeDelegate::DcTreeDelegate(QWidget *parent, QAbstractProxyModel *model) 
                        : QItemDelegate(parent) , m_model(model)
{
}

void DcTreeDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
                  const QModelIndex &index ) const
{
   
    if (index.column() != 0) {
        QItemDelegate::paint(painter, option, index);
        return;
    }
    DcItemModel *item_model = static_cast< DcItemModel *>(m_model->sourceModel());   
    DcItem *item = static_cast< DcItem * >
                    (item_model->itemFromIndex(m_model->mapToSource(index)));

    if (!item) return;
    
    // display the icon, name, and description a better way
    // QIcon icon = item->icon();
    QString name = qfu(item->m_hub->name);
    QString description = qfu(item->m_hub->description);
    QString country = qfu(item->m_hub->country);

    painter->drawPixmap(QRectF( 0,0 , ICON_WIDTH, ICON_HEIGHT), UiDcNet::ui_dc_hub_pixmap(country), QRectF( 0,0 , ICON_WIDTH, ICON_HEIGHT));
}

*/


DcItemModel::DcItemModel(QWidget *parent)
    : QStandardItemModel(0, 6 , parent) // FIXME : handle the color on online
{
    QStringList headers;
    headers << qtr("Name") << qtr("Description") << qtr("Address") 
            << qtr("Users") << qtr("Status") << qtr("Country");
    setHorizontalHeaderLabels(headers); 
}

void DcItemModel::dc_item_hub_add(dc_hub_t* hub)
{
    QList<QStandardItem *> items;

    DcItem *name;
    QStandardItem *description;
    QStandardItem *address;
    QStandardItem *users;
    QStandardItem *status;
    QStandardItem *country;

    if (!hub) return;

    if (object_get_data(OBJ(hub), "HubEntry")) return;    

    // FIXME : QStandardItem deosn't handle these tags, use a Delegate as a QLabel or handle font foregtround etc
   // QString name_desc = QString("%1\n<span size=\"smaller\" foreground=\"#99aa99\">%2</span>")
   //                         .arg(qfu(hub->name)).arg(qfu(hub->description));
    name = new DcItem(UiDcNet::ui_dc_hub_icon(qfu(hub->country)), qfu(hub->name)); 
    description = new QStandardItem(qfu(hub->description));
    address = new QStandardItem(qfu("%1:%2").arg(hub->address).arg(ntohs(TSOCK(hub)->port)));
    users = new QStandardItem(qfu("%1").arg(dc_hub_get_user_cnt(hub)));
    status = new QStandardItem(QString(""));
    country = new QStandardItem(qfu(hub->country));
 
    name->set_hub(hub);

    items.append(name);
    items.append(description);
    items.append(address);
    items.append(users);
    items.append(status);
    items.append(country);
    // Add the items
    appendRow(items);
    dc_item_hub_update(hub);
}

void DcItemModel::dc_item_hub_update(dc_hub_t* hub)
{
    DcItem *name;
    QStandardItem *description;
    QStandardItem *address;
    QStandardItem *users;
    QStandardItem *status;
    QStandardItem *country;

    name = static_cast< DcItem *>(object_get_data(OBJ(hub), "HubEntry"));
    if (!name) return;

    QModelIndex model_index = indexFromItem(name);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    
    
    // We have the row, let get all the items and update them
    description = item(row, 1); 
    address = item(row, 2); 
    users = item(row, 3); 
    status = item(row, 4); 
    country = item(row, 5); 

    // FIXME : QStandardItem deosn't handle these tags, use a Delegate as a QLabel or handle font foregtround etc
    // QString name_desc = QString("%1\n<span size=\"smaller\" foreground=\"#99aa99\">%2</span>") 
     //                        .arg(qfu(hub->name)).arg(qfu(hub->description));
    // update the items
    name->setText(qfu(hub->name));
    // name->set_description(qfu(hub->description) /*qfu(hub->name)*/ );
    description->setText(qfu(hub->description));
    address->setText(qfu("%1:%2").arg(hub->address).arg(ntohs(TSOCK(hub)->port)));
    users->setText(qfu("%1").arg(dc_hub_get_user_cnt(hub)));
    if (hub->status == DC_HUB_ONLINE)
    {
        status->setText(qtr("Online"));
    }
    else {
        status->setText(QString(""));
    }
    country->setText(qfu(hub->country));
}

void DcItemModel::dc_item_hub_delete(dc_hub_t* hub)
{
    DcItem *name = static_cast< DcItem *>(object_get_data(OBJ(hub), "HubEntry"));
    if (!name) return;

    QModelIndex model_index = indexFromItem(name);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    removeRow(row);   
}


UiDcNet::UiDcNet(QWidget *parent)
    : QWidget(parent)
{
    setupUi(this);
    proxy_model = new HubSortFilterProxyModel;
    proxy_model->setDynamicSortFilter(true);

    hubs_view->setRootIsDecorated(false);
    hubs_view->setModel(proxy_model);
    hubs_view->setSortingEnabled(true);  
	hubs_view->setIconSize(QSize(30,20));

    combo_state->addItem(qtr("Show all hubs"));
    combo_state->addItem(qtr("Show bookmarks"));
    combo_state->addItem(qtr("Show online hubs"));

    source_model = new DcItemModel(this);
    proxy_model->setSourceModel(source_model);
    
    hubs_view->ui_treeview_set_model(source_model,false);
    hubs_view->setWordWrap(true);

 //   hubs_view->setItemDelegate(new DcTreeDelegate(hubs_view, proxy_model));

    action_item_t* actions = action_item_search("DirectConnect hub actions");
    hubs_view->ui_tv_handle_button(1, &actions, NULL);
    
    connect(line_name, SIGNAL(textChanged(const QString &)),this, SLOT(filter_reg_exp_changed(const QString &)));
    connect(combo_state, SIGNAL(currentIndexChanged(int)),this, SLOT(filter_state_changed(int)));
    connect(combo_country, SIGNAL(currentIndexChanged(int)),this, SLOT(filter_country_changed(int)));
    connect(tool_clean,SIGNAL(clicked()), this, SLOT(filter_reset()));

    connect(tool_w_ok,SIGNAL(clicked()),this,SLOT(hubs_refresh()));
    tool_w_ok->setEnabled(false);
    connect(tool_w_clean,SIGNAL(clicked()),this,SLOT(weblist_remove()));
    tool_w_clean->setEnabled(false);
    connect(combo_web,SIGNAL(currentIndexChanged(const QString&)),this,SLOT(weblist_changed(const QString&)));
    connect(combo_web,SIGNAL(editTextChanged(const QString&)),this,SLOT(weblist_changed(const QString&)));  
}

void UiDcNet::ui_dc_net_init()
{
    NetworkPage::ui_network_add_page(DC_ID, GetDcNet());
}

void UiDcNet::ui_dc_net_cleanup()
{
    UiDcNet* net = GetDcNet(); 
    NetworkPage::ui_network_remove_page(net);
    delete net;
}

UiDcNet *UiDcNet::GetDcNet()
{
   static UiDcNet* net = NULL;
   if (net == NULL)
   {
        net = new UiDcNet(); 
   }
   return net;
}

void UiDcNet::hubs_refresh()
{
    QString str = combo_web->currentText();
    if (!str.isEmpty())
    {
        dc_weblist_get_list(qtu(str));
    }
}

void UiDcNet::weblist_remove()
{
    int index = combo_web->currentIndex();
    if (index != -1) {
        url_t* url;
        QVariant v = combo_web->itemData(index);
        if ( !v.isValid())
        {
            combo_web->removeItem(index);
            return;
        }
        url = (url_t*) v.value<void*>();
        object_kill(OBJ(url));
    }
}


void UiDcNet::weblist_changed(const QString &text)
{
    tool_w_ok->setEnabled(!text.isEmpty());
    tool_w_clean->setEnabled(!text.isEmpty());
}

void UiDcNet::ui_dc_weblist_hide(url_t* web)
{
    int *index = (int *) object_get_data(OBJ(web), "HubListUrl");
    QComboBox *combo = GetDcNet()->combo_web;
    combo->removeItem(*index);   
    object_free_data(OBJ(web), "HubListUrl", NULL);
}

void UiDcNet::ui_dc_weblist_update(url_t* web)
{
    int *index = (int *) object_get_data(OBJ(web), "HubListUrl");
    if (!index) return;
    QString str = qfu("http://%1%2").arg(web->host).arg(web->doc);
    QComboBox *combo = GetDcNet()->combo_web;
    combo->insertItem(*index , str, QVariant::fromValue((void*)web));   
}

void UiDcNet::ui_dc_weblist_show(url_t* web)
{
    int *index = NULL;
    index = (int *) object_get_data(OBJ(web), "HubListUrl");  
    if (index && *index >= 0) return;

    QComboBox *combo = GetDcNet()->combo_web;
    int count = combo->count();
    object_set_data(OBJ(web), "HubListUrl", &count);
    ui_dc_weblist_update(web);
}

void UiDcNet::ui_dc_weblist_new(url_t* web)
{
    Q_ASSERT(web);
    
    object_notify_d(OBJ(web), (ObjectFunc)ui_dc_weblist_hide, NULL);
    object_monitor_d(OBJ(web), (ObjectFunc)ui_dc_weblist_update, NULL);

    ui_dc_weblist_show(web);
}

void UiDcNet::ui_dc_hub_update(dc_hub_t* hub)
{
    GetDcNet()->source_model->dc_item_hub_update(hub);
}

void UiDcNet::ui_dc_hub_destroy(dc_hub_t* hub)
{
    GetDcNet()->ui_dc_hub_unregister_flag(hub);
    GetDcNet()->source_model->dc_item_hub_delete(hub);
}

void UiDcNet::ui_dc_hub_unregister_flag(dc_hub_t* hub)
{
    if (!hub->country || *hub->country == 0) return;
    QString country = qfu(hub->country);
    int index = combo_country->findText(country);
    if (index != -1)
    { 
        combo_country->removeItem(index);
    }    
}

void UiDcNet::ui_dc_hub_register_flag(dc_hub_t* hub)
{
    if (!hub->country || *hub->country == 0) return;
    QString country = qfu(hub->country);
    int index = combo_country->findText(country);
    if (index == -1)
    {
        QIcon icon = ui_dc_hub_icon(country);        
        combo_country->addItem(icon,country);
    }
}

void UiDcNet::ui_dc_hub_new(dc_hub_t* hub)
{
    Q_ASSERT(hub);

    GetDcNet()->ui_dc_hub_register_flag(hub);   

    object_notify_d(OBJ(hub), (ObjectFunc)ui_dc_hub_destroy, NULL);
    object_monitor_d(OBJ(hub), (ObjectFunc)ui_dc_hub_update, NULL);

    GetDcNet()->source_model->dc_item_hub_add(hub);
}

void UiDcNet::ui_dc_hub_set_status(dc_hub_t* hub, const char* message)
{
    //TODO
  /*  dc_hub_gui_t* data = object_get_data(OBJ(hub), "HubEntry");
    
    if (!data) return;

    gtk_list_store_set(data->store, &data->iter,
                       4, message,
                       5, (hub->status == DC_HUB_ONLINE),
                       -1); */
}

void UiDcNet::ui_dc_hub_edit(dc_hub_t* hub)
{
    //TODO
}

void UiDcNet::filter_reg_exp_changed(const QString &text)
{
    QRegExp regExp(text, Qt::CaseInsensitive, QRegExp::RegExp);
    proxy_model->setFilterHubName(regExp);
}
void UiDcNet::filter_state_changed(int index)
{
    QString state = combo_country->itemText(index);
    if (!state.isEmpty())
    {
        proxy_model->setFilterHubState(state);
    }
}
void UiDcNet::filter_country_changed(int index)
{
    QString country = combo_country->itemText(index);
    if (!country.isEmpty())
    {
        proxy_model->setFilterHubCountry(country);
    }
}

void UiDcNet::filter_reset()
{
    line_name->setText(QString(""));
    combo_state->setCurrentIndex(0); // all
    combo_country->setCurrentIndex(0); // blank
}

QIcon UiDcNet::ui_dc_hub_icon(const QString &name)
{
	QIcon icon = HubsIcons[name];
    if (icon.isNull())
    {
	    icon = QIcon(QString(":/Flag%1.png").arg(name));
		if (!icon.isNull())
		{
			HubsIcons[name] = icon;
		} 
		else {
			icon = HubsIcons["FlagUnknown"];
			if (!icon.isNull())
			{
				icon = QIcon(QString(":/FlagUnknown.png"));
				HubsIcons["FlagUnknown"] = icon;
			}
		}
	}
    return icon;
}

/*
QPixmap UiDcNet::ui_dc_hub_pixmap(const QString &name)
{
	QPixmap pix = HubsPixmaps[name];
    if (pix.isNull())
    {
	    pix = QPixmap(QString(":/Flag%1.png").arg(name));
		if (!pix.isNull())
		{
			HubsPixmaps[name] = pix;
		} 
		else {
			pix = HubsPixmaps["FlagUnknown"];
			if (!pix.isNull())
			{
				pix = QPixmap(QString(":/FlagUnknown.png"));
				HubsPixmaps["FlagUnknown"] = pix;
			}
		}
	}
    return pix;
}
*/