/* Copyright (C) 2007   The Lopster Team
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _UI_DC_NET_HPP_
#define _UI_DC_NET_HPP_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QSortFilterProxyModel>

#include "lcore/l_http.h"
#include "dc_network.h"

#include "widgets/DcNetworkWidget.h"

class DcItemModel;

class HubSortFilterProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
public:
    HubSortFilterProxyModel(QObject *parent = 0);
    QRegExp filterHubName() const { return hub_name;}
    void setFilterHubName(const QRegExp &name) {
        hub_name = name;
        invalidateFilter();
    }
    QString filterHubState() const { return hub_state;}
    void setFilterHubState(const QString &state) {
        hub_state = state;
        invalidateFilter();
    }
    QString filterHubCountry() const { return hub_country;}
    void setFilterHubCountry(const QString &country) {
        hub_country = country;
        invalidateFilter();
    }
protected:
    bool filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const;
    bool lessThan(const QModelIndex &left, const QModelIndex &right) const;

private:
    bool filter_check_state(const QString &state) const;
    bool filter_check_country(const QString &country) const;
    QRegExp hub_name;
    QString hub_state;
    QString hub_country;
};


class UiDcNet : public QWidget, private Ui::DcNetworkWidget
{
    Q_OBJECT
public:
    UiDcNet(QWidget *parent=0);

    static UiDcNet *GetDcNet();
    
    static void ui_dc_net_init();
    static void ui_dc_net_cleanup();

    static QIcon ui_dc_hub_icon(const QString &name);
//    static QPixmap ui_dc_hub_pixmap(const QString &name);

    static void ui_dc_weblist_hide(url_t* web);
    static void ui_dc_weblist_update(url_t* web);
    static void ui_dc_weblist_show(url_t* web);
    static void ui_dc_weblist_new(url_t* web);

    static void ui_dc_hub_new(dc_hub_t* hub);
    static void ui_dc_hub_update(dc_hub_t* hub);
    static void ui_dc_hub_destroy(dc_hub_t* hub);

    static void ui_dc_hub_edit(dc_hub_t* hub);
    static void ui_dc_hub_set_status(dc_hub_t* hub, const char* message);

    void ui_dc_hub_register_flag(dc_hub_t* hub);
    void ui_dc_hub_unregister_flag(dc_hub_t* hub);
    //void ui_dc_flag_destroy();

    DcItemModel *source_model;
private slots:
    void filter_reg_exp_changed(const QString &);
    void filter_state_changed(int);
    void filter_country_changed(int);
    void filter_reset();
    void hubs_refresh();
    void weblist_remove();
    void weblist_changed(const QString&);

private:
    HubSortFilterProxyModel *proxy_model;
    QString name_filter;
    QString state_filter;
    QString country_filter;
};

#endif
