/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#include <string.h>

#include "tiger.h"
#include "tth.h"

#define TTH_BLOCK_SIZE   1024

typedef struct {
  guint64 size;
  guint8 hash[TIGER_HASH_SIZE];
} tth_block_t;

#define MERGE_BLOCKS(tiger, block1, block2) { \
  tiger_init(tiger); \
  tiger_update(tiger, "\1", 1); \
  tiger_update(tiger, block1->hash, TIGER_HASH_SIZE); \
  tiger_update(tiger, block2->hash, TIGER_HASH_SIZE); \
  memcpy(block1->hash, tiger_finalize(tiger), TIGER_HASH_SIZE); \
  block1->size = block1->size + block2->size; \
  g_free(block2); \
}

static void tth_add_block(tth_t* tth, guint8* data, unsigned len) {
  static tiger_t tiger;
  tth_block_t* block1;
  tth_block_t* block2;

  block1 = g_malloc(sizeof(*block1));
  memcpy(block1->hash, data, TIGER_HASH_SIZE);
  block1->size = len;
  tth->blocks = g_list_prepend(tth->blocks, block1);
  
  while (tth->blocks->next &&
	 (block1 = tth->blocks->data)->size == 
	 (block2 = tth->blocks->next->data)->size) {
    MERGE_BLOCKS(&tiger, block2, block1);
    tth->blocks = g_list_remove(tth->blocks, block1);
  }
}

void tth_init(tth_t* tth) {
  //  printf("--init\n");
  tth->blocks = NULL;
  tth->last = FALSE;
}

void tth_update(tth_t* tth, const void* data, unsigned len) {
  const guint8* buf = data;
  static tiger_t tiger;
  
  //  printf("--update %u bytes\n", len);
  g_assert(!tth->last);

  while (len >= TTH_BLOCK_SIZE) {
    tiger_init(&tiger);
    tiger_update(&tiger, "\0", 1);
    tiger_update(&tiger, buf, TTH_BLOCK_SIZE);
    tth_add_block(tth, tiger_finalize(&tiger), TTH_BLOCK_SIZE);
    buf += TTH_BLOCK_SIZE;
    len -= TTH_BLOCK_SIZE;
  }
  if (len > 0 || data == buf) {
    tth->last = TRUE;
    tiger_init(&tiger);
    tiger_update(&tiger, "\0", 1);
    tiger_update(&tiger, buf, len);
    tth_add_block(tth, tiger_finalize(&tiger), len);
  }
}

guint8* tth_finalize(tth_t* tth) {
  static tiger_t tiger;
  tth_block_t* block1;
  tth_block_t* block2;

  //  printf("--fin\n");
  // if there is no block, we didn't call tth_update()
  // lets do that manually
  if (!tth->blocks) tth_update(tth, "", 0);

  while (tth->blocks->next) {
    block1 = tth->blocks->data;
    block2 = tth->blocks->next->data;
    MERGE_BLOCKS(&tiger, block2, block1);
    tth->blocks = g_list_remove(tth->blocks, block1);
  }
  block1 = tth->blocks->data;
  memcpy(tth->res, block1->hash, TIGER_HASH_SIZE);
  tth->blocks = g_list_remove(tth->blocks, block1);
  g_free(block1);
  
  return tth->res;
}
