/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.16 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "lcore/chat.h"
#include "lcore/preferences.h"
#include "lcore/string_list.h"
#include "lcore/lopster.h"
#include "lcore/l_i18n.h"
#include "lcore/gtk2_ui/ui_chat.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "lcore/gtk2_ui/ui_layout.h"
#include "lcore/gtk2_ui/ui_treeview.h"
#include "nap_network.h"
#include "nap.h"
#include "nap_chat.h"
#include "ui_nap_chat.h"

const char NapCModeChars[NAP_CMODE_NUMBER] = {
  't', 'r', 'p', 'i', 'm',
};

const char* NapCModeNames[NAP_CMODE_NUMBER] = {
  "T", "R", "P", "I", "M",
};
const char* NapCModeNamesLong[NAP_CMODE_NUMBER] = {
  N_("topic restricted"), N_("registered"), N_("private"), N_("invite only"), N_("moderated"),
};

// thats the gui data for a public page
// server, motd, private, other pages simply use chat_gui_t
typedef struct {
  chat_gui_t _data;
  // user list in public pages
  GtkListStore* online_store;
  // mode/level/limit in public pages
  GtkWidget* public_settings;
  GtkWidget* chop;
  GtkWidget* mbutton[NAP_CMODE_NUMBER];
  GtkWidget* combo;
  GtkWidget* lentry;
  GtkTreeViewColumn* col[3];
  GtkWidget* top_paned;
  GtkWidget* chat_paned;
  GtkWidget* users;
  GtkWidget* topic;
  GtkWidget* counter;
  GtkLabel* info_label;
} nap_public_gui_t;

// thats the gui data for a channel page
/* typedef struct { */
/*   // channel list and settings gui */
/*   GtkWindow* win; */
/*   GtkTreeViewColumn* col[5]; */
/*   GtkListStore* channel_store; */
/*   GtkWidget* clabel; */
/*   GtkWidget* rbutton; */
/*   GtkWidget* abutton; */
/*   GtkWidget* min_spin; */
/*   GtkWidget* max_spin; */
/* } nap_channel_gui_t; */


/** registers tags for chat prints
 * network(nap_net_t*)
 *    [net->name]
 * netstr(nap_net_t*, char*)
 *    [net->name:text]
 */
static void
ui_nap_chat_print_net(GtkTextBuffer* buffer, GtkTextIter* iter, va_list ap) {
  nap_net_t* net;
  // prints [network]
  // args: nap_net_t* net
  net = va_arg(ap, void*);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, "[", 1, "c_bracket", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, OBJ(net)->name, -1, "c_text", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, "]", 1, "c_bracket", NULL);
}
static void
ui_nap_chat_print_netstr(GtkTextBuffer* buffer, GtkTextIter* iter,
			 va_list ap) {
  nap_net_t* net = va_arg(ap, void*);
  const char* text = va_arg(ap, char*);

  // prints [network:text]

  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, "[", 1, "c_bracket", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, OBJ(net)->name, -1, "c_text", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, ":", 1, "c_text", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, text, -1, "c_text", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, "]", 1, "c_bracket", NULL);
}
static void
ui_nap_chat_print_channel(GtkTextBuffer* buffer, GtkTextIter* iter, va_list ap) {
  char* channel = va_arg(ap, char*);
  nap_net_t* net = va_arg(ap, void*);  // FIXME
  char* text = va_arg(ap, char*);

  // prints channel
  if (text && text[0])
    gtk_text_buffer_insert_with_tags_by_name
      (buffer, iter, text, 1, "c_bracket", NULL);
  gtk_text_buffer_insert_with_tags_by_name
    (buffer, iter, channel, -1, "c_text", NULL);
  if (text && text[1])
    gtk_text_buffer_insert_with_tags_by_name
      (buffer, iter, text+1, 1, "c_bracket", NULL);
}

void ui_nap_chat_init() {
  ui_chat_register_print_func("nap_net", ui_nap_chat_print_net);
  ui_chat_register_print_func("nap_netstr", ui_nap_chat_print_netstr);
  ui_chat_register_print_func("nap_channel", ui_nap_chat_print_channel);
}

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

static void on_chmode_clicked(GtkToggleButton* togglebutton, gpointer data) {
  int set;
  nap_pub_t* page;
  int mode = GPOINTER_TO_INT(data);

  if (gtk_toggle_button_get_active(togglebutton)) set = 1;
  else set = 0;

  page = g_object_get_data(G_OBJECT(togglebutton), "page");
  g_assert(page);
  nap_channel_set_mode(page, set?mode:0, set?0:mode);
}

static void on_chlevel_changed(GtkComboBox* combo, nap_pub_t* page) {
  nap_channel_set_level(page, gtk_combo_box_get_active(combo));
}

static void on_chlimit_clicked(GtkEntry* entry) {
  nap_pub_t* page = g_object_get_data(G_OBJECT(entry), "page");
  g_assert(page);
  nap_channel_set_limit(page, atoi(gtk_entry_get_text(entry)));
}

static void ui_nap_user_hide(nap_cu_t* cu, nap_pub_t* npp);

static void ui_nap_public_remove(chat_page_t* page)
{
    nap_public_gui_t* ndata;
    chat_gui_t* data;
    
    g_list_foreach(NPP(page)->users, (GFunc)object_free_data_common, "ChatUserEntry");
    
    ndata = object_get_data(OBJ(page), "ChatPage");
    if (!ndata) return;
    data = (chat_gui_t*)ndata;

    ui_layout_unmonitor("nap:chat:col:user:size", ndata->col[0]);
    ui_layout_unmonitor("nap:chat:col:user:vis", ndata->col[0]);
    ui_layout_unmonitor("nap:chat:col:shared:size", ndata->col[1]);
    ui_layout_unmonitor("nap:chat:col:shared:vis", ndata->col[1]);
    ui_layout_unmonitor("nap:chat:col:speed:size", ndata->col[2]);
    ui_layout_unmonitor("nap:chat:col:speed:vis", ndata->col[2]);
    ui_layout_unmonitor("nap:chat:paned:pos", ndata->top_paned);
    ui_layout_unmonitor("nap:chat:op:paned:pos", ndata->chat_paned);
    ui_layout_unmonitor("nap:chat:users:vis", ndata->users);
    ui_layout_unmonitor("nap:chat:chop:vis", ndata->chop);
    
    ui_chat_page_remove(page);
}


void ui_nap_create_public_page(nap_pub_t* page)
{
    GtkWidget* chat_paned;
    GtkWidget* text;
    nap_net_t* net;
    int i1;
    nap_public_gui_t* ndata;
    chat_gui_t* data;
    GtkWidget* scrolledwindow;
    GtkWidget* hbox;
    GtkListStore* store;
    GtkWidget* tv;
    GtkWidget* entry;
    GtkCellRenderer* renderer;
    GtkTreeViewColumn* column;
    GtkWidget* hpaned;
    GtkTreeSelection *select;
    GtkWidget* label;
    GtkWidget* tt;

    ndata = g_malloc0(sizeof(nap_public_gui_t));
    data = (chat_gui_t*)ndata;

    object_set_data(OBJ(page), "ChatPage", ndata);
    net = NCP(page)->net;


    data->toplevel = ndata->top_paned = hpaned = gtk_hpaned_new();
    gtk_widget_show(hpaned);
    ui_layout_monitor("nap:chat:paned:pos", hpaned);
  
    // RIGHT USER LIST
    ndata->users = scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(scrolledwindow);
    ui_layout_monitor("nap:chat:users:vis", scrolledwindow);
    gtk_paned_pack2(GTK_PANED(hpaned), scrolledwindow, FALSE, TRUE);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow),
                                         GTK_SHADOW_IN);

    store = gtk_list_store_new(4,
                               G_TYPE_POINTER,  // pointer to page
                               G_TYPE_STRING,
                               G_TYPE_INT,
                               G_TYPE_STRING);
    ndata->online_store = store;
  
    tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
    //  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
    gtk_container_add (GTK_CONTAINER (scrolledwindow), tv);
    gtk_widget_show(tv);
    gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tv), TRUE);
    //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);

    select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
    gtk_tree_selection_set_mode (select, GTK_SELECTION_MULTIPLE);
    {
        action_item_t* actions = 
            action_item_search("Online nap users actions");
        ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &actions, page);
    }

    renderer = gtk_cell_renderer_text_new ();
    ndata->col[0] = column = gtk_tree_view_column_new_with_attributes 
        (_("User"), renderer, "text", 1, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    ui_tv_config_default_col(column, 1, "nap:chat:col:user", FALSE);

    renderer = gtk_cell_renderer_text_new ();
    ndata->col[1] = column = gtk_tree_view_column_new_with_attributes 
        (_("Shared"), renderer, "text", 2, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    ui_tv_config_default_col(column, 2, "nap:chat:col:shared", FALSE);

    renderer = gtk_cell_renderer_text_new ();
    ndata->col[2] = column = gtk_tree_view_column_new_with_attributes 
        (_("Speed"), renderer, "text", 3, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    ui_tv_config_default_col(column, 3, "nap:chat:col:speed", TRUE);


    data->no_buffers = 2;
    data->buffer = g_malloc0(sizeof(GtkTextBuffer)*data->no_buffers);
    data->view = g_malloc0(sizeof(GtkTextView)*data->no_buffers);


    // THE CHAT PAGES
    ndata->chat_paned = chat_paned = gtk_vpaned_new();
    gtk_widget_show(chat_paned);
    ui_layout_monitor("nap:chat:op:paned:pos", chat_paned);
    gtk_paned_pack1(GTK_PANED(hpaned), chat_paned, TRUE, TRUE);

    ndata->chop = scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
    //  gtk_widget_show(scrolledwindow);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrolledwindow),
                                        GTK_SHADOW_IN);
    gtk_paned_pack1(GTK_PANED(chat_paned), scrolledwindow, FALSE, TRUE);
    ui_layout_monitor("nap:chat:chop:vis", scrolledwindow);

    data->buffer[NAP_B_OP] = ui_chat_new_buffer();
    text = ui_chat_new_text_view(data->buffer[NAP_B_OP]);
    gtk_text_view_set_indent(GTK_TEXT_VIEW(text), -50);
    gtk_widget_show(text);
    gtk_container_add(GTK_CONTAINER(scrolledwindow), text);
    data->view[NAP_B_OP] = GTK_TEXT_VIEW(text);

    scrolledwindow = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(scrolledwindow);

    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolledwindow),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW (scrolledwindow),
                                        GTK_SHADOW_IN);
    gtk_paned_pack2(GTK_PANED(chat_paned), scrolledwindow, TRUE, TRUE);

    data->buffer[NAP_B_PUBLIC] = ui_chat_new_buffer();
    text = ui_chat_new_text_view(data->buffer[NAP_B_PUBLIC]);
    gtk_text_view_set_indent(GTK_TEXT_VIEW(text), -50);
    gtk_widget_show(text);
    gtk_container_add(GTK_CONTAINER(scrolledwindow), text);
    data->view[NAP_B_PUBLIC] = GTK_TEXT_VIEW(text);


    // TOPIC
    tt = hbox = gtk_hbox_new(FALSE, 10);
    gtk_widget_show(hbox);
    ndata->topic = label = gtk_label_new("");
    gtk_widget_show(label);
    gtk_widget_set_size_request(label, 100, -1);
    gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_label_set_selectable(GTK_LABEL(label), TRUE);
    ndata->counter = label = gtk_label_new("");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);


    // CONTROLS
    {
        GtkWidget* combo;
        GtkWidget* button;
        GtkWidget* hbox2;

        ndata->public_settings = hbox = gtk_hbox_new(FALSE, 10);
/*         gtk_container_set_border_width(GTK_CONTAINER(hbox), 5); */
        if (net->level <= NAP_L_USER)
            gtk_widget_set_sensitive(GTK_WIDGET(hbox), FALSE);
    
        label = gtk_label_new(_("<b>Mode:</b>"));
        gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
        gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    
        hbox2 = gtk_hbox_new(TRUE, 0);
        gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 0);

        for (i1 = 0; i1 < NAP_CMODE_NUMBER; i1++)
        {
            button = gtk_toggle_button_new_with_label(NapCModeNames[i1]);
            gtk_box_pack_start(GTK_BOX(hbox2), button, TRUE, TRUE, 0);
            ndata->mbutton[i1] = button;
            g_object_set_data(G_OBJECT(button), "page", (gpointer)page);
            g_signal_connect(G_OBJECT(button), "toggled",
                             G_CALLBACK(on_chmode_clicked), GINT_TO_POINTER(1<<i1));
            gtk_widget_set_tooltip_text(button, _(NapCModeNamesLong[i1]));
        }
    
    
        label = gtk_label_new(_("<b>Level:</b>"));
        gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
        gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    
        combo = gtk_combo_box_new_text();
        for (i1 = NAP_L_LEECH; i1 <= NAP_L_ELITE; i1++)
        {
            gtk_combo_box_append_text(GTK_COMBO_BOX(combo), NapLevel(i1));
        }
        gtk_box_pack_start(GTK_BOX(hbox), combo, FALSE, FALSE, 0);
        ndata->combo = combo;
        gtk_combo_box_set_active(GTK_COMBO_BOX(combo), page->level);
        g_signal_connect(G_OBJECT(combo), "changed",
                         G_CALLBACK(on_chlevel_changed), page);


        label = gtk_label_new(_("<b>Limit:</b>"));
        gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
        gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

        hbox2 = gtk_hbox_new(FALSE, 2);
        gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 0);
    
        entry = gtk_entry_new();
        gtk_box_pack_start(GTK_BOX(hbox2), entry, TRUE, TRUE, 0);
        ndata->lentry = entry;
        g_object_set_data(G_OBJECT(entry), "page", (gpointer)page);
        g_signal_connect(G_OBJECT(entry), "activate",
                         G_CALLBACK(on_chlimit_clicked), NULL);

/*     label = ui_create_event_label */
/*       ("change user limit", G_CALLBACK(on_chlimit_clicked), entry, TRUE); */
/*     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0); */

        label = gtk_label_new(_("(0 = unlimited)"));
        gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);

        label = gtk_label_new(_("you are not operator!"));
        ndata->info_label = GTK_LABEL(label);
        gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);

        gtk_widget_show_all(hbox);

        if (net->level <= NAP_L_USER)
            gtk_widget_show(GTK_WIDGET(ndata->info_label));
    }
    //
  
    ui_chat_page_add
        (PAGE(page),
         ndata->public_settings, tt,
         ui_create_show_hide_label(_("show op-chat"), _("hide op-chat"), ndata->chop),
         ui_create_show_hide_label(_("show users"), _("hide users"), ndata->users),
         NULL
        );
  
    object_notify_d(OBJ(page), (ObjectFunc)ui_nap_public_remove, NULL);
}

void ui_nap_cp_update_op(nap_pub_t* npp)
{
    nap_public_gui_t* data;
    gboolean opped;
    int i1;
    
    if (!npp || NCP(npp)->type != NAP_PAGE_PUBLIC) return;
    data = object_get_data(OBJ(npp), "ChatPage");
    if (!data || !data->public_settings) return;
    opped = ((NCP(npp)->net && NCP(npp)->net->level >= NAP_L_MOD) || npp->opped);
  
    gtk_widget_set_sensitive(data->public_settings, opped);
    if (opped)
        gtk_widget_hide(GTK_WIDGET(data->info_label));
    else
        gtk_widget_show(GTK_WIDGET(data->info_label));

    for (i1 = 0; i1 < NAP_CMODE_NUMBER; i1++)
    {
        if (npp->chmode & (1<<i1))
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->mbutton[i1]), TRUE);
        else
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(data->mbutton[i1]), FALSE);
    }
    
    gtk_entry_set_text(GTK_ENTRY(data->lentry), l_printf("%d", npp->limit));
    if (NapIsLevel(npp->level))
        gtk_combo_box_set_active(GTK_COMBO_BOX(data->combo), npp->level);
}

static void ui_nap_update_counter(GtkLabel* label, int counter)
{
    const char* l = l_printf(ngettext("%d user", "%d users", counter), counter);
    gtk_label_set_markup(label, l_printf("<b>%s</b>", l));
}

void ui_nap_channel_joined(nap_pub_t* npp)
{
    nap_public_gui_t* data;
    
    if (!npp) return;
    data = object_get_data(OBJ(npp), "ChatPage");
    if (!data) return;
    if (data->online_store) 
        gtk_list_store_clear(data->online_store);
    ui_nap_update_counter(GTK_LABEL(data->counter), npp->online);

    ui_nap_cp_update_op(npp);
}

static void ui_nap_user_hide(nap_cu_t* cu, nap_pub_t* npp)
{
    nap_public_gui_t* data;
    GtkTreeIter* iter;
    
    if (!npp || !cu) return;
    data = object_get_data(OBJ(npp), "ChatPage");
    if (!data || !data->online_store) return;
  
    iter = object_get_data(OBJ(cu), "ChatUserEntry");
    if (!iter) return;
    
    gtk_list_store_remove(data->online_store, iter);
    ui_nap_update_counter(GTK_LABEL(data->counter), npp->online);
    object_free_data_common(OBJ(cu), "ChatUserEntry");
}

static void ui_nap_user_update(nap_cu_t* cu, nap_pub_t* npp)
{
    nap_public_gui_t* data;
    GtkTreeIter* iter;
    
    if (!npp) return;
    data = object_get_data(OBJ(npp), "ChatPage");
    if (!data || !data->online_store) return;
  
    iter = object_get_data(OBJ(cu), "ChatUserEntry");
    if (!iter) return;

    gtk_list_store_set(data->online_store, iter,
                       0, cu,
                       1, OBJ(cu)->name,
                       2, cu->shared,
                       3, _(NapSpeedStr(cu->speed)),
                       -1);
}

static void ui_nap_user_show(nap_cu_t* cu, nap_pub_t* npp)
{
    nap_public_gui_t* data;
    GtkTreeIter* iter;
    
    data = object_get_data(OBJ(npp), "ChatPage");
    if (!data || !data->online_store) return;
  
    if (object_get_data(OBJ(cu), "ChatUserEntry")) return;

    object_set_data(OBJ(cu), "ChatUserEntry", iter = g_malloc(sizeof(*iter)));
    gtk_list_store_append(data->online_store, iter);
    ui_nap_update_counter(GTK_LABEL(data->counter), npp->online);
    
    ui_nap_user_update(cu, npp);
}

void ui_nap_user_new(nap_cu_t* cu, nap_pub_t* npp)
{
    g_assert(npp && cu);
    
    object_notify_o(OBJ(cu), (ObjectFunc)ui_nap_user_hide, OBJ(npp)); // of no use?
    object_monitor_o(OBJ(cu), (ObjectFunc)ui_nap_user_update, OBJ(npp));
    
    ui_nap_user_show(cu, npp);
}

void ui_nap_new_topic(nap_pub_t* npp)
{
    nap_public_gui_t* data = object_get_data(OBJ(npp), "ChatPage");
    if (!data) return;
    gtk_label_set_text(GTK_LABEL(data->topic), npp->topic?npp->topic:"");
}


void ui_nap_user_parted(nap_pub_t* npp, nap_cu_t* cu)
{
    if (!npp || !cu) return;

    if (!(pref_get_ref("/core/chat/hide", int) & HIDE_PARTS)){
        chat_print(PAGE(npp),
                   "+\0part",
                   "+\1nick", OBJ(cu)->name, "[]",
                   "+\1c_part", _("has left"),
                   "+\1c_text", "(",
                   "+\0c_text", l_printf("%d", cu->shared),
                   "+\0c_text", "/",
                   "+\0c_text", NapSpeedStrShort(cu->speed),
                   "+\0c_text", ")",
                   "+\0nl", NULL);
    }
}

void ui_nap_user_joined(nap_pub_t* npp, nap_cu_t* cu)
{
    if (!(pref_get_ref("/core/chat/hide", int) & HIDE_JOINS)){
        chat_print(PAGE(npp),
                   "+\0join",
                   "+\1nick", OBJ(cu)->name, "[]",
                   "+\1c_join", _("has joined"),
                   "+\1c_text", "(",
                   "+\0c_text", l_printf("%d", cu->shared),
                   "+\0c_text", "/",
                   "+\0c_text", NapSpeedStrShort(cu->speed),
                   "+\0c_text", ")",
                   "+\0nl", NULL);
    }
}

static char* int2chmode(int set, int clear)
{
    static char mode[2048];
    char* pos;
    int cnt;
    
    pos = mode;
    
    cnt = 0;
    if (set) *pos++ = '+';
    while (set)
    {
        if (set & 1) *pos++ = NapCModeChars[cnt];
        cnt++;
        set >>= 1;
    }
    cnt = 0;
    if (clear) *pos++ = '-';
    while (clear)
    {
        if (clear & 1) *pos++ = NapCModeChars[cnt];
        cnt++;
        clear >>= 1;
    }
    
    *pos = 0;
    
    if (*mode) return mode;
    else return NULL;
}


void ui_nap_cp_update_mode(nap_pub_t* npp)
{
    nap_public_gui_t* data = object_get_data(OBJ(npp), "ChatPage");
    int i1;
    
    if (!data) return;

    for (i1 = 0; i1 < NAP_CMODE_NUMBER; i1++)
    {
        gtk_toggle_button_set_active
            (GTK_TOGGLE_BUTTON(data->mbutton[i1]), (npp->chmode & (1<<i1)));
    }
    chat_print(PAGE(npp),
               "+\0server",
               "+\1c_text", _("Channel mode :"),
               "+\1c_server", int2chmode(npp->chmode, 0),
               "+\0nl", NULL);
}

void ui_nap_cp_update_limit(nap_pub_t* npp)
{
    nap_public_gui_t* data = object_get_data(OBJ(npp), "ChatPage");
    char* text;
    
    if (!data) return;

    text = l_printf("%d", npp->limit);
    gtk_entry_set_text(GTK_ENTRY(data->lentry), text);

    chat_print(PAGE(npp),
               "+\0server",
               "+\1c_text", _("Channel limit:"),
               "+\1c_server", text,
               "+\0nl", NULL);
}

void ui_nap_cp_update_level(nap_pub_t* npp)
{
    nap_public_gui_t* data = object_get_data(OBJ(npp), "ChatPage");

    if (!data) return;

    if (NapIsLevel(npp->level))
        gtk_combo_box_set_active(GTK_COMBO_BOX(data->combo), npp->level);
    
    chat_print(PAGE(npp), "+\0server",
               "+\1c_text", _("Channel level:"),
               "+\1c_server", NapLevel(npp->level),
               "+\0nl", NULL);
}




typedef struct {
  // channel list and settings gui
  GtkWindow* win;
  GtkTreeViewColumn* col[5];
  GtkListStore* channel_store;
  GtkLabel* nlabel;
  GtkWidget* rlabel;
  GtkSpinButton* min_spin;
  GtkSpinButton* max_spin;
  GtkWidget* alabel;
/*   GtkWidget* clabel; */
} nap_channel_gui_t;

static void on_channel_list_apply(nap_channels_t* channels)
{
    nap_channel_gui_t* ndata = object_get_data(OBJ(channels), "ChannelList");
    if (ndata)
    {
        channels->min = gtk_spin_button_get_value(GTK_SPIN_BUTTON(ndata->min_spin));
        channels->max = gtk_spin_button_get_value(GTK_SPIN_BUTTON(ndata->max_spin));
        object_set_dirty(OBJ(channels));
    }
}

static void ui_nap_channels_update(nap_channels_t* channels)
{
    nap_channel_gui_t* ndata;
    GtkTreeIter iter;
    int shown=0, hidden=0;
    
    ndata = object_get_data(OBJ(channels), "ChannelList");
    if (!ndata) return;

    gtk_window_present(ndata->win);
    
    gtk_list_store_clear(ndata->channel_store);
    if (!channels->list && !channels->refreshing)
    {
        gtk_list_store_append(ndata->channel_store, &iter);
        gtk_list_store_set(ndata->channel_store, &iter,
                           0, NULL,
                           1, _("No Entries found!"),
                           -1);
    }
    else
    {
        GList* dlist;
        
        for (dlist = channels->list; dlist; dlist = dlist->next)
        {
            nap_ce_t* channel = dlist->data;
            if (channel->online < channels->min ||
                (channels->max && channel->online > channels->max))
            {
                hidden++;
                continue;
            }
            shown++;
            
            gtk_list_store_append(ndata->channel_store, &iter);
            gtk_list_store_set(ndata->channel_store, &iter,
                               0, channel,
                               1, channel->name,
                               2, channel->online,
                               3, channel->limit,
                               4, NapLevelShort(channel->level),
                               5, channel->topic,
                               -1);
        }
    }
    if (channels->refreshing)
    {
        gtk_label_set_text(GTK_LABEL(ndata->nlabel), _("Refreshing list..."));
        ui_event_label_set_sensitive(ndata->rlabel, FALSE);
        ui_event_label_set_sensitive(ndata->alabel, FALSE);
    }
    else
    {
        gtk_label_set_text(GTK_LABEL(ndata->nlabel), 
                           l_printf(_("%d shown - %d hidden"), shown, hidden));
        ui_event_label_set_sensitive(ndata->rlabel, TRUE);
        ui_event_label_set_sensitive(ndata->alabel, TRUE);
    }
}

static void ui_nap_channels_destroy(nap_channels_t* channels)
{
    nap_channel_gui_t* ndata = object_get_data(OBJ(channels), "ChannelList");
    
    if (ndata)
    {
        ui_layout_unmonitor("nap:chan:col:name:size", ndata->col[0]);
        ui_layout_unmonitor("nap:chan:col:online:size", ndata->col[1]);
        ui_layout_unmonitor("nap:chan:col:limit:size", ndata->col[2]);
        ui_layout_unmonitor("nap:chan:col:level:size", ndata->col[3]);
        ui_layout_unmonitor("nap:chan:col:topic:size", ndata->col[4]);
        ui_layout_unmonitor("nap:chan:col:name:vis", ndata->col[0]);
        ui_layout_unmonitor("nap:chan:col:online:vis", ndata->col[1]);
        ui_layout_unmonitor("nap:chan:col:limit:vis", ndata->col[2]);
        ui_layout_unmonitor("nap:chan:col:level:vis", ndata->col[3]);
        ui_layout_unmonitor("nap:chan:col:topic:vis", ndata->col[4]);
        
        object_free_data_common(OBJ(channels), "ChannelList");
    }
}

static void ui_nap_channels_destroyed(nap_channels_t* channels)
{
    nap_channel_gui_t* ndata = object_get_data(OBJ(channels), "ChannelList");
    
    if (ndata)
    {
        GtkWidget* win = GTK_WIDGET(ndata->win);
        gtk_widget_destroy(win);
    }
}


void ui_nap_channel_list_new(nap_channels_t* channels)
{
    GtkWidget* win;
    GtkWidget* hbox;
    GtkWidget* vbox;
    GtkWidget* vbox2;
    GtkWidget* item;
    GtkWidget* sw;
    nap_channel_gui_t* ndata;
    GtkObject *adj;
    GtkWidget* tv;
    GtkCellRenderer* renderer;
    GtkTreeViewColumn* column;
    GtkListStore* store;
    GtkTreeSelection* select;
    const char* stock;
    char* temp;
    
    ndata = object_get_data(OBJ(channels), "ChannelList");
    if (ndata)
    {
        gtk_window_present(ndata->win);
        return;
    }
    object_set_data(OBJ(channels), "ChannelList", ndata = g_malloc(sizeof(*ndata)));

    win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
    gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
    if (ui_main_get_win())
    {
        gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
        gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
    }

    ndata->win = GTK_WINDOW(win);
    
    temp = l_printf(_("%s - %s - List of channels"),
                    plugin_get_name(plugin_get(NAP_ID)), OBJ(channels->net)->name);
    gtk_window_set_title(GTK_WINDOW (win), temp);
    gtk_window_set_default_size(GTK_WINDOW(win), 600, 400);

    g_signal_connect_swapped(G_OBJECT(win), "destroy",
                             G_CALLBACK(ui_nap_channels_destroy), channels);
    
    vbox = gtk_vbox_new(FALSE, 10);
    gtk_widget_show(vbox);
    gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
    gtk_container_add(GTK_CONTAINER(win), vbox);
    
    // the header line
    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    stock = ui_main_get_plugin_stock(NAP_ID);
    if (stock)
    {
        item = gtk_image_new_from_stock(stock, GTK_ICON_SIZE_MENU);
        gtk_widget_show(item);
        gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    }
    temp = g_markup_printf_escaped(_("<b>%s - List of channels</b>"), OBJ(channels->net)->name);
    item = gtk_label_new(temp);
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    g_free(temp);
    
    item = ui_create_event_label
        (_("refresh list"), G_CALLBACK(nap_net_get_channels), channels->net, TRUE);
    gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    gtk_widget_show(item);
    ndata->rlabel = item;
    
    vbox2 = gtk_vbox_new(FALSE, 10);
    gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, FALSE, 0);
    
    item = ui_create_show_hide_label(_("show filter"), _("hide filter"), vbox2);
    gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    gtk_widget_show(item);
    
    item = gtk_label_new("");
    gtk_widget_show(item);
    gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    ndata->nlabel = GTK_LABEL(item);
    
    // the control section line
    /*   gtk_container_set_border_width(GTK_CONTAINER(controls), 5); */
    item = gtk_hseparator_new();
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);
    
    hbox = gtk_hbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
    gtk_widget_show(hbox);
    
    item = gtk_label_new(_("<b>Filter</b>"));
    gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    gtk_widget_show(item);
    
/*   item = gtk_label_new ("name"); */
/*   gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0); */
/*   gtk_widget_show(item); */
    
/*   item = gtk_entry_new(); */
/*   gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0); */
/*   gtk_widget_show(item); */
/*   ndata->nentry = GTK_ENTRY(item); */
    
    item = gtk_label_new(_("min/max users"));
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    gtk_widget_show(item);
    
    adj = gtk_adjustment_new(channels->min, 0, 10000, 1, 100, 100);
    item = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    gtk_widget_show(item);
    ndata->min_spin = GTK_SPIN_BUTTON(item);
    
    adj = gtk_adjustment_new(channels->max, 0, 10000, 1, 100, 100);
    item = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    gtk_widget_show(item);
    ndata->max_spin = GTK_SPIN_BUTTON(item);
    
/*   item = gtk_label_new ("topic"); */
/*   gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0); */
/*   gtk_widget_show(item); */

/*   item = gtk_entry_new(); */
/*   gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0); */
/*   gtk_entry_set_text(GTK_ENTRY(item), "<not implemented>"); */
/*   gtk_widget_set_sensitive(item, FALSE); */
/*   gtk_widget_show(item); */

    item = ui_create_event_label
        (_("apply"), G_CALLBACK(on_channel_list_apply), channels, TRUE);
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    ndata->alabel = item;
    
  /*   label = ui_create_event_label */
  //     ("reset", NULL/*G_CALLBACK(on_channel_list_apply)*/, page, TRUE);
  /*   gtk_widget_show (label); */
  /*   gtk_box_pack_start(GTK_BOX (hbox), label, FALSE, FALSE, 0); */

    
    // the channel list
    sw = gtk_scrolled_window_new (NULL, NULL);
    gtk_widget_show(sw);
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
    gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
    
    store = gtk_list_store_new(6,
                               G_TYPE_POINTER,  // data pointer
                               G_TYPE_STRING,   // name
                               G_TYPE_INT,      // online
                               G_TYPE_INT,      // limit
                               G_TYPE_STRING,   // level
                               G_TYPE_STRING);  // topic
    ndata->channel_store = store;
    
    tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
    //  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
    gtk_container_add(GTK_CONTAINER(sw), tv);
    gtk_widget_show(tv);
    gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW(tv), TRUE);
    //  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tv), TRUE);
    select = gtk_tree_view_get_selection(GTK_TREE_VIEW(tv));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
    
    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes 
        (_("Channel"), renderer, "text", 1, NULL);
    ui_tv_config_default_col(column, 1, "nap:chan:col:name", FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW(tv), column);
    ndata->col[0] = column;
    
    renderer = gtk_cell_renderer_text_new();
    column = gtk_tree_view_column_new_with_attributes 
        (_("Users"), renderer, "text", 2, NULL);
    ui_tv_config_default_col(column, 2, "nap:chan:col:online", FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
    ndata->col[1] = column;
    
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes 
        (_("Limit"), renderer, "text", 3, NULL);
    ui_tv_config_default_col(column, 3, "nap:chan:col:limit", FALSE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    ndata->col[2] = column;
    
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes 
        (_("Level"), renderer, "text", 4, NULL);
    ui_tv_config_default_col(column, 4, "nap:chan:col:level", FALSE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    ndata->col[3] = column;
    
    renderer = gtk_cell_renderer_text_new ();
    column = gtk_tree_view_column_new_with_attributes 
        (_("Topic"), renderer, "text", 5, NULL);
    ui_tv_config_default_col(column, 5, "nap:chan:col:topic", FALSE);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    ndata->col[4] = column;
    
    {
        action_item_t* actions = action_item_search("Nap channel list actions");
        ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &actions, channels);
    }
    
    
    hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    
    item = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
    gtk_container_add(GTK_CONTAINER(hbox), item);
    gtk_widget_show(item);

    g_signal_connect_swapped(G_OBJECT(item), "clicked",
                             G_CALLBACK(gtk_widget_destroy), win);
    
    gtk_widget_show(win);
    
    object_monitor_d(OBJ(channels), (ObjectFunc)ui_nap_channels_update, win);
    object_notify_d(OBJ(channels), (ObjectFunc)ui_nap_channels_destroyed, NULL);
}

/* void ui_nap_create_private_page(nap_cp_t* ncp) { */
/*   ui_chat_create_simple_page */
/*     (PAGE(ncp), l_printf(_("%s - Operator messages"), OBJ(net)->name)); */
/* } */
