/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.5 $
 * $Date: 2006/07/22 17:14:04 $
 * $Author: sid4u $
 */

#include <string.h>

#include <gtk/gtk.h>

#include "lcore/buddy.h"
#include "lcore/attribute.h"
#include "lcore/preferences.h"
#include "lcore/xml_handler.h"
#include "lcore/l_i18n.h"
#include "lcore/legacy.h"
#include "lcore/gtk2_ui/ui_buddy.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "lcore/gtk2_ui/ui_init.h"
#include "nap.h"
#include "nap_hotlist.h"
#include "nap_chat.h"
#include "nap_whois.h"
#include "nap_browse.h"

typedef struct {
  GtkToggleButton* timport;
  GtkToggleButton* tnothing;
  gboolean done;
} ui_hot_t;

static GtkEntry* NickEntry = NULL;
static int Row = 0;

static GtkWidget* ui_nap_buddy_edit(buddy_t* buddy) {
  GtkWidget* table;
  GtkWidget* entry;
  GtkWidget* item;
  attribute_t* attr;

  table = gtk_table_new(1, 1, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);

  entry = gtk_entry_new();
  NickEntry = GTK_ENTRY(entry);
  attr = attr_find(ATTRC(buddy), "buddy", "NapNick");
  if (attr) gtk_entry_set_text(NickEntry, attr_get_ptr(attr, char));
  g_signal_connect(G_OBJECT(entry), "destroy",
                   G_CALLBACK(gtk_widget_destroyed), &NickEntry);

  item = ui_create_headed_item(_("monitor nicks (space separated list)"), entry);
  gtk_table_attach(GTK_TABLE(table), item, 0, 1, 0, 1,
                   GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

  gtk_widget_show_all(table);
  return table;
}

static void ui_nap_buddy_apply(buddy_t* buddy) {
  const char* nick;

  if (!buddy || !NickEntry) return;

  nick = gtk_entry_get_text(NickEntry);

  if (nick && *nick) {
    attr_store(ATTRC(buddy), "buddy", "NapNick", nick);
  } else {
    attribute_t* attr = attr_find(ATTRC(buddy), "buddy", "NapNick");
    if (attr) attr_remove(ATTRC(buddy), attr);
  }
}

static void on_buddy_chat(GObject* item, nap_user_t* user) {
  nap_net_t* net = g_object_get_data(item, "net");
  if (net && user) nap_cp_new_private(net, OBJ(user)->name);
}
static void on_buddy_whois(GObject* item, nap_user_t* user) {
  nap_net_t* net = g_object_get_data(item, "net");
  if (net && user)
    nap_whois_user(user, net, NAP_WHOIS_WINDOW, FALSE);
}
static void on_buddy_browse(GObject* item, nap_user_t* user) {
  nap_net_t* net = g_object_get_data(item, "net");
  if (net) nap_user_set_net(user, net);
  if (user) nap_browse_user(user);
}

static GtkWidget* ui_create_chat_popup(nap_user_t* user) {
  GtkWidget* popup;
  GList* dlist;
  
  popup = gtk_menu_new();
  for (dlist = user->nets; dlist; dlist = dlist->next) {
    nap_net_t* net = dlist->data;
    GtkWidget* item = gtk_menu_item_new_with_label(OBJ(net)->name);
    
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup), item);

    g_object_set_data(G_OBJECT(item), "net", net);
    g_signal_connect(G_OBJECT(item), "activate",
		     G_CALLBACK(on_buddy_chat), user);
  }

  return popup;
}
static GtkWidget* ui_create_whois_popup(nap_user_t* user) {
  GtkWidget* popup;
  GList* dlist;
  
  popup = gtk_menu_new();
  for (dlist = user->nets; dlist; dlist = dlist->next) {
    nap_net_t* net = dlist->data;
    GtkWidget* item = gtk_menu_item_new_with_label(OBJ(net)->name);
    
    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup), item);

    g_object_set_data(G_OBJECT(item), "net", net);
    g_signal_connect(G_OBJECT(item), "activate",
		     G_CALLBACK(on_buddy_whois), user);
  }

  return popup;
}
static GtkWidget* ui_create_browse_popup(nap_user_t* user) {
  GtkWidget* popup;
  GList* dlist;
  
  popup = gtk_menu_new();
  for (dlist = user->nets; dlist; dlist = dlist->next) {
    nap_net_t* net = dlist->data;
    GtkWidget* item = gtk_menu_item_new_with_label(OBJ(net)->name);

    gtk_widget_show(item);
    gtk_container_add(GTK_CONTAINER(popup), item);

    g_object_set_data(G_OBJECT(item), "net", net);
    g_signal_connect(G_OBJECT(item), "activate",
		     G_CALLBACK(on_buddy_browse), user);
  }

  return popup;
}

static gboolean ui_nap_user_show(nap_user_t* user, GtkTable* table) {
  GtkWidget* hbox;
  GtkWidget* label;
  user_t* u = USER(user);

  label = gtk_label_new(u->print(u));
  gtk_widget_set_name(label, "OnlineLabel");
  if (u->online) gtk_widget_set_state(label, GTK_STATE_ACTIVE);
  else gtk_widget_set_state(label, GTK_STATE_NORMAL);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, Row, Row+1,
                   GTK_FILL, GTK_FILL, 0, 0);

  hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, Row, Row+1,
                   GTK_FILL, GTK_FILL, 0, 0);
  label = gtk_label_new(u->online?_("is online"):_("is not active"));
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
  
  if (u->online) {
    GtkWidget* button =
      ui_create_popup_button(gtk_label_new(_("chat on")), POS_LEFT_DOWN,
                             (PopupFunction)ui_create_chat_popup, user);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    
    button =
      ui_create_popup_button(gtk_label_new(_("whois on")), POS_LEFT_DOWN,
                             (PopupFunction)ui_create_whois_popup, user);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    
    button =
      ui_create_popup_button(gtk_label_new(_("browse on")), POS_LEFT_DOWN,
                             (PopupFunction)ui_create_browse_popup, user);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
    
  }
  Row++;
  return FALSE;
}

static GtkWidget* ui_nap_buddy_show(buddy_t* buddy) {
  GtkWidget* table;
  
  g_assert(buddy);
  if (!buddy_has_user(buddy, NAP_ID)) return NULL;

  table = gtk_table_new(1, 2, FALSE);
  gtk_widget_show(table);
/*   gtk_container_set_border_width(GTK_CONTAINER(table), 5); */
  //  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table), 2);

  Row = 0;
  buddy_foreach_user(buddy, NAP_ID, (UserForeach)ui_nap_user_show, table);
  
  return table;
}


static GtkWidget* ui_hot_create(ui_page_t* page) {
  GtkWidget* toggle;
  GtkWidget* vbox;
  ui_hot_t* data = g_malloc0(sizeof(*data));

  ui_page_set_data(page, data);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);

  toggle = gtk_radio_button_new_with_label
    (NULL, _("Import hotlist from Lopster 1"));
  data->timport = GTK_TOGGLE_BUTTON(toggle);
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);

  toggle = gtk_radio_button_new_with_label
    (gtk_radio_button_get_group(GTK_RADIO_BUTTON(toggle)), _("Don't import the hotlist"));
  data->tnothing = GTK_TOGGLE_BUTTON(toggle);
  gtk_widget_show(toggle);
  gtk_box_pack_start(GTK_BOX(vbox), toggle, FALSE, FALSE, 0);

  gtk_toggle_button_set_active(data->timport, TRUE);

  return vbox;
}

static const char* ui_hot_enter(ui_page_t* page, gboolean next) {
  ui_hot_t* data = ui_page_get_data(page);
  (void)next;
  if (data->done) {
    gtk_widget_set_sensitive(GTK_WIDGET(data->timport), FALSE);
    gtk_widget_set_sensitive(GTK_WIDGET(data->tnothing), FALSE);
    return _("The hotlist already was imported from Lopster 1");
  }
  return NULL;
}

static void ui_hot_add(const char* value) {
  char* name = l_strdup(value);
  char* des = strchr(name, ':');

  if (des) *des++ = 0;
  nap_hot_new(nap_user_find(name, NULL), des);
}

static const char* ui_hot_leave(ui_page_t* page, gboolean next) {
  ui_hot_t* data = ui_page_get_data(page);

  if (!next) return NULL;
  if (data->done) return NULL;
  if (!gtk_toggle_button_get_active(data->timport)) return NULL;

  legacy_pref_foreach("User", ui_hot_add);
  
  return NULL;
}

void ui_nap_hotlist_init() {

  ui_buddy_register_funcs(NAP_ID, ui_nap_buddy_edit,
                          ui_nap_buddy_apply,
                          ui_nap_buddy_show
                          );
  
  if (plugin_get_old_version(plugin_get(NAP_ID)) != 0 ||
      lopster_xml_loaded("core:buddies")) return;

  if (legacy_pref_available()) {
    ui_init_add(_("Napster hotlist"),
                _("If you wish Lopster 2 can import your hotlist from Lopster 1."),
                ui_hot_create, ui_hot_enter, ui_hot_leave);
  }

}
