/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.8 $
 * $Date: 2007/04/29 09:28:37 $
 * $Author: sgop $
 */

#include <gtk/gtk.h>

#include "lcore/l_i18n.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_preferences.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "ui_nap_pref.h"
#include "nap_network.h"
#include "nap.h"

static GtkWidget* ui_general_user(gboolean* expand)
{
    GtkWidget* vbox;
    GtkWidget* hbox;
    GtkWidget* table;
    GtkWidget* item;
    
    vbox = ui_create_section
        (_("User settings"), 
         _("If you leave the user or password empty then the global "
           "default setting is used. The dataport specifies the port "
           "the napster module will accept connections. This port must "
           "lie in the allowed ports range from the core. If you don't "
           "like to accept connections you have to deactivate it in the core settings."),
         &hbox, NULL);
    
    table = gtk_table_new(1, 3, FALSE);
    gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 0);
    gtk_table_set_col_spacings(GTK_TABLE(table), 5);
    
    item = ui_create_headed_item
        (_("username"), ui_pref_string_entry("/nap/username", _("Using default username")));
    gtk_table_attach(GTK_TABLE(table), item, 0, 1, 0, 1,
                     GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);
    
    item = ui_create_headed_item
        (_("password"), ui_pref_string_entry("/nap/password", _("Using default password")));
    gtk_table_attach(GTK_TABLE(table), item, 1, 2, 0, 1,
                     GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

    item = ui_create_headed_item
        (_("data port"), ui_pref_int_spin("/nap/listen/port", 1, 65535));
    gtk_table_attach(GTK_TABLE(table), item, 2, 3, 0, 1,
                     GTK_FILL,GTK_FILL,0,0);
    
    gtk_widget_show_all(vbox);
    
    if (expand) *expand = FALSE;
    return vbox;
}

static GtkWidget* ui_general_chat(gboolean* expand) {
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* table;
  GtkWidget* item;
  
  vbox = ui_create_section
    (_("Chat settings and user mode"),
     _("The min and max channel users setting is a default "
       "filter for the channel list page."),
     &hbox, NULL);

  table = gtk_table_new(1, 3, FALSE);
  gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 0);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);

  item = ui_create_headed_item
    (_("min channel users"),
     ui_pref_int_spin("/nap/chat/channels/min", 0, G_MAXINT));
  gtk_table_attach(GTK_TABLE(table), item, 0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

  item = ui_create_headed_item
    (_("max channel users"),
     ui_pref_int_spin("/nap/chat/channels/max", 0, G_MAXINT));
  gtk_table_attach(GTK_TABLE(table), item, 1, 2, 0, 1,
		   GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

  item = ui_create_headed_item
    (_("user mode"),
     ui_pref_int_flags("/nap/usermode", FALSE, POS_RIGHT_DOWN,
                       17, _("Click to set ..."), 0,
                       0, NapUserModes(0), (1<<0),
                       0, NapUserModes(1), (1<<1),
                       0, NapUserModes(2), (1<<2),
                       0, NapUserModes(3), (1<<3),
                       0, NapUserModes(4), (1<<4),
                       0, NapUserModes(5), (1<<5),
                       0, NapUserModes(6), (1<<6),
                       0, NapUserModes(7), (1<<7),
                       0, NapUserModes(8), (1<<8),
                       0, NapUserModes(9), (1<<9),
                       0, NapUserModes(10), (1<<10),
                       0, NapUserModes(11), (1<<11),
                       0, NapUserModes(12), (1<<12),
                       0, NapUserModes(13), (1<<13),
                       0, NULL, 0,
                       1, _("Additional Opennap-ng modes"), 0,
                       0, NapUserModes(21), (1<<21),
                       7, _("Additional SlavaNap modes"), 0,
                       0, NapUserModes(14), (1<<14),
                       0, NapUserModes(15), (1<<15),
                       0, NapUserModes(16), (1<<16),
                       0, NapUserModes(17), (1<<17),
                       0, NapUserModes(17), (1<<18),
                       0, NapUserModes(19), (1<<19),
                       0, NapUserModes(20), (1<<20)));
  
  gtk_table_attach(GTK_TABLE(table), item, 2, 3, 0, 1,
		   GTK_FILL,GTK_FILL|GTK_EXPAND,0,0);
  
  gtk_widget_show_all(vbox);

  if (expand) *expand = FALSE;
  return vbox;
}

static GtkWidget* ui_general_charset(gboolean* expand) {
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* table;
  GtkWidget* item;
  
  vbox = ui_create_section
    (_("Default character set"),
     _("Lopster needs to know how to convert messages from other"
       "users into the local format.\n\n"
       "If unsure or if lopster warns you, try 'ISO-8859-1'\n\n"
       "Note: This is a default setting for all networks, you can override it "
       "for every network in the 'edit network' dialog."),
     &hbox, NULL);

  table = gtk_table_new(1, 1, FALSE);
  gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 0);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);

  item = ui_create_headed_item
    (_("Character encoding"), ui_pref_charset_combo("/nap/charset"));
  gtk_table_attach(GTK_TABLE(table), item, 0, 1, 0, 1,
                   GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

  gtk_widget_show_all(vbox);
  if (expand) *expand = FALSE;
  return vbox;
}

void ui_nap_pref_init() {
  ui_pref_hook_plugin
    (NAP_ID, ui_pref_register_leaf
     (    "Napster", FALSE, _("Settings for the opennap network plugin"),
          ui_general_user,
          ui_general_chat,
          ui_general_charset,
          NULL)
     );
}

