/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of
 * the GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.5 $
 * $Date: 2007/04/29 09:28:37 $
 * $Author: sgop $
 */

#include <string.h>

#include <gtk/gtk.h>

#include "lcore/l_i18n.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "nap_whois.h"
#include "nap_browse.h"
#include "nap_chat.h"
#include "nap_network.h"

typedef struct {
  nap_user_t* user;
  GtkWidget* win;
  GtkWidget* main;
  GtkWidget* refresh;
} ui_nap_whois_t;

static GList* Whois = NULL;

static ui_nap_whois_t* ui_nap_whois_search(nap_user_t* user) {
  GList* dlist;
  for (dlist = Whois; dlist; dlist = dlist->next) {
    ui_nap_whois_t* w = dlist->data;
    if (w->user == user) return w;
  }
  return NULL;
}

static gboolean on_whois_close(ui_nap_whois_t* w) {
  Whois = g_list_remove(Whois, w);
  gtk_widget_destroy(w->win);
  user_unref(USER(w->user));
  g_free(w);
  return TRUE;
}

static ui_nap_whois_t* ui_nap_whois_new(nap_user_t* user) {
  GtkWidget* win;
  ui_nap_whois_t* w;
  
  w = g_malloc(sizeof(*w));
  w->user = user_ref(USER(user));
  Whois = g_list_append(Whois, w);

  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (win), _("Whois"));
  gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
  if (ui_main_get_win())
  {
      gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
      gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
  }
  g_signal_connect_swapped(G_OBJECT(win), "delete-event",
                           G_CALLBACK(on_whois_close), w);

  w->win = win;
  w->main = NULL;
  return w;
}

static gboolean on_refresh(ui_nap_whois_t* w) {
  nap_whois_user(w->user, NAP_WHOIS(w->user->whois)->net,
                 NAP_WHOIS_WINDOW, TRUE);
  return TRUE;
}

static gboolean on_chat(ui_nap_whois_t* w) {
  nap_whois_t* whois = w->user->whois;
  nap_cp_new_private(whois->net, OBJ(w->user)->name);
  return TRUE;
}
static gboolean on_browse(ui_nap_whois_t* w) {
  nap_whois_t* whois = w->user->whois;
  if (whois->net) nap_user_set_net(w->user, whois->net);
  nap_browse_user(w->user);
  return TRUE;
}

static void ui_nap_whois_show_real(ui_nap_whois_t* w) {
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* label;
  GtkWidget* button;
  GtkWidget* sep;
  nap_user_t* user = w->user;
  nap_whois_t* whois = user->whois;

  if (w->main) gtk_widget_destroy(w->main);

  vbox = gtk_vbox_new(FALSE, 10);
  w->main = vbox;
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  gtk_container_add(GTK_CONTAINER(w->win), vbox);
  
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  
  label = gtk_label_new(l_printf("<b>%s</b>", OBJ(user)->name));
  gtk_widget_set_name(label, "LightLabel");
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  
  if (whois->status == NAP_WHOIS_TIMEOUT) {
    label = gtk_label_new(_("timed out"));
  } else if (whois->status == NAP_WHOIS_ONLINE) {
    label = gtk_label_new(_("is online"));
  } else if (whois->status == NAP_WHOIS_OFFLINE) {
    label = gtk_label_new(_("is offline"));
  } else {
    label = gtk_label_new(_("is unknown"));
  }
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
    
  if (whois->status == NAP_WHOIS_ONLINE) {
    label = ui_create_event_label
      (_("browse"), G_CALLBACK(on_browse), w, TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

    label = ui_create_event_label
      (_("chat"), G_CALLBACK(on_chat), w, TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  }

  label = ui_create_event_label
    (_("refresh"), G_CALLBACK(on_refresh), w, TRUE);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  if (whois->timer) gtk_widget_set_sensitive(label, FALSE);
  
  if (whois->timer) {
    label = gtk_label_new(l_printf(_("currently refreshing on %s"),
                                   whois->net?OBJ(whois->net)->name:"???"));
  } else if (whois->status == NAP_WHOIS_ONLINE) {
    label = gtk_label_new(l_printf(_("currently on network %s"),
                                   whois->net?OBJ(whois->net)->name:"???"));
  } else {
    label = gtk_label_new(l_printf(_("currently not on network %s"),
                                   whois->net?OBJ(whois->net)->name:"???"));
  }
  //  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);

  sep = gtk_hseparator_new();
  gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);

  if (whois->status == NAP_WHOIS_ONLINE) {
    GtkWidget* table = gtk_table_new(8, 4, FALSE);

    gtk_table_set_col_spacings(GTK_TABLE(table), 16);
    gtk_table_set_row_spacings(GTK_TABLE(table), 5);
    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
  
    label = gtk_label_new(_("<b>Level:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new
      (l_printf("%s (%s)", whois->str_level?whois->str_level:"?",
                NapLevel(whois->level)));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 2, 0, 1,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Status:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 2, 3, 0, 1,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(whois->stat);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, 0, 1,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Shared:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(l_printf(_("%d files"), user->shared));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 2, 1, 2,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Connection:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 2, 3, 1, 2,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_(NapSpeedStr(user->speed)));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, 1, 2,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Channels:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(whois->channel);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 4, 2, 3,
                     GTK_FILL, GTK_FILL, 0, 0);
      
    label = gtk_label_new(_("<b>Online:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(print_duration_unit_exact(&(whois->online)));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 2, 3, 4,
                     GTK_FILL, GTK_FILL, 0, 0);
    
    label = gtk_label_new(_("<b>Downloads:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 2, 3, 3, 4,
                     GTK_FILL, GTK_FILL, 0, 0);

    if (whois->dl >= 0)
      label = gtk_label_new(l_printf("%d", whois->dl));
    else
      label = gtk_label_new(_("unknown"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, 3, 4,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Client:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(user->client);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 2, 4, 5,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Uploads:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 2, 3, 4, 5,
                     GTK_FILL, GTK_FILL, 0, 0);

    if (whois->ul >= 0)
      label = gtk_label_new(l_printf("%d", whois->ul));
    else
      label = gtk_label_new(_("unknown"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, 4, 5,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>IP:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6,
                     GTK_FILL, GTK_FILL, 0, 0);
      
    if (user_ip(user) == 0)
      label = gtk_label_new(_("unknown"));
    else
      label = gtk_label_new(ntoa(user_ip(user)));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 2, 5, 6,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Total DL:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 2, 3, 5, 6,
                     GTK_FILL, GTK_FILL, 0, 0);

    if (whois->tdl >= 0)
      label = gtk_label_new(l_printf("%d", whois->tdl));
    else
      label = gtk_label_new(_("unknown"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, 5, 6,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Server:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 6, 7,
                     GTK_FILL, GTK_FILL, 0, 0);
    
    label = gtk_label_new(whois->server?whois->server:_("unknown"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 2, 6, 7,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Total UL:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 2, 3, 6, 7,
                     GTK_FILL, GTK_FILL, 0, 0);
    
    if (whois->tul >= 0)
      label = gtk_label_new(l_printf("%d", whois->tul));
    else
      label = gtk_label_new(_("unknown"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, 6, 7,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Server port:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 7, 8,
                     GTK_FILL, GTK_FILL, 0, 0);

    if (whois->server_port)
      label = gtk_label_new(l_printf("%d", whois->server_port));
    else
      label = gtk_label_new(_("unknown"));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 2, 7, 8,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(_("<b>Data port:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 2, 3, 7, 8,
                     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new(l_printf("%d", user_port(user)));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 3, 4, 7, 8,
                     GTK_FILL, GTK_FILL, 0, 0);
  } else if (whois->status == NAP_WHOIS_OFFLINE) {
    char* text;
    GtkWidget* table = gtk_table_new(2, 2, FALSE);

    gtk_table_set_col_spacings(GTK_TABLE(table), 16);
    gtk_table_set_row_spacings(GTK_TABLE(table), 5);
    gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 10);
  
    label = gtk_label_new(_("<b>Last seen:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                     GTK_FILL, GTK_FILL, 0, 0);

    if (whois->online > 0) {
      text = l_printf("%s", ctime(&whois->online));
      text[strlen(text)-1] = 0;
    } else {
      text = l_strdup(_("unknown"));
    }

    label = gtk_label_new(text);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 2, 0, 1,
                     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);

    label = gtk_label_new(_("<b>Level:</b>"));
    gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
		     GTK_FILL, GTK_FILL, 0, 0);

    label = gtk_label_new
      (l_printf("%s (%s)", whois->str_level?whois->str_level:"?",
		NapLevel(whois->level)));
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
    gtk_table_attach(GTK_TABLE(table), label, 1, 2, 1, 2,
		     GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
    
  }

  hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
  gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

  button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
  gtk_container_add(GTK_CONTAINER(hbox), button);
  g_signal_connect_swapped(G_OBJECT(button), "clicked",
			   G_CALLBACK(on_whois_close), w);

  gtk_widget_show_all(vbox);

  gtk_widget_show(w->win);
}

void ui_nap_whois_show(nap_user_t* user) {
  nap_whois_t* whois = user->whois;
  if (whois->mode & NAP_WHOIS_WINDOW) {
    ui_nap_whois_t* w = ui_nap_whois_search(user);
    if (!w) w = ui_nap_whois_new(user);
    ui_nap_whois_show_real(w);
  }

}
