/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.18 $
 * $Date: 2007/11/21 20:20:06 $
 * $Author: sgop $
 */

#include "lcore/buddy.h"
#include "lcore/preferences.h"
#include "lcore/load.h"
#include "lcore/browse.h"
#include "lcore/version.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "nap.h"
#include "nap_napi.h"
#include "nap_network.h"
#include "nap_commands.h"
#include "nap_proto.h"
#include "nap_search.h"
#include "nap_hotlist.h"
#include "nap_actions.h"
#include "nap_chat.h"
#include "nap_ui.h"
#include "nap_whois.h"
#include "nap_users.h"
#include "nap_download.h"
#ifdef WIN32
#include "napalias.h"
#endif

#ifndef WIN32
int NAP_ID = -1;
#else
int nap_id = -1;
#endif


char* convert_quotes(char* text) {
  char* res = text;
  while (*text) {
    if (*text == '\"') *text = '\'';
    text++;
  }
  return res;
}

static void nap_speed_observer(pref_node_t* node) {
  int speed = *(int*)(node->value);
  nap_net_send(NULL, NAP_S_SPEED, FALSE, "%d", NapSpeedFromType(speed));
}

static void nap_gpass_observer(pref_node_t* node) {
  const char* pass = (char*)(node->value);
  nap_net_global_pass_changed(pass);
}

static void nap_lpass_observer(pref_node_t* node) {
  const char* pass = (char*)(node->value);
  nap_net_local_pass_changed(pass);
}

static void nap_umode_observer(pref_node_t* node) {
  int mode = *(int*)(node->value);
  nap_usermode_send(NULL, mode);
}

static void nap_init()
{
    pref_register_int("/nap/chat/channels/min", TRUE, 5);
    pref_register_int("/nap/chat/channels/max", TRUE, 0);
    pref_register_int("/nap/usermode", TRUE, NAP_MODE_MASK_ALL);
    pref_register_string("/nap/charset", TRUE, NULL);
    
    nap_actions_init();
}

static void nap_run() {
  pref_observe("/core/user/pass", (ObjectFunc)nap_gpass_observer, NULL);
  pref_observe("/core/line/type", (ObjectFunc)nap_speed_observer, NULL);
  pref_observe("/nap/password", (ObjectFunc)nap_lpass_observer, NULL);
  pref_observe("/nap/usermode", (ObjectFunc)nap_umode_observer, NULL);
}

static void nap_exit() {
  nap_user_stat_print();
}

static void nap_plugin_start(plugin_t* plugin) {
  plugin_init(plugin, "Napster Module",
              (ModuleFunc)nap_init, (ModuleFunc)nap_run, (ModuleFunc)nap_exit);

#ifndef WIN32
  NAP_ID = plugin->ID;
#else
  nap_id = plugin->ID;
#endif
  plugin->connect = nap_connect_change;

  module_require(base_module_instance);
  module_require(pref_module_instance);
  module_require(nap_command_module_instance);
  module_require(nap_hot_module_instance);
  module_require(nap_ui_module_instance);
  module_require(nap_search_module_instance);
  module_require(nap_net_module_instance);
  module_require(nap_user_module_instance);
  module_require(nap_download_module_instance);
  module_require(buddy_module_instance);
  //  module_require(MODULE(plugin), upload_module_instance);
  //  module_require(MODULE(plugin), download_module_instance);
  module_require(browse_module_instance);
  module_require(library_module_instance);
}

void plugin_initialize(plugin_info_t* info)
{
    info->name       = g_strdup("OpenNap");
    info->sname      = g_strdup("Nap");
    info->copyright  = g_strdup("Copyright \302\251 2003-2007 Markus Lausser");
    info->if_version = IF_VERSION;
    info->pl_version = 0x00080500; // 0.8.5-0
    info->icon       = g_strdup("nap_icon.png");
    info->license    = g_strdup("GPL (GNU General Public License)");
    info->website    = g_strdup("http://lopster.sourceforge.net");
    info->comment    = g_strdup(N_("Plugin which connects to napster-like servers such as OpenNap and SlavaNap."));
    info->authors    = g_list_append
        (info->authors, g_strdup("Markus Lausser <sgop@users.sourceforge.net>"));
    info->start      = nap_plugin_start;
#ifdef PACKAGE_SOURCE_DIR
    lopster_pixmap_dir_add(SDIR(PACKAGE_SOURCE_DIR) SDIR("plugin") 
                           SDIR("napster") "pixmaps");
#endif
}

#ifdef WIN32
int nap_plugin_get_id( )
{
    return nap_id;
}
#endif

#ifdef WIN32
#define _NAP_C_
#include "napaliasdef.c"
#endif

