/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.7 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#include "lcore/action_item.h"
#include "lcore/string_list.h"
#include "lcore/l_i18n.h"
#include "nap.h"
#include "nap_actions.h"
#include "nap_chat.h"
#include "nap_whois.h"
#include "nap_browse.h"
#include "nap_hotlist.h"
#include "nap_upload.h"
#include "nap_download.h"

static gboolean on_nap_check_not_buddy(nap_user_t* nu)
{
    return (!nap_hot_in_hotlist(nu));
}
static gboolean on_nap_check_mod(nap_user_t* nu)
{
    return (nu->net && nu->net->level >= NAP_L_MOD);
}

static void nap_chat(nap_user_t* nu)
{
    if (nu->net) nap_cp_new_private(nu->net, OBJ(nu)->name);
}
static void nap_whois(nap_user_t* nu)
{
    if (nu->net) nap_whois_user(nu, nu->net, NAP_WHOIS_WINDOW, FALSE);
}
static void nap_buddy(nap_user_t* nu)
{
    if (!nap_hot_in_hotlist(nu)) nap_hot_new(nu, NULL);
}
static void nap_browse(nap_user_t* nu)
{
    if (nu->net) nap_browse_user(nu);
}
static void nap_muzzle(nap_user_t* nu)
{
    if (nu->net) nap_user_muzzle(nu, nu->net, NULL);
}
static void nap_kill(nap_user_t* nu)
{
    if (nu->net) nap_user_kill(nu, nu->net, NULL);
}
static void nap_ban(nap_user_t* nu)
{
    if (nu->net) nap_user_ban(nu, nu->net, NULL, 0);
}
static void nap_bankill(nap_user_t* nu)
{
    if (nu->net) 
    {
        nap_user_ban(nu, nu->net, NULL, 0);
        nap_user_kill(nu, nu->net, NULL);
    }
}
static void nap_nuke(nap_user_t* nu)
{
    if (nu->net) nap_user_nuke(nu, nu->net);
}

ACTION_PROCESS_LIST(nap_chat, nap_user_t);
ACTION_PROCESS_LIST(nap_whois, nap_user_t);
ACTION_PROCESS_LIST(nap_buddy, nap_user_t);
ACTION_PROCESS_LIST(nap_browse, nap_user_t);
ACTION_PROCESS_LIST(nap_muzzle, nap_user_t);
ACTION_PROCESS_LIST(nap_kill, nap_user_t);
ACTION_PROCESS_LIST(nap_ban, nap_user_t);
ACTION_PROCESS_LIST(nap_bankill, nap_user_t);
ACTION_PROCESS_LIST(nap_nuke, nap_user_t);


static action_item_t* nap_actions_user_menu(const char* name)
{
    action_item_t* item;
    
    item = action_item_root(name);
    action_item_simple(N_("Open chat..."), item, 0,
                       (ActionItemFunc)nap_chat);
    action_item_simple(N_("Send message..."), item, AI_RESTRICT_1,
                       NULL);
    action_item_sep(item);
    action_item_simple(N_("Whois user"), item, 0,
                       (ActionItemFunc)on_nap_whois);
    action_item_new(N_("Add to buddies"), item, 0,
                    (ActionItemFunc)on_nap_buddy,
                    (ActionItemCheck)on_nap_check_not_buddy);
    action_item_simple(N_("Browse user"), item, 0,
                       (ActionItemFunc)on_nap_browse);
  /*
  action_item_sep(item);
  action_item_new("Muzzle user on network", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_muzzle,
                  (ActionItemCheck)on_nap_check_mod);
  action_item_new("Kick user from network", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_kill,
                  (ActionItemCheck)on_nap_check_mod);
  action_item_new("Ban user from network", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_ban,
                  (ActionItemCheck)on_nap_check_mod);
  action_item_new("Ban/Kick user from network", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_bankill,
                  (ActionItemCheck)on_nap_check_mod);
  action_item_new("Nuke user account", item, AI_HIDE_0,
                  (ActionItemFunc)on_nap_nuke,
                  (ActionItemCheck)on_nap_check_mod);
  */
  return item;
}




static gboolean on_check_not_joined(nap_ce_t* ce)
{
    return (!nap_cp_search_net(ce->net, ce->name, NAP_PAGE_PUBLIC) &&
            NAP_NET_ONLINE(ce->net));
}
static gboolean on_check_joined(nap_ce_t* ce)
{
    return (nap_cp_search_net(ce->net, ce->name, NAP_PAGE_PUBLIC) != NULL);
}
static gboolean on_check_not_auto(nap_ce_t* ce)
{
    return (!string_slist_contains(ce->net->autojoin, ce->name));
}
static gboolean on_check_auto(nap_ce_t* ce)
{
    return (string_slist_contains(ce->net->autojoin, ce->name) != NULL);
}

static void join_channel(nap_ce_t* ce)
{
    if (!nap_cp_search_net(ce->net, ce->name, NAP_PAGE_PUBLIC))
        nap_join_channel(ce->net, ce->name);
}
static void part_channel(nap_ce_t* ce)
{
    nap_cp_t* page = nap_cp_search_net(ce->net, ce->name, NAP_PAGE_PUBLIC);
    if (!page) return;
    chat_page_close(PAGE(page));
}
static void add_channel(nap_ce_t* ce)
{
    if (!string_slist_contains(ce->net->autojoin, ce->name)) {
        ce->net->autojoin =
            g_slist_append(ce->net->autojoin, g_strdup(ce->name));
    }
}
static void remove_channel(nap_ce_t* ce)
{
    GSList* dl = string_slist_contains(ce->net->autojoin, ce->name);
    if (dl)
    {
        g_free(dl->data);
        ce->net->autojoin = g_slist_remove(ce->net->autojoin, dl->data);
    }
}

ACTION_PROCESS_LIST(join_channel, nap_ce_t);
ACTION_PROCESS_LIST(part_channel, nap_ce_t);
ACTION_PROCESS_LIST(add_channel, nap_ce_t);
ACTION_PROCESS_LIST(remove_channel, nap_ce_t);


/* static void on_refresh_channels(GList* dummy, nap_cl_t* ncl) { */
/*   (void)dummy; */
/*   if (ncl) nap_get_channel_list(ncl); */
/* } */

static void nap_actions_channel_list()
{
    action_item_t* item;
    //  action_item_t* sub;
    
    item = action_item_root("Nap channel list actions");
    action_item_new(N_("Join channel"), item, 0,
                    (ActionItemFunc)on_join_channel,
                    (ActionItemCheck)on_check_not_joined);
    action_item_new(N_("Part channel"), item, 0,
                    (ActionItemFunc)on_part_channel,
                    (ActionItemCheck)on_check_joined);
    action_item_sep(item);
    action_item_new(N_("Add to autojoin"), item, AI_HIDE_0,
                    (ActionItemFunc)on_add_channel,
                    (ActionItemCheck)on_check_not_auto);
    action_item_new(N_("Remove from autojoin"), item, AI_HIDE_0,
                    (ActionItemFunc)on_remove_channel,
                    (ActionItemCheck)on_check_auto);
  //FIXME
/*   action_item_sep(item); */
/*   action_item_simple("Refresh list", item, AI_GLOBAL, */
/*                      (ActionItemFunc)on_refresh_channels); */
}


gboolean nap_check_start_group(nap_ng_t* group)
{
    return !(group->nets_auto);
}
gboolean nap_check_start_net(nap_net_t* net)
{
    return !(net->flags & NAP_NET_AUTO);
}
gboolean nap_check_connect_server(nap_server_t* server)
{
    return (server && server->status != NAP_SERVER_CONNECTING &&
            server->status != NAP_SERVER_ONLINE);
}
gboolean nap_check_disconnect_server(nap_server_t* server)
{
    return (!server ||
            server->status == NAP_SERVER_CONNECTING ||
            server->status == NAP_SERVER_ONLINE);
}
gboolean nap_check_include_group(nap_ng_t* group)
{
    return !(group->flags & NAP_NG_AUTO_CONNECT);
}
gboolean nap_net_check_private(nap_net_t* net)
{
    return !(net->flags & NAP_NET_PRIVATE);
}
gboolean nap_net_check_emu_winmx(nap_net_t* net)
{
    return !(net->flags & NAP_NET_EMU_WINMX);
}
gboolean nap_net_check_own_server(nap_net_t* net)
{
    return !(net->flags & NAP_NET_OWN_SERVER);
}
gboolean nap_net_check_ign_global(nap_net_t* net)
{
    return !(net->flags & NAP_NET_IGN_GLOBAL);
}

static void nap_net_private(nap_net_t* net)
{
    net->flags ^= NAP_NET_PRIVATE;
    object_set_dirty(OBJ(net));
    nap_network_save();
}
static void nap_net_emu_winmx(nap_net_t* net)
{
    net->flags ^= NAP_NET_EMU_WINMX;
    object_set_dirty(OBJ(net));
    nap_network_save();
}
static void nap_net_own_server(nap_net_t* net)
{
    net->flags ^= NAP_NET_OWN_SERVER;
    object_set_dirty(OBJ(net));
    nap_network_save();
}
static void nap_net_ign_global(nap_net_t* net)
{
    net->flags ^= NAP_NET_IGN_GLOBAL;
    object_set_dirty(OBJ(net));
    nap_network_save();
}
static void nap_group_activate(nap_ng_t* group)
{
    if (group->nets_auto)
        nap_ng_stop_connect(group);
    else
        nap_ng_start_connect(group);
}
static void nap_net_activate(nap_net_t* net)
{
    if (net->flags & NAP_NET_AUTO)
        nap_net_disconnect(net);
    else
        nap_net_start_connect(net);
}
static void nap_ng_include(nap_ng_t* ng)
{
    if (ng->flags & NAP_NG_AUTO_CONNECT)
        nap_ng_set_auto(ng, FALSE);
    else
        nap_ng_set_auto(ng, TRUE);
}

ACTION_PROCESS_LIST(object_kill, object_t);
ACTION_PROCESS_LIST(nap_ng_include, nap_ng_t);
ACTION_PROCESS_LIST(nap_group_activate, nap_ng_t);
ACTION_PROCESS_LIST(nap_net_activate, nap_net_t);
ACTION_PROCESS_LIST(nap_server_connect, nap_server_t);
ACTION_PROCESS_LIST(nap_server_disconnect, nap_server_t);
ACTION_PROCESS_LIST(nap_net_private, nap_net_t);
ACTION_PROCESS_LIST(nap_net_emu_winmx, nap_net_t);
ACTION_PROCESS_LIST(nap_net_own_server, nap_net_t);
ACTION_PROCESS_LIST(nap_net_ign_global, nap_net_t);


static void nap_actions_nets()
{
    action_item_t* item;
    
    item = action_item_root("Napster group actions");
    action_item_new(N_("Auto-connect"), item, AI_CHECK,
                    (ActionItemFunc)on_nap_group_activate,
                    (ActionItemCheck)nap_check_start_group);
    
    item = action_item_root("Napster group conf actions");
    action_item_new(N_("Include in Auto-Connect"), item, AI_CHECK,
                    (ActionItemFunc)on_nap_ng_include,
                    (ActionItemCheck)nap_check_include_group);
    action_item_simple(N_("Stay connected to"), item, 0,
                       (ActionItemFunc)NULL);

    item = action_item_root("Napster net actions");
    action_item_new(N_("Auto-connect"), item, AI_CHECK,
                    (ActionItemFunc)on_nap_net_activate,
                    (ActionItemCheck)nap_check_start_net);
/*       item = gtk_menu_item_new_with_label("Get Global User List"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
/*       if (net->level >= NAP_L_MOD) { */
/*         g_signal_connect(G_OBJECT(item), "activate", */
/*                          G_CALLBACK(on_global_list), net); */
/*       } else { */
/*         gtk_widget_set_sensitive(item, FALSE); */
/*       } */
      
/*       if (net->level < NAP_L_MOD) */
/*         gtk_widget_set_sensitive(item, FALSE); */
      
/*       item = gtk_menu_item_new_with_label("Get Banned Users (NI)"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
      /*
        gtk_signal_connect(GTK_OBJECT(item), "activate",
        GTK_SIGNAL_FUNC(on_banned_users), net);
      */      
/*       item = gtk_menu_item_new_with_label("Get Client Statistic (NI)"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
      /*
        gtk_signal_connect(GTK_OBJECT(item), "activate",
        GTK_SIGNAL_FUNC(on_client_list), net);
      */
/*       if (net->level < NAP_L_MOD) */
/*         gtk_widget_set_sensitive(item, FALSE); */
      
/*       item = gtk_menu_item_new_with_label("Refresh linked servers (NI)"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
      /*
        gtk_signal_connect(GTK_OBJECT(item), "activate",
        GTK_SIGNAL_FUNC(on_server_links), net);
      */      
/*       item = gtk_menu_item_new_with_label("Show server stats (NI)"); */
/*       gtk_widget_show(item); */
/*       gtk_container_add(GTK_CONTAINER(popup2), item); */
      /*
        gtk_signal_connect(GTK_OBJECT(item), "activate",
        GTK_SIGNAL_FUNC(on_server_stats), net);
      */


    item = action_item_root("Napster net conf actions");
    action_item_new(N_("Has persistent servers"), item, AI_CHECK,
                    (ActionItemFunc)on_nap_net_private,
                    (ActionItemCheck)nap_net_check_private);
    action_item_new(N_("Don't emulate WinMX v2.6"), item, AI_CHECK,
                    (ActionItemFunc)on_nap_net_emu_winmx,
                    (ActionItemCheck)nap_net_check_emu_winmx);
    action_item_new(N_("Own server page"), item, AI_CHECK,
                    (ActionItemFunc)on_nap_net_own_server,
                    (ActionItemCheck)nap_net_check_own_server);
    action_item_new(N_("Ignore global messages"), item, AI_CHECK,
                    (ActionItemFunc)on_nap_net_ign_global,
                    (ActionItemCheck)nap_net_check_ign_global);
    //FIXME: delete/move network

    item = action_item_root("Napster server actions");
/*     action_item_new(N_("Connect"), item, AI_HIDE_0|AI_RESTRICT_1, */
/*                     (ActionItemFunc)on_nap_server_connect, */
/*                     (ActionItemCheck)nap_check_connect_server); */
/*     action_item_new(N_("Disconnect"), item, AI_HIDE_0|AI_RESTRICT_1, */
/*                     (ActionItemFunc)on_nap_server_disconnect, */
/*                     (ActionItemCheck)nap_check_disconnect_server); */

    item = action_item_root("Napster server conf actions");
}




static nap_user_t* nap_cu_to_nu(nap_cu_t* cu, nap_cp_t* page)
{
    nap_user_t* user =
        nap_user_find(OBJ(cu)->name, page?page->net:NULL);
    return user;
}

static nap_user_t* nap_upload_to_nu(nap_upload_t* upload)
{
    return NAP_USER(UPLOAD(upload)->user);
}

static nap_user_t* nap_source_to_nu(nap_source_t* source)
{
    return NAP_USER(SOURCE(source)->user);
}

static nap_user_t* nap_result_to_nu(search_result_t* result)
{
    return NAP_USER(result->user);
}

static nap_user_t* nap_browse_to_nu(file_t* file, browse_t* browse)
{
    (void)file;
    return NAP_USER(browse->user);
}

void nap_actions_init()
{
    action_item_t* item;
    
    nap_actions_nets();
    nap_actions_channel_list();

    item = nap_actions_user_menu("Online nap users actions");
    item->convert = (ActionObjectConvert)nap_cu_to_nu;
    
    item = nap_actions_user_menu("Upload nap users actions");
    item->convert = (ActionObjectConvert)nap_upload_to_nu;
    user_action_upload_register(NAP_ID, item);
    
    item = nap_actions_user_menu("Source nap users actions");
    item->convert = (ActionObjectConvert)nap_source_to_nu;
    user_action_source_register(NAP_ID, item);
    
    item = nap_actions_user_menu("Result nap users actions");
    item->convert = (ActionObjectConvert)nap_result_to_nu;
    user_action_result_register(NAP_ID, item);
    
    item = nap_actions_user_menu("Browse nap users actions");
    item->convert = (ActionObjectConvert)nap_browse_to_nu;
    user_action_browse_register(NAP_ID, item);
}
