/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifndef _NAP_CHAT_H_
#define _NAP_CHAT_H_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "lcore/chat.h"
#include "nap_network.h"

#define NAP_CMODE_TOPIC_REST 0x01
#define NAP_CMODE_REGISTERED 0x02
#define NAP_CMODE_PRIVATE    0x04
#define NAP_CMODE_INVITE     0x08
#define NAP_CMODE_MODERATED  0x10
#define NAP_CMODE_MASK       0x1f
#define NAP_CMODE_NUMBER        5

#define NAP_PAGE_OTHER          0
#define NAP_PAGE_PUBLIC         1
#define NAP_PAGE_PRIVATE        2
#define NAP_PAGE_WALLOP         3

#define NAP_B_PUBLIC            0
#define NAP_B_OP                1

#define NAP_CU_OP      0x01
#define NAP_CU_VOICE   0x02
#define NAP_CU_MUZZLE  0x04
#define NAP_CU_BAN     0x08

typedef struct {
  object_t _object;
  int speed;
  int shared;
  int flags;
} nap_cu_t;

#define NCP(arg) ((nap_cp_t*)(arg))
typedef struct {
  chat_page_t _page;
  int type;
  nap_net_t* net;
} nap_cp_t;

#define NPP(arg) ((nap_pub_t*)(arg))
typedef struct {
  nap_cp_t _ncp;
  GList* users;
  char* topic;
  int online;
  int limit;
  int level;
  int chmode;
  gboolean opped;
  gboolean leaving;
} nap_pub_t;

G_BEGIN_DECLS

int NapChannelModeInt(const char* mode);

nap_cp_t* nap_cp_search_net(nap_net_t* net, const char* name, int type);
nap_cp_t* nap_cp_search(const char* name, int type);
nap_cp_t* nap_cp_new_private(nap_net_t* net, const char* nick);
void nap_join_channel(nap_net_t* net, const char* channel);
void nap_chat_left_net(nap_net_t* net);
void nap_chat_joined_net(nap_net_t* net);
void nap_channel_joined(nap_pub_t* npp);
void nap_channel_part(nap_pub_t* page);
nap_cu_t* nap_cu_add(nap_pub_t* npp, const char* nick,
                     int shared, int speed);
void nap_cu_joined(nap_pub_t* npp, const char* nick,
                   int shared, int speed);
void nap_cu_parted(nap_pub_t* npp, const char* nick,
                   int shared, int speed);
void nap_channel_new_topic(nap_pub_t* npp, const char* topic, const char* who);

void nap_server_message (nap_net_t* net, char* message);
void nap_motd_message   (nap_net_t* net, char* message);
void nap_emote_message  (nap_net_t* net, char* channel, char* user, 
                         char* message);
void nap_public_message (nap_net_t* net, char* channel,
                         char* user, char* message);
void nap_chop_message   (nap_net_t* net, char* channel,
                         char* user, char* message);
void nap_private_message(nap_net_t* net, char* user, char* message);
void nap_global_message (nap_net_t* net, char* user, char* message);
void nap_wallop_message (nap_net_t* net, char* user,
                         char* message);


void nap_send_public_message (nap_cp_t* ncp, char* message);
void nap_send_chop_message   (nap_cp_t* ncp, char* message);
void nap_send_emote_message  (nap_cp_t* ncp, char* message);
void nap_send_wallop_message (nap_net_t* net, char* message);
void nap_send_global_message (nap_net_t* net, char* message);
void nap_send_private_message(nap_net_t* net, chat_page_t* page,
                              const char* user, char* message);

void nap_channel_update_op(nap_net_t* net);

void nap_channel_set_mode (nap_pub_t* page, int set, int clear);
void nap_channel_set_limit(nap_pub_t* page, int limit);
void nap_channel_set_level(nap_pub_t* page, int level);

void nap_channel_mode_changed (nap_pub_t* page, const char* nick,
                               int set, int clear);
void nap_channel_limit_changed(nap_pub_t* page, const char* nick,
                               int limit);
void nap_channel_level_changed(nap_pub_t* page, const char* nick,
                               int level);
void nap_channel_user_flagged(nap_pub_t* page, const char* who,
                              const char* user, gboolean set,
                              int flag, const char* reason);

G_END_DECLS

#endif
