/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.6 $
 * $Date: 2006/07/23 00:17:10 $
 * $Author: sgop $
 */

#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <glib.h>

#include "lcore/utils.h"
#include "lcore/chat.h"
#include "lcore/commands.h"
#include "lcore/preferences.h"
#include "lcore/l_i18n.h"
#include "nap_chat.h"
#include "nap_proto.h"
#include "nap.h"
#include "nap_network.h"
#include "nap_ping.h"
#include "nap_whois.h"
#include "nap_users.h"

typedef void (*NapCommandFunc)(chat_page_t*, nap_net_t*, char*);
#define NAP_COMM(f) static void f (chat_page_t* page, nap_net_t* net, char *data)

#define NAP_COMMAND(c) ((nap_command_t*)(c))
typedef struct {
  command_t _command;
  NapCommandFunc handle;
} nap_command_t;

typedef struct {
  module_t _module;
} nap_command_module_t;

static nap_command_module_t* NapCommandModule = NULL;

#define CHECK_NET COMMAND_ERROR(!net, page, _("No network specified"))
#define CHECK_PUBLIC COMMAND_ERROR(page->ID != NAP_ID || NCP(page)->type != NAP_PAGE_PUBLIC || NCP(page)->net==NULL, page, "This is not a valid napster public page")



static void nap_command_handle(chat_page_t* page, command_t* comm, char *data) {
  nap_cp_t* ncp;
  nap_command_t* command = NAP_COMMAND(comm);
  
  if (page && page->ID == NAP_ID) ncp = NCP(page);
  else ncp = NULL;

  g_log("NAP_COM", G_LOG_LEVEL_DEBUG,
        "handle: [%s] [%s]", command?COMMAND(command)->name:"(null)",
        data?data:"(null)");
  if (command) {
    nap_net_t* net = NULL;
    if (data && *data == ':' && data[1] != 0 && data[1] != ' ') {
      data = arg(data+1, 0);
      net = nap_get_net(atoi(data));
      if (!net) {
        command_error(page, _("No network with such index"));
        return;
      }
      data = arg(NULL, 1);
    } else if (ncp) {
      net = ncp->net;
    }
    command->handle(page, net, data);
  } else if (ncp && ncp->net) {
    // normal line, pass to chat page
    if (ncp->type == NAP_PAGE_PRIVATE) {
      nap_send_private_message(ncp->net, page, OBJ(page)->name, data);
    } else if (ncp->type == NAP_PAGE_PUBLIC) {
      nap_send_public_message(ncp, data);
    } else if (ncp->type == NAP_PAGE_WALLOP) {
      nap_send_wallop_message(ncp->net, data);
/*     } else if (!strcmp("__Global__", OBJ(page)->name)) { */
/*       nap_send_global_message(ncp->net, data); */
    }
  }
}
  
static void
nap_command_new(const char* name, const char* syntax,
                const char* des, NapCommandFunc func) {
  nap_command_t *comm;
  
  g_assert(name);

  comm = g_malloc0(sizeof(*comm));
  command_init(COMMAND(comm), NAP_ID, name, syntax, des);
  comm->handle = func;
}

NAP_COMM(nap_com_msg) {
  char* nick;
  char* message;
  
  CHECK_NET;

  nick = arg(data, 0);
  message = arg(NULL, 1);

  COMMAND_ERROR(!nick, page, _("Missing nickname!"));
  COMMAND_ERROR(!message, page, _("Missing message!"));
  
  nap_send_private_message(net, page, nick, message);
}

NAP_COMM(nap_com_join) {
  char *channel;

  CHECK_NET;

  channel = arg(data, 0);
  COMMAND_ERROR(!net, page, _("No network specified"));
  nap_join_channel(net, channel);
}

NAP_COMM(nap_com_emote) {
  char *message;

  CHECK_NET;

  message = arg(data, 1);

  COMMAND_ERROR(!message, page, _("Missing message!"));
  COMMAND_ERROR((!page || page->ID != NAP_ID ||
                 NCP(page)->type != NAP_PAGE_PUBLIC), page,
                _("This is not a napster channel page"));

  nap_send_emote_message(NCP(page), message);
}

NAP_COMM(nap_com_wallop) {
  char *message;
  
  CHECK_NET; // remove?

  message = arg(data, 1);
  COMMAND_ERROR(!message, page, _("Missing message!"));

  nap_send_wallop_message(net, message);
}

NAP_COMM(nap_com_global) {
  char *message;

  CHECK_NET; // remove?

  message = arg(data, 1);
  COMMAND_ERROR(!message, page, _("Missing message!"));
  
  nap_send_global_message(net, message);
}

NAP_COMM(nap_com_pass) {
  char* nick;
  char* passwd;

  CHECK_NET;

  nick = arg(data, 0);
  
  COMMAND_ERROR(!nick, page, _("Missing argument"));
  passwd = arg(NULL, 0);
  if (!passwd) {
    nap_net_set_password(net, nick);
    // FIXME: notify user
  } else {
    CHECK_NET;
    nap_net_send(net, NAP_S_ALTER_PASSWORD, TRUE, "%s %s", nick, passwd);
  }
}

NAP_COMM(nap_com_npass) {
  (void)page;
  (void)net;
  pref_store_str("/nap/password", arg(data, 0));
}

NAP_COMM(nap_com_topic) {
  char *topic;

  (void)net;
  CHECK_PUBLIC;

  topic = arg(data, 1);
  COMMAND_ERROR(!topic, page, _("Missing argument"));
  nap_net_send(NCP(page)->net, NAP_S_SET_TOPIC, TRUE,
               "%s %s", OBJ(page)->name, topic);
}

NAP_COMM(nap_com_ping) {
  char *nick;

  CHECK_NET;
  
  (void)page;

  nick = arg(data, 0);
  COMMAND_ERROR(!nick, page, _("Missing argument"));

  nap_ping_user(nick, net);
}

NAP_COMM(nap_com_whois) {
  char* nick;

  CHECK_NET;
  
  (void)page;

  nick = arg(data, 0);
  COMMAND_ERROR(!nick, page, _("Missing argument"));
  nap_whois_user(nap_user_find(nick, NULL),
                 net, NAP_WHOIS_WINDOW, 0);
}

NAP_COMM(nap_com_guser) {
  char *message;

  CHECK_NET;

  (void)page;

  message = arg(data, 1);
  nap_net_get_global_users(net, message);
}

NAP_COMM(nap_com_chwallop) {
  char* message;

  CHECK_NET;
  CHECK_PUBLIC;

  message = arg(data, 1);
  COMMAND_ERROR(!message, page, _("Missing argument"));

  nap_send_chop_message(NCP(page), message);
}

static void
nap_pack_raw(b_socket_t* sock, guint16 type, const char* arg) {
  buffer_t* buffer;
  guint16 added;
  char* pos;

  added = arg?strlen(arg):0;
  buffer = b_socket_get_out_buffer(sock, added+1+4);
  pos = buffer_write_pos(buffer);
  if (arg) memcpy(pos+4, arg, added);
  pos[added+4] = 0;

  if (type != NAP_S_SHARE_FILE)
    g_log("NAP_PROTO", G_LOG_LEVEL_DEBUG,
          "S: %5u L: %4d RAW[%s]", type, added, pos+4);
  
  pack_u16(pos, added);
  pack_u16(pos+2, type);
  buffer->size += (added+4);
}

NAP_COMM(nap_com_raw) {
  char* id;
  char* args;

  CHECK_NET;

  id = arg(data, 0);
  args = arg(NULL, 1);

  COMMAND_ERROR(!id, page, _("Missing argument"));

  nap_pack_raw(BSOCK(net), atoi(id), args);
  b_socket_start_sending(BSOCK(net));
}

NAP_COMM(nap_com_sstats) {
  CHECK_NET;
  nap_net_send(net, NAP_S_GET_USAGE_STATS, FALSE, "");
}

NAP_COMM(nap_com_kill) {
  char* nick = arg(data, 0);
  char* reason = arg(NULL, 1);
  nap_user_t* user;

  CHECK_NET;
  COMMAND_ERROR(!nick, page, _("Missing nickname"));

  user = nap_user_find(nick, NULL);
  nap_user_kill(user, net, reason);
}

NAP_COMM(nap_com_muzzle) {
  char* nick = arg(data, 0);
  char* reason = arg(NULL, 1);
  nap_user_t* user;

  CHECK_NET;
  COMMAND_ERROR(!nick, page, _("Missing nickname"));

  user = nap_user_find(nick, NULL);
  nap_user_muzzle(user, net, reason);
}

NAP_COMM(nap_com_unmuzzle) {
  char* nick = arg(data, 0);
  char* reason = arg(NULL, 1);
  nap_user_t* user;

  CHECK_NET;
  COMMAND_ERROR(!nick, page, _("Missing nickname"));

  user = nap_user_find(nick, NULL);
  nap_user_unmuzzle(user, net, reason);
}

NAP_COMM(nap_com_ban) {
  char* nick = arg(data, 0);
  char* reason = arg(NULL, 1);
  nap_user_t* user;

  CHECK_NET;
  COMMAND_ERROR(!nick, page, _("Missing nickname"));

  user = nap_user_find(nick, NULL);
  nap_user_ban(user, net, reason, 0);
}

NAP_COMM(nap_com_unban) {
  char* nick = arg(data, 0);
  char* reason = arg(NULL, 1);
  nap_user_t* user;

  CHECK_NET;
  COMMAND_ERROR(!nick, page, _("Missing nickname"));

  user = nap_user_find(nick, NULL);
  nap_user_unban(user, net, reason);
}


NAP_COMM(nap_com_op) {
  char* nicks = arg(data, 1);
  
  CHECK_NET;
  CHECK_PUBLIC;

  if (nicks)
    nap_net_send(NCP(page)->net, NAP_S_CH_OP, TRUE,
                 "%s %s", OBJ(page)->name, nicks);
  else
    nap_net_send(NCP(page)->net, NAP_S_CH_OP, TRUE,
                 "%s", OBJ(page)->name);
}

NAP_COMM(nap_com_deop) {
  char* nicks = arg(data, 1);
  
  CHECK_NET;
  CHECK_PUBLIC;

  COMMAND_ERROR(!nicks, page, _("Missing nickname(s)"));

  nap_net_send(NCP(page)->net, NAP_S_CH_DEOP, TRUE,
               "%s %s", OBJ(page)->name, nicks);
}

NAP_COMM(nap_com_voice) {
  char* nicks = arg(data, 1);
  
  CHECK_NET;
  CHECK_PUBLIC;

  if (nicks)
    nap_net_send(NCP(page)->net, NAP_S_CH_VOICE, TRUE,
                 "%s %s", OBJ(page)->name, nicks);
  else
    nap_net_send(NCP(page)->net, NAP_S_CH_VOICE, TRUE,
                 "%s", OBJ(page)->name);
}

NAP_COMM(nap_com_devoice) {
  char* nicks = arg(data, 1);
  
  CHECK_NET;
  CHECK_PUBLIC;

  COMMAND_ERROR(!nicks, page, _("Missing nickname(s)"));

  nap_net_send(NCP(page)->net, NAP_S_CH_DEVOICE, TRUE,
               "%s %s", OBJ(page)->name, nicks);
}

NAP_COMM(nap_com_invite) {
  char* nick = arg(data, 0);
  
  CHECK_NET;
  CHECK_PUBLIC;

  COMMAND_ERROR(!nick, page, _("Missing nickname"));

  nap_net_send(NCP(page)->net, NAP_S_CH_INVITE, TRUE,
               "%s %s", OBJ(page)->name, nick);
}

NAP_COMM(nap_com_chmuzzle) {
  char* nick = arg(data, 0);
  char* reason = arg(NULL, 1);

  CHECK_NET;
  CHECK_PUBLIC;

  if (reason)
    nap_net_send(NCP(page)->net, NAP_S_CH_MUZZLE, TRUE,
                 "%s %s \"%s\"", OBJ(page)->name, nick, convert_quotes(reason));
  else if (nick)
    nap_net_send(NCP(page)->net, NAP_S_CH_MUZZLE, TRUE,
                 "%s %s", OBJ(page)->name, nick);
  else
    nap_net_send(NCP(page)->net, NAP_S_CH_MUZZLE, TRUE,
                 "%s", OBJ(page)->name);
}

NAP_COMM(nap_com_chunmuzzle) {
  char* nick = arg(data, 0);
  char* reason = arg(NULL, 1);

  CHECK_NET;
  CHECK_PUBLIC;
  COMMAND_ERROR(!nick, page, _("Missing nickname"));

  if (reason)
    nap_net_send(NCP(page)->net, NAP_S_CH_UNMUZZLE, TRUE,
                 "%s %s \"%s\"", OBJ(page)->name, nick, convert_quotes(reason));
  else
    nap_net_send(NCP(page)->net, NAP_S_CH_UNMUZZLE, TRUE,
                 "%s %s", OBJ(page)->name, nick);
}

NAP_COMM(nap_com_chban) {
  char* nick = arg(data, 0);
  char* reason = arg(NULL, 1);

  CHECK_NET;
  CHECK_PUBLIC;
  //  COMMAND_ERROR(!nick, page, "Missing nickname");

  if (reason)
    nap_net_send(NCP(page)->net, NAP_S_CH_BAN, TRUE,
                 "%s %s \"%s\"", OBJ(page)->name, nick, convert_quotes(reason));
  else if (nick)
    nap_net_send(NCP(page)->net, NAP_S_CH_BAN, TRUE,
                 "%s %s", OBJ(page)->name, nick);
  else
    nap_net_send(NCP(page)->net, NAP_S_CH_BAN, TRUE,
                 "%s", OBJ(page)->name);
}

NAP_COMM(nap_com_chunban) {
  char* nick = arg(data, 0);
  char* reason = arg(NULL, 1);

  CHECK_NET;
  CHECK_PUBLIC;
  COMMAND_ERROR(!nick, page, _("Missing nickname"));

  if (reason)
    nap_net_send(NCP(page)->net, NAP_S_CH_UNBAN, TRUE,
                 "%s %s \"%s\"", OBJ(page)->name, nick, convert_quotes(reason));
  else
    nap_net_send(NCP(page)->net, NAP_S_CH_UNBAN, TRUE,
                 "%s %s", OBJ(page)->name, nick);
}


static void nap_commands_init() {
  command_handler_install(NAP_ID, (CommandHandler)nap_command_handle);
}

static void nap_commands_run() {
  nap_command_new("msg", "<user> <text>",
                  _("Send a private message <text> to the User <user>"),
                  nap_com_msg);
  nap_command_new("join", "<channel>",
                  _("Join channel <channel>"),
                  nap_com_join);
  nap_command_new("me", "<text>",
                  _("Send action description <text> on the current channel."),
                  nap_com_emote);
  nap_command_new("wallop", "<text>",
                  _("Send a message to all users with level moderator or higher"),
                  nap_com_wallop);
  nap_command_new("global", "<text>",
                  _("Send global message to all users"),
                  nap_com_global);
  nap_command_new("passwd", "[<user>] <pass>",
                  _("Change the password of the given user or your own if no user is given"),
                  nap_com_pass);
  nap_command_new("nappasswd", "<pass>",
                  _("Change the global napster password"),
                  nap_com_npass);
  nap_command_new("topic", "<topic>",
                  _("Set the topic of the current channel"),
                  nap_com_topic);
  nap_command_new("ping", "<user>",
                  _("Ping a user"),
                  nap_com_ping);
  nap_command_new("whois", "<user>",
                  _("Get information about user"),
                  nap_com_whois);
  nap_command_new("guser", "[<server> [<flags>]]",
                  _("List users. <server> is a server name in the network and may be '*' for any server. <flags> may be a list of the characters: e=ELITE, a=ADMIN, m=MODERATOR, l=LEECH, z=MUZZLED, c=CLOAKED"),
                  nap_com_guser);
  nap_command_new("chwallop", "<message>",
                  _("Send message to all channel operator"),
                  nap_com_chwallop);
  nap_command_new("raw", "<id> <args>",
                  _("send a raw command to the server"),
                  nap_com_raw);
  nap_command_new("sstats", NULL,
                  _("Request some server statistics (moderator or higher)"),
                  nap_com_sstats);
  nap_command_new("kill", "<user> [<reason>]",
                  _("Kill (disconnect) <user> with lower level than yours"),
                  nap_com_kill);
  nap_command_new("muzzle", "<user> [<reason>]",
                  _("Muzzle a user with lower level than yours"),
                  nap_com_muzzle);
  nap_command_new("unmuzzle", "<user>",
                  _("Unmuzzle a user with lower level than yours"),
                  nap_com_unmuzzle);
  nap_command_new("ban", "<user> [<reason>]",
                  _("Ban a user with lower level than yours from the network."),
                  nap_com_ban);
  nap_command_new("unban", "<user>",
                  _("Unban a user with lower level than yours."),
                  nap_com_unban);
  nap_command_new("op", "[<list of users>]",
                  _("Set one or more users as operator on the current channel. If no user is given the operator list is requested"),
                  nap_com_op);
  nap_command_new("deop", "<list of users>",
                  _("Remove on or more users as operator on the current channel."),
                  nap_com_deop);
  nap_command_new("voice", "[<list of users>]",
                  _("Give one or more users voice to speak on the current channel. If no user is specified the voice list is requested."),
                  nap_com_voice);
  nap_command_new("devoice", "<list of users>",
                  _("Remove voice from one or more users."),
                  nap_com_devoice);
  nap_command_new("invite", "<user>",
                  _("Invite a user to the current channel"),
                  nap_com_invite);
  nap_command_new("chmuzzle", "<user> [<reason>]",
                  _("Muzzle a user with lower level than yours in the current channel."),
                  nap_com_chmuzzle);
  nap_command_new("chunmuzzle", "<user>",
                  _("Unmuzzle a user with lower level than yours in the current channel."),
                  nap_com_chunmuzzle);
  nap_command_new("chban", "<user> [<reason>]",
                  _("Ban a user with lower level than yours from the current channel."),
                  nap_com_chban);
  nap_command_new("chunban", "<user>",
                  _("Unban a user with lower level than yours from the current channel."),
                  nap_com_chunban);
  /*
  nap_command_new("ban", "<user|ip> [[<timeout>[s|m|h|d|w]] <reason>]",
                  "Ban user or ip from server", 
                  nap_com_ban);
  */
}
static void nap_commands_exit() {
  //FIXME: cleanup
}

static void nap_command_module_init(nap_command_module_t* module) {
  module_init(MODULE(module), "Napster Command Module",
              (ModuleFunc)nap_commands_init,
              (ModuleFunc)nap_commands_run,
              (ModuleFunc)nap_commands_exit);
  
  module_require(pref_module_instance);
  module_require(command_module_instance);
  module_require(chat_module_instance);
}


module_t* nap_command_module_instance() {
  if (!NapCommandModule) {
    NapCommandModule = g_malloc0(sizeof(*NapCommandModule));
    nap_command_module_init(NapCommandModule);
  }
  return MODULE(NapCommandModule);
}



#undef NAP_COMMAND
#undef NAP_COMM
#undef CHECK_NET
#undef CHECK_NET2
