/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2006/07/22 17:14:03 $
 * $Author: sid4u $
 */

#include <string.h>

#include "lcore/buddy.h"
#include "lcore/utils.h"
#include "lcore/ui.h"
#include "nap.h"
#include "nap_hotlist.h"
#include "nap_proto.h"
#ifdef WIN32
#include "napalias.h"
#endif

typedef struct {
  module_t _module;
} nap_hot_module_t;

static nap_hot_module_t* NapHotModule = NULL;

static gboolean collect_users(nap_user_t* user, GSList** list) {
  *list = g_slist_append(*list, user);
  return FALSE;
}

static void nap_hot_change_user(buddy_t* buddy) {
  attribute_t* attr = attr_find(ATTRC(buddy), "buddy", "NapNick");
  const char* nicks = attr?attr_get_ptr(attr, char):NULL;
  GSList* old_users = NULL;
  GSList* new_users = NULL;
  GSList* dlist;

  g_assert(NapHotModule);
  buddy_foreach_user
    (buddy, NAP_ID, (UserForeach)collect_users, &old_users);

  make_list_from_string(&new_users, nicks, " ");
  for (dlist = new_users; dlist; dlist = dlist->next) {
    const char* nick = dlist->data;
    nap_user_t* user = nap_user_find(nick, NULL);
    if (!g_slist_find(old_users, user)) {
      buddy_add_user(buddy, USER(user));
    } else {
      old_users = g_slist_remove(old_users, user);
    }
  }
  for (dlist = old_users; dlist; dlist = dlist->next) {
    user_t* user = dlist->data;
    buddy_remove_user(buddy, USER(user));
  }
  g_slist_free(old_users);
  g_slist_free(new_users);
}

void nap_hot_user_online(nap_user_t* user, nap_net_t* net) {
  if (!g_list_find(user->nets, net)) {
    user->nets = g_list_append(user->nets, net);
    user_set_online(USER(user), TRUE);
  }
}

void nap_hot_user_offline(nap_user_t* user, nap_net_t* net) {
  if (g_list_find(user->nets, net)) {
    user->nets = g_list_remove(user->nets, net);
    if (!user->nets) user_set_online(USER(user), FALSE);
  }
}

static gboolean nap_hot_user_add(nap_user_t* user, nap_net_t* net) {
  nap_net_send(net, NAP_S_HOT_ADD_SEQ, TRUE, "%s", OBJ(user)->name);
  return FALSE;
}

static gboolean nap_hot_user_remove(nap_user_t* user, nap_net_t* net) {
  nap_hot_user_offline(user, net);
  return FALSE;
}

void nap_hot_left_net(nap_net_t* net) {
  users_foreach_monitored
    (NAP_ID, (UserForeach)nap_hot_user_remove, net);
}
void nap_hot_joined_net(nap_net_t* net) {
  users_foreach_monitored
    (NAP_ID, (UserForeach)nap_hot_user_add, net);
}

void nap_hot_new(nap_user_t* user, const char* des) {
  buddy_t* buddy;
  
  g_assert(NapHotModule);
  buddy = buddy_find_from_user(USER(user));
  if (buddy) return;

  buddy = buddy_new(OBJ(user)->name);
  if (des && *des) buddy_set_description(buddy, des);
  attr_store(ATTRC(buddy), "buddy", "NapNick", OBJ(user)->name);
  nap_hot_change_user(buddy);
}

gboolean nap_hot_in_hotlist(nap_user_t* user) {
  g_assert(NapHotModule);
  return (buddy_find_from_user(USER(user)) != NULL);
}
  
static void nap_hots_init() {
  attr_register_str("buddy", "NapNick", ATTR_SKIP, NULL);
  buddy_register_update(nap_hot_change_user);
}

static void nap_hot_module_init(nap_hot_module_t* module) {
  module_init(MODULE(module), "Nap Hotlist Module",
              (ModuleFunc)nap_hots_init, NULL, NULL);

  module_require(buddy_module_instance);
  module_require(nap_net_module_instance);
  module_require(nap_user_module_instance);
}


module_t* nap_hot_module_instance() {
  if (!NapHotModule) {
    NapHotModule = g_malloc0(sizeof(*NapHotModule));
    nap_hot_module_init(NapHotModule);
  }
  return MODULE(NapHotModule);
}

#ifdef WIN32
#define _NAP_HOTLIST_C_
#include "napaliasdef.c"
#endif

