/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.11 $
 * $Date: 2007/11/23 12:42:11 $
 * $Author: sid4u $
 */

#ifndef WIN32
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#endif
#include <stdlib.h>
#include <string.h>

#include <glib.h>

#include "lcore/l_http.h"
#include "lcore/ui.h"
#include "lcore/preferences.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "nap_napi.h"
#include "nap_network.h"
#include "nap_ui.h"


void nap_napi_process_body(http_t* http) {
  nap_net_t* net;
  nap_ng_t* ng;
  nap_server_t* server;
  char* data;
  char* pos;
  buffer_t* buffer = b_socket_get_in_buffer(BSOCK(http));

  //  print_raw("napi body", buffer->data, buffer->size);

  data = buffer->data;
  pos = data;

  while ((pos = strchr(pos, '\n')) != NULL) {
    char *n1, *n2, *n3, *n4;
    char *ip, *port;
    char* nname;
    char* line;
    char *users, *files, *gigs;
    char* addy;
    char* admin;
    char* www;
    char* utf8name;

    *pos = 0;
    n1    = arg(data, 0);
    ip    = arg(NULL, 0);
    port  = arg(NULL, 0);
    nname = arg(NULL, 0);
    line  = arg(NULL, 0);
    users = arg(NULL, 0);
    files = arg(NULL, 0);
    gigs  = arg(NULL, 0);
    addy  = arg(NULL, 0);
    n2    = arg(NULL, 0); // unknown
    n3    = arg(NULL, 0); // unknown
    n4    = arg(NULL, 0); // unknown
    admin = arg(NULL, 0);
    www   = arg(NULL, 0);
    utf8name = auto_text(nap_net_to_utf8(NULL, nname));

    if (n4 && utf8name) {
      // own network for servers in group 'n/a'
      if (!strcmp(utf8name, "n/a") || !strcmp(utf8name, "N/A")) 
        utf8name = addy;

      net = nap_find_net(utf8name);
      if (!net) {
        ng = nap_ng_create(_("New Networks"), NAP_NG_AUTO_CONNECT,
                           NAP_CREATE_REPLACE);
        net = nap_net_create(ng, utf8name, NAP_CREATE_ALWAYS);
      }
      server = nap_server_create(net, addy, NAP_CREATE_REPLACE);

      server->ip = inet_addr(ip);
      server->port = atoi(port);
      server->users = atoi(users);
      server->files = atoi(files);
      server->gigs = atoi(gigs);
      server->linespeed = atoi(line);
      g_free(server->admin);
      server->admin = g_strdup(admin);
      g_free(server->web);
      server->web = to_utf8(www, -1, NULL, NULL, 2, "utf-8", "latin1");

      object_set_dirty(OBJ(server));
    }

    data = pos+1;
    pos = data;
  }
  
  // comment the follwing as it's commented in nap_network
  // nap_net_remove_not_napi();

  nap_network_save();

}

static gboolean nap_napi_end(http_t* http, const char* message) {

  if (socket_get_status(SOCK(http)) == ST_INPUT_COMPLETE) {
    nap_napi_process_body(http);
  } else {
    lopster_notify(LOPSTER_MESSAGE_INFO, _("Error"),
                   _("Failed to get the server list."));
  }
  socket_set_status(SOCK(http), ST_UNKNOWN);
  return http_end(http, message);
}

void nap_napi_get_list(url_t* url)
{
    http_t* http = http_new_from_url(url);
    if (!http) return;

    socket_set_end(SOCK(http), (SockEndFunc)nap_napi_end);

    g_log("NAPI", G_LOG_LEVEL_DEBUG, "getting server list");
    // set the function that handles the received data
    http_get(http);
}
