/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.9 $
 * $Date: 2008/12/23 13:38:14 $
 * $Author: sgop $
 */

#ifndef _NAP_NET_H_
#define _NAP_NET_H_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

typedef struct _nap_ng_t nap_ng_t;
typedef struct _nap_net_t nap_net_t;
typedef struct _nap_server_t nap_server_t;
typedef struct _nap_ce_t nap_ce_t;

#include "lcore/tcp_socket.h"
#include "lcore/file_tree.h"
#include "lcore/library.h"
#include "lcore/search.h"
#include "lcore/module.h"
#include "nap_search.h"
#include "nap_users.h"


#define NAP_TYPE_UNKNOWN           0
#define NAP_TYPE_OPENNAP           1
#define NAP_TYPE_OPENNAP_NG        2
#define NAP_TYPE_SLAVANAP          3

#define NAP_SERVER_OFFLINE         0
#define NAP_SERVER_RESOLVING       1
#define NAP_SERVER_WAITING         2
#define NAP_SERVER_CONNECTING      3
#define NAP_SERVER_ONLINE          4

#define NAP_CREATE_ALWAYS          0
#define NAP_CREATE_UNIQUE          1
#define NAP_CREATE_REPLACE         2

#define NAP_SERVER_META         0x01

#define NAP_NET_OWN_SERVER      0x01
#define NAP_NET_IGN_GLOBAL      0x02
#define NAP_NET_EMU_WINMX       0x04
#define NAP_NET_PRIVATE         0x08
#define NAP_NET_AUTO            0x10

#define NAP_NG_AUTO_CONNECT      0x1

#define NAP_L_UNKNOWN -1
#define NAP_L_LEECH    0
#define NAP_L_USER     1
#define NAP_L_MOD      2
#define NAP_L_ADMIN    3
#define NAP_L_ELITE    4

#define NAP_MODE_ERROR       0x000001
#define NAP_MODE_BAN         0x000002
#define NAP_MODE_CHANGE      0x000004
#define NAP_MODE_KILL        0x000008
#define NAP_MODE_LEVEL       0x000010
#define NAP_MODE_SERVER      0x000020
#define NAP_MODE_MUZZLE      0x000040
#define NAP_MODE_PORT        0x000080
#define NAP_MODE_WALLOP      0x000100
#define NAP_MODE_CLOAK       0x000200
#define NAP_MODE_FLOOD       0x000400
#define NAP_MODE_PING        0x000800
#define NAP_MODE_MSG         0x001000
#define NAP_MODE_WHOIS       0x002000
#define NAP_MODE_ANNOUNCE    0x004000
#define NAP_MODE_BROWSE      0x008000
#define NAP_MODE_MOTD        0x010000
#define NAP_MODE_FRIEND      0x020000
#define NAP_MODE_CHANNEL     0x040000
#define NAP_MODE_REGISTER    0x080000
#define NAP_MODE_VAR         0x100000
#define NAP_MODE_ABUSE       0x200000
#define NAP_MODE_NUMBER      22
#define NAP_MODE_MASK_ON     0x003fff
#define NAP_MODE_MASK_SN     0x1fffff
#define NAP_MODE_MASK_NG     0x203fff
#define NAP_MODE_MASK_ALL    0x3fffff
#define NAP_MODE_MASK_NAP    (NAP_MODE_MASK_ON & NAP_MODE_MASK_SN & NAP_MODE_MASK_NG)
#define NAP_MODE_MASK_NONE   0x000000

#define NAP_NET_ONLINE(net) ((net) && (net)->active && (net)->active->status == NAP_SERVER_ONLINE)

#define S_NET_USER_DISCONNECT  300

typedef struct {
    object_t _object;
    GList* list;
    time_t refreshed;
    int min, max;
    gboolean refreshing;
    nap_net_t* net;
} nap_channels_t;

struct _nap_ce_t {
    nap_net_t* net;
    char* name;
    char* topic;
    int online;
    int limit;
    int level;
};

struct _nap_ng_t {
    object_t _object;
    GList* nets;
    GList* nets_auto;
    int flags;
    int online;
    int keep;
};

struct _nap_net_t {
    tcp_socket_t _socket;
    nap_ng_t* group;
    char* charset;
    gboolean charset_warned;
    int type;
    int major, minor, micro;
    char* cur_nick;
    char* cur_pass;
    char* nick;
    char* pass;
    GSList* autojoin;
    int level;
    int flags;
    GList* servers;
    nap_server_t* active;
    int max_searches;
    int cur_searches;
    int max_searches_pm;
    int searches_pm;
    int skip_search_end;
    GList* active_searches;
    GList* queued_searches;
    time_t last_success;
    unsigned int number_fails;
    buffer_t* temp_buffer;
    nap_channels_t* channel_list;
};

struct _nap_server_t {
    object_t _object;
    ip_t ip;
    port_t port;
    nap_net_t* net;
    int relogin;
    int flags;
    int users, files, gigs;
    int status;
    int linespeed;
    char* message;
    char* admin;
    char* web;
    time_t last_success;
    unsigned number_fails;
};

#include "nap_users.h"

G_BEGIN_DECLS

module_t* nap_net_module_instance();

int         NapSpeedFromType(int no);
const char* NapSpeedStr     (int no);
const char* NapSpeedStrShort(int no);

const char* NapLevel     (int no);
const char* NapLevelShort(int no);
int         NapIsLevel   (int no);
const char* NapUserModes (int no);

int NapLevelInt(const char* level);

void nap_update_stats();

nap_ng_t*  nap_ng_create        (const char* name, int flags, int mode);
int        nap_ng_stats         (nap_ng_t* ng, int mode);
void       nap_ng_set_auto      (nap_ng_t* ng, gboolean val);
GList*     nap_get_groups       (void);
void       nap_ng_start_connect (nap_ng_t* ng);
void       nap_ng_stop_connect  (nap_ng_t* ng);

nap_net_t* nap_net_create       (nap_ng_t* ng, const char* name, int mode);
int        nap_net_success      (nap_net_t* net);
void       nap_net_go           (nap_net_t* net);
char*      nap_net_print_version(nap_net_t* net);
int        nap_net_stats        (nap_net_t* net, int mode);
void       nap_net_set_level    (nap_net_t* net, int level);
void       nap_net_send         (nap_net_t* net, guint16 type, gboolean convert, const char* fmt, ...);
void       nap_net_disconnect   (nap_net_t* net);
void       nap_net_move         (nap_net_t* net, nap_ng_t* to);
void       nap_net_start_connect(nap_net_t* net);
void       nap_net_stop_connect (nap_net_t* net);
void       nap_net_show_channels(nap_net_t* net);
void       nap_net_get_global_users(nap_net_t* net, char *args);
gboolean   nap_net_validate     (nap_net_t* net);
void       nap_net_finish_channel(nap_net_t* net);
void       nap_net_add_channel  (nap_net_t* net, const char* name,
                                 int online, int level, int limit, const char* topic);
char*      nap_net_get_nick     (nap_net_t* net);
char*      nap_net_get_pass     (nap_net_t* net);
void       nap_net_add_search   (nap_net_t* net, search_t* search);
void       nap_net_remove_search(nap_net_t* net, search_t* search);
void       nap_net_search_finish(nap_net_t* net);
void       nap_net_search_finish_latest(nap_net_t* net);
void       nap_net_new_result   (file_t* file, nap_user_t* user);
void       nap_net_set_password (nap_net_t* net, const char* passwd);
void       nap_net_set_nick     (nap_net_t* net, const char* nick);
void       nap_usermode_send    (nap_net_t* net, int mode);
void       nap_net_get_channels(nap_net_t* net);

char* nap_net_to_utf8(nap_net_t* net, const char* data);


nap_server_t* nap_server_create (nap_net_t* net, const char* address, int mode);
void          nap_server_connect(nap_server_t* server);
void          nap_server_disconnect(nap_server_t* server);

void       nap_network_save      (void);
nap_net_t* nap_get_net           (int index);

void nap_net_remove_not_napi();
nap_net_t* nap_find_net(const char* name);


void nap_net_global_pass_changed(const char* passwd);
void nap_net_local_pass_changed(const char* passwd);


//GList*     nap_available_servers(char* net);
//GList*     nap_available_links  (GList* source, char* server_name, nap_net_t* net);

port_t nap_get_listen_port(void);
void nap_service_get_list(const char* url);
void nap_connect_change(unsigned status);

G_END_DECLS

#endif
