/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.7 $
 * $Date: 2007/11/21 20:20:07 $
 * $Author: sgop $
 */

#include <string.h>

#include "lcore/chat.h"
#include "lcore/utils.h"
#include "lcore/l_i18n.h"
#include "lcore/l_time.h"
#include "nap_ping.h"
#include "nap_proto.h"

static GList* NapPings = NULL;

typedef struct {
  char* user;
  struct timeval tv;
} nap_ping_t;


static nap_ping_t* nap_ping_new(const char *user) {
  nap_ping_t* result;

  result = g_malloc(sizeof(*result));
  result->user = g_strdup(user);
  NapPings = g_list_append(NapPings, result);

  return result;
}

static void nap_ping_destroy(nap_ping_t* ping) {
  NapPings = g_list_remove(NapPings, ping);
  g_free(ping->user);
  g_free(ping);
}

static nap_ping_t* nap_ping_search(const char* user) {
  GList* dlist;
  nap_ping_t* result;

  for (dlist = NapPings; dlist; dlist = dlist->next) {
    result = (nap_ping_t *) (dlist->data);
    if (!g_strcasecmp(result->user, user))
      return result;
  }

  return NULL;
}

static nap_ping_t* nap_ping_touch(const char* user) {
  nap_ping_t* ping;

  ping = nap_ping_search(user);
  if (!ping) ping = nap_ping_new(user);

  gettimeofday(&(ping->tv), 0);
  return ping;
}

void nap_ping_user(const char* user, nap_net_t* net) {
  nap_ping_t* ping;

  ping = nap_ping_touch(user);

  chat_print(NULL, "+\0client",
             "+\1c_text", _("Pinging"),
             "+\1nick", user, "\0\0",
             "+\0nl", NULL);
  
  nap_net_send(net, NAP_S_PING, TRUE, "%s", user);
}

static int nap_ping_diff(nap_ping_t* ping) {
  struct timeval tv;
  int result;

  gettimeofday(&(tv), 0);

  result = (tv.tv_sec - ping->tv.tv_sec) * 1000 +
    (tv.tv_usec - ping->tv.tv_usec) / 1000;
  return result;
}

void nap_pong_received(nap_net_t* net, const char* user, const char* add) {
  nap_ping_t* ping = nap_ping_search(user);

  if (!ping) {
    nap_user_t* u = nap_user_find(user, net);
    // we are used to unrequested pongs from winmx26
    if (u->client_id != NAP_CLIENT_WINMX_26) {
      chat_print(NULL, "+\0client",
                 "+\1c_text", _("Received unrequested pong from user"),
                 "+\1nick", user, "\0\0",
                 NULL);
      if (add)
        chat_print(NULL,
                   "+\1c_text", "(",
                   "+\0c_text", add,
                   "+\0c_text", ")",
                   NULL);
      chat_print(NULL, "+0nl", NULL);
    }
  } else {
    chat_print(NULL, "+\0client",
               "+\1c_text", _("Pong from"),
               "+\1nick", user, "\0\0",
               NULL);
    if (add) 
      chat_print(NULL,
                 "+\1c_text", "(",
                 "+\0c_text", add,
                 "+\0c_text", ")",
                 NULL);
    
    chat_print(NULL,
               "+\1c_text", l_printf("[%d ms]", nap_ping_diff(ping)),
               NULL);
    chat_print(NULL, "+0nl", NULL);
    nap_ping_destroy(ping);
  }
  
}

void nap_ping_received(nap_net_t* net, const char* user, const char* add) {
  
  // just reply slavanap server pings
  if (strcmp(user, "server")) {
    if (add) {
      chat_print(NULL, "+\0client",
                 "+\1nick", user, "\0\0",
                 "+\1c_text", l_printf("(%s)", add),
                 "+\1c_text", _("has pinged you!"),
                 "+\0nl", NULL);
    } else {
      chat_print(NULL, "+\0client",
                 "+\1nick", user, "\0\0",
                 "+\1c_text", _("has pinged you!"),
                 "+\0nl", NULL);
    }
  }
  if (add) {
    nap_net_send(net, NAP_S_PONG, TRUE, "%s %s", user, add);
  } else {
    nap_net_send(net, NAP_S_PONG, TRUE, "%s", user);
  }
}
