/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.10 $
 * $Date: 2006/07/22 17:14:03 $
 * $Author: sid4u $
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifndef WIN32
#include <netinet/in.h>
#endif

#include "lcore/utils.h"
#include "lcore/preferences.h"
#include "lcore/search.h"
#include "lcore/l_base.h"
#include "lcore/attribute.h"
#include "lcore/version.h"
#include "lcore/l_i18n.h"
#include "nap_proto.h"
#include "nap_hotlist.h"
#include "nap_whois.h"
#include "nap_browse.h"
#include "nap_upload.h"
#include "nap_server_parse.h"
#include "nap.h"
#include "nap_ui.h"
#include "nap_chat.h"
#include "nap_ping.h"
#include "nap_download.h"
#include "nap_users.h"

/* #define NAP_R_INVALID        0xffff */
#define NAP_R_ERROR		            0
#define NAP_R_LOGIN_ACK           3
#define NAP_R_UNSHARE_ALL_ACK   110
#define NAP_R_SEARCH_RESULT     201
#define NAP_R_SEARCH_END        202
#define NAP_R_DOWNLOAD_ACK      204
#define NAP_R_PRIVATE_MSG       205
#define NAP_R_DOWNLOAD_ERROR    206
#define NAP_R_USER_SIGNON       209
#define NAP_R_USER_SIGNOFF      210
#define NAP_R_BROWSE_RESPONSE   212
#define NAP_R_BROWSE_END        213
#define NAP_R_SERVER_STATS      214
#define NAP_R_HOTLIST_ERROR     302
#define NAP_R_PART_CHANNEL      401
#define NAP_R_HOTLIST_ACK       301
#define NAP_R_PUBLIC_MSG        403
#define NAP_R_SERVER_MSG        404
#define NAP_R_JOIN_ACK          405
#define NAP_R_USER_JOIN         406
#define NAP_R_USER_PART         407
#define NAP_R_CHANNEL_USER      408
#define NAP_R_USER_END          409
#define NAP_R_TOPIC             410
#define NAP_R_UPLOAD_FIREWALL   501
#define NAP_R_WHOIS_RESPONSE    604
#define NAP_R_WHOWAS_RESPONSE   605
#define NAP_R_UPLOAD_REQUEST    607
#define NAP_R_UPLOAD_ERROR      609
#define NAP_R_CHANNEL_SHORT_END 617
#define NAP_R_CHANNEL_SHORT     618
#define NAP_R_REMOTE_QUEUED     620
#define NAP_R_MOTD              621
#define NAP_R_WALLOP_MSG        627
#define NAP_R_GLOBAL_MSG        628
#define NAP_R_BROWSE_DIRECT     640
#define NAP_R_BROWSE_DIRECT_ERR 642
#define NAP_R_BROWSE_DIRECT_OK  641
#define NAP_R_LOGIN_ATTEMPT     748
#define NAP_R_PING              751
#define NAP_R_PONG              752
#define NAP_R_EMOTE_MSG         824
#define NAP_R_CHANNEL_END       827
#define NAP_R_CHANNEL_ENTRY     828
#define NAP_R_USAGE_STATS		  10115

/* unused
#define NAP_USER_SPEED		  601
 */

//#define NAP_REGISTER_OK		  8
//#define NAP_REGISTER_FAIL	  9
//#define NAP_BAD_NICK		  10
//#define NAP_DISCONNECTING	  316
//#define NAP_CHANNEL_BAN_LIST_END  420
//#define NAP_CHANNEL_BAN_LIST	  421
//#define NAP_CHANNEL_ALT_TOPIC     425
//#define NAP_ALTER_PORT		  613
//#define NAP_BANLIST		  615
//#define NAP_IP_BANLIST		  616
//#define NAP_DATA_PORT_ERROR	  626
//#define NAP_NICK_BANLIST	  629
//#define NAP_PING_SERVER		  750
//#define NAP_REDIRECT		  821
//#define NAP_CYCLE		  822
//#define NAP_NAMES_LIST		  825
//#define NAP_NAMES_LIST_END	  830
//#define NAP_GUSER_LIST_END	  831
//#define NAP_GUSER_LIST   	  832
//#define NAP_BLOCKMD5              931
//#define NAP_LINKS		  10112
//#define NAP_VERSION_STATS         10118
//#define NAP_WHO_WAS               10121
//#define NAP_HISTOGRAM		  10123
//#define NAP_HISTOGRAM_END	  10124
//#define NAP_SHISTOGRAM		  10125
//#define NAP_SHISTOGRAM_END	  10126
//#define NAP_CLASS_LIST            10252
//#define NAP_DLINE_LIST            10255
//#define NAP_ILINE_LIST            10258
//#define NAP_ELINE_LIST            10261
//#define NAP_BROWSE_NEW		  10301
//#define NAP_BROWSE_RESULT_NEW	  10302


#define HANDLER(f) static void f (nap_net_t* net, char* data)
#define NAP_CALL(f) f(net, command->message)
#define NAP_CALLC(f) f(net, auto_text(nap_net_to_utf8(net, command->message)))

#define CHECK(expr) \
  if (!(expr)) {g_log("NAP_PROTO", G_LOG_LEVEL_WARNING, "invalid message in %s:%s at line %u", __FILE__, __FUNCTION__, __LINE__); return;}

nap_message_t* nap_consume_message(buffer_t* buffer) {
  nap_message_t* result;
  guint16 len;
  unsigned avail;
  char* data;

  //  g_log("NAP_PROTO", G_LOG_LEVEL_DEBUG, "using charset '%s'", charset);
  if (!buffer) return NULL;
  
  avail = buffer_avail(buffer);
  if (avail< 4) return NULL;
  data = buffer_read_pos(buffer);

  len = unpack_u16(&data, &avail);
  if (avail < len + 2) return NULL;


  result = g_malloc0(sizeof(nap_message_t));
  result->type = unpack_u16(&data, &avail);
/*   result->message = to_utf8(data, len, &s, &error, 4, "utf-8", charset, c2, "latin1"); */
/*   result->len = (guint16)s; */
  result->message = g_malloc(len+1);
  memcpy(result->message, data, len);
  result->message[len] = 0;
  result->len = len;

/*   if (error) { */
/*     result->message = g_strdup(error->message); */
/*     result->len = result->message?strlen(result->message):0; */
/*     result->type = NAP_R_INVALID; */
/*     g_error_free(error); */
/*   } */

  buffer_consume_virt(buffer, len + 4);

  return result;
}

void nap_pack_command(buffer_t* buffer, guint16 type, const char* message) {
  char* pos;
  gsize len = message?strlen(message):0;

  if (type != NAP_S_SHARE_FILE && type != NAP_S_SHARE_GENERIC &&
      type != NAP_S_DOWNLOAD_STARTED && type != NAP_S_DOWNLOAD_STOPPED &&
      type != NAP_S_UPLOAD_STARTED && type != NAP_S_UPLOAD_STOPPED)
    g_log("NAP_PROTO", G_LOG_LEVEL_DEBUG, "S: %5u [%s]", type, message);
  
  //  printf("pack command at %u %u %u\n", buffer->consumed, buffer->size, buffer->max);
  pos = buffer_need_space(buffer, 4);
  pack_u16(pos, len);
  pack_u16(pos+2, type);
  buffer->size += 4;
  if (len) buffer_add_data(buffer, message, len);
}


HANDLER(handle_unknown) {
  (void)data;
  (void)net;
}

HANDLER(handle_login_ack) {
  (void)data;
  nap_net_success(net);
}

HANDLER(handle_server_stats) {
  char* users = arg(data, 0);
  char* files = arg(NULL, 0);
  char* gigs  = arg(NULL, 0);
  
  if (!net->active) return;

  CHECK(gigs);

  net->active->users = atoi(users);
  net->active->files = atoi(files);
  net->active->gigs = atoi(gigs);

  object_set_dirty(OBJ(net->active));
  object_set_dirty(OBJ(net));
  object_set_dirty(OBJ(net->group));

  nap_update_stats();
}

static void nap_extract_version(nap_net_t* net, const char* string) {
  if (!strncasecmp(string, "Slavanap ", 9)) {
    net->type = NAP_TYPE_SLAVANAP;
    sscanf(string+9, "%d.%d.%d", 
	   &(net->major), &(net->minor), &(net->micro));
  } else if (!strncasecmp(string, "Opennap ", 8)) {
    net->type = NAP_TYPE_OPENNAP;
    sscanf(string+8, "%d.%d", &(net->major), &(net->minor));
  } else if (!strncasecmp(string, "Opennap-ng ", 11)) {
    net->type = NAP_TYPE_OPENNAP_NG;
    sscanf(string+11, "%d.%d", &(net->major), &(net->minor));
  } else {
    g_log("NAP_PROTO", G_LOG_LEVEL_WARNING,
          "got unknown server type: '%s'\n", string);
    net->type = NAP_TYPE_UNKNOWN;
  }
}

HANDLER(handle_motd) {
  if (data && !strncmp(data, "VERSION ", 8)) {
    nap_extract_version(net, data+8);
    nap_net_go(net);
  }
  nap_motd_message(net, data);
}

HANDLER(handle_server_message) {
  if (!data) return;
  if (nap_check_skip(net, data)) {
    ;
  } else if (nap_check_level_set(net, data)
             || nap_check_channel_wallop(net, data)
             || nap_check_channel_mode2(net, data)
             || nap_check_channel_limit_level(net, data)
             || nap_check_channel_mode(net, data)
             || nap_check_channel_limit(net, data)
             || nap_check_channel_level(net, data)
             || nap_check_channel_topic(net, data)
             || nap_check_flag(net, data)
             || nap_check_too_many_pending(net, data)
	     ) {
    nap_server_message(net, l_strdup("*** the next line was parsed by lopster ***"));
    //    return;
  }
  
  nap_server_message(net, data); 
}

HANDLER(handle_channel_list_entry) {
  char* name   = arg(data, 0);
  char* online = arg(NULL, 0);
  char* temp   = arg(NULL, 0);
  char* level  = arg(NULL, 0);
  char* limit  = arg(NULL, 0);
  char* topic  = arg(NULL, 0);

  (void)temp;
  CHECK(online);
  
  nap_net_add_channel(net, name, atoi(online),
                      level?atoi(level):NAP_L_LEECH,
                      limit?atoi(limit):0, topic);
}

HANDLER(handle_channel_list_end) {
  nap_net_finish_channel(net);
}

HANDLER(handle_channel_list_short) {
  char* name   = arg(data, 0);
  char* online = arg(NULL, 0);
  char* topic  = arg(NULL, 1);

  CHECK(online);

  nap_net_add_channel(net, name, atoi(online), NAP_L_UNKNOWN, 0, topic);
}

HANDLER(handle_channel_join_ack)
{
    char* channel = arg(data, 0);
    nap_pub_t* npp;
    
    CHECK(channel);
    
    npp = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
    if (!npp)
    {
        nap_net_send(net, NAP_S_PART_CHANNEL, TRUE, "%s", channel);
    }
    else
    {
        nap_channel_joined(npp);
    }
}

HANDLER(handle_channel_user) {
  char* channel = arg(data, 0);
  char* nick    = arg(NULL, 0);
  char* shared  = arg(NULL, 0);
  char* speed   = arg(NULL, 0);
  nap_pub_t* npp;

  CHECK(speed);

  npp = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
  if (!npp) return;

  nap_cu_add(npp, nick, atoi(shared), atoi(speed));
}

HANDLER(handle_topic) {
  char* channel = arg(data, 0);
  char* topic   = arg(NULL, 1);
  nap_pub_t* npp;


  CHECK(channel);

  npp = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
  if (!npp) return;

  nap_channel_new_topic(npp, topic, NULL);
}

HANDLER(handle_user_join)
{
    char* channel = arg(data, 0);
    char* nick    = arg(NULL, 0);
    char* shared  = arg(NULL, 0);
    char* speed   = arg(NULL, 0);
    nap_pub_t* npp;
    
    CHECK(speed);
    
    npp = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
    if (!npp) return;
    
    nap_cu_joined(npp, nick, atoi(shared), atoi(speed));
}

HANDLER(handle_user_part) {
  char* channel = arg(data, 0);
  char* nick    = arg(NULL, 0);
  char* shared  = arg(NULL, 0);
  char* speed   = arg(NULL, 0);
  nap_pub_t* npp;
  
  CHECK(speed);

  npp = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
  if (!npp) return;

  nap_cu_parted(npp, nick, atoi(shared), atoi(speed));
}

HANDLER(handle_public_message) {
  char* channel = arg(data, 0);
  char* nick    = arg(NULL, 0);
  char* message = arg(NULL, 1);
  
  CHECK(nick);

  nap_public_message(net, channel, nick, message);
}

HANDLER(handle_emote_message) {
  char* channel = arg(data, 0);
  char* nick    = arg(NULL, 0);
  char* message = arg(NULL, 0);
  
  CHECK(nick);
  
  nap_emote_message(net, channel, nick, message);
}

HANDLER(handle_wallop_message) {
  char* nick    = arg(data, 0);
  char* message = arg(NULL, 1);
  
  CHECK(nick);

  nap_wallop_message(net, nick, message);
}

HANDLER(handle_global_message) {
  char* nick    = arg(data, 0);
  char* message = arg(NULL, 1);
  
  CHECK(nick);

  nap_global_message(net, nick, message);
}

HANDLER(handle_private_message) {
  char* nick    = arg(data, 0);
  char* message = arg(NULL, 1);

  CHECK(nick);

  nap_private_message(net, nick, message);
}

HANDLER(handle_part_channel) {
  char* channel = arg(data, 0);
  nap_pub_t* npp;

  CHECK(channel);
  
  npp = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
  if (!npp) return;
  nap_channel_part(npp);
}

HANDLER(handle_search_result)
{
    char* name  = arg(data, 0);
    char* md5   = arg(NULL, 0);
    char* size  = arg(NULL, 0);
    char* rate  = arg(NULL, 0);
    char* freq  = arg(NULL, 0);
    char* dur   = arg(NULL, 0);
    char* nick  = auto_text(nap_net_to_utf8(net, arg(NULL, 0)));
    char* ip    = arg(NULL, 0);
    char* speed = arg(NULL, 0);
    int bitrate;
    file_t* file;
    nap_user_t* user;
    
    (void)md5;
    file = file_new();
    file_set_origname(file, g_strdup(name));
    file_set_locname(file, nap_net_to_utf8(net, name));
    file->size = strtoll(size, NULL, 10);
    
    bitrate = atoi(rate);
    if (bitrate != 8 && bitrate != 24)
    {
        attr_store_str(ATTRC(file), "file", "Bitrate", rate);
        attr_store_str(ATTRC(file), "file", "Frequency", freq);
        attr_store_str(ATTRC(file), "file", "Duration", dur);
    }
  
    user = nap_user_find(nick, net);
    nap_user_setup(user, IP_FROM_LE(strtoul(ip, NULL, 10)), atoi(speed));
    nap_net_new_result(file, user);
    
    object_kill(OBJ(file));
}

HANDLER(handle_search_end)
{
    (void)data;
    if (net->skip_search_end)
    {
        net->skip_search_end--;
    }
    else
    {
        nap_net_search_finish(net);
    }
}

HANDLER(handle_user_on) {
  char* nick  = arg(data, 0);
  char* speed = arg(NULL, 0);  // ingored at the moment

  (void)speed;
  CHECK(nick);

  nap_hot_user_online(nap_user_find(nick, NULL), net);
}

HANDLER(handle_whois_response) {
  char* nick    = arg(data, 0);
  char* level   = arg(NULL, 0);
  char* online  = arg(NULL, 0);
  char* channel = arg(NULL, 0);
  char* status  = arg(NULL, 0);
  char* shared  = arg(NULL, 0);
  char* dl      = arg(NULL, 0);
  char* ul      = arg(NULL, 0);
  char* link    = arg(NULL, 0);
  char* client  = arg(NULL, 0);
  char* tdl     = arg(NULL, 0);
  char* tul     = arg(NULL, 0);
  char* ip      = arg(NULL, 0);
  char* sport   = arg(NULL, 0);
  char* dport   = arg(NULL, 0);
  char* email   = arg(NULL, 0);
  char* server  = arg(NULL, 0);
  nap_whois_t* whois;
  nap_user_t* user;

  CHECK(client);

  user = nap_user_find(nick, net);
  whois = user->whois;
  if (!whois) return;
  if (whois->net != net) return;  //discard whois if net has changed.
  
  whois->level = NapLevelInt(level);
  g_free(whois->str_level);
  whois->str_level = g_strdup(level);
  whois->online = online?atoi(online):0;
  g_free(whois->channel);
  whois->channel = g_strdup(channel);
  whois->status = NAP_WHOIS_ONLINE;
  g_free(whois->stat);
  whois->stat = g_strdup(status);
  user->shared = atoi(shared);
  whois->dl = atoi(dl);
  whois->ul = atoi(ul);
  user->speed = atoi(link);
  nap_user_set_client(user, client);
  // advanced
  whois->tdl = tdl?atoi(tdl):-1;
  whois->tul = tul?atoi(tul):-1;
  if (ip && dport) user_set_ip_port(USER(user), inet_addr(ip), atoi(dport));
  else if (ip) user_set_ip(USER(user), inet_addr(ip));
  else if (dport) user_set_port(USER(user), atoi(dport));
  whois->server_port = sport?atoi(sport):0;
  g_free(whois->email);
  whois->email = g_strdup(email);
  g_free(whois->server);
  whois->server = g_strdup(server);

  nap_whois_received(user);
}

HANDLER(handle_whowas_response) {
  char* nick    = arg(data, 0);
  char* level   = arg(NULL, 0);
  char* online  = arg(NULL, 0);
  nap_whois_t* whois;
  nap_user_t* user;

  CHECK(online);

  user = nap_user_find(nick, net);
  whois = user->whois;
  if (!whois) return;
  if (whois->net != net) return;  //discard whois if net has changed.
  
  whois->level = NapLevelInt(level);
  g_free(whois->str_level);
  whois->str_level = g_strdup(level);
  whois->online = online?atoi(online):0;
  g_free(whois->channel);
  whois->status = NAP_WHOIS_OFFLINE;

  nap_whois_received(user);
}

HANDLER(handle_user_off) {
  char* nick = arg(data, 0);
  
  CHECK(nick);
  nap_hot_user_offline(nap_user_find(nick, NULL), net);
}

HANDLER(handle_browse_direct) {
  char* nick = arg(data, 0);
  char* ip   = arg(NULL, 0);
  char* port = arg(NULL, 0);
  nap_user_t* user;

  CHECK(nick);

  user = nap_user_find(nick, NULL);
  if (ip && port) {
    nap_browse_outgoing_send
      (net, IP_FROM_LE(strtoul(ip, NULL, 10)), htons(atoi(port)));
  } else {
    nap_browse_set_last_net(net);
    nap_net_send(net, NAP_S_BROWSE_DIRECT_OK, TRUE, "%s", nick);
  }
}

HANDLER(handle_browse_direct_ok) {
  char* nick = arg(data, 0);
  char* ip   = arg(NULL, 0);
  char* port = arg(NULL, 0);
  nap_user_t* user;

  CHECK(port);
  CHECK(atoi(port));

  user = nap_user_find(nick, net);

  nap_browse_outgoing_get
    (user, IP_FROM_LE(strtoul(ip, NULL, 10)), htons(atoi(port)));
}

HANDLER(handle_browse_direct_err) {
  char* nick    = arg(data, 0);
  char* message = arg(NULL, 1);
  nap_user_t* user;
  (void)net;

  CHECK(nick);
  user = nap_user_find(nick, NULL);
  nap_browse_error(user, message);
}

HANDLER(handle_browse_response) {
  char* nick = auto_text(nap_net_to_utf8(net, arg(data, 0)));
  char* name = arg(NULL, 0);
  char* md5  = arg(NULL, 0);
  char* size = arg(NULL, 0);
  char* rate = arg(NULL, 0);
  char* freq = arg(NULL, 0);
  char* dur  = arg(NULL, 0);
  browse_t* browse;
  file_t* file;
  nap_user_t* user;

  (void)net;
  (void)md5;

  CHECK(dur);
  
  user = nap_user_find(nick, NULL);
  browse = browse_find(USER(user));
  if (!browse) return;

  file = file_new();
  file_set_origname(file, g_strdup(name));
  file_set_locname(file, nap_net_to_utf8(net, name));
  file->size = strtoll(size, NULL, 10);
  // ignore md5
  //    file->md5 = g_strdup(md5);

  attr_store_str(ATTRC(file), "file", "Bitrate", rate);
  attr_store_str(ATTRC(file), "file", "Frequency", freq);
  attr_store_str(ATTRC(file), "file", "Duration", dur);

  nap_browse_insert_file(NAP_BROWSE(browse), file);
}

HANDLER(handle_browse_end) {
  char* nick = arg(data, 0);
  char* ip   = arg(NULL, 0);
  nap_user_t* user;
  
  (void)net;

  CHECK(nick);

  user = nap_user_find(nick, NULL);
  
  if (ip) user_set_ip(USER(user), IP_FROM_LE(strtoul(ip, NULL, 10)));
  
  nap_browse_finished(user, NULL);

}

HANDLER(handle_upload_request) {
  char* nick    = auto_text(nap_net_to_utf8(net, arg(data, 0)));
  char* winname = arg(NULL, 0);
  char* speed   = arg(NULL, 0);
  upload_t* upload;
  nap_user_t* user;

  CHECK(winname);

  user = nap_user_find(nick, net);
  nap_user_setup(user, 0, speed?atoi(speed):0);

  upload = upload_find(USER(user), winname);
  if (!upload) {
    file_t* file = library_winname_requested(winname);
    if (!file) {
      g_log("UPLOAD", G_LOG_LEVEL_WARNING, "'%s' requested file not shared '%s'",
            nick, winname);
      return;  // not found or not shared or whatever
    }
    upload = UPLOAD(nap_upload_new(user, file));
    upload_insert(upload, TRUE);
  } else {
    upload_renew(upload);
  }
}

HANDLER(handle_hotlist_error) {
  char* user = arg(data, 0);

  CHECK(user);

  lopster_notify(LOPSTER_MESSAGE_ERROR, _("Napster hotlist error"),
                 l_printf(_("User <b>%s</b> could not be added to your hotlist on network <b>%s</b>"),
                          user, OBJ(net)->name));
}

HANDLER(handle_pong) {
  char* nick = arg(data, 0);
  char* add  = arg(NULL, 1);

  CHECK(nick);

  nap_pong_received(net, nick, add);
}

HANDLER(handle_ping) {
  char* nick = arg(data, 0);
  char* add  = arg(NULL, 1);
  
  CHECK(nick);

  nap_ping_received(net, nick, add);
}

HANDLER(handle_login_attempt) {
  (void)data;
  normal_message(NULL, OBJ(net)->name, NULL, 
                 _("Someone else is trying to login with your nick"));
}

HANDLER(handle_remote_queued) {
  char* nick     = auto_text(nap_net_to_utf8(net, arg(data, 0)));
  char* filename = arg(NULL, 0);
  char* temp     = arg(NULL, 0);
  char* queue    = arg(NULL, 0);
  source_t* source;
  unsigned num;
  nap_user_t* user;

  (void)temp;
  CHECK(filename);
  
  user = nap_user_find(nick, net);
  source = source_find(USER(user), filename);
  if (!source) return;
  
  num = queue?atoi(queue):0;

  if (!queue)
    nap_user_stat_cnt(user, NS_N_QUEUE);
  else if (num >= 10000)
    nap_user_stat_cnt(user, NS_R_10000);
  else if (num == 0)
    nap_user_stat_cnt(user, NS_R_0);
  
  if (num >= 10000) source->start(source);
  else source_set_remote(source, 60*5, num);
    
}

HANDLER(handle_download_ack) {
  char* nick    = auto_text(nap_net_to_utf8(net, arg(data, 0)));
  char* ip      = arg(NULL, 0);
  char* port    = arg(NULL, 0);
  char* winname = arg(NULL, 0);
  char* md5     = arg(NULL, 0);    // is ignored
  char* line    = arg(NULL, 0);
  source_t* source;
  nap_user_t* user;

  (void)md5;
  (void)line;

  CHECK(winname);
  user = nap_user_find(nick, net);
  source = source_find(USER(user), winname);
  if (!source) {
    g_log("NAP_DOWN", G_LOG_LEVEL_WARNING,
          "source '%s' from '%s' not found", nick, winname);
    return;
  }
  nap_source_ack(source, net, IP_FROM_LE(strtoul(ip, NULL, 10)),
                 ntohs(atoi(port)));
}

HANDLER(handle_firewalled_upload) {
  char* nick    = auto_text(nap_net_to_utf8(net, arg(data, 0)));
  char* ip      = arg(NULL, 0);
  char* port    = arg(NULL, 0);
  char* winname = arg(NULL, 0);
  char* md5     = arg(NULL, 0);    // is ignored
  char* speed   = arg(NULL, 0);
  nap_upload_t* upload;
  nap_user_t* user;

  (void)md5;

  CHECK(winname);

  user = nap_user_find(nick, net);
  nap_user_setup(user, 0, speed?atoi(speed):0);

  upload = NAP_UPLOAD(upload_find(USER(user), winname));
  if (!upload) return;

  // FIXME: must be in waiting state.
  nap_upload_outgoing(upload, IP_FROM_LE(strtoul(ip, NULL, 10)), htons(atoi(port)));
		      
}

HANDLER(handle_upload_error) {
  char* nick    = auto_text(nap_net_to_utf8(net, arg(data, 0)));
  char* winname = arg(NULL, 1);
  nap_user_t* user;
  upload_t* upload;

  CHECK(winname);

  user = nap_user_find(nick, net);
  upload = upload_find(USER(user), winname);
  if (!upload) return;

  socket_done(SOCK(upload), ST_UL_START_ERR, "peer canceled");
}

HANDLER(handle_download_error) {
  char* nick    = auto_text(nap_net_to_utf8(net, arg(data, 0)));
  char* winname = arg(NULL, 0);
  nap_user_t* user;
  source_t* source;

  CHECK(winname);

  user = nap_user_find(nick, net);
  source = source_find(USER(user), winname);
  if (!source) {
    g_log("NAP_DOWN", G_LOG_LEVEL_WARNING,
          "source '%s' from user '%s' not found", winname, nick);
    return;
  }

  socket_done(SOCK(source), ST_ERROR, "unavailable");
}

/* HANDLER(handle_invalid) { */
/*   if (!net->charset_warned) { */
/*     lopster_notify(LOPSTER_MESSAGE_WARNING, (data && *data)?data:_("Invalid encoding"), */
/*                    g_markup_printf_escaped */
/*                    (_("Could not convert message from network <b>%s</b>, " */
/*                       "make sure you have chosen the correct default character " */
/*                       "set encoding (napster preferences).\n\n" */
/*                       "You might also consider to adjust the encoding for " */
/*                       "this network only (edit dialog of the network)."), */
/*                     OBJ(net)->name) */
/*                    ); */
/*     net->charset_warned = TRUE; */
/*   } */
/* } */

static void print_stat_message(const char* tag, const char* message) {
  if (message) {
    chat_print(NULL, "+\0client",
               "+\1c_client", tag,
               "+\1c_text", message,
               "+\0nl", NULL);
  }
}

static const char* get_tag(int id) {
  switch (id) {
  case 0:  return _("Local Clients    :");
  case 1:  return _("Local Servers    :");
  case 2:  return _("Users            :");
  case 3:  return _("Files            :");
  case 4:  return _("Channels         :");
  case 5:  return _("Boot Time        :");
  case 6:  return _("Uptime           :");
  case 7:  return _("Memory           :");
  case 8:  return _("Registered Users :");
  case 9:  return _("KBytes In/sec    :");
  case 10: return _("KBytes Out/sec   :");
  case 11: return _("Searches/sec     :");
  case 12: return _("Traffic In       :");
  case 13: return _("Traffic Out      :");
  case 14: return _("Pending Searches :");
  default: return _("Unknown          :");
  }
}

HANDLER(handle_usage_stats) {
  char* pos;
  char* pos2;
  time_t stime;
  int t1;
  file_size_t bytes;
  const char* temp;
  char* temp2;

  (void)net;

  pos = arg(data, 0); CHECK(pos);
  print_stat_message(get_tag(0), pos);

  pos = arg(NULL, 0); CHECK(pos);
  print_stat_message(get_tag(1), pos);

  pos = arg(NULL, 0); CHECK(pos);
  print_stat_message(get_tag(2), pos);

  pos = arg(NULL, 0);
  pos2 = arg(NULL, 0); CHECK(pos2);
  bytes = strtoll(pos2, NULL, 10);
  temp = print_filesize(&bytes);
  print_stat_message(get_tag(3), l_printf("%s (%s)", pos, temp));

  pos = arg(NULL, 0); CHECK(pos);
  print_stat_message(get_tag(4), pos);

  pos = arg(NULL, 0);
  stime = strtol(pos, NULL, 10);
  temp2 = l_strdup(ctime(&stime));
  temp2[strlen(temp2) - 1] = 0;
  print_stat_message(get_tag(5), temp2);

  pos = arg(NULL, 0);
  t1 = strtoul(pos, NULL, 10);
  temp = print_duration_unit(&t1);
  print_stat_message(get_tag(6), temp);

  pos = arg(NULL, 0); if (!pos) return;
  print_stat_message(get_tag(7), pos);

  pos = arg(NULL, 0); if (!pos) return;
  print_stat_message(get_tag(8), pos);

  pos = arg(NULL, 0); if (!pos) return;
  print_stat_message(get_tag(9), pos);

  pos = arg(NULL, 0); if (!pos) return;
  print_stat_message(get_tag(10), pos);

  pos = arg(NULL, 0); if (!pos) return;
  print_stat_message(get_tag(11), pos);

  pos = arg(NULL, 0); if (!pos) return;
  bytes = strtoll(pos, NULL, 10);
  temp = print_filesize(&bytes);
  print_stat_message(get_tag(12), temp);

  pos = arg(NULL, 0); if (!pos) return;
  bytes = strtoll(pos, NULL, 10);
  temp = print_filesize(&bytes);
  print_stat_message(get_tag(13), temp);

  pos = arg(NULL, 0); if (!pos) return;
  print_stat_message(get_tag(14), pos);
}

#define NAP_REC(level) \
 g_log("NAP_PROTO", level, "R: %5u L: %4d [%s]", command->type, command->len, command->message)

const char* nap_handle_message(nap_net_t* net, nap_message_t* command)
{

    switch (command->type)
    {
/*   case NAP_R_INVALID: */
/*     NAP_REC(G_LOG_LEVEL_WARNING); */
/*     NAP_CALL(handle_invalid); */
/*     break; */
    case NAP_R_SERVER_MSG:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_server_message);
        break;
    case NAP_R_ERROR:
        if (!command->message) return "login error";
        else return auto_text(nap_net_to_utf8(net, command->message));
        break;
    case NAP_R_LOGIN_ACK:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_login_ack);
        break;
    case NAP_R_SERVER_STATS:
        //    NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_server_stats);
        break;
    case NAP_R_REMOTE_QUEUED:
        //    NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_remote_queued);
        break;
    case NAP_R_MOTD:
        //    NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_motd);
        break;
    case NAP_R_CHANNEL_ENTRY:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_channel_list_entry);
        break;
    case NAP_R_CHANNEL_SHORT:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_channel_list_short);
        break;
    case NAP_R_DOWNLOAD_ACK:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_download_ack);
        break;
    case NAP_R_CHANNEL_SHORT_END:
    case NAP_R_CHANNEL_END:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_channel_list_end);
        break;
    case NAP_R_JOIN_ACK:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_channel_join_ack);
        break;
    case NAP_R_CHANNEL_USER:
        //    NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_channel_user);
        break;
    case NAP_R_USER_END:
        //    NAP_REC(G_LOG_LEVEL_DEBUG);
        break;
    case NAP_R_TOPIC:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_topic);
        break;
    case NAP_R_USER_JOIN:
        //    NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_user_join);
        break;
    case NAP_R_USER_PART:
        //    NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_user_part);
        break;
    case NAP_R_PUBLIC_MSG:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_public_message);
        break;
    case NAP_R_EMOTE_MSG:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_emote_message);
        break;
    case NAP_R_WALLOP_MSG:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_wallop_message);
        break;
    case NAP_R_GLOBAL_MSG:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_global_message);
        break;
    case NAP_R_PRIVATE_MSG:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_private_message);
        break;
    case NAP_R_DOWNLOAD_ERROR:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_download_error);
        break;
    case NAP_R_PART_CHANNEL:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_part_channel);
        break;
    case NAP_R_SEARCH_RESULT:
        // NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_search_result);
        break;
    case NAP_R_SEARCH_END:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_search_end);
        break;
    case NAP_R_HOTLIST_ACK:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        // do nothing
        break;
    case NAP_R_USER_SIGNON:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_user_on);
        break;
    case NAP_R_USER_SIGNOFF:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_user_off);
        break;
    case NAP_R_WHOIS_RESPONSE:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_whois_response);
        break;
    case NAP_R_WHOWAS_RESPONSE:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_whowas_response);
        break;
    case NAP_R_BROWSE_DIRECT:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_browse_direct);
        break;
    case NAP_R_BROWSE_DIRECT_OK:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_browse_direct_ok);
        break;
    case NAP_R_BROWSE_DIRECT_ERR:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_browse_direct_err);
        break;
    case NAP_R_BROWSE_RESPONSE:
        //    NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_browse_response);
        break;
    case NAP_R_BROWSE_END:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_browse_end);
        break;
    case NAP_R_UPLOAD_REQUEST:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_upload_request);
        break;
    case NAP_R_UPLOAD_ERROR:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_upload_error);
        break;
    case NAP_R_HOTLIST_ERROR:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_hotlist_error);
        break;
    case NAP_R_PING:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_ping);
        break;
    case NAP_R_PONG:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_pong);
        break;
    case NAP_R_LOGIN_ATTEMPT:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_login_attempt);
        break;
    case NAP_R_UNSHARE_ALL_ACK:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        break;
    case NAP_R_UPLOAD_FIREWALL:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALL(handle_firewalled_upload);
        break;
    case NAP_R_USAGE_STATS:
        NAP_REC(G_LOG_LEVEL_DEBUG);
        NAP_CALLC(handle_usage_stats);
        break;
    default:
        NAP_REC(G_LOG_LEVEL_WARNING);
        NAP_CALL(handle_unknown);
        break;
    }
    return NULL;
}

