/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2006/07/22 17:14:03 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdlib.h>

#include "lcore/utils.h"
#include "nap_network.h"
#include "nap_chat.h"

gboolean nap_check_too_many_pending(nap_net_t* net, char *data) {
  if (!data) return FALSE;

  if (!strcmp(data, "search failed: too many pending searches")) {
    switch (net->type) {
    case NAP_TYPE_SLAVANAP:
      // we do too many searches per minute.
      if (net->max_searches_pm > 1) net->max_searches_pm--;
      net->skip_search_end++;
    case NAP_TYPE_OPENNAP:
    case NAP_TYPE_OPENNAP_NG:
      // we do too many concurrent searches
      if (net->max_searches > 1) net->max_searches--;
    default: /* UNKNOWN server type below*/
      if (net->max_searches > 1) {
        // we have assumed opennap style (default on unknown servers)
        net->max_searches--;
      } else if (net->max_searches_pm > 1) {
        // we had switched to slavanap style, now adjust the values
        net->max_searches_pm--;
        net->skip_search_end++;
      } else if (net->max_searches_pm == 0) {
        // we have opennap style, but it seems the wrong one,
        // switch to slavanap style now.
        net->max_searches_pm = 3;
        net->skip_search_end++;
      }
    }
    
    g_log("SEARCH", G_LOG_LEVEL_DEBUG,
          "too many pending on %s [%d][%d][%d]",
          OBJ(net)->name, net->cur_searches,
          net->max_searches, net->max_searches_pm);
    nap_net_search_finish_latest(net);
    return TRUE;
  }
  return FALSE;
}

gboolean nap_check_skip(nap_net_t* net, char* data) {
  (void)net;
  if (!strncmp(data, "Connection from ", 16)) return TRUE;
  return FALSE;
}

gboolean nap_check_level_set(nap_net_t* net, char* data) {
  char* pos;
  char* lev;
  char* temp;
  int level;
  
  if ((pos = strstr(data, " changed your user level to "))) {
    temp = l_strdup(pos+28);
  } else if ((pos = strstr(data, " set your user level to "))) {
    temp = l_strdup(pos+24);
  } else {
    temp = NULL;
  }
  if (!temp) return FALSE;
  
  lev = arg(temp, 0);
  if (!lev) return TRUE;

  level = NapLevelInt(lev);
  if (level == NAP_L_UNKNOWN) {
    // thats a unknown level (slavanap), check the int value
    temp = arg(NULL, 0);
    if (!temp) return TRUE;  // no int value found
    if (temp[0] != '(') return TRUE; // there should be a '('
    level = atoi(temp+1);
  }
  if (!NapIsLevel(level)) return TRUE;
  nap_net_set_level(net, level);
  return TRUE;
}

gboolean nap_check_channel_wallop(nap_net_t* net, char* data) {
  char* pos;
  char* nick;
  char* channel;

  pos = strstr(data, " [ops/");
  if (!pos) return FALSE;
  pos = l_strdup(data);

  nick = arg(pos, 0);
  g_assert(nick);

  channel = arg(NULL, 1);
  if (!channel) return FALSE;
  if (strncmp(channel, "[ops/", 5)) return FALSE;
  channel += 5;

  pos = strchr(channel, ']');
  if (!pos) return TRUE;
  *pos++ = 0;
  if (pos[0] != ':') return TRUE;
  if (pos[1] != ' ') return TRUE;
  pos += 2;

  nap_chop_message(net, channel, nick, pos);

  return TRUE;
}

gboolean nap_check_channel_mode2(nap_net_t* net, char* data) {
  char* nick;
  char* pos;
  char* temp;
  char* channel;
  char* mode;
  int chmode1 = 0;
  int chmode2 = 0;
  nap_pub_t* page;

  pos = strchr(data, ' ');
  if (!pos) return FALSE;
  if (strncmp(pos, " changed mode on channel ", 25)) return FALSE;

  *pos = 0;
  nick = l_strdup(data);
  *pos = ' ';
  temp = l_strdup(pos+25);

  channel = arg(temp, 0);
  if (!channel) return TRUE;
  channel[strlen(channel)-1] = 0;
  data = arg(NULL, 1);

  page = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
  if (!page) return TRUE;

  while ((mode = arg(data, 0)) != NULL) {
    if (*mode == '+')
      chmode1 |= NapChannelModeInt(mode);
    else if (*mode == '-')
      chmode2 |= NapChannelModeInt(mode);
    data = NULL;
  }

  if (chmode1 & NAP_CMODE_TOPIC_REST) {
    chmode1 &= ~NAP_CMODE_TOPIC_REST;
    chmode2 |= NAP_CMODE_TOPIC_REST;
  } else if (chmode2 & NAP_CMODE_TOPIC_REST) {
    chmode2 &= ~NAP_CMODE_TOPIC_REST;
    chmode1 |= NAP_CMODE_TOPIC_REST;
  }

  nap_channel_mode_changed(page, nick, chmode1, chmode2);
  
  return TRUE;
}

gboolean nap_check_channel_mode(nap_net_t* net, char* data) {
  char* channel;
  char* mode;
  char* temp;
  int chmode = 0;
  nap_pub_t* page;

  if (strncmp("mode for channel ", data, 17)) return FALSE;

  temp = l_strdup(data+17);
  channel = arg(temp, 0);
  data = arg(NULL, 1);
  if (!channel) return FALSE;

  page = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
  if (!page) return TRUE;
  
  while ((mode = arg(data, 0)) != NULL) {
    chmode |= NapChannelModeInt(mode);
    data = NULL;
  }

  chmode ^= NAP_CMODE_TOPIC_REST;   // toggle topic flag

  nap_channel_mode_changed(page, NULL, chmode, NAP_CMODE_MASK^chmode);

  return TRUE;
}

gboolean nap_check_channel_limit_level(nap_net_t* net, char* data) {
  char* channel;
  char* temp;
  char* rest;
  nap_pub_t* page;

  if (strncmp("Channel ", data, 8)) return FALSE;
  temp = l_strdup(data+8);
  channel = arg(temp, 0);
  if (!channel) return FALSE;

  rest = arg(NULL, 1);
  if (!rest) return FALSE;

  if (!strncmp("has limit ", rest, 10)) {
    page = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
    if (page)
      nap_channel_limit_changed(page, NULL, atoi(rest+10));
    return TRUE;
  } else if (!strncmp("is level ", rest, 9)) {
    page = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
    if (page)
      nap_channel_level_changed(page, NULL, NapLevelInt(rest+9));
    return TRUE;
  } else {
    return FALSE;
  }
  g_assert(FALSE);
}

gboolean nap_check_channel_limit(nap_net_t* net, char* data) {
  char* nick;
  char* pos;
  char* temp;
  char* channel;
  nap_pub_t* page;

  pos = strchr(data, ' ');
  if (!pos) return FALSE;
  if (strncmp(pos, " set limit on channel ", 22)) return FALSE;

  *pos = 0;
  nick = l_strdup(data);
  *pos = ' ';
  temp = l_strdup(pos+22);

  channel = arg(temp, 0);
  if (!channel) return FALSE;
  data = arg(NULL, 0);
  if (!data || strcmp(data, "to")) return FALSE;
  data = arg(NULL, 0);
  if (!data) return FALSE;
  
  page = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
  if (!page) return TRUE;
  
  nap_channel_limit_changed(page, nick, atoi(data));
  
  return TRUE;
}

gboolean nap_check_channel_level(nap_net_t* net, char* data) {
  char* nick;
  char* pos;
  char* temp;
  char* channel;
  nap_pub_t* page;

  pos = strchr(data, ' ');
  if (!pos) return FALSE;
  if (strncmp(pos, " set channel ", 13)) return FALSE;

  *pos = 0;
  nick = l_strdup(data);
  *pos = ' ';
  temp = l_strdup(pos+13);

  channel = arg(temp, 0);
  if (!channel) return FALSE;
  data = arg(NULL, 0);
  if (!data || strcmp(data, "to")) return FALSE;
  data = arg(NULL, 0);
  if (!data || strcmp(data, "level")) return FALSE;
  data = arg(NULL, 0);
  if (!data) return FALSE;
  
  page = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
  if (!page) return TRUE;

  nap_channel_level_changed(page, nick, NapLevelInt(data));
  
  return TRUE;
}

gboolean nap_check_flag(nap_net_t* net, char *data) {
  char* data2;
  char* who;
  char* pos1;
  char* nick = NULL;
  nap_pub_t* page;
  int flag = -1;
  int set = 0;

  if (!data) return FALSE;
  data2 = l_strdup(data);
  who = arg(data2, 0);
  if (!who) return FALSE;

  if (!strcmp("Server", who)) who = arg(NULL, 0);

  pos1 = arg(NULL, 1);
  if (!pos1) return FALSE;

  if (!strncmp(pos1, "opped ", 6)) {
    nick = arg(pos1+6, 0);
    flag = NAP_CU_OP;
    set = 1;
  } else if (!strncmp(pos1, "deopped ", 8)) {
    nick = arg(pos1+8, 0);
    flag = NAP_CU_OP;
    set = 0;
  } else if (!strncmp(pos1, "voiced ", 7)) {
    nick = arg(pos1+7, 0);
    flag = NAP_CU_VOICE;
    set = 1;
  } else if (!strncmp(pos1, "devoiced ", 9)) {
    nick = arg(pos1+9, 0);
    flag = NAP_CU_VOICE;
    set = 0;
  } else if (!strncmp(pos1, "muzzled ", 8)) {
    nick = arg(pos1+7, 0);
    flag = NAP_CU_MUZZLE;
    set = 1;
  } else if (!strncmp(pos1, "unmuzzled ", 10)) {
    nick = arg(pos1+9, 0);
    flag = NAP_CU_MUZZLE;
    set = 0;
  }
  if (flag != -1) {
    pos1 = arg(NULL, 1);
    if (!pos1) return FALSE;

    // nick cant be NULL here, unless pos1 is NULL
    if (!strncmp(pos1, "on channel ", 11)) {
      char* channel;
      char* reason;
      channel = arg(pos1+11, 0);
      if (!channel) return FALSE;
      if (channel[strlen(channel)-1] == ':') {
        channel[strlen(channel)-1] = 0;
        reason = arg(NULL, 1);
      } else {
        reason = NULL;
      }

      page = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
      if (page)
        nap_channel_user_flagged(page, who, nick, set, flag, reason);
      return TRUE;
    }
    return FALSE;
  }

  // ok, we had none of the obove flags, now check for BAN
  // which has a different format at the end.
  flag = -1;
  if (!strncmp(pos1, "banned ", 7)) {
    nick = arg(pos1+7, 0);
    flag = NAP_CU_BAN;
    set = 1;
  } else if (!strncmp(pos1, "unbanned ", 9)) {
    nick = arg(pos1+9, 0);
    flag = NAP_CU_BAN;
    set = 0;
  }

  if (flag != -1) {
    pos1 = arg(NULL, 1);
    if (!pos1) return FALSE;
    // nick cant be NULL here, unless pos1 is NULL
    if (!strncmp(pos1, "from ", 5)) {
      char* channel;
      char* reason;
      channel = arg(pos1+5, 0);
      if (!channel) return 0;
      channel[strlen(channel)-1] = 0;
      reason = arg(NULL, 1);

      page = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
      if (page)
        nap_channel_user_flagged(page, who, nick, set, flag, reason);
      return TRUE;
    }
    return FALSE;
  }


  if ((who = strstr(pos1, " as operator on channel ")) != NULL) {
    nick = arg(pos1, 0);
    nick = arg(NULL, 0);
    nick = arg(NULL, 0);
    page = NPP(nap_cp_search_net(net, who+24, NAP_PAGE_PUBLIC));
    if (page)
      nap_channel_user_flagged(page, NULL, "you", 1, NAP_CU_OP, NULL);
    return TRUE;
  }
  return FALSE;
}

gboolean nap_check_channel_topic(nap_net_t* net, char* data) {
  char* who;
  char* pos;
  char* temp;
  char* channel;
  char* topic;
  nap_pub_t* npp;
  
  pos = strstr(data, " set topic on ");
  if (!pos) return FALSE;

  temp = l_strdup(data);
  pos = strstr(temp, " set topic on ");
  *pos = 0;

  who = temp;
  pos += 14;

  channel = arg(pos, 0);
  if (!channel) return FALSE;
  channel[strlen(channel)-1] = 0;
  topic = arg(NULL, 1);

  npp = NPP(nap_cp_search_net(net, channel, NAP_PAGE_PUBLIC));
  if (npp) nap_channel_new_topic(npp, topic, who);

  return TRUE;
}

