/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.7 $
 * $Date: 2007/05/23 20:42:29 $
 * $Author: sid4u $
 */

#include <string.h>
#include <stdio.h>

#include <glib.h>

#include "lcore/l_base.h"
#include "nap_whois.h"
#include "nap_proto.h"
#include "nap_ui.h"

#define NAP_WHOIS_RECENT    60
#define NAP_WHOIS_LIFE   60*60

#ifdef WIN32
#include "napalias.h"
#endif

typedef struct {
  char* client;
  unsigned cnt[NS_NUM];
} nap_user_stat_t;

static GHashTable* UserStat = NULL;


static void nap_whois_notify(nap_net_t* net, nap_whois_t* whois)
{
    if (whois->net == net) whois->net = NULL;
}

static void nap_whois_set_net(nap_whois_t* whois, nap_net_t* net)
{
    if (whois->net == net) return;

    if (whois->net)
        object_unnotify_d
            (OBJ(whois->net), (ObjectFunc)nap_whois_notify, whois);

    whois->net = net;

    if (whois->net)
        object_notify_d
            (OBJ(whois->net), (ObjectFunc)nap_whois_notify, whois);
}

static void nap_whois_init(nap_whois_t* whois, nap_net_t* net, int mode)
{
    whois->net = NULL;
    whois->mode = mode;
    whois->level = NAP_L_USER;
    whois->status = NAP_WHOIS_UNKNOWN;
    whois->dl = whois->ul = whois->tdl = whois->tul = -1;
    nap_whois_set_net(whois, net);
}

static nap_whois_t* nap_whois_new(nap_net_t* net, int mode)
{
    nap_whois_t* whois = g_malloc0(sizeof(*whois));
    nap_whois_init(whois, net, mode);
    return whois;
}

static void nap_whois_action(nap_user_t* user)
{
    nap_whois_t* whois = user->whois;
    
    if ((whois->mode & NAP_WHOIS_WINDOW) ||
        (whois->mode & NAP_WHOIS_TEXT))
    {
        NAPUI_whois_show(user);
    }
    nap_user_stat_cnt(user, NS_WHOIS);
}

static gboolean nap_whois_timer(nap_user_t* user)
{
    nap_whois_t* whois = user->whois;
    
    whois->status = NAP_WHOIS_TIMEOUT;
    nap_whois_received(user);
    return FALSE;
}

static void nap_whois_do(nap_user_t* user)
{
    nap_whois_t* whois = user->whois;
    
    g_assert(whois);
    g_assert(!whois->timer);
    
    if (!whois->net)
    {
        whois->status = NAP_WHOIS_UNKNOWN;
    }
    else
    {
        nap_net_send(whois->net, NAP_S_WHOIS, TRUE, "%s", OBJ(user)->name);
        whois->timer =
            lopster_timeout_add(NAP_WHOIS_RECENT*1000,
                          (GSourceFunc)nap_whois_timer, user);
    }
}

nap_whois_t* nap_whois_user(nap_user_t* user, nap_net_t* net,
                            int mode, gboolean force)
{
    nap_whois_t* whois = user->whois;
    
    if (!net) return NULL;
    
    if (!user->whois)
    {
        user->whois = whois = nap_whois_new(net, mode);
        //FIXME: we currently never unref users that were whois'ed
        user_ref(USER(user));
    }
    else if (!whois->timer)
    {
        whois->mode = mode;
    }
    else
    {
        whois->mode |= mode;
    }

    if (whois->net != net)
    {
        nap_whois_set_net(whois, net);
        //    force = TRUE;  // always whois when net has changed
        if (whois->timer)
        {
            lopster_timeout_remove(whois->timer);
            whois->timer = 0;
        }
    }
    
    if (!whois->timer &&
        (force || whois->refreshed+NAP_WHOIS_LIFE < CurrentTime))
    {
        nap_whois_do(user);
    }
    NAPUI_whois_show(user);
    
    return whois;
}

void nap_whois_received(nap_user_t* user)
{
    nap_whois_t* whois = user->whois;
    
    whois->refreshed = CurrentTime;
    if (whois->timer)
    {
        lopster_timeout_remove(whois->timer);
        whois->timer = 0;
    }
    
    nap_whois_action(user);
}

static void nap_user_stat_destroy(nap_user_stat_t* nus)
{
    g_free(nus->client);
    g_free(nus);
}

static nap_user_stat_t* nap_user_stat_find(const char* client)
{
    nap_user_stat_t* nus;
    
    if (!UserStat)
    {
        UserStat =
            g_hash_table_new_full
            ((GHashFunc)g_str_hash, (GEqualFunc)g_str_equal,
             (GDestroyNotify)NULL, (GDestroyNotify)nap_user_stat_destroy);
    }    
    if (!client) return NULL;
    nus = g_hash_table_lookup(UserStat, client);
    if (!nus)
    {
        nus = g_malloc0(sizeof(*nus));
        nus->client = g_strdup(client);
        g_hash_table_insert(UserStat, nus->client, nus);
    }
    return nus;
}

void nap_user_stat_cnt(nap_user_t* user, unsigned sec)
{
    nap_user_stat_t* nus;
    
    if (sec >= NS_NUM) return;
    //  if (whois->status != NAP_WHOIS_ONLINE) return;
    if (!user->client) return;
    nus = nap_user_stat_find(user->client);
    nus->cnt[sec]++;
}

static void nap_stat_print(const char* client, nap_user_stat_t* nus)
{
    g_log("NAP", G_LOG_LEVEL_DEBUG,
          "%21s : %4u %4u %4u %4u %4u %4u", client, 
          nus->cnt[NS_WHOIS],
          nus->cnt[NS_DOWN],
          nus->cnt[NS_R_0],
          nus->cnt[NS_R_10000],
          nus->cnt[NS_N_QUEUE],
          nus->cnt[NS_UP]
        );
}

void nap_user_stat_print()
{
    g_log("NAP", G_LOG_LEVEL_DEBUG,
          "----------------Client- -who down --0- -10k -noq -up- -");
    if (UserStat)
        g_hash_table_foreach(UserStat, (GHFunc)nap_stat_print, NULL);
    g_log("NAP", G_LOG_LEVEL_DEBUG,
          "-------------------------------------------------------");
}

#ifdef WIN32
#define _NAP_WHOIS_C_
#include "napaliasdef.c"
#endif
