#include <QtGui>

#include "lcore/utils.h"
#include "lcore/buffer.h"
#include "lcore/preferences.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "lcore/xml_handler.h"
#include "nap_network.h"
#include "nap_napi.h"
#include "nap_chat.h"
#include "nap.h"

#include "lcore/qt4_ui/ui_main.hpp"
#include "lcore/qt4_ui/ui_utils.hpp"
#include "lcore/qt4_ui/ui_network.hpp"
#include "lcore/qt4_ui/ui_style.hpp"
#include "NetworkDataAccessor.hpp"
#include "ui_nap_chat.hpp"
#include "NapNetworkTreeWidget.hpp"


// TODO : handle status pixmap for server 
/*
#include "pixmaps/server_perm.xpm"
#include "pixmaps/server_deact.xpm"
#include "pixmaps/server_napi.xpm"
#include "pixmaps/server_nnapi.xpm"
*/

NapTreeItem::NapTreeItem(NapNetworkTreeWidget *tree, void *object, NetworkObjectType type) 
                        :QTreeWidgetItem(tree)
{ 
    m_type=type; 
    m_object=object;
}

NapTreeItem::NapTreeItem(NapTreeItem *item, void *object, NetworkObjectType type) 
                        :QTreeWidgetItem(item)
{ 
    m_type=type; 
    m_object=object;
}

NapTreeItem::~NapTreeItem()
{
    QString name;
    if (m_type == GROUP_OBJECT)
    {
       object_free_data(OBJ((nap_ng_t*)m_object), "NapGroupEntry", NULL);
    }
    else if (m_type == NETWORK_OBJECT)
    {
       object_free_data(OBJ((nap_net_t*)m_object), "NapNetEntry", NULL);
    }
    else if (m_type == SERVER_OBJECT)
    {
       object_free_data(OBJ((nap_server_t*)m_object), "NapServerEntry", NULL);   
    }
}

NetworkObjectType NapTreeItem::object_get_type( ) { 
    return m_type;
}

void* NapTreeItem::object_get_object( ) { 
    return m_object;
}

void NapTreeItem::set_online(bool value)
{
    if (!value)
    {
        return;
    }
    // FIXME : this is for testing if the style could be changed here and it works
    // now need to change the way it should behave on connect , disconnect
   /* if (StyleCore::GetStyle()->ui_style_sets("network:entry:online", qtr("Text"))) {   
        for (int i=0; i<columnCount () ; i++ )
        {
            setForeground(i, 
                    QBrush(
                        StyleCore::GetStyle()->ui_style_get_color("network:entry:online", qtr("Text"))
                          )
                         );
        }
    }*/
}


ItemAction::ItemAction(const QString &text, QObject *parent)
            : QAction(text,parent)
{
    this->m_callback = NULL;
    this->m_object = NULL;
    this->m_extra = NULL;
}

void ItemAction::item_action_set_list_callback_with_extra(item_callback_extra func, 
                            QList<QTreeWidgetItem* > selection, void* extra, bool check)
{
    this->m_extra = extra;
    this->m_callback_extra = func;
    item_action_set_list_callback(NULL,selection,check); 
}

void ItemAction::item_action_set_list_callback(item_callback func, 
                            QList<QTreeWidgetItem* > selection, bool check)
{
    this->m_callback = func;
    this->m_selection = selection;
    if (check)
    {
        connect(this,SIGNAL(toggled(bool)),this,SLOT(item_list_action_on_toggled(bool)));
    }
    else {
        connect(this,SIGNAL(triggered()),this,SLOT(item_list_action_on_triggered()));
    }
    
}

void ItemAction::item_action_set_callback(item_callback func, void* data, bool check)
{
    this->m_callback = func;
    this->m_object = data;
    if (check)
    {
        connect(this,SIGNAL(toggled(bool)),this,SLOT(item_action_on_toggled(bool)));
    }
    else {
        connect(this,SIGNAL(triggered()),this,SLOT(item_action_on_triggered()));
    }
}

void ItemAction::item_action_set_callback_with_extra(item_callback_extra func, void* data, void* extra, bool check)
{
    this->m_extra = extra;
    this->m_callback_extra = func;
    item_action_set_callback(NULL,data,check);
}

void ItemAction::item_list_action_on_triggered()
{
    if (!m_selection.isEmpty())
    {
        for (int i=0;i < m_selection.size();i++)
        {
            NapTreeItem *item = static_cast<NapTreeItem *> (m_selection.at(i));
            if (this->m_extra) {
                m_callback_extra(item, this->m_extra);
            }
            else if (!this->m_callback)
            {
                m_callback_extra(item, NULL);
            }
            else {
                m_callback(item);       
            }
        }
    }
}

// For now it's a copy of the other function but may be modified or merged if no changed needed
void ItemAction::item_list_action_on_toggled(bool checked)
{
    if (!m_selection.isEmpty())
    {
        for (int i=0;i < m_selection.size();i++)
        {
            NapTreeItem *item = static_cast<NapTreeItem *> (m_selection.at(i));
            if (this->m_extra) {
                m_callback_extra(item, this->m_extra);
            }
            else if (!this->m_callback)
            {
                m_callback_extra(item, NULL);
            }
            else {
                m_callback(item);
            }
        }
    }
}

void ItemAction::item_action_on_triggered()
{
  /*  if (m_object)
    {*/
        if (this->m_extra) {
            m_callback_extra(this->m_object, this->m_extra);
        }
        else if (!this->m_callback)
        {
            m_callback_extra(this->m_object, NULL);
        }
        else {
            m_callback(this->m_object);
        }
 /*   }*/
}
// For now it's a copy of the other function but may be modified or merged if no changed needed
void ItemAction::item_action_on_toggled(bool checked)
{
   /* if (m_object)
    {*/
        if (this->m_extra) {
            m_callback_extra(this->m_object, this->m_extra);
        }
        else if (!this->m_callback)
        {
            m_callback_extra(this->m_object, NULL);
        }
        else {
            m_callback(m_object);
        }
/*    }*/
}

NapNetworkTreeWidget::NapNetworkTreeWidget(QWidget *parent )
    : QTreeWidget(parent)
{
    /*
	connect(this, SIGNAL(itemClicked(QTreeWidgetItem *,  int )),
            this, SLOT(ui_nap_item_selected(QTreeWidgetItem*)));
    connect(this, SIGNAL(itemPressed(QTreeWidgetItem *,  int )),
            this, SLOT(ui_nap_item_selected(QTreeWidgetItem*)));
	connect(this, SIGNAL(itemActivated(QTreeWidgetItem *,  int )),
            this, SLOT(ui_nap_item_selected(QTreeWidgetItem*)));
    connect(this, SIGNAL(itemSelectionChanged( )),
            this, SLOT(objectSelectionChanged()));*/

    setColumnCount(6);
    QStringList labels;
    labels << qtr("Name") << qtr("Status") << qtr("Failures") << qtr("Users") << qtr("Files") << qtr("GiB");

    setSelectionMode(QAbstractItemView::ExtendedSelection);
    setHeaderLabels(labels);


    //FIXME : Change icon style to some better ones ;)
    // folderIcon.addPixmap(style()->standardPixmap(QStyle::SP_DirClosedIcon),
    //                      QIcon::Normal, QIcon::Off);
    // folderIcon.addPixmap(style()->standardPixmap(QStyle::SP_DirOpenIcon),
    //                      QIcon::Normal, QIcon::On);
    // bookmarkIcon.addPixmap(style()->standardPixmap(QStyle::SP_FileIcon));

}

void NapNetworkTreeWidget::on_create_edit_group(nap_ng_t* ng)
{
    GroupAccessorDialog *dialog = new GroupAccessorDialog(ng, GetNetworkTree( ) );
    dialog->exec();
}

void NapNetworkTreeWidget::ui_nap_create_top_actions( QMenu *menu )
{
    ItemAction *item = new ItemAction(qtr("Create group"), menu);
    item->item_action_set_callback((item_callback)on_create_edit_group,NULL);
    menu->addAction(item);
    menu->addSeparator();
}

void NapNetworkTreeWidget::on_keep(nap_ng_t* ng, int *hm) {
    int data = *hm;
    ng->keep = data;
    object_set_dirty(OBJ(ng));
    nap_network_save();
}

void NapNetworkTreeWidget::on_create_net(nap_ng_t* ng) {
    on_create_edit_net(NULL, ng);
}

void NapNetworkTreeWidget::on_create_edit_net(nap_net_t* net, nap_ng_t* ng) {
    NetworkAccessorDialog *dialog = new NetworkAccessorDialog(net, ng );
    dialog->exec();
}

void NapNetworkTreeWidget::on_ng_toggle_auto(nap_ng_t* ng) {
    ng->flags ^= NAP_NG_AUTO_CONNECT;
    object_set_dirty(OBJ(ng));
    nap_network_save();
}

QMenu *NapNetworkTreeWidget::ui_nap_create_group_actions(nap_ng_t* ng)
{
#define SUB_LEN  20
    ItemAction *item;
    QMenu   *popup;
    QMenu   *popup2;
    QMenu   *popup3;
    int i1;
    ItemAction *items[SUB_LEN+1];
    int no_nets, inc;
    QString str;
  
    popup = new QMenu(this);
    ui_nap_create_top_actions(popup);

    no_nets = g_list_length(ng->nets);
    inc = (no_nets+SUB_LEN-1)/SUB_LEN;
  
    item = new ItemAction(qtr("Group action"), popup);
    popup->addAction(item);
    {
        popup2 = new QMenu(popup);
        item->setMenu(popup2);
        if (ng->nets) {
            item = new ItemAction(qtr("Start connect"), popup2);
            item->item_action_set_callback((item_callback)nap_ng_start_connect,ng);
            popup2->addAction(item);
            item = new ItemAction(qtr("Stop connect"), popup2);
            item->item_action_set_callback((item_callback)nap_ng_stop_connect,ng);
            popup2->addAction(item);
        } else {
            item =  new ItemAction(qtr("Remove Group"), popup2);
            item->item_action_set_callback((item_callback) object_kill,ng);
            popup2->addAction(item);
        }

        popup2->addSeparator();
    
        item =  new ItemAction(qtr("Edit Group"), popup2);
        item->item_action_set_callback((item_callback)on_create_edit_group,ng);
        popup2->addAction(item);

        item =  new ItemAction(qtr("Add Network"), popup2);
        item->item_action_set_callback((item_callback)on_create_net,ng);
        popup2->addAction(item);
    }

    item = new ItemAction(qtr("Quick configure"), popup);
    popup->addAction(item);
    {
        popup2 = new QMenu(popup);
        item->setMenu(popup2);

        item = new ItemAction(qtr("Stay connected to"), popup2);
        popup2->addAction(item);

        {
            popup3 = new QMenu(popup2);
            item->setMenu(popup3);
     
            if (ng->keep == -1) str = qtr("All networks");
            else str = qtr(ngettext("%1 network", "%1 networks", ng->keep)).arg(ng->keep);
            
            item = new ItemAction(str, popup3);
            item->setEnabled(false);
            popup3->addAction(item);
            popup3->addSeparator();
      
            items[0] = new ItemAction(qtr("All networks"), popup3);
            int v = -1;
            items[0]->item_action_set_callback_with_extra((item_callback_extra)on_keep,ng,&v);
            popup3->addAction(items[0]);
      
            for (i1 = 0; i1*inc < no_nets; i1++) {
                str = qtr(ngettext("%1 network", "%1 networks", i1*inc)).arg(i1*inc);
                items[i1+1] = new ItemAction(str, popup3);
                int w = i1*inc;
                items[i1+1]->item_action_set_callback_with_extra((item_callback_extra)on_keep,ng,&w);
                popup3->addAction(items[i1+1]);
            }    
        }

        popup2->addSeparator();

        item =  new ItemAction(qtr("Include in Autoconnect"), popup2);
        item->setCheckable(true);
        if (ng->flags & NAP_NG_AUTO_CONNECT) {
            item->setChecked(true);
        }
        item->item_action_set_callback((item_callback)on_ng_toggle_auto,ng, true);
        popup2->addAction(item);
    }

    return popup;
}

void NapNetworkTreeWidget::on_net_toggle_pers(NapTreeItem *item)
{
    nap_net_t* net = (nap_net_t*) item->object_get_object();
//     net->flags ^= NAP_NET_PERSISTENT;
    object_set_dirty(OBJ(net));
}

void NapNetworkTreeWidget::on_net_toggle_winmx(NapTreeItem *item)
{
    nap_net_t* net = (nap_net_t*) item->object_get_object();
//     net->flags ^= NAP_NET_NO_WINMX;
    object_set_dirty(OBJ(net));
}

void NapNetworkTreeWidget::on_net_toggle_server(NapTreeItem *item)
{
    nap_net_t* net = (nap_net_t*) item->object_get_object();
    net->flags ^= NAP_NET_OWN_SERVER;
    object_set_dirty(OBJ(net));
}

void NapNetworkTreeWidget::on_net_toggle_global(NapTreeItem *item)
{
    nap_net_t* net = (nap_net_t*) item->object_get_object();
    net->flags ^= NAP_NET_IGN_GLOBAL;
    object_set_dirty(OBJ(net));
}

void NapNetworkTreeWidget::on_net_move(NapTreeItem *item, nap_ng_t* ng) {
    nap_net_t* net = (nap_net_t*) item->object_get_object();
    if (!net || net->group == ng) return;
    delete item;
    if (!ng) {
        object_kill(OBJ(net));
    } else {
        nap_net_move(net, ng);
    }
}

void NapNetworkTreeWidget::on_net_stop_connect(NapTreeItem *item) {
    nap_net_t* net = (nap_net_t*) item->object_get_object();
    if (!net ) return;
    nap_net_start_connect(net);
}

void NapNetworkTreeWidget::on_net_start_connect(NapTreeItem *item) {
    nap_net_t* net = (nap_net_t*) item->object_get_object();
    if (!net ) return;
    nap_net_stop_connect(net);
}

void NapNetworkTreeWidget::on_net_disconnect(NapTreeItem *item) {
    nap_net_t* net = (nap_net_t*) item->object_get_object();
    if (!net ) return;
    nap_net_stop_connect(net);
    nap_net_disconnect(net);
}

void NapNetworkTreeWidget::on_edit_net(nap_net_t* net) {
    on_create_edit_net(net, NULL);
}

void NapNetworkTreeWidget::on_show_channels(nap_net_t* net)
{
    if (!net) return;
    nap_net_show_channels(net);
}

QMenu *NapNetworkTreeWidget::ui_nap_create_net_actions(nap_net_t* net)
{
    ItemAction *item;
    QMenu *popup;
    QMenu *popup2;
    GList* dlist;
    nap_ng_t* ng;
    QList<QTreeWidgetItem* > select;

    popup = new QMenu(this);
    ui_nap_create_top_actions(popup);

    item = new ItemAction(qtr("Network action"), popup);
    popup->addAction(item);

    select = selectedItems();
    {
        popup2 = new QMenu(popup);
        item->setMenu(popup2);

        if (net->servers) {
            if (net->active)
            {
                item = new ItemAction(qtr("Stop connect"),popup2);
                item->item_action_set_list_callback((item_callback)on_net_disconnect,select);
                popup2->addAction(item);
            }
            else
            {
                item = new ItemAction(qtr("Start connect"),popup2);
                item->item_action_set_list_callback((item_callback)on_net_start_connect,select);
                popup2->addAction(item);
            }
            popup2->addSeparator();
        }

        item = new ItemAction(qtr("Edit Network"),popup2);
        item->item_action_set_callback((item_callback)on_edit_net,net);
        popup2->addAction(item);

        item = new ItemAction(qtr("Add Server"),popup2);
        item->item_action_set_callback((item_callback)on_create_server,net);
        popup2->addAction(item);

        if (net->active && net->active->status == NAP_SERVER_ONLINE) {
            popup2->addSeparator();

            item = new ItemAction(qtr("Show Channel List"),popup2);
            item->item_action_set_callback((item_callback)on_show_channels,net);
            popup2->addAction(item);
        }
    }

    popup->addSeparator();

    item = new ItemAction(qtr("Quick configure"), popup);
    popup->addAction(item);
    {
        popup2 = new QMenu(popup);
        item->setMenu(popup2);
        /*---------------*/
//         item =  new ItemAction(qtr("Has persistent servers"), popup2);
//         item->setCheckable(true);
//         if (net->flags & NAP_NET_PERSISTENT) {
//             item->setChecked(true);
//         }
//         item->item_action_set_list_callback((item_callback)on_net_toggle_pers,select, true);
//         popup2->addAction(item);
        /*---------------*/
//         item =  new ItemAction(qtr("Don't emulate WinMX v2.6"), popup2);
//         item->setCheckable(true);
//         if (net->flags & NAP_NET_NO_WINMX) {
//             item->setChecked(true);
//         }
//         item->item_action_set_list_callback((item_callback)on_net_toggle_winmx,select, true);
//         popup2->addAction(item);
        /*---------------*/        
        item =  new ItemAction(qtr("Own server page"), popup2);
        item->setCheckable(true);
        if (net->flags & NAP_NET_OWN_SERVER) {
            item->setChecked(true);
        }
        item->item_action_set_list_callback((item_callback)on_net_toggle_server,select, true);
        popup2->addAction(item);
        /*---------------*/ 
        item =  new ItemAction(qtr("Ignore global messages"), popup2);
        item->setCheckable(true);
        if (net->flags & NAP_NET_IGN_GLOBAL) {
            item->setChecked(true);
        }
        item->item_action_set_list_callback((item_callback)on_net_toggle_global,select, true);
        popup2->addAction(item);
        /*---------------*/ 
    }

    item = new ItemAction(qtr("Move Network"), popup);
    popup->addAction(item);
    {
        popup2 = new QMenu(popup);
        item->setMenu(popup2);
     
        item = new ItemAction(qtr("<Delete Network>"), popup2);
        item->item_action_set_list_callback_with_extra((item_callback_extra)on_net_move,select,NULL);
        popup2->addAction(item);

        for (dlist = nap_get_groups(); dlist; dlist = dlist->next) {
            ng = (nap_ng_t*) dlist->data;
            item = new ItemAction(qtr("%1").arg(OBJ(ng)->name), popup2);
            item->item_action_set_list_callback_with_extra((item_callback_extra)on_net_move,select,ng);
            popup2->addAction(item);
            if (net->group == ng) item->setEnabled(false);
        }
    }
  
    if (!net->active || net->active->status != NAP_SERVER_ONLINE)
        return popup;

    {
        nap_server_t* server = net->active;

        item = new ItemAction(qtr("Active Server"), popup);
        popup->addAction(item);

        popup2 = ui_nap_create_server_actions(server);
        item->setMenu(popup2);
    }

    return popup;
}

void NapNetworkTreeWidget::on_server_disconnect(nap_server_t* server) {
    nap_net_disconnect(server->net);
}

void NapNetworkTreeWidget::on_server_connect(nap_server_t* server) {
    nap_server_connect(server);
}

void NapNetworkTreeWidget::on_create_edit_server(nap_server_t* server, nap_net_t* net) {
    ServerAccessorDialog *dialog = new ServerAccessorDialog(server, net );
    dialog->exec();
}

void NapNetworkTreeWidget::on_create_server(nap_net_t* net) {
    on_create_edit_server(NULL, net);
}

void NapNetworkTreeWidget::on_edit_server(nap_server_t* server) {
    on_create_edit_server(server, NULL);
}

void NapNetworkTreeWidget::on_server_toggle_ignore(nap_server_t* server) {
    if (server) {
//         server->flags ^= NAP_SERVER_IGNORE;
        object_set_dirty(OBJ(server));
        nap_network_save();
    }
}


QMenu *NapNetworkTreeWidget::ui_nap_create_server_actions(nap_server_t* server)
{
    ItemAction *item;
    int server_status;
    nap_net_t* net;
    QMenu *popup;
    QMenu *popup2;

    popup = new QMenu(this);
    ui_nap_create_top_actions(popup);

    net = server->net;
    server_status = server->status;
  
    item = new ItemAction(qtr("Server action"), popup);
    popup->addAction(item);
    {
        popup2 = new QMenu(popup);
        item->setMenu(popup2);
  
        if (server_status == NAP_SERVER_CONNECTING ||
                    server_status == NAP_SERVER_ONLINE) {
            item = new ItemAction(qtr("Disconnect"),popup2);
            item->item_action_set_callback((item_callback)on_server_disconnect,server);
            popup2->addAction(item);
        } else {
            item = new ItemAction(qtr("Connect"),popup2);
            item->item_action_set_callback((item_callback)on_server_connect,server);
            popup2->addAction(item);
        }

        popup2->addSeparator();
    
        item = new ItemAction(qtr("Edit Server"),popup2);
        item->item_action_set_callback((item_callback)on_edit_server,server);
        popup2->addAction(item);

        item = new ItemAction(qtr("Remove Server"),popup2);
        item->item_action_set_callback((item_callback)object_kill,server);
        popup2->addAction(item);

    }  

    item = new ItemAction(qtr("Quick configure"), popup);
    popup->addAction(item);
    {
        popup2 = new QMenu(popup);
        item->setMenu(popup2);

//         item =  new ItemAction(qtr("Deactivated"), popup2);
//         item->setCheckable(true);
//         if (server->flags & NAP_SERVER_IGNORE) {
//             item->setChecked(true);
//         }
//         item->item_action_set_callback((item_callback)on_server_toggle_ignore,server, true);
//         popup2->addAction(item);
    } 
    return popup;
}


void NapNetworkTreeWidget::contextMenuEvent( QContextMenuEvent * e )
{
    QList<QTreeWidgetItem* > item_list = selectedItems();

    if (item_list.isEmpty())
    {
        QMenu *menu = new QMenu(this);
        ui_nap_create_top_actions(menu);
        menu->exec(e->globalPos());
    }
    else {
        NapTreeItem* item = (NapTreeItem*) item_list.takeFirst();
        if (item->object_get_type() == GROUP_OBJECT)
        {
            QMenu *menu = ui_nap_create_group_actions((nap_ng_t*) item->object_get_object());
            menu->exec(e->globalPos());
        }
        else if (item->object_get_type() == NETWORK_OBJECT)
        {
            QMenu *menu = ui_nap_create_net_actions((nap_net_t*) item->object_get_object());
            menu->exec(e->globalPos());
        }
        else if (item->object_get_type() == SERVER_OBJECT)
        {
            QMenu *menu = ui_nap_create_server_actions((nap_server_t*)item->object_get_object());
            menu->exec(e->globalPos());
        } 
    }
}

NapNetworkTreeWidget* NapNetworkTreeWidget::GetNetworkTree( ) 
{
   static NapNetworkTreeWidget* tree = NULL;
   if (tree == NULL)
   {
        tree = new NapNetworkTreeWidget(); 
        tree->setObjectName("NapNetTreeWidget");
   }
   return tree;
}

void NapNetworkTreeWidget::ui_nap_add_group(nap_ng_t* ng)
{
    NapNetworkTreeWidget* tree = GetNetworkTree( );
    if (tree == NULL) { return; }

    object_notify_d(OBJ(ng), (ObjectFunc)ui_nap_delete_group, NULL);
    object_monitor_d(OBJ(ng), (ObjectFunc)ui_nap_update_group, NULL);
    ui_nap_show_group(tree, ng);
}

void NapNetworkTreeWidget::ui_nap_show_group(NapNetworkTreeWidget* tree, nap_ng_t* ng)
{
    NapTreeItem *node;
    GList* dlist;
    
    Q_ASSERT(ng);
   
    node = static_cast<NapTreeItem *> (object_get_data(OBJ(ng), "NapGroupEntry"));

    if (!node)
    {
        node = new NapTreeItem(tree, ng, GROUP_OBJECT );
        node->setText(0, qfu(OBJ(ng)->name));
        object_set_data(OBJ(ng), "NapGroupEntry", node);
        ui_nap_update_group(ng);
    }

    for (dlist = ng->nets; dlist; dlist = dlist->next)
    {
        nap_net_t* net = (nap_net_t *) dlist->data;
        if (!object_get_data(OBJ(net), "NapNetEntry")) ui_nap_add_network(net);
    }
}

void NapNetworkTreeWidget::ui_nap_update_group(nap_ng_t* ng)
{
    NapNetworkTreeWidget* tree = GetNetworkTree( );
    if (tree == NULL) { return; }
    NapTreeItem *node;
    int gigs = 0;
    int files = 0;
    int users = 0;
    QString str;
    
    Q_ASSERT(ng);
    
    node = static_cast<NapTreeItem *> (object_get_data(OBJ(ng), "NapGroupEntry"));

    if (!node)
    {
        ui_nap_show_group(tree,ng);
        return;
    }
  
    if (ng->online > 0)
    {
        users = nap_ng_stats(ng, 0);
        files = nap_ng_stats(ng, 1);
        gigs  = nap_ng_stats(ng, 2);
    }
    
    if (ng->online)
    {
        int keep = ng->keep==-1?(int)g_list_length(ng->nets):ng->keep;
        str = qtr("%1/%2  connected")
             .arg(ng->online)
             .arg( keep );
    }
    else
    {
        str = QString("");
    }

    node->setText(0, qfu(OBJ(ng)->name));
    node->setText(1, str);
    node->setText(3, qfu("%1").arg(users));
    node->setText(4, qfu("%1").arg(files));
    node->setText(5, qfu("%1").arg(gigs));

}

void NapNetworkTreeWidget::ui_nap_delete_group(nap_ng_t* ng)
{
    NapNetworkTreeWidget* tree = GetNetworkTree( );
    if (tree == NULL) { return; }
    NapTreeItem *node = static_cast<NapTreeItem *> ( object_get_data(OBJ(ng), "NapGroupEntry"));
    if (node)
    {
        delete node;
    }
}

void NapNetworkTreeWidget::ui_nap_add_network(nap_net_t* net)
{
    NapNetworkTreeWidget* tree = GetNetworkTree( );
    if (tree == NULL) { return; }

    object_notify_d(OBJ(net), (ObjectFunc)ui_nap_delete_network, NULL);
    object_monitor_d(OBJ(net), (ObjectFunc)ui_nap_update_network, NULL);
    
    ui_nap_show_network(tree, net);
}

void NapNetworkTreeWidget::ui_nap_show_network(NapNetworkTreeWidget* tree, nap_net_t* net)
{
    NapTreeItem *node;
    NapTreeItem *parent;
    GList* dlist;

    if (!net ) return;
    if ( !net->group) return;
    parent = static_cast<NapTreeItem *> (object_get_data(OBJ(net->group), "NapGroupEntry"));
    if (!parent) return;
    node = static_cast<NapTreeItem *> (object_get_data(OBJ(net), "NapNetEntry"));
    
    if (!node)
    {
        node = new NapTreeItem(parent, net, NETWORK_OBJECT );
        node->setText(0, qfu("%1").arg(OBJ(net)->name));
        object_set_data(OBJ(net), "NapNetEntry", node);
        ui_nap_update_network(net);
    }
    
    for (dlist = net->servers; dlist; dlist = dlist->next)
    {
        nap_server_t* server = (nap_server_t*) dlist->data;
        if (!object_get_data(OBJ(server), "NapServerEntry"))
            ui_nap_show_server(tree,server);
    } 
}

void NapNetworkTreeWidget::ui_nap_update_network( nap_net_t* net)
{
    NapNetworkTreeWidget* tree = GetNetworkTree( );
    if (tree == NULL) { return; }
   
    NapTreeItem *node;
    int gigs = 0;
    int files = 0;
    int users = 0;
    QString status;
    int diff=0;
    
    if (!net) return;

    node = static_cast<NapTreeItem *> (object_get_data(OBJ(net), "NapNetEntry"));

    // check whether node is shown
    if (!node)
    {
        ui_nap_show_network(tree,net);
        return;
    }
    else if (!net->group)
    {
        ui_nap_delete_network(net);
        return;
    }

    // check whether the node has changed its parent
    /*
      if (!ui_iter_is_parent_to(GTK_TREE_MODEL(ui_nap_get_net_tree()), parent, node)) {
      g_assert(FALSE);
      ui_nap_net_hide(net);
      ui_nap_net_show(net);
      return;
      }
    */
    
  
    if (net->active && net->active->status == NAP_SERVER_ONLINE)
    {
        status = qtr("Online");
    }
    else if (net->active)
    {
        status = qtr("Connecting");
    }
//     else if (net->flags & NAP_NET_AUTO_CONNECT)
//     {
//         status = qtr("Autoconnect");
//     }
    
    users = nap_net_stats(net, 0);
    files = nap_net_stats(net, 1);
    gigs  = nap_net_stats(net, 2);
    
    if (net->last_success)
        diff = CurrentTime - net->last_success;
   
    QString msg = diff>0?qtr("since %1").arg(print_duration_unit(&diff)):"";

    node->setText(0, qfu(OBJ(net)->name));
    node->setText(1, status);
    node->setText(1, qfu("%1  %2").arg(net->number_fails).arg(msg));
    node->setText(3, QString("%1").arg(users));
    node->setText(4, QString("%1").arg(files));
    node->setText(5, QString("%1").arg(gigs));


/*
    TODO : update these icons
    2, (net->active && net->active->status == NAP_SERVER_ONLINE),
    8, (net->flags & NAP_NET_PERSISTENT)?ServerPerm:NULL,
*/  
}

void NapNetworkTreeWidget::ui_nap_delete_network( nap_net_t* net)
{
    NapNetworkTreeWidget* tree = GetNetworkTree( );
    if (tree == NULL) { return; }

    NapTreeItem *node = static_cast<NapTreeItem *> (object_get_data(OBJ(net), "NapNetEntry"));
    if (node)
    {
        delete node;
    }
}

void NapNetworkTreeWidget::ui_nap_add_server( nap_server_t* server )
{
    NapNetworkTreeWidget* tree = GetNetworkTree( );
    if (tree == NULL) { return; }

    object_notify_d(OBJ(server), (ObjectFunc)ui_nap_delete_server, NULL);
    object_monitor_d(OBJ(server), (ObjectFunc)ui_nap_update_server, NULL);
    
    ui_nap_show_server(tree, server);
}

void NapNetworkTreeWidget::ui_nap_show_server(NapNetworkTreeWidget* tree, nap_server_t* server)
{
    NapTreeItem *node = static_cast<NapTreeItem *> (object_get_data(OBJ(server), "NapServerEntry"));
    NapTreeItem *parent;
    
    if (!server || !server->net) return;
    if (node) return;
    
    parent = static_cast<NapTreeItem *> (object_get_data(OBJ(server->net), "NapNetEntry"));
    if (!parent) return;
    
    node = new NapTreeItem(parent, server, SERVER_OBJECT );
    object_set_data(OBJ(server), "NapServerEntry", node );   
    node->setText(0, qfu(OBJ(server)->name));
    
    ui_nap_update_server(server);
}


void NapNetworkTreeWidget::ui_nap_update_server(nap_server_t* server)
{
    NapNetworkTreeWidget* tree = GetNetworkTree( );
    if (tree == NULL) { return; }

    NapTreeItem *node = static_cast<NapTreeItem *> (object_get_data(OBJ(server), "NapServerEntry"));
    
    // check whether node is shown
    if (!node)
    {
        ui_nap_show_server(tree,server);
        return;
    }
    else if (!server->net)
    {
        ui_nap_delete_server(server);
        return;
    }

    node->set_online(server->status == NAP_SERVER_ONLINE);

    node->setText(0, qfu(OBJ(server)->name));
    node->setText(1, qfu("%1").arg(server->message));
    node->setText(3, qfu("%1").arg(server->users));
    node->setText(4, qfu("%1").arg(server->files));
    node->setText(5, qfu("%1").arg(server->gigs));
/*
    TODO : update these icons
    2, (server->status == NAP_SERVER_ONLINE),
    7, (server->flags & NAP_SERVER_NAPI)?ServerNapi:ServerNNapi,
    8, (server->flags & NAP_SERVER_IGNORE)?ServerIgn:NULL,
*/
    
}

void NapNetworkTreeWidget::ui_nap_delete_server(nap_server_t* server)
{
    NapNetworkTreeWidget* tree = GetNetworkTree( );
    if (tree == NULL) { return; }

    NapTreeItem *node = static_cast<NapTreeItem *> (object_get_data(OBJ(server), "NapServerEntry"));
    if (node)
    {
        delete node;
    }
}
