#ifndef _NAP_NET_TREE_H_
#define _NAP_NET_TREE_H_

#include <QIcon>
#include <QTreeWidget>
#include "nap_network.h"
#include "ui_nap_net.hpp"

class NapTreeItem;

class NapNetworkTreeWidget : public QTreeWidget
{
    Q_OBJECT

public:
    NapNetworkTreeWidget( QWidget *parent = 0);
    static NapNetworkTreeWidget* GetNetworkTree();

    void ui_nap_create_top_actions( QMenu *menu );

    static void on_keep(nap_ng_t* ng, int *hm);
    static void on_create_edit_group(nap_ng_t* group);
    static void on_create_net(nap_ng_t* ng);
    static void on_create_edit_net(nap_net_t* net, nap_ng_t* ng);

    static void on_ng_toggle_auto(nap_ng_t* ng);

    QMenu *ui_nap_create_group_actions(nap_ng_t* ng);
    
    static void on_net_toggle_pers(NapTreeItem *item);
    static void on_net_toggle_winmx(NapTreeItem *item);
    static void on_net_toggle_server(NapTreeItem *item);
    static void on_net_toggle_global(NapTreeItem *item);
    static void on_net_move(NapTreeItem *item, nap_ng_t* ng=NULL);

    static void on_net_stop_connect(NapTreeItem *item);
    static void on_net_start_connect(NapTreeItem *item);
    static void on_net_disconnect(NapTreeItem *item);
    static void on_edit_net(nap_net_t* net);
    static void on_create_server(nap_net_t* net);
    static void on_show_channels(nap_net_t* net);

    QMenu *ui_nap_create_net_actions(nap_net_t* net);

    static void on_create_edit_server(nap_server_t* server, nap_net_t* net);

    static void on_server_disconnect(nap_server_t* server);
    static void on_server_connect(nap_server_t* server);
    static void on_edit_server(nap_server_t* server);
    static void on_server_toggle_ignore(nap_server_t* server);

    QMenu *ui_nap_create_server_actions(nap_server_t* server);

    static void ui_nap_add_group     ( nap_ng_t* ng);
    static void ui_nap_show_group    ( NapNetworkTreeWidget* tree, nap_ng_t* ng);
    static void ui_nap_update_group  ( nap_ng_t* ng );
    static void ui_nap_delete_group  ( nap_ng_t* ng );
    
    static void ui_nap_add_network   ( nap_net_t* net );
    static void ui_nap_show_network  ( NapNetworkTreeWidget* tree, nap_net_t* net);
    static void ui_nap_update_network( nap_net_t* net );
    static void ui_nap_delete_network( nap_net_t* net );

    static void ui_nap_add_server    ( nap_server_t* server );
    static void ui_nap_show_server   ( NapNetworkTreeWidget* tree, nap_server_t* server );
    static void ui_nap_update_server ( nap_server_t* server );
    static void ui_nap_delete_server ( nap_server_t* server );
    
    /* void setNetworkContent(NapNetworkContent *networkContent); */

    QIcon group_icon;
    QIcon net_icon;
    QIcon server_icon;

    void contextMenuEvent( QContextMenuEvent * e );

private slots:

    /* void ui_nap_item_selected(QTreeWidgetItem * item = 0); */
	/* void objectSelectionChanged( ); */

   /* void connectServer( );
    void disconnectServer( );
    
    void connectNetwork( );
    void disconnectNetwork( );
    void displayChannelList();

    void connectGroup( );
    void disconnectGroup( ); */

};

enum NetworkObjectType {
    GROUP_OBJECT,
    NETWORK_OBJECT,
    SERVER_OBJECT
};

class NapTreeItem : public QTreeWidgetItem
{
public:
    NapTreeItem(NapNetworkTreeWidget *tree, void *object, NetworkObjectType type);
    NapTreeItem(NapTreeItem *item, void *object, NetworkObjectType type);
    ~NapTreeItem();
    NetworkObjectType object_get_type( );
    void* object_get_object( );
    void  set_online(bool value);
private:
    void* m_object;
    NetworkObjectType m_type;
};

typedef void (*item_callback)(void* data);
typedef void (*item_callback_extra)(void* data, void* extra);

class ItemAction : public QAction
{
    Q_OBJECT
public:
    ItemAction(const QString &text, QObject *parent);
    void item_action_set_list_callback_with_extra(item_callback_extra func, 
                            QList<QTreeWidgetItem* > selection, void* extra, bool check=false);
    void item_action_set_list_callback(item_callback func, QList<QTreeWidgetItem* > selection, bool check=false);
    void item_action_set_callback_with_extra(item_callback_extra func, void* data, void* extra, bool check=false);
    void item_action_set_callback(item_callback func, void* data, bool check=false);
private slots:
    void item_list_action_on_triggered();
    void item_list_action_on_toggled(bool);
    void item_action_on_triggered();
    void item_action_on_toggled(bool);
private:
    QList<QTreeWidgetItem* > m_selection;
    void* m_extra;
    void* m_object;
    item_callback m_callback;
    item_callback_extra m_callback_extra;
};

#endif //_NAP_NET_TREE_H_
