#include "lcore/l_i18n.h"
#include "lcore/l_base.h"

#include "lcore/qt4_ui/ui_main.hpp"
#include "NetworkDataAccessor.hpp"

GroupAccessorDialog::GroupAccessorDialog(nap_ng_t* ng, QWidget *parent) 
    : QDialog(parent), m_ng(ng)
{
    setupUi(this);   
    if (ng) {
        setWindowTitle(qtr("Edit Group"));
        line_group_name->setText(qfu(OBJ(ng)->name));
        spin_group_keep->setValue(ng->keep);
        if (ng->flags & NAP_NG_AUTO_CONNECT)
            check_group_auto->setChecked(true);
    }
    else {
        setWindowTitle(qtr("Create Group"));
    }
    connect(push_ok,SIGNAL(clicked()),this,SLOT(edit_group()));
    connect(push_cancel,SIGNAL(clicked()),this,SLOT(reject()));   
}

void GroupAccessorDialog::edit_group()
{
    QString name = line_group_name->text();

    if (name.isEmpty())
    {
        lopster_notify(LOPSTER_MESSAGE_INFO, _("Invalid input"),
                       _("The group name is empty")); 
        return;
    }
   
    if (!m_ng)
    {
        m_ng = nap_ng_create(qtu(name), 0, NAP_CREATE_UNIQUE);
        if (!m_ng)
        {
            lopster_notify(LOPSTER_MESSAGE_INFO, _("Invalid input"),
                        _("A group with this name already exists"));
            return;
        }
    }
    else
    {
        object_set_name(OBJ(m_ng), qtu(name));
    }
    m_ng->keep = spin_group_keep->value();

    if (check_group_auto->checkState() == Qt::Checked)
        m_ng->flags |= NAP_NG_AUTO_CONNECT;
    else
        m_ng->flags &= ~NAP_NG_AUTO_CONNECT;
    
    object_set_dirty(OBJ(m_ng));
    nap_network_save();
    accept ();
}

NetworkAccessorDialog::NetworkAccessorDialog(nap_net_t* net, nap_ng_t* ng, QWidget *parent) 
    : QDialog(parent), m_net(net) , m_ng(ng)
{
    setupUi(this);
    if (net)
    {
        setWindowTitle(qtr("Edit Network"));
        line_net_name->setText(qfu(OBJ(net)->name));
        char* str = make_string_from_list(net->autojoin, " ");
        line_net_auto->setText(qfu(str));
        g_free(str);
//         if (net->flags & NAP_NET_PERSISTENT)
//             check_remove_auto->setChecked(true);
        if (net->flags & NAP_NET_OWN_SERVER) 
            check_own_server->setChecked(true);
        if (net->flags & NAP_NET_IGN_GLOBAL)
            check_ignore_global->setChecked(true);
//         if (net->flags & NAP_NET_NO_WINMX)
//             check_emul_winmx26->setChecked(true);
        if (net->nick)
        {
            line_net_login->setText(qfu(net->nick));
        }
        if (net->pass)
        {
            line_net_passwd->setText(qfu(net->pass));
        }
        if (net->charset)
        {
            widget_charset->charset_set_selection(qfu(net->charset));
        }       
    }
    else
    {
        setWindowTitle(qtr("Create Network"));
    }
    connect(push_ok,SIGNAL(clicked()),this,SLOT(edit_network()));
    connect(push_cancel,SIGNAL(clicked()),this,SLOT(reject()));   
}

void NetworkAccessorDialog::edit_network()
{
    QString name = line_net_name->text();
    QString user = line_net_login->text();
    QString pass =  line_net_passwd->text();

    if (user.contains(' '))
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                        _("The username may not contain any spaces"));
        return;
    }
    if (pass.contains(' '))
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                        _("The password may not contain any spaces"));
        return;
    }
    
    if (name.isEmpty())
    {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                        _("The network name is empty"));
        return;
    }
    if (!m_net)
    {
        m_net = nap_net_create(m_ng, qtu(name), NAP_CREATE_UNIQUE);
        if (!m_net)
        {
            lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                        _("This group already contains a network with this name"));
            return;
        }
    }
    else
    {
        object_set_name(OBJ(m_net), qtu(name));
    }

    nap_net_set_nick(m_net, qtu(user));
    nap_net_set_password(m_net,qtu(pass) );
        
    make_list_from_string(&m_net->autojoin, qtu(line_net_auto->text()) , " ");

    if (check_own_server->checkState() == Qt::Checked )
        m_net->flags |= NAP_NET_OWN_SERVER;
    else
        m_net->flags &= ~NAP_NET_OWN_SERVER;

    if (check_ignore_global->checkState() == Qt::Checked )
        m_net->flags |= NAP_NET_IGN_GLOBAL;
    else
        m_net->flags &= ~NAP_NET_IGN_GLOBAL;

//     if (check_emul_winmx26->checkState() == Qt::Checked )
//         m_net->flags |= NAP_NET_NO_WINMX;
//     else
//         m_net->flags &= ~NAP_NET_NO_WINMX;
    
//     if (check_remove_auto->checkState() == Qt::Checked )
//         m_net->flags |= NAP_NET_PERSISTENT;
//     else
//         m_net->flags &= ~NAP_NET_PERSISTENT;
    g_free(m_net->charset);
    m_net->charset = g_strdup(qtu(widget_charset->charset_get_selection()));
    m_net->charset_warned = FALSE; 
    
    object_set_dirty(OBJ(m_net));
    nap_network_save();
    
    accept ();
}



ServerAccessorDialog::ServerAccessorDialog(nap_server_t* server, nap_net_t* net, QWidget *parent) 
    : QDialog(parent) , m_server(server) , m_net(net)
{
    setupUi(this);
    if (server)
    {
        setWindowTitle(qtr("Edit Server"));
        line_server_addr->setText(qfu(OBJ(server)->name));
        spin_server_port->setValue(server->port);
        if (server->flags & NAP_SERVER_META)
            check_server_redir->setChecked(true);
//         if (server->flags & NAP_SERVER_IGNORE)
//             check_server_deac->setChecked(true);
    }
    else {
        setWindowTitle(qtr("Create Server"));
    }
    connect(push_ok,SIGNAL(clicked()),this,SLOT(edit_server()));
    connect(push_cancel,SIGNAL(clicked()),this,SLOT(reject()));
}

void ServerAccessorDialog::edit_server( ) {
    QString name = line_server_addr->text();

    if (name.isEmpty()) {
        lopster_notify(LOPSTER_MESSAGE_ERROR, _("Invalid input"),
                    _("The server address is empty")); 
        return;
    }

    if (!m_server) {
        m_server = nap_server_create(m_net, qtu(name), NAP_CREATE_ALWAYS);
    } else {
        object_set_name(OBJ(m_server), qtu(name));
        m_server->ip = 0;
    }
    m_server->port = spin_server_port->value();

    if (check_server_redir->checkState() == Qt::Checked)
        m_server->flags |= NAP_SERVER_META;
    else
        m_server->flags &= ~NAP_SERVER_META;

//     if (check_server_deac->checkState() == Qt::Checked)
//         m_server->flags |= NAP_SERVER_IGNORE;
//     else
//         m_server->flags &= ~NAP_SERVER_IGNORE;

    object_set_dirty(OBJ(m_server));
    nap_network_save();  
    
    accept ();
}
