/* Copyright (C) 2005 sgop@users.sourceforge.net
/* Copyright (C) 2007 Lopster Team. This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.19 $
 * $Date: 2007/10/11 12:01:59 $
 * $Author: sid4u $
 */

#include "lcore/qt4_ui/ModuleWidget.hpp"
#include "lcore/qt4_ui/ui_main.hpp"

#include "ui_nap_chat.hpp"

#define FIND_NAP_CHAT_GUI(p,d) \
            char* name = (char*) object_get_data(OBJ(p), "ChatGui"); \
            d =  qobject_cast<NapChatGui *> (findWidgetByName(QString("%1").arg(name))); \
            Q_ASSERT(d);


class NapChatUserItemModel : public QStandardItemModel
{
public:
    NapChatUserItemModel(QObject *parent=0);
    // Will automaticall handle the StandardItems (by column and row)
    void nap_item_chat_user_clean();
    void nap_item_chat_user_add(nap_cu_t* cu, nap_pub_t* npp);
    void nap_item_chat_user_update(nap_cu_t* cu, nap_pub_t* npp);
    void nap_item_chat_user_delete(nap_cu_t* cu, nap_pub_t* npp);
};

NapChatUserItemModel::NapChatUserItemModel(QObject *parent)
    : QStandardItemModel(0, 3, parent) 
{
    QStringList headers;
    headers << qtr("User") << qtr("Shared") << qtr("Speed");
    setHorizontalHeaderLabels(headers); 
}

void NapChatUserItemModel::nap_item_chat_user_clean()
{
    removeRows(1,rowCount());
}


void NapChatUserItemModel::nap_item_chat_user_delete(nap_cu_t* cu, nap_pub_t* npp)
{
    QStandardItem *user;

    user = static_cast< QStandardItem *>(object_get_data(OBJ(cu), "ChatUserEntry"));
    if (!user) return;

    QModelIndex model_index = indexFromItem(user);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    object_free_data(OBJ(cu),"ChatUserEntry",NULL);
    removeRow(row);   
}

void NapChatUserItemModel::nap_item_chat_user_update(nap_cu_t* cu, nap_pub_t* npp)
{
    QStandardItem *user;
    QStandardItem *shared;
    QStandardItem *speed;

    user = static_cast< QStandardItem *>(object_get_data(OBJ(cu), "ChatUserEntry"));
    if (!user) return;

    QModelIndex model_index = indexFromItem(user);
    if (!model_index.isValid()) return;
    int row = model_index.row();
    if (row == -1) return;
    
    
    // We have the row, let get all the items and update them
    shared = item(row, 1); 
    speed = item(row, 2); 

    // Set default data (for right action menu)
    user->setData(QVariant::fromValue((void*)cu));

    // update the items
    user->setText(qfu(OBJ(cu)->name));
    shared->setText(qfu("%1").arg(cu->shared));
    speed->setText(qtr(NapSpeedStr(cu->speed)));
}

void NapChatUserItemModel::nap_item_chat_user_add(nap_cu_t* cu, nap_pub_t* npp)
{
    QList<QStandardItem *> items;

    QStandardItem *user;
    QStandardItem *shared;
    QStandardItem *speed;

    if (!cu) return;

    if (object_get_data(OBJ(cu), "ChatUserEntry")) return;    
    
    user = new QStandardItem(qfu(OBJ(cu)->name)); 
    shared = new QStandardItem(qfu("%1").arg(cu->shared));
    speed = new QStandardItem(qtr(NapSpeedStr(cu->speed)));
    
    items.append(user);
    items.append(shared);
    items.append(speed);

    // Add the items
    appendRow(items);
    object_set_data(OBJ(cu), "ChatUserEntry", user);
    nap_item_chat_user_update(cu,npp);
}


NapChatUserTree::NapChatUserTree(QWidget *parent)
    : UiTreeView(parent)
{
    ui_treeview_set_model(new NapChatUserItemModel(this));
}

void NapChatUserTree::ui_nap_user_init(nap_pub_t* page)
{
    action_item_t* actions = 
        action_item_search("Online nap users actions");
    ui_tv_handle_button(1, &actions, page);
}

void NapChatUserTree::ui_nap_user_hide(nap_cu_t* cu, nap_pub_t* npp)
{
    NapChatUserTree *tree;   
    NapChatGui *ndata;
    
    if (!npp || !cu) return;
    FIND_NAP_CHAT_GUI(npp,ndata);
    if (!ndata || !ndata->user_tree()) return;

    tree = ndata->user_tree();
    static_cast<NapChatUserItemModel *>(tree->model())->nap_item_chat_user_delete(cu, npp);

    UiNapChat::ui_nap_update_counter(ndata->label_counter(), npp->online);
}

void NapChatUserTree::ui_nap_user_update(nap_cu_t* cu, nap_pub_t* npp)
{
    NapChatUserTree *tree;   
    NapChatGui *ndata;
    
    if (!npp) return;
    FIND_NAP_CHAT_GUI(npp,ndata);

    if (!ndata || !ndata->user_tree()) return;
    
    tree = ndata->user_tree();
    static_cast<NapChatUserItemModel *>(tree->model())->nap_item_chat_user_update(cu, npp);
}

void NapChatUserTree::ui_nap_user_show(nap_cu_t* cu, nap_pub_t* npp)
{
    NapChatUserTree *tree;   
    NapChatGui *ndata;
    FIND_NAP_CHAT_GUI(npp,ndata);
    if (!ndata) return;
  
    tree = ndata->user_tree();
    static_cast<NapChatUserItemModel *>(tree->model())->nap_item_chat_user_add(cu, npp);

    UiNapChat::ui_nap_update_counter(ndata->label_counter(), npp->online);   
    ui_nap_user_update(cu, npp);
}

void NapChatUserTree::ui_nap_user_new(nap_cu_t* cu, nap_pub_t* npp)
{
    Q_ASSERT(npp && cu);
    
    object_notify_o(OBJ(cu), (ObjectFunc)ui_nap_user_hide, OBJ(npp)); // of no use?
    object_monitor_o(OBJ(cu), (ObjectFunc)ui_nap_user_update, OBJ(npp));
    
    ui_nap_user_show(cu, npp);
}

NapChatGui::NapChatGui(nap_pub_t* page, QWidget *parent )
    : ChatGui(parent), m_page(page)
{
    // Need to fill in the inherited parent body
    body.setupUi(ui_chat_gui_get_body());
    this->no_views = 2;
    this->view[NAP_B_OP] = body.text_op;
    this->view[NAP_B_PUBLIC] = body.text_chan;
    //FIXME : check from layout preferences if hidden or not
    body.widget_op_tools->setVisible(false); 
    body.text_op->setVisible(false); 
    body.splitter_main->setStretchFactor(0,3);

    body.tree_users->ui_nap_user_init(page);
    mbutton[0] = body.tool_t; 
    mbutton[1] = body.tool_r; 
    mbutton[2] = body.tool_p; 
    mbutton[3] = body.tool_i; 
    mbutton[4] = body.tool_m; 

    connect(mbutton[0],SIGNAL(toggled(bool)),this,SLOT(on_chmode_t_clicked(bool)));
    connect(mbutton[1],SIGNAL(toggled(bool)),this,SLOT(on_chmode_r_clicked(bool)));
    connect(mbutton[2],SIGNAL(toggled(bool)),this,SLOT(on_chmode_p_clicked(bool)));
    connect(mbutton[3],SIGNAL(toggled(bool)),this,SLOT(on_chmode_i_clicked(bool)));
    connect(mbutton[4],SIGNAL(toggled(bool)),this,SLOT(on_chmode_m_clicked(bool)));

    for (int i1= NAP_L_LEECH; i1 <= NAP_L_ELITE; i1++)
    {
        body.combo_level->addItem(qtr(NapLevel(i1)));
    }

    body.line_limit->setValidator(new QIntValidator(body.line_limit)); 
    connect(body.combo_level,SIGNAL(currentIndexChanged(int)),this,SLOT(on_chlevel_changed(int)));
    connect(body.line_limit,SIGNAL(returnPressed()),this,SLOT(on_chlimit_clicked()));

    // Connect the CommandLink Push buttons to display hide the above widgets
    connect(body.push_showctrl,SIGNAL(clicked()),this,SLOT(ui_nap_chat_toggle_ctrl()));
    connect(body.push_showop,SIGNAL(clicked()),this,SLOT(ui_nap_chat_toggle_op()));
    connect(body.push_showusers,SIGNAL(clicked()),this,SLOT(ui_nap_chat_toggle_users()));
}

NapChatUserTree *NapChatGui::user_tree()
{
    return body.tree_users;
}

QLabel *NapChatGui::label_counter()
{
    return body.label_usersno;
}

QWidget *NapChatGui::ui_nap_chat_get_op_tools()
{
    return body.widget_op_tools;
}

QLineEdit *NapChatGui::ui_nap_chat_get_op_limit()
{
    return body.line_limit;
}

QComboBox *NapChatGui::ui_nap_chat_get_op_level()
{
    return body.combo_level;
}

void NapChatGui::ui_nap_chat_toggle_ctrl()
{
    body.widget_op_tools->setVisible(!body.widget_op_tools->isVisible());
    body.push_showctrl->setText(body.widget_op_tools->isVisible()?
                                qtr("Hide controls"):qtr("Show controls"));
}

void NapChatGui::ui_nap_chat_toggle_op()
{
    body.text_op->setVisible(!body.text_op->isVisible());
    body.push_showop->setText(body.text_op->isVisible()?
                                qtr("Hide op-chat"):qtr("Show op-chat"));
}

void NapChatGui::ui_nap_chat_toggle_users()
{
    body.tree_users->setVisible(!body.tree_users->isVisible());
    body.push_showusers->setText(body.tree_users->isVisible()?
                                qtr("Hide users"):qtr("Show users"));
}

void NapChatGui::ui_nap_chat_gui_update_topic(const QString &title)
{
    body.label_topic->setText(title);
}

void NapChatGui::ui_nap_chat_print_net(QTextCursor cursor, va_list ap) {
    nap_net_t* net;
    // prints [network]
    // args: nap_net_t* net
    net = (nap_net_t*) va_arg(ap, void*);

    QTextCharFormat format = ChatPage::ui_chat_get_format_from_tag("c_bracket"); 
    cursor.insertText("[", format);   
    format = ChatPage::ui_chat_get_format_from_tag("c_text"); 
    cursor.insertText(qfu("%1").arg(OBJ(net)->name), format);   
    format = ChatPage::ui_chat_get_format_from_tag("c_bracket"); 
    cursor.insertText("]", format);   
}
void NapChatGui::ui_nap_chat_print_netstr(QTextCursor cursor, va_list ap) {
    nap_net_t* net = (nap_net_t*) va_arg(ap, void*);
    const char* text = va_arg(ap, char*);

    // prints [network:text]

    QTextCharFormat format = ChatPage::ui_chat_get_format_from_tag("c_bracket"); 
    cursor.insertText("[", format);   
    format = ChatPage::ui_chat_get_format_from_tag("c_text"); 
    cursor.insertText(qfu("%1").arg(OBJ(net)->name), format);   
    cursor.insertText(":", format);   
    cursor.insertText(qfu("%1").arg(text), format);   
    format = ChatPage::ui_chat_get_format_from_tag("c_bracket"); 
    cursor.insertText("]", format);   
}

void NapChatGui::ui_nap_chat_print_channel(QTextCursor cursor, va_list ap) {
    char* channel = va_arg(ap, char*);
    nap_net_t* net = (nap_net_t*) va_arg(ap, void*);  // FIXME
    char* text = va_arg(ap, char*);
    QTextCharFormat format;

    // prints channel
    if (text && text[0]) {
        format = ChatPage::ui_chat_get_format_from_tag("c_bracket"); 
        cursor.insertText(qfu("%1").arg(text[0]), format);  
    }
    format = ChatPage::ui_chat_get_format_from_tag("c_text");
    cursor.insertText(qfu("%1").arg(channel), format);  
    if (text && text[1]) {
        format = ChatPage::ui_chat_get_format_from_tag("c_bracket"); 
        cursor.insertText(qfu("%1").arg(text+1), format);  
    }
}

void NapChatGui::on_chmode_t_clicked(bool checked)
{
    on_chmode_clicked(checked,0);
}
void NapChatGui::on_chmode_r_clicked(bool checked)
{
    on_chmode_clicked(checked,1);
}
void NapChatGui::on_chmode_p_clicked(bool checked)
{
    on_chmode_clicked(checked,2);
}
void NapChatGui::on_chmode_i_clicked(bool checked)
{
    on_chmode_clicked(checked,2);
}
void NapChatGui::on_chmode_m_clicked(bool checked)
{
    on_chmode_clicked(checked,4);
}

void NapChatGui::on_chmode_clicked(bool checked, int mode) {
    int set;
    nap_pub_t* page;
    if (checked) set = 1;
    else set = 0;
    Q_ASSERT(this->m_page);
    nap_channel_set_mode(this->m_page, set?mode:0, set?0:mode); 
}

void NapChatGui::on_chlevel_changed(int index) {
    Q_ASSERT(this->m_page);
    nap_channel_set_level(this->m_page, index); 
}

void NapChatGui::on_chlimit_clicked() {
    Q_ASSERT(this->m_page);
    nap_channel_set_limit(this->m_page, body.line_limit->text().toInt()); 
}

void UiNapChat::ui_nap_chat_init() {
    ChatPage::ui_chat_register_print_func("nap_net", (PrintFunc) NapChatGui::ui_nap_chat_print_net);
    ChatPage::ui_chat_register_print_func("nap_netstr", (PrintFunc) NapChatGui::ui_nap_chat_print_netstr);
    ChatPage::ui_chat_register_print_func("nap_channel", (PrintFunc) NapChatGui::ui_nap_chat_print_channel);
}

void UiNapChat::ui_nap_public_remove(chat_page_t* page)
{
    NapChatGui *ndata;
    FIND_NAP_CHAT_GUI(page,ndata);
    // g_list_foreach(NPP(page)->users, (GFunc)object_free_data_common, "ChatUserEntry");   
    if (!ndata) return;

    ChatPage::s_ui_chat_page_tab_remove(qobject_cast<ChatGui *>(ndata));
    delete ndata;
    object_free_data_common(OBJ(page), "ChatGui");
}

void UiNapChat::ui_nap_create_public_page(nap_pub_t* page)
{
    // Create the NapPublicChatPage, and update the object
    // No need to set parent here, as it will be deleted when closing in tab
    NapChatGui *tab = new NapChatGui(page);
    char* object_name = g_strdup_printf("NapChatGui_%x",tab);
    tab->setObjectName(QString("%1").arg(object_name));
    object_set_data(OBJ(page), "ChatGui", object_name);
    // Then add it to the main chat entry page
    ChatPage::ui_chat_page_add (PAGE(page), tab);
    object_notify_d(OBJ(page), (ObjectFunc)ui_nap_public_remove, NULL);
}


void UiNapChat::ui_nap_cp_update_op(nap_pub_t* npp)
{
    NapChatGui *data;
    bool opped;
    int i1;
    
    if (!npp || NCP(npp)->type != NAP_PAGE_PUBLIC) return;
    FIND_NAP_CHAT_GUI(npp,data);
    if (!data /* || !data->public_settings */) return;
    opped = ((NCP(npp)->net && NCP(npp)->net->level >= NAP_L_MOD) || npp->opped);
    
    QWidget *op_tools = data->ui_nap_chat_get_op_tools();
    op_tools->setEnabled(opped);

    for (i1 = 0; i1 < NAP_CMODE_NUMBER; i1++)
    {
        if (npp->chmode & (1<<i1))
            data->mbutton[i1]->setEnabled(true);
        else
            data->mbutton[i1]->setEnabled(false);
    }
    
    QLineEdit *lentry = data->ui_nap_chat_get_op_limit();
    lentry->setText(QString("%1").arg(npp->limit));
    QComboBox *combo = data->ui_nap_chat_get_op_level();
    if (NapIsLevel(npp->level))
        combo->setCurrentIndex(npp->level);
}

void UiNapChat::ui_nap_update_counter(QLabel *label, int counter)
{
    QString str = qtr(ngettext("%1 user", "%1 users", counter)).arg(counter);
    label->setText( QString("<b>%1</b>").arg(str));
}

void UiNapChat::ui_nap_channel_joined(nap_pub_t* npp)
{
    NapChatGui *data;
    FIND_NAP_CHAT_GUI(npp,data);
    if (!data) return;
    
    if (data->user_tree()) {
        NapChatUserItemModel *model = static_cast<NapChatUserItemModel *>(data->user_tree()->model());
        model->nap_item_chat_user_clean();
    }

    ui_nap_update_counter(data->label_counter(), npp->online);
    ui_nap_cp_update_op(npp);
}

void UiNapChat::ui_nap_new_topic(nap_pub_t* npp)
{
    NapChatGui *ndata;
    FIND_NAP_CHAT_GUI(npp,ndata);
    if (!ndata) return;
    ndata->ui_nap_chat_gui_update_topic(npp->topic?qfu(npp->topic):QString(""));
}

void UiNapChat::ui_nap_user_parted(nap_pub_t* npp, nap_cu_t* cu)
{
    if (!npp || !cu) return;

    if (!(pref_get_ref("/core/chat/hide", int) & HIDE_PARTS)){
        chat_print(PAGE(npp),
                   "+\0part",
                   "+\1nick", OBJ(cu)->name, "[]",
                   "+\1c_part", _("has left"),
                   "+\1c_text", "(",
                   "+\0c_text", l_printf("%d", cu->shared),
                   "+\0c_text", "/",
                   "+\0c_text", NapSpeedStrShort(cu->speed),
                   "+\0c_text", ")",
                   "+\0nl", NULL);
    }
}

void UiNapChat::ui_nap_user_joined(nap_pub_t* npp, nap_cu_t* cu)
{
    if (!(pref_get_ref("/core/chat/hide", int) & HIDE_JOINS)){
        chat_print(PAGE(npp),
                   "+\0join",
                   "+\1nick", OBJ(cu)->name, "[]",
                   "+\1c_join", _("has joined"),
                   "+\1c_text", "(",
                   "+\0c_text", l_printf("%d", cu->shared),
                   "+\0c_text", "/",
                   "+\0c_text", NapSpeedStrShort(cu->speed),
                   "+\0c_text", ")",
                   "+\0nl", NULL);
    }
}

class NapChannelItemModel : public QStandardItemModel
{
public:
    NapChannelItemModel(QObject *parent=0);
    void nap_item_channels_update(nap_ce_t* channel);
    void nap_item_channels_empty();
    void nap_item_channels_clean();
};


NapChannelItemModel::NapChannelItemModel(QObject *parent)
    : QStandardItemModel(0, 5, parent) 
{
    setHeaderData(0, Qt::Horizontal, qtr("Channel"));
    setHeaderData(1, Qt::Horizontal, qtr("Users"));
    setHeaderData(2, Qt::Horizontal, qtr("Limit"));
    setHeaderData(3, Qt::Horizontal, qtr("Level"));
    setHeaderData(4, Qt::Horizontal, qtr("Topic"));
}

void NapChannelItemModel::nap_item_channels_update(nap_ce_t* channel)
{
    QList<QStandardItem *> items;

    QStandardItem *chan;
    QStandardItem *users;
    QStandardItem *limit;
    QStandardItem *level;
    QStandardItem *topic;
    if (!channel)
    {
        return;
    }
    
    chan = new QStandardItem(qfu(channel->name)); 
    users   = new QStandardItem(qfu("%1").arg(channel->online));
    limit   = new QStandardItem(qfu("%1").arg(channel->limit));
    level   = new QStandardItem(qtr(NapLevelShort(channel->level)));
    topic   = new QStandardItem(qfu(channel->topic));
    
    chan->setData(QVariant::fromValue((void*)channel));

    items.append(chan);
    items.append(users);
    items.append(limit);
    items.append(level);
    items.append(topic);

    // Add the items
    appendRow(items);
}

void NapChannelItemModel::nap_item_channels_empty()
{
    QStandardItem *empty;   
    empty = new QStandardItem(qtr("No Entries found!")); 
    // Set default data (for right action menu)
    empty->setData(QVariant::fromValue((void*)NULL));
    appendRow(empty);
}

void NapChannelItemModel::nap_item_channels_clean()
{
    removeRows(1,rowCount());
}

ChannelsListDlg::ChannelsListDlg( nap_channels_t* channels,  QWidget *parent ) 
    : QDialog(parent), chans(channels)
{
	setupUi(this);
    setAttribute(Qt::WA_DeleteOnClose);

    char* object_name = g_strdup_printf("ChannelsListDlg_%x",this);
    this->setObjectName(QString("%1").arg(object_name));
    object_set_data(OBJ(channels), "ChannelList", object_name);

    // setup model
    model_channel = new NapChannelItemModel(this);
    channel_list->ui_treeview_set_model(model_channel);

    action_item_t* actions = action_item_search("Nap channel list actions");
    channel_list->ui_tv_handle_button( 1, &actions, channels);
}

ChannelsListDlg::~ChannelsListDlg() 
{
    object_free_data_common(OBJ(chans), "ChannelList");    
}

void ChannelsListDlg::ui_nap_channels_update(nap_channels_t* channels)
{
    char* object_name = (char*) object_get_data(OBJ(channels), "ChannelList");
    ChannelsListDlg *dlg = qobject_cast<ChannelsListDlg *> 
                    (findWidgetByName(QString("%1").arg(object_name)));
    
    if (!dlg) return;

    dlg->model_channel->nap_item_channels_clean();
    if (!channels->list && !channels->refreshing)
    {
       dlg->model_channel->nap_item_channels_empty();
    }
    else
    {
        GList* dlist;
        
        for (dlist = channels->list; dlist; dlist = dlist->next)
        {            
            nap_ce_t* channel = (nap_ce_t*) dlist->data;    
            dlg->model_channel->nap_item_channels_update(channel);
        }
    } 

    //TODO : when channels is refreshing, define a label that states it
}

void ChannelsListDlg::ui_nap_channel_list_new(nap_channels_t* channels)
{
    ChannelsListDlg *dlg = new ChannelsListDlg(channels);

    object_monitor_d(OBJ(channels), (ObjectFunc)ui_nap_channels_update, NULL);
    dlg->show();
}
