/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.11 $
 * $Date: 2007/09/11 07:36:25 $
 * $Author: sid4u $
 */
#include "lcore/qt4_ui/ui_main.hpp"
#include "ui_nap.hpp"
#include "ui_nap_hotlist.hpp"


NapUserEditorWidget::NapUserEditorWidget(QWidget *parent) 
    : BasicUserEditorWidget(parent)
{ 
    group_plugin->setTitle(QString("Opennap"));
}

void NapUserEditorWidget::ui_buddy_user_populate(buddy_t *buddy)
{
    if (!buddy) return;

    attribute_t* attr = attr_find(ATTRC(buddy), "buddy", "NapNick");
    if (attr) {
        QString str = qfu(attr_get_ptr(attr, char));
        line_users->setText(str);
    }
}

void NapUserEditorWidget::ui_buddy_user_apply(buddy_t *buddy)
{
    QString nicks = line_users->text();
    if (!nicks.isEmpty()) {
        attr_store(ATTRC(buddy), "buddy", "NapNick", qtu(nicks));
    } else {
        attribute_t* attr = attr_find(ATTRC(buddy), "buddy", "NapNick");
        if (attr) attr_remove(ATTRC(buddy), attr);
    }
}

void NapUserFuncs::ui_buddy_plugin_user_update(user_t* user, UserItem *item)
{
    nap_user_t* nap_user = NAP_USER(user);
    QMenu *menu;

    if (item->context)
        delete item->context;
    if (user->online)
    {
        item->setIcon(0, NICON(":/nap_icon.png", LOPSTER_TREEVIEW_ICON));
        NapHostlistActions *actions = new NapHostlistActions(nap_user);
        item->context = actions;
    }
    else {
        // Set Offline Icon
        item->setIcon(0, NICONG(":/nap_icon.png", LOPSTER_TREEVIEW_ICON));
        menu = new QMenu();
        QAction *action = menu->addAction(qtr("User is offline"));
        action->setEnabled(false);
        item->context = menu;
    }   
}

AbstractUserEditorWidget *NapUserFuncs::ui_buddy_plugin_edit_user()
{
    NapUserEditorWidget *editor = new NapUserEditorWidget();
    return editor;
}

NapHostlistActions::NapHostlistActions(nap_user_t* user)
    : QMenu()
{
    this->user = user;
    ui_create_chat_popup( );
    ui_create_whois_popup( );
    ui_create_browse_popup( );
}

void NapHostlistActions::on_buddy_chat(QAction *action) {
    nap_net_t* net = net_actions[action];
    if (net && user) nap_cp_new_private(net, OBJ(user)->name);
}

void NapHostlistActions::ui_create_chat_popup() {
  GList* dlist;
  
  QMenu *chat = addMenu(qtr("Chat user on"));
  for (dlist = user->nets; dlist; dlist = dlist->next) {
    nap_net_t* net = (nap_net_t*) dlist->data;
    
    QAction *action = new QAction(qfu("<%1>").arg(OBJ(net)->name), this);
    net_actions[action] = net;
    chat->addAction(action);
  }
  connect(chat, SIGNAL(triggered(QAction *)), this, SLOT(on_buddy_chat(QAction *)));
}

void NapHostlistActions::on_buddy_whois(QAction *action) {
    nap_net_t* net = net_actions[action];
    if (net && user)
        nap_whois_user(user, net, NAP_WHOIS_WINDOW, FALSE);
}

void NapHostlistActions::ui_create_whois_popup() {
  GList* dlist;
  
  QMenu *whois = addMenu(qtr("Whois user on"));
  for (dlist = user->nets; dlist; dlist = dlist->next) {
    nap_net_t* net = (nap_net_t*) dlist->data;    
    QAction *action = new QAction(qfu("<%1>").arg(OBJ(net)->name), this);
    net_actions[action] = net;
    whois->addAction(action);
  }
  connect(whois, SIGNAL(triggered(QAction *)), this, SLOT(on_buddy_whois(QAction *)));
}

void NapHostlistActions::on_buddy_browse(QAction *action) {
    nap_net_t* net = net_actions[action];
    if (net) nap_user_set_net(user, net);
    if (user) nap_browse_user(user);
}

void NapHostlistActions::ui_create_browse_popup() {
    GList* dlist;
    QMenu *browse = addMenu(qtr("Browse user on"));
    for (dlist = user->nets; dlist; dlist = dlist->next) {
        nap_net_t* net = (nap_net_t*) dlist->data;        
        QAction *action = new QAction(qfu("<%1>").arg(OBJ(net)->name), this);
        net_actions[action] = net;
        browse->addAction(action);
    }
    connect(browse, SIGNAL(triggered(QAction *)), this, SLOT(on_buddy_browse(QAction *)));
}

void UiNapHotlist::ui_nap_hotlist_init() 
{
    NapUserFuncs *funcs = new NapUserFuncs();
    BuddiesPage::ui_buddy_register_funcs(NAP_ID,funcs);
}

void UiNapHotlist::ui_nap_hotlist_cleanup()
{
}
