/* Copyright (C) 2007 The Lopster Team
 * This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.15 $
 * $Date: 2007/10/11 12:01:59 $
 * $Author: sid4u $
 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <QWidget>
#include <QDockWidget>
#include <QMessageBox>
#include <QMenu>
#include "lcore/utils.h"
#include "lcore/buffer.h"
#include "lcore/preferences.h"
#include "lcore/l_base.h"
#include "lcore/l_i18n.h"
#include "lcore/xml_handler.h"
#include "nap_network.h"
#include "nap_napi.h"
#include "nap_chat.h"
#include "nap.h"

#include "lcore/qt4_ui/ui_network.hpp"
#include "lcore/qt4_ui/ui_main.hpp"
#include "NapNetworkTreeWidget.hpp"
#include "ui_nap_net.hpp"


UiNapNetwork::UiNapNetwork(QWidget* parent)
    : QWidget(parent)
{
	setupUi(this);

	QVBoxLayout *tree_layout = new QVBoxLayout(net_tree_cnt);    
    nap_tree = NapNetworkTreeWidget::GetNetworkTree( ); 
    nap_tree->setParent(net_tree_cnt);
    tree_layout->addWidget(nap_tree);

    tool_delete_service->setIcon(QIcon(QString::fromUtf8(":/close.png")));
    tool_nap_mode->setIcon(QIcon(QString::fromUtf8(":/downarrow.png")));
    ui_nap_init_mode_popup();

    connect(tool_nap_service,SIGNAL(clicked()),this,SLOT(server_refresh()));
    tool_nap_service->setEnabled(false);
    connect(tool_delete_service,SIGNAL(clicked()),this,SLOT(service_remove()));
    tool_delete_service->setEnabled(false);
    connect(combo_nap_service,SIGNAL(currentIndexChanged(const QString&)),this,SLOT(service_changed(const QString&)));
    connect(combo_nap_service,SIGNAL(editTextChanged(const QString&)),this,SLOT(service_changed(const QString&)));   
}

UiNapNetwork::~UiNapNetwork()
{
    if (nap_mode_popup)
    {
        delete nap_mode_popup;
    }
} 

void UiNapNetwork::ui_nap_net_cleanup()
{
    UiNapNetwork* net = GetNapNetwork(); 
    NetworkPage::ui_network_remove_page(net);
    delete net;
}

UiNapNetwork *UiNapNetwork::GetNapNetwork()
{
   static UiNapNetwork* net = NULL;
   if (net == NULL)
   {
        net = new UiNapNetwork(); 
        //net->setObjectName("nap_net_content");
   }
   return net;
}

void UiNapNetwork::ui_nap_net_init() {
    NetworkPage::ui_network_add_page(NAP_ID, GetNapNetwork());
}

void UiNapNetwork::ui_nap_server_new(nap_server_t* server)
{
    NapNetworkTreeWidget::ui_nap_add_server(server);
}

void UiNapNetwork::ui_nap_net_new(nap_net_t* net)
{
    NapNetworkTreeWidget::ui_nap_add_network(net);
}

void UiNapNetwork::ui_nap_ng_new(nap_ng_t* ng)
{
    NapNetworkTreeWidget::ui_nap_add_group(ng);
}

void UiNapNetwork::service_remove() {
    int index = combo_nap_service->currentIndex();
    if (index != -1) {
        url_t* url;
        QVariant v = combo_nap_service->itemData(index);
        if ( !v.isValid())
        {
            combo_nap_service->removeItem(index);
            return;
        }
        url = (url_t*) v.value<void*>();
        object_kill(OBJ(url));
    }
}

void UiNapNetwork::service_changed(const QString &text)
{
    tool_nap_service->setEnabled(!text.isEmpty());
    tool_delete_service->setEnabled(!text.isEmpty());
}

void UiNapNetwork::server_refresh()
{
    QComboBox *combo = GetNapNetwork()->combo_nap_service;
    QString str = combo->currentText();
    if (!str.isEmpty())
    {
        nap_service_get_list(qtu(str));
    }
}

void UiNapNetwork::ui_nap_service_hide(url_t* serv)
{
    int *index = (int *) object_get_data(OBJ(serv), "NapServiceEntry");
    QComboBox *combo = GetNapNetwork()->combo_nap_service;
    combo->removeItem(*index);   
    object_free_data(OBJ(serv), "NapServiceEntry", NULL);
}

void UiNapNetwork::ui_nap_service_update(url_t* serv) {
    int *index = (int *) object_get_data(OBJ(serv), "NapServiceEntry");
    if (!index) return;
    QString str = qfu("http://%1%2").arg(serv->host).arg(serv->doc);
    QComboBox *combo = GetNapNetwork()->combo_nap_service;
    combo->insertItem(*index , str, QVariant::fromValue((void*)serv));   
}

void UiNapNetwork::ui_nap_service_show(url_t* serv) {  
    int *index = (int *) object_get_data(OBJ(serv), "NapServiceEntry");
  
    if (*index >= 0) return;
    QComboBox *combo = GetNapNetwork()->combo_nap_service;
    int count = combo->count();
    object_set_data(OBJ(serv), "NapServerEntry", &count);
    ui_nap_service_update(serv);
}

void UiNapNetwork::ui_nap_service_new(url_t* serv)
{
    Q_ASSERT(serv);
    
    object_notify_d(OBJ(serv), (ObjectFunc)ui_nap_service_hide, NULL);
    object_monitor_d(OBJ(serv), (ObjectFunc)ui_nap_service_update, NULL);
    
    ui_nap_service_show(serv);
}

void UiNapNetwork::mode(QAction *action) {
    int flag = action->data().toInt();
    int mode = pref_get_ref("/nap/usermode", int);

    if (mode & flag) mode &= ~flag;
    else mode |= flag;
    pref_store("/nap/usermode", mode);
}

void UiNapNetwork::ui_nap_init_mode_popup() {
//    QMenu *popup;
    QAction *mode;
    int cmode = pref_get_ref("/nap/usermode", int);
    int i1;
    QMenu *popup_on=NULL;
    QMenu *popup_sn=NULL;
    QMenu *popup_ng=NULL;

    nap_mode_popup = new QMenu(tool_nap_mode);
    for (i1 = 0; i1 < NAP_MODE_NUMBER; i1++) {
        int m = (1<<i1);
        if ((m & NAP_MODE_MASK_NAP)) {
            mode =  nap_mode_popup->addAction(qtr(NapUserModes(i1)));
            mode->setCheckable(true);
            if (cmode & m) {
                 mode->setChecked(true);
            }
            mode->setData(QVariant(m));
        } else {
            if ((m & NAP_MODE_MASK_ON)) {
                if (!popup_on) {
                    popup_on = nap_mode_popup->addMenu(qtr("Additional Opennap modes"));
                }
                mode = popup_on->addAction(qtr(NapUserModes(i1)));
                mode->setCheckable(true);
                if (cmode & m) {
                    mode->setChecked(true);
                }
                mode->setData(QVariant(m));
            }
            if ((m & NAP_MODE_MASK_NG)) {
                if (!popup_ng) {
                    popup_ng = nap_mode_popup->addMenu(qtr("Additional Opennap-ng modes"));
                }
                mode = popup_ng->addAction(qtr(NapUserModes(i1)));
                mode->setCheckable(true);
                if (cmode & m) {
                    mode->setChecked(true);
                }
                mode->setData(QVariant(m));
            }
            if ((m & NAP_MODE_MASK_SN)) {
                if (!popup_sn) {
                    popup_sn = nap_mode_popup->addMenu(qtr("Additional SlavaNap modes"));
                }
                mode = popup_sn->addAction(qtr(NapUserModes(i1)));
                mode->setCheckable(true);
                if (cmode & m) {
                    mode->setChecked(true);
                }
                mode->setData(QVariant(m));
            }
        } // else
    } // for
    connect(nap_mode_popup,SIGNAL(triggered(QAction*)),this,SLOT(mode(QAction*)));
    if (popup_on) {
        connect(popup_on,SIGNAL(triggered(QAction*)),this,SLOT(mode(QAction*)));
    }
    if (popup_ng) {
        connect(popup_ng,SIGNAL(triggered(QAction*)),this,SLOT(mode(QAction*)));
    }
    if (popup_sn) {
        connect(popup_sn,SIGNAL(triggered(QAction*)),this,SLOT(mode(QAction*)));
    }
    tool_nap_mode->setMenu(nap_mode_popup);
}
