/* Copyright (C) 2007 The Lopster Team This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.5 $
 * $Date: 2007/09/11 07:36:25 $
 * $Author: sid4u $
 */

#include <QMenu>
#include "lcore/l_i18n.h"

#include "lcore/qt4_ui/ui_main.hpp"
#include "ui_nap_whois.hpp"

QList<UiNapWhois*> whois_list;

UiNapWhois::UiNapWhois(QWidget *parent)
    : QDialog(parent)
{
    setupUi(this);

    //UPDATE icons
    tool_actions->setIcon(QIcon(QString::fromUtf8(":/downarrow.png")));
    tool_actions->setMenu(ui_nap_whois_actions());
    label_lastseen->setVisible(false);
    connect(push_close,SIGNAL(clicked()),this, SLOT(on_whois_close()));
}

void UiNapWhois::Update(nap_user_t* user)
{
    this->m_user = user;
    nap_whois_t* whois = (nap_whois_t*) user->whois;
    label_username->setText(qfu("<b>%1</b>").arg(OBJ(user)->name));
    
    if (whois->status == NAP_WHOIS_TIMEOUT) {
        label_userstatus->setText(qtr("timed out"));
    } else if (whois->status == NAP_WHOIS_ONLINE) {
        label_userstatus->setText(qtr("is online"));
    } else if (whois->status == NAP_WHOIS_OFFLINE) {
        label_userstatus->setText(qtr("is offline"));
    } else {
        label_userstatus->setText(qtr("is unknown"));
    }
    if (whois->status == NAP_WHOIS_ONLINE) {
        browse->setEnabled(true);
        chat->setEnabled(true);
    }
    else {
        browse->setEnabled(false);
        chat->setEnabled(false);
    }
    if (whois->timer) {
        label_situation->setText(qtr("currently refreshing on %1").
                                arg(whois->net?OBJ(whois->net)->name:"???"));
    } else if (whois->status == NAP_WHOIS_ONLINE) {
        label_situation->setText(qtr("currently on network %1").
                                arg(whois->net?OBJ(whois->net)->name:"???"));
    } else {
        label_situation->setText(qtr("currently not on network %1").
                                arg(whois->net?OBJ(whois->net)->name:"???"));
    }

    if (whois->status == NAP_WHOIS_ONLINE) {
        label_lastseen->setVisible(false);
        label_level->setText(qfu("%1 (%2)")
                        .arg(whois->str_level?whois->str_level:"?")
                        .arg(NapLevel(whois->level)));
        label_shared->setText(qtr("%1 files").arg(user->shared)); 
        label_channels->setText(qfu(whois->channel));
        label_online->setText(qfu(print_duration_unit_exact((int*)&(whois->online))));     
        label_client->setText(qfu(user->client));
        if (user_ip(user) == 0)
            label_ip->setText(qtr("unknown"));
        else
            label_ip->setText(qfu(ntoa(user_ip(user))));
        label_server->setText(whois->server?qfu(whois->server):qtr("unknown"));
        if (whois->server_port)
            label_server_port->setText(qfu("%1").arg(whois->server_port));
        else
            label_server_port->setText(qtr("unknown"));
        label_status->setText(qfu(whois->stat));
        label_connection->setText(qtr(NapSpeedStr(user->speed)));
        if (whois->dl >= 0)
            label_download->setText(QString("%1").arg(whois->dl));
        else
            label_download->setText(qtr("unknown"));
        if (whois->ul >= 0)
            label_upload->setText(QString("%1").arg(whois->ul)); 
        else
            label_upload->setText(qtr("unknown")); 
        if (whois->tdl >= 0)
            label_totaldl->setText(QString("%1").arg(whois->tdl));
        else
            label_totaldl->setText(qtr("unknown"));
        if (whois->tul >= 0)
            label_totalul->setText(QString("%1").arg(whois->tul));
        else
            label_totalul->setText(qtr("unknown"));
        label_dataport->setText(QString("%1").arg(user_port(user)));
                
    } else if (whois->status == NAP_WHOIS_OFFLINE) {
        label_lastseen->setVisible(true);
        if (whois->online > 0) {
            label_lastseen->setText(qfu("<b>Last seen: %1</b>").arg(ctime(&whois->online)));
        } else {
            label_lastseen->setText(qtr("unknown"));
        }
        label_level->setText(qfu("%1 (%2)")
                        .arg(whois->str_level?whois->str_level:"?")
                        .arg(NapLevel(whois->level)));      
        label_shared->setText(""); 
        label_channels->setText(""); 
        label_online->setText(""); 
        label_client->setText(""); 
        label_ip->setText(""); 
        label_server->setText(""); 
        label_server_port->setText(""); 
        label_status->setText(""); 
        label_connection->setText(""); 
        label_download->setText(""); 
        label_totaldl->setText(""); 
        label_totalul->setText(""); 
        label_dataport->setText(""); 
    }
}

QMenu *UiNapWhois::ui_nap_whois_actions()
{
    QMenu *menu = new QMenu(tool_actions);
    browse = new QAction(qtr("browse"),menu);
    chat = new QAction(qtr("chat"),menu);
    refresh = new QAction(qtr("refresh"),menu);
    menu->addAction(browse);
    menu->addAction(chat);
    menu->addAction(refresh);
    connect(browse,SIGNAL(triggered()),this, SLOT(on_browse()));
    connect(chat,SIGNAL(triggered()),this, SLOT(on_chat()));
    connect(refresh,SIGNAL(triggered()),this, SLOT(on_refresh()));
    return menu;
}

void UiNapWhois::on_whois_close( ) {
    if (whois_list.indexOf(this) != -1)
    {
        whois_list.removeAt(whois_list.indexOf(this));
    }
    accept();
}

void UiNapWhois::on_refresh() {
    nap_whois_user(m_user, NAP_WHOIS(m_user->whois)->net,
                 NAP_WHOIS_WINDOW, TRUE);
}

void UiNapWhois::on_chat() {
    nap_whois_t* whois = (nap_whois_t*) m_user->whois;
    nap_cp_new_private(whois->net, OBJ(m_user)->name);
}

void UiNapWhois::on_browse() {
    nap_whois_t* whois = (nap_whois_t*) m_user->whois;
    if (whois->net) nap_user_set_net(m_user, whois->net);
    nap_browse_user(m_user);
}

UiNapWhois *UiNapWhois::ui_nap_whois_search(nap_user_t* user) {
    for (int i=0; i < whois_list.size(); i++) {
        UiNapWhois *w = whois_list.at(i);
        if (w->ui_nap_whois_get_user() == user) return w;
    }
    return NULL;
}

//FIXME : find a trick to free memory
void UiNapWhois::ui_nap_whois_show(nap_user_t* user) {
    nap_whois_t* whois = (nap_whois_t*) user->whois;
    if (whois->mode & NAP_WHOIS_WINDOW) {
        UiNapWhois *w = ui_nap_whois_search(user);
        if (!w) {
            w = new UiNapWhois();
            whois_list.append(w);
        }
        w->Update(user);
        if (!w->isVisible())
        {
            w->show();
        }
    }
}
