/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.1 $
 * $Date: 2007/04/29 09:54:21 $
 * $Author: sgop $
 */

#include <string.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "lcore/chat.h"
#include "lcore/preferences.h"
#include "lcore/string_list.h"
#include "lcore/lopster.h"
#include "lcore/utils.h"
#include "lcore/action_item.h"
#include "lcore/gtk2_ui/ui_chat.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "lcore/gtk2_ui/ui_layout.h"
#include "lcore/gtk2_ui/ui_treeview.h"
#include "slsk_network.h"
#include "slsk_hotlist.h"
#include "slsk_proto.h"
#include "slsk_peer.h"
#include "slsk.h"
#include "slsk_chat.h"
#include "slsk_users.h"
#include "ui_slsk_chat.h"
#include "ui_slsk.h"

typedef struct {
  int width;
  GtkWidget* widget;
} slsk_ticker_gui_t;

// thats the gui data for a public page
// server, motd, private, other pages simply use chat_gui_t
typedef struct {
  chat_gui_t _data;
  GtkListStore* online_store;
  GtkFixed* ticker_box;
  GtkSpinButton* ticker_spin1;
  GtkSpinButton* ticker_spin2;
  GList* ticker_list;
  GtkTreeViewColumn* col[3];
  int timer;
  int ticker_pos;
  int ticker_delay;
  int ticker_move;
  gboolean ticker_resize;
} slsk_public_gui_t;

typedef struct {
  chat_gui_t _data;
  GtkWidget* status;
  int istatus;
  GtkWidget* action[2];
  slsk_user_t* user;
} slsk_priv_gui_t;

typedef struct {
  slsk_user_t* user;
} slsk_cu_gui_t;

/*
static gboolean
ui_slsk_chat_print(GtkTextBuffer* buffer, GtkTextIter* iter,
		   const char* tag, va_list* ap) {
  return FALSE;
}
*/

void ui_slsk_chat_init() {
  static char called = 0;

  if (!called) {
    //    ui_chat_register_print_func(ui_slsk_chat_print);
    called = 1;
  }
  return;

}

//////////////////////////////////////////////////////////////

static void ui_slsk_user_hide(slsk_user_t* cu, slsk_cp_t* scp);
static void ui_slsk_ticker_hide(slsk_ct_t* ticker, slsk_cp_t* scp);

static void ui_slsk_public_remove(chat_page_t* page)
{
    slsk_public_gui_t* ndata = object_get_data(OBJ(page), "ChatPage");
    chat_gui_t* data = (chat_gui_t*)ndata;
    GList* dlist;
    
    // we need to remove all users gui_data
    for (dlist = SCP(page)->users; dlist; dlist = dlist->next)
    {
        slsk_user_t* user = dlist->data;
        object_free_data_common(OBJ(user), l_printf("ChatUser_%x", page));
    }

    g_list_foreach(SCP(page)->ticker, (GFunc)ui_slsk_ticker_hide, page); //slow
    
    
    if (!data) return;
    ui_layout_unmonitor("slsk:chat:col:user:size", ndata->col[0]);
    ui_layout_unmonitor("slsk:chat:col:user:vis", ndata->col[0]);
    ui_layout_unmonitor("slsk:chat:col:shared:size", ndata->col[1]);
    ui_layout_unmonitor("slsk:chat:col:shared:vis", ndata->col[1]);
    ui_layout_unmonitor("slsk:chat:col:speed:size", ndata->col[2]);
    ui_layout_unmonitor("slsk:chat:col:speed:vis", ndata->col[2]);
    ui_layout_unmonitor("slsk:chat:paned:pos", data->toplevel);
    ui_layout_unmonitor("slsk:chat:ticker:delay", ndata->ticker_spin1);
    ui_layout_unmonitor("slsk:chat:ticker:move", ndata->ticker_spin2);
    ui_layout_unmonitor("slsk:chat:ticker:vis", ndata->ticker_box);
    
    ui_chat_page_remove(page);
}

static void fixed_size_request(GtkFixed* fixed, GtkRequisition* requisition,
                               slsk_public_gui_t* data) {
  GtkFixedChild *child;
  GList* children;
  GtkRequisition child_requisition;

  requisition->width = 0;
  requisition->height = 0;

  if (data->ticker_resize || TRUE) {
    children = fixed->children;
    while (children) {
      child = children->data;
      children = children->next;
      
      if (GTK_WIDGET_VISIBLE (child->widget)) {
	gtk_widget_size_request(child->widget, &child_requisition);
	
	requisition->height = MAX(requisition->height,
				  child_requisition.height);
      }
    }
    data->ticker_resize = FALSE;
  } else {
    requisition->width = GTK_WIDGET(fixed)->requisition.width;
    requisition->height = GTK_WIDGET(fixed)->requisition.height;
  }

  requisition->height += GTK_CONTAINER(fixed)->border_width * 2;
  requisition->width += GTK_CONTAINER(fixed)->border_width * 2;
}

static gboolean ticker_timer(slsk_public_gui_t* data);
static void slsk_ticker_delay_changed(GtkSpinButton* spin, 
                                      slsk_public_gui_t* ndata) {

  g_assert(spin && ndata && ndata->ticker_spin1);
  ndata->ticker_delay = gtk_spin_button_get_value_as_int(spin);
  if (ndata->ticker_delay < 1) ndata->ticker_delay = 1;

  if (ndata->timer) {
    g_source_remove(ndata->timer);
    ndata->timer = 
      g_timeout_add(ndata->ticker_delay, (GSourceFunc)ticker_timer, ndata);
  }
}

static void slsk_ticker_move_changed(GtkSpinButton* spin, 
                                     slsk_public_gui_t* ndata) {

  g_assert(spin && ndata && ndata->ticker_spin2);
  ndata->ticker_move = gtk_spin_button_get_value_as_int(spin);
  if (ndata->ticker_move < 1) ndata->ticker_move = 1;
}

static void on_ticker_set(GtkEntry* entry, slsk_cp_t* page) {
  slsk_set_ticker(page, gtk_entry_get_text(entry));
  gtk_entry_set_text(entry, "");
}

static void on_encoding_changed(const char* charset, slsk_cp_t* page) {
  slsk_cp_set_encoding(page, charset);
}

void ui_slsk_create_public_page(slsk_cp_t* page)
{
    GtkWidget* toplevel;
    GtkWidget* text;
    slsk_net_t* net;
    slsk_public_gui_t* ndata;
    chat_gui_t* data;
    GtkWidget* sw;
    GtkListStore* store;
    GtkWidget* tv;
    GtkWidget* fixed;
    GtkWidget* hbox;
    GtkWidget* label;
    GtkWidget* spin;
    GtkTreeViewColumn* column;
    GtkTreeSelection *select;
    GtkWidget* entry;
    
    ndata = g_malloc0(sizeof(slsk_public_gui_t));
    data = (chat_gui_t*)ndata;
    ndata->ticker_delay = 250;
    ndata->ticker_move = 5;
    
    object_set_data(OBJ(page), "ChatPage", ndata);
    net = page->net;
    
    toplevel = gtk_hpaned_new();
    gtk_widget_show(toplevel);
    data->toplevel = toplevel;
    ui_layout_monitor("slsk:chat:paned:pos", toplevel);
    
    data->no_buffers = 1;
    data->buffer = g_malloc0(sizeof(GtkTextBuffer)*data->no_buffers);
    data->view = g_malloc0(sizeof(GtkTextView)*data->no_buffers);
    
    // CHAT AREA
    sw = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(sw);
    gtk_paned_pack1(GTK_PANED(toplevel), sw, TRUE, TRUE);
    
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
    
    data->buffer[0] = ui_chat_new_buffer();
    text = ui_chat_new_text_view(data->buffer[0]);
    gtk_widget_show(text);
    gtk_container_add(GTK_CONTAINER(sw), text);
    data->view[0] = GTK_TEXT_VIEW(text);
    
    // USER LIST
    sw = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(sw);
    gtk_paned_pack2(GTK_PANED(toplevel), sw, FALSE, TRUE);
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
    
    store = gtk_list_store_new (5,
                                G_TYPE_POINTER,  // pointer to page
                                G_TYPE_STRING,
                                G_TYPE_INT,
                                G_TYPE_STRING,
                                G_TYPE_OBJECT); // pixmap
    ndata->online_store = store;
    
    tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL (store));
    //  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
    gtk_container_add (GTK_CONTAINER(sw), tv);
    gtk_widget_show(tv);
    gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tv), TRUE);
    //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);
    
    select = gtk_tree_view_get_selection(GTK_TREE_VIEW (tv));
    gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
    
    ndata->col[0] = column = ui_tv_column_new
        ("User",
         gtk_cell_renderer_pixbuf_new(), "pixbuf", 4, FALSE,
         gtk_cell_renderer_text_new(), "text", 1, TRUE,
         NULL);
    ui_tv_config_default_col(column, 1, "slsk:chat:col:user", FALSE);
    gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
    
    ndata->col[1] = column = ui_tv_column_new
        ("Shared", gtk_cell_renderer_text_new(), "text", 2, TRUE, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    ui_tv_config_default_col(column, 2, "slsk:chat:col:shared", FALSE);
    
    ndata->col[2] = column = ui_tv_column_new
        ("Speed", gtk_cell_renderer_text_new(), "text", 3, TRUE, NULL);
    gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
    ui_tv_config_default_col(column, 3, "slsk:chat:col:speed", TRUE);
    
    {
        action_item_t* actions = 
            action_item_search("Online slsk users actions");
        ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &actions, page);
    }
    
    
    fixed = gtk_fixed_new();
    gtk_widget_show(fixed);
    ndata->ticker_box = GTK_FIXED(fixed);
    ui_layout_monitor("slsk:chat:ticker:vis", fixed);
    
    g_signal_connect(G_OBJECT(fixed), "size-request",
                     G_CALLBACK(fixed_size_request), ndata);
    
    
    hbox = gtk_hbox_new(FALSE, 3);
    gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
    
    label = gtk_label_new("Ticker move delay (ms)");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    
    spin = gtk_spin_button_new_with_range(1, 100000, 1);
    gtk_widget_show(spin);
    gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
    ndata->ticker_spin1 = GTK_SPIN_BUTTON(spin);
    gtk_spin_button_set_value(ndata->ticker_spin1, ndata->ticker_delay);
    
    g_signal_connect(G_OBJECT(spin), "value-changed", 
                     G_CALLBACK(slsk_ticker_delay_changed), ndata);
    ui_layout_monitor("slsk:chat:ticker:delay", spin);
    
    label = gtk_label_new("Ticker move width (px)");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    
    spin = gtk_spin_button_new_with_range(1, 20, 1);
    gtk_widget_show(spin);
    gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, FALSE, 0);
    ndata->ticker_spin2 = GTK_SPIN_BUTTON(spin);
    gtk_spin_button_set_value(ndata->ticker_spin2, ndata->ticker_move);
    
    g_signal_connect(G_OBJECT(spin), "value-changed", 
                     G_CALLBACK(slsk_ticker_move_changed), ndata);
    ui_layout_monitor("slsk:chat:ticker:move", spin);
    
    label = gtk_label_new("Your ticker");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    
    entry = gtk_entry_new();
    gtk_widget_show(entry);
    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
    g_signal_connect(G_OBJECT(entry), "activate",
                     G_CALLBACK(on_ticker_set), page);
    
    
    entry = ui_create_charset_control(page->charset, (CharsetFunc)on_encoding_changed, page);
    gtk_widget_show(entry);
    gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, FALSE, 0);
    
    ui_chat_page_add(PAGE(page), 
                     hbox, fixed,
                     ui_create_show_hide_label("show ticker", "hide ticker", fixed),
                     ui_create_show_hide_label("show users", "hide users", sw),
                     NULL
                    );
    object_notify_d(OBJ(page), (ObjectFunc)ui_slsk_public_remove, NULL);
}


static void ui_slsk_user_update(slsk_user_t* cu, slsk_cp_t* scp)
{
    slsk_public_gui_t* data = object_get_data(OBJ(scp), "ChatPage");
    GtkTreeIter* iter = object_get_data(OBJ(cu), l_printf("ChatUser_%x", scp));
    
    if (!data || !iter || !data->online_store) return;
    
    gtk_list_store_set(data->online_store, iter,
                       0, cu,
                       1, OBJ(cu)->name,
                       2, cu->shared,
                       3, print_speed(&(cu->speed)),
                       4, ui_slsk_get_status_pixmap(cu),
                       -1);
}

static void ui_slsk_user_hide(slsk_user_t* cu, slsk_cp_t* scp)
{
    slsk_public_gui_t* data = object_get_data(OBJ(scp), "ChatPage");
    GtkTreeIter* iter = object_get_data(OBJ(cu), l_printf("ChatUser_%x", scp));
    
    if (iter && data && data->online_store)
    {
        object_unnotify_o(OBJ(cu), (ObjectFunc)ui_slsk_user_hide, OBJ(scp));
        object_unmonitor_o(OBJ(cu), (ObjectFunc)ui_slsk_user_update, OBJ(scp));
        
        gtk_list_store_remove(data->online_store, iter);
        object_free_data_common(OBJ(cu), l_printf("ChatUser_%x", scp));
    }
}


void ui_slsk_user_show(slsk_cp_t* scp, slsk_user_t* user)
{
    slsk_public_gui_t* data;
    GtkTreeIter* iter;
    
    if (object_get_data(OBJ(user), l_printf("ChatUser_%x", scp))) return;

    data = object_get_data(OBJ(scp), "ChatPage");
    if (!data || !data->online_store) return;

    object_set_data(OBJ(user),l_printf("ChatUser_%x", scp), 
                    iter = g_malloc(sizeof(*iter)));
    gtk_list_store_append(data->online_store, iter);
    
    object_notify_o(OBJ(user), (ObjectFunc)ui_slsk_user_hide, OBJ(scp));
    object_monitor_o(OBJ(user), (ObjectFunc)ui_slsk_user_update, OBJ(scp));
    ui_slsk_user_update(user, scp);
}

void ui_slsk_user_joined(slsk_cp_t* scp, slsk_user_t* user) {

  if (!scp) return;

  if (!(pref_get_ref("/core/chat/hide", int) & HIDE_JOINS)) {
    chat_print(PAGE(scp), "+\0join",
               "+\1nick", OBJ(user)->name, "[]",
               "+\1c_join", "has joined",
               "+\1c_text", "(",
               "+\0c_text", l_printf("%d", user->shared),
               "+\3c_text", "files -",
               "+\0c_text", print_speed(&(user->speed)),
               "+\0c_text", ")",
               "+\0nl", NULL);
  }
}

void ui_slsk_user_parted(slsk_cp_t* scp, slsk_user_t* user) {

  if (!scp) return;
  
  if (!(pref_get_ref("/core/chat/hide", int) & HIDE_PARTS)) {
    chat_print(PAGE(scp), "+\0part",
               "+\1nick", OBJ(user)->name, "[]",
               "+\1c_part", "has left",
               "+\1c_text", "(",
               "+\0c_text", l_printf("%d", user->shared),
               "+\3c_text", "files -",
               "+\0c_text", print_speed(&(user->speed)),
               "+\0c_text", ")",
               "+\0nl", NULL);
  }
  ui_slsk_user_hide(user, scp);
}

void ui_slsk_channel_joined(slsk_cp_t* scp)
{
    slsk_public_gui_t* data = object_get_data(OBJ(scp), "ChatPage");

    if (data && data->online_store) 
        gtk_list_store_clear(data->online_store);
}

static void ui_slsk_ticker_hide(slsk_ct_t* ticker, slsk_cp_t* scp)
{
    slsk_public_gui_t* data = object_get_data(OBJ(scp), "ChatPage");
    slsk_ticker_gui_t* tdata = object_get_data(OBJ(ticker), "Ticker");
    
    if (!data || !data->ticker_box) return;
    if (!tdata) return;

    data->ticker_resize = TRUE;
    
    gtk_widget_destroy(tdata->widget);
    object_free_data_common(OBJ(ticker), "Ticker");
    
    data->ticker_list = g_list_remove(data->ticker_list, ticker);
    if (!data->ticker_list && data->timer)
    {
        g_source_remove(data->timer);
        data->timer = 0;
    }
}

static gboolean ticker_timer(slsk_public_gui_t* data)
{
    GList* dlist;
    slsk_ct_t* ticker;
    slsk_ticker_gui_t* tdata;
    int pos;
    
#define TSPACE 15
    if (!data->ticker_list) return TRUE;
    
    data->ticker_pos -= data->ticker_move;
    ticker = data->ticker_list->data;
    tdata = object_get_data(OBJ(ticker), "Ticker");

    if (data->ticker_pos < -tdata->width)
    {
        // this ticker is outside visible range, append to right side
        data->ticker_pos += tdata->width;
        data->ticker_pos += TSPACE;
        data->ticker_list = g_list_remove(data->ticker_list, ticker);
        data->ticker_list = g_list_append(data->ticker_list, ticker);
    }

    pos = data->ticker_pos;
    for (dlist = data->ticker_list; dlist; dlist = dlist->next)
    {
        ticker = dlist->data;
        tdata = object_get_data(OBJ(ticker), "Ticker");
        gtk_fixed_move(data->ticker_box, tdata->widget, pos, 0);
        pos += tdata->width;
        pos += TSPACE;
        // we should break, if we leave visible area
    }
#undef TSPACE
    return TRUE;
}

static void ui_slsk_ticker_update(slsk_ct_t* ticker, GtkLabel* label)
{
    slsk_ticker_gui_t* tdata = object_get_data(OBJ(ticker), "Ticker");
    
    gtk_label_set_text(label, ticker->message?ticker->message:"");
    if (tdata)
    {
        GtkRequisition req;
        gtk_widget_size_request(tdata->widget, &req);
        tdata->width = req.width;
    }
}


void ui_slsk_ticker_show(slsk_cp_t* scp, slsk_ct_t* ticker)
{
    slsk_public_gui_t* data;
    GtkWidget* label;
    GtkWidget* hbox;
    slsk_ticker_gui_t* tdata;
    GtkRequisition req;
    
    g_assert(scp && ticker);
    
    if (object_get_data(OBJ(ticker), "Ticker")) return;

    data = object_get_data(OBJ(scp), "ChatPage");
    if (!data || !data->ticker_box) return;
    data->ticker_resize = TRUE;

    hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_ref(hbox);
    gtk_widget_show(hbox);
    object_set_data(OBJ(ticker), "Ticker", tdata = g_malloc0(sizeof(*tdata)));
    tdata->widget = hbox;

    {
        char* text = g_markup_printf_escaped("<b>%s:</b>", OBJ(ticker)->name);
        label = gtk_label_new(text);
        g_free(text);
    }
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
    gtk_label_set_selectable(GTK_LABEL(label), TRUE);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    
    gtk_fixed_put(data->ticker_box, hbox, 0, 100);
    
    label = gtk_label_new(ticker->message?ticker->message:"");
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    //  gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
    gtk_label_set_selectable(GTK_LABEL(label), TRUE);
    //  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
    gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
    
    gtk_widget_size_request(hbox, &req);
    tdata->width = req.width;
    
    if (!data->timer && !data->ticker_list)
    {
        data->timer = g_timeout_add(data->ticker_delay, (GSourceFunc)ticker_timer, data);
    }
    data->ticker_list = g_list_append(data->ticker_list, ticker);
    
    object_notify_o(OBJ(ticker), (ObjectFunc)ui_slsk_ticker_hide, OBJ(scp));
    object_monitor_d(OBJ(ticker), (ObjectFunc)ui_slsk_ticker_update, label);
}

static void on_user_changed(slsk_user_t* user, slsk_cp_t* scp)
{
    slsk_priv_gui_t* data = object_get_data(OBJ(scp), "ChatPage");
    GtkWidget* pix;
    gboolean online;
    
    if (!data) return;
    
    online = slsk_user_online(user);
    if (data->istatus != user->status)
    {
        data->istatus = user->status;
        pix = create_pixmap
            (!online?"slsk_offline.gif":
             (user->status == SLSK_USER_ONLINE)?"slsk_online.gif":
             (user->status == SLSK_USER_AWAY)?"slsk_away.gif":"slsk_offline.gif");
        gtk_widget_show(pix);
        gtk_widget_destroy(gtk_bin_get_child(GTK_BIN(data->status)));
        gtk_container_add(GTK_CONTAINER(data->status), pix);
    }
    gtk_widget_set_sensitive(data->action[0], online);
    gtk_widget_set_sensitive(data->action[1], online);
}

void ui_slsk_create_private_page(slsk_cp_t* scp)
{
    GtkWidget* sw;
    GtkWidget* text;
    slsk_priv_gui_t* data;
    chat_gui_t* data2;
    slsk_user_t* user;
    GtkWidget* pix;
    GtkWidget* frame;
    
    data = g_malloc0(sizeof(slsk_priv_gui_t));
    data2 = (chat_gui_t*)data;
    object_set_data(OBJ(scp), "ChatPage", data);
    
    sw = gtk_scrolled_window_new(NULL, NULL);
    gtk_widget_show(sw);
    gtk_scrolled_window_set_policy
        (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_scrolled_window_set_shadow_type
        (GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);
    data2->toplevel = sw;
    
    data2->no_buffers = 1;
    data2->buffer = g_malloc0(sizeof(GtkTextBuffer)*data2->no_buffers);
    data2->view = g_malloc0(sizeof(GtkTextView)*data2->no_buffers);
    
    data2->buffer[0] = ui_chat_new_buffer();
    text = ui_chat_new_text_view(data2->buffer[0]);
    gtk_widget_show(text);
    //  gtk_text_view_set_indent(GTK_TEXT_VIEW(text), -50);
    
    gtk_container_add(GTK_CONTAINER(sw), text);
    data2->view[0] = GTK_TEXT_VIEW(text);
    gtk_text_view_set_wrap_mode(data2->view[0], GTK_WRAP_WORD);
    gtk_text_view_set_editable(data2->view[0], FALSE);
    gtk_text_view_set_cursor_visible(data2->view[0], FALSE);
    
    data->user = user = slsk_user_find(OBJ(scp)->name);
    if (!user->net) slsk_user_set_net(user, scp->net);
    
    data->istatus = user->status;
    pix = create_pixmap
        ((user->status == SLSK_USER_ONLINE)?"slsk_online.gif":
         (user->status == SLSK_USER_AWAY)?"slsk_away.gif":"slsk_offline.gif");
    gtk_widget_show(pix);
    frame = gtk_frame_new(NULL);
    gtk_widget_show(frame);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);
    gtk_container_add(GTK_CONTAINER(frame), pix);
    data->status = frame;
    
    data->action[0] = 
        ui_create_event_label("browse", (GCallback)slsk_browse_user, user, TRUE),
        data->action[1] = 
        ui_create_event_label("whois", (GCallback)slsk_peer_whois, user, TRUE);
    ui_chat_page_add
        (PAGE(scp),
         NULL, NULL,
         data->action[0], data->action[1],
         frame,
         NULL
        );
    
    object_monitor_o(OBJ(user), (ObjectFunc)on_user_changed, OBJ(scp));
    object_notify_d(OBJ(scp), (ObjectFunc)ui_chat_page_remove, NULL);
}
