/* Copyright (C) 2005 sgop@users.sourceforge.net This is free software
 * distributed under the terms of the GNU Public License.  See the
 * file COPYING for details.
 */
/* $Revision: 1.1 $
 * $Date: 2007/04/29 09:54:21 $
 * $Author: sgop $
 */

#include <string.h>

#include <gtk/gtk.h>

#include "lcore/buddy.h"
#include "lcore/attribute.h"
#include "lcore/l_i18n.h"
#include "lcore/gtk2_ui/ui_buddy.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "slsk.h"
#include "slsk_hotlist.h"
#include "slsk_chat.h"
#include "slsk_browse.h"
#include "slsk_peer.h"

static GtkEntry* NickEntry = NULL;

static GtkWidget* ui_slsk_buddy_edit(buddy_t* buddy) {
  GtkWidget* label;
  GtkWidget* table;
  GtkWidget* entry;
  attribute_t* attr;

  table = gtk_table_new(2, 1, FALSE);
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);

  label = gtk_label_new("monitor nick");
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
		   GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);

  entry = gtk_entry_new();
  gtk_table_attach(GTK_TABLE(table), entry, 0, 1, 1, 2,
		   GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);
  NickEntry = GTK_ENTRY(entry);
  attr = attr_find(ATTRC(buddy), "buddy", "SlskNick");
  if (attr) gtk_entry_set_text(NickEntry, attr_get_ptr(attr, char));
  g_signal_connect(G_OBJECT(entry), "destroy",
		   G_CALLBACK(gtk_widget_destroyed), &NickEntry);
		   
  gtk_widget_show_all(table);
  return table;
}

static void ui_slsk_buddy_apply(buddy_t* buddy) {
  const char* nick;

  if (!buddy || !NickEntry) return;

  nick = gtk_entry_get_text(NickEntry);

  if (nick && *nick) {
    attr_store(ATTRC(buddy), "buddy", "SlskNick", nick);
  } else {
    attribute_t* attr = attr_find(ATTRC(buddy), "buddy", "SlskNick");
    if (attr) attr_remove(ATTRC(buddy), attr);
  }
}

static gboolean on_buddy_chat(slsk_user_t* user) {
  if (!user->net) return FALSE;
  slsk_cp_new_private(user->net, OBJ(user)->name);
  return TRUE;
}

static GtkWidget* ui_slsk_buddy_show(buddy_t* buddy) {
  GtkWidget* table;
  GtkWidget* item;
  GtkWidget* hbox;
  gboolean online;
  slsk_user_t* user;

  g_assert(buddy);
  user = slsk_hot_get_user(buddy);
  if (!user) return NULL;

  online = slsk_user_online(user);

  table = gtk_table_new(1, 2, FALSE);
  gtk_widget_show(table);
/*   gtk_container_set_border_width(GTK_CONTAINER(table), 5); */
  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
  gtk_table_set_row_spacings(GTK_TABLE(table), 2);

  item = gtk_label_new(OBJ(user)->name);
  gtk_widget_set_name(item, "OnlineLabel");
  if (online)
    gtk_widget_set_state(item, GTK_STATE_ACTIVE);
  else
    gtk_widget_set_state(item, GTK_STATE_NORMAL);

  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_widget_show(item);
  gtk_table_attach(GTK_TABLE(table), item, 0, 1, 0, 1,
                   GTK_FILL, GTK_FILL, 0, 0);
  
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 0, 1,
                   GTK_FILL, GTK_FILL, 0, 0);
  
  if (user->status == SLSK_USER_UNKNOWN)
    item = gtk_label_new(Q_("slskhot|is unknown"));
  else if (user->status == SLSK_USER_OFFLINE)
    item = gtk_label_new(Q_("slskhot|is offline"));
  else
    item = gtk_label_new(Q_("slskhot|is online"));
  
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  
  item = create_pixmap
    (!online?"slsk_offline.gif":
     (user->status == SLSK_USER_ONLINE)?"slsk_online.gif":
     (user->status == SLSK_USER_AWAY)?"slsk_away.gif":
     "slsk_offline.gif");
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  if (online) {
    char* l = l_printf(ngettext("browse %d file", "browse %d files", user->shared), user->shared);

    item = ui_create_event_label
      (_("chat"), G_CALLBACK(on_buddy_chat), user, TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    
    item = ui_create_event_label
      (_("whois"), G_CALLBACK(slsk_peer_whois), user, TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
    
    item = ui_create_event_label(l, G_CALLBACK(slsk_browse_user), user, TRUE);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  } else if (user->status != SLSK_USER_UNKNOWN) {
    char* l = l_printf(ngettext("%d file", "%d files", user->shared), user->shared);

    item = gtk_label_new(l_printf("(%s)", l));
    gtk_widget_show(item);
    gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  }
  
  
  return table;
}

void ui_slsk_hotlist_init() {
  ui_buddy_register_funcs(SLSK_ID, ui_slsk_buddy_edit,
                          ui_slsk_buddy_apply,
                          ui_slsk_buddy_show
                          );
}
