/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.1 $
 * $Date: 2007/04/29 09:54:21 $
 * $Author: sgop $
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "lcore/preferences.h"
#include "lcore/string_list.h"
#include "lcore/l_i18n.h"
#include "lcore/gtk2_ui/ui_network.h"
#include "lcore/gtk2_ui/ui_main.h"
#include "lcore/gtk2_ui/ui_layout.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_treeview.h"
#include "slsk_network.h"
#include "slsk_chat.h"
#include "slsk_til.h"
#include "slsk.h"
#include "ui_slsk.h"

static GtkListStore* ChannelStore = NULL;
static GtkListStore* LikeUserStore = NULL;
static GtkListStore* LikeRecoStore = NULL;
static slsk_net_t* Net = NULL;
static GtkWidget* GLabel1 = NULL;
static GtkWidget* GLabel2 = NULL;
static GtkWidget* GLabel3 = NULL;
static GtkWidget* CLabel = NULL;
static GtkLabel* ServerLabel = NULL;
static GtkLabel* StatusLabel = NULL;
static GtkLabel* PrivilegesLabel = NULL;
static GtkWidget* PrivLabel = NULL;
static GtkWidget* ConnectLabel = NULL;

typedef struct {
  slsk_user_t* user;
  GtkTreeIter iter;
} ui_slsk_similar_t;

void ui_slsk_net_show(slsk_net_t* net);

void ui_slsk_show_channels(slsk_net_t* net) {
  GList* dlist;
  GtkTreeIter iter;
  int cnt = 0;

  if (!net) return;
  
  gtk_list_store_clear(ChannelStore);
  for (dlist = net->channels; dlist; dlist = dlist->next) {
    slsk_channel_t* channel = dlist->data;
    
    gtk_list_store_append(ChannelStore, &iter);
    gtk_list_store_set(ChannelStore, &iter,
		       0, channel,
		       1, channel->name,
		       2, channel->online,
		       -1);
    cnt++;
  }
  gtk_label_set_markup(GTK_LABEL(CLabel), l_printf("<b>%d</b>", cnt));
}

static void ui_slsk_priv_observer(slsk_net_t* net, GtkLabel* label) {
  int days = net->privileges;
  
  gtk_label_set_text(label, days<0?_("unknown"):
                     days==0?_("no privileges"):
                     days==1?_("1 day"):
                     l_printf(_("%d days"), days));

  if (!net) return;

  gtk_label_set_text(StatusLabel, net->status_message?net->status_message:_("Unknown"));

  if (net->status == SLSK_NET_OFFLINE) {
    ui_event_label_set_text(ConnectLabel, _("connect"));
  } else {
    ui_event_label_set_text(ConnectLabel, _("disconnect"));
  }
  if (net->status != SLSK_NET_ONLINE) {
    ui_event_label_set_sensitive(PrivLabel, FALSE);
    ui_event_label_set_sensitive(GLabel1, FALSE);
    ui_event_label_set_sensitive(GLabel2, FALSE);
    ui_event_label_set_sensitive(GLabel3, FALSE);
  } else {
    ui_event_label_set_sensitive(PrivLabel, TRUE);
    ui_event_label_set_sensitive(GLabel1, TRUE);
    ui_event_label_set_sensitive(GLabel2, TRUE);
    ui_event_label_set_sensitive(GLabel3, TRUE);
  }
  
}

static void ui_slsk_show_pending_nets() {
/*   GList* dlist; */
  
/*   for (dlist = PendingNets; dlist; dlist = dlist->next) { */
/*     slsk_net_t* net = dlist->data; */
/*     ui_slsk_net_show(net); */
/*   } */
}

static void ui_slsk_net_connect() {
  if (!Net) return;
  if (Net->status == SLSK_NET_OFFLINE)
    slsk_net_connect(Net);
  else
    slsk_net_disconnect(Net);
}

static void on_net_check_privileges() {
  if (Net) slsk_net_check_privileges(Net);
}

static void ui_slsk_create_net_page(void) {
  GtkWidget* vbox0;
  GtkWidget* vbox;
  GtkWidget* vbox2;
  GtkWidget* hbox2;
  GtkWidget* paned;
  GtkWidget* hbox0;
  GtkWidget* hbox;
  GtkWidget* item;
  GtkWidget* sw;
  GtkListStore* store;
  GtkWidget* tv;
  GtkTreeViewColumn* column;
  GtkTreeSelection *select;

  ///////////////
  vbox0 = gtk_vbox_new(FALSE, 5);
  gtk_widget_show(vbox0);
  //  gtk_container_set_border_width(GTK_CONTAINER(paned), 5);

  paned = gtk_hpaned_new();
  gtk_widget_show(paned);
  ui_layout_monitor("slsk:net:paned:pos", paned);
  gtk_box_pack_start(GTK_BOX(vbox0), paned, TRUE, TRUE, 0);


  // left side
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_widget_show(vbox);
  gtk_paned_pack1(GTK_PANED(paned), vbox, TRUE, TRUE);
  
  hbox = gtk_hbox_new(FALSE, 20);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

/*   item = ui_create_header_label(_("Server")); */
/*   gtk_widget_show(item); */
/*   gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0); */

  hbox2 = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox2);
  gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 0);

  item = gtk_label_new(_("<b>Address:</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

  item = gtk_label_new(_("not set"));
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
  ServerLabel = GTK_LABEL(item);
  
  ConnectLabel = item = ui_create_event_label
    (_("connect"), G_CALLBACK(ui_slsk_net_connect), NULL, TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
  

  hbox2 = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox2);
  gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 0);

  item = gtk_label_new(_("<b>Status:</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

  item = gtk_label_new(_("offline"));
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
  StatusLabel = GTK_LABEL(item);
  

  hbox2 = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox2);
  gtk_box_pack_start(GTK_BOX(hbox), hbox2, FALSE, FALSE, 0);

  item = gtk_label_new(_("<b>Privileges:</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);

  item = gtk_label_new(_("unknown"));
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
  PrivilegesLabel = GTK_LABEL(item);

  PrivLabel = item = ui_create_event_label
    (_("check"), G_CALLBACK(on_net_check_privileges), NULL, TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox2), item, FALSE, FALSE, 0);
  ui_event_label_set_sensitive(item, FALSE);


  item = gtk_hseparator_new();
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(vbox), item, FALSE, FALSE, 5);


  hbox0 = gtk_hbox_new(TRUE, 5);
  gtk_widget_show(hbox0);
  gtk_box_pack_start(GTK_BOX(vbox), hbox0, TRUE, TRUE, 0);

  vbox2 = gtk_vbox_new(TRUE, 10);
  gtk_widget_show(vbox2);
  gtk_box_pack_start(GTK_BOX(hbox0), vbox2, TRUE, TRUE, 0);
  
  // like
  item = ui_string_list_show
    (GTK_BOX(vbox2), "slsk:til", "Soulseek til actions", FALSE);
  gtk_widget_show(item);
  // dislike
  item = ui_string_list_show
    (GTK_BOX(vbox2), "slsk:tih", "Soulseek tih actions", FALSE);
  gtk_widget_show(item);


  // recommendations
  vbox2 = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox2);
  gtk_box_pack_start(GTK_BOX(hbox0), vbox2, TRUE, TRUE, 0);
  
  item = gtk_label_new(_("<b>Recommendations</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
/*   item = ui_create_header_label("Recommendations"); */
  gtk_widget_show(item);
  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);
  
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(sw);
  gtk_box_pack_start(GTK_BOX(vbox2), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);

  store = gtk_list_store_new(2,
                             G_TYPE_STRING,
                             G_TYPE_INT);
  LikeRecoStore = store;

  tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
  //  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
  gtk_container_add(GTK_CONTAINER(sw), tv);
  gtk_widget_show(tv);
  gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tv), TRUE);
  //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);

  select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
  gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);

  column = ui_tv_column_new
    ("Recommended", gtk_cell_renderer_text_new(), "text", 0, TRUE, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
  ui_tv_config_default_col(column, 0, "slsk:net:col:reco", FALSE);

  column = ui_tv_column_new
    ("Rating", gtk_cell_renderer_text_new(), "text", 1, TRUE, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
  ui_tv_config_default_col(column, 1, "slsk:net:col:rating", TRUE);

  {
    action_item_t* actions =
      action_item_search("Soulseek til similar actions");
    ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &actions, NULL);
  }

  // similar users
  vbox2 = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox2);
  gtk_box_pack_start(GTK_BOX(hbox0), vbox2, TRUE, TRUE, 0);

  item = gtk_label_new(_("<b>Similar Users</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
/*   item = ui_create_header_label("Similar Users"); */
  gtk_widget_show(item);
  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox2), item, FALSE, FALSE, 0);
  
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(sw);
  gtk_box_pack_start(GTK_BOX(vbox2), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);

  store = gtk_list_store_new(5,
                             G_TYPE_POINTER,  // slsk_user_t*
                             G_TYPE_STRING,
                             G_TYPE_INT,
                             G_TYPE_OBJECT,   // pixmap
                             G_TYPE_POINTER); // ui_slsk_similar_t*
  LikeUserStore = store;

  tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
  //  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
  gtk_container_add (GTK_CONTAINER(sw), tv);
  gtk_widget_show(tv);
  gtk_tree_view_set_headers_clickable(GTK_TREE_VIEW (tv), TRUE);
  //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);

  select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
  gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);

  column = ui_tv_column_new
    ("User",
     gtk_cell_renderer_pixbuf_new(), "pixbuf", 3, FALSE,
     gtk_cell_renderer_text_new(), "text", 1, TRUE,
     NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
  ui_tv_config_default_col(column, 1, "slsk:net:col:similar", FALSE);

  column = ui_tv_column_new
    ("Rating", gtk_cell_renderer_text_new(), "text", 2, TRUE, NULL);
  gtk_tree_view_append_column(GTK_TREE_VIEW (tv), column);
  ui_tv_config_default_col(column, 2, "slsk:net:col:urating", TRUE);

  {
    action_item_t* actions =
      action_item_search("Similar slsk users actions");
    ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &actions, NULL);
  }


  /////////////////////////////////
  // right side
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);
  gtk_paned_pack2(GTK_PANED(paned), vbox, FALSE, TRUE);
  ui_layout_monitor("slsk:net:channels:vis", vbox);

  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  
  item = gtk_label_new(_("<b>List of channels</b>"));
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
/*   item = ui_create_header_label("List of channels"); */
  gtk_widget_show(item);
  gtk_misc_set_alignment(GTK_MISC(item), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(hbox), item, TRUE, TRUE, 0);
  
  CLabel = item = gtk_label_new("<b>0</b>");
  gtk_label_set_use_markup(GTK_LABEL(item), TRUE);
  gtk_widget_show(item);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 5);
  

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show(sw);
  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_IN);

  store = gtk_list_store_new(3,
                             G_TYPE_POINTER,  // data pointer
                             G_TYPE_STRING,   // name
                             G_TYPE_INT);     // users
  ChannelStore = store;
  
  tv = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
  //  gtk_tree_view_columns_autosize(GTK_TREE_VIEW(tv));
  gtk_container_add(GTK_CONTAINER(sw), tv);
  gtk_widget_show(tv);
  gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (tv), TRUE);
  //  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tv), TRUE);

  select = gtk_tree_view_get_selection (GTK_TREE_VIEW (tv));
  gtk_tree_selection_set_mode (select, GTK_SELECTION_MULTIPLE);

  column = ui_tv_column_new
    ("Channel", gtk_cell_renderer_text_new(), "text", 1, TRUE, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
  ui_tv_config_default_col(column, 1, "slsk:net:col:channel", FALSE);

  column = ui_tv_column_new
    ("Users", gtk_cell_renderer_text_new(), "text", 2, TRUE, NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (tv), column);
  ui_tv_config_default_col(column, 2, "slsk:net:col:users", TRUE);

  {
    action_item_t* actions =
      action_item_search("Soulseek channel list actions");
    ui_tv_handle_button(GTK_TREE_VIEW(tv), 1, &actions, NULL);
  }


  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox0), hbox, FALSE, FALSE, 0);
  
  GLabel1 = item = ui_create_event_label
    ("get recommendations", G_CALLBACK(slsk_til_get_recommendation), NULL, TRUE);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  GLabel2 = item = ui_create_event_label
    ("get global recommendation", G_CALLBACK(slsk_til_get_global_recommendation), NULL, TRUE);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  GLabel3 = item = ui_create_event_label
    ("get similar users", G_CALLBACK(slsk_til_get_similar_users), NULL, TRUE);
  gtk_box_pack_start(GTK_BOX(hbox), item, FALSE, FALSE, 0);

  item = ui_create_show_hide_label("show channels", "hide channels", vbox);
  gtk_widget_show(item);
  gtk_box_pack_end(GTK_BOX(hbox), item, FALSE, FALSE, 0);
  
  ui_event_label_set_sensitive(GLabel1, FALSE);
  ui_event_label_set_sensitive(GLabel2, FALSE);
  ui_event_label_set_sensitive(GLabel3, FALSE);


  ui_add_network_subpage(SLSK_ID, vbox0);

  ui_slsk_show_pending_nets();

}

void ui_slsk_net_init() {

  ui_slsk_create_net_page();
  
}

void ui_slsk_net_show(slsk_net_t* net)
{
    if (!net) return;
    
    Net = net;
    if (!ServerLabel)
    {
        g_assert(FALSE);
        return;
    }
    
    gtk_label_set_text(ServerLabel, l_printf("%s:%d", net->address, net->port));
    object_monitor_d(OBJ(net), (ObjectFunc)ui_slsk_priv_observer, PrivilegesLabel);
}

void ui_slsk_reco_clear() {
  gtk_list_store_clear(LikeRecoStore);
}

void ui_slsk_reco_add(const char* item, int rating) {
  GtkTreeIter iter;

  gtk_list_store_append(LikeRecoStore, &iter);
  gtk_list_store_set(LikeRecoStore, &iter,
		     0, item,
		     1, rating,
		     -1);
}
static void on_similar_changed(slsk_user_t* user, ui_slsk_similar_t* similar)
{
    g_assert(similar->user == user);
    gtk_list_store_set(LikeUserStore, &similar->iter,
                       3, ui_slsk_get_status_pixmap(user),
                       -1);
}

static gboolean
on_similar_delete(GtkTreeModel* model, GtkTreePath* path,
                  GtkTreeIter* iter)
{
    ui_slsk_similar_t* similar;
    (void)path;
    gtk_tree_model_get(model, iter, 4, &similar, -1);
    object_unmonitor_d(OBJ(similar->user), (ObjectFunc)on_similar_changed, similar);
    user_stop_monitor(USER(similar->user));
    g_free(similar);
    return FALSE;
}


void ui_slsk_reco_user_clear()
{
    gtk_tree_model_foreach(GTK_TREE_MODEL(LikeUserStore),
                           (GtkTreeModelForeachFunc)on_similar_delete,
                           NULL);
    gtk_list_store_clear(LikeUserStore);
}

void ui_slsk_reco_user_add(slsk_user_t* user, int rating)
{
    ui_slsk_similar_t* similar = g_malloc(sizeof(*similar));
    
    similar->user = user;
    gtk_list_store_append(LikeUserStore, &similar->iter);
    gtk_list_store_set(LikeUserStore, &similar->iter,
                       0, user,
                       1, OBJ(user)->name,
                       2, rating,
                       3, ui_slsk_get_status_pixmap(user),
                       4, similar,
                       -1);
    object_monitor_d(OBJ(user), (ObjectFunc)on_similar_changed, similar);
    user_start_monitor(USER(user));
}
