/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.3 $
 * $Date: 2007/11/16 11:59:11 $
 * $Author: sid4u $
 */

#include <string.h>

#include <gtk/gtk.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include "lcore/utils.h"
#include "lcore/gtk2_ui/ui_support.h"
#include "lcore/gtk2_ui/ui_main.h"

#include "slsk_whois.h"

typedef struct {
  slsk_user_t* user;
  unsigned progress;
  unsigned image_size;
  GdkPixbufLoader* loader;

  GtkWidget* win;
  GtkWidget* sep;
  GtkBox* hbox;
  GtkProgressBar* bar;
  GtkImage* pic;
  GtkLabel* slots;
  GtkLabel* queued;
  GtkLabel* has_slots;
  GtkLabel* des;
  GtkWidget* save;
} ui_slsk_whois_t;

static GList* Whois = NULL;

static ui_slsk_whois_t* ui_slsk_whois_search(slsk_user_t* user) {
  GList* dlist;
  for (dlist = Whois; dlist; dlist = dlist->next) {
    ui_slsk_whois_t* whois = dlist->data;
    if (whois->user == user) return whois;
  }
  return NULL;
}

static void on_whois_destroy(ui_slsk_whois_t* whois) {
  gtk_widget_destroy(whois->win);
  user_unref(USER(whois->user));
  g_free(whois);
  Whois = g_list_remove(Whois, whois);
}

void ui_slsk_whois_new(slsk_user_t* user) {
  ui_slsk_whois_t* data;
  GtkWidget* win;
  GtkWidget* vbox0;
  GtkWidget* hbox;
  GtkWidget* table;
  GtkWidget* vbox;
  GtkWidget* sw;
  GtkWidget* label;
  GtkWidget* bar;
  GtkWidget* button;

  data = ui_slsk_whois_search(user);
  if (data) {
    if (data->pic) gtk_widget_destroy(GTK_WIDGET(data->pic));
    data->pic = NULL;
    data->progress = 0;
    return; //FIXME
  }

  data = g_malloc0(sizeof(*data));
  Whois = g_list_append(Whois, data);
  data->user = user_ref(USER(user));

  win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(win), "User info");
  gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DIALOG);
  gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER_ON_PARENT);
  if (ui_main_get_win())
  {
      gtk_window_set_transient_for(GTK_WINDOW(win), ui_main_get_win());
      gtk_window_set_destroy_with_parent(GTK_WINDOW(win), TRUE);
  }
  data->win = win;

  vbox0 = gtk_vbox_new(FALSE, 10);
  gtk_widget_show(vbox0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox0), 10);
  gtk_container_add(GTK_CONTAINER(win), vbox0);
  
  hbox = gtk_hbox_new(FALSE, 10);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX (vbox0), hbox, TRUE, TRUE, 0);
  data->hbox = GTK_BOX(hbox);

  // DESCRIPTION
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
  
  {
    char* text = g_markup_printf_escaped("<b>%s</b>", OBJ(user)->name);
    label = gtk_label_new(text);
    g_free(text);
  }
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX (vbox), label, FALSE, FALSE, 0);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  bar = gtk_progress_bar_new();
  gtk_widget_show(bar);
  gtk_box_pack_start(GTK_BOX(vbox), bar, FALSE, FALSE, 0);
  data->bar = GTK_PROGRESS_BAR(bar);
  gtk_progress_bar_set_text(data->bar, "getting info...");
  
  //
  table = gtk_table_new(3, 2, FALSE);
  gtk_widget_show(table);
  gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);

  label = gtk_label_new("Upload slots");
  //  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
		   GTK_FILL, 0, 0, 0);

  label = gtk_label_new("-");
  gtk_widget_show(label);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 1, 2, 0, 1,
		   GTK_FILL, 0, 0, 0);
  data->slots = GTK_LABEL(label);

  label = gtk_label_new("Uploads queued");
  //  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
		   GTK_FILL, 0, 0, 0);

  label = gtk_label_new("-");
  gtk_widget_show(label);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 1, 2, 1, 2,
		   GTK_FILL, 0, 0, 0);
  data->queued = GTK_LABEL(label);

  label = gtk_label_new("Upload slots available");
  //  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
		   GTK_FILL, 0, 0, 0);

  label = gtk_label_new("-");
  gtk_widget_show(label);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_table_attach(GTK_TABLE(table), label, 1, 2, 2, 3,
		   GTK_FILL, 0, 0, 0);
  data->has_slots = GTK_LABEL(label);


  label = gtk_label_new("<b>Description</b>");
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);

  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_show(sw);
  gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
  gtk_scrolled_window_set_policy
    (GTK_SCROLLED_WINDOW(sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_shadow_type
    (GTK_SCROLLED_WINDOW(sw), GTK_SHADOW_NONE);
  
    
  label = gtk_label_new("");
  gtk_widget_show(label);
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
  data->des = GTK_LABEL(label);

  gtk_scrolled_window_add_with_viewport
    (GTK_SCROLLED_WINDOW(sw), label);
  gtk_viewport_set_shadow_type
    (GTK_VIEWPORT(gtk_bin_get_child(GTK_BIN(sw))), GTK_SHADOW_NONE);

  g_signal_connect_swapped(G_OBJECT(win), "delete-event",
			   G_CALLBACK(on_whois_destroy), data);


  hbox = ui_create_bbox(10, GTK_BUTTONBOX_END);
  gtk_widget_show(hbox);
  gtk_box_pack_end(GTK_BOX(vbox0), hbox, FALSE, FALSE, 0);

  button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
  gtk_widget_show(button);
  gtk_container_add(GTK_CONTAINER(hbox), button);

  g_signal_connect_swapped(G_OBJECT(button), "clicked",
			   G_CALLBACK(on_whois_destroy), data);

  gtk_widget_show(win);
}

static void
whois_size_prepared(GdkPixbufLoader* loader, int width, int height,
		    ui_slsk_whois_t* whois) {
  GtkWidget* pic;
  GtkWidget* vbox;
  GtkWidget* hbox;
  GtkWidget* frame;
  GtkWidget* label;
  int w, h;

  if (whois->pic) return;

  w = width;
  h = height;

  while (width > 800 || height > 600) {
    width /= 2;
    height /= 2;
  }
  if (width != w)
    gdk_pixbuf_loader_set_size(loader, width, height);

  pic = gtk_image_new();
  gtk_widget_set_size_request(pic, width, height); // neccessary?
  gtk_widget_show(pic);
    
  vbox = gtk_vbox_new(FALSE, 5);
  gtk_widget_show(vbox);
  gtk_box_pack_start(GTK_BOX(whois->hbox), vbox, FALSE, FALSE, 0);
  
  hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
  
  label = gtk_label_new("<b>Picture</b>");
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  
  {
    file_size_t bytes = whois->image_size;
    label = gtk_label_new(print_filesize(&bytes));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  }
  {
    label = gtk_label_new
      (gdk_pixbuf_format_get_name(gdk_pixbuf_loader_get_format(loader)));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  }
  {
    label = gtk_label_new(l_printf("%dx%d", w, h));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  }
  if (w != width) {
    label = gtk_label_new(l_printf("(display: %dx%d)", width, height));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  }
  whois->save = label =
    ui_create_event_label("save image", NULL, NULL, TRUE);
  gtk_widget_show(label);
  gtk_box_pack_end(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  ui_event_label_set_sensitive(GTK_WIDGET(label), FALSE);

  frame = gtk_frame_new(NULL);
  gtk_widget_show(frame);
  gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);

  gtk_window_set_resizable(GTK_WINDOW(whois->win), FALSE);
    
  gtk_container_add(GTK_CONTAINER(frame), pic);
  whois->pic = GTK_IMAGE(pic);

}

static void
whois_area_prepared(GdkPixbufLoader* loader, ui_slsk_whois_t* whois) {
  if (whois->pic) {
    GdkPixbuf* buf = gdk_pixbuf_loader_get_pixbuf(loader);
    if (buf) gtk_image_set_from_pixbuf(whois->pic, buf);
  }
}
		    
void ui_slsk_whois_pic_progress(slsk_user_t* user, char* buffer,
				guint32 progress, guint32 max) {
  ui_slsk_whois_t* uiw;
  guint32 len;

  uiw = ui_slsk_whois_search(user);
  if (!uiw) return;
  if (!uiw->loader) {
    uiw->loader = gdk_pixbuf_loader_new();
    uiw->progress = 0;
    g_signal_connect(G_OBJECT(uiw->loader), "size-prepared",
		     G_CALLBACK(whois_size_prepared), uiw);
    g_signal_connect(G_OBJECT(uiw->loader), "area-prepared",
		     G_CALLBACK(whois_area_prepared), uiw);
    gtk_progress_bar_set_text(uiw->bar, "getting image...");
    uiw->image_size = max;
  }
  if (progress > max) progress = max;
  
  if (uiw->progress >= progress) return;

  gtk_progress_bar_set_fraction(uiw->bar, (double)progress/(double)max);
  
  len = progress - uiw->progress;
  gdk_pixbuf_loader_write(uiw->loader, buffer+uiw->progress, len, NULL);
  uiw->progress = progress;

  if (uiw->pic)
    gtk_widget_queue_draw_area
      (GTK_WIDGET(uiw->pic), 0, 0,
       GTK_WIDGET(uiw->pic)->allocation.width,
       GTK_WIDGET(uiw->pic)->allocation.height);
}
				
void ui_slsk_whois_finish(slsk_user_t* user) {
  ui_slsk_whois_t* uiw;
  
  uiw = ui_slsk_whois_search(user);
  if (!uiw) return;
  if (uiw->loader) {
    if (!gdk_pixbuf_format_is_writable
	(gdk_pixbuf_loader_get_format(uiw->loader)) && uiw->save)
      ui_event_label_set_sensitive(uiw->save, TRUE);

    gdk_pixbuf_loader_close(uiw->loader, NULL);
  }
      
  gtk_label_set_text(uiw->slots, l_printf("%u", user->all_slots));
  gtk_label_set_text(uiw->queued, l_printf("%u", user->queued));
  gtk_label_set_text(uiw->has_slots, user->has_slots?"Yes":"No");
  gtk_label_set_text(uiw->des, user->description);

  gtk_progress_bar_set_fraction(uiw->bar, 1.0);
  gtk_progress_bar_set_text(uiw->bar, "done");

  gtk_widget_hide(GTK_WIDGET(uiw->bar));
}
