/* Copyright (C) 2007   The Lopster Team
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. 
*/

/* $Revision: 1.3 $
 * $Date: 2007/11/19 15:38:48 $
 * $Author: sid4u $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <QStandardItemModel>

#include "lcore/preferences.h"
#include "lcore/string_list.h"
#include "lcore/l_i18n.h"

#include "lcore/qt4_ui/ui_network.hpp"
#include "lcore/qt4_ui/ui_main.hpp"

#include "ui_slsk.hpp"
#include "ui_slsk_net.hpp"

class SimilarUserItem : public QStandardItem
{
public:
    SimilarUserItem(const QString &text)
        : QStandardItem(text) {}
    SimilarUserItem(const QIcon &icon, const QString &text)
        : QStandardItem(icon,text) {}
    ~SimilarUserItem() {
        object_free_data(OBJ(user),"SoulSeekItem", NULL);
        user_stop_monitor(USER(user));
    }
    void set_user(slsk_user_t* user) {
        object_set_data(OBJ(user),"SoulSeekItem", this);
        this->user = user;
        this->setData(QVariant::fromValue((void*)user));
        user_start_monitor(USER(user));
    }
private:
    slsk_user_t* user;
};

class SimilarUserModel : public QStandardItemModel
{
public:
    SimilarUserModel(QObject *parent=0);
    void similar_user_clear();
    void similar_user_add(slsk_user_t* user, int rating);
    void similar_user_update(slsk_user_t* slsk_user);
};

class RecommendationModel : public QStandardItemModel
{
public:
    RecommendationModel(QObject *parent=0);
    void recommendation_clear();
    void recommendation_add(const char* item, int rating);
};

class ChannelModel : public QStandardItemModel
{
public:
    ChannelModel(QObject *parent=0);
    void channel_add(slsk_channel_t* channel);
    void channel_clear();
};


ChannelModel::ChannelModel(QObject *parent)
    : QStandardItemModel(0, 2 , parent) 
{
    QStringList headers;
    headers << qtr("Channel") << qtr("Users");
    setHorizontalHeaderLabels(headers); 

}

void ChannelModel::channel_add(slsk_channel_t* channel)
{
    QList<QStandardItem *> items;   
    QStandardItem *item_channel;
    QStandardItem *users;

    item_channel = new QStandardItem(qfu(channel->name)); 
    users = new QStandardItem(qfu("%1").arg(channel->online));
    
    item_channel->setData(QVariant::fromValue((void*)channel));

    items.append(item_channel);
    items.append(users);

    // Add the items
    appendRow(items);
}

void ChannelModel::channel_clear()
{
    removeRows(1,rowCount());
}

SimilarUserModel::SimilarUserModel(QObject *parent)
    : QStandardItemModel(0, 2 , parent) 
{
    QStringList headers;
    headers << qtr("User") << qtr("Rating");
    setHorizontalHeaderLabels(headers); 
}

void SimilarUserModel::similar_user_clear()
{
    removeRows(1,rowCount());
}

void SimilarUserModel::similar_user_add(slsk_user_t* slsk_user, int rating)
{
    if (object_get_data(OBJ(slsk_user), "SoulseekItem")) return;    

    QList<QStandardItem *> items;   
    SimilarUserItem *user;
    QStandardItem *item_rating;

    user = new SimilarUserItem(ui_slsk_get_status_pixmap(slsk_user), qfu(OBJ(slsk_user)->name)); 
    item_rating = new QStandardItem(qfu("%1").arg(rating));
    
    items.append(user);
    items.append(item_rating);

    // Add the items
    appendRow(items);
}

void SimilarUserModel::similar_user_update(slsk_user_t* slsk_user)
{
    SimilarUserItem *user = static_cast< SimilarUserItem *>(object_get_data(OBJ(slsk_user), "SoulseekItem"));
    if (!user) return;

    QModelIndex model_index = indexFromItem(user);
    if (!model_index.isValid()) return;

    user->setIcon(ui_slsk_get_status_pixmap(slsk_user));
}

RecommendationModel::RecommendationModel(QObject *parent)
    : QStandardItemModel(0, 2 , parent) 
{
    QStringList headers;
    headers << qtr("Recommended") << qtr("Rating");
    setHorizontalHeaderLabels(headers); 
}

void RecommendationModel::recommendation_clear()
{
    removeRows(1,rowCount());
}

void RecommendationModel::recommendation_add(const char* item, int rating)
{
    QList<QStandardItem *> items;   
    QStandardItem *reco;
    QStandardItem *item_rating;

    reco = new QStandardItem(qfu(item)); 
    item_rating = new QStandardItem(qfu("%1").arg(rating));

    reco->setData(QVariant::fromValue((void*)item));

    items.append(reco);
    items.append(item_rating);

    // Add the items
    appendRow(items);
}


UiSlSkNet::UiSlSkNet(QWidget *parent)
    : QWidget(parent)
{
    setupUi(this);

    // setup string list
    widget_like->setup(QString(""), QString(""), "slsk:til", "Soulseek til actions");
    widget_dislike->setup(QString(""), QString(""), "slsk:tih", "Soulseek tih actions");
    widget_like->set_header_visible(false);
    widget_dislike->set_header_visible(false);

    // set models
    model_reco = new RecommendationModel(this);
    tree_recommend->ui_treeview_set_model(model_reco);

    model_similar = new SimilarUserModel(this);
    tree_similar->ui_treeview_set_model(model_similar);

    model_channel = new ChannelModel(this);
    tree_channels->ui_treeview_set_model(model_channel);
    
    // actions
    {
        action_item_t* actions = action_item_search("Soulseek til similar actions");
        tree_recommend->ui_tv_handle_button(1, &actions, NULL);
    }
    
    {
        action_item_t* actions = action_item_search("Similar slsk users actions");
        tree_similar->ui_tv_handle_button(1, &actions, NULL);
    }
    {
        action_item_t* actions = action_item_search("Soulseek channel list actions");
        tree_channels->ui_tv_handle_button(1, &actions, NULL);
    }


}

void UiSlSkNet::ui_slsk_net_init()
{
    NetworkPage::ui_network_add_page(SLSK_ID, GetSlSkNet());
}

void UiSlSkNet::ui_slsk_net_cleanup()
{
    UiSlSkNet* net = GetSlSkNet(); 
    NetworkPage::ui_network_remove_page(net);
    delete net;
}

UiSlSkNet *UiSlSkNet::GetSlSkNet()
{
   static UiSlSkNet* net = NULL;
   if (net == NULL)
   {
        net = new UiSlSkNet(); 
   }
   return net;
}

void UiSlSkNet::ui_slsk_show_channels(slsk_net_t* net) {
    GList* dlist;
    int cnt = 0;

    if (!net) return;
  
    GetSlSkNet()->model_channel->channel_clear();

    for (dlist = net->channels; dlist; dlist = dlist->next) 
    {
        slsk_channel_t* channel = (slsk_channel_t*) dlist->data;
        GetSlSkNet()->model_channel->channel_add(channel);
        cnt++;
    }
    GetSlSkNet()->label_nochannels->setText(QString("<b>%1</b>").arg(cnt));
}

void UiSlSkNet::ui_slsk_reco_add(const char* item, int rating)
{
    GetSlSkNet()->model_reco->recommendation_add(item,rating);
}

void UiSlSkNet::ui_slsk_reco_clear()
{
    GetSlSkNet()->model_reco->recommendation_clear();
}

void UiSlSkNet::ui_slsk_reco_user_add(slsk_user_t* user, int rating)
{
    GetSlSkNet()->model_similar->similar_user_add(user, rating);

    object_monitor_d(OBJ(user), (ObjectFunc)ui_slsk_reco_user_update, NULL);
    user_start_monitor(USER(user));
}

void UiSlSkNet::ui_slsk_reco_user_update(slsk_user_t* user)
{
    GetSlSkNet()->model_similar->similar_user_update(user);
}

void UiSlSkNet::ui_slsk_reco_user_clear()
{
    GetSlSkNet()->model_similar->similar_user_clear();
}

void UiSlSkNet::ui_slsk_net_update(slsk_net_t* net)
{
    if (!net) return;
    int days = net->privileges;
  
    GetSlSkNet()->label_privileges->setText(days<0?qtr("unknown"):
                                            days==0?qtr("no privileges"):
                                            days==1?qtr("1 day"):
                                            qtr("%1 days").arg(days));

    if (!net) return;

    GetSlSkNet()->label_status->setText(net->status_message?qtr(net->status_message):qtr("Unknown"));

    if (net->status == SLSK_NET_OFFLINE) {
        GetSlSkNet()->tool_connect->setText(qtr("connect"));
    } else {
        GetSlSkNet()->tool_connect->setText(qtr("disconnect"));
    }
    if (net->status != SLSK_NET_ONLINE) {
        GetSlSkNet()->tool_check->setEnabled(false);
        //ui_event_label_set_sensitive(GLabel1, FALSE);
        //ui_event_label_set_sensitive(GLabel2, FALSE);
        //ui_event_label_set_sensitive(GLabel3, FALSE);
    } else {
        GetSlSkNet()->tool_check->setEnabled(true);
        //ui_event_label_set_sensitive(GLabel1, TRUE);
        //ui_event_label_set_sensitive(GLabel2, TRUE);
        //ui_event_label_set_sensitive(GLabel3, TRUE);
    }    
}

void UiSlSkNet::ui_slsk_net_show(slsk_net_t* net)
{
    if (!net) return;
    
    GetSlSkNet()->m_net = net;
    GetSlSkNet()->label_address->setText(QString("%1:%2").arg(net->address).arg(net->port));   
    object_monitor_d(OBJ(net), (ObjectFunc)ui_slsk_net_update, NULL);
}
