/* Copyright (C) 2005-2007 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.3 $
 * $Date: 2007/06/02 18:41:25 $
 * $Author: sgop $
 */

#include <string.h>

#include "lcore/utils.h"
#include "lcore/preferences.h"
#include "lcore/l_listen.h"
#include "lcore/search.h"
#include "lcore/buddy.h"
#include "lcore/string_list.h"
#include "lcore/l_base.h"
#include "lcore/version.h"
#include "lcore/l_i18n.h"
#include "slsk.h"
#include "slsk_network.h"
#include "slsk_proto.h"
#include "slsk_ui.h"
#include "slsk_commands.h"
#include "slsk_search.h"
#include "slsk_peer.h"
#include "slsk_hotlist.h"
#include "slsk_users.h"
#include "slsk_actions.h"

#ifdef WIN32
#include "slskalias.h"
#endif

#ifndef WIN32
int SLSK_ID = -1;
#else
int slsk_id = -1;
#endif

static char* CharSet = NULL;

static void slsk_away_observer(pref_node_t* node) {
  char* away = node->value;
  if (away && *away) {
    slsk_net_send(NULL, SLSK_S_SET_STATUS, SLSK_USER_AWAY);
  } else {
    slsk_net_send(NULL, SLSK_S_SET_STATUS, SLSK_USER_ONLINE);
  }
}

const char* slsk_charset_get() {
  if (CharSet && *CharSet) return CharSet;
  else return "UTF-8";
}

static void slsk_charset_observer(pref_node_t* node) {
  const char* charset = (char*)(node->value);
  g_free(CharSet);
  if (charset && *charset)
    CharSet = g_strdup(charset);
  else
    CharSet = NULL;
}

static void slsk_init()
{
/*   pref_register_int("/slsk/protocol", FALSE, 155); */
/*   pref_register_int("/slsk/protocol", FALSE, 198); */
    pref_register_int("/slsk/protocol", FALSE, 181);
    pref_register_string("/slsk/charset", TRUE, "WINDOWS-1252");
    
    string_list_register("slsk:til", N_("Things i like"), TRUE);
    string_list_register("slsk:tih", N_("Things i dislike"), TRUE);
    
    slsk_actions_init();
}

static void slsk_run() {
  pref_observe("/core/user/away", (ObjectFunc)slsk_away_observer, NULL);
  CharSet = g_strdup(pref_get_ptr("/slsk/charset", char));
  pref_observe("/slsk/charset", (ObjectFunc)slsk_charset_observer, NULL);
}

static void slsk_exit() {
}

static void slsk_plugin_start(plugin_t* plugin) {
  plugin_init(plugin, "Soulseek Module",
              (ModuleFunc)slsk_init, (ModuleFunc)slsk_run, (ModuleFunc)slsk_exit);

#ifndef WIN32
  SLSK_ID = plugin->ID;
#else
  slsk_id = plugin->ID;
#endif
  plugin->connect = slsk_connect_change;
  
  module_require(base_module_instance);
  module_require(pref_module_instance);
  module_require(slsk_command_module_instance);
  module_require(buddy_module_instance);
  module_require(search_module_instance);
  module_require(slsk_ui_module_instance);
  module_require(slsk_net_module_instance);
  module_require(slsk_user_module_instance);
  module_require(slsk_hot_module_instance);
  module_require(slsk_download_module_instance);
  module_require(slsk_search_module_instance);
  module_require(slsk_chat_module_instance);
  //  module_require(MODULE(plugin), upload_module_instance);
  //  module_require(MODULE(plugin), download_module_instance);
  //  module_require(MODULE(plugin), browse_module_instance);
  //  module_require(MODULE(plugin), library_module_instance);

}


void plugin_initialize(plugin_info_t* info) {
  info->name       = g_strdup("Soulseek");
  info->sname      = g_strdup("Slsk");
  info->copyright  = g_strdup("Copyright \302\251 2003-2007 Markus Lausser");
  info->if_version = IF_VERSION;
  info->pl_version = 0x00050000;  // 0.5.0-0
  info->icon       = g_strdup("slsk_icon.png");
  info->license    = g_strdup("GPL (GNU General Public License)");
  info->website    = g_strdup("http://lopster.sourceforge.net");
  info->comment    = g_strdup("Plugin which connects to the Soulseek\342\204\242 network (www.slsknet.org)");
  info->authors    = g_list_append
    (info->authors, g_strdup("Markus Lausser <sgop@users.sourceforge.net>"));
  info->start      = slsk_plugin_start;
#ifdef PACKAGE_SOURCE_DIR
  lopster_pixmap_dir_add(SDIR(PACKAGE_SOURCE_DIR) SDIR("plugin") 
                         SDIR("soulseek") "pixmaps");
#endif
}

#ifdef WIN32
int slsk_plugin_get_id( )
{
    return slsk_id;
}
#endif

#ifdef WIN32
#define _SLSK_C_
#include "slskaliasdef.c"
#endif


