/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.1 $
 * $Date: 2007/04/29 09:55:05 $
 * $Author: sgop $
 */

#include "lcore/search.h"
#include "lcore/action_item.h"
#include "slsk.h"
#include "slsk_til.h"
#include "slsk_hotlist.h"
#include "slsk_chat.h"
#include "slsk_peer.h"
#include "slsk_upload.h"
#include "slsk_browse.h"
#include "slsk_download.h"

static void on_slsk_chat(GList* users) {
  GList* dlist;
  for (dlist = users; dlist; dlist = dlist->next) {
    slsk_user_t* user = dlist->data;
    if (user && user->net && OBJ(user)->name)
      slsk_cp_new_private(user->net, OBJ(user)->name);
  }
}

#define USER_LIST_ACTION(func) static void						\
  on_ ## func (GList* users) {												\
    GList* dlist;																			\
    for (dlist = users; dlist; dlist = dlist->next) {	\
      slsk_user_t* user = dlist->data;								\
      func(user);																			\
    }																									\
  }

USER_LIST_ACTION(slsk_peer_whois);
USER_LIST_ACTION(slsk_hot_new);
USER_LIST_ACTION(slsk_browse_user);



static gboolean on_slsk_check_not_buddy(slsk_user_t* user) {
  return (!slsk_hot_in_hotlist(user));
}

static gboolean on_til_check(char* item) {
  return (!slsk_til_in_list(item));
}

static gboolean on_til_check2(char* item) {
  return (slsk_til_in_list(item));
}

static gboolean on_tih_check(char* item) {
  return (!slsk_tih_in_list(item));
}

static gboolean on_tih_check2(char* item) {
  return (slsk_tih_in_list(item));
}

static action_item_t* slsk_actions_user_menu(const char* name) {
  action_item_t* item;

  item = action_item_root(name);
  action_item_simple("Open chat...", item, 0,
		     (ActionItemFunc)on_slsk_chat);
  action_item_simple("Send message...", item, AI_RESTRICT_1,
		     NULL);
  action_item_sep(item);
  action_item_simple("Whois user", item, 0,
		     (ActionItemFunc)on_slsk_peer_whois);
  action_item_new("Add to buddies", item, 0,
		  (ActionItemFunc)on_slsk_hot_new,
		  (ActionItemCheck)on_slsk_check_not_buddy);
  action_item_simple("Browse user", item, 0,
		     (ActionItemFunc)on_slsk_browse_user);

  return item;
}


/*
static slsk_user_t* slsk_str_to_u(const char* nick) {
  slsk_user_t* user = slsk_user_find(nick);
  if (!user->net) slsk_user_set_net(user, slsk_net_get_any_online());
  return user;
}
*/
static slsk_user_t* slsk_cu_to_u(slsk_user_t* cu, slsk_cp_t* page) {
  slsk_user_set_net(cu, page->net);
  return cu;
}
static slsk_user_t* slsk_upload_to_u(slsk_upload_t* upload) {
  return SLSK_USER(UPLOAD(upload)->user);
}

static slsk_user_t* slsk_source_to_u(slsk_source_t* source) {
  return SLSK_USER(SOURCE(source)->user);
}

static slsk_user_t* slsk_result_to_u(search_result_t* result) {
  return SLSK_USER(result->user);
}

static slsk_user_t* slsk_browse_to_u(file_t* file, browse_t* browse) {
  (void)file;
  return SLSK_USER(browse->user);
}


static void on_slsk_join(GList* channels) {
  GList* dlist;
  for (dlist = channels; dlist; dlist = dlist->next) {
    slsk_channel_t* channel = dlist->data;
    slsk_join_channel(channel->net, channel->name);
  }
}

#define TIL_LIST_ACTION(func) static void             \
  on_ ## func (GList* tils) {                         \
    GList* dlist;                                     \
    for (dlist = tils; dlist; dlist = dlist->next) {  \
      char* til = dlist->data;                        \
      func(til);                                      \
    }                                                 \
  }

TIL_LIST_ACTION(slsk_til_add);
TIL_LIST_ACTION(slsk_tih_add);
TIL_LIST_ACTION(slsk_til_remove);
TIL_LIST_ACTION(slsk_tih_remove);
TIL_LIST_ACTION(slsk_til_get_item_recommendation);
TIL_LIST_ACTION(slsk_til_get_user_recommendation);
TIL_LIST_ACTION(slsk_til_search);


void slsk_actions_init() {
  action_item_t* item;


  item = action_item_root("Soulseek channel list actions");
  action_item_simple("Join room", item, 0,
                     (ActionItemFunc)on_slsk_join);
  action_item_sep(item);
  action_item_simple("Refresh list", item, AI_GLOBAL,
                     (ActionItemFunc)NULL);
  //////////
  item = action_item_root("Soulseek til actions");
  action_item_simple("Remove item", item, 0,
		     (ActionItemFunc)on_slsk_til_remove);
  action_item_sep(item);
  action_item_simple("Get similar recommendations", item, AI_RESTRICT_1,
		     (ActionItemFunc)on_slsk_til_get_item_recommendation);
  action_item_simple("Get users recommending this item", item, AI_RESTRICT_1,
		     (ActionItemFunc)on_slsk_til_get_user_recommendation);

  item = action_item_root("Soulseek tih actions");
  action_item_simple("Remove item", item, 0,
		     (ActionItemFunc)on_slsk_tih_remove);
  action_item_sep(item);
  action_item_simple("Get users recommending this item", item, AI_RESTRICT_1,
		     (ActionItemFunc)on_slsk_til_get_user_recommendation);

  //////////
  item = action_item_root("Soulseek til similar actions");
  action_item_simple("Search for item", item, 0,
		     (ActionItemFunc)on_slsk_til_search);
  action_item_simple("Get similar recommendations", item, AI_RESTRICT_1,
		     (ActionItemFunc)on_slsk_til_get_item_recommendation);
  action_item_simple("Get users recommending this item", item, AI_RESTRICT_1,
		     (ActionItemFunc)on_slsk_til_get_user_recommendation);
  action_item_sep(item);
  action_item_new("Add to things i like", item, 0,
		  (ActionItemFunc)on_slsk_til_add,
		  (ActionItemCheck)on_til_check);
  action_item_new("Remove from things i like", item, 0,
		  (ActionItemFunc)on_slsk_til_remove,
		  (ActionItemCheck)on_til_check2);
  action_item_sep(item);
  action_item_new("Add to things i don't like", item, 0,
		  (ActionItemFunc)on_slsk_tih_add,
		  (ActionItemCheck)on_tih_check);
  action_item_new("Remove from things i don't like", item, 0,
		  (ActionItemFunc)on_slsk_tih_remove,
		  (ActionItemCheck)on_tih_check2);
  
  //////////
  item = slsk_actions_user_menu("Similar slsk users actions");
  //  item->convert = (ActionObjectConvert)slsk_str_to_u; // no conversion needed
  
  item = slsk_actions_user_menu("Online slsk users actions");
  item->convert = (ActionObjectConvert)slsk_cu_to_u;
  
  item = slsk_actions_user_menu("Upload slsk users actions");
  item->convert = (ActionObjectConvert)slsk_upload_to_u;
  user_action_upload_register(SLSK_ID, item);

  item = slsk_actions_user_menu("Source slsk users actions");
  item->convert = (ActionObjectConvert)slsk_source_to_u;
  user_action_source_register(SLSK_ID, item);

  item = slsk_actions_user_menu("Result slsk users actions");
  item->convert = (ActionObjectConvert)slsk_result_to_u;
  user_action_result_register(SLSK_ID, item);

  item = slsk_actions_user_menu("Browse slsk users actions");
  item->convert = (ActionObjectConvert)slsk_browse_to_u;
  user_action_browse_register(SLSK_ID, item);

  

}
