/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2007/05/23 20:42:29 $
 * $Author: sid4u $
 */

#include <string.h>

#include "lcore/ui.h"
#include "slsk_browse.h"
#include "slsk_peer.h"

#ifdef WIN32
#include "slskalias.h"
#endif

struct _slsk_browse_t {
  browse_t _browse;
};

static GList* SlskBrowses = NULL;

slsk_browse_t* slsk_browse_search(slsk_user_t* user) {
  GList* dlist;

  for (dlist = SlskBrowses; dlist; dlist = dlist->next) {
    slsk_browse_t* browse = dlist->data;
    if (BROWSE(browse)->user == USER(user)) return browse;
  }
  return NULL;
}

static void slsk_browse_destroy(slsk_browse_t* browse) {
  SlskBrowses = g_list_remove(SlskBrowses, browse);
  // make sure we kill a sockets that point to this browse
  slsk_peer_browse_removed(browse);
  browse_destroy(BROWSE(browse));
}

static void slsk_browse_init(slsk_browse_t* browse, slsk_user_t* user) {
  browse_init(BROWSE(browse), USER(user));
  OBJ(browse)->destroy = (ObjectFunc)slsk_browse_destroy;
  SlskBrowses = g_list_append(SlskBrowses, browse);
}

static slsk_browse_t* slsk_browse_new(slsk_user_t* user) {
  slsk_browse_t* browse = g_malloc0(sizeof(*browse));
  slsk_browse_init(browse, user);
  return browse;
}

void slsk_browse_user(slsk_user_t* user) {
  slsk_browse_t* browse;
  g_assert(user);
  browse = slsk_browse_search(user);
  if (browse) {
    // FIXME
    g_log("BROWSE", G_LOG_LEVEL_WARNING,
          "browse '%s' already existent", OBJ(user)->name);
    return;
  }
  browse = slsk_browse_new(user);
  slsk_peer_browse(user, browse);
}

void slsk_browse_set_progress(slsk_user_t* user, double prog, const char* status) {
  static slsk_browse_t* browse = NULL;
  if (!browse || BROWSE(browse)->user != USER(user)) {
    // unmonitor old browse
    if (browse) object_unset_ptr((object_t**)(&browse));
    // search browse
    browse = slsk_browse_search(user);
    // monitor new browse
    if (browse) object_set_ptr((object_t**)(&browse));
  }
  // set progress
  if (browse) browse_set_progress(BROWSE(browse), prog, status);
}

#ifdef WIN32
#define _SLSK_BROWSE_C_
#include "slskaliasdef.c"
#endif
