/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <stdlib.h>

#include <glib.h>

#include "lcore/utils.h"
#include "lcore/chat.h"
#include "lcore/commands.h"
#include "lcore/preferences.h"
#include "lcore/commands.h"
#include "slsk_chat.h"
#include "slsk.h"
#include "slsk_commands.h"
#include "slsk_network.h"
#include "slsk_peer.h"
#include "slsk_users.h"

typedef void (*SlskCommandFunc)(chat_page_t*, slsk_net_t*, char*);
#define SLSK_COMM(f) static void f (chat_page_t* page, slsk_net_t* net, char *data)

#define SLSK_COMMAND(c) ((slsk_command_t*)(c))
typedef struct {
  command_t _command;
  SlskCommandFunc handle;
} slsk_command_t;

typedef struct {
  module_t _module;
} slsk_command_module_t;

static slsk_command_module_t* SlskCommandModule = NULL;

#define CHECK_NET COMMAND_ERROR(!net, page, "No network specified")

static void slsk_command_handle(chat_page_t* page, command_t* comm, char *data) {
  slsk_cp_t* scp;
  slsk_net_t* net;
  slsk_command_t* command = SLSK_COMMAND(comm);
  
  if (page && page->ID == SLSK_ID) scp = SCP(page);
  else scp = NULL;

  g_log("SLSK_COM", G_LOG_LEVEL_DEBUG,
        "handle: [%s] [%s]", command?COMMAND(command)->name:"(null)",
        data?data:"(null)");
  if (command) {
    net = scp?scp->net:NULL;
    if (!net) net = slsk_net_get_online();
    command->handle(page, net, data);
  } else if (scp) {
    // normal line, pass to chat page
    if (scp->type == SLSK_PAGE_PRIVATE) {
      slsk_send_private_message(scp->net, OBJ(page)->name, data);
    } else if (scp->type == SLSK_PAGE_PUBLIC) {
      slsk_send_public_message(scp, data);
    }
  }
  return;
}
  
static void
slsk_command_new(const char* name, const char* syntax,
                 const char* des, SlskCommandFunc func) {
  slsk_command_t *comm;
  
  g_assert(name);

  comm = g_malloc0(sizeof(*comm));
  command_init(COMMAND(comm), SLSK_ID, name, syntax, des);
  comm->handle = func;
}

SLSK_COMM(slsk_com_whois) {
  char* nick;
  slsk_user_t* user;
  
  nick = arg(data, 1);
  COMMAND_ERROR(!net, page, "not online");
  COMMAND_ERROR(!nick, page, "Missing nickname");
  user = slsk_user_find(nick);
  if (!user->net) slsk_user_set_net(user, net);
  slsk_peer_whois(user);
}

SLSK_COMM(slsk_com_me) {
  (void)net;

  COMMAND_ERROR(!page || page->ID != SLSK_ID || !page->active ||
                SCP(page)->type != SLSK_PAGE_PUBLIC, page,
                "This is not a valid soulseek channel");
  if (!data) return;
  slsk_send_public_message(SCP(page), l_printf("/me %s", data));
}

SLSK_COMM(slsk_com_ticker) {
  (void)net;
  
  COMMAND_ERROR(!page || page->ID != SLSK_ID || !page->active ||
                SCP(page)->type != SLSK_PAGE_PUBLIC, page,
                "This is not a valid soulseek channel");
  slsk_set_ticker(SCP(page), data);
}

SLSK_COMM(slsk_com_nsearch) {
  char* user;
  char* args;

  user = arg(data, 0);
  args = arg(NULL, 1);
  COMMAND_ERROR(!net, page, "You need to specify a network");
  COMMAND_ERROR(!user, page, "You need to specify a user");
  COMMAND_ERROR(!args, page, "You need to specify a search term");

  slsk_peer_search(user, args);
}

SLSK_COMM(slsk_com_privileges) {
  char* user;

  user = arg(data, 1);
  COMMAND_ERROR(!net, page, "You need to specify a network");

  //  COMMAND_ERROR(!user, page, "You need to specify a user");
  if (!user) {
    GList* dlist;
    slsk_cp_t* scp;
    COMMAND_ERROR(!page || page->ID != SLSK_ID || !page->active ||
                  SCP(page)->type != SLSK_PAGE_PUBLIC, page,
                  "This is not a valid soulseek channel");
    scp = SCP(page);
    for (dlist = scp->users; dlist; dlist = dlist->next) {
      slsk_user_t* cu = dlist->data;
      if (cu->privileges == SLSK_PRIV_ON)
        chat_print(PAGE(page), "+\0client",
                   "+\1c_client", "user",
                   "+\1nick", OBJ(cu)->name, "\0\0",
                   "+\1c_client", "has privileges",
                   "+\0nl", NULL);
    }
  } else {
    int priv = slsk_user_get_privileges(user);
    chat_print(PAGE(page), "+\0client",
               "+\1c_client", "user",
               "+\1nick", user, "\0\0",
               "+\1c_client", 
               (priv==SLSK_PRIV_ON)?"has privileges":
               (priv==SLSK_PRIV_OFF)?"has no privileges":
               "has unknown privileges",
               "+\0nl", NULL);
  }

}

SLSK_COMM(slsk_com_parent) {
  char* nick = arg(data, 0);
  char* ip = arg(NULL, 0);
  char* port = arg(NULL, 0);

  COMMAND_ERROR(!net, page, "You need to specify a network");
  COMMAND_ERROR(!ip, page, "You need to specify an ip");
  COMMAND_ERROR(!port, page, "You need to specify a port");

  slsk_peer_add_host(net, nick, inet_addr(ip),
                     htons(atoi(port)));
  slsk_peer_make_d();
}

SLSK_COMM(slsk_com_abort_parent) {
  (void)page;
  (void)net;
  (void)data;
  slsk_peer_abort_d();
}

SLSK_COMM(slsk_com_join) {
  char* channel = arg(data, 0);

  COMMAND_ERROR(!channel, page, "You need to specify a channel");
  slsk_join_channel(net, channel);
}

SLSK_COMM(slsk_com_test) {
  char* number = arg(data, 0);

  (void)net;
  COMMAND_ERROR(!number, page, "You need to specify a number");
  pref_store("/slsk/protocol", atoi(number));
}


static void slsk_commands_init() {
  command_handler_install(SLSK_ID, (CommandHandler)slsk_command_handle);
}

static void slsk_commands_run() {
  slsk_command_new("whois", "<user>",
                   "Get information about the user",
                   slsk_com_whois);
  slsk_command_new("me", "<message>",
                   "Send action description to current channel",
                   slsk_com_me);
  slsk_command_new("ticker", "[<message>]",
                   "Set your ticker for a channel. No message argument to remove the ticker",
                   slsk_com_ticker);
  slsk_command_new("nsearch", "<nick> <arguments>",
                   "Search in the files of a user",
                   slsk_com_nsearch);
  slsk_command_new("privileges", "[<nick>]",
                   "Check whether the user has privileges",
                   slsk_com_privileges);
  slsk_command_new("parent", "<nick> <ip> <port>",
                   "Set parent ip node to connect to",
                   slsk_com_parent);
  slsk_command_new("abortparent", NULL,
                   "Abort current parent connection",
                   slsk_com_abort_parent);
  slsk_command_new("join", "<channel>",
                   "Join the channel <channel>",
                   slsk_com_join);
  slsk_command_new("test", "<number>",
                   "Set login protocol number to <number>",
                   slsk_com_test);
}
static void slsk_commands_exit() {
  //FIXME: cleanup
}

static void slsk_command_module_init(slsk_command_module_t* module) {
  module_init(MODULE(module), "Soulseek Command Module",
              (ModuleFunc)slsk_commands_init, 
              (ModuleFunc)slsk_commands_run,
              (ModuleFunc)slsk_commands_exit);
  
  module_require(pref_module_instance);
  module_require(command_module_instance);
  module_require(chat_module_instance);
}


module_t* slsk_command_module_instance() {
  if (!SlskCommandModule) {
    SlskCommandModule = g_malloc0(sizeof(*SlskCommandModule));
    slsk_command_module_init(SlskCommandModule);
  }
  return MODULE(SlskCommandModule);
}


#undef SLSK_COMMAND
#undef SLSK_COMM
#undef CHECK_NET
