/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2007/05/23 20:42:29 $
 * $Author: sid4u $
 */

#include <string.h>

#include "lcore/buddy.h"
#include "lcore/utils.h"
#include "lcore/ui.h"
#include "slsk.h"
#include "slsk_hotlist.h"
#include "slsk_proto.h"

#ifdef WIN32
#include "slskalias.h"
#endif

typedef struct {
  module_t _module;
} slsk_hot_module_t;

static slsk_hot_module_t* SlskHotModule = NULL;

static gboolean get_user(user_t* user, user_t** slsk_user) {
  *slsk_user = user;
  return TRUE;
}

slsk_user_t* slsk_hot_get_user(buddy_t* buddy) {
  slsk_user_t* old_user = NULL;
  buddy_foreach_user(buddy, SLSK_ID, (UserForeach)get_user, &old_user);
  return old_user;
}

static void slsk_hot_change_user(buddy_t* buddy) {
  attribute_t* attr = attr_find(ATTRC(buddy), "buddy", "SlskNick");
  const char* nick = attr?attr_get_ptr(attr, char):NULL;
  slsk_user_t* new_user = nick?slsk_user_find(nick):NULL;
  slsk_user_t* old_user = slsk_hot_get_user(buddy);
  
  if (old_user == new_user) return;
  
  if (old_user) buddy_remove_user(buddy, USER(old_user));
  if (new_user) buddy_add_user(buddy, USER(new_user));
}

void slsk_hot_new(slsk_user_t* user) {
  buddy_t* buddy;
  
  g_assert(SlskHotModule);
  buddy = buddy_find_from_user(USER(user));
  if (buddy) return;

  buddy = buddy_new(OBJ(user)->name);
  attr_store(ATTRC(buddy), "buddy", "SlskNick", OBJ(user)->name);
  slsk_hot_change_user(buddy);
}

static gboolean slsk_hot_user_add(slsk_user_t* user, slsk_net_t* net)
{
    slsk_user_set_net(user, net);
    slsk_net_send(NULL, SLSK_S_ADD_USER, OBJ(user)->name);
    return FALSE;
}

static gboolean slsk_hot_user_remove(slsk_user_t* user, slsk_net_t* net)
{
    if (user->net == net)
    {
        slsk_user_set_net(user, NULL);
        slsk_user_set_status(user, SLSK_USER_OFFLINE);
    }
    return FALSE;
}


void slsk_hot_joined_net(slsk_net_t* net)
{
    users_foreach_monitored
        (SLSK_ID, (UserForeach)slsk_hot_user_add, net);
}

void slsk_hot_left_net(slsk_net_t* net) {
  users_foreach_monitored
    (SLSK_ID, (UserForeach)slsk_hot_user_remove, net);
}

gboolean slsk_hot_in_hotlist(slsk_user_t* user) {
  g_assert(SlskHotModule);
  return (buddy_find_from_user(USER(user)) != NULL);
}


static void slsk_hots_init() {
  attr_register_str("buddy", "SlskNick", ATTR_SKIP, NULL);
  buddy_register_update(slsk_hot_change_user);
}

static void slsk_hot_module_init(slsk_hot_module_t* module) {
  module_init(MODULE(module), "Slsk Hotlist Module",
	      (ModuleFunc)slsk_hots_init, NULL, NULL);

  module_require(buddy_module_instance);
  module_require(slsk_user_module_instance);
  module_require(slsk_net_module_instance);
}


module_t* slsk_hot_module_instance() {
  if (!SlskHotModule) {
    SlskHotModule = g_malloc0(sizeof(*SlskHotModule));
    slsk_hot_module_init(SlskHotModule);
  }
  return MODULE(SlskHotModule);
}

#ifdef WIN32
#define _SLSK_HOTLIST_C_
#include "slskaliasdef.c"
#endif
