/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2007/05/23 20:42:29 $
 * $Author: sid4u $
 */

#ifndef _SLSK_PEER_H_
#define _SLSK_PEER_H_

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

typedef struct _slsk_peer_t slsk_peer_t;

#include "lcore/tcp_socket.h"
#include "slsk_network.h"
#include "slsk_browse.h"
#include "slsk_whois.h"
#include "slsk_users.h"
#include "slsk_download.h"

G_BEGIN_DECLS

struct _slsk_peer_t {
  slsk_con_t _con;
  slsk_user_t* user;
  GList* actions;
  GList* actions_save;
  guint32 token;
  int direction;
  slsk_browse_t* browse;
  unsigned search_cnt;
};


#define S_BROWSE_REMOVED   200

#define SLSK_ACTION_NONE       0
#define SLSK_ACTION_WHOIS      1
#define SLSK_ACTION_BROWSE     2
#define SLSK_ACTION_DISTRIB    3
#define SLSK_ACTION_RESULTS    4
#define SLSK_ACTION_SEARCH     5
#define SLSK_ACTION_UPLOAD     6
#define SLSK_ACTION_QUEUED     7
#define SLSK_ACTION_REQUEST    8
#define SLSK_ACTION_QUEUE_POS  9
#define SLSK_ACTION_DOWNLOAD  10
#define SLSK_ACTION_UP_FAILED 11

typedef struct _slsk_action_t slsk_action_t;


void slsk_peer_remove_save_action(slsk_peer_t* peer);

/** incoming, peer initiated */
const char* slsk_peer_requested(const char* nick,
				tcp_socket_t* socket, char type);
/** outgoing, peer initiated */
void slsk_peer_request_fw(slsk_user_t* user, char type, guint32 token);

const char* slsk_peer_replied(tcp_socket_t* socket, guint32 token);

void slsk_peer_whois(slsk_user_t* user);
void slsk_peer_browse(slsk_user_t* user, slsk_browse_t* browse);
void slsk_peer_browse_removed(slsk_browse_t* browse);
void slsk_peer_got_address(slsk_net_t* net, const char* nick, ip_t ip, port_t port);

void slsk_peer_add_host(slsk_net_t* net, const char* nick,
			ip_t ip, port_t port);
void slsk_peer_make_d();

void slsk_peer_set_parent_timeout (unsigned timeout);
void slsk_peer_set_parent_interval(unsigned interval);
void slsk_peer_set_min_parents    (unsigned number);

void slsk_peer_search(const char* nick, const char* term);
void slsk_peer_send_results(slsk_user_t* user, guint32 token, GList* results);
void slsk_peer_upload(guint32 token, slsk_user_t* user, file_size_t size);
void slsk_peer_queued(guint32 token, slsk_user_t* user);
void slsk_peer_request(guint32 token, slsk_user_t* user, file_t* file, int dir);
void slsk_peer_download(slsk_source_t* source);
void slsk_peer_queue_pos(slsk_source_t* source);

G_END_DECLS
#endif
