/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2007/11/16 11:49:27 $
 * $Author: sid4u $
 */

#define SLSK_P_CONNECT     0x01
#define SLSK_P_INCOMING    0x02

#include <string.h>
#include <stdio.h>

#include "lcore/utils.h"
#include "lcore/buffer.h"
#include "lcore/zbuffer.h"
#include "lcore/files.h"
#include "lcore/preferences.h"
#include "lcore/library.h"
#include "lcore/l_base.h"
#include "slsk_peer.h"
#include "slsk_proto.h"
#include "slsk_peer_proto.h"
#include "slsk.h"
#include "slsk_search.h"
#include "slsk_ui.h"
#include "slsk_browse.h"
#include "slsk_upload.h"
#include "slsk_download.h"
#include "slsk_con.h"

#define SLSK_R_DISTRIB_ALIVE    0
#define SLSK_R_DISTRIB_SEARCH   3

#define PHANDLER(f) static void f (slsk_peer_t* peer, char* data, unsigned dlen)
#define SLSK_PCALL(f) f(peer, command->data, command->length)

typedef struct {
  file_node_t* folder;
  GList* files;
} fbucket_t;

PHANDLER(handle_search_result) {
  char* nick;
  guint32 token;
  guint32 number;
  guint32 speed;
  gboolean has_slots;
  gboolean queued;
  GList* results = NULL;
  GList* dlist;

  nick = unpack_string_u32(&data, &dlen);
  CHECK(nick);
  REQ_LEN(8);
  token = unpack_u32(&data, &dlen);
  number = unpack_u32(&data, &dlen);
  
  // nicotine has a bug and sends our nick here, do we don't
  // use 'nick' but the nick from the connection
  //sfs = slsk_fs_new(slsk_user_find(nick));
  while (number > 0) {
    file_t* file = slsk_unpack_file(&data, &dlen);
    CHECK(file);
    results = g_list_append(results, file);
    number--;
  }
  
  REQ_LEN(9);
  has_slots = unpack_byte(&data, &dlen); // upload slots available
  speed = unpack_u32(&data, &dlen);  //FIXME: really speed?
  queued = unpack_u32(&data, &dlen); // queued files
  if (dlen >= 4) {
    int arg = unpack_u32(&data, &dlen);
    if (arg != 0)
      g_log("SLSK_PROTO", G_LOG_LEVEL_WARNING, "%s last result arg is %d", nick, arg);
  }
  CHECK_LEN(0);

  slsk_user_set_whois(peer->user, peer->user->all_slots, queued, has_slots);
  
  for (dlist = results; dlist; dlist = dlist->next) {
    file_t* file = dlist->data;
    slsk_search_insert_result(token, file, peer->user);
    object_kill(OBJ(file));
  }
  g_list_free(results);

}

PHANDLER(handle_whois_result) {
  char* des;
  char pic;
  guint32 len;
  char* filename = NULL;
  FILE* file;
  guint32 slots;
  guint32 queued;
  gboolean has_slots;

  des = unpack_string_u32(&data, &dlen);
  CHECK(des); REQ_LEN(1);
  pic = unpack_byte(&data, &dlen);
  if (pic) {
    REQ_LEN(4);
    len = unpack_u32(&data, &dlen);
    REQ_LEN(len);
    filename = l_printf(SDIR("%s") "image.tmp", g_get_tmp_dir());
    file = fopen(filename, "w");
    if (!file) return;
    
    fwrite(data, len, 1, file);
    fclose(file);

    data += len; dlen -= len;
  }
  if (des && *des) {
    line_insert_lf(des, 50);
    peer->user->description = 
      to_utf8(des, -1, NULL, NULL, 4,
              "utf-8", slsk_charset_get(), "latin1", "windows-1252");
  } else {
    peer->user->description = g_strdup("no description");
  }
  REQ_LEN(9);
  slots = unpack_u32(&data, &dlen);
  queued = unpack_u32(&data, &dlen);
  has_slots = unpack_byte(&data, &dlen);
  slsk_user_set_whois(peer->user, slots, queued, has_slots);

  CHECK_LEN(0);
  
  slsk_whois_finish(peer->user);
  
}

PHANDLER(handle_browse_result) {
  guint32 dir_number;
  slsk_browse_t* browse;

  REQ_LEN(4);

  browse = slsk_browse_search(peer->user);
  if (!browse) {
    g_log("SLSK_P", G_LOG_LEVEL_WARNING, "got unknown browse");
    return;
  }
  g_assert(browse == peer->browse);

  //FIXME: correct encoding

  dir_number = unpack_u32(&data, &dlen);
/*   g_message("got %u dirs", dir_number); */
  while (dir_number > 0) {
    char* dir;
    int file_number;

    REQ_LEN(8);
    dir = unpack_string_u32(&data, &dlen);
    file_number = unpack_u32(&data, &dlen);
    // we dont show empty dirs
    while (file_number > 0) {
      file_t* file = slsk_unpack_file(&data, &dlen);
      char* n;

      CHECK(file);
      n = g_strdup_printf("%s\\%s", dir, file->netname);
      file_set_origname(file, n);
      file_set_locname(file, to_utf8(n, -1, NULL, NULL, 3,
                                     "utf-8", slsk_charset_get(), "latin1"));
      browse_insert_file(BROWSE(browse), file);
      file_number--;
    }
    dir_number--;
  }

  CHECK_LEN(0);

  browse_set_status(BROWSE(browse), "Complete");
  browse_set_progress(BROWSE(browse), 0, NULL);
  peer->browse = NULL;

}

static file_node_t*
collect_shared_files(file_tree_t* tree, file_node_t* node, GList** folder) {
  GList* dlist;
  fbucket_t* bucket = NULL;

  (void)tree;

  if (!node->parent) {
    g_log("SLSK_P", G_LOG_LEVEL_WARNING,
          "a file in library has no parent");
    return NULL;
  }
  for (dlist = *folder; dlist; dlist = dlist->next) {
    bucket = dlist->data;
    if (bucket->folder == node->parent) break;
  }
  if (!dlist) bucket = NULL;
  if (!bucket) {
    bucket = g_malloc0(sizeof(*bucket));
    bucket->folder = node->parent;
    *folder = g_list_append(*folder, bucket);
  }
  bucket->files = g_list_append(bucket->files, node);
  return NULL;
}

PHANDLER(handle_browse_request) {
  GList* folder = NULL;
  buffer_t* buffer = buffer_new(1024);
  buffer_t* out;
  guint32 num_folder;
  compress_t* deflate;
  GList* dlist;
  GList* dlist2;
  int sstart;

  (void)data;
  (void)dlen;

  out = b_socket_get_out_buffer(BSOCK(peer), 1024);
  sstart = out->size;
  out->size += 8;
  
  deflate = compress_new(COMPRESS_ZLIB, TRUE, buffer, out);
  if (!deflate) {
    buffer_destroy(buffer);
    return;
  }

  library_foreach_shared((FileNodeTraverse)collect_shared_files, &folder);

  // now pack the files
  num_folder = g_list_length(folder);
  buffer->size = pack_u32(buffer->data+buffer->size, num_folder);

  for (dlist = folder; dlist; dlist = dlist->next) {
    fbucket_t* bucket = dlist->data;
    guint32 num_files = g_list_length(bucket->files);
    int dlen = 0;
    char* dir = file_node_get_folder(bucket->folder, &dlen);
    char* temp;

    // make sure we have enough space in buffer
    buffer_need_space(buffer, dlen+100);
    
    local2win(dir);  // convert this for the windows user
    
    // remove the trailing (back)slash
    if (dir[dlen-1] == '\\') dir[dlen-1] = 0;
    temp = auto_text(from_utf8(dir, -1, NULL, NULL, 2, slsk_charset_get(), "utf-8"));  //FIXME:
    buffer->size += pack_string_u32(buffer->data+buffer->size, temp);
    g_free(dir);

    buffer->size += pack_u32(buffer->data+buffer->size, num_files);
    for (dlist2 = bucket->files; dlist2; dlist2 = dlist2->next) {
      file_node_t* node = dlist2->data;
      file_t* file = node->file;

      buffer_need_space(buffer, strlen(file->filename) + 200);
      buffer->size += slsk_pack_file(buffer->data+buffer->size, file, FALSE);

      if (!dlist2->next && !dlist->next) {
        compress_action(deflate, TRUE);
      } else {
        compress_action(deflate, FALSE);
      }
    }
    g_list_free(bucket->files);
    g_free(bucket);
  }
  // need to finish the buffer if we have no folders.
  if (!folder) compress_action(deflate, TRUE);

  { // debug output
    slsk_message_t command;
    pack_u32((char*)&num_folder, num_folder);
    command.type = SLSK_X_BROWSE_RESULT;
    command.length = buffer->size;
    command.data = (char*)&num_folder;
    slsk_message(&command, 'S', SLSK_CON(peer), G_LOG_LEVEL_DEBUG, 
		 VAL_INT, VAL_INC);
  }

  g_assert(!buffer_avail(buffer));
  g_list_free(folder);

  compress_destroy(deflate);
  buffer_destroy(buffer);

  // finally add the header length
  pack_u32(out->data+sstart, out->size-4-sstart);
  pack_u32(out->data+sstart+4, SLSK_X_BROWSE_RESULT);
}

PHANDLER(handle_whois_request) {
  buffer_t* buffer;
  int sstart;
  char* temp;
  char* pic = NULL;
  gsize size = 0;

  (void)data;
  (void)dlen;

  buffer = b_socket_get_out_buffer(BSOCK(peer), 1024);

  sstart = buffer->size;
  buffer->size += 4;
  buffer->size += pack_u32(buffer->data+buffer->size, SLSK_X_WHOIS_RESULT);

  temp = pref_get_ptr("/core/user/description", char);
  buffer_need_space(buffer, strlen(temp?temp:"") + 4);
  buffer->size += pack_string_u32(buffer->data+buffer->size, temp?temp:"");

  temp = pref_get_ptr("/core/user/picture", char);
  if (temp && g_file_get_contents(temp, &pic, &size, NULL)) {
    buffer_need_space(buffer, size+5 + 9);
    buffer->size += pack_byte(buffer->data+buffer->size, 1);
    buffer->size += pack_u32(buffer->data+buffer->size, size);
    buffer->size += pack_string_raw(buffer->data+buffer->size, pic, size);
    g_free(pic);
  } else {
    buffer_need_space(buffer, 1 + 9);
    buffer->size += pack_byte(buffer->data+buffer->size, 0);
  }
  // slots 
  buffer->size += pack_u32(buffer->data+buffer->size, uploads_slots_number());
  // queued
  buffer->size += pack_u32(buffer->data+buffer->size, uploads_queue_size());
  // have slots
  buffer->size += pack_byte(buffer->data+buffer->size, uploads_slots_free()<=0?0:1);

  // finally add the header length
  pack_u32(buffer->data+sstart, buffer->size-4-sstart);

}


PHANDLER(handle_search_request) {
  int token;
  char* item;

  REQ_LEN(4);
  token = unpack_u32(&data, &dlen);
  item = unpack_string_u32(&data, &dlen);
  CHECK(item);

  CHECK_LEN(0);
  
  // the next function automatically uses this open con for the results
  slsk_search_process(peer->user, token, item, 49);
}

PHANDLER(handle_queue_upload) {
  char* filename;
  upload_t* upload;
  
  filename = unpack_string_u32(&data, &dlen);
  CHECK(filename);

  upload = upload_find(USER(peer->user), filename);
  
  if (!upload) {
    file_t* file = library_winname_requested(filename);
    if (!file) {
      g_log("SLSK_UPLOAD", G_LOG_LEVEL_WARNING, "'%s' requested file not shared '%s'",
            OBJ(peer->user)->name, filename);
      slsk_peer_send(peer, SLSK_S_QUEUE_FAILED, filename, "File not shared");
      return;  // not found or not shared or whatever
    }
    upload = UPLOAD(slsk_upload_new(peer, file, 0));
    upload_insert(upload, FALSE);
  } else {
    g_log("SLSK_UPLOAD", G_LOG_LEVEL_DEBUG, "enable '%s' for '%s'",
          upload->file->filename,  OBJ(peer->user)->name);
    //    upload_renew(upload);
  }
}

PHANDLER(handle_queue_in) {
  char* filename;
  upload_t* upload;
  
  filename = unpack_string_u32(&data, &dlen);
  CHECK(filename);

  upload = upload_find(USER(peer->user), filename);

  if (!upload) {
    file_t* file = library_winname_requested(filename);
    if (!file) {
      g_log("SLSK_UPLOAD", G_LOG_LEVEL_WARNING,
	    "'%s' requested file not shared '%s'",
	    OBJ(peer->user)->name, filename);
      slsk_peer_send(peer, SLSK_S_QUEUE_FAILED, filename,
		     "File not shared");
      return;  // not found or not shared or whatever
    }
    upload = UPLOAD(slsk_upload_new(peer, file, 0));
    upload_insert(upload, FALSE);
  } else {
    g_log("SLSK_UPLOAD", G_LOG_LEVEL_DEBUG, "re-queue '%s' for '%s'", 
          upload->file->filename,  OBJ(peer->user)->name);
    //    upload_renew(upload, TRUE);
  }
  
}

PHANDLER(handle_upload_failed) {
  char* filename = unpack_string_u32(&data, &dlen);

  CHECK(filename);
  g_log("SLSK_DOWN", G_LOG_LEVEL_WARNING,
        "'%s' upload failed", filename);

  // FIXME: do something
}

PHANDLER(handle_transfer_request) {
  guint32 dir;
  guint32 token;
  char* filename;
  
  REQ_LEN(8);
  dir = unpack_u32(&data, &dlen);
  token = unpack_u32(&data, &dlen);
  filename = unpack_string_u32(&data, &dlen);
  CHECK(filename);

  if (dir == 0) {
    upload_t* upload = upload_find(USER(peer->user), filename);
    if (!upload) {
      file_t* file = library_winname_requested(filename);
      if (!file) {
        g_log("SLSK_UPLOAD", G_LOG_LEVEL_WARNING,
              "'%s' requested file not shared '%s'",
              OBJ(peer->user)->name, filename);
        slsk_peer_send(peer, SLSK_X_TRANS_RESPONSE, token,
                       0, "File not shared.");
        return;  // not found or not shared or whatever
      }
      g_log("SLSK_UPLOAD", G_LOG_LEVEL_WARNING,
            "create un-queued upload '%s' for '%s'",
            filename, OBJ(peer->user)->name);
      upload = UPLOAD(slsk_upload_new(peer, file, token));
      upload_insert(upload, TRUE);
    } else {
      g_log("SLSK_UPLOAD", G_LOG_LEVEL_DEBUG, "enable '%s' for '%s'",
            upload->file->filename, OBJ(peer->user)->name);
      SLSK_UPLOAD(upload)->token = token;
      upload_renew(upload);
    }
    CHECK_LEN(0);
  } else {
    file_size_t size;
    slsk_source_t* source;

    REQ_LEN(8);
    size = unpack_u32d(&data, &dlen);
    CHECK_LEN(0);

    source = SLSK_SOURCE(source_find(USER(peer->user), filename));
    if (source) {
      source->id = token;
      slsk_peer_download(source);
    } else {
      g_log("SLSK_DOWN", G_LOG_LEVEL_WARNING,
            "'%s' offered file not requested '%s'",
            OBJ(peer->user)->name, filename);
    }
  }

}

PHANDLER(handle_transfer_response) {
  guint32 token;
  char allowed;
  file_size_t size = 0;
  char* reason = NULL;
  slsk_source_t* source;

  REQ_LEN(5);
  token = unpack_u32(&data, &dlen);
  allowed = unpack_byte(&data, &dlen);
  source = slsk_source_find_by_id(token);

  if (!source) {
    g_log("SLSK_DOWN", G_LOG_LEVEL_WARNING,
          "source for token %u not found", token);
    return;
  }
  if (SOURCE(source)->user != USER(peer->user)) {
    g_log("SLSK_DOWN", G_LOG_LEVEL_WARNING,
          "%s responded to download %s owned by %s", OBJ(peer->user)->name,
          SOURCE(source)->file->filename, OBJ(SOURCE(source)->user)->name);
    return;
  }

  if (allowed) {
    REQ_LEN(8);
    size = unpack_u32d(&data, &dlen);
    CHECK_LEN(0);
    if (SOURCE(source)->file->size != size) {
      g_log("SLSK_DOWN", G_LOG_LEVEL_WARNING, "file size mismatch "FF" "FF, 
            size, SOURCE(source)->file->size);
      return;
    }
    slsk_peer_download(source);
  } else {
    reason = unpack_string_u32(&data, &dlen);
    CHECK(reason);
    CHECK_LEN(0);
    if (!strcmp("File not shared.", reason)) {
      socket_done(SOCK(source), ST_ERROR, "file not shared");
    } else if (!strcmp("Queued", reason)) {
      socket_done(SOCK(source), ST_SOURCE_REMOTE, "remotely queued");
    } else {
      socket_done(SOCK(source), ST_ERROR, l_strdup(reason));
    }
  }
}

PHANDLER(handle_place_inqueue) {
  char* filename;
  unsigned pos;
  source_t* source;

  filename = unpack_string_u32(&data, &dlen);
  CHECK(filename);
  
  REQ_LEN(4);
  pos = unpack_u32(&data, &dlen);

  CHECK_LEN(0);
  
  source = source_find(USER(peer->user), filename);
  if (!source) {
    g_log("SLSK_PEER", G_LOG_LEVEL_WARNING,
          "source %s not found", filename);
    return;
  }
  source_set_remote(source, 0, pos);
}

static const char* slsk_p_handle_message(slsk_peer_t* peer, slsk_message_t* command) {
  guint32 len;
  char* data;
  slsk_con_t* con = SLSK_CON(peer);

  switch (command->type) {
  case SLSK_X_SEARCH_RESULT:
    if (!(peer->direction & SLSK_P_INCOMING)) {
      g_log("SLSK_P", G_LOG_LEVEL_WARNING,
            "this should be an incoming connection (search result)");
      peer->direction |= SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
                 VAL_STRING, VAL_INT, VAL_INT, VAL_INC);
    SLSK_PCALL(handle_search_result);
    slsk_peer_remove_save_action(peer);
    break;
  case SLSK_X_BROWSE_RESULT:
    if (peer->direction & SLSK_P_INCOMING) {
      peer->direction &= ~SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, VAL_INT, VAL_INC);
    SLSK_PCALL(handle_browse_result);
    slsk_peer_remove_save_action(peer);
    break;
  case SLSK_X_WHOIS_RESULT:
    if (peer->direction & SLSK_P_INCOMING) {
      peer->direction &= ~SLSK_P_INCOMING;
    }
    // check whether we have a pic, to customize the debug output
    data = command->data;
    len = 4;
    len = unpack_u32(&data, &len);
    data += len;
    if (*data)
      slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
		   VAL_STRING, VAL_BYTE, VAL_BIN, 
		   VAL_INT, VAL_INT, VAL_BYTE, VAL_NONE);
    else
      slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
		   VAL_STRING, VAL_BYTE, 
		   VAL_INT, VAL_INT, VAL_BYTE, VAL_NONE);
    SLSK_PCALL(handle_whois_result);
    slsk_peer_remove_save_action(peer);
    break;
  case SLSK_X_TRANS_RESPONSE:
    if (peer->direction & SLSK_P_INCOMING) {
      peer->direction &= ~SLSK_P_INCOMING;
    }
    if (command->data[4]) {
      slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
                   VAL_INT, VAL_BYTE, VAL_DINT, VAL_NONE);
    } else {
      slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
                   VAL_INT, VAL_BYTE, VAL_STRING, VAL_NONE);
    }
    SLSK_PCALL(handle_transfer_response);
    slsk_peer_remove_save_action(peer);
    break;
  case SLSK_X_BROWSE_REQUEST:
    if (!(peer->direction & SLSK_P_INCOMING)) {
      g_log("SLSK_P", G_LOG_LEVEL_WARNING,
	    "this should be an incoming connection (browse_request)");
      peer->direction |= SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
		 VAL_NONE);
    SLSK_PCALL(handle_browse_request);
    break;
  case SLSK_X_WHOIS_REQUEST:
    if (!(peer->direction & SLSK_P_INCOMING)) {
      g_log("SLSK_P", G_LOG_LEVEL_WARNING,
	    "this should be an incoming connection (whois_request)");
      peer->direction |= SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
		 VAL_NONE);
    SLSK_PCALL(handle_whois_request);
    break;
  case SLSK_X_SEARCH_REQUEST:
    if (!(peer->direction & SLSK_P_INCOMING)) {
      g_log("SLSK_P", G_LOG_LEVEL_WARNING,
            "this should be an incoming connection (search_request)");
      peer->direction |= SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
                 VAL_INT, VAL_STRING, VAL_NONE);
    SLSK_PCALL(handle_search_request);
    break;
  case SLSK_R_QUEUE_UPLOAD:  // subsequent request?
    if (!(peer->direction & SLSK_P_INCOMING)) {
      g_log("SLSK_P", G_LOG_LEVEL_WARNING,
            "this should be an incoming connection (queue_upload)");
      peer->direction |= SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
		 VAL_STRING, VAL_NONE);
    SLSK_PCALL(handle_queue_upload);
    break;
  case SLSK_X_QUEUE_IN:      // used initially?
    if (!(peer->direction & SLSK_P_INCOMING)) {
      g_log("SLSK_P", G_LOG_LEVEL_WARNING,
            "this should be an incoming connection (queue_upload)");
      peer->direction |= SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
                 VAL_STRING, VAL_NONE);
    SLSK_PCALL(handle_queue_in);
    break;
  case SLSK_X_UPLOAD_FAILED:
    if (peer->direction & SLSK_P_INCOMING) {
      peer->direction &= ~SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
                 VAL_STRING, VAL_NONE);
    SLSK_PCALL(handle_upload_failed);
    break;
  case SLSK_X_TRANS_REQUEST:
    if (!(peer->direction & SLSK_P_INCOMING)) {
      g_log("SLSK_P", G_LOG_LEVEL_WARNING,
            "this should be an incoming connection (trans_request)");
      peer->direction |= SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
                 VAL_INT, VAL_INT, VAL_STRING, VAL_NONE);
    SLSK_PCALL(handle_transfer_request);
    break;
  case SLSK_R_PLACE_INQUEUE:
    if (peer->direction & SLSK_P_INCOMING) {
      peer->direction &= ~SLSK_P_INCOMING;
    }
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
                 VAL_STRING, VAL_INT, VAL_NONE);
    SLSK_PCALL(handle_place_inqueue);
    // we don't request the place in queue, so don't remove any action
/*     slsk_peer_remove_save_action(peer); */
    break;
  default:
    slsk_message(command, 'R', con, G_LOG_LEVEL_WARNING,
		 VAL_NONE);
    break;
  }
  return NULL;
}

PHANDLER(handle_distrib_search) {
  static unsigned cnt;
  static unsigned cnt1;
  static unsigned cnt2;
  static time_t start_time;

  guint32 temp;
  char* user;
  int token;
  char* items;
#define SLSK_HARD_LIMIT 10
  static int SearchLimit = SLSK_HARD_LIMIT;
  static int SearchTime = 0;

  peer->search_cnt++;

  REQ_LEN(4);
  temp = unpack_u32(&data, &dlen);
  user = unpack_string_u32(&data, &dlen);
  CHECK(user); REQ_LEN(4);
  token = unpack_u32(&data, &dlen);
  items = unpack_string_u32(&data, &dlen);
  CHECK(items);

  if (++cnt == 1) start_time = CurrentTime;
  switch (slsk_user_get_privileges(user)) {
  case SLSK_PRIV_ON:
    cnt1++; break;
  case SLSK_PRIV_OFF:
    cnt2++; break;
  }
  if ((cnt % 100) == 0)
    g_log("SLSK_PEER", G_LOG_LEVEL_WARNING,
	  "-- %u searches in %lu secs (%lu/min)"
	  " | %u from priv (%.1f%%) | %u from non-priv (%.1f%%)",
	  cnt, (CurrentTime-start_time),
	  (start_time<CurrentTime)?(cnt*60/(CurrentTime-start_time)):0,
	  cnt1, (double)cnt1/(double)cnt*100.,
	  cnt2, (double)cnt2/(double)cnt*100.);

  if (SearchTime < CurrentTime) {
    SearchTime = CurrentTime;
    SearchLimit = SLSK_HARD_LIMIT;
  }
  if (!SearchLimit) return;
  SearchLimit--;
  
  slsk_search_process(slsk_user_find(user), token, items, temp);
}

static const char*
slsk_d_handle_message(slsk_peer_t* peer, slsk_message_t* command) {
  slsk_con_t* con = SLSK_CON(peer);

  switch (command->type) {
  case SLSK_R_DISTRIB_ALIVE:
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, VAL_NONE);
    // TEST: close connection if we didnt get searches between two pings
    if (peer->search_cnt == 0) return "got no search requests";
    else peer->search_cnt = 0;
    break;
  case SLSK_R_DISTRIB_SEARCH:
    /*
    slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, 
		 VAL_INT, VAL_STRING, VAL_INT, VAL_STRING, VAL_NONE);
    */
    SLSK_PCALL(handle_distrib_search);
    break;
  default:
    slsk_message(command, 'R', con, G_LOG_LEVEL_WARNING,
		 VAL_NONE);
    break;
  }
  return NULL;
}

static const char*
slsk_f_handle_message(slsk_peer_t* peer, slsk_message_t* command) {
  slsk_con_t* con = SLSK_CON(peer);
  switch (command->type) {
  default:
    slsk_message(command, 'R', con, G_LOG_LEVEL_WARNING,
		 VAL_NONE);
    break;
  }
  return NULL;
}

const char* slsk_peer_consume_input(slsk_peer_t* peer) {
  buffer_t* buffer = b_socket_get_in_buffer(BSOCK(peer));
  static slsk_message_t result;
  guint32 len;
  char* data;
  compress_t* inflate;
  buffer_t* decomp;
  const char* message;
  unsigned avail;


  if (!buffer) return NULL;
  avail = buffer_avail(buffer);

  if (avail < 4) return NULL;
/*   print_raw(l_printf("%p", con), */
/*             buffer_read_pos(buffer), buffer_avail(buffer)); */

  while (avail > 4) {
    data = buffer_read_pos(buffer);

    len = unpack_u32(&data, &avail);
    if (SLSK_CON(peer)->type == 'D') {
      if (len < 1) return "got len < 1"; /*not possible*/
      result.type = unpack_byte(&data, &avail);
      len -= 1;
    } else {
      if (len < 4) return "got len < 4";
      result.type = unpack_u32(&data, &avail);
      len -= 4;
      if (result.type == SLSK_X_WHOIS_RESULT)
        slsk_whois_set_progress(peer->user, data, avail);
      if (result.type == SLSK_X_BROWSE_RESULT && avail < len)
        slsk_browse_set_progress(peer->user,(double)avail/(double)len, NULL);
    }
    if (avail < len) return NULL;

    buffer_consume_virt(buffer, buffer_avail(buffer)-avail);

    decomp = NULL;
    switch (result.type) {
      /* compressed stuff */
    case SLSK_X_SEARCH_RESULT:
    case SLSK_X_BROWSE_RESULT:
      decomp = buffer_new(1024);
      inflate = compress_new(COMPRESS_ZLIB, FALSE, buffer, decomp);
      if (!inflate) {
        buffer_destroy(decomp);
        return "could not init zlib";
      }
      if (!compress_action(inflate, TRUE)) {
        compress_destroy(inflate);
        buffer_destroy(decomp);
        return "could not inflate";
      }
      compress_destroy(inflate);

      result.length = decomp->size;
      result.data = decomp->data;
      break;
      /* not compressed stuff */
    case SLSK_X_WHOIS_RESULT:
    case SLSK_X_BROWSE_REQUEST:
    case SLSK_X_WHOIS_REQUEST:
    case SLSK_X_SEARCH_REQUEST:
    case SLSK_R_DISTRIB_ALIVE:
    case SLSK_R_DISTRIB_SEARCH:
    case SLSK_R_QUEUE_UPLOAD:
    case SLSK_X_QUEUE_IN:
    case SLSK_X_TRANS_REQUEST:
    case SLSK_X_TRANS_RESPONSE:
    case SLSK_X_UPLOAD_FAILED:
      result.data = buffer_read_pos(buffer);
      result.length = len;
      buffer_consume_virt(buffer, len);
      break;
    default:
      printf("unhandled %u len %u\n", result.type, len);
      result.data = 0;
      if (len > 0) print_raw(NULL,  data, len);
      buffer_consume_virt(buffer, len);
      break;
    }

    if (result.data) {
      switch (SLSK_CON(peer)->type) {
      case 'P':
        message = slsk_p_handle_message(peer, &result);
        break;
      case 'D':
        message = slsk_d_handle_message(peer, &result);
        break;
      case 'F':
        message = slsk_f_handle_message(peer, &result);
        break;
      default:
        message = NULL;
      }
    } else {
      message = NULL;
    }
    if (decomp) buffer_destroy(decomp);
    if (message) return message;
    avail = buffer_avail(buffer);
  }

  buffer_consume(buffer, 0);

  // check whether we are going to send something
  if (buffer_avail(b_socket_get_out_buffer(BSOCK(peer), 0)))
    b_socket_start_sending(BSOCK(peer));

  return NULL;
}

/*
SLSK_PEER  | R:   1 L: 00019 D: 'b0ne99' 'P' 0
SOCK       | ending 0x83e3d40 : (tsocket->slsk_peer) status:7 fd:-1
SOCK       | destroying 0x83e3d40
unhandled 1537 len 16
[[00]b0ne99[01][00][00][00]P[00][00][00][00]]

*** SLSK_PEER *** : got unknown attribute: 3 - 16187264 - c:\documents and settings\owner\desktop\soulseek\=good looking records=\glr036 - makoto - enterprise & sweet changes\01. enterprise.mp3
*** SLSK_PEER *** : got unknown attribute: 3 - 16187264 - c:\documents and settings\owner\desktop\soulseek\=good looking records=\glr036 - makoto - enterprise & sweet changes\02. sweet changes.mp3


*/
static void
slsk_pack_peer_command(slsk_peer_t* peer, guint32 type, va_list ap) {
  buffer_t* buffer;
  char* pos;
  slsk_message_t m;
  int token;
  slsk_con_t* con = SLSK_CON(peer);

  // FIXME: we assume that we dont need more space than 4096 bytes
  buffer = b_socket_get_out_buffer(BSOCK(peer), 4096);
  pos = buffer_write_pos(buffer) + 4;

  if (type == SLSK_X_PIERCE_FW || type == SLSK_X_PEER_INIT) {
    pos += pack_byte(pos, (char)type);
  } else {
    pos += pack_u32(pos, type);
  }
  m.type = type;
  m.data = pos;
  m.length = 0;
  switch (type) {
  case SLSK_X_PIERCE_FW:
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		 VAL_INT, VAL_NONE);
    break;
  case SLSK_X_PEER_INIT:
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    m.length += pack_u32(pos+m.length, 1);
    m.length += pack_byte(pos+m.length, va_arg(ap, int));
    m.length += pack_u32(pos+m.length, 0);
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		 VAL_STRING, VAL_STRING, VAL_INT, VAL_NONE);
    break;
  case SLSK_X_WHOIS_REQUEST:
  case SLSK_X_BROWSE_REQUEST:
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG, VAL_NONE);
    break;
  case SLSK_S_QUEUE_FAILED:
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		 VAL_STRING, VAL_STRING, VAL_NONE);
    break;
  case SLSK_X_SEARCH_REQUEST:
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		 VAL_INT, VAL_STRING, VAL_NONE);
    break;
  case SLSK_X_UPLOAD_FAILED:
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		 VAL_STRING, VAL_NONE);
    break;
  case SLSK_X_QUEUE_IN:
  case SLSK_S_QUEUE_POS:
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		 VAL_STRING, VAL_NONE);
    break;
  case SLSK_X_TRANS_REQUEST:
    token = va_arg(ap, int);
    m.length += pack_u32(pos+m.length, token);
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    if (token == 1) {// upload
      m.length += pack_u32d(pos+m.length, va_arg(ap, file_size_t));
      slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		   VAL_INT, VAL_INT, VAL_STRING, VAL_DINT, VAL_NONE);
    } else {
      slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		   VAL_INT, VAL_INT, VAL_STRING, VAL_NONE);
    }
    break;
  case SLSK_X_TRANS_RESPONSE:
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    token = va_arg(ap, int);
    m.length += pack_byte(pos+m.length, token);
    if (token) {
      m.length += pack_u32d(pos+m.length, va_arg(ap, file_size_t));
      slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		   VAL_INT, VAL_BYTE, VAL_DINT, VAL_NONE);
    } else {
      m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
      slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		   VAL_INT, VAL_BYTE, VAL_STRING, VAL_NONE);
    }
    break;
  case SLSK_S_SEARCH: //from slsk_proto.h
    // int : string
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
		 VAL_INT, VAL_STRING, VAL_NONE);
    break;
  default:
    g_log("SLSK_P", G_LOG_LEVEL_WARNING,
	  "unknown command %d", type);
    return;
  }

  pos = buffer_write_pos(buffer);
  if (type == SLSK_X_PIERCE_FW || type == SLSK_X_PEER_INIT) {
    pack_u32(pos, m.length+1);
    buffer->size += (m.length+5);
  } else {
    pack_u32(pos, m.length+4);
    buffer->size += (m.length+8);
  }
}

void slsk_peer_send(slsk_peer_t* con, guint32 type, ...) {
  va_list ap;
  
  va_start(ap, type);
  slsk_pack_peer_command(con, type, ap);
  va_end(ap);
  b_socket_start_sending(BSOCK(con));
}
