/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.2 $
 * $Date: 2007/05/23 20:42:29 $
 * $Author: sid4u $
 */

#include <string.h>
#include <stdlib.h>
#ifndef WIN32
#include <netinet/in.h>
#endif
#include <stdio.h>

#include "lcore/utils.h"
#include "lcore/preferences.h"
#include "lcore/search.h"
#include "lcore/mtypes.h"
#include "lcore/attribute.h"
#include "slsk_proto.h"
#include "slsk_chat.h"
#include "slsk.h"
#include "slsk_network.h"
#include "slsk_peer.h"
#include "slsk_users.h"
#include "slsk_hotlist.h"
#include "slsk_til.h"
#include "slsk_ui.h"
#include "slsk_search.h"
#include "slsk_users.h"
#include "slsk_con.h"

#define SLSK_R_LOGIN                 1
#define SLSK_R_GET_ADDRESS           3
#define SLSK_R_ADD_USER              5
#define SLSK_R_USER_STATUS           7
#define SLSK_R_PUBLIC_MESSAGE       13
#define SLSK_R_CHANNEL_USER_LIST    14
#define SLSK_R_PART_CHANNEL         15
#define SLSK_R_USER_JOIN            16
#define SLSK_R_USER_PART            17
#define SLSK_R_PEER_REQUEST         18
#define SLSK_R_PRIVMSG              22
#define SLSK_R_SEARCH               26
#define SLSK_R_USER_STATS           36
#define SLSK_R_LOCAL_RECO           54
#define SLSK_R_GLOBAL_RECO          56
#define SLSK_R_ROOM_ADDED           62
#define SLSK_R_ROOM_REMOVED         63
#define SLSK_R_ROOM_LIST            64
#define SLSK_R_EXACT_SEARCH         65
#define SLSK_R_GLOBAL               66
#define SLSK_R_USER_LIST            69
#define SLSK_R_PARENT_TIMEOUT       86
#define SLSK_R_MIN_PARENTS          88
#define SLSK_R_PARENT_INTERVAL      90
#define SLSK_R_NEW_PRIVILEGED       91
#define SLSK_R_CHECK_PRIVILEGES     92
#define SLSK_R_NET_INFO            102
#define SLSK_R_SIMILAR_USERS       110
#define SLSK_R_ITEM_RECO           111
#define SLSK_R_ITEM_USERS          112
#define SLSK_R_TICKER_STATE        113
#define SLSK_R_TICKER_ADD          114
#define SLSK_R_TICKER_REMOVE       115

/*
SetStatus:28,ServerPing:32,SendSpeed:34,
QueuedDownloads:40,Relogged:41,
PlaceInLineResponse:60,
AdminMessage:66, 
GlobalUserList:67,TunneledMessage:68,
,
SearchInactivityTimeout:87,
MinParentsInCache:88,
HaveNoParent:71,SearchRequest:93,
WishlistSearch:103,WishlistInterval:104

ID:26 LEN:27
 Merius
 [e5][04][00][00]
 middleton
*/
/*
received message 5: 20 byte more than expected.
 Q   [09][00][00] [f6][0e][00][00] [00][00][00][00] J   [09][00][00] L   [01][00][00]
 [fc]6   [00][00] \   [00][00][00] [00][00][00][00] [9e][00][00][00] [0f][00][00][00]
 /   [05][00][00] [04]u   [00][00] [00][00][00][00] [dc]U   [00][00] [89][07][00][00]
 [00][00][00][00] [00][00][00][00] [00][00][00][00] 3   [02][00][00] D   [00][00][00]
 [10][1f][00][00] [8a]e   [00][00] [00][00][00][00] [f5][10][00][00] G   [01][00][00]
 [80][19][00][00] [04][00][00][00] [00][00][00][00] [00][00][00][00] [00][00][00][00]
 [f7][09][00][00] 3   [00][00][00] [00][00][00][00] [c9][02][00][00] Q   [00][00][00]
 [c9][19][00][00] ^   [06][00][00] [00][00][00][00] [e9][03][00][00] I   [00][00][00]
das_benny (priv)
 7   I   [00][00] e   [04][00][00] [00][00][00][00] [ca][00][00][00] [07][00][00][00]
dj24jp (priv)
 [94][ed][00][00] [12][00][00][00] [00][00][00][00] [\n][00][00][00] [01][00][00][00]
jazzus
 +   [06][00][00] [a6]@   [00][00] [00][00][00][00] [b0][0e][00][00] [e4][01][00][00]]

 */

#define HANDLER(f) static void f (slsk_net_t* net, char* data, unsigned dlen)
#define SLSK_CALL(f) f(net, command->data, command->length)

HANDLER(handle_got_address) {
  char* user;
  unsigned long ip;
  int port;
  
  (void)net;
  user = unpack_string_u32(&data, &dlen);
  CHECK(user); REQ_LEN(8);
  ip = unpack_u32(&data, &dlen);
  port = unpack_u32(&data, &dlen);
  CHECK_LEN(0);

  if (ip && port) {
    slsk_peer_got_address(net, user, htonl(ip), 
			  htons((unsigned short)port));
  }
}

HANDLER(handle_user_status)
{
    char* nick;
    int status;
    gboolean priv;
    slsk_user_t* user;
    
    (void)net;
    
    nick = unpack_string_u32(&data, &dlen);
    CHECK(nick); REQ_LEN(5);
    status = unpack_u32(&data, &dlen);
    priv = unpack_byte(&data, &dlen);
    CHECK_LEN(0);
    
    user = slsk_user_find(nick);
    slsk_user_set_net(user, net);
    slsk_user_set_privileges(user, priv?SLSK_PRIV_ON:SLSK_PRIV_OFF);
    slsk_user_set_status(user, status);
}

HANDLER(handle_public_message) {
  char* channel;
  char* user;
  char* message;
  
  channel = unpack_string_u32(&data, &dlen);
  CHECK(channel);
  user = unpack_string_u32(&data, &dlen);
  CHECK(user);
  message = unpack_string_u32(&data, &dlen);
  CHECK(message);

  CHECK_LEN(0);

  if (!strncmp(message, "/me ", 4))
    slsk_emote_message(net, channel, user, message+4);
  else
    slsk_public_message(net, channel, user, message);

}

HANDLER(handle_channel_user_list) {
  unsigned number, i1;
  char* string;
  slsk_user_t** cu;
  slsk_cp_t* scp;
  
  string = unpack_string_u32(&data, &dlen);
  CHECK(string);
  scp = slsk_cp_search_net(net, string, SLSK_PAGE_PUBLIC);
  if (!scp) {
    // we havent requested the join 
    // (or maybe we closed the page ourselfs)
    slsk_net_send(net, SLSK_S_PART_CHANNEL, string);
    return;
  }

  REQ_LEN(4);
  number = unpack_u32(&data, &dlen);
  cu = g_malloc(sizeof(*cu)*number);
  for (i1 = 0; i1 < number; i1++) {
    string = unpack_string_u32(&data, &dlen);
    CHECK(string);
    cu[i1] = slsk_user_find(string);
    slsk_user_set_net(cu[i1], net);
  }
  
  REQ_LEN(4);
  number = unpack_u32(&data, &dlen);
  REQ_LEN(4*number);
  for (i1 = 0; i1 < number; i1++) {
    slsk_user_set_status(cu[i1], unpack_u32(&data, &dlen));
  }

  REQ_LEN(4);
  number = unpack_u32(&data, &dlen);
  REQ_LEN(number*4*5);
  for (i1 = 0; i1 < number; i1++) {
    int speed, down, shared, dirs;
    speed = unpack_u32(&data, &dlen);
    down = unpack_u32(&data, &dlen);
    unpack_u32(&data, &dlen);
    shared = unpack_u32(&data, &dlen);
    dirs = unpack_u32(&data, &dlen);
    slsk_user_set_info(cu[i1], speed, down, shared, dirs);
  }
  
  REQ_LEN(4);
  number = unpack_u32(&data, &dlen);
  REQ_LEN(4*number);
  for (i1 = 0; i1 < number; i1++) {
    unpack_u32(&data, &dlen);
    slsk_user_joined(scp, cu[i1], FALSE);
  }
  CHECK_LEN(0);
  g_free(cu);
}

HANDLER(handle_part_channel) {
  slsk_cp_t* scp;
  char* channel;

  channel = unpack_string_u32(&data, &dlen);
  CHECK(channel);
  CHECK_LEN(0);

  scp = slsk_cp_search_net(net, channel, SLSK_PAGE_PUBLIC);

  if (!scp) return;

  slsk_channel_parted(scp);

}

HANDLER(handle_user_join) {
  slsk_cp_t* scp;
  char* channel;
  slsk_user_t* cu;
  static gboolean skip = TRUE;
  
  // just a workaround since soulseek sends the join message twice
  skip = !skip;
  if (skip) return;

  channel = unpack_string_u32(&data, &dlen);
  CHECK(channel);

  scp = slsk_cp_search_net(net, channel, SLSK_PAGE_PUBLIC);
  if (scp) {
    char* nick = unpack_string_u32(&data, &dlen);
    int speed, down, shared, dirs;

    CHECK(nick);
    REQ_LEN(7*4);

    cu = slsk_user_find(nick);
    slsk_user_set_net(cu, net);
    slsk_user_set_status(cu, unpack_u32(&data, &dlen));
    
    speed = unpack_u32(&data, &dlen);
    down = unpack_u32(&data, &dlen);
    unpack_u32(&data, &dlen);
    shared = unpack_u32(&data, &dlen);
    dirs = unpack_u32(&data, &dlen);
    unpack_u32(&data, &dlen);
    
    slsk_user_set_info(cu, speed, down, shared, dirs);

    if (!strcmp(OBJ(cu)->name, net->nick))
      slsk_channel_joined(scp);
    slsk_user_joined(scp, cu, TRUE);
  }
  CHECK_LEN(0);
}

HANDLER(handle_user_part) {
  slsk_cp_t* scp;
  char* channel;
  char* user;
  
  channel = unpack_string_u32(&data, &dlen);
  CHECK(channel);
  user = unpack_string_u32(&data, &dlen);
  CHECK(user);

  CHECK_LEN(0);

  scp = slsk_cp_search_net(net, channel, SLSK_PAGE_PUBLIC);
  if (scp) slsk_user_parted(scp, user);
}

HANDLER(handle_peer_request) {
  char* nick;
  unsigned long ip;
  int port;
  int token;
  char* mode;
  char priv;
  slsk_user_t* user;

  (void)net;
  nick = unpack_string_u32(&data, &dlen);
  CHECK(nick);
  mode = unpack_string_u32(&data, &dlen);
  CHECK(mode);
  REQ_LEN(12);
  ip = htonl(unpack_u32(&data, &dlen));
  port = htons((unsigned short)(unpack_u32(&data, &dlen)));
  token = unpack_u32(&data, &dlen);
  priv = unpack_byte(&data, &dlen);
  CHECK_LEN(0);

  user = slsk_user_find(nick);
  slsk_user_set_net(user, net);
  slsk_user_set_ip_port(user, ip, port);
  slsk_user_set_privileges(user, priv?SLSK_PRIV_ON:SLSK_PRIV_OFF);
  slsk_peer_request_fw(user, *mode, token);
}

HANDLER(handle_priv_msg) {
  char* user;
  char* message;
  int id;
  int timestamp;

  REQ_LEN(8);

  id = unpack_u32(&data, &dlen);
  timestamp = unpack_u32(&data, &dlen);
  user = unpack_string_u32(&data, &dlen);
  CHECK(user);
  message = unpack_string_u32(&data, &dlen);
  CHECK(user);

  CHECK_LEN(0);
  
  slsk_private_message(net, user, message, timestamp?timestamp:1);
  slsk_net_send(net, SLSK_S_PRIVMSG_ACK, id);
}

HANDLER(handle_user_stats) {
  char* nick;
  slsk_user_t* user;
  int speed, down, shared, dirs;

  (void)net;
  nick = unpack_string_u32(&data, &dlen);
  CHECK(nick);
  REQ_LEN(4*5);

  user = slsk_user_find(nick);
  slsk_user_set_net(user, net);

  speed = unpack_u32(&data, &dlen);
  down = unpack_u32(&data, &dlen);
  unpack_u32(&data, &dlen);
  shared = unpack_u32(&data, &dlen);
  dirs = unpack_u32(&data, &dlen);
  slsk_user_set_info(user, speed, down, shared, dirs);

  CHECK_LEN(0);
}

HANDLER(handle_room_added) {
  char* name;
  slsk_channel_t* channel;

  name = unpack_string_u32(&data, &dlen);
  CHECK(name);
  CHECK_LEN(0);

  channel = g_malloc0(sizeof(*channel));
  channel->name = g_strdup(name);
  channel->net = net;

  slsk_net_channel_add(net, channel, TRUE);
}

HANDLER(handle_room_removed) {
  char* channel;

  channel = unpack_string_u32(&data, &dlen);
  CHECK(channel);
  CHECK_LEN(0);

  slsk_net_channel_remove(net, channel);
}

HANDLER(handle_room_list)
{
    unsigned number, i1;
    slsk_channel_t** channel;
    
    // FIXME: free data correctly on errors
    REQ_LEN(4);
    number = unpack_u32(&data, &dlen);
    channel = g_malloc(sizeof(*channel)*number);
    for (i1 = 0; i1 < number; i1++)
    {
        char* name;
        
        name = unpack_string_u32(&data, &dlen);
        CHECK(name);
        
        channel[i1] = g_malloc(sizeof(slsk_channel_t));
        channel[i1]->name = g_strdup(name);
        channel[i1]->online = 0;
        channel[i1]->net = net;
    }
    REQ_LEN(4);
    number = unpack_u32(&data, &dlen);
    REQ_LEN(4*number);
    for (i1 = 0; i1 < number; i1++)
    {
        channel[i1]->online = unpack_u32(&data, &dlen);
        slsk_net_channel_add(net, channel[i1], (i1==number-1)?TRUE:FALSE);
    }
    
    CHECK_LEN(0);
    
    g_free(channel);
    
    // SUCCESS
    //  slsk_net_update_shares(net);
    slsk_net_check_privileges(net);
    slsk_til_send(net);
    slsk_tih_send(net);
    
    slsk_net_send(net, SLSK_S_HAVE_NO_PARENT, 1);
}

HANDLER(handle_user_list) {
  int number;
  int i1;

  (void)net;

  REQ_LEN(4);
  number = unpack_u32(&data, &dlen);
  for (i1 = 0; i1 < number; i1++) {
    char* string = unpack_string_u32(&data, &dlen);
    CHECK(string);
    slsk_user_set_privileges(slsk_user_find(string), SLSK_PRIV_ON);
  }
  slsk_user_set_know_privileges();

  CHECK_LEN(0);
}

HANDLER(handle_new_privileged) {
  char* user;

  (void)net;
  user = unpack_string_u32(&data, &dlen);
  CHECK(user);
  slsk_user_set_privileges(slsk_user_find(user), SLSK_PRIV_ON);
  CHECK_LEN(0);
}

HANDLER(handle_add_user) {
  char* nick;
  char exist;
  slsk_user_t* user;

  (void)net;
  nick = unpack_string_u32(&data, &dlen);
  CHECK(nick);
  REQ_LEN(1);
  exist = unpack_byte(&data, &dlen);

  user = slsk_user_find(nick);
  slsk_user_set_net(user, net);

  if (exist) {
    int status, speed, down, shared, dirs;
    REQ_LEN(24);

    status = unpack_u32(&data, &dlen);
    slsk_user_set_status(user, status);

//  <nick><exist>[<status><[88].[00][00]><[bf][01][00][00]><[00][00][00][00]>[8f][17][00][00][fb][01][00][00]]
//  <nick><exist>[<status><speed?><down?><something?><shared><dirs>

    speed = unpack_u32(&data, &dlen);
    down = unpack_u32(&data, &dlen);
    status = unpack_u32(&data, &dlen);  // unknown (privileges? check!)
    if (status != 0) 
      g_log("SLSK_PROTO", G_LOG_LEVEL_WARNING,
            "user %s with unknown number = %d", OBJ(user)->name, status);
    shared = unpack_u32(&data, &dlen);
    dirs = unpack_u32(&data, &dlen);
    slsk_user_set_info(user, speed, down, shared, dirs);

/*     slsk_net_send(net, SLSK_S_GET_STATUS, OBJ(user)->name); */
  } else {
    slsk_user_set_status(user, SLSK_USER_UNKNOWN);
  }
  CHECK_LEN(0);
}

HANDLER(handle_global_msg)
{
    char* message;
    
    (void)net;
    message = unpack_string_u32(&data, &dlen);
    CHECK(message);
    CHECK_LEN(0);
    slsk_global_message(net, message);
}

HANDLER(handle_check_privileges)
{
    int days;
    
    (void)net;
    REQ_LEN(4);
    days = unpack_u32(&data, &dlen);
    CHECK_LEN(0);
    slsk_net_set_privileges(net, days);
}

HANDLER(handle_ticker_state)
{
    char* channel;
    int num, i1;
    slsk_cp_t* scp;
    
    channel = unpack_string_u32(&data, &dlen);
    CHECK(channel);
    scp = slsk_cp_search_net(net, channel, SLSK_PAGE_PUBLIC);
    if (!scp) return;
    
    REQ_LEN(4);
    num = unpack_u32(&data, &dlen);
    for (i1 = 0; i1 < num; i1++)
    {
        char* nick;
        char* message;
        
        nick = unpack_string_u32(&data, &dlen);
        CHECK(nick);
        message = unpack_string_u32(&data, &dlen);
        CHECK(message);
        
        slsk_ticker_new(scp, nick, message);
    }
    CHECK_LEN(0);
}

HANDLER(handle_ticker_add)
{
    char* channel;
    slsk_cp_t* scp;
    char* nick;
    char* message;
    
    channel = unpack_string_u32(&data, &dlen);
    CHECK(channel);
    scp = slsk_cp_search_net(net, channel, SLSK_PAGE_PUBLIC);
    if (!scp) return;
    
    nick = unpack_string_u32(&data, &dlen);
    CHECK(nick);
    message = unpack_string_u32(&data, &dlen);
    CHECK(message);
    
    CHECK_LEN(0);
    slsk_ticker_new(scp, nick, message);
}

HANDLER(handle_ticker_remove)
{
    char* channel;
    char* user;
    slsk_cp_t* scp;
    
    channel = unpack_string_u32(&data, &dlen);
    CHECK(channel);
    scp = slsk_cp_search_net(net, channel, SLSK_PAGE_PUBLIC);
    if (!scp) return;
    
    user = unpack_string_u32(&data, &dlen);
    CHECK(user);
    
    CHECK_LEN(0);
    
    slsk_ticker_remove(scp, user);
}

HANDLER(handle_item_reco)
{
    char* item;
    int num, i1;
    
    (void)net;
    SLSKUI.reco_clear();
    
    item = unpack_string_u32(&data, &dlen);
    CHECK(item);
    REQ_LEN(4);
    num = unpack_u32(&data, &dlen);
    for (i1 = 0; i1 < num; i1++)
    {
        char* key;
        int rating;
        
        key = unpack_string_u32(&data, &dlen);
        CHECK(key);
        REQ_LEN(4);
        rating = unpack_u32(&data, &dlen);
        if (rating > num/500)
        {
            char* enc = to_utf8(key, -1, NULL, NULL, 4,
                                "utf-8", slsk_charset_get(), "latin1", "windows-1252");
            if (enc) SLSKUI.reco_add(enc, rating);
            g_free(enc);
        }
    }
    CHECK_LEN(0);
}

HANDLER(handle_item_users)
{
    char* item;
    int num, i1;
    
    (void)net;
    SLSKUI.reco_user_clear();
    
    item = unpack_string_u32(&data, &dlen);
    CHECK(item);
    REQ_LEN(4);
    num = unpack_u32(&data, &dlen);
    for (i1 = 0; i1 < num; i1++)
    {
        char* key = unpack_string_u32(&data, &dlen);
        slsk_user_t* user;
        CHECK(key);
        user = slsk_user_find(key);
        slsk_user_set_net(user, net);
        SLSKUI.reco_user_add(user, 0);
    }
    CHECK_LEN(0);
}

HANDLER(handle_reco)
{
    int num, i1;
    
    (void)net;
    SLSKUI.reco_clear();
    
    REQ_LEN(4);
    num = unpack_u32(&data, &dlen);
    for (i1 = 0; i1 < num; i1++)
    {
        char* key;
        int rating;
        
        key = unpack_string_u32(&data, &dlen);
        CHECK(key); REQ_LEN(4);
        rating = unpack_u32(&data, &dlen);
        
        if (rating > num/500)
        {
            char* enc = to_utf8(key, -1, NULL, NULL, 4,
                                "utf-8", slsk_charset_get(), "latin1", "windows-1252");
            if (enc) SLSKUI.reco_add(enc, rating);
            g_free(enc);
        }
    }
    CHECK_LEN(0);
}

HANDLER(handle_similar_users)
{
    int num, i1;
    
    (void)net;
    SLSKUI.reco_user_clear();
    
    REQ_LEN(4);
    num = unpack_u32(&data, &dlen);
    for (i1 = 0; i1 < num; i1++)
    {
        char* key;
        int rating;
        slsk_user_t* user;
        
        key = unpack_string_u32(&data, &dlen);
        CHECK(key); REQ_LEN(4);
        rating = unpack_u32(&data, &dlen);
        user = slsk_user_find(key);
        slsk_user_set_net(user, net);
        SLSKUI.reco_user_add(user, rating);
    }
    CHECK_LEN(0);
}


HANDLER(handle_parent_timeout)
{
    unsigned interval;
    
    (void)net;
    
    REQ_LEN(4);
    interval = unpack_u32(&data, &dlen);
    CHECK_LEN(0);
    
    slsk_peer_set_parent_timeout(interval);
}

HANDLER(handle_parent_interval) {
  unsigned interval;
  
  (void)net;

  REQ_LEN(4);
  interval = unpack_u32(&data, &dlen);
  CHECK_LEN(0);

  slsk_peer_set_parent_interval(interval);
}

HANDLER(handle_exact_search) {
  char* file;
  char* folder;
  char* user;
  guint32 size1;
  guint32 size2;
  guint32 checksum;
  guint32 token;

  (void)net;
  user = unpack_string_u32(&data, &dlen);
  CHECK(user); REQ_LEN(4);
  token = unpack_u32(&data, &dlen);
  file = unpack_string_u32(&data, &dlen);
  CHECK(file);
  folder = unpack_string_u32(&data, &dlen);
  CHECK(folder); REQ_LEN(12);
  
  size1 = unpack_u32(&data, &dlen);
  size2 = unpack_u32(&data, &dlen);
  checksum = unpack_u32(&data, &dlen);
  
  CHECK_LEN(0);

  slsk_exact_search_answer(net, user, file, size1, size2, checksum);
}

HANDLER(handle_net_info) {
  guint32 number, i1;

  REQ_LEN(4);
  number = unpack_u32(&data, &dlen);
  for (i1 = 0; i1 < number; i1++) {
    char* nick;
    unsigned long ip;
    unsigned short port;

    nick = unpack_string_u32(&data, &dlen);
    CHECK(nick); REQ_LEN(8);
    ip = htonl(unpack_u32(&data, &dlen));
    port = (unsigned short)unpack_u32(&data, &dlen);
    
    slsk_peer_add_host(net, nick, ip, htons(port));
  }
  CHECK_LEN(0);

  slsk_peer_make_d();
}

HANDLER(handle_min_parents) {
  unsigned number;
  
  (void)net;

  REQ_LEN(4);
  number = unpack_u32(&data, &dlen);
  CHECK_LEN(0);

  slsk_peer_set_min_parents(number);
}

HANDLER(handle_search) {
  char* nick;
  guint32 token;
  char* term;

  (void)net;
  nick = unpack_string_u32(&data, &dlen);
  CHECK(nick); REQ_LEN(4);
  token = unpack_u32(&data, &dlen);
  term = unpack_string_u32(&data, &dlen);
  CHECK(term);

  CHECK_LEN(0);

  slsk_search_process(slsk_user_find(nick), token, term, 0);
}

const char* slsk_handle_message(slsk_net_t* net, slsk_message_t* command)
{
    char* data;
    int success;
    char* message;
    unsigned dlen = command->length;
    slsk_con_t* con = SLSK_CON(net);
    
    switch (command->type)
    {
    case SLSK_R_LOGIN:
    {
        data = command->data;
        if (dlen < 1) return "invalid login message";
        success = unpack_byte(&data, &dlen);
        
        if (success)
        {
            slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                         VAL_BYTE, VAL_STRING, VAL_IP, VAL_NONE);
            
            message = unpack_string_u32(&data, &dlen);
            if (!message) return "invalid login message";
            
            if (dlen < 4) return "invalid login message";
            // damnit! slsk also sends ip's in little endian format
            set_local_ip(htonl(unpack_u32(&data, &dlen)));
            slsk_net_success(net, message);
        }
        else
        {
            slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                         VAL_BYTE, VAL_STRING, VAL_NONE);
            
            message = unpack_string_u32(&data, &dlen);
            if (!message) return "invalid login message";
            return l_strdup(message);
        }
        break;
    }
    case SLSK_R_GET_ADDRESS:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_IP, VAL_INT, VAL_NONE);
        SLSK_CALL(handle_got_address);
        break;
    case SLSK_R_SEARCH:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_INT, VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_search);
        break;
    case SLSK_R_EXACT_SEARCH:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_INT, VAL_STRING,
                     VAL_STRING, VAL_INT, VAL_INT, VAL_INT, VAL_NONE);
        SLSK_CALL(handle_exact_search);
        break;
    case SLSK_R_USER_STATUS:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_INT, VAL_BYTE, VAL_NONE);
        SLSK_CALL(handle_user_status);
        break;
    case SLSK_R_PUBLIC_MESSAGE:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_STRING, VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_public_message);
        break;
    case SLSK_R_CHANNEL_USER_LIST:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_INT, VAL_INC);
        SLSK_CALL(handle_channel_user_list);
        break;
    case SLSK_R_PART_CHANNEL:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_part_channel);
        break;
    case SLSK_R_USER_JOIN:
/*     slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, */
/*                  VAL_STRING, VAL_STRING, VAL_INT, VAL_INT, VAL_INT,  */
/*                  VAL_INT, VAL_INT, VAL_INT, VAL_INT, VAL_NONE); */
        SLSK_CALL(handle_user_join);
        break;
    case SLSK_R_USER_PART:
/*     slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG, */
/*                  VAL_STRING, VAL_STRING, VAL_NONE); */
        SLSK_CALL(handle_user_part);
        break;
    case SLSK_R_PEER_REQUEST:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_STRING, VAL_IP, VAL_INT,
                     VAL_INT, VAL_BYTE, VAL_NONE);
        SLSK_CALL(handle_peer_request);
        break;
    case SLSK_R_PRIVMSG:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_INT, VAL_STRING, VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_priv_msg);
        break;
    case SLSK_R_NET_INFO:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_INC);
        SLSK_CALL(handle_net_info);
        break;
    case SLSK_R_USER_STATS:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_INT, VAL_INT,
                     VAL_INT, VAL_INT, VAL_INT, VAL_NONE);
        SLSK_CALL(handle_user_stats);
        break;
    case SLSK_R_ROOM_ADDED:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_room_added);
        break;
    case SLSK_R_ROOM_REMOVED:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_room_removed);
        break;
    case SLSK_R_ROOM_LIST:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_INC);
        SLSK_CALL(handle_room_list);
        break;
    case SLSK_R_USER_LIST:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_INC);
        SLSK_CALL(handle_user_list);
        break;
    case SLSK_R_NEW_PRIVILEGED:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_new_privileged);
        break;
    case SLSK_R_ADD_USER:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_BYTE, VAL_INC);
        SLSK_CALL(handle_add_user);
        break;
    case SLSK_R_GLOBAL:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_global_msg);
        break;
    case SLSK_R_CHECK_PRIVILEGES:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_NONE);
        SLSK_CALL(handle_check_privileges);
        break;
    case SLSK_R_TICKER_STATE:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_INT, VAL_INC);
        SLSK_CALL(handle_ticker_state);
        break;
    case SLSK_R_TICKER_ADD:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_STRING, VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_ticker_add);
        break;
    case SLSK_R_TICKER_REMOVE:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_STRING, VAL_NONE);
        SLSK_CALL(handle_ticker_remove);
        break;
    case SLSK_R_ITEM_RECO:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_INT, VAL_INC);
        SLSK_CALL(handle_item_reco);
        break;
    case SLSK_R_ITEM_USERS:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_STRING, VAL_INT, VAL_INC);
        SLSK_CALL(handle_item_users);
        break;
    case SLSK_R_LOCAL_RECO:
    case SLSK_R_GLOBAL_RECO:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_INC);
        SLSK_CALL(handle_reco);
        break;
    case SLSK_R_SIMILAR_USERS:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_INC);
        SLSK_CALL(handle_similar_users);
        break;
    case SLSK_R_PARENT_TIMEOUT:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_INC);
        SLSK_CALL(handle_parent_timeout);
        break;
    case SLSK_R_PARENT_INTERVAL:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_INC);
        SLSK_CALL(handle_parent_interval);
        break;
    case SLSK_R_MIN_PARENTS:
        slsk_message(command, 'R', con, G_LOG_LEVEL_DEBUG,
                     VAL_INT, VAL_INC);
        SLSK_CALL(handle_min_parents);
    case 83:
        // m_parentMinSpeed = 1
    case 84:
        // m_parentSpeedConnectionRatio = 10
    case 87:
        // m_secondsServerInactivityBeforeDisconnect = 20000
    case 104:
        slsk_message(command, 'R', con, G_LOG_LEVEL_WARNING,
                     VAL_INT, VAL_NONE);
        break;
    default:
        slsk_message(command, 'R', con, G_LOG_LEVEL_WARNING,
                     VAL_NONE);
        break;
    }
    return NULL;
}

void slsk_pack_command(slsk_net_t* net, guint32 type, va_list ap) {
  buffer_t* buffer;
  char* pos;
  slsk_message_t m;
  slsk_con_t* con = SLSK_CON(net);

  // FIXME: we assume that we dont need more space than 4096 bytes
  buffer = b_socket_get_out_buffer(BSOCK(net), 4096);
  pos = buffer_write_pos(buffer) + 4;

  pos += pack_u32(pos, type);
  m.type = type;
  m.data = pos;
  m.length = 0;
  switch (type) {
  case SLSK_S_LOGIN:
    // string : string : int
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
                 VAL_STRING, VAL_STRING, VAL_INT, VAL_NONE);
    break;
  case SLSK_S_PUBLIC:
  case SLSK_S_PRIVMSG:
  case SLSK_S_SET_TICKER:
    // string : string
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
                 VAL_STRING, VAL_STRING, VAL_NONE);
    break;
  case SLSK_S_JOIN_CHANNEL:
  case SLSK_S_PART_CHANNEL:
  case SLSK_S_GET_ADDRESS:
  case SLSK_S_REMOVE_USER:
  case SLSK_S_ADD_USER:
  case SLSK_S_ADD_LIKE:
  case SLSK_S_REMOVE_LIKE:
  case SLSK_S_ADD_HATE:
  case SLSK_S_REMOVE_HATE:
  case SLSK_S_ITEM_RECO:
  case SLSK_S_ITEM_USERS:
  case SLSK_S_GET_STATUS:
    // string
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
                 VAL_STRING, VAL_NONE);
    break;
  case SLSK_S_CHECK_PRIVILEGES:
  case SLSK_S_LOCAL_RECO:
  case SLSK_S_GLOBAL_RECO:
  case SLSK_S_SIMILAR_USERS:
  case SLSK_S_PING:
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
                 VAL_NONE);
    break;
  case SLSK_S_SHARE_LIB:
    // int : int
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
                 VAL_INT, VAL_INT, VAL_NONE);
    break;
  case SLSK_S_REQUEST_PEER:
    // int : string : char
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    m.length += pack_u32(pos+m.length, 1);
    m.length += pack_byte(pos+m.length, va_arg(ap, int));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
                 VAL_INT, VAL_STRING, VAL_STRING, VAL_NONE);
    break;
  case SLSK_S_SEARCH:
  case SLSK_S_CANT_CONNECT:
    // int : string
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    m.length += pack_string_u32(pos+m.length, va_arg(ap, char*));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
                 VAL_INT, VAL_STRING, VAL_NONE);
    break;
  case SLSK_S_SET_STATUS:
  case SLSK_S_SET_LISTEN:
  case SLSK_S_PRIVMSG_ACK:
    // int
    m.length += pack_u32(pos+m.length, va_arg(ap, int));
    slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG,
                 VAL_INT, VAL_NONE);
    break;
/*   case SLSK_S_HAVE_NO_PARENT: */
/*     // byte */
/*     m.length += pack_byte(pos+m.length, (char)va_arg(ap, int)); */
/*     slsk_message(&m, 'S', con, G_LOG_LEVEL_DEBUG, */
/*                  VAL_BYTE, VAL_NONE); */
/*     break; */
  default:
    g_log("SLSK_PROTO", G_LOG_LEVEL_WARNING,
          "unknown command %u", type);
    return;
  }

  pos = buffer->data + buffer->size;
  pack_u32(pos, m.length+4);
  buffer->size += (m.length+8);
}

#undef REQ_LEN
#undef CHECK
#undef CHECK_LEN
