/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.1 $
 * $Date: 2007/04/29 09:55:06 $
 * $Author: sgop $
 */

#include <string.h>

#include "lcore/attribute.h"
#include "lcore/search.h"
#include "lcore/library.h"
#include "lcore/zbuffer.h"
#include "lcore/upload.h"
#include "lcore/lib_search.h"
#include "lcore/download.h"
#include "slsk_network.h"
#include "slsk.h"
#include "slsk_proto.h"
#include "slsk_peer.h"
#include "slsk_peer_proto.h"
#include "slsk_search.h"
#include "slsk_download.h"

typedef struct {
  int token;
  search_t* search;
} slsk_search_t;

typedef struct {
  module_t _module;
  GList* searches;
  int token;
} slsk_search_module_t;

static slsk_search_module_t* SlskSearchModule = NULL;


static void slsk_search_remove(search_t* search) {
  GList* dlist;

  for (dlist = SlskSearchModule->searches; dlist; dlist = dlist->next) {
    slsk_search_t* ssearch = dlist->data;
    if (ssearch->search == search) {
      SlskSearchModule->searches = g_list_delete_link(SlskSearchModule->searches, dlist);
      g_free(ssearch);
      return;
    }
  }
}

static void slsk_search_queue(search_t* search)
{
    slsk_search_t* ssearch;
    char* term;
    
    if (search->download) return; //FIXME?
    
    if (!SEARCHP(search)->include) return;
    
    ssearch = g_malloc(sizeof(*ssearch));
    ssearch->search = search;
    ssearch->token = SlskSearchModule->token++;
    
    SlskSearchModule->searches = g_list_prepend(SlskSearchModule->searches, ssearch);
    object_notify_d(OBJ(search), (ObjectFunc)slsk_search_remove, NULL);
    
    if (SEARCHP(search)->exclude)
    {
        char* inc = make_string_from_list(SEARCHP(search)->include, " ");
        char* exc = make_string_from_list(SEARCHP(search)->exclude, " -");
        term = l_printf("%s -%s", inc, exc);
    }
    else
    {
        term = make_string_from_list(SEARCHP(search)->include, " ");
    }
    slsk_net_send(NULL, SLSK_S_SEARCH, ssearch->token, term);
}

void slsk_search_insert_result(int token, file_t* file, slsk_user_t* user) {
  GList* dlist;

  g_assert(SlskSearchModule);

  if (!file) return;

  for (dlist = SlskSearchModule->searches; dlist; dlist = dlist->next) {
    slsk_search_t* search = dlist->data;
    //    if (search->search->download) continue;
    if (search->token != token) continue;

    if (search->search->download) {
      if (download_matches_source(search->search->download, file, USER(user)))
        download_add_source(search->search->download, file, USER(user));
    } else {
      if (search_pattern_check_file(SEARCHP(search->search), file, TRUE))
        search_add_result(search->search, file, USER(user), 1, user->has_slots);
    }
    break;
  }
}

/*
static file_node_t*
collect_results(file_tree_t* tree, file_node_t* node, 
		slsk_search_request_t* request) {
  file_t* file;

  (void)tree;
  file = node->file;
  g_assert(file);
  
  // check media type
  if (request->media != MEDIA_NONE && file->media != request->media)
    return NULL;
  
  if (strcmp(file->filename, request->file)) return NULL;
  
  // this is a result, add;
  request->results = g_list_prepend(request->results, file);
  request->max_results--;
  if (request->max_results <= 0) return node; // stop searching

  return NULL;
}
*/
void slsk_exact_search_answer(slsk_net_t* net, const char* user, const char* file,
                              file_size_t size1, file_size_t size2, guint32 checksum) {
  g_assert(SlskSearchModule);
  /*
  GList* dlist;
  static slsk_search_request_t req = {
    NULL, 0, 0, NULL
  };

  req.file = to_utf8(file);
  req.media = mtype_find_from_suffix(file);
  req.max_results = 100;
  g_assert(!req.results);

  lib_foreach_shared((FileNodeFunc)collect_results, &req);
  
  for (dlist = req.results; dlist; dlist = dlist->next) {
    file_t* file = dlist->data;
    //FIXME: dont know how to anwser this search
  }
  g_list_free(req.results);
  req.results = NULL;
  */
}

void slsk_search_answer(slsk_peer_t* con, guint32 token, GList* results) {
  int sstart;
  buffer_t* buffer = buffer_new(2048);
  buffer_t* out;
  GList* dlist;
  compress_t* deflate;
  
  buffer->size +=
    pack_string_u32(buffer->data+buffer->size, slsk_net_get_username(con->user->net));
  buffer->size += 
    pack_u32(buffer->data+buffer->size, token);
  buffer->size += 
    pack_u32(buffer->data+buffer->size, g_list_length(results));

  for (dlist = results; dlist; dlist = dlist->next) {
    file_t* file = dlist->data;
    buffer_need_space(buffer, 2048);
    buffer->size += slsk_pack_file(buffer->data+buffer->size, file, TRUE);
  }

  buffer->size += pack_byte(buffer->data+buffer->size, uploads_slots_free()<=0?0:1);
  buffer->size += pack_u32(buffer->data+buffer->size, 0);  // FIXME: speed
  buffer->size += pack_u32(buffer->data+buffer->size, uploads_queue_size());
  // FIXME: don't know meaning of next one
/*   buffer->size += pack_u32(buffer->data+buffer->size, 0); */

  { // debug output
    slsk_message_t command;
    command.type = SLSK_X_SEARCH_RESULT;
    command.length = buffer->size;
    command.data = buffer->data;
    slsk_message(&command, 'S', SLSK_CON(con), G_LOG_LEVEL_DEBUG, 
                 VAL_STRING, VAL_INT, VAL_INT, VAL_INC);
  }

  out = b_socket_get_out_buffer(BSOCK(con), 1024);
  sstart = out->size;
  out->size += 8;

  deflate = compress_new(COMPRESS_ZLIB, TRUE,  buffer, out);
  if (!deflate) {
    buffer_destroy(buffer);
    return;
  }
  compress_action(deflate, TRUE);
  g_assert(buffer_avail(buffer) == 0);

  compress_destroy(deflate);
  buffer_destroy(buffer);

  // finally add the header length
  pack_u32(out->data+sstart, out->size-4-sstart);
  pack_u32(out->data+sstart+4, SLSK_X_SEARCH_RESULT);

  b_socket_start_sending(BSOCK(con));
  b_socket_close_when_sent(BSOCK(con), ST_DONE);  // FIXME: really close?
  
}

void slsk_search_process(slsk_user_t* user, guint32 token,
                         const char* items, guint32 temp) {

  search_pattern_t* search;
  GList* results;

  (void)temp;

  g_assert(SlskSearchModule);
  if (!items || *items == 0) return;

  search = search_pattern_new();
  search_string_tokenize(l_strdup(items), &search->include, &search->exclude);

  search->media = MEDIA_ANY;
  //FIXME: need to parse more.
  if (search->include) {
    if (!strcmp("mp3", (char*)search->include->data)) {
      search->media = MEDIA_AUDIO;
      g_free(search->include->data);
      search->include = g_slist_delete_link(search->include, search->include);
    } else if (!strcmp("avi", (char*)search->include->data) ||
               !strcmp("mpg", (char*)search->include->data)) {
      search->media = MEDIA_VIDEO;
      g_free(search->include->data);
      search->include = g_slist_delete_link(search->include, search->include);
    }
  }

  results = lib_search_do(search);
  if (results) slsk_peer_send_results(user, token, results);
}


static void slsk_searches_init() {
  search_register_action(slsk_search_queue);
}

static void slsk_search_module_init(slsk_search_module_t* module) {
  module_init(MODULE(module), "Slsk Search Module",
              (ModuleFunc)slsk_searches_init, NULL, NULL);
  
  module->searches = NULL;
  module->token = 0;

  module_require(search_module_instance);
  module_require(slsk_net_module_instance);
  module_require(lib_search_module_instance);
  module_require(download_module_instance);
}


module_t* slsk_search_module_instance() {
  if (!SlskSearchModule) {
    SlskSearchModule = g_malloc0(sizeof(*SlskSearchModule));
    slsk_search_module_init(SlskSearchModule);
  }
  return MODULE(SlskSearchModule);
}
