/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#include <string.h>

#include <libxml/parser.h>
#include <glib.h>

#include "lcore/utils.h"
#include "lcore/lopster.h"
#include "lcore/string_list.h"
#include "lcore/action_item.h"
#include "lcore/search.h"
#include "lcore/mtypes.h"
#include "slsk_ui.h"
#include "slsk_proto.h"

#ifdef WIN32
#include "slskalias.h"
#endif

void slsk_til_add(const char* item) {
  g_assert(item);
  if (string_list_add("slsk:til", item))
    slsk_net_send(NULL, SLSK_S_ADD_LIKE, item);
}

void slsk_tih_add(const char* item) {
  g_assert(item);
  if (string_list_add("slsk:tih", item))
    slsk_net_send(NULL, SLSK_S_ADD_HATE, item);
}

void slsk_til_remove(const char* item) {
  g_assert(item);
  if (string_list_remove("slsk:til", item))
    slsk_net_send(NULL, SLSK_S_REMOVE_LIKE, item);
}

void slsk_tih_remove(const char* item) {
  g_assert(item);
  if (string_list_remove("slsk:tih", item))
    slsk_net_send(NULL, SLSK_S_REMOVE_HATE, item);
}

void slsk_til_send(slsk_net_t* net) {
  const GList* dlist;
  for (dlist = string_list_get_items("slsk:til"); dlist; dlist = dlist->next) {
    char* item = dlist->data;
    slsk_net_send(net, SLSK_S_ADD_LIKE, item);
  }
}

void slsk_tih_send(slsk_net_t* net) {
  const GList* dlist;
  for (dlist = string_list_get_items("slsk:tih"); dlist; dlist = dlist->next) {
    char* item = dlist->data;
    slsk_net_send(net, SLSK_S_ADD_HATE, item);
  }
}

void slsk_til_get_item_recommendation(const char* item) {
  if (!item) return;
  slsk_net_send(NULL, SLSK_S_ITEM_RECO, item);
}

void slsk_til_get_user_recommendation(const char* item) {
  if (!item) return;
  slsk_net_send(NULL, SLSK_S_ITEM_USERS, item);
}

void slsk_til_get_global_recommendation() {
  slsk_net_send(NULL, SLSK_S_GLOBAL_RECO);
}

void slsk_til_get_recommendation() {
  slsk_net_send(NULL, SLSK_S_LOCAL_RECO);
}

void slsk_til_get_similar_users() {
  slsk_net_send(NULL, SLSK_S_SIMILAR_USERS);
}

void slsk_til_search(const char* item) {
  search_pattern_t* pattern;

  if (!item || *item == 0) return;
  
  pattern = search_pattern_new();

  // we have to setup the pattern manually
  make_list_from_string(&pattern->include, item, " ");
  object_set_name(OBJ(pattern), item);
  pattern->media = MEDIA_AUDIO;
  search_new_from_pattern(pattern, -1);

  object_kill(OBJ(pattern));
}

gboolean slsk_til_in_list(const char* item) {
  if (string_list_check("slsk:til", item)) return TRUE;
  else return FALSE;
}

gboolean slsk_tih_in_list(const char* item) {
  if (string_list_check("slsk:tih", item)) return TRUE;
  else return FALSE;
}

#ifdef WIN32
#define _SLSK_TIL_C_
#include "slskaliasdef.c"
#endif

