/* Copyright (C) 2005 sgop@users.sourceforge.net
 * This is free software distributed under the terms of the
 * GNU Public License.  See the file COPYING for details.
 */
/* $Revision: 1.4 $
 * $Date: 2007/09/05 14:34:10 $
 * $Author: sid4u $
 */

#include "lcore/plugin.h"
#include "lcore/ui.h"
#include "lcore/dynamic.h"

#include "slsk_ui.h"

ui_slsk_t SLSKUI;

typedef struct {
  module_t _module;
  void (*init)(void);
  void (*run)(void);
  void (*exit)(void);
} slsk_ui_module_t;

static slsk_ui_module_t* SlskUIModule = NULL;

#define CHECK_SYMBOL(sym) \
  if (!(sym)) {\
    g_log("SLSK", G_LOG_LEVEL_WARNING, "could not find symbol '%s'", #sym);\
    g_assert(FALSE);\
  }

static void dummy_func() {}

void slsk_ui_load_syms()
{
    const char* filename;
    void* handle;
    
    SLSKUI.create_public_page = dummy_func;
    SLSKUI.create_private_page = dummy_func;
    SLSKUI.channel_joined = dummy_func;
    SLSKUI.user_show = dummy_func;
    SLSKUI.user_joined = dummy_func;
    SLSKUI.user_parted = dummy_func;
    SLSKUI.whois_new = dummy_func;
    SLSKUI.whois_pic_progress = dummy_func;
    SLSKUI.whois_finish = dummy_func;
    SLSKUI.net_show = dummy_func;
    
    filename = ui_find_library_file("slsk");
    if (!filename)
    {
        g_log("SLSK", G_LOG_LEVEL_INFO, "starting without interface (could not find lib)");
        return;
    }
    
    handle = lopster_dll_open(filename, G_MODULE_BIND_LAZY|G_MODULE_BIND_LOCAL);
    if (!handle)
    {
        g_log("SLSK", G_LOG_LEVEL_WARNING, "%s", g_module_error());
        g_log("SLSK", G_LOG_LEVEL_INFO, "starting plugin without interface (could not load lib)");
        return;
    }
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_init", (void**)&SlskUIModule->init));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_run", (void**)&SlskUIModule->run));
   
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_exit", (void**)&SlskUIModule->exit));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_create_public_page", (void**)&SLSKUI.create_public_page));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_create_private_page", (void**)&SLSKUI.create_private_page));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_channel_joined", (void**)&SLSKUI.channel_joined));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_user_show", (void**)&SLSKUI.user_show));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_user_joined", (void**)&SLSKUI.user_joined));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_user_parted", (void**)&SLSKUI.user_parted));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_net_show", (void**)&SLSKUI.net_show));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_show_channels", (void**)&SLSKUI.update_channels));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_ticker_show", (void**)&SLSKUI.ticker_show));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_reco_clear", (void**)&SLSKUI.reco_clear));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_reco_add", (void**)&SLSKUI.reco_add));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_reco_user_clear", (void**)&SLSKUI.reco_user_clear));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_reco_user_add", (void**)&SLSKUI.reco_user_add));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_whois_new", (void**)&SLSKUI.whois_new));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_whois_pic_progress", (void**)&SLSKUI.whois_pic_progress));
    
    CHECK_SYMBOL(g_module_symbol(handle, "ui_slsk_whois_finish", (void**)&SLSKUI.whois_finish));
}


static void slsk_ui_init() {
    if (SlskUIModule->init) SlskUIModule->init();
}
static void slsk_ui_run() {
    if (SlskUIModule->run) SlskUIModule->run();
}
static void slsk_ui_exit() {
    if (SlskUIModule->exit) SlskUIModule->exit();
}

static void slsk_ui_module_init(slsk_ui_module_t* module)
{
    module_init(MODULE(module), "Slsk User Interface Module",
                (ModuleFunc)slsk_ui_init, (ModuleFunc)slsk_ui_run, slsk_ui_exit);
    
    module->init = dummy_func;
    module->run = dummy_func;
    module->exit = dummy_func;
    
    module_require(ui_module_instance);
    module_require(dynamic_module_instance);
    
    slsk_ui_load_syms();
}

module_t* slsk_ui_module_instance() {
  if (!SlskUIModule) {
    SlskUIModule = g_malloc0(sizeof(*SlskUIModule));
    slsk_ui_module_init(SlskUIModule);
  }
  return MODULE(SlskUIModule);
}

#undef CHECK_SYMBOL

