/* Copyright (C) 2005 sgop@users.sourceforge.net
   This is free software distributed under the terms of the
   GNU Public License.  See the file COPYING for details. */

#include <string.h>
#include <stdlib.h>

#include "lcore/file_segment.h"
#include "slsk_upload.h"
#include "slsk_proto.h"
#include "slsk_peer_proto.h"

static unsigned Token = 1000;

slsk_upload_t* slsk_upload_find_user_req(slsk_user_t* user, guint32 req) {
  GList* dlist;

  for (dlist = USER(user)->uploads; dlist; dlist = dlist->next) {
    slsk_upload_t* upload = dlist->data;
    if (UPLOAD(upload)->status != UPLOAD_REMOTE &&
        upload->token == req && !socket_connected(SOCK(upload)))
      return upload;
  }
  return NULL;
}

static void slsk_upload_queue(slsk_upload_t* upload, int pos, int slots) {
  slsk_peer_queued(upload->token, SLSK_USER(UPLOAD(upload)->user));
}

static gboolean slsk_upload_start(slsk_upload_t* upload) {
  if (upload->token > 0) {
    slsk_peer_upload(upload->token, SLSK_USER(UPLOAD(upload)->user),
                     UPLOAD(upload)->file->size);
  } else {
    upload->token = Token++;
    slsk_peer_request(upload->token, SLSK_USER(UPLOAD(upload)->user),
                      UPLOAD(upload)->file, 1);
  }
  return TRUE;
}

static gboolean slsk_upload_end(slsk_upload_t* upload, const char* message) {
  if (socket_get_status(SOCK(upload)) != ST_UL_SEG_FIN)
    slsk_peer_upload_failed(upload);

  return upload_end(UPLOAD(upload), message);
}

static void slsk_upload_destroy(slsk_upload_t* upload) {
  upload_t* next = upload_get_next_from_user(UPLOAD(upload));
  upload_destroy(UPLOAD(upload));
  if (next) upload_wake_up(next);
}

static void
slsk_upload_init(slsk_upload_t* upload, slsk_peer_t* con, file_t* file, guint32 token) {
  upload_init(UPLOAD(upload), file, USER(con->user));
  object_add_type(OBJ(upload), "SlskUpload");
  OBJ(upload)->destroy = (ObjectFunc)slsk_upload_destroy;
  socket_set_end(SOCK(upload), (SockEndFunc)slsk_upload_end);
  UPLOAD(upload)->queue = (UploadQueueFunc)slsk_upload_queue;
  UPLOAD(upload)->start = (UploadStartFunc)slsk_upload_start;
  upload->token = token;

}

slsk_upload_t* slsk_upload_new(slsk_peer_t* con, file_t* file, guint32 token) {
  slsk_upload_t* upload = g_malloc0(sizeof(*upload));
  slsk_upload_init(upload, con, file, token);
  return upload;
}

static const char* slsk_upload_test_receiver(slsk_upload_t* upload) {
  buffer_t* buffer = b_socket_get_in_buffer(BSOCK(upload));
  print_raw("unexpected upload input", buffer_read_pos(buffer), buffer_avail(buffer));
  buffer_clear(buffer);
  return NULL;
}

static const char* slsk_upload_get_offset(slsk_upload_t* upload) {
  buffer_t* buffer = b_socket_get_in_buffer(BSOCK(upload));
  guint32 dlen;
  char* data;
  file_size_t offset;
  const char* res;
  
  dlen = buffer_avail(buffer);
  data = buffer_read_pos(buffer);
  
  if (dlen < 8) return NULL;
  offset = unpack_u32d(&data, &dlen);
  buffer_consume(buffer, 8);

  g_log("SLSK_UPLOAD", G_LOG_LEVEL_DEBUG, "got offset "FF, offset);
  if (!upload_add_segment(UPLOAD(upload), offset,
			  UPLOAD(upload)->file->size, FALSE)) {
    return "could not add segment";
  }

  res = upload_prepare_segment(UPLOAD(upload));
  if (res) return res;

  b_socket_set_consume
    (BSOCK(upload), (BConsumeInputFunc)slsk_upload_test_receiver);
  b_socket_start_sending(BSOCK(upload));
  return NULL;
}

const char*
slsk_upload_adapt_peer(slsk_upload_t* upload, slsk_peer_t* con) {
  g_assert(!socket_connected(SOCK(upload)));
  
  b_socket_set_consume
    (BSOCK(upload), (BConsumeInputFunc)slsk_upload_get_offset);
  tcp_socket_move(TSOCK(upload), TSOCK(con), TRUE);
  b_socket_start_receiving(BSOCK(upload), 1024);
  
  return "slsk_incoming->slsk_upload";
}
